//----- fastdata.c ----- 06/17/10 -----(12/09/06)-----

//-----------------------------------------------------------------------------
//
//  Module to provide a fast, lower-overhead method to provide a continuous
//  stream of data to a remote client.
//
//-----------------------------------------------------------------------------
//
//  2010-06-17 - Mark Davis
//               - Added ability to read the current values from a chunk of
//                 memory
//
//  2009-10-26 - Mark Davis
//               - changes to work with register names ref via far ptr
//
//  2009-05-27 - Mark Davis
//               - fixed header so the names of the channels are correct
//
//  2006-12-09 - Mark Davis
//               - original version started
//
//-----------------------------------------------------------------------------

#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include <string.h>

#include <sys/backup.h>
#include <sys/hdwdefs.h>

#include <sys/modbus.h>


#define MaxCmdMsgLen    64
#define MaxRespMsgLen  256



//--- Save 16-bit value to a Byte array in Big-Endian (MSB-1st) format ---
#define  SetWordVal(offset,val,buf)  \
  (((buf)[offset] = (val) >> 8), ((buf)[(offset)+1] = (val) & 0xff))


#include <sys/farprintf.h>


#define  BufSize  512

//-----------------------------------------------------------------------------
//  send the client a copy of the current contents of portion of memory
//-----------------------------------------------------------------------------
void readMem(uchar *buf)
{


}


//-----------------------------------------------------------------------------
void _task fastdatad(int argc, char **argv)
{
  uint  u, numSamples, capIdx, lastCapIdx, numBytes;
  int  stat;
  RegInfo  *reg;
  uchar far  *fromPtr;
  char  *ptr;
  uchar  buf[BufSize];


#define  FastDataVers  2

  redirectStdio();  // so printf, perror, etc don't end up in data stream

  _EnterCritical_();  capIdx = captureIdx;  _ExitCritical_();
  lastCapIdx = capIdx;


//--- check to see if the client wants something besides a continuous update ---
//--- of the values specified in a currently running scope command ---
  stat = read_(0, buf, 8, 1000);
  if (stat == 8)  readMem(buf);


  ptr = buf + 2;

//--- Send the version # and the # of values in each sample ---
  addShort(&ptr, FastDataVers);
  addShort(&ptr, (short)numScopeRegs);

//--- and the name, size, and data type for each value ---
  for (u=0,reg=scopeList[0]; u<numScopeRegs; ++u,reg=scopeList[u])  {
    strcpyfn(ptr, reg->name);  ptr += strlen(ptr) + 1;
    addShort(&ptr, reg->numRegs * 2);
    addShort(&ptr, reg->dataType);
  }
  numBytes = ptr - buf;

//--- go back and insert the size of the header in the 1st 2 bytes ---
  ptr = buf;  addShort(&ptr, numBytes);

//--- and send the header ---
  if (write(1, buf, numBytes) != numBytes)  {
    fstrcpy(buf, strerror(errno));
    fprintf(DiagLog, "write(): %s\n", buf);  Exit(-1);
  }


//--- Now just loop, continuously sending any new data values to the client ---
  for (;;)  {
    _EnterCritical_();  capIdx = captureIdx;  _ExitCritical_();
    if (capIdx == lastCapIdx)  { sleepMS(5);  continue; }

    numSamples = (capIdx > lastCapIdx) ?
                   (capIdx - lastCapIdx) : (scopeBufSampleSize - lastCapIdx);

    if (numSamples * bytesPerScopeSample > sizeof(buf))
      numSamples = sizeof(buf) / bytesPerScopeSample;

    numBytes = numSamples * bytesPerScopeSample;
    fromPtr = scopeBuf + lastCapIdx * bytesPerScopeSample;

    KLOCK();  fmemcpy(buf, fromPtr, numBytes);  KUNLOCK();

    if (write(1, buf, numBytes) != numBytes)  {
      fstrcpy(buf, strerror(errno));
      fprintf(DiagLog, "write(): %s\n", buf);  Exit(-1);
    }

    if ((lastCapIdx += numSamples) >= scopeBufSampleSize)  lastCapIdx = 0;
  }

}

//-----------------------------------------------------------------------------
