//----- RFAC.c ----- 02/28/11 -----(02/15/11)-----

//-----------------------------------------------------------------------------
//  Code to handle hardware-specific implementation of generic hdw functions
//-----------------------------------------------------------------------------
//
//  2011-02-15 - Mark Davis -
//               - Began work on RF Amplifier Circulator (RFAC) controller
//                 platform code.
//
//-----------------------------------------------------------------------------

#include <common.h>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include <sys/backup.h>

#include <sys/spi.h>
#include <sys/hdwDefs.h>

#include <busybox.h>

#include <Rabbit.h>

#include <dcdefs.h>

#include <stcpip.h>

#include <sys/socket.h>
#include <sys/srfs.h>
#include <sys/flashFile.h>

#include <sys/farprintf.h>

//--- see hdwInit() for Digital I/O pin usage ---

const char ArchName[] = { "RFAC" };

#define  NUM_ADC    1
#define  NUM_DAC    1

#define  MAX_DIN   24
#define  MAX_DOUT   8


//--- these values can be set LOWER by the application ---
const uint  NumHdwADC = NUM_ADC;
const uint  NumHdwDAC = NUM_DAC;

const uint  MaxDIN =  MAX_DIN;
const uint  MaxDOUT = MAX_DOUT;


unsigned char  __brdInitFlag;

bool  normalRead = true;

#include <hdwDefs.c>


//SerialInfo  *spiPort;

//--- to insert a delay to avoid a WAIT response from the Rabbit RIO chip ---
#define  RIO_DELAY()  { nop(); nop(); nop(); nop(); nop(); nop(); }


//--- Rabbit RIO External Register addresses.  These are registers that ---
//--- can be accessed directly via an External I/O read/write.

//    NOTE:  As recommended in the Rabbit RIO manual, PB7 is connected
//           to the G//B pin on the Rabbit RIO chips.  Not sure why
//           they recommend this, as it means that the address for
//           External I/O reads/writes - which uses PB[7:2] - cannot
//           use the 5th address bit (PB6) to communicate with the
//           Rabbit RIO chips - so the External Register addresses
//           as shown below (to match the manual) have to be manipulated
//           before using them so the 5th bit drives PB7 rather than PB6.

#define  RIO_MCR    0x10    // Master Control Register
#define  RIO_MSR    0x11    // Master Satus Register
#define  RIO_MPR    0x12    // Master Prescale Register
#define  RIO_MADR   0x13    // Master Alternate Data Register
#define  RIO_MPCR   0x14    // Master Protection Command Register
#define  RIO_MPPR   0x15    // Master Protection Prescale Register

#define  RIO_WDT0R  0x1C    // Watchdog Timer 0 Register
#define  RIO_WDT1R  0x1D    // Watchdog Timer 1 Register
#define  RIO_WDT2R  0x1E    // Watchdog Timer 2 Register

#define  RIO_PR0    0x00    // Block 0 Pointer Register
#define  RIO_PR1    0x02    // Block 1 Pointer Register
#define  RIO_PR2    0x04    // Block 2 Pointer Register
#define  RIO_PR3    0x06    // Block 3 Pointer Register
#define  RIO_PR4    0x08    // Block 4 Pointer Register
#define  RIO_PR5    0x0A    // Block 5 Pointer Register
#define  RIO_PR6    0x0C    // Block 6 Pointer Register
#define  RIO_PR7    0x0E    // Block 7 Pointer Register

#define  RIO_IR0    0x01    // Block 0 Indirect Register
#define  RIO_IR1    0x03    // Block 1 Indirect Register
#define  RIO_IR2    0x05    // Block 2 Indirect Register
#define  RIO_IR3    0x07    // Block 3 Indirect Register
#define  RIO_IR4    0x09    // Block 4 Indirect Register
#define  RIO_IR5    0x0B    // Block 5 Indirect Register
#define  RIO_IR6    0x0D    // Block 6 Indirect Register
#define  RIO_IR7    0x0F    // Block 7 Indirect Register

//--- Interal Rabbit RIO registers.  Each of the 8 blocks has its own ---
//--- instance of these registers.  To access an Internal register for ---
//--- one of the blocks, write the register's address to the block's ---
//--- Pointer Register (PR0-PR7) and then do reads/writes to the block's ---
//--- Indirect Register (IR0-IR7) ---
#define  RIO_CR     0x00    // (WO) Command Regiser
#define  RIO_MR     0x01    // (RW) Mode Register
#define  RIO_IER    0x02    // (RW) Interrupt Enable Register
#define  RIO_SR     0x03    // (RW) Status Register
#define  RIO_CTR    0x04    // (WO) Counter Toggle Register
#define  RIO_SCR    0x05    // (WO) Synch Control Register
#define  RIO_ICR    0x06    // (WO) Incr/In-Phase/Begin Control Register
#define  RIO_DCR    0x07    // (WO) Decr/Quadrature/End Control Register

#define  RIO_S0CR   0x08    // (WO) Status 0 Control Register
#define  RIO_S1CR   0x09    // (WO) Status 1 Control Register
#define  RIO_S2CR   0x0A    // (WO) Status 2 Control Register
#define  RIO_S3CR   0x0B    // (WO) Status 3 Control Register

#define  RIO_P0CR   0x0C    // (RW) Pin 0 Control Register
#define  RIO_P1CR   0x0D    // (RW) Pin 1 Control Register
#define  RIO_P2CR   0x0E    // (RW) Pin 2 Control Register
#define  RIO_P3CR   0x0F    // (RW) Pin 3 Control Register

#define  RIO_M0LR   0x10    // (WO) Match 0 LSB Register
#define  RIO_M0MR   0x11    // (WO) Match 0 MSB Register
#define  RIO_M1LR   0x12    // (WO) Match 1 LSB Register
#define  RIO_M1MR   0x13    // (WO) Match 1 MSB Register
#define  RIO_M2LR   0x14    // (WO) Match 2 LSB Register
#define  RIO_M2MR   0x15    // (WO) Match 2 MSB Register
#define  RIO_M3LR   0x16    // (WO) Match 3 LSB Register
#define  RIO_M3MR   0x17    // (WO) Match 3 MSB Register

#define  RIO_CLLR   0x18    // (WO) Count Limit LSB Register
#define  RIO_CLMR   0x19    // (WO) Count Limit MSB Register
#define  RIO_CBLR   0x1A    // (RO) Count Begin LSB Register
#define  RIO_CBMR   0x1B    // (RO) Count Begin MSB Register
#define  RIO_CELR   0x1C    // (RO) Count End LSB Register
#define  RIO_CEMR   0x1D    // (RO) Count End MSB Register
#define  RIO_CVLR   0x1E    // (RO) Count Value LSB Register
#define  RIO_CVMR   0x1F    // (RO) Count Value MSB Register




//=============================================================================
_nearcall float adcGetRaw(uint ui)  { return (float) adcRaw[ui]; }

_nearcall float adcMinRaw(uint ui)  { return 0.0; }
_nearcall float adcMaxRaw(uint ui)  { return (float)0xffff; }

_nearcall void adcDefaults(uint ui, float *gain, float *offset)
{ *gain = 0.0003046;  *offset = 32786.0; }

_nearcall void adcCalibPoints(uint ui, float *low, float *high)
{ *low = -9.5;  *high = 9.5; }

_nearcall char *adcUnits(uint ui, char *buf)  { return "V"; }

//=============================================================================
_nearcall float dacGetRaw(uint ui)  { return (float) dacRaw[ui]; }

_nearcall float dacMinRaw(uint ui)  { return 0.0; }
_nearcall float dacMaxRaw(uint ui)  { return (float)0xffffL; }

_nearcall void dacDefaults(uint ui, float *gain, float *offset)
{  *gain = 0.000311;  *offset = 32768.0;  }

_nearcall void dacCalibPoints(uint ui, float *low, float *high)
  {  *low = -9.5;  *high = 9.5; }

_nearcall char *dacUnits(uint ui, char *buf)  { return "V"; }

//=============================================================================


//-----------------------------------------------------------------------------

/* START FUNCTION DESCRIPTION ********************************************
Function: int _readExtioPort ( int port );

Note:
This function is setup to provide additional hold time when
accessing external I/O addresses 0000 and 0002. This routine
must stay         and not be changed, for the additional
address hold-time to be valid.

Improved by Softools to not require a copy to RAM because
real tools have align and other stuff for real programmers.
************************************************************/
/*
unsigned readExtIO (unsigned port)      // Port is 0 or 2
{
#asm
    push    ip
    ipset   3
    ld      a,l
    and     a,0x03
    jr      z,..read0
    jr      ..skip
    .align  4           ; Align to clear A0 & A1
..read0:
    nop                 ; A0 = 0, A1 = 0
    nop                 ; A0 = 1, A1 = 0
    ioe     ld l,(hl)   ; A0 = 0, A1 = 1
    pop     ip          ; A0 = 0, A1 = 0
    jr      ..exit
    .align  4           ; Align to clear A0 & A1
..skip:
    ioe     ld l,(hl)   ; A0 = 0, A1 = 0 (2 bytes)
    pop     ip          ; A0 = 0, A1 = 1
..exit:
#endasm
}
*/

//-----------------------------------------------------------------------------
//  Set/Read Rabbit RIO registers
//-----------------------------------------------------------------------------
void _task rio(int argc, char **argv)
{
  uchar  setVal, readVal = 0;
  uint  extAddr, reg = 0;


  if (argc > 2) reg = ATOU(argv[2]) & 0x3f;
  if (argc > 3) setVal = ATOU(argv[3]);

// Current wiring is as recommended in the manual which uses
// PB7,5,4,3,2 rather than PB6,5,4,3,2 for the External I/O
// mode address bits (?!?!?!?!?)
// The 0x2000 is to cause PE1 (used for the Chip Select) to
// be active (low) during the transaction
  extAddr = ((reg & 0x10) << 1) | (reg & 0x0f) | 0x2000;


  if (argc > 1)  {

  //--- read a register value ---
    if (StrEq(argv[1], "read"))  if (argc > 2)  {
      if (StrEq(argv[2], "on"))
        normalRead = true;  else
      if (StrEq(argv[2], "off"))
        normalRead = false;  else
      {
        KLOCK();  outi(SPCR, 0x8C);
        readVal = ine(extAddr | 0x2000);
        outi(SPCR, srSPCR);  KUNLOCK();

        printf("  Reg 0x%02X:  Read:0x%02X\r\n", reg, readVal);
      }
      Exit(0);
    }

  //--- write to a register and then read it back ---
    if (StrEq(argv[1], "write"))  if (argc > 3)  {
      setVal = ATOU(argv[3]);

      KLOCK();  outi(SPCR, 0x8C);
      oute(extAddr, setVal);  RIO_DELAY();  readVal = ine(extAddr);
      outi(SPCR, srSPCR);  KUNLOCK();

      printf("  Reg 0x%02X:  Wrote:0x%02X  Read:0x%02X\r\n",
             reg, setVal, readVal);

      Exit(0);
    }

  //--- pulse PD4 low then high to reset the Rabbit RIO chip ---
    if (StrEq(argv[1], "reset"))  {
      outi(PDDR, srPDDR &= ~0x10);
      outi(PDDR, srPDDR |=  0x10);

      Exit(0);
    }

  }

  printf("\n"
         "usage:  rio read  <reg>|on|off\r\n"
         "            write <reg> <val>\r\n"
         "            reset\r\n"
        );

  Exit(-1);
}


//-----------------------------------------------------------------------------
//  determine the device ID based on external hardware settings
//-----------------------------------------------------------------------------
ulong ReadDeviceID()
{
  uchar  inF, devID;


  inF = RdPortI(PFDR);

//--- device ID comes from 8-bit switch input ----
  devID = ((inF & 0x01) << 7)
        | ((inF & 0x02) << 5)
        | ((inF & 0x04) << 3)
        | ((inF & 0x08) << 1)
        | ((inF & 0x10) >> 1)
        | ((inF & 0x20) >> 3)
        | ((inF & 0x40) >> 5)
        | ((inF & 0x80) >> 7);
  devID = ~devID;

  return (ulong) devID;
}

//-----------------------------------------------------------------------------
//  Return the current state of the digital inputs
//-----------------------------------------------------------------------------
ulong ReadDigInputs()
{
  uint  extAddr;
  int  blk;
  ulong  inBits0, inBits1;
  uchar  *inPtr;
  uchar  pinStates[16];


  inBits0 = inBits1 = 0;

  if (normalRead)  {
  //--- Read the state of all the Rabbit RIO pins ---

  //--- The Rabbit RIO chip has 8 blocks of 4 bits each.  Each block has ---
  //--- a Pointer Register and an Indirect Register that are used to ---
  //--- access the Internal registers for each block. ---
  //--- Reads and Writes from/to a block's Internal registers is done by ---
  //--- reading and writing from/to the single Indirect Register for the ---
  //--- block.  Which Internal register is actually accessed is determined ---
  //--- by the value currently in the block's Pointer Register. ---
  //--- So to access an Internal register, you write the address of the ---
  //--- Internal register to the block's Pointer register and then read ---
  //--- and/or write to the block's Indirect register. ---

  //--- Read the contents of the Internal P0CR (Pin 0 Control Register) ---
  //--- for each of the 8 blocks by writing its address to the block's ---
  //--- Pointer register and then reading the block's Indirect register ---
  //--- NOTE:  The 0x2000 offset causes PE1 to be used as the Chip Select ---
  //--- for the external read and write operations, while an offset of ---
  //--- 0xE000 causes PE7 to be used. ---

    KLOCK();  outi(SPCR, 0x8C);

    inPtr = pinStates;

    for (extAddr=0x2000; extAddr<0x200F; extAddr+=2)  {
      oute(extAddr, 0x0C);  RIO_DELAY();  *inPtr++ = ine(extAddr+1); }

    for (extAddr=0xE000; extAddr<0xE00F; extAddr+=2)  {
      oute(extAddr, 0x0C);  RIO_DELAY();  *inPtr++ = ine(extAddr+1); }

    outi(SPCR, srSPCR);  KUNLOCK();

  //--- blocks 5 to 0 are used on RIO chip U1A ---
    inPtr = pinStates + 5;
    for (blk=5; blk>=0; --blk)  inBits0 = (inBits0 << 4) | (*inPtr-- >> 4);

  //--- blocks 5 to 3 are used on RIO chip U2A ---
    inPtr = pinStates + 13;
    for (blk=5; blk>=0; --blk)  inBits1 = (inBits1 << 4) | (*inPtr-- >> 4);
    inBits1 >>= 14;
  }

  inBits0 = (inBits0 & ~forcedDigInMask) | forcedDigInBits;
  inBits1 = (inBits1 & ~forcedExtraInMask) | forcedExtraInBits;

  extraIn = inBits1;

  return inBits0;
}


//-----------------------------------------------------------------------------
//  set the state of the digital outputs
//-----------------------------------------------------------------------------
void WriteDigOutputs(ulong digOut)
{
  digOut = (digOut & ~forcedDigOutMask) | forcedDigOutBits;

  outi(PGDR, srPGDR = (uchar)digOut);
}


//-----------------------------------------------------------------------------
//  Read analog inputs.
//-----------------------------------------------------------------------------
void readAnalogInputs(ulong mask)
{
//  if (! spiPort)  return;

  adcRaw[0] = 0;
}


//-----------------------------------------------------------------------------
//  Update the state of the analog outputs
//-----------------------------------------------------------------------------
void writeAnalogOutputs(ulong mask)
{

}


//-----------------------------------------------------------------------------
/*
float ADCtoVolts(uchar chan, uint rawADC)
{
  return (ADCOffset(chan) - (float)(rawADC)) * ADCGain(chan);
}
*/

//-----------------------------------------------------------------------------
//  Set an External 2/4-wire RS-485 adapter to transmit mode
//-----------------------------------------------------------------------------
void setExt485Tx(void)
{
}
//-----------------------------------------------------------------------------
//  Set an External 2/4-wire RS-485 adapter to receive mode
//-----------------------------------------------------------------------------
void setExt485Rx(void)
{
}

//-----------------------------------------------------------------------------
//  functions to enable/disable the transmitter for an RS-485 connection
//-----------------------------------------------------------------------------
void _nearcall SerSetDirA(int enableXmit)  {  }
void _nearcall SerSetDirB(int enableXmit)  {  }
void _nearcall SerSetDirC(int enableXmit)  {  }
void _nearcall SerSetDirD(int enableXmit)
{  // PF5 controls RS-485 line driver for port D(3) RS-485 connections
//  char  val;
//  val = enableXmit ? (PFDRShadow | 0x20) : (PFDRShadow & ~0x20);
//  outi(PFDR, srPFDR = val);
}


//-----------------------------------------------------------------------------
void hdwInit(ulong digOutState)
{
//  uint  pin, extAddr;
  hdwType = HT_PS8_RACK;


  __brdInitFlag = TRUE;

// (NI) = Normal Input
// (NO) = Normal Output
// (AI) = Alternate Input
// (AO) = Alternate Output
// (xx) = Used by ZWorld core - do not change

//---------------------------------------------------------------
// General approach to setting values for configuring the ports:
// -------------------------------------------------------------
//   AND the existing shadow register value with 1's for the
//   bits that we don't want to change (leaving everything
//   else 0's) then OR it with 1's where we want 1's
//---------------------------------------------------------------


/*
  SPCR:  Slave Port Control Register (?)
      Use to enable/disable the I/O bus
      (does it need to stay enabled because other code assumes that it is
       always enabled?)
       ---<0x8C>---
           1: SMODE pins ignored
          00: Read: get SMODE pins state,  Write: (always write 00)
           0: PE7 used for /SCS (Slave Chip Select) (when slave port is enabled)
          11: Enable external I/O (Port A for data, B[7:2] for address)
          00: Slave port interrupts are disabled
*/
//--- Port A --- used as External I/O data bus (for Rabbit RIO chips) ---
//  PA7 - (xx) *
//  PA6 - (xx) *
//  PA5 - (xx) *
//  PA4 - (xx) *
//  PA3 - (xx) *
//  PA2 - (xx) *
//  PA1 - (xx) *
//  PA0 - (xx) *
  // Enable External I/O:
  //   Forces PA[7:0] to be External I/O Data bus
  //   Forces PB[7:2] to be External I/O Address bus
  outi(SPCR, srSPCR = 0x8C);

//--- Port B ---
//  PB7 - (xO)  IA5: Rabbit RIO: Global/Block
//  PB6 - (xO)  IA4:  (not used)
//  PB5 - (xO)  IA3: Rabbit RIO: Block Address 2
//  PB4 - (xO)  IA2: Rabbit RIO: Block Address 1
//  PB3 - (xO)  IA1: Rabbit RIO: Block Address 0
//  PB2 - (xO)  IA0: Rabbit RIO: Pointer/Indirect
//  PB1 - (?O)  Serial-A (Diag/Prog) clock
//  PB0 - (xO)  Serial-B (SPI) clock - provides Rabbit RIO CLK
  outi(PBDDR, srPBDDR |= 0xFF); // dir: OOOO-OOOO  (1111-1111)
// NOTE: see SPCR setting above

//--- Port C ---
//  PC7 - (xI)  Serial-A (Diag/Prog) receive
//  PC6 - (AO)  Serial-A (Diag/Prog) transmit
//  PC5 - (xI)  Serial-B (SPI) receive
//  PC4 - (AO)  Serial-B (SPI) transmit
//  PC3 - (xI)  Serial-C (RS-232) receive
//  PC2 - (AO)  Serial-C (RS-232) transmit
//  PC1 - (xI)  Serial-D (RS-232/485) receive
//  PC0 - (AO)  Serial-D (RS-232/485) transmit
  // (Input/Output modes are fixed for each signal)
  outi(PCFR, srPCFR |= 0x55);  // func: xAxA-xAxA (x1x1-0x1x1)

//--- Port D ---
//  PD7 - (xx)  Ethernet Controller (TPIN+)
//  PD6 - (xx)  Ethernet Controller (TPIN-)
//  PD5 - (NO)  to GSYNC on both Rabbit RIO chips
//  PD4 - (NO)  to /RESET on both Rabbit RIO chips
//  PD3 - (xx)  Ethernet Controller (TPOUT+)
//  PD2 - (xx)  Ethernet Controller (TPOUT-)
//  PD1 - (xx)  Not Avail (pulled high - not even connected???)
//  PD0 - (xx)  Ethernet Controller Reset
  outi(PDDDR, srPDDDR |= 0x30);  // dir:  xxOO-xxxx  (xx11-xxx)
  outi(PDFR, srPDFR &= 0xCF);    // func: xxNN-xxxx  (xx00-xxx)
  outi(PDDR, srPDDR = (srPDDR & ~0x20) | 0x10); // GYNC low, /RESET high

//--- Port E ---
//  PE7 - (AO)  to /CS (chip select) on Rabbit RIO chip (U2A)
//  PE6 - (NI)  /WAIT(1) from Rabbit RIO (U1A)
//  PE5 - (AI)  /INT from Rabbit RIO (U1A)
//  PE4 - (AI)  /INT from Rabbit RIO (U2A)
//  PE3 - (NI)  /WAIT(2) from Rabbit RIO (U2A)
//  PE2 - (xx)  Ethernet Controller Chip Select (output)
//  PE1 - (AO)  to /CS (chip select) on Rabbit RIO chip (U1A)
//  PE0 - (xx)  Interrupt Input 0A (Ethernet ctlr Interrupt)
  outi(PEDDR, srPEDDR = (srPEDDR & 0x05) | 0x82); // dir:  OIII-IxOx (1000-0x1x)
  outi(PEFR, srPEFR = (srPEFR & 0x05) | 0xB2);    // func: ANAA-NxAx (1011-0x1x)
  outi(PEDR, srPEDR |= 0x82);  // set /CS high for both RIO chips (1xxx-xx1x)

//--- Port F --- reads Device ID switches ---
//  PF7 - (NI)  LSB Device ID
//  PF6 - (NI)  --- Device ID
//  PF5 - (NI)  --- Device ID
//  PF4 - (NI)  --- Device ID
//  PF3 - (NI)  --- Device ID
//  PF2 - (NI)  --- Device ID
//  PF1 - (NI)  --- Device ID
//  PF0 - (NI)  MSB Device ID
  outi(PFDDR, srPFDDR = 0x00);  // dir:  IIII-IIII (0000-0000)
  outi(PFFR, srPFFR = 0x00);    // func: NNNN-NNNN (0000-0000)

//--- Port G --- used as generic digital outputs ---
//  PG7 - (NO)  RF Disable
//  PG6 - (NO)    (unused)
//  PG5 - (NO)  Activity LED
//  PG4 - (NO)    (unused)
//  PG3 - (NO)    (unused)
//  PG2 - (NO)    (unused)
//  PG1 - (NO)    (unused)
//  PG0 - (NO)    (unused)
  outi(PGDDR, srPGDDR = 0xFF);  // dir:  OOOO-OOOO (1111-1111)
  outi(PGFR, srPGFR = 0x00);    // func: NNNN-NNNN (0000-0000)
  outi(PGDR, srPDDR = 0x80);    // disable RF


//--- all parallel port outputs driven hi/lo (no open-drain outputs) ---
  outi(PDDCR, srPDDCR = 0x00);
  outi(PFDCR, srPFDCR = 0x00);
  outi(PGDCR, srPGDCR = 0x00);


//--- initialize state of digital outputs ---
  digitalOut = digOutState;  WriteDigOutputs(digitalOut);

//    ser485Rx();  // disable the RS485 transmitter
//    setExt485Rx();  // set external 2/4-wire RS-485 adapter to Rx mode

//    spiPort = NULL;  // set later by a call from the kernel process


//--- Generate a continuous clock pulse on PB0: freq = CLK/(n+1)/4 ---
//--- where n = 0-255, providing CLK/4 to CLK/1024
  // timer A5 divide by 8
  outi(TAT5R, srTAT5R=0);
  // start xmit, disable rcvr, internal sync clk, serial B int disabled
  outi(SBCR, srSBCR=0xAC);

//--- configure timing, etc for external I/O to blocks 1 and 7 ---
  outi(IB1CR, srIB1CR = 0x08);
  outi(IB7CR, srIB7CR = 0x08);
/*
  IB#CR:  I/O Bank # Control Register
      Uses PE# for stobe
      Ext I/O addr range covers 0x2000 for each block
        (block 1 ix 0x2000-0x3FFF, block 7 is 0xE000-0xFFFF)
      Use to set I/O timing, mode, etc
      Default mode:
        Chip Select with 15 wait statues
        Read-Only
        Active-low signaling
        Use external I/O bus
        ---<0x08>---
          7:6  00: 15 wait states
          5:4  00: Chip Select mode
            3   1: Writes allowed
            2   0: Active low
            1   0: Normal I/O read timing
            0   0: Use I/O bus if enabled

  PECR:  Sets clock used to drive port E output mode signals
         (peripheral clock or Timer A1, B1, or B2)

  PEALR:
  PEAHR:

  I0CR, I1CR:  Select Port E pin as an external interrupt
*/

//--- pulse PD4 low then high to reset the Rabbit RIO chips ---
  outi(PDDR, srPDDR &= ~0x10);
  outi(PDDR, srPDDR |=  0x10);

/* not really needed as the default mode is all inputs
//--- configure all Rabbit RIO pins as inputs ---
  for (extAddr = 0x2000; extAddr<0x200F; extAddr+=2)  {
    for (pin=0; pin<4; ++pin)  {
      // enable access to the Indirect Pin Control Register for the pin
      oute(extAddr, RIO_P0CR | pin);  // IR# ==> P#CR (Pin # Control Register)
      RIO_DELAY();
      // configure the pin as an input pin
      oute(extAddr+1, 0x0);
      RIO_DELAY();
    }
  }
*/

  flashSize = 524288L;

//  resetADC();

//optional - disable spectrum spreader to make debugging simpler (?)
//  outi(GCM1R, srGCM1R = 0);   //temp


  printf("\r\nInitialized %s hardware\r\n", ArchName);
}


//-----------------------------------------------------------------------------
//  Initialize and register hardware-dependent file systems
//-----------------------------------------------------------------------------
void registerFileSystems(void)
{
  socketInit_();
  srfsInit_();
  serialInit_();

  flashFileInit_();
}

//-----------------------------------------------------------------------------
//  Initialize the network hardware
//-----------------------------------------------------------------------------
void netHdwInit()
{
//  Select ethernet hardware:
//  usingAsix, usingDevKit, usingRealtek, usingSMSC
//  usingAll() can be used to support all cmd core modules, but req's more code

  usingAsix();
}


//-----------------------------------------------------------------------------
//  allocate any resources that should be owned by the kernel process
//
//  To be called ONLY by the kernel process!!!!
//-----------------------------------------------------------------------------
void hdwInit2(void)
{
//  fd_t  spifd;


//  spifd = _serial_(ADC_DAC_SPIPort, 0, SER_SPI, SER_NO_FLOW);

//  spiPort = _FileFromFD_(spifd)->ioDev;

//  printf("spiPort: %04lX (%d)\r\n", (ulong)spiPort, spiPort->port);  //tdebug
}

//-----------------------------------------------------------------------------
