  .comment |
----- clockTicks.asm ----- 11/06/06 -----(04/02/06)-----

  Read the 32 least significant bits from the Real Time Clock counter

  Note that the Rabbit uses a 48-bit ripple counter and that a ripple can
  be in progress at the time the counts are transfered to the holding register
  for reading.  To be sure this did not happen, we have to repeat the process
  until we get the same value twice in a row.
-------------------------------------------------------|
  .maclib c-rabbit
  .include  Rabbit.inc

  .FPROC  clockTicks
;
  ld  a,3     ; up to 3 times
;  .tstate on

..loop:
;--- Load the clock counter and read the 32 least significant bits ---
;--- Any write to RTC0R causes the counts to be copied to the 6-byte holding
;--- register.  Since a ripple can be in-progress when this is done, we
;--- have to do this multiple times until we get the same result twice in
;--- a row.
  ioi ld  (RTC0R),a  ; value written not important so just use A

  ioi ld  de,(RTC0R)  ; Read RTC0R and RTC1R: bits  0-15 (W1)
  ioi ld  bc,(RTC2R)  ; Read RTC2R and RTC3R: bits 16-31 (W2)

;--- load holding register again and see if the values changed ---
  ioi ld  (RTC0R),a ; Reload again
;  .tstate off   ; T-States for 2 stores to holding regs

  ioi ld  hl,(RTC0R)
  or  a
  sbc hl,de   ; Check W1
  jr  nz,..failed

  ioi ld  hl,(RTC2R)
  sbc hl,bc   ; Check W2
  jr  nz,..failed

;--- all 32-bits match - just return the value already in BCDE
  .EXIT

..failed:
  dec a   ; Failed one of the compares
  jr  nz,..loop
  .EXIT

  .ENDP
;
  .end

