//----- timerb.c ----- 02/15/12 -----(xx/xx/06)-----

//-----------------------------------------------------------------------------
//  TimerB interrupt handler.  Original version (used almost verbatim here)
//  from SHDesigns TimerB sample code.
//-----------------------------------------------------------------------------
//  Change log (most recent first)
//
//
//  2012-06-11 - Mark Davis
//               - _TimerMilliSeconds[1] and _TimerSeconds are now updated
//                 within _TimerBISR().
//               - Changed the name of timer_interrupt() to sched_tick().
//
//  2012-02-15 - Mark Davis
//               - Made timer_interrupt() a far func to free up some root
//                 memory space.
//
//  2010-02-27 - Mark Davis
//               - ipres is now done regardless of whether timer_interrupt()
//                 is called.  This is required because timer_interrupt()
//                 now reenables the interrupt just long enough to call
//                 schedule() (and not at all if it doesn't call schedule()).
//
//  2007-03-02 - Mark Davis
//               - initial use of slightly modified version of this code (?)
//
//-----------------------------------------------------------------------------

#include "dcdefs.h"
#include "TIMERB.h"
#include "Rabbit.h"

#include <sys/types.h>



#define XTAL_FREQ ((long)_serial19200TC * 307200l)

#if DEBUG_RST
  #define MIN_TIMER 200
#else
  #define MIN_TIMER 165
#endif

static int _timerb_inc;
// used by isr
static unsigned  int _timerb_ctr;
unsigned int  _timerb_scale; // made public for demo
static float  _timerb_divisor;
static int  _timerb_match = 0;

uchar  intTicks;

/*----------------------------------------------------------------------------
***** WARNING *****
  If this is changed at all, be sure to update the TimerBISR_PushCount macro
  in the header file!!!
***** WARNING *****
 -----------------------------------------------------------------------------
 --- This is what the stack look likes when we call sched_tick() ---

                ret_addr (LSB)  (addr inside _TimerBISR())
                ret_addr (MSB)

                XPC

                AF' (LSB)
                AF' (MSB)

                BC' (LSB)
                BC' (MSB)

                DE' (LSB)
                DE' (MSB)

                HL' (LSB)
                HL' (MSB)

                BC (LSB)
                BC (MSB)

                IY (LSB)
                IY (MSB)

                IX (LSB)
                IX (MSB)
           --------------------
                HL (LSB)
                HL (MSB)

                DE (LSB)
                DE (MSB)

                AF (LSB == F)
                AF (MSB == A)
*/

//-----------------------------------------------------------------------------
// *** WARNING ***
//     ANY changes to this will almost certainly require changes to the
//     dumpFault() and postMortem() functions in unistd.c!!!
// *** WARNING ***
//-----------------------------------------------------------------------------
void near _nostep _TimerBISR(void)
{
#asm
    push   af                    ;[7]
    push   de                    ;[7]
    push   hl                    ;[7]

    ld     hl,(__timerb_match)   ;[11]
    ld     a,h                   ;[4]
    or     0x3f  // will cause carry to bit 8 to propigate to bit 14  ;[4]
    ld     h,a                   ;[4]
    ld     de,(__timerb_inc)     ;[13]
    add    hl,de                 ;[2]
    ld     (__timerb_match),hl   ;[13]

    push   ip

    ipset  3
    ioi ld a, (TBCSR)     ; load B1, B2 interrupt flags (clears flag)
    ld     a,h
    ioi ld (TBM1R), a     ; NOTE:  you _need_ to reload the match
    ld     a,l            ; next low in a
    ioi ld (TBL1R), a     ; set up next B1 match (at timer=0000h)
                          ; register after every interrupt!
    pop    ip

; are we scaling int calls to user routine?
    ld     hl,(__timerb_scale)
    bool   hl
    jr     z,no_scale

; ok so we need to count down
    ld     hl,(__timerb_ctr)
    dec    hl
    ld     (__timerb_ctr),hl
    bool   hl
    jr     nz,skip_user

    ld     hl,(__timerb_scale)
    ld     (__timerb_ctr),hl

no_scale:
;  ++_TimerMilliSeconds[1]
    ld     hl,__TimerMilliSeconds##+4
    inc    (hl)
    jr     nz,..x

    inc    hl
    inc    (hl)
    jr     nz,..x

    inc    hl
    inc    (hl)
    jr     nz,..x

    inc    hl
    inc    (hl)
..x:

;  if (++milliTicks == 1000)  { ++_TimerSeconds;  milliTicks = 0; }
    ld     hl,(_milliTicks##)
    inc    hl
    ld     (_milliTicks##),hl
    ld     de,0x3e8
    or     a,a
    sbc    hl,de
    jr     nz,..endSecChk

    ld     hl,__TimerSeconds##
    inc    (hl)
    jr     nz,..y

    inc    hl
    inc    (hl)
    jr     nz,..y

    inc    hl
    inc    (hl)
    jr     nz,..y

    inc    hl
    inc    (hl)
 ..y:

    bool    hl
    ld      l,h
    ld      (_milliTicks##),hl
..endSecChk:

;//--- Are we due to invoke the scheduler?  If not, nothing more to do ---
;  if (++intTicks < SchedMS)  return;
    ld      a,(_intTicks)
    inc     a
    ld      (_intTicks),a
    bool    hl
    ld      l,a
    ld      de,(_SchedMS##)
    or      a,a
    sbc     hl,de
    jp      c,skip_user

;  intTicks = 0;
    xor     a,a
    ld      (_intTicks),a


;;===== 158 cycles to call sched_tick() =====
    push   ix                    ;[12]
    push   iy                    ;[12]
    push   bc                    ;[10]

    exx                          ;[2]
    push   hl                    ;[10]
    push   de                    ;[10]
    push   bc                    ;[10]
    exx                          ;[2]

    ex     af,af'                ;[2]
    push   af                    ;[10]
    ex     af,af'                ;[2]

;;  call   _sched_tick##      ; do external func
    lcall   $sched_tick##   ;[19]   ; do external func

    ex     af,af'                ;[2]
    pop    af                    ;[7]
    ex     af,af'                ;[2]

    exx                          ;[2]
    pop    bc                    ;[7]
    pop    de                    ;[7]
    pop    hl                    ;[7]
    exx                          ;[2]

    pop    bc                    ;[7]
    pop    iy                    ;[7]
    pop    ix                    ;[7]
;;===========================================================

skip_user:
    pop    hl
    pop    de
    pop    af

    ipres

#endasm
}


//-----------------------------------------------------------------------------
int _nostep select_timer(unsigned long usec, char intlevel)
{
  static char  timera_mod = 0;
  unsigned long  divisor;
  int  timeradiv;
  unsigned long  intscale;

  _timerb_match = 0;
  ini(TBCSR);             // clear any pending int
  ioSrOutI(TBCSR, 0x00);  // disable timer B and its interrupts
  ioSrOutI(TBCR, 0x04);   // disable timer B and its interrupts (use timer A clock)
  ini(TBCSR);             // clear any pending int
  _timerb_scale = 0;      // no scale be default

  // if serial ports are using timer A1 we can't modify it.
  timeradiv = (int)srTAT1R + 1;
  if (timeradiv == 1)  timera_mod = 1; // we can change it
  if (timera_mod)  timeradiv = 256;  // use max as default

  intscale = (unsigned long) // divisor is # of 1024 count timerb rollovers
     ((float)usec * (float)XTAL_FREQ / 1.024e9);

  if (intscale < timeradiv) {  // max for using both timer A and B
    divisor = (unsigned long) (((float)usec * (float)XTAL_FREQ) / 1.0e6+0.5);
    if (intscale == 0L)   {  // timer b only
      intlevel &= 3;    // mask valid
      _timerb_divisor = divisor;
      // not enough instrs for accurate time between ints?
      if (divisor < MIN_TIMER)  return 1;
    }
    else  if (intscale < 8L)  { // TimerB clk/16
      intlevel = 0x8 | (intlevel & 3);    // mask valid set pclk/2
      divisor = (unsigned long) (((float)usec * (float)XTAL_FREQ) / 8.0e6+0.5);
      _timerb_divisor = divisor * 8L;
    }
    else  {  // use TimerA
      if (timera_mod)  { // this is unused so we can modify it
        timeradiv = (int)((divisor + 1023L) / 1024L);
        ioSrOutI(TAT1R, timeradiv - 1);  // use new val
      }
      intlevel = 0x4 | (intlevel & 3);    // mask valid set timera
      // try to calc best value
      divisor = (divisor + (timeradiv>>1)) /timeradiv; // nearest mult of timeradiv
      _timerb_divisor = divisor * timeradiv;
    }
  }
  else {  // TimerA + TimerB + timerscale
    intlevel = 0x4 | (intlevel & 3);  // mask valid set TimerA
    ioSrOutI(TAT1R, timeradiv - 1);   // use new val

    intscale = (unsigned long)
            (((float)usec * (float)XTAL_FREQ) / (timeradiv * 1.024e9) + 1.0);
    if (intscale > 65535L)  return 1;  // max for using both timer A & B and scale

    divisor = (unsigned long)
      (((float)usec * (float)XTAL_FREQ) / (timeradiv * 1.0e6 * intscale) + 0.5);

    _timerb_scale = (unsigned int) intscale;  // init first

#ifdef TIMERB_IMMEDIATE
    _timerb_ctr = 1;  // call isr on first int
#else
    _timerb_ctr = _timerb_scale;
    if (divisor != 1024)  _timerb_ctr++;
#endif

    _timerb_divisor = (float)divisor * (float)intscale * (float)timeradiv;
  }

  _timerb_inc = (int)((divisor & 0xff) | ((divisor << 6) & 0xc000));  // val for isr

//  ipset3();

#asm
  ioi ld  a,(TBCLR)
  ld      l,a
  ioi ld  a,(TBCMR)
  or      0x3f
  ld      h,a
  ld      de,(__timerb_inc)
  add     hl,de
  ld      (__timerb_match),hl
  ld      a,h
  ioi ld  (TBM1R),a
  ld      a,l
  ioi ld  (TBL1R),a
  ld      a,0x3
  ioi ld  (TBCSR),a
  ld      (_srTBCSR),a
#endasm

  setInterruptHandler(INT_TIMER_B, _TimerBISR);    // set up ISR
  ioSrOutI(TBCR, intlevel);  // clock timer B with clock source and int level

//  ipres();

#ifdef TIMER_DEBUG
  printf("Timer B divisor=%ld, _timerb_scale=%u, timeradiv=%d, match=%x\n",
          divisor, _timerb_scale, timeradiv, _timerb_match);
#endif

  return 0; // :-)
}

//-----------------------------------------------------------------------------
float TimerBRate()
{
  return (_timerb_divisor * 1000000.0) / ((float) XTAL_FREQ);
}


//-----------------------------------------------------------------------------
void TimerBUninit(void)
{
  outi(TBCSR, srTBCSR = 0x00);  // disable timer B and its interrupts
}

//-----------------------------------------------------------------------------
