//----- SHD_DLM.h ----- 03/12/09 -----(xx/xx/xx)-----

//----- original code from SHDesigns -----

//--- additions, modifications, and reformatting by Mark Davis, NSCL ---


/*** BeginHeader  ********************************************/
#ifndef __SHD_DLM
#define __SHD_DLM
/*** EndHeader ***********************************************/


#include <DLMiii.h>


#define DNLD_UDP_PORT  2000  // arbitrary

#define  DOPEN_FILES   4  // max # of flash files open at once

typedef struct {
  unsigned char  cmd, status;
  unsigned short length;  // data size
  unsigned long  address;  // write address
  unsigned char  data[ETH_MTU-8];  // can be up to 512 bytes.
} UDP_DBG_HEADER;

#define UDP_DBG_HEADER_SIZE (sizeof(UDP_DBG_HEADER)-ETH_MTU+8)
#define FRAG_MASK  0xff20

// command definitions
#define UDPDNLD_QUERY            1 // query for all boards (broadcast) (old DLM)
#define UDPDNLD_DOWNLOAD_RAM     2  // download RAM code
#define UDPDNLD_XMEM_SIZE        3   // xmem size needed for RAM code
#define UDPDNLD_DOWNLOAD_FLASH   4  // download FLASH code (data len=0 when done)
#define UDPDNLD_RUN              5   // run download RAM code
#define UDPDNLD_REBOOT           6  // restart with new code
#define UDPDNLD_SET_IP           7  // set the board to diff IP
#define UDPDNLD_SET_SPECIFIC_IP  8 // set the board to diff IP
#define UDPDNLD_GET_USERBLOCK    9
#define UDPDNLD_READ            10   // read memory
#define UDPDNLD_FLUSH           11  // flush last block
#define UDPDNLD_QUERY3          12 // query for all boards (broadcast)
#define UDPDNLD_SET_RTC         15
#define UDPDNLD_GET_PARAMS      16
#define UDPDNLD_NULL           127  // do nothing command to resolve ARP

// query status
#define UDPDNLD_STATUS_NULL        0 // not set
#define UDPDNLD_STATUS_NEED_CODE   1 // board needs code downloaded to RAM
#define UDPDNLD_STATUS_HAS_CODE    2 // board has co
#define UDPDNLD_STATUS_RAM_CODE    3 // ram code already running
#define UDPDNLD_STATUS_NOMEM       4 // no xmem available
#define UDPDNLD_STATUS_REBOOT      5 // rebooting
#define UDPDNLD_STATUS_USER_CODE  11 // running user code

// download status
#define UDPDNLD_STATUS_ACK               6 // download acknowledge
#define UDPDNLD_STATUS_NACK              7 // download fail acknowledge
#define UDPDNLD_STATUS_SEQUENCE          8 // dld sequence error
#define UDPDNLD_STATUS_RAM_CODE_IN_XMEM  9 // RAM loader in flash
#define UDPDNLD_STATUS_FRAG             10  // fragmented packet received

#define UDPDNLD_MODE_RUN_DLM      0
#define UDPDNLD_MODE_RESTART      1
#define UDPDNLD_MODE_RUN_BACKUP   2
#define UDPDNLD_MODE_RUN_PRIMARY  3
#define UDPDNLD_MODE_RUN_OK      -1


typedef struct {
  unsigned entry_num; // file entry #
  unsigned block_num;
  unsigned long pos;
  char far * sector_buff;
  char far * buff_ptr;
  int flags;
} FILE_DATA;

//--- list of open references to files ---
//--- The ffile "handles" are indexes in to this list ---
extern FILE_DATA  _ffile_list[DOPEN_FILES];


#define FFLAG_OPEN     0x0001
#define FFLAG_WRITE    0x0002
#define FFLAG_UPDATED  0x0004

#define FERR_NOTFOUND      -1
#define FERR_TOOMANYFILES  -2
#define FERR_NOXMEM        -3
#define FERR_FILE_EXISTS   -4
#define FERR_SEEK          -5

#define FSEEK_SET  0
#define FSEEK_CUR  1
#define FSEEK_END  2

int  DLM_Init(char * str);
int  DLM_Tick(void);
void DLM_SetPort(unsigned port);
int  DLM_Reboot(int mode);


//
// file I/O functions
//

// open existing file for reading/writing
int ffile_open(char * name);

// read from file, returns byte read
unsigned  ffile_read(int handle, char far * buff, unsigned bytes);

// write to file, returns bytes written
unsigned  ffile_write(int handle, char far * buff, unsigned bytes);

// create a new file for writing
int ffile_create(char * name);

// delete a file
int ffile_delete(char * name);
int ffile_delete_index(int index);

// return current file position
long ffile_tell(int handle);

// seek within file
// returns 0 if no error, FERR_SEEK if beyond file
int ffile_seek(int handle, long pos, int mode);

// close a file opend for read or write
void ffile_close(int handle);

// get entry number for a file
int ffile_index(char * name);

// set entry number as primary or backup
int ffile_setPrimary(int index);
int ffile_setBackup(int index);

// return free space in flash
long int  ffile_free(void);

// create a link to a FAT file, returns index
int ffile_FATLink(char * fname);


int ffile_getPrimary(void);
int ffile_getBackup(void);

// dl_flags bits
#define DL_RUN_PENDING  0x01
#define DL_SOCKET_OPEN  0x04



//----- NSCL ----- added following funcs -----

#include <sys/types.h>

FILE_ENTRY far *ffile_info(unsigned idx);
int  ffile_fstat(int handle, struct stat *fsbuf);

long ffile_unread(int handle);

unsigned long  getIPAddr(void);
int  setIPAddr(unsigned long ipAddr);

char *ffile_getPrimaryName(char *name, int nameLen);
char *ffile_getBackupName(char *name, int nameLen);

int ffile_rename(char const *oldpath, const char *newpath);

int ffile_makePrimary(const char *fileName);
int ffile_makeBackup(const char *fileName);

int  ffile_utime(const char *fileName, const struct utimbuf *buf);

int   ffile_setName(char *newName);
char *ffile_getName(char *name, int maxLen);

int ffile_fsck(bool repair);



#endif
