//--- limits.h --- 06/06/12 ---(11/14/07)---

#ifndef  _LimitsH_
#define  _LimitsH_


//-----------------------------------------------------------------------------
//  Update log (most recent first):
//
//
//  2012-06-06 - Mark Davis
//               - Added MaxListenTCP and MaxTCPCons
//
//  2012-01-11 - Mark Davis
//               - Increased WQ_POOL_SIZE from 20 to 24
//
//  2010-03-02 - Mark Davis
//               - Increased MaxUDPSockets to 4
//
//  2010-02-24 - Mark Davis
//               - Increased PATH_MAX to 64 (from 48)
//
//  2009-03-24 - Mark Davis
//               - increased MaxSRFSFiles from 5 to 6 to try to minimize the
//                 # of instances where code fails because it cannot open
//                 a connection to the SRFS server
//
//-----------------------------------------------------------------------------

//--- names commonly used in linux source ---

#define NR_OPEN           16  /* max # of open files for a single process */

#define NR_FILES          64  /* max # of open files for ALL processes */


#define NAME_MAX          24  /* max # chars in a file name incl nul*/
#define PATH_MAX          64  /* max # chars in a path name including nul */

//#define PIPE_BUF        1024  /* max # bytes in atomic write to a pipe */

#define ARG_MAX          128  /* max # bytes of args + environ for exec() */

//#define NGROUPS_MAX    65536  /* supplemental group IDs are available */
//#define CHILD_MAX        999    /* no limit :-) */
//#define LINK_MAX         127  /* # links a file may have */
//#define MAX_CANON        255  /* size of the canonical input queue */
//#define MAX_INPUT        255  /* size of the type-ahead buffer */
//#define XATTR_NAME_MAX   255  /* # chars in an extended attribute name */
//#define XATTR_SIZE_MAX 65536  /* size of an extended attribute value (64k) */
//#define XATTR_LIST_MAX 65536  /* size of extended attribute namelist (64k) */

//#define RTSIG_MAX   32


#define OPEN_MAX     NR_OPEN

#define FD_SETSIZE   NR_OPEN




//--- additional names not commonly used in linux ---

#define MAX_FILE_SYS  8  // max # of file systems


// socket limits for all processes
#define  MaxListenTCP     6   // Max # of TCP Listening sockets
#define  MaxTCPCons      12   // max established cons (1-127)
#define  MaxTCPSockets   (MaxTCPCons + MaxListenTCP)  // max incl listeners

#define  MaxUDPSockets    4   // max for all processes (1-127)


#define  MaxOpenFiles   NR_FILES

#define  StdOutBufSize       128  // size of stdout buffer for each process

#define  StacksPer4K           3  // how many stacks to fit in each 4K block


//=== settings to control # of resources in kernel ===
// Max NUM_TASKS = 16 * StacksPer4K (and is also limited by root memory space)
//-----------------------------
#define NUM_TASKS           20
//-----------------------------
#define  TaskIdxMask    0x001f  // bits in pid comprising index into tasks[]

//--- stacks must be <= 4096 and cannot cross 4K boundary ---
//--- NOTE that task 0 (the kernel task) will always have a 4K stack (cannot ---
//---      change this), and all other tasks will have the size given here ---
#define  STACK_SIZE       ((4096/StacksPer4K) & 0xFFE)

#define  NUM_PRIORITIES     16  // # of distinct priorities

#define  MaxArgBytes   ARG_MAX  // max length of cmd line arguments


#define  MaxSRFSFiles        6  // max # SRFS connections for all processes


#define  MaxFIFOs            8

#define  FIFOSize         1024


//#define NR_FIFO            6  // max # of unique FIFO's for all processes

//#define FIFO_BUF    PIPE_BUF  // max # bytes in atomic write to a FIFO

#define  WQ_POOL_SIZE       24  // max # wait queues in use by all tasks

#define  RLIMIT_NOFILE       8  // max # of fds one task can block on


#endif


