//----- signal.h ----- 04/30/07 -----(03/28/06)-----

#ifndef _signalH_
#define _signalH_


#define  SIGHUP      1  // Terminate   Hangup (see termio(7I))
#define  SIGINT      2  // Terminate   Interrupt (see termio(7I))
#define  SIGQUIT     3  // Dump        Quit (see termio(7I))
#define  SIGILL      4  // Dump        Illegal Instruction
#define  SIGTRAP     5  // Dump        Trace or Breakpoint Trap
#define  SIGABRT     6  // Dump        Abnormal termination
#define  SIGIOT      6  // Dump        (same as SIGABRT)
#define  SIGBUS      7  // Dump        Bus Error
#define  SIGFPE      8  // Dump        Floating Point Exception
#define  SIGKILL     9  // Terminate   Forced process termination
#define  SIGUSR1    10  // Terminate   User Signal 1
#define  SIGSEGV    11  // Dump        Invalid memory reference
#define  SIGUSR2    12  // Terminate   User Signal 2
#define  SIGPIPE    13  // Terminate   Write to pipe with no readers
#define  SIGALRM    14  // Terminate   Real-timer/Alarm Clock
#define  SIGTERM    15  // Terminate   Process Termination
#define  SIGSTKFLT  16  // Terminate   Stack Fault
#define  SIGCHLD    17  // Ignore      Child process stopped or terminated
#define  SIGCONT    18  // Continue    Continued
#define  SIGSTOP    19  // Stop        Stop process execution
#define  SIGTSTP    20  // Stop        Stop issued from tty (see termio)
#define  SIGTTIN    21  // Stop        Bkgnd proc needs tty input (see termio)
#define  SIGTTOU    22  // Stop        Bkgnd proc needs tty output (see termio)
#define  SIGURG     23  // Ignore      Urgent condition on Socket
#define  SIGXCPU    24  // Dump        CPU time limit exceeded
#define  SIGXFSZ    25  // Dump        File size limit exceeded
#define  SIGVTALRM  26  // Terminate   Virtual Timer Expired
#define  SIGPROF    27  // Terminate   Profile timer clock
#define  SIGWINCH   28  // Ignore      Window Size Change
#define  SIGIO      29  // Terminate   I/O now possible
#define  SIGPOLL    29  // Terminate   (Pollable Event - same as SIGPIO)
#define  SIGPWR     30  // Terminate   Power Fail or Restart
#define  SIGSYS     31  // Dump        Bad System Call
#define  SIGUNUSED  31  // Dump        (same as SIGUNUSED)



typedef unsigned long  sigset_t;

typedef void (*sighandler_t)(int);

struct sigaction {
    sighandler_t   sa_handler;
    unsigned long  sa_flags;
    void           (*sa_restorer)(void);  /* osbolete */
    sigset_t       sa_mask;   /* mask last for extensibility */
};
typedef  struct sigaction  sigaction_t;


#define sigmask(sig)  (1UL << ((sig) - 1))


int sigemptyset(sigset_t *set);

int sigfillset(sigset_t *set);

int sigaddset(sigset_t *set, int signum);

int sigdelset(sigset_t *set, int signum);

int sigismember(const sigset_t *set, int signum);



sighandler_t signal(int signum, sighandler_t handler);


int sigaction(int signum, const struct sigaction *act,
              struct sigaction *oldact);

int sigprocmask(int how, const sigset_t *set, sigset_t *oldset);

//--- values for how ---
#define  SIG_BLOCK     1  // add to existing set of blocked signals
#define  SIG_UNBLOCK   2  // remove from existing set of blocked signals
#define  SIG_SETMASK   3  // replace the existing set of blocked signals


int sigpending(sigset_t *set);

int sigsuspend(const sigset_t *mask);


#endif
