//----- backup.h ----- 03/30/10 -----(11/02/06)-----

#ifndef _backupH_
#define _backupH_


//finish:  For future use, I will probably want to include a pointer to
//         a func to GET a value, as an alternative to a pointer to the
//         actual value.  The parameters to the existing PERSISTENT_ macros
//         can stay the same, and I can just add another for values that
//         provide a GET func rather than the addr of the data.

typedef void (*RestoreVal)(void far *to, char *from, uint count);

typedef   struct {
  short      id;     // unique ID within a file of persistent values
  short      size;   // total # of bytes
  short      gran;   // granularity: safe subset of size to read: 0 means all
  void far  *data;   // ptr to where data is stored
  char far  *name;   // name of value
  RestoreVal restoreVal;  // to do actual restore handling for special cases
} ParamData;              // use RESTORE_FUNC(), PVRF(), and NEW_VAL() macros!

#define END_PARAM_DATA  { 0, 0, 0, NULL, NULL }


//--- struct at beginning of block pointed to by ParamFile.bbmCache ---
typedef struct {
  ulong   time;     // time the BBRAM cache was last updated
  uint    size;     // # of bytes that follow this structure
  uint    devType;  // devType from code that wrote to the cache
  uint    crc;      // 16-bit CRC of the bytes following this structure
} BBM_Cache_Hdr;


typedef struct {
  char           *baseName;        // base name of file data is stored in
  uchar           category;        // category of data (see below)
  const ParamData far  *params;    // pointer to data to be saved/restored
  uint            writeCount[16];  // to track # of writes over last 16 minutes
  fd_t            fd;              // fd opened for the file
  BBM_Cache_Hdr far *bbmCache;     // copy stored in Battery-Backed Memory
  uint            cacheSize;       // # bytes in cache NOT incl BBM_Cache_Hdr
  uint            offset;          // offset for next read/write
} ParamFile;


//--- values for ParamdData ParamFiles.category ---
#define  PDC_SBC    1  // SBC-specfic data (e.g. ADC, DAC calib)
#define  PDC_CTLR   2  // ctlr-specific (e.g. BCM FrontEnd board calib)
#define  PDC_DEV    3  // device specific (e.g. gain/offset for Power Supply)
#define  PDC_SET    4  // settings that should be restored during startup
#define  PDC_STATE  5  // for restoring state of ctlr following a "soft" reset



// Macros for simpler declaring/reading of ParamData list elements

//NOTE:  The "gran"ularity parameter for the _TBL macros indicates the
//       smallest amount that should be read at a time to avoid corruption
//       to individual elements of a table

#define  FCS  (const unsigned far char *)

#define  PERSISTENT_VAR(id, var, func)  \
        { (id), sizeof(var),0,  VFPV(var), FCS#var, func  }

#define  PERSISTENT_VARx(id, var, func)  \
        { (id), sizeof(var),64,  VFPV(var), FCS#var, func  }

#define  PERSISTENT_TBL(id, tbl, gran, func)  \
        { (id), sizeof(tbl),(gran), VFP(tbl), FCS#tbl, func  }

#define  PERSISTENT_TBLN(id, typ,tbl, gran, func)  \
        { (id), sizeof(tbl),(gran), VFPN(tbl,typ), FCS#tbl, func  }


#define NEW_VAL(type)  ( *((type *)newVal) )

#define  RESTORE_FUNC(name)  \
  _nearcall void name##Restore(void far *curVal, char *newVal, uint count)

#define  PVRF(name)  name##Restore


//--- declaration only: actual definition must be in the application code ---
extern const ParamData far  hdwCfgParams[];

extern ParamFile *paramFiles[];
extern const int NumParamFiles;


int initParamModule(char *buf, uint bufSize);

int saveParams(char *buf, uint bufSize, ParamFile *paramFile, bool forceFlush);
//--- values for the "forceFlush" parameter ---
#define  ForceFlush   true
#define  DontFlush    false

int loadParams(char *buf, uint bufSize, ParamFile *paramFile, bool justCompare);
//--- values for the "justCompare" parameter ---
#define  JustCompare  true
#define  Reload       false


void loadAllParams(char *buf, uint bufSize);

void saveAllParams(char *buf, uint bufSize);
void flushAllParams(char *buf, uint bufSize);


int loadAllParamsMsg(void);
int loadParamsMsg(ParamFile *paramFile);



void _task backupd(int argc, char **argv);



#endif

