//----- list.c ----- 04/30/10 -----(04/19/07)-----

#include <linux/list.h>

//-----------------------------------------------------------------------------
//  DEBROS implementations of standard Linux doubly-linked list functions
//-----------------------------------------------------------------------------
//  Update log (most recent first)
//
//
//  2010-04-29 - Mark Davis
//               - Cleaned up a bit and changed __list_del() to a macro.
//
//  2007-04-23 - Mark Davis
//               - original version
//
//-----------------------------------------------------------------------------


/*---------------------------------------------------------------------------*
 * Insert a new entry between two known consecutive entries.
 *
 * This is only for internal list manipulation where we know
 * the prev/next entries already!
 *---------------------------------------------------------------------------*/
void __list_add(list_head_t *new_, list_head_t *prev_, list_head_t *next_)
{
  next_->prev = new_;
  new_->next = next_;
  new_->prev = prev_;
  prev_->next = new_;
}


/*---------------------------------------------------------------------------*
 * list_empty_careful - tests whether a list is empty and not being modified
 * @head: the list to test
 *
 * Description:
 * tests whether a list is empty _and_ checks that no other CPU might be
 * in the process of modifying either member (next or prev)
 *
 * NOTE: using list_empty_careful() without synchronization
 * can only be safe if the only activity that can happen
 * to the list entry is list_del_init(). Eg. it cannot be used
 * if another CPU could re-list_add() it.
 *---------------------------------------------------------------------------*/
int list_empty_careful(list_head_t *head)
{
  list_head_t *next = head->next;
  return (next == head) && (next == head->prev);
}


/*---------------------------------------------------------------------------*
 * list_splice - join two lists
 * @list: the new list to add.
 * @head: the place to add it in the first list.
 *---------------------------------------------------------------------------*/
void list_splice(list_head_t *list, list_head_t *head)
{
  list_head_t *first = list->next;

  if (first != list) {
    list_head_t *last = list->prev;
    list_head_t *at = head->next;

    first->prev = head;
    head->next = first;

    last->next = at;
    at->prev = last;
  }
}


