//----- bootimg.c ----- 03/20/09 -----(03/14/06)-----

//-----------------------------------------------------------------------------
//
//  2009-xx-xx - Mark Davis
//               - original version of ...
//
//-----------------------------------------------------------------------------

#include <time.h>
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <utime.h>


#include <busybox.h>

#include <sys/hdwDefs.h>

#include <dcdefs.h>

#include <ctype.h>
#include <limits.h>

#include <sys/SHD_headers.h>

#include <applic.h>

#include <sys/farprintf.h>


//-----------------------------------------------------------------------------
//  Get (and optionally display) the names of the files containing boot images
//
//  Returns a copy of the file names that contain the primary and backup
//  boot images (if they exist)
//
//  (also used in update.c)
//-----------------------------------------------------------------------------
void bootFileNames(char *primaryName, char *backupName, bool showNames)
{
  char  temp[32];


//--- get names of files with primary and backup version of firmware ---
  *primaryName = *backupName = 0;
  ffile_getPrimaryName(primaryName, PATH_MAX);
  ffile_getBackupName(backupName, PATH_MAX);

//--- show current state ---
  if (showNames)  {
    if (*primaryName)  {
      fileInfoStr(primaryName, temp);
      printf("Primary:  %s %s  %s\r\n",
              temp, primaryName, (runningPrimary ? "(active)" : "") );
    }
    if (*backupName)  {
      fileInfoStr(backupName, temp);
      printf("Backup:   %s %s  %s\r\n",
              temp, backupName, (runningPrimary ? "" : "(active)") );
    }
  }

}


//-----------------------------------------------------------------------------
//  Show or Set the Primary or Backup boot image file
//-----------------------------------------------------------------------------
void _task _bootimg(int argc, char **argv)
{
  bool  setPrimary, setBackup;
  char  primaryName[PATH_MAX], backupName[PATH_MAX];


  if (argc > 1)  if (showHelp(3, argc, argv))  {
    printf("\nusage: bootimg primary|backup file\r\n");  Exit(0);  }

  if (argc > 2)  {
    setPrimary = (strcmp(argv[1], "primary") == 0);
    setBackup = (strcmp(argv[1], "backup") == 0);

    if (!setPrimary AND !setBackup)  {
      printf("\nusage: bootimg primary|backup file\r\n");  Exit(-1);  }

    if (setPrimary)
      if (ffile_makePrimary(argv[2]))  { perror(argv[2]);  Exit(-2); }

    if (setBackup)
      if (ffile_makeBackup(argv[2]))  { perror(argv[2]);  Exit(-3); }
  }

  bootFileNames(primaryName, backupName, true);

  Exit(0);
}

//-----------------------------------------------------------------------------
