//----- float.c ----- 06/15/12 -----(03/14/06)-----

//-----------------------------------------------------------------------------
//  Update log (most recent first):
//
//
//  2012-02-22 - Mark Davis
//               - Added timing test for long (32-bit) integer additions and
//                 subtractions.
//
//  2009-xx-xx - Mark Davis
//               - original version of ...
//
//-----------------------------------------------------------------------------

#include <time.h>
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <utime.h>


#include <busybox.h>

#include <sys/hdwDefs.h>

#include <dcdefs.h>

#include <ctype.h>
#include <limits.h>

#include <sys/SHD_headers.h>

#include <applic.h>

#include <sys/farprintf.h>


#pragma warn -757

//-----------------------------------------------------------------------------
//  Test algorithm for fast long division (currently only works for unsigned
//  values).
//-----------------------------------------------------------------------------
ulong  uldiv_(ulong num, ulong div)
{
  ulong  result = 0, mult = 1;


  if (!num)  return 0;
  if (!div)  return 0xFFFFFFFFL;

  while ((div ^ num) > div)  { div <<= 1;  mult <<= 1; }

  for (;;)  {
    while (div > num)  {
      if (!(mult >>= 1))  return result;
      div >>= 1;
    }
    if (!div)  return result;
    result += mult;  num = num - div;
    if (!(mult >>= 1))  return result;
    div >>= 1;
  }

//  printf("uldiv_() error:\r\n"
//         "  num: %08lX\r\n"
//         "  div: %08lX\r\n"
//         "  mult: %08lX\r\n"
//         "  result: %08lX\r\n",
//         num, div, mult, result );

//  return 0;
}


//-----------------------------------------------------------------------------
void _task _float(int argc, char **argv)
{
  uint  ui, startTime, ttlms;
  short  i1, i2, i3;
  long  l1, l2, l3;
  ulong  ul1, ul2, ul3;
  float f1, f2, f3;
  char  temp1[16], temp2[16], temp3[16];


  if (showHelp(1, argc, argv))  {
    printf("\nusage:  float [[[f [f]] [l]] [l]]\r\n");  Exit(-1); }

  f1 = (argc > 1) ? atof(argv[1]) : 0.0001234567;
  f2 = (argc > 2) ? atof(argv[2]) : -765.4321;

//--- testing floating point operations, compiler lib and fast-int-mult ---
  l1 = *((long *)&f1);  l2 = *((long *)&f2);

  printf("\r\n %s [%08lX] * %s [%08lX]\r\n",
         Engineering(f1, 7, temp1), l1,  Engineering(f2, 7, temp2), l2 );

  f3 = f1 * f2;  l3 = *((long *)&f3);
  printf(" __fmlt:  %s [%08lX]\r\n", Engineering(f3, 7, temp3), l3 );

  f3 = fmult(f1, f2);  l3 = *((long *)&f3);
  printf("  fmult:  %s [%08lX]\r\n", Engineering(f3, 7, temp3), l3 );


  printf("\r\nAprox times microseconds:\r\n");

//--- test speed of floating point operations ---
  printf("\r\n floats:\r\n");

  KLOCK();
  startTime = GetMS_();
  for (ui=0; ui<1000; ++ui)  f3 = f1 * f2;
  ttlms = msSince_(startTime);
  KUNLOCK();
  printf("   __fmlt: %3u\r\n", ttlms);

  KLOCK();
  startTime = GetMS_();
  for (ui=0; ui<1000; ++ui)  f3 = f1 / f2;
  ttlms = msSince_(startTime);
  KUNLOCK();
  printf("   __fdiv: %3u\r\n", ttlms);

  KLOCK();
  startTime = GetMS_();
  for (ui=0; ui<1000; ++ui)  f3 = f1 + f2;
  ttlms = msSince_(startTime);
  KUNLOCK();
  printf("   __fadd: %3u\r\n", ttlms);

  KLOCK();
  startTime = GetMS_();
  for (ui=0; ui<1000; ++ui)  f3 = f1 - f2;
  ttlms = msSince_(startTime);
  KUNLOCK();
  printf("   __fsub: %3u\r\n", ttlms);


//--- test speed of long int operations ---
  printf("\r\n long ints:\r\n");

  l1 = (argc > 1) ? atol(argv[1]) : 123456L;
  l2 = (argc > 2) ? atol(argv[2]) : 78;

  ul1 = *((ulong *)&l1);  ul2 = *((ulong *)&l2);

  KLOCK();
  startTime = GetMS_();
  for (ui=0; ui<1000; ++ui)  l3 = l1 * l2;
  ttlms = msSince_(startTime);
  KUNLOCK();
  printf("  __ulmlt: %3u\r\n", ttlms);

  KLOCK();
  startTime = GetMS_();
  for (ui=0; ui<1000; ++ui)  l3 = l1 / l2;
  ttlms = msSince_(startTime);
  KUNLOCK();
  printf("   __ldiv: %3u\r\n", ttlms);

  KLOCK();
  startTime = GetMS_();
  for (ui=0; ui<1000; ++ui)  l3 = l1 + l2;
  ttlms = msSince_(startTime);
  KUNLOCK();
  printf("   __ladd: %3u\r\n", ttlms);

  KLOCK();
  startTime = GetMS_();
  for (ui=0; ui<1000; ++ui)  l3 = l1 - l2;
  ttlms = msSince_(startTime);
  KUNLOCK();
  printf("   __lsub: %3u\r\n", ttlms);


//--- test speed of short int mult and div ---
  printf("\r\n short ints:\r\n");

  i1 = (int) l1;  i2 = (int) l2;

  KLOCK();
  startTime = GetMS_();
  for (ui=0; ui<1000; ++ui)  i3 = i1 * i2;
  ttlms = msSince_(startTime);
  KUNLOCK();
  printf("      mul: %3u\r\n", ttlms);

  KLOCK();
  startTime = GetMS_();
  for (ui=0; ui<1000; ++ui)  i3 = i1 / i2;
  ttlms = msSince_(startTime);
  KUNLOCK();
  printf("   __idiv: %3u\r\n", ttlms);


//--- test speed/correctness of fast long div ---
  printf("\r\n test fast ulong div: [%08lX/%08lX]\r\n", ul1, ul2);
  fflush(stdout);
  KLOCK();
  startTime = GetMS_();
  for (ui=0; ui<500; ++ui)  ul3 = uldiv_(ul1, ul2);
  ttlms = msSince_(startTime) << 1;
  KUNLOCK();
  printf("   uldiv_: %3u (%lu/%lu)\r\n", ttlms, ul3, ul1/ul2);

  Exit(0);
}

#pragma warn 757

//-----------------------------------------------------------------------------
