//----- nc.c ----- 03/30/11 -----(03/30/11)-----

//-----------------------------------------------------------------------------
//
//  2011-03-11 - Mark Davis
//               - original version of a minimal implementation of the nc
//                 (net connect) command.
//
//-----------------------------------------------------------------------------

#include <time.h>
#include <common.h>

#include <sys/types.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <arpa/inet.h>
#include <errno.h>
#include <signal.h>
#include <utime.h>


#include <busybox.h>

#include <sys/hdwDefs.h>

#include <sys/srfs.h>

#include <dcdefs.h>

#include <ctype.h>
#include <limits.h>

#include <sys/SHD_headers.h>

#include <sys/modbus.h>

#include <tsh.h>

#include <applic.h>

#include <sys/farprintf.h>


//-----------------------------------------------------------------------------
//  cmd to connect stdio to the specified file/device
//-----------------------------------------------------------------------------
void _task _nc(int argc, char **argv)
{
  bool  checkForETX, echo;
  int  sockfd, stdinFlags, stdoutFlags, sockFlags;
  uint  i;
  ulong  remoteAddr;
  struct sockaddr_in  srvAddr;
  IFConfig  curConfig;


  if (showHelp(3, argc, argv))  {
    printf("\nusage: nc ipAddr port [-etx] [-echo]\r\n");  Exit(-1); }

  getIFConfig(IF_ETH0, &curConfig);
  if (curConfig.state != IF_UP)  Exit(-1);

  errno = 0;
  remoteAddr = 0;  inet_ptoh(AF_INET, argv[1], &remoteAddr);

  srvAddr.sin_family = AF_INET;
  srvAddr.sin_port   = htons(atoi(argv[2]));
  srvAddr.sin_addr   = htonl(remoteAddr);

  if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)  {
    perror("socket()");  Exit(-2);  }

  if (connect(sockfd, (struct sockaddr *) &srvAddr, sizeof(srvAddr)) < 0)  {
    perror("connect()");  close(sockfd);  Exit(-3);  }



  checkForETX = echo = false;
  for (i=3; i<argc; ++i)  {
    if (StriEq(argv[i], "-etx"))  checkForETX = true;
    if (StriEq(argv[i], "-echo"))  echo = true;
  }

  stdinFlags = fcntl(0, F_GETFL, 0);
  stdoutFlags = fcntl(1, F_GETFL, 0);
  sockFlags = fcntl(sockfd, F_GETFL, 0);

  for (;;)  {
    if (copyData(0, stdinFlags, sockfd, sockFlags, checkForETX, echo) < 0)  break;
    if (copyData(sockfd, sockFlags, 1, stdoutFlags, false, false) < 0)  break;
    sleepMS(10);
  }

  Exit(0);
}


//-----------------------------------------------------------------------------
