	.objcmt	Softools, Inc. ** Control Cross-C Globally Optimizing Compiler ** Version 2.18.00
	.objcmt	Copyright (c) 1989 - 2003, Softools, Inc.  All rights reserved.
	.debug	P 5
	.debug	X 11 10
	.debug	X 12 10
	.debug	N 'C:\work\dev\Rabbit_ST\appl\LLRF2\LLRF2_Appl.c' 23 30 16 8 1 33
;1 //----- LLRF2_Appl.c ----- 01/08/13 -----(03/05/06)----
;2 
;3 //-------------------------------------------------------------------------
;4 //  Application code to measure and control phase of RF frequency signals.
;5 //  Incorporates the following hardware:
;6 //
;7 //  - Single Board Computer (RCM3200: 44 MHz 8-bit CPU, 768K RAM, 512K flas
;8 //  - 2 Serial EEPROMs MicroChip SST 25VF016B-50 (2048K bytes)
;9 //  - Xilinx Spartan-3A-DSP 3400 FPGA
;10 //  - Texas Instruments TMS320C6748 DSP
;11 //-------------------------------------------------------------------------
;12 //  Update log (most recent first):
;13 //
;14 //
;15 //  2013-01-08 - Mark Davis
;16 //               - updateCurState() now reads a waveform OR does the period
;17 //                 I/O in a pass, but not both.
;18 //
;19 //  2012-11-16 - Mark Davis
;20 //               - Cleaned up state display a bit, and added the data for a
;21 //                 calls to the functions to read/write reg valus from/to t
;22 //                 FPGA.
;23 //
;24 //  2012-11-15 - Mark Davis
;25 //               - Brought up to date with changes to earlier version (the
;26 //                 PhaseCtlrAppl.c) so it handles waveform data properly.
;27 //
;28 //  2012-11-14 - Mark Davis
;29 //               - Commented out outdated readwave() function/task.  Wavefo
;30 //                 data is now read using DirectCA which uses direct CA TCP
;31 //                 connections to the ca_tcpd task.
;32 //               - Added typical and noise values to the read-only DSP regi
;33 //                 definitions to provide more reasonable values when testi
;34 //                 on other hardware platforms.
;35 //
;36 //  2012-09-14 - Mark Davis
;37 //               - Replaced getLock() and releaseLock() with the newer one 
;38 //                 the PhaseCtlr appl that uses shared/global semaphores.
;39 //                 Prior to this there were still instances where the locks
;40 //                 would not be correctly released.
;41 //
;42 //  2012-01-16 - Mark Davis
;43 //               - Changed argument for releaseLock() to uint.  This is so 
;44 //                 is consistent with getLock() and to avoid any possible
;45 //                 compiler errors (and excess code) converting between uch
;46 //                 and uint.
;47 //               - Changed all getLock calls so a return value  of 0 is not
;48 //                 treated as meaning that it got the lock.  This shouldn't
;49 //                 possible, but better safe than sorry.
;50 //
;51 //  2011-09-02 - Mark Davis
;52 //               - Created using the PhaseCtlr code as a starting point.  T
;53 //                 module incorporates the functionality of both the older
;54 //                 (separate) LLRF and PhaseCtlr modules.
;55 //                 In addition to a newer more capable FPGA, this version u
;56 //                 an 8-bit parallel read/write interface to communicate wi
;57 //                 the FPGA (the previous version used a serial SPI connect
;58 //
;59 //-------------------------------------------------------------------------
;60 
;61 /*
;62 ===================
;63 === Terminology ===
;64 ===================
;65 
;66    Feedback Control Loop types:
;67       ADRC: Active Disturbance Rejection Control
;68       PID:  Proportional, Integral, Differential
;69 
;70 
;71 ============================
;72 === Operational Overview ===
;73 ============================
;74 
;75    The LLRF2 controller replaces the Digital Phase Controller and LLRF Turn
;76    modules.  It contains a more powerful FPGA, DSP, and 2 EEPROMs.
;77 
;78    The Rabbit Core Module (RCM) and this application code is responsible fo
;79    ongoing communication with the FPGA and DSP and for maintaining the conf
;80    "files" stored in the 2 EEPROMS.  This is done using 2 SPI interfaces an
;81    one parallel interface.
;82 
;83    The connections to the SPI interfaces are as follows:
;84 
;85       SPI1 (the RCM's Serial Port B):
;86          Master:  RCM
;87          Slave:   DSP
;88 
;89       SPI2 (the RCM's Serial port C):
;90          Masters:  RCM, FPGA, DSP
;91          Slaves:   the EEPROMS
;92 
;93     16 of the RCMs I/O signals are used as a parallel I/O bus to communicat
;94     with the FPGA (8 for data, 6 for an address, 2 for R/W controls).
;95 
;96 
;97     The EEPROMS contain the configuration files for the FPGA and the DSP
;98     (one FPGA, one for the DSP).  The RCM provides the means to copy a file
;99     from a remote server to either of the EEPROMs and restart the FPGA and
;100     the DSP so they use the new configuration.
;101 
;102     When not updating one of the EEPROMs, the RCM's primary task is to
;103     continually update its copy of a set of values provided by the DSP and
;104     the FPGA, and to send them any changes to writable values made by
;105     external entities (e.g. the control system or a person typing a shell
;106     command).
;107 
;108 
;109 ==========================
;110 === Summary of signals ===
;111 ==========================
;112 
;113    ADDR0-ADDR11 (INPUTS):
;114       Used to read the state of the 12 DIP switches that determine the
;115       controller's ID.  The combination of the controller type and its ID m
;116       be unique for all controllers.
;117 
;118 
;119    /DSP_RESET (OUTPUT):
;120       A low level holds the DSP in RESET and stops it from using the SPI
;121       interfaces.  A low-to-high transition causes to reload its configurat
;122       from its EEPROM.
;123 
;124    PROG_B (OUTPUT):
;125       A low level holds the FPGA in RESET and stops it from using the SPI
;126       interfaces.  A low-to-high transition causes to reload its configurat
;127       from its EEPROM.
;128 
;129 
;130    Rab_SPI1_DSP_EN (OUTPUT):
;131       Effectively a Chip Select for the DSP's connection to the SPI1 bus.
;132       This is asserted to enable the RCM to communicate with the DSP via th
;133       SPI1 interface.
;134 
;135 
;136    Rab_SPI2.DSPen (Hi-Z state or OUTPUT driven low):
;137       The Chip Select for the EEPROM connected to the DSP.  A low level ena
;138       the EEPROM.  This can be asserted by the RCM or the DSP (but not both
;139       once).  The RCM must hold this in a high impedance state when it is n
;140       using the SPI1 interface.  Before driving it low, it must drive both
;141       /DSP_RESET and PROG_B low to insure neither the DSP nor the FPGA is
;142       trying to use the SPI2 interface.
;143 
;144    Rab_SPI2.FPGAen (Hi-Z state or OUTPUT driven low):
;145       The Chip Select for the EEPROM connected to the FPGA.  A low level en
;146       the EEPROM.  This can be asserted by the RCM or the FPGA (but not bot
;147       once).  The RCM must hold this in a high impedance state when it is n
;148       using the SPI1 interface.  Before driving it low, it must drive both
;149       /DSP_RESET and PROG_B low to insure neither the DSP nor the FPGA is
;150       trying to use the SPI2 interface.
;151 
;152    FPGA_DONE (INPUT):
;153       A low-level state indicates the FPGA is (re)loading its configuration
;154       from its EEPROM.  A high-level state indicates it is busy doing so an
;155       the Rabbit should wait until it is finished before attempting to
;156       communicate with the FPGA or its EEPROM.
;157 
;158 
;159    Rab_SPI1_CLK (Hi-Z state or OUTPUT):
;160       Clock for SPI1 interface.  When using the SPI1 interface, the RCM dri
;161       this high/low.  The rest of the time it holds it in a Hi-Z state.
;162 
;163    Rab_SPI1_MOSI (OUTPUT):
;164       Used to write data to the DSP (and possibly the FPGA in the future?).
;165 
;166       NOTE:  The RCM cannot set this to a Hi-Z state, so if something else
;167       needs to write to the SPI1 bus, something external to the Rabbit will
;168       have to control when the Rabbit is connected to this signal.
;169 
;170    Rab_SPI1_MISO (INPUT):
;171       Used to read data from the DSP (and possibly the FPGA in the future?)
;172 
;173 
;174    Rab_SPI2.CLK (Hi-Z or OUTPUT):
;175       Clock for SPI2 interface.  When using the SPI1 interface, the RCM dri
;176       this high/low.  The rest of the time it holds it in a Hi-Z state.
;177 
;178    Rab_SPI2.MOSI (OUTPUT):
;179       Used to write data to one of the EEPROMS.  Which EEPROM the data is
;180       written to is controlled by driving either the Rab_SPI2.FPGAen or the
;181       Rab_SPI2.DSPen signal low.
;182 
;183       NOTE:  The RCM cannot set this to a Hi-Z state, so something external
;184       the RCM must disconnect the RCM when it is not using the SPI2 bus.  T
;185       easiest way to do this would be to connect the RCM to this signal ONL
;186       when it is driving both the /DSP_RESET and PROG_B outputs low.
;187 
;188    Rab_SPI2.MISO (INPUT):
;189       Used to read from one of the EEPROMS.  Which EEPROM the data is read 
;190       is controlled by driving either the Rab_SPI2.FPGAen or the Rab_SPI2.D
;191       signal low.
;192 
;193 
;194    CPU_RDY (OUTPUT):
;195       A high-level state means the Rabbit is currently running and is ready
;196       communicate with the FPGA and DSP.
;197 
;198    ACTY (OUTPUT):
;199       Connected to an LED, the Rabbit pulses this to cause the LED to blink
;200       various rates.  Used as a simple means of indicating the state of the
;201       controller to someone within view of the controller.
;202 
;203 
;204    FPGA.A0-FPGA.A5 (OUTPUTS):
;205       FPGA Address:  A 6-bit address that specifies which value the Rabbit
;206       wishes to read from or write to the FPGA.
;207 
;208    FPGA.D0-FPGA.D7 (I/O):
;209       FPGA Data:  An 8-bit bidirectional data bus for reading from and writ
;210       to the FPGA.  The Rabbit will leave these signals in a high impedance
;211       state except when it sets FPGA.WE high.  The FPGA will leave them in 
;212       high impedance state except when the Rabbit sets FPGA.OE high.
;213 
;214    FPGA.OE (OUTPUT):
;215       FPGA Output Enable:  Used to read an 8-bit value from the FPGA.  Norm
;216       in a low-level state, a value is read from the FPGA as follows:
;217 
;218         - Assert the address of the value to read on FPGA.A0-FPGA.A5
;219         - Set FPGA.OE high and wait at least 106ns
;220         - Read the value on FPGA.D0-FPGA.D7
;221         - Set FPFGA.OE low
;222 
;223    FPGA.WE (OUTPUT):
;224       FPGA Write Enable:  Used to write an 8-bit value to the FPGA.  Normal
;225       in a low-level state, a value is written to the FPGA as follows:
;226 
;227         - Assert the address of the value to read on FPGA.A0-FPGA.A5
;228         - Assert the value to be written on FPGA.D0-FPGA.D7
;229         - Pulse FPGA.WE high for at least 27ns
;230 
;231    FPGA.CLK (OUTPUT):
;232       FPGA Clock:  Not currently used.
;233 
;234       NOTE: This is connected to a Rabbit signal that CANNOT be configured 
;235       a clock signal for any of the Rabbit's serial ports.
;236 
;237 
;238 =======================================
;239 === Outline of RCM responsibilities ===
;240 =======================================
;241 
;242   Unlike most of the RCM-based controllers, the RCM code for this controlle
;243   is not directly responsible for much in terms of actual real-time control
;244   of the device.  The primary role of the RCM is to manage the configuratio
;245   data for and communication with the FPGA and DSP, which handle all the
;246   high-speed computational tasks needed to make the device work in real tim
;247 
;248   What the RCM IS responsible for:
;249 
;250    - With the noted exception of real-time control of the device(s) the
;251      controller is connected to, the RCM is responsible same things it is
;252      in ALL RCM-based controllers:
;253        - Communication with external entities such as the Control System
;254        - Managing persistent configuration values
;255        - Monitoring the state of the device
;256        - Run-time diagnostics
;257 
;258    - On startup and on demand:  Reset the FPGA and DSP and cause them to
;259      reload their configurations from their respective EEPROMs.
;260 
;261    - Copy the contents of a DSP or FPGA configuration file (usually located
;262      a remote server) to one of the EEPROMS.
;263 
;264    - Continually R/W values from/to the FPGA and the DSP.
;265 
;266      NOTES:
;267 
;268        Communication with the FPGA is via an "ad-hoc" parallel bus (i.e. it
;269        uses general I/O signals for the data and address signals rather tha
;270        the Rabbit's built-in I/O capability), so it is relatively slow.
;271 
;272        Comnunication with the DSP is the same as the earlier PhaseCtlr modu
;273        using the SPI1 bus.
;274 
;275        For values that do not trigger some action within the DSP or FPGA, t
;276        current value is normally read and/or written to the DSP or FPGA eac
;277        time through the main application task's loop.  This ensures that th
;278        most recent value is always known/applied.
;279 
;280   -------------------------------------------------------------------------
;281 
;282   This is the first controller where I will try adopting a new strategy in
;283   terms of the separation of the hardware and application logic layers.
;284 
;285   Prior to this, the separation was only as deep as the signal level.  The
;286   hardware layer managed reading the state of the available analaog and
;287   digital inputs and storing them in some global variables, and updating
;288   the state of the available analog and digital outputs using values stored
;289   in some global variables.
;290 
;291   This approach meant that the application layer did not have to worry abou
;292   how the digital and analog signals where implemented by any particular
;293   hardware.  It just assumed that the signals existed and that, on request,
;294   any changes would be propogated in both directions by the hardware layer.
;295 
;296 
;297   While not as efficient as directly manipulating the hardware-specific
;298   registers and chips needed to control or read the signal states, it meant
;299   the application code could be built for and run on any platform that
;300   supported a very basic set of I/O signals without any changes to the code
;301 
;302   But that model is built on the assumption that the application logic is
;303   tightly coupled to the state of the analog and digital inputs and outputs
;304 
;305   In the broadest sense all the controller applications can be described as
;306   a continuous loop that reads the state of a set of inputs, computes what
;307   the next desired state should be, then updating the state of the outputs
;308   with the goal of bringing about the desired changes.
;309 
;310   Note that this holds true regardless of whether the inputs and outputs
;311   represent actual digital and analog inputs and outputs or simply values
;312   in memory that are accessible to other tasks (and, indirectly, logic
;313   external to the controllers).
;314 
;315 
;316   For most controllers, the current and next state of these hardware signal
;317   is the focus of the application logic, so it makes logical sense to place
;318   the hardware abstraction at that level.  Also, most applications only
;319   update the states of these signals once each pass, so the additional
;320   overhead of the abstraction layer is not significant.
;321 
;322 
;323   But the focus of the LLRF2 application task is communuication with the
;324   DSP, FPGA, and their EEPROMs.  It does not care what signals are needed t
;325   implement communication, or how they are used.  It only cares about when 
;326   NEEDS to communicate and when it CAN.
;327 
;328   Also, the communication requires manipulating the hardware signals dozens
;329   even hundreds of time each pass, so the overhead of abstracting them can
;330   add up very quickly.
;331 
;332 
;333   Last but not least is the issue of testing the application logic.
;334 
;335   For many applications basic testing can be accomplished by manually
;336   manipulating the state of the inputs and monitoring the state of the outp
;337   But even in the cases where this is useful, it is not practical to simula
;338   many of the behaviors of the real hardware.  In the case of the LLRF2
;339   application, there is no practical way to manually simulate much of anyth
;340   the actual hardware does, so very little testing can be done on any other
;341   platform.
;342 
;343   A more practical approach in this case that may also be more practical fo
;344   other applications is to move the level of abstraction closer to the
;345   application logic.  Rather than simply providing an abstraction of the an
;346   and digital signals, the hardware layer API will include application
;347   specific functions that previously were implemented in the application la
;348 
;349   This has the following advantages over the simpler signal-abstraction
;350   approach:
;351 
;352     - More efficient:  They can manipulate the signals more directly so
;353       functions that have to manipulate the signals many times each call
;354       will be significantly faster.
;355 
;356     - Each platform the application is built for can provide its own versio
;357       that operates the most efficiently for that platform.  For platforms
;358       that simply don't have the equivalent hardware, the function could
;359       provide a simulation of the hardware, providing a more realistic test
;360       on other platorms.
;361 
;362       NOTES:
;363          The ioctl() model could be used to make it possible to maintain th
;364          one advantage of the current model:  Build and run any application
;365          on any platform.  Of course, this approach simply amounts to run-t
;366          linking, and any platform that does not implement the (indirectly)
;367          specified function will simply return an error, so it is of dubiou
;368          value.
;369 
;370          A better approach would be to provide a generic version of the
;371          functions that could be compiled for any platform and which would
;372          provide at least a minimal simulation of the hardware it was meant
;373          for.  This would also avoid the overhead of figuring out which
;374          function is being indirectly called each time.  And given that the
;375          function ID would have to be unique for ALL applications, that cou
;376          turn in to a significant amount of overhead.
;377 
;378   ------------------------------------------------------------------------
;379 
;380   Questions/Issues:
;381 
;382     - Eliminate the functionality for writing to the DSP memory?  I assume
;383       this is superceeded by the DSP having its own dedicated EEPROM.
;384 
;385     - Keep in mind that some communication with the FPGA and/or DSP will
;386       require checking the value of a polled register value.  It is probabl
;387       not practical to implent that level within the hardware abstraction
;388       layer (although such functions will still call hardware layer functio
;389       to actually read/write individual values or blocks of values, includi
;390       the register being polled).
;391 
;392     - Should I still map at least some of the signals to the digitalIn and
;393       digitalOut values so I can monitor them while testing other functions
;394       I COULD make the digitalOut read-only.
;395 
;396       I could have a separate function that returns the state of a bunch
;397       of the signals, but that would just require duplicating much of same
;398       functionality that already works for the digitalIn and digitalOut
;399       values.
;400 
;401     - Eliminate everything to do with the PLL?
;402 
;403     - DSP program number?  Is this concept still valid?  How is it handled
;404       now that the Rabbit writes directly to the EEPROM?  Just assume a
;405       fixed size and divide the EEPROM space in to equal size parts?
;406 
;407     - dspComErrors?
;408 
;409 
;410 ---------------------------------------------------------------------------
;411 (from older DigPhaseCtlr:  needs to be revised for this version...)
;412 
;413   - The EEPROM contains the code for the FPGA.  On startup, one of the digi
;414     inputs to the FPGA (FPGA_SPI) is toggled, which causes it to take over 
;415     SPI connection to the EEPROM and load the binary image stored there.  A
;416     digital output from the FPGA is monitored to indicate when this has
;417     completed (it takes a little less than 2 seconds).
;418 
;419   - The code for the DSP is stored in the DSP itself.  Similar to how the
;420     DEBROS update command can be used to update the DEBROS boot image, the 
;421     code provides a mechanism for updating the copies of the DSP code store
;422     within the DSP.  When the DSP is running a working copy of the DSP code
;423     the progdsp command can be used to download a new copy of DSP code in t
;424     the DSP itself.
;425 
;426     So updating or storing the initial working copy of DSP code requires th
;427     a working copy of DSP code is already running in the DSP.  If a working
;428     copied is stored at the correct place within the DSP memory, then the D
;429     will load it and run it after being reset.  But if there isn't a workin
;430     copy in the DSP memory, then getting it to run a working copy can only 
;431     accomplished by running the appropriate development software on a deskt
;432     PC and use of a special cable between the PC and the controller (again,
;433     very similar to how the DLM is downloaded to a Rabbit Core Module).
;434 
;435 
;436 Communicating with the DSP:
;437 
;438   - Exchange of data with the DSP is done using a bi-directional (simultane
;439     send/recv) clocked SPI interface.  All data is exchanged in 32-bit (4-b
;440     packets.  While we are sending a 32-bit packet to the DSP, it is using 
;441     same clock pulses to simultaneously send a 32-bit packet in response.
;442 
;443     Each exchange of a 32-bit packet consists of setting the DSP /Enable si
;444     low, writing (while simultaneously reading) 32-bits of data, and then
;445     setting the DSP /Enable signal high.
;446 
;447     Due to the way the DSP buffers are set up, all responses from the DSP a
;448     always 2 packets "behind" the ones that contained the commands that the
;449     are a response to.  We can write as many commands as we want in a
;450     sequence of packets, but the response to each one will always be receiv
;451     2 packets later.  Given that we generally want to read the response to
;452     a set of commands as soon as possible, we normally have to add some
;453     dummy or filler packets to the sequence to get the desired response
;454     packets.  For extended responses (like reading of waveform data), we mu
;455     send lots of filler packets to get all the data.
;456 
;457 
;458 Revamped register conventions (Jan 2010):
;459 
;460   - Communication with the DSP is done by reading and writing "register"
;461     values, each of which has an "address".  Even numbered addresses are fo
;462     reading and odd numbered ones are for writing (i.e. the least significa
;463     bit specifies a read or write operation).
;464 
;465   - Reading a register value is done by sending the 4-byte address of the
;466     register and then reading (2 packets later) its 4 byte value.
;467 
;468   - Writing to a register is done by sending the 4-byte address and then th
;469     4-byte value.  The DSP responds (with the usual 2-packet delay) with th
;470     4-byte address and 4-byte value as a means of verifying that we are
;471     communicating with the DSP and that it received the correct values.
;472 
;473     The meaning of the 4-byte value is dependent on the register.  The most
;474     common representations used are 32-bit floats, 32 and 16-bit integers,
;475     and bit-mapped values.  The values are always in network byte order (mo
;476     significant first).
;477 
;478 
;479   - Registers come in several varieties:
;480 
;481       Read-Only-and-Continuously:
;482          The value should be read each pass to keep track of the current st
;483 
;484       Write-Only-and-Continuously:
;485          These represent settings that we want asserted all the time.  If t
;486          DSP resets, these values will be quickly re-asserted because they 
;487          written to the DSP each pass.
;488 
;489       Write-Once, Read-never:
;490          Writing to the value triggers some action so writing to them each
;491          pass is not an option.  These registers generally provide the mean
;492          to write to one instance in a table of values maintained within th
;493          DSP itself, where the row written to is based on the current state
;494          of the DSP.
;495 
;496       Read/Write-continuously:
;497          Such registers have both a separate read and write register and of
;498          corresponding separate read and write ModBus values.  The usual ca
;499          is that the write value represents a setpoint and read value
;500          represents the current state or readback value.
;501 
;502       Read-Trigger:
;503          Reading from the register triggers some event such as starting or
;504          stopping a calculation, so it is done only when we want to trigger
;505          the corresponding process or event.
;506 
;507 
;508   - Most the DSP registers have a corresponding ModBus register.  Tables ar
;509     used to provide the information that links them together.  About 20 tim
;510     per second the main application task uses these tables to read and writ
;511     values from/to the DSP.
;512 
;513     The table for the read-only registers causes the ModBus register values
;514     that correspond to each DSP register to be updated.  The table for the
;515     writable registers is used to write the current value of each of the
;516     corresponding ModBus registers to the DSP.  This means that, if the DSP
;517     resets for any reason, the current state will be restored very quickly.
;518 
;519   - The dspread and dspwrite commands provide a mechanism to manually read
;520     values from and write values to the DSP.  Just be careful about using
;521     registers that trigger some event within the DSP as this can interfere
;522     with normal operation.
;523 
;524   - To facilitate testing, the normal periodic reading from and writing to 
;525     DSP registers can be disabled or enabled using the dspio command.
;526 
;527 
;528   - The register values provide information about the state of the DSP.
;529     One of the registers is a bit-mapped status value, with bits that
;530     indicate if new waveform data is available for reading.
;531 
;532     For each waveform we check to see if we have an open connection to a
;533     client for the waveform and if there is new data or a fixed interval
;534     of time has elapsed.  If these conditions are met, then we read the
;535     corresponding waveform data as follows:
;536 
;537       - Using the write/read 4-bytes methodology described above,
;538         read the # of 32-bit words of data available for the waveform.
;539 
;540       - Using the value 0x0010##00 for the 4 bytes we write each
;541         time (where ## indicates which waveform we want data for),
;542         use the write/read 4-bytes methodology to read the
;543         indicated # of 32-bit words of data.
;544 
;545     A couple of the waveforms take around 40 seconds to compute and the
;546     DSP must be told when to compute them.  For these an additional
;547     register provides the %complete the computation is so we can monitor
;548     the progress of the computation and determine when it is available
;549     for reading.
;550 
;551 
;552  - Some of the bit-mapped registers we can write to represent multiple valu
;553    each represented by more than one bit.  When writing to these registers,
;554    all 0's for a given group means do not change the value of any of the
;555    bits in that group.
;556 
;557    To insure that the DSP state is properly restored if it is reset, the la
;558    value of each bit-group within such registers must be kept track of.  Th
;559    minimizes problems caused by DSP resets.
;560 
;561 
;562  - The SPI interface used to communicate with the DSP is also used to
;563    communicate with the FPGA, the PLL, and the ADC's.  To allow multiple ta
;564    to communicate with with these devices, a locking mechanism is provided 
;565    insure that only one task is using the shared SPI connection and related
;566    data values at any given time.
;567 
;568 
;569  - The DSP xxxCalib value registers (0x0010022A - 0x001000239) are used
;570    to write a given value in to a table of values based on the current stat
;571    of another value (attenuations?), so they cannot be included in the set
;572    that is constantly being written to.  Writing to one of these registers
;573    is done just once when you need to write a value to the table stored in
;574    the DSP.  Saving the table is done by reading from the register at the
;575    end of that range (SaveCalibToDSP: 0x0010023A).
;576 
;577 ---------------------------------------------------------------------------
;578 
;579 
;580 //-------------------------------------------------------------------------
;581 //  Work in progress:
;582 //
;583 //  - Need to deal with multiple sources of R/W requests for the DSP.  The
;584 //    situation is not as simple as representing the DSP as a std streaming
;585 //    device that reads() and writes() can be interleaved on.  The followin
;586 //    constraints exist:
;587 //
;588 //    - SPI port 1 is connected to the FPGA, PLL, DSP, and ADC's
;589 //
;590 //    - Different control bits and clock modes are required for different
;591 //      devices
;592 //
;593 //    - Reads and Writes to the DSP are not independant.  In at least some
;594 //      cases, you have to know how many bytes to read from the DSP at the
;595 //      time you begin writing to it.
;596 //
;597 //    - Reads and Writes to the DSP always consist of an even number of byt
;598 //
;599 //    - Reads and Writes to the DSP require a control bit be toggled betwee
;600 //      each pair of bytes transferred
;601 //
;602 //
;603 //  Events/processes that require use of the SPI connection:
;604 //
;605 //  - Writing commands to the ADC's
;606 //
;607 //  - Writing commands to the FPGA
;608 //
;609 //  - Writing commands to and reading data from the DSP (where some of the
;610 //    responses can be up to 1024 bytes in length)
;611 //
;612 //  - Downloading a new config file to the FPGA (or the DSP?)
;613 //
;614 //  - A write to a Modbus register that mirrors a value in the DSP
;615 //
;616 //
;617 //  Two options and issues regarding sharing this single resource that many
;618 //  tasks need to use or affect:
;619 //
;620 //  - Extension of current approach:  Use a semaphore to limit access to a
;621 //    single task at a time.
;622 //
;623 //  - Limit all access to a single task and send msgs to the task requestin
;624 //    certain actions
;625 //
;626 //  The semaphore approach has one major drawback:  Tasks that cannot block
;627 //  (like the main applic process) must either give up on an action or keep
;628 //  data needed to process the request and retry until successful or an err
;629 //  occurs.  It has the advantage that it means less code for tasks that CA
;630 //  block without a problem.
;631 //
;632 //  The message approach has the advantage that a task can send a message a
;633 //  the request and pretty much forget about it.  The disadvantages are tha
;634 //  their is no easy way to provide feedback to the requesting task, and th
;635 //  all the work has to be done in one task.
;636 //
;637 //
;638 //  Basic problem scenario:
;639 //
;640 //    A command is issued to download a new image to the FPGA or the DSP.
;641 //    This can tie up the SPI for several seconds.  While this is going on,
;642 //    several Modbus writes to registers mirroring DSP values happen.  Thes
;643 //    writes cause event messages to be written to the main application's
;644 //    event msg queue.  The application loop, which is still running while
;645 //    the download to the FPGA is going on, cannot use the SPI line, so it
;646 //    cannot read values from the DSP to update its state data, much less
;647 //    process the messages to write new values to the DSP.  In order to
;648 //    not loose the write events, it has to stop processing the event
;649 //    messages, which means the queue could overflow, causing processes
;650 //    that write to the queue to block.
;651 //
;652 //  I don't see any way around this.  The SPI line serves several purposes
;653 //  and it is either in use or it is not.  If it is in use, a task that
;654 //  wishes to use it can either block or not.  For processes that cannot
;655 //  (or should not) block, they can either ignore/forget what they were
;656 //  going to do with the SPI connection or keep track of what they were
;657 //  about to do and keep polling periodically, waiting for their chance
;658 //  to use the SPI connection.
;659 //
;660 //  This is really nothing but the basic limitation of any resource that
;661 //  must be protected against concurrent usage.  A locking mechanism is
;662 //  required that insures only one user at a time, and tasks contending for
;663 //  the resource have 3 choices:
;664 //
;665 //  - block on the resource (and get woken up when it is available)
;666 //
;667 //  - poll in a loop, waiting until they get access, possibly doing other
;668 //    useful stuff that is not dependent on the the event that cannot be
;669 //    completed until the locked resource is available
;670 //
;671 //  - forget what they needed it for and continue on
;672 //
;673 //
;674 //  So, when the main application process needs the SPI line, it will eithe
;675 //  have to block on a wait queue, or simply suspend processing of event
;676 //  messages.  The advantage of the blocking approach is that the order of
;677 //  requests for the SPI line will be enforced by the wait queue.  On the
;678 //  other hand, I might want to grant access based on priority as well, whe
;679 //  higher priority tasks get unblocked before lower priority ones.  But as
;680 //  a general rule, ignoring the sequence of requests can lead to other
;681 //  problems.
;682 //
;683 //-------------------------------------------------------------------------
;684 //
;685 //  More notes on SPI port usage:
;686 //
;687 //  The model I am using for the SPI port is not right.  It should be treat
;688 //  more like a disk drive or a network connection.
;689 //
;690 //  When a serial port is used as a point-to-point connection between 2 dev
;691 //  it often makes sense to allow the application code to directly manage t
;692 //  connection.  But when there are multiple devices connected via the same
;693 //  communications medium, then the connection is a bus, and the connection
;694 //  needs to be handled by a driver that provides a layer of abstraction fo
;695 //  communicating with the devices attached to the bus.
;696 //
;697 //  In this case, something more like a device special file per device atta
;698 //  to the SPI port makes more sense, where the major device # is for the S
;699 //  port and the minor specifies which device on the "bus".  Then individua
;700 //  architecture modules can deal with them however I want (for devices tha
;701 //  a particular board does not have, it can either return errors for attem
;702 //  to access the device, or I could have a test mode where they mimic the
;703 //  device to some degree, or simply return no errors).
;704 //
;705 //-------------------------------------------------------------------------
;706 // Keeping Modbus register values (global variables) in sync with the DSP
;707 // settings vs reading waveform data after a settings
;708 //
;709 // - ampl and phase values in DSP and ModBus register (global variable) val
;710 //   must be kept in sync
;711 //
;712 // - when we want to read the waveform data from the DSP to see the effect
;713 //   of changing the ampl and/or phase value, we have to make the settings
;714 //   and read the waveform data as a single transaction
;715 //
;716 // - one possibility is to read the waveform data EVERY time the ampl or
;717 //   phase setpoints are altered.  Then there is no distinction between "ju
;718 //   setting" one of them vs setting-and-reading
;719 //   BUT:  The problem with that is it doesn't allow for changing BOTH the
;720 //   the phase and amplitude and reading the result.
;721 //
;722 //-------------------------------------------------------------------------
;723 //
;724 //  "Pulsed" tuning test mode:
;725 //
;726 //    The DSP normally controls the Amplitude and Phase values using a stan
;727 //    PID feedback-loop algorithm.  To facilitate the tuning of the PID wei
;728 //    used by the feedback loop, the DSP and controller provide a "pulsed"
;729 //    tuning/test mode where the DSP periodically cycles the Amplitude and/
;730 //    Phase setpoints between two settings and captures a portion of the
;731 //    resulting outputs in a buffer that the controller's CPU can read and
;732 //    send to a remote client for the purpose of providing a near real-time
;733 //    plot of the resulting waveforms.
;734 //
;735 //
;736 //  Basic operation:
;737 //
;738 //    The DSP contains all the logic and code for actually implementing the
;739 //    pulsed test functionality.  It requires the following values to be se
;740 //    before this mode is enabled:
;741 //
;742 //    - Nominal Amplitude and Phase setpoints:  These are the "normal"
;743 //      setpoints, used when the pulsed mode is not active.
;744 //
;745 //    - Pulse Amplitude and Phase setpoints:  When the pulsed test mode is
;746 //      activated, the DSP toggles the setpoints between the Nominal values
;747 //      and Pulsed setpoints.
;748 //
;749 //    - Pulse Frequency:  Controls the # of times per second that the
;750 //      Amplitude and/or Phase setpoints are toggled between the 2 setpoint
;751 //
;752 //    - Trigger position:  Specifies the # of samples that should be taken
;753 //      before changing the setpoint(s).  This provides a simple means of
;754 //      specifying a trigger delay, or a sliding window around the trigger
;755 //      event.
;756 //
;757 //    The pulsed test mode is activated by setting one or both of the run b
;758 //    in the DSP's cmd word.  When activated, the DSP samples the outputs a
;759 //    writes the values to one of two buffers.  One buffer is for the sampl
;760 //    window around the rising edge, and one is for the window around the
;761 //    falling edge.  When the DSP has filled one of the buffers, it sets a 
;762 //    in the status word indicating the buffer contains new data to be read
;763 //    The DSP must then leave the contents of the buffer untouched until it
;764 //    has been read by the controller CPU (see next paragraph).
;765 //
;766 //    The controller CPU periodically reads the status word from the DSP.
;767 //    Whenever it sees that the new-data bit for one of the buffers is set,
;768 //    it reads the data for that buffer from the DSP and then sets a bit in
;769 //    the command word indicating it has read the contents of the buffer.
;770 //    To avoid having the controller CPU attempt to read the same data from
;771 //    the same buffer the next cycle, the DSP must respond by immeadiately
;772 //    resetting the new-data bit in the status word.
;773 //
;774 //    Once the new-data bit has been reset, the DSP is free to write new
;775 //    data to the buffer, repeating the above cycle.
;776 //
;777 //-------------------------------------------------------------------------
;778 
;779 
;780 
;781 #include <common.h>
;782 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\time.h' 15 48 18 10 12 22
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\string.h' 26 27 14 2 6 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stddef.h' 21 20 12 5 6 22
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\math.h' 19 23 18 21 9 21
	.debug	I
	.debug	I
;783 #include <unistd.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\sched.h' 3 49 6 8 6 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\signal.h' 6 21 8 17 3 29
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\list.h' 21 49 15 26 7 30
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\dummy.h' 25 58 21 12 10 27
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\list.h' 21 49 15 26 7 30
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Rabbit.h' 24 40 11 19 5 30
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stdio.h' 22 16 12 9 4 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\SoftoolsIDblock.h' 18 52 15 4 6 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\FlashBlock.h' 5 10 16 30 11 24
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Flash.h' 0 55 21 17 7 26
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Stdlib.h' 2 30 16 6 7 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\dcdefs.h' 18 42 23 19 7 26
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Rabbit.h' 24 40 11 19 5 30
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\SoftoolsIDBlock.h' 18 52 15 4 6 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\errors.h' 25 8 15 18 11 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stdlib.h' 2 30 16 6 7 24
	.debug	I
	.debug	# 109
_LIN2SEG	.macro

	.debug	# 110
add hl,hl

	.debug	# 111
rla

	.debug	# 112
add hl,hl

	.debug	# 113
rla

	.debug	# 114
add hl,hl

	.debug	# 115
rla

	.debug	# 116
add hl,hl

	.debug	# 117
rla

	.debug	# 118
sub 0x0E

	.debug	# 119
or a

	.debug	# 120
rr hl

	.debug	# 121
scf

	.debug	# 122
rr hl

	.debug	# 123
scf

	.debug	# 124
rr hl

	.debug	# 125
scf

	.debug	# 126
rr hl

	.debug	# 127
.endm

	.cseg
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\hdwdefs.h' 20 2 12 27 10 31
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\sched.h' 3 49 6 8 6 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\fcntl.h' 29 48 12 5 12 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\ioctl.h' 2 15 10 21 5 30
	.debug	I
	.debug	I
	.debug	I
;784 #include <sys/types.h>
;785 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
;786 #include <linux/wait.h>
;787 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
;788 #include <semaphore.h>
;789 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\sched.h' 3 49 6 8 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I
;790 
;791 
;792 #ifndef  ApplicSource
;793   #define  ApplicSource  "LLRF2_Appl.c"
;794 #endif
;795 
;796 #ifndef DidRegDefs
;797   #define  DidRegDefs
;798   #include <modbusRegs.c>
	.debug	I 'C:\work\dev\Rabbit_ST\daemons\modbusRegs.c' 29 38 11 7 12 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\modbus.h' 21 56 10 7 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\appl\LLRF2\LLRF2_Appl.c' 23 30 16 8 1 33
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I
	.dseg
_DoNothing_CoilInfo::
	.dw	word _DoNothing
	.db	0
	.dw	word (phys @1)
	.dw	phys @1 >> 16
	.dw	word _setCoil
	.db	0x2
	.debug	G _DoNothing_CoilInfo 32 0
_dlmVersTime_RegInfo::
	.dw	word _dlmVersTime
	.dw	0
	.dw	word (phys @1+10)
	.dw	phys @1+10 >> 16
	.dw	0
	.db	0x3
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _dlmVersTime_ChgEventVal
	.dw	word _dlmVersTime_MinChg
	.debug	G _dlmVersTime_RegInfo 33 0
_rtosVersTime_RegInfo::
	.dw	word _rtosVersTime
	.dw	0x2
	.dw	word (phys @1+22)
	.dw	phys @1+22 >> 16
	.dw	0
	.db	0x3
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _rtosVersTime_ChgEventVal
	.dw	word _rtosVersTime_MinChg
	.debug	G _rtosVersTime_RegInfo 33 0
_applVersTime_RegInfo::
	.dw	word _applVersTime
	.dw	0x4
	.dw	word (phys @1+35)
	.dw	phys @1+35 >> 16
	.dw	0
	.db	0x3
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _applVersTime_ChgEventVal
	.dw	word _applVersTime_MinChg
	.debug	G _applVersTime_RegInfo 33 0
_devType__RegInfo::
	.dw	word _devType_
	.dw	0x6
	.dw	word (phys @1+48)
	.dw	phys @1+48 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _devType__ChgEventVal
	.dw	word _devType__MinChg
	.debug	G _devType__RegInfo 33 0
_deviceID__RegInfo::
	.dw	word _deviceID_
	.dw	0x7
	.dw	word (phys @1+57)
	.dw	phys @1+57 >> 16
	.dw	0
	.db	0x3
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _deviceID__ChgEventVal
	.dw	word _deviceID__MinChg
	.debug	G _deviceID__RegInfo 33 0
_upSecs_RegInfo::
	.dw	word _upSecs
	.dw	0x9
	.dw	word (phys @1+67)
	.dw	phys @1+67 >> 16
	.dw	0
	.db	0x3
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _upSecs_ChgEventVal
	.dw	word _upSecs_MinChg
	.debug	G _upSecs_RegInfo 33 0
_percentIdle__RegInfo::
	.dw	word _percentIdle_
	.dw	0xb
	.dw	word (phys @1+74)
	.dw	phys @1+74 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _percentIdle__ChgEventVal
	.dw	word _percentIdle__MinChg
	.debug	G _percentIdle__RegInfo 33 0
_numChans_RegInfo::
	.dw	word _numChans
	.dw	0xc
	.dw	word (phys @1+87)
	.dw	phys @1+87 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _numChans_ChgEventVal
	.dw	word _numChans_MinChg
	.debug	G _numChans_RegInfo 33 0
_coilChanMult_RegInfo::
	.dw	word _coilChanMult
	.dw	0xd
	.dw	word (phys @1+96)
	.dw	phys @1+96 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _coilChanMult_ChgEventVal
	.dw	word _coilChanMult_MinChg
	.debug	G _coilChanMult_RegInfo 33 0
_reg3ChanMult_RegInfo::
	.dw	word _reg3ChanMult
	.dw	0xe
	.dw	word (phys @1+109)
	.dw	phys @1+109 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _reg3ChanMult_ChgEventVal
	.dw	word _reg3ChanMult_MinChg
	.debug	G _reg3ChanMult_RegInfo 33 0
_reg4ChanMult_RegInfo::
	.dw	word _reg4ChanMult
	.dw	0xf
	.dw	word (phys @1+122)
	.dw	phys @1+122 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _reg4ChanMult_ChgEventVal
	.dw	word _reg4ChanMult_MinChg
	.debug	G _reg4ChanMult_RegInfo 33 0
_digInBits_RegInfo::
	.dw	word _digInBits
	.dw	0x10
	.dw	word (phys @1+135)
	.dw	phys @1+135 >> 16
	.dw	0
	.db	0x3
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _digInBits_ChgEventVal
	.dw	word _digInBits_MinChg
	.debug	G _digInBits_RegInfo 33 0
_statusBitsStdA_RegInfo::
	.dw	word _statusBitsStdA
	.dw	0x12
	.dw	word (phys @1+145)
	.dw	phys @1+145 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _statusBitsStdA_ChgEventVal
	.dw	word _statusBitsStdA_MinChg
	.debug	G _statusBitsStdA_RegInfo 33 0
_statusBitsStdB_RegInfo::
	.dw	word _statusBitsStdB
	.dw	0x13
	.dw	word (phys @1+160)
	.dw	phys @1+160 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _statusBitsStdB_ChgEventVal
	.dw	word _statusBitsStdB_MinChg
	.debug	G _statusBitsStdB_RegInfo 33 0
_statusBitsAppl_RegInfo::
	.dw	word _statusBitsAppl
	.dw	0x14
	.dw	word (phys @1+175)
	.dw	phys @1+175 >> 16
	.dw	0
	.db	0x3
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _statusBitsAppl_ChgEventVal
	.dw	word _statusBitsAppl_MinChg
	.debug	G _statusBitsAppl_RegInfo 33 0
_latency_RegInfo::
	.dw	word _latency
	.dw	0x16
	.dw	word (phys @1+190)
	.dw	phys @1+190 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _latency_ChgEventVal
	.dw	word _latency_MinChg
	.debug	G _latency_RegInfo 33 0
_tcpTickTime_RegInfo::
	.dw	word _tcpTickTime
	.dw	0x17
	.dw	word (phys @1+198)
	.dw	phys @1+198 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _tcpTickTime_ChgEventVal
	.dw	word _tcpTickTime_MinChg
	.debug	G _tcpTickTime_RegInfo 33 0
_applLoopTime_RegInfo::
	.dw	word _applLoopTime
	.dw	0x18
	.dw	word (phys @1+210)
	.dw	phys @1+210 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _applLoopTime_ChgEventVal
	.dw	word _applLoopTime_MinChg
	.debug	G _applLoopTime_RegInfo 33 0
_kernelLoopTime_RegInfo::
	.dw	word _kernelLoopTime
	.dw	0x19
	.dw	word (phys @1+223)
	.dw	phys @1+223 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _kernelLoopTime_ChgEventVal
	.dw	word _kernelLoopTime_MinChg
	.debug	G _kernelLoopTime_RegInfo 33 0
_statusBitsFPGA_RegInfo::
	.dw	word _statusBitsFPGA
	.dw	0x1a
	.dw	word (phys @1+238)
	.dw	phys @1+238 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _statusBitsFPGA_ChgEventVal
	.dw	word _statusBitsFPGA_MinChg
	.debug	G _statusBitsFPGA_RegInfo 33 0
_statusBitsTuner_RegInfo::
	.dw	word _statusBitsTuner
	.dw	0x1b
	.dw	word (phys @1+253)
	.dw	phys @1+253 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _statusBitsTuner_ChgEventVal
	.dw	word _statusBitsTuner_MinChg
	.debug	G _statusBitsTuner_RegInfo 33 0
_statusBitsRF_RegInfo::
	.dw	word _statusBitsRF
	.dw	0x1c
	.dw	word (phys @1+269)
	.dw	phys @1+269 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _statusBitsRF_ChgEventVal
	.dw	word _statusBitsRF_MinChg
	.debug	G _statusBitsRF_RegInfo 33 0
_statusBitsILK_RegInfo::
	.dw	word _statusBitsILK
	.dw	0x1d
	.dw	word (phys @1+282)
	.dw	phys @1+282 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _statusBitsILK_ChgEventVal
	.dw	word _statusBitsILK_MinChg
	.debug	G _statusBitsILK_RegInfo 33 0
_spark0_ctr_RegInfo::
	.dw	word _spark0_ctr
	.dw	0x1e
	.dw	word (phys @1+296)
	.dw	phys @1+296 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _spark0_ctr_ChgEventVal
	.dw	word _spark0_ctr_MinChg
	.debug	G _spark0_ctr_RegInfo 33 0
_spark1_ctr_RegInfo::
	.dw	word _spark1_ctr
	.dw	0x1f
	.dw	word (phys @1+307)
	.dw	phys @1+307 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _spark1_ctr_ChgEventVal
	.dw	word _spark1_ctr_MinChg
	.debug	G _spark1_ctr_RegInfo 33 0
_cavlow_ctr_RegInfo::
	.dw	word _cavlow_ctr
	.dw	0x20
	.dw	word (phys @1+318)
	.dw	phys @1+318 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavlow_ctr_ChgEventVal
	.dw	word _cavlow_ctr_MinChg
	.debug	G _cavlow_ctr_RegInfo 33 0
_rev_high_ctr_RegInfo::
	.dw	word _rev_high_ctr
	.dw	0x21
	.dw	word (phys @1+329)
	.dw	phys @1+329 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _rev_high_ctr_ChgEventVal
	.dw	word _rev_high_ctr_MinChg
	.debug	G _rev_high_ctr_RegInfo 33 0
_dvdt_ctr_RegInfo::
	.dw	word _dvdt_ctr
	.dw	0x22
	.dw	word (phys @1+342)
	.dw	phys @1+342 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _dvdt_ctr_ChgEventVal
	.dw	word _dvdt_ctr_MinChg
	.debug	G _dvdt_ctr_RegInfo 33 0
_fwd_low_ctr_RegInfo::
	.dw	word _fwd_low_ctr
	.dw	0x23
	.dw	word (phys @1+351)
	.dw	phys @1+351 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwd_low_ctr_ChgEventVal
	.dw	word _fwd_low_ctr_MinChg
	.debug	G _fwd_low_ctr_RegInfo 33 0
_fwd_high_ctr_RegInfo::
	.dw	word _fwd_high_ctr
	.dw	0x24
	.dw	word (phys @1+363)
	.dw	phys @1+363 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwd_high_ctr_ChgEventVal
	.dw	word _fwd_high_ctr_MinChg
	.debug	G _fwd_high_ctr_RegInfo 33 0
_PLC_ctr_RegInfo::
	.dw	word _PLC_ctr
	.dw	0x25
	.dw	word (phys @1+376)
	.dw	phys @1+376 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _PLC_ctr_ChgEventVal
	.dw	word _PLC_ctr_MinChg
	.debug	G _PLC_ctr_RegInfo 33 0
_safety_ctr_RegInfo::
	.dw	word _safety_ctr
	.dw	0x26
	.dw	word (phys @1+384)
	.dw	phys @1+384 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _safety_ctr_ChgEventVal
	.dw	word _safety_ctr_MinChg
	.debug	G _safety_ctr_RegInfo 33 0
_fwdPhase_RegInfo::
	.dw	word _fwdPhase
	.dw	0x27
	.dw	word (phys @1+395)
	.dw	phys @1+395 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdPhase_ChgEventVal
	.dw	word _fwdPhase_MinChg
	.debug	G _fwdPhase_RegInfo 33 0
_revPhase_RegInfo::
	.dw	word _revPhase
	.dw	0x29
	.dw	word (phys @1+404)
	.dw	phys @1+404 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _revPhase_ChgEventVal
	.dw	word _revPhase_MinChg
	.debug	G _revPhase_RegInfo 33 0
_cavPhase_RegInfo::
	.dw	word _cavPhase
	.dw	0x2b
	.dw	word (phys @1+413)
	.dw	phys @1+413 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavPhase_ChgEventVal
	.dw	word _cavPhase_MinChg
	.debug	G _cavPhase_RegInfo 33 0
_ref0Phase_RegInfo::
	.dw	word _ref0Phase
	.dw	0x2d
	.dw	word (phys @1+422)
	.dw	phys @1+422 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref0Phase_ChgEventVal
	.dw	word _ref0Phase_MinChg
	.debug	G _ref0Phase_RegInfo 33 0
_spare0Phase_RegInfo::
	.dw	word _spare0Phase
	.dw	0x2f
	.dw	word (phys @1+432)
	.dw	phys @1+432 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare0Phase_ChgEventVal
	.dw	word _spare0Phase_MinChg
	.debug	G _spare0Phase_RegInfo 33 0
_spare1Phase_RegInfo::
	.dw	word _spare1Phase
	.dw	0x31
	.dw	word (phys @1+444)
	.dw	phys @1+444 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare1Phase_ChgEventVal
	.dw	word _spare1Phase_MinChg
	.debug	G _spare1Phase_RegInfo 33 0
_spare2Phase_RegInfo::
	.dw	word _spare2Phase
	.dw	0x33
	.dw	word (phys @1+456)
	.dw	phys @1+456 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare2Phase_ChgEventVal
	.dw	word _spare2Phase_MinChg
	.debug	G _spare2Phase_RegInfo 33 0
_ref1Phase_RegInfo::
	.dw	word _ref1Phase
	.dw	0x35
	.dw	word (phys @1+468)
	.dw	phys @1+468 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref1Phase_ChgEventVal
	.dw	word _ref1Phase_MinChg
	.debug	G _ref1Phase_RegInfo 33 0
_fwdAmpl_RegInfo::
	.dw	word _fwdAmpl
	.dw	0x37
	.dw	word (phys @1+478)
	.dw	phys @1+478 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdAmpl_ChgEventVal
	.dw	word _fwdAmpl_MinChg
	.debug	G _fwdAmpl_RegInfo 33 0
_revAmpl_RegInfo::
	.dw	word _revAmpl
	.dw	0x39
	.dw	word (phys @1+486)
	.dw	phys @1+486 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _revAmpl_ChgEventVal
	.dw	word _revAmpl_MinChg
	.debug	G _revAmpl_RegInfo 33 0
_cavAmpl_RegInfo::
	.dw	word _cavAmpl
	.dw	0x3b
	.dw	word (phys @1+494)
	.dw	phys @1+494 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavAmpl_ChgEventVal
	.dw	word _cavAmpl_MinChg
	.debug	G _cavAmpl_RegInfo 33 0
_ref0Ampl_RegInfo::
	.dw	word _ref0Ampl
	.dw	0x3d
	.dw	word (phys @1+502)
	.dw	phys @1+502 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref0Ampl_ChgEventVal
	.dw	word _ref0Ampl_MinChg
	.debug	G _ref0Ampl_RegInfo 33 0
_spare0Ampl_RegInfo::
	.dw	word _spare0Ampl
	.dw	0x3f
	.dw	word (phys @1+511)
	.dw	phys @1+511 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare0Ampl_ChgEventVal
	.dw	word _spare0Ampl_MinChg
	.debug	G _spare0Ampl_RegInfo 33 0
_spare1Ampl_RegInfo::
	.dw	word _spare1Ampl
	.dw	0x41
	.dw	word (phys @1+522)
	.dw	phys @1+522 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare1Ampl_ChgEventVal
	.dw	word _spare1Ampl_MinChg
	.debug	G _spare1Ampl_RegInfo 33 0
_spare2Ampl_RegInfo::
	.dw	word _spare2Ampl
	.dw	0x43
	.dw	word (phys @1+533)
	.dw	phys @1+533 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare2Ampl_ChgEventVal
	.dw	word _spare2Ampl_MinChg
	.debug	G _spare2Ampl_RegInfo 33 0
_ref1Ampl_RegInfo::
	.dw	word _ref1Ampl
	.dw	0x45
	.dw	word (phys @1+544)
	.dw	phys @1+544 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref1Ampl_ChgEventVal
	.dw	word _ref1Ampl_MinChg
	.debug	G _ref1Ampl_RegInfo 33 0
_maxFwdAmpl_RegInfo::
	.dw	word _maxFwdAmpl
	.dw	0x47
	.dw	word (phys @1+553)
	.dw	phys @1+553 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _maxFwdAmpl_ChgEventVal
	.dw	word _maxFwdAmpl_MinChg
	.debug	G _maxFwdAmpl_RegInfo 33 0
_maxRevAmpl_RegInfo::
	.dw	word _maxRevAmpl
	.dw	0x49
	.dw	word (phys @1+564)
	.dw	phys @1+564 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _maxRevAmpl_ChgEventVal
	.dw	word _maxRevAmpl_MinChg
	.debug	G _maxRevAmpl_RegInfo 33 0
_maxCavAmpl_RegInfo::
	.dw	word _maxCavAmpl
	.dw	0x4b
	.dw	word (phys @1+575)
	.dw	phys @1+575 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _maxCavAmpl_ChgEventVal
	.dw	word _maxCavAmpl_MinChg
	.debug	G _maxCavAmpl_RegInfo 33 0
_maxSpare0Ampl_RegInfo::
	.dw	word _maxSpare0Ampl
	.dw	0x4d
	.dw	word (phys @1+586)
	.dw	phys @1+586 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _maxSpare0Ampl_ChgEventVal
	.dw	word _maxSpare0Ampl_MinChg
	.debug	G _maxSpare0Ampl_RegInfo 33 0
_maxSpare1Ampl_RegInfo::
	.dw	word _maxSpare1Ampl
	.dw	0x4f
	.dw	word (phys @1+600)
	.dw	phys @1+600 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _maxSpare1Ampl_ChgEventVal
	.dw	word _maxSpare1Ampl_MinChg
	.debug	G _maxSpare1Ampl_RegInfo 33 0
_maxSpare2Ampl_RegInfo::
	.dw	word _maxSpare2Ampl
	.dw	0x51
	.dw	word (phys @1+614)
	.dw	phys @1+614 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _maxSpare2Ampl_ChgEventVal
	.dw	word _maxSpare2Ampl_MinChg
	.debug	G _maxSpare2Ampl_RegInfo 33 0
_maxOutAmpl_RegInfo::
	.dw	word _maxOutAmpl
	.dw	0x53
	.dw	word (phys @1+628)
	.dw	phys @1+628 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _maxOutAmpl_ChgEventVal
	.dw	word _maxOutAmpl_MinChg
	.debug	G _maxOutAmpl_RegInfo 33 0
_statusBitsDSP_RegInfo::
	.dw	word _statusBitsDSP
	.dw	0x55
	.dw	word (phys @1+639)
	.dw	phys @1+639 >> 16
	.dw	0
	.db	0x3
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _statusBitsDSP_ChgEventVal
	.dw	word _statusBitsDSP_MinChg
	.debug	G _statusBitsDSP_RegInfo 33 0
_dspLoad_RegInfo::
	.dw	word _dspLoad
	.dw	0x57
	.dw	word (phys @1+653)
	.dw	phys @1+653 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _dspLoad_ChgEventVal
	.dw	word _dspLoad_MinChg
	.debug	G _dspLoad_RegInfo 33 0
_pcbTemp_RegInfo::
	.dw	word _pcbTemp
	.dw	0x59
	.dw	word (phys @1+661)
	.dw	phys @1+661 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _pcbTemp_ChgEventVal
	.dw	word _pcbTemp_MinChg
	.debug	G _pcbTemp_RegInfo 33 0
_fan0Spd_RegInfo::
	.dw	word _fan0Spd
	.dw	0x5b
	.dw	word (phys @1+669)
	.dw	phys @1+669 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _fan0Spd_ChgEventVal
	.dw	word _fan0Spd_MinChg
	.debug	G _fan0Spd_RegInfo 33 0
_fan1Spd_RegInfo::
	.dw	word _fan1Spd
	.dw	0x5c
	.dw	word (phys @1+677)
	.dw	phys @1+677 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _fan1Spd_ChgEventVal
	.dw	word _fan1Spd_MinChg
	.debug	G _fan1Spd_RegInfo 33 0
_rfq_t1_output_RegInfo::
	.dw	word _rfq_t1_output
	.dw	0x5d
	.dw	word (phys @1+685)
	.dw	phys @1+685 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _rfq_t1_output_ChgEventVal
	.dw	word _rfq_t1_output_MinChg
	.debug	G _rfq_t1_output_RegInfo 33 0
_rfq_t2_output_RegInfo::
	.dw	word _rfq_t2_output
	.dw	0x5f
	.dw	word (phys @1+699)
	.dw	phys @1+699 >> 16
	.dw	0
	.db	0x4
	.db	0x2
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _rfq_t2_output_ChgEventVal
	.dw	word _rfq_t2_output_MinChg
	.debug	G _rfq_t2_output_RegInfo 33 0
_piezoReadPos_RegInfo::
	.dw	word _piezoReadPos
	.dw	0x61
	.dw	word (phys @1+713)
	.dw	phys @1+713 >> 16
	.dw	0
	.db	0
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _piezoReadPos_ChgEventVal
	.dw	word _piezoReadPos_MinChg
	.debug	G _piezoReadPos_RegInfo 33 0
_stepperReadPos_RegInfo::
	.dw	word _stepperReadPos
	.dw	0x62
	.dw	word (phys @1+726)
	.dw	phys @1+726 >> 16
	.dw	0
	.db	0
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _stepperReadPos_ChgEventVal
	.dw	word _stepperReadPos_MinChg
	.debug	G _stepperReadPos_RegInfo 33 0
_percentFFT_RegInfo::
	.dw	word _percentFFT
	.dw	0x63
	.dw	word (phys @1+741)
	.dw	phys @1+741 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _percentFFT_ChgEventVal
	.dw	word _percentFFT_MinChg
	.debug	G _percentFFT_RegInfo 33 0
_sizeFFT_RegInfo::
	.dw	word _sizeFFT
	.dw	0x64
	.dw	word (phys @1+752)
	.dw	phys @1+752 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _sizeFFT_ChgEventVal
	.dw	word _sizeFFT_MinChg
	.debug	G _sizeFFT_RegInfo 33 0
_romProgErrs_RegInfo::
	.dw	word _romProgErrs
	.dw	0x65
	.dw	word (phys @1+760)
	.dw	phys @1+760 >> 16
	.dw	0
	.db	0x1
	.db	0x1
	.db	0
	.db	0x2
	.db	0
	.db	0
	.dw	word _romProgErrs_ChgEventVal
	.dw	word _romProgErrs_MinChg
	.debug	G _romProgErrs_RegInfo 33 0
_timeZone__RegInfo::
	.dw	word _timeZone_
	.dw	0
	.dw	word (phys @1+772)
	.dw	phys @1+772 >> 16
	.dw	word _MB_timeZone
	.db	0
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _timeZone__ChgEventVal
	.dw	word _timeZone__MinChg
	.debug	G _timeZone__RegInfo 33 0
_curTime_RegInfo::
	.dw	word _curTime
	.dw	0x1
	.dw	word (phys @1+782)
	.dw	phys @1+782 >> 16
	.dw	word _setU32
	.db	0x3
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _curTime_ChgEventVal
	.dw	word _curTime_MinChg
	.debug	G _curTime_RegInfo 33 0
_ipAddr_RegInfo::
	.dw	word _ipAddr
	.dw	0x3
	.dw	word (phys @1+790)
	.dw	phys @1+790 >> 16
	.dw	word _setU32
	.db	0x3
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ipAddr_ChgEventVal
	.dw	word _ipAddr_MinChg
	.debug	G _ipAddr_RegInfo 33 0
_netmask_RegInfo::
	.dw	word _netmask
	.dw	0x5
	.dw	word (phys @1+797)
	.dw	phys @1+797 >> 16
	.dw	word _setU32
	.db	0x3
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _netmask_ChgEventVal
	.dw	word _netmask_MinChg
	.debug	G _netmask_RegInfo 33 0
_gateway_RegInfo::
	.dw	word _gateway
	.dw	0x7
	.dw	word (phys @1+805)
	.dw	phys @1+805 >> 16
	.dw	word _setU32
	.db	0x3
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _gateway_ChgEventVal
	.dw	word _gateway_MinChg
	.debug	G _gateway_RegInfo 33 0
_setACoil_RegInfo::
	.dw	word _setACoil
	.dw	0x9
	.dw	word (phys @1+813)
	.dw	phys @1+813 >> 16
	.dw	word _setU16
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _setACoil_ChgEventVal
	.dw	word _setACoil_MinChg
	.debug	G _setACoil_RegInfo 33 0
_resetACoil_RegInfo::
	.dw	word _resetACoil
	.dw	0xa
	.dw	word (phys @1+822)
	.dw	phys @1+822 >> 16
	.dw	word _setU16
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _resetACoil_ChgEventVal
	.dw	word _resetACoil_MinChg
	.debug	G _resetACoil_RegInfo 33 0
_digOutBits_RegInfo::
	.dw	word _digOutBits
	.dw	0xb
	.dw	word (phys @1+833)
	.dw	phys @1+833 >> 16
	.dw	word _setU32
	.db	0x3
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _digOutBits_ChgEventVal
	.dw	word _digOutBits_MinChg
	.debug	G _digOutBits_RegInfo 33 0
_cmdBitsStd_RegInfo::
	.dw	word _cmdBitsStd
	.dw	0xd
	.dw	word (phys @1+844)
	.dw	phys @1+844 >> 16
	.dw	word _MB_cmdBitsStd
	.db	0x3
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _cmdBitsStd_ChgEventVal
	.dw	word _cmdBitsStd_MinChg
	.debug	G _cmdBitsStd_RegInfo 33 0
_cmdBitsAppl_RegInfo::
	.dw	word _cmdBitsAppl
	.dw	0xf
	.dw	word (phys @1+855)
	.dw	phys @1+855 >> 16
	.dw	word _MB_cmdBitsAppl
	.db	0x3
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _cmdBitsAppl_ChgEventVal
	.dw	word _cmdBitsAppl_MinChg
	.debug	G _cmdBitsAppl_RegInfo 33 0
_ADC_err_rst_RegInfo::
	.dw	word _ADC_err_rst
	.dw	0x11
	.dw	word (phys @1+867)
	.dw	phys @1+867 >> 16
	.dw	word _setU16
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ADC_err_rst_ChgEventVal
	.dw	word _ADC_err_rst_MinChg
	.debug	G _ADC_err_rst_RegInfo 33 0
_RF_soft_on_RegInfo::
	.dw	word _RF_soft_on
	.dw	0x12
	.dw	word (phys @1+879)
	.dw	phys @1+879 >> 16
	.dw	word _MB_RF_soft_on
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _RF_soft_on_ChgEventVal
	.dw	word _RF_soft_on_MinChg
	.debug	G _RF_soft_on_RegInfo 33 0
_RF_ilk_rst_RegInfo::
	.dw	word _RF_ilk_rst
	.dw	0x13
	.dw	word (phys @1+890)
	.dw	phys @1+890 >> 16
	.dw	word _setU16
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _RF_ilk_rst_ChgEventVal
	.dw	word _RF_ilk_rst_MinChg
	.debug	G _RF_ilk_rst_RegInfo 33 0
_ilk_ctr_rst1_RegInfo::
	.dw	word _ilk_ctr_rst1
	.dw	0x14
	.dw	word (phys @1+901)
	.dw	phys @1+901 >> 16
	.dw	word _setU16
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ilk_ctr_rst1_ChgEventVal
	.dw	word _ilk_ctr_rst1_MinChg
	.debug	G _ilk_ctr_rst1_RegInfo 33 0
_ilk_ctr_rst2_RegInfo::
	.dw	word _ilk_ctr_rst2
	.dw	0x15
	.dw	word (phys @1+914)
	.dw	phys @1+914 >> 16
	.dw	word _setU16
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ilk_ctr_rst2_ChgEventVal
	.dw	word _ilk_ctr_rst2_MinChg
	.debug	G _ilk_ctr_rst2_RegInfo 33 0
_trigPos_RegInfo::
	.dw	word _trigPos
	.dw	0x16
	.dw	word (phys @1+927)
	.dw	phys @1+927 >> 16
	.dw	word _MB_DSP_REG
	.db	0x3
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _trigPos_ChgEventVal
	.dw	word _trigPos_MinChg
	.debug	G _trigPos_RegInfo 33 0
_warmLevel_RegInfo::
	.dw	word _warmLevel
	.dw	0x18
	.dw	word (phys @1+935)
	.dw	phys @1+935 >> 16
	.dw	word _MB_maxAmpl
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _warmLevel_ChgEventVal
	.dw	word _warmLevel_MinChg
	.debug	G _warmLevel_RegInfo 33 0
_warmTime_RegInfo::
	.dw	word _warmTime
	.dw	0x1a
	.dw	word (phys @1+945)
	.dw	phys @1+945 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _warmTime_ChgEventVal
	.dw	word _warmTime_MinChg
	.debug	G _warmTime_RegInfo 33 0
_rampTime_RegInfo::
	.dw	word _rampTime
	.dw	0x1c
	.dw	word (phys @1+954)
	.dw	phys @1+954 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rampTime_ChgEventVal
	.dw	word _rampTime_MinChg
	.debug	G _rampTime_RegInfo 33 0
_fwdAtten_RegInfo::
	.dw	word _fwdAtten
	.dw	0x1e
	.dw	word (phys @1+963)
	.dw	phys @1+963 >> 16
	.dw	word _MB_DSP_REG
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdAtten_ChgEventVal
	.dw	word _fwdAtten_MinChg
	.debug	G _fwdAtten_RegInfo 33 0
_revAtten_RegInfo::
	.dw	word _revAtten
	.dw	0x1f
	.dw	word (phys @1+972)
	.dw	phys @1+972 >> 16
	.dw	word _MB_DSP_REG
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _revAtten_ChgEventVal
	.dw	word _revAtten_MinChg
	.debug	G _revAtten_RegInfo 33 0
_cavAtten_RegInfo::
	.dw	word _cavAtten
	.dw	0x20
	.dw	word (phys @1+981)
	.dw	phys @1+981 >> 16
	.dw	word _MB_DSP_REG
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavAtten_ChgEventVal
	.dw	word _cavAtten_MinChg
	.debug	G _cavAtten_RegInfo 33 0
_spare0Atten_RegInfo::
	.dw	word _spare0Atten
	.dw	0x21
	.dw	word (phys @1+990)
	.dw	phys @1+990 >> 16
	.dw	word _MB_DSP_REG
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare0Atten_ChgEventVal
	.dw	word _spare0Atten_MinChg
	.debug	G _spare0Atten_RegInfo 33 0
_spare1Atten_RegInfo::
	.dw	word _spare1Atten
	.dw	0x22
	.dw	word (phys @1+1002)
	.dw	phys @1+1002 >> 16
	.dw	word _MB_DSP_REG
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare1Atten_ChgEventVal
	.dw	word _spare1Atten_MinChg
	.debug	G _spare1Atten_RegInfo 33 0
_spare2Atten_RegInfo::
	.dw	word _spare2Atten
	.dw	0x23
	.dw	word (phys @1+1014)
	.dw	phys @1+1014 >> 16
	.dw	word _MB_DSP_REG
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare2Atten_ChgEventVal
	.dw	word _spare2Atten_MinChg
	.debug	G _spare2Atten_RegInfo 33 0
_outAtten_RegInfo::
	.dw	word _outAtten
	.dw	0x24
	.dw	word (phys @1+1026)
	.dw	phys @1+1026 >> 16
	.dw	word _MB_DSP_REG
	.db	0x1
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _outAtten_ChgEventVal
	.dw	word _outAtten_MinChg
	.debug	G _outAtten_RegInfo 33 0
_phaseSet_RegInfo::
	.dw	word _phaseSet
	.dw	0x25
	.dw	word (phys @1+1035)
	.dw	phys @1+1035 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _phaseSet_ChgEventVal
	.dw	word _phaseSet_MinChg
	.debug	G _phaseSet_RegInfo 33 0
_amplSet_RegInfo::
	.dw	word _amplSet
	.dw	0x27
	.dw	word (phys @1+1044)
	.dw	phys @1+1044 >> 16
	.dw	word _MB_maxAmpl
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _amplSet_ChgEventVal
	.dw	word _amplSet_MinChg
	.debug	G _amplSet_RegInfo 33 0
_pulseFreq_RegInfo::
	.dw	word _pulseFreq
	.dw	0x29
	.dw	word (phys @1+1052)
	.dw	phys @1+1052 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _pulseFreq_ChgEventVal
	.dw	word _pulseFreq_MinChg
	.debug	G _pulseFreq_RegInfo 33 0
_pulseDuty_RegInfo::
	.dw	word _pulseDuty
	.dw	0x2b
	.dw	word (phys @1+1062)
	.dw	phys @1+1062 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _pulseDuty_ChgEventVal
	.dw	word _pulseDuty_MinChg
	.debug	G _pulseDuty_RegInfo 33 0
_rf_P_RegInfo::
	.dw	word _rf_P
	.dw	0x2d
	.dw	word (phys @1+1072)
	.dw	phys @1+1072 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rf_P_ChgEventVal
	.dw	word _rf_P_MinChg
	.debug	G _rf_P_RegInfo 33 0
_rf_I_RegInfo::
	.dw	word _rf_I
	.dw	0x2f
	.dw	word (phys @1+1077)
	.dw	phys @1+1077 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rf_I_ChgEventVal
	.dw	word _rf_I_MinChg
	.debug	G _rf_I_RegInfo 33 0
_rf_D_RegInfo::
	.dw	word _rf_D
	.dw	0x31
	.dw	word (phys @1+1082)
	.dw	phys @1+1082 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rf_D_ChgEventVal
	.dw	word _rf_D_MinChg
	.debug	G _rf_D_RegInfo 33 0
_tuner_P_RegInfo::
	.dw	word _tuner_P
	.dw	0x33
	.dw	word (phys @1+1087)
	.dw	phys @1+1087 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _tuner_P_ChgEventVal
	.dw	word _tuner_P_MinChg
	.debug	G _tuner_P_RegInfo 33 0
_tuner_I_RegInfo::
	.dw	word _tuner_I
	.dw	0x35
	.dw	word (phys @1+1095)
	.dw	phys @1+1095 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _tuner_I_ChgEventVal
	.dw	word _tuner_I_MinChg
	.debug	G _tuner_I_RegInfo 33 0
_tuner_D_RegInfo::
	.dw	word _tuner_D
	.dw	0x37
	.dw	word (phys @1+1103)
	.dw	phys @1+1103 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _tuner_D_ChgEventVal
	.dw	word _tuner_D_MinChg
	.debug	G _tuner_D_RegInfo 33 0
_rf_B0_RegInfo::
	.dw	word _rf_B0
	.dw	0x39
	.dw	word (phys @1+1111)
	.dw	phys @1+1111 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rf_B0_ChgEventVal
	.dw	word _rf_B0_MinChg
	.debug	G _rf_B0_RegInfo 33 0
_rf_WCtrl_RegInfo::
	.dw	word _rf_WCtrl
	.dw	0x3b
	.dw	word (phys @1+1117)
	.dw	phys @1+1117 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rf_WCtrl_ChgEventVal
	.dw	word _rf_WCtrl_MinChg
	.debug	G _rf_WCtrl_RegInfo 33 0
_rf_WObs_RegInfo::
	.dw	word _rf_WObs
	.dw	0x3d
	.dw	word (phys @1+1126)
	.dw	phys @1+1126 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rf_WObs_ChgEventVal
	.dw	word _rf_WObs_MinChg
	.debug	G _rf_WObs_RegInfo 33 0
_tuner_B0_RegInfo::
	.dw	word _tuner_B0
	.dw	0x3f
	.dw	word (phys @1+1134)
	.dw	phys @1+1134 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _tuner_B0_ChgEventVal
	.dw	word _tuner_B0_MinChg
	.debug	G _tuner_B0_RegInfo 33 0
_tuner_WCtrl_RegInfo::
	.dw	word _tuner_WCtrl
	.dw	0x41
	.dw	word (phys @1+1143)
	.dw	phys @1+1143 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _tuner_WCtrl_ChgEventVal
	.dw	word _tuner_WCtrl_MinChg
	.debug	G _tuner_WCtrl_RegInfo 33 0
_tuner_WObs_RegInfo::
	.dw	word _tuner_WObs
	.dw	0x43
	.dw	word (phys @1+1155)
	.dw	phys @1+1155 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _tuner_WObs_ChgEventVal
	.dw	word _tuner_WObs_MinChg
	.debug	G _tuner_WObs_RegInfo 33 0
_pulseAmplSet_RegInfo::
	.dw	word _pulseAmplSet
	.dw	0x45
	.dw	word (phys @1+1166)
	.dw	phys @1+1166 >> 16
	.dw	word _MB_maxAmpl
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _pulseAmplSet_ChgEventVal
	.dw	word _pulseAmplSet_MinChg
	.debug	G _pulseAmplSet_RegInfo 33 0
_pulsePhaseSet_RegInfo::
	.dw	word _pulsePhaseSet
	.dw	0x47
	.dw	word (phys @1+1179)
	.dw	phys @1+1179 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _pulsePhaseSet_ChgEventVal
	.dw	word _pulsePhaseSet_MinChg
	.debug	G _pulsePhaseSet_RegInfo 33 0
_cavLowIlkLevel_RegInfo::
	.dw	word _cavLowIlkLevel
	.dw	0x49
	.dw	word (phys @1+1193)
	.dw	phys @1+1193 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavLowIlkLevel_ChgEventVal
	.dw	word _cavLowIlkLevel_MinChg
	.debug	G _cavLowIlkLevel_RegInfo 33 0
_cavLowIlkTime_RegInfo::
	.dw	word _cavLowIlkTime
	.dw	0x4b
	.dw	word (phys @1+1208)
	.dw	phys @1+1208 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavLowIlkTime_ChgEventVal
	.dw	word _cavLowIlkTime_MinChg
	.debug	G _cavLowIlkTime_RegInfo 33 0
_dvdtIlkRate_RegInfo::
	.dw	word _dvdtIlkRate
	.dw	0x4d
	.dw	word (phys @1+1222)
	.dw	phys @1+1222 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _dvdtIlkRate_ChgEventVal
	.dw	word _dvdtIlkRate_MinChg
	.debug	G _dvdtIlkRate_RegInfo 33 0
_revHighIlkRatio_RegInfo::
	.dw	word _revHighIlkRatio
	.dw	0x4f
	.dw	word (phys @1+1234)
	.dw	phys @1+1234 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _revHighIlkRatio_ChgEventVal
	.dw	word _revHighIlkRatio_MinChg
	.debug	G _revHighIlkRatio_RegInfo 33 0
_revHighIlkTime_RegInfo::
	.dw	word _revHighIlkTime
	.dw	0x51
	.dw	word (phys @1+1250)
	.dw	phys @1+1250 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _revHighIlkTime_ChgEventVal
	.dw	word _revHighIlkTime_MinChg
	.debug	G _revHighIlkTime_RegInfo 33 0
_fwdLowIlkRatio_RegInfo::
	.dw	word _fwdLowIlkRatio
	.dw	0x53
	.dw	word (phys @1+1265)
	.dw	phys @1+1265 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdLowIlkRatio_ChgEventVal
	.dw	word _fwdLowIlkRatio_MinChg
	.debug	G _fwdLowIlkRatio_RegInfo 33 0
_fwdLowIlkTime_RegInfo::
	.dw	word _fwdLowIlkTime
	.dw	0x55
	.dw	word (phys @1+1280)
	.dw	phys @1+1280 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdLowIlkTime_ChgEventVal
	.dw	word _fwdLowIlkTime_MinChg
	.debug	G _fwdLowIlkTime_RegInfo 33 0
_fwdHighIlkLevel_RegInfo::
	.dw	word _fwdHighIlkLevel
	.dw	0x57
	.dw	word (phys @1+1294)
	.dw	phys @1+1294 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdHighIlkLevel_ChgEventVal
	.dw	word _fwdHighIlkLevel_MinChg
	.debug	G _fwdHighIlkLevel_RegInfo 33 0
_fwdHighIlkTime_RegInfo::
	.dw	word _fwdHighIlkTime
	.dw	0x59
	.dw	word (phys @1+1310)
	.dw	phys @1+1310 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdHighIlkTime_ChgEventVal
	.dw	word _fwdHighIlkTime_MinChg
	.debug	G _fwdHighIlkTime_RegInfo 33 0
_fwdCavPhase_RegInfo::
	.dw	word _fwdCavPhase
	.dw	0x5b
	.dw	word (phys @1+1325)
	.dw	phys @1+1325 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdCavPhase_ChgEventVal
	.dw	word _fwdCavPhase_MinChg
	.debug	G _fwdCavPhase_RegInfo 33 0
_ffPhase_RegInfo::
	.dw	word _ffPhase
	.dw	0x5d
	.dw	word (phys @1+1337)
	.dw	phys @1+1337 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ffPhase_ChgEventVal
	.dw	word _ffPhase_MinChg
	.debug	G _ffPhase_RegInfo 33 0
_ffGain_RegInfo::
	.dw	word _ffGain
	.dw	0x5f
	.dw	word (phys @1+1345)
	.dw	phys @1+1345 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ffGain_ChgEventVal
	.dw	word _ffGain_MinChg
	.debug	G _ffGain_RegInfo 33 0
_freqOffset_RegInfo::
	.dw	word _freqOffset
	.dw	0x61
	.dw	word (phys @1+1352)
	.dw	phys @1+1352 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _freqOffset_ChgEventVal
	.dw	word _freqOffset_MinChg
	.debug	G _freqOffset_RegInfo 33 0
_rfq_manTuneSet_RegInfo::
	.dw	word _rfq_manTuneSet
	.dw	0x63
	.dw	word (phys @1+1363)
	.dw	phys @1+1363 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rfq_manTuneSet_ChgEventVal
	.dw	word _rfq_manTuneSet_MinChg
	.debug	G _rfq_manTuneSet_RegInfo 33 0
_rfq_t1_offset_RegInfo::
	.dw	word _rfq_t1_offset
	.dw	0x65
	.dw	word (phys @1+1378)
	.dw	phys @1+1378 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rfq_t1_offset_ChgEventVal
	.dw	word _rfq_t1_offset_MinChg
	.debug	G _rfq_t1_offset_RegInfo 33 0
_rfq_t2_offset_RegInfo::
	.dw	word _rfq_t2_offset
	.dw	0x67
	.dw	word (phys @1+1392)
	.dw	phys @1+1392 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rfq_t2_offset_ChgEventVal
	.dw	word _rfq_t2_offset_MinChg
	.debug	G _rfq_t2_offset_RegInfo 33 0
_rfq_t1_feedback_RegInfo::
	.dw	word _rfq_t1_feedback
	.dw	0x69
	.dw	word (phys @1+1406)
	.dw	phys @1+1406 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rfq_t1_feedback_ChgEventVal
	.dw	word _rfq_t1_feedback_MinChg
	.debug	G _rfq_t1_feedback_RegInfo 33 0
_rfq_t2_feedback_RegInfo::
	.dw	word _rfq_t2_feedback
	.dw	0x6b
	.dw	word (phys @1+1422)
	.dw	phys @1+1422 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _rfq_t2_feedback_ChgEventVal
	.dw	word _rfq_t2_feedback_MinChg
	.debug	G _rfq_t2_feedback_RegInfo 33 0
_setDSPState_RegInfo::
	.dw	word _setDSPState
	.dw	0x6d
	.dw	word (phys @1+1438)
	.dw	phys @1+1438 >> 16
	.dw	word _MB_setDSPState
	.db	0x3
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _setDSPState_ChgEventVal
	.dw	word _setDSPState_MinChg
	.debug	G _setDSPState_RegInfo 33 0
_piezoSetPos_RegInfo::
	.dw	word _piezoSetPos
	.dw	0x6f
	.dw	word (phys @1+1450)
	.dw	phys @1+1450 >> 16
	.dw	word _MB_DSP_REG
	.db	0
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _piezoSetPos_ChgEventVal
	.dw	word _piezoSetPos_MinChg
	.debug	G _piezoSetPos_RegInfo 33 0
_stepperSetPos_RegInfo::
	.dw	word _stepperSetPos
	.dw	0x70
	.dw	word (phys @1+1462)
	.dw	phys @1+1462 >> 16
	.dw	word _MB_DSP_REG
	.db	0
	.db	0x1
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _stepperSetPos_ChgEventVal
	.dw	word _stepperSetPos_MinChg
	.debug	G _stepperSetPos_RegInfo 33 0
_fwdMagCalib_RegInfo::
	.dw	word _fwdMagCalib
	.dw	0x71
	.dw	word (phys @1+1476)
	.dw	phys @1+1476 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdMagCalib_ChgEventVal
	.dw	word _fwdMagCalib_MinChg
	.debug	G _fwdMagCalib_RegInfo 33 0
_fwdPhaseCalib_RegInfo::
	.dw	word _fwdPhaseCalib
	.dw	0x73
	.dw	word (phys @1+1488)
	.dw	phys @1+1488 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdPhaseCalib_ChgEventVal
	.dw	word _fwdPhaseCalib_MinChg
	.debug	G _fwdPhaseCalib_RegInfo 33 0
_revMagCalib_RegInfo::
	.dw	word _revMagCalib
	.dw	0x75
	.dw	word (phys @1+1502)
	.dw	phys @1+1502 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _revMagCalib_ChgEventVal
	.dw	word _revMagCalib_MinChg
	.debug	G _revMagCalib_RegInfo 33 0
_revPhaseCalib_RegInfo::
	.dw	word _revPhaseCalib
	.dw	0x77
	.dw	word (phys @1+1514)
	.dw	phys @1+1514 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _revPhaseCalib_ChgEventVal
	.dw	word _revPhaseCalib_MinChg
	.debug	G _revPhaseCalib_RegInfo 33 0
_cavMagCalib_RegInfo::
	.dw	word _cavMagCalib
	.dw	0x79
	.dw	word (phys @1+1528)
	.dw	phys @1+1528 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavMagCalib_ChgEventVal
	.dw	word _cavMagCalib_MinChg
	.debug	G _cavMagCalib_RegInfo 33 0
_cavPhaseCalib_RegInfo::
	.dw	word _cavPhaseCalib
	.dw	0x7b
	.dw	word (phys @1+1540)
	.dw	phys @1+1540 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavPhaseCalib_ChgEventVal
	.dw	word _cavPhaseCalib_MinChg
	.debug	G _cavPhaseCalib_RegInfo 33 0
_spare0MagCalib_RegInfo::
	.dw	word _spare0MagCalib
	.dw	0x7d
	.dw	word (phys @1+1554)
	.dw	phys @1+1554 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare0MagCalib_ChgEventVal
	.dw	word _spare0MagCalib_MinChg
	.debug	G _spare0MagCalib_RegInfo 33 0
_spare0PhaseCalib_RegInfo::
	.dw	word _spare0PhaseCalib
	.dw	0x7f
	.dw	word (phys @1+1569)
	.dw	phys @1+1569 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare0PhaseCalib_ChgEventVal
	.dw	word _spare0PhaseCalib_MinChg
	.debug	G _spare0PhaseCalib_RegInfo 33 0
_spare1MagCalib_RegInfo::
	.dw	word _spare1MagCalib
	.dw	0x81
	.dw	word (phys @1+1586)
	.dw	phys @1+1586 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare1MagCalib_ChgEventVal
	.dw	word _spare1MagCalib_MinChg
	.debug	G _spare1MagCalib_RegInfo 33 0
_spare1PhaseCalib_RegInfo::
	.dw	word _spare1PhaseCalib
	.dw	0x83
	.dw	word (phys @1+1601)
	.dw	phys @1+1601 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare1PhaseCalib_ChgEventVal
	.dw	word _spare1PhaseCalib_MinChg
	.debug	G _spare1PhaseCalib_RegInfo 33 0
_spare2MagCalib_RegInfo::
	.dw	word _spare2MagCalib
	.dw	0x85
	.dw	word (phys @1+1618)
	.dw	phys @1+1618 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare2MagCalib_ChgEventVal
	.dw	word _spare2MagCalib_MinChg
	.debug	G _spare2MagCalib_RegInfo 33 0
_spare2PhaseCalib_RegInfo::
	.dw	word _spare2PhaseCalib
	.dw	0x87
	.dw	word (phys @1+1633)
	.dw	phys @1+1633 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare2PhaseCalib_ChgEventVal
	.dw	word _spare2PhaseCalib_MinChg
	.debug	G _spare2PhaseCalib_RegInfo 33 0
_outMagCalib0_RegInfo::
	.dw	word _outMagCalib0
	.dw	0x89
	.dw	word (phys @1+1650)
	.dw	phys @1+1650 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _outMagCalib0_ChgEventVal
	.dw	word _outMagCalib0_MinChg
	.debug	G _outMagCalib0_RegInfo 33 0
_outPhaseCalib0_RegInfo::
	.dw	word _outPhaseCalib0
	.dw	0x8b
	.dw	word (phys @1+1663)
	.dw	phys @1+1663 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _outPhaseCalib0_ChgEventVal
	.dw	word _outPhaseCalib0_MinChg
	.debug	G _outPhaseCalib0_RegInfo 33 0
_outMagCalib1_RegInfo::
	.dw	word _outMagCalib1
	.dw	0x8d
	.dw	word (phys @1+1678)
	.dw	phys @1+1678 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _outMagCalib1_ChgEventVal
	.dw	word _outMagCalib1_MinChg
	.debug	G _outMagCalib1_RegInfo 33 0
_outPhaseCalib1_RegInfo::
	.dw	word _outPhaseCalib1
	.dw	0x8f
	.dw	word (phys @1+1691)
	.dw	phys @1+1691 >> 16
	.dw	word _MB_DSP_REG
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _outPhaseCalib1_ChgEventVal
	.dw	word _outPhaseCalib1_MinChg
	.debug	G _outPhaseCalib1_RegInfo 33 0
_fwdPhaseGain_RegInfo::
	.dw	word _fwdPhaseGain
	.dw	0x91
	.dw	word (phys @1+1706)
	.dw	phys @1+1706 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdPhaseGain_ChgEventVal
	.dw	word _fwdPhaseGain_MinChg
	.debug	G _fwdPhaseGain_RegInfo 33 0
_fwdPhaseOffset_RegInfo::
	.dw	word _fwdPhaseOffset
	.dw	0x93
	.dw	word (phys @1+1719)
	.dw	phys @1+1719 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdPhaseOffset_ChgEventVal
	.dw	word _fwdPhaseOffset_MinChg
	.debug	G _fwdPhaseOffset_RegInfo 33 0
_fwdAmplGain_RegInfo::
	.dw	word _fwdAmplGain
	.dw	0x95
	.dw	word (phys @1+1734)
	.dw	phys @1+1734 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdAmplGain_ChgEventVal
	.dw	word _fwdAmplGain_MinChg
	.debug	G _fwdAmplGain_RegInfo 33 0
_fwdAmplOffset_RegInfo::
	.dw	word _fwdAmplOffset
	.dw	0x97
	.dw	word (phys @1+1746)
	.dw	phys @1+1746 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _fwdAmplOffset_ChgEventVal
	.dw	word _fwdAmplOffset_MinChg
	.debug	G _fwdAmplOffset_RegInfo 33 0
_revPhaseGain_RegInfo::
	.dw	word _revPhaseGain
	.dw	0x99
	.dw	word (phys @1+1760)
	.dw	phys @1+1760 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _revPhaseGain_ChgEventVal
	.dw	word _revPhaseGain_MinChg
	.debug	G _revPhaseGain_RegInfo 33 0
_revPhaseOffset_RegInfo::
	.dw	word _revPhaseOffset
	.dw	0x9b
	.dw	word (phys @1+1773)
	.dw	phys @1+1773 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _revPhaseOffset_ChgEventVal
	.dw	word _revPhaseOffset_MinChg
	.debug	G _revPhaseOffset_RegInfo 33 0
_revAmplGain_RegInfo::
	.dw	word _revAmplGain
	.dw	0x9d
	.dw	word (phys @1+1788)
	.dw	phys @1+1788 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _revAmplGain_ChgEventVal
	.dw	word _revAmplGain_MinChg
	.debug	G _revAmplGain_RegInfo 33 0
_revAmplOffset_RegInfo::
	.dw	word _revAmplOffset
	.dw	0x9f
	.dw	word (phys @1+1800)
	.dw	phys @1+1800 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _revAmplOffset_ChgEventVal
	.dw	word _revAmplOffset_MinChg
	.debug	G _revAmplOffset_RegInfo 33 0
_cavPhaseGain_RegInfo::
	.dw	word _cavPhaseGain
	.dw	0xa1
	.dw	word (phys @1+1814)
	.dw	phys @1+1814 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavPhaseGain_ChgEventVal
	.dw	word _cavPhaseGain_MinChg
	.debug	G _cavPhaseGain_RegInfo 33 0
_cavPhaseOffset_RegInfo::
	.dw	word _cavPhaseOffset
	.dw	0xa3
	.dw	word (phys @1+1827)
	.dw	phys @1+1827 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavPhaseOffset_ChgEventVal
	.dw	word _cavPhaseOffset_MinChg
	.debug	G _cavPhaseOffset_RegInfo 33 0
_cavAmplGain_RegInfo::
	.dw	word _cavAmplGain
	.dw	0xa5
	.dw	word (phys @1+1842)
	.dw	phys @1+1842 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavAmplGain_ChgEventVal
	.dw	word _cavAmplGain_MinChg
	.debug	G _cavAmplGain_RegInfo 33 0
_cavAmplOffset_RegInfo::
	.dw	word _cavAmplOffset
	.dw	0xa7
	.dw	word (phys @1+1854)
	.dw	phys @1+1854 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _cavAmplOffset_ChgEventVal
	.dw	word _cavAmplOffset_MinChg
	.debug	G _cavAmplOffset_RegInfo 33 0
_ref0PhaseGain_RegInfo::
	.dw	word _ref0PhaseGain
	.dw	0xa9
	.dw	word (phys @1+1868)
	.dw	phys @1+1868 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref0PhaseGain_ChgEventVal
	.dw	word _ref0PhaseGain_MinChg
	.debug	G _ref0PhaseGain_RegInfo 33 0
_ref0PhaseOffset_RegInfo::
	.dw	word _ref0PhaseOffset
	.dw	0xab
	.dw	word (phys @1+1882)
	.dw	phys @1+1882 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref0PhaseOffset_ChgEventVal
	.dw	word _ref0PhaseOffset_MinChg
	.debug	G _ref0PhaseOffset_RegInfo 33 0
_ref0AmplGain_RegInfo::
	.dw	word _ref0AmplGain
	.dw	0xad
	.dw	word (phys @1+1898)
	.dw	phys @1+1898 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref0AmplGain_ChgEventVal
	.dw	word _ref0AmplGain_MinChg
	.debug	G _ref0AmplGain_RegInfo 33 0
_ref0AmplOffset_RegInfo::
	.dw	word _ref0AmplOffset
	.dw	0xaf
	.dw	word (phys @1+1911)
	.dw	phys @1+1911 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref0AmplOffset_ChgEventVal
	.dw	word _ref0AmplOffset_MinChg
	.debug	G _ref0AmplOffset_RegInfo 33 0
_spare0PhaseGain_RegInfo::
	.dw	word _spare0PhaseGain
	.dw	0xb1
	.dw	word (phys @1+1926)
	.dw	phys @1+1926 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare0PhaseGain_ChgEventVal
	.dw	word _spare0PhaseGain_MinChg
	.debug	G _spare0PhaseGain_RegInfo 33 0
_spare0PhaseOffset_RegInfo::
	.dw	word _spare0PhaseOffset
	.dw	0xb3
	.dw	word (phys @1+1942)
	.dw	phys @1+1942 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare0PhaseOffset_ChgEventVal
	.dw	word _spare0PhaseOffset_MinChg
	.debug	G _spare0PhaseOffset_RegInfo 33 0
_spare0AmplGain_RegInfo::
	.dw	word _spare0AmplGain
	.dw	0xb5
	.dw	word (phys @1+1960)
	.dw	phys @1+1960 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare0AmplGain_ChgEventVal
	.dw	word _spare0AmplGain_MinChg
	.debug	G _spare0AmplGain_RegInfo 33 0
_spare0AmplOffset_RegInfo::
	.dw	word _spare0AmplOffset
	.dw	0xb7
	.dw	word (phys @1+1975)
	.dw	phys @1+1975 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare0AmplOffset_ChgEventVal
	.dw	word _spare0AmplOffset_MinChg
	.debug	G _spare0AmplOffset_RegInfo 33 0
_spare1PhaseGain_RegInfo::
	.dw	word _spare1PhaseGain
	.dw	0xb9
	.dw	word (phys @1+1992)
	.dw	phys @1+1992 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare1PhaseGain_ChgEventVal
	.dw	word _spare1PhaseGain_MinChg
	.debug	G _spare1PhaseGain_RegInfo 33 0
_spare1PhaseOffset_RegInfo::
	.dw	word _spare1PhaseOffset
	.dw	0xbb
	.dw	word (phys @1+2008)
	.dw	phys @1+2008 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare1PhaseOffset_ChgEventVal
	.dw	word _spare1PhaseOffset_MinChg
	.debug	G _spare1PhaseOffset_RegInfo 33 0
_spare1AmplGain_RegInfo::
	.dw	word _spare1AmplGain
	.dw	0xbd
	.dw	word (phys @1+2026)
	.dw	phys @1+2026 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare1AmplGain_ChgEventVal
	.dw	word _spare1AmplGain_MinChg
	.debug	G _spare1AmplGain_RegInfo 33 0
_spare1AmplOffset_RegInfo::
	.dw	word _spare1AmplOffset
	.dw	0xbf
	.dw	word (phys @1+2041)
	.dw	phys @1+2041 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare1AmplOffset_ChgEventVal
	.dw	word _spare1AmplOffset_MinChg
	.debug	G _spare1AmplOffset_RegInfo 33 0
_spare2PhaseGain_RegInfo::
	.dw	word _spare2PhaseGain
	.dw	0xc1
	.dw	word (phys @1+2058)
	.dw	phys @1+2058 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare2PhaseGain_ChgEventVal
	.dw	word _spare2PhaseGain_MinChg
	.debug	G _spare2PhaseGain_RegInfo 33 0
_spare2PhaseOffset_RegInfo::
	.dw	word _spare2PhaseOffset
	.dw	0xc3
	.dw	word (phys @1+2074)
	.dw	phys @1+2074 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare2PhaseOffset_ChgEventVal
	.dw	word _spare2PhaseOffset_MinChg
	.debug	G _spare2PhaseOffset_RegInfo 33 0
_spare2AmplGain_RegInfo::
	.dw	word _spare2AmplGain
	.dw	0xc5
	.dw	word (phys @1+2092)
	.dw	phys @1+2092 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare2AmplGain_ChgEventVal
	.dw	word _spare2AmplGain_MinChg
	.debug	G _spare2AmplGain_RegInfo 33 0
_spare2AmplOffset_RegInfo::
	.dw	word _spare2AmplOffset
	.dw	0xc7
	.dw	word (phys @1+2107)
	.dw	phys @1+2107 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _spare2AmplOffset_ChgEventVal
	.dw	word _spare2AmplOffset_MinChg
	.debug	G _spare2AmplOffset_RegInfo 33 0
_ref1PhaseGain_RegInfo::
	.dw	word _ref1PhaseGain
	.dw	0xc9
	.dw	word (phys @1+2124)
	.dw	phys @1+2124 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref1PhaseGain_ChgEventVal
	.dw	word _ref1PhaseGain_MinChg
	.debug	G _ref1PhaseGain_RegInfo 33 0
_ref1PhaseOffset_RegInfo::
	.dw	word _ref1PhaseOffset
	.dw	0xcb
	.dw	word (phys @1+2138)
	.dw	phys @1+2138 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref1PhaseOffset_ChgEventVal
	.dw	word _ref1PhaseOffset_MinChg
	.debug	G _ref1PhaseOffset_RegInfo 33 0
_ref1AmplGain_RegInfo::
	.dw	word _ref1AmplGain
	.dw	0xcd
	.dw	word (phys @1+2154)
	.dw	phys @1+2154 >> 16
	.dw	word _MB_setGain
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref1AmplGain_ChgEventVal
	.dw	word _ref1AmplGain_MinChg
	.debug	G _ref1AmplGain_RegInfo 33 0
_ref1AmplOffset_RegInfo::
	.dw	word _ref1AmplOffset
	.dw	0xcf
	.dw	word (phys @1+2167)
	.dw	phys @1+2167 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _ref1AmplOffset_ChgEventVal
	.dw	word _ref1AmplOffset_MinChg
	.debug	G _ref1AmplOffset_RegInfo 33 0
_maxAmplSet_RegInfo::
	.dw	word _maxAmplSet
	.dw	0xd1
	.dw	word (phys @1+2182)
	.dw	phys @1+2182 >> 16
	.dw	word _setF32
	.db	0x4
	.db	0x2
	.db	0x1
	.db	0x2
	.db	0
	.db	0
	.dw	word _maxAmplSet_ChgEventVal
	.dw	word _maxAmplSet_MinChg
	.debug	G _maxAmplSet_RegInfo 33 0
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\appl\LLRF2\LLRF2_Appl.c' 23 30 16 8 1 33
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I
_pvStrTest::
	.db	"....Initial Value....",0
	.debug	G _pvStrTest 34 0
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\appl\LLRF2\LLRF2_Appl.c' 23 30 16 8 1 33
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\appl\LLRF2\LLRF2_Appl.c' 23 30 16 8 1 33
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\appl\LLRF2\LLRF2_Appl.c' 23 30 16 8 1 33
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I
_c0Coils::
	.dw	word _DoNothing_CoilInfo
	.debug	I
	.dw	0
	.debug	G _c0Coils 35 0
	.debug	I 'C:\work\dev\Rabbit_ST\appl\LLRF2\LLRF2_Appl.c' 23 30 16 8 1 33
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I
_r3Regs::
	.dw	word _dlmVersTime_RegInfo
	.dw	word _dlmVersTime_RegInfo
	.dw	word _rtosVersTime_RegInfo
	.dw	word _rtosVersTime_RegInfo
	.dw	word _applVersTime_RegInfo
	.dw	word _applVersTime_RegInfo
	.dw	word _devType__RegInfo
	.dw	word _deviceID__RegInfo
	.dw	word _deviceID__RegInfo
	.dw	word _upSecs_RegInfo
	.dw	word _upSecs_RegInfo
	.dw	word _percentIdle__RegInfo
	.dw	word _numChans_RegInfo
	.dw	word _coilChanMult_RegInfo
	.dw	word _reg3ChanMult_RegInfo
	.dw	word _reg4ChanMult_RegInfo
	.dw	word _digInBits_RegInfo
	.dw	word _digInBits_RegInfo
	.dw	word _statusBitsStdA_RegInfo
	.dw	word _statusBitsStdB_RegInfo
	.dw	word _statusBitsAppl_RegInfo
	.dw	word _statusBitsAppl_RegInfo
	.dw	word _latency_RegInfo
	.dw	word _tcpTickTime_RegInfo
	.dw	word _applLoopTime_RegInfo
	.dw	word _kernelLoopTime_RegInfo
	.dw	word _statusBitsFPGA_RegInfo
	.dw	word _statusBitsTuner_RegInfo
	.dw	word _statusBitsRF_RegInfo
	.dw	word _statusBitsILK_RegInfo
	.dw	word _spark0_ctr_RegInfo
	.dw	word _spark1_ctr_RegInfo
	.dw	word _cavlow_ctr_RegInfo
	.dw	word _rev_high_ctr_RegInfo
	.dw	word _dvdt_ctr_RegInfo
	.dw	word _fwd_low_ctr_RegInfo
	.dw	word _fwd_high_ctr_RegInfo
	.dw	word _PLC_ctr_RegInfo
	.dw	word _safety_ctr_RegInfo
	.dw	word _fwdPhase_RegInfo
	.dw	word _fwdPhase_RegInfo
	.dw	word _revPhase_RegInfo
	.dw	word _revPhase_RegInfo
	.dw	word _cavPhase_RegInfo
	.dw	word _cavPhase_RegInfo
	.dw	word _ref0Phase_RegInfo
	.dw	word _ref0Phase_RegInfo
	.dw	word _spare0Phase_RegInfo
	.dw	word _spare0Phase_RegInfo
	.dw	word _spare1Phase_RegInfo
	.dw	word _spare1Phase_RegInfo
	.dw	word _spare2Phase_RegInfo
	.dw	word _spare2Phase_RegInfo
	.dw	word _ref1Phase_RegInfo
	.dw	word _ref1Phase_RegInfo
	.dw	word _fwdAmpl_RegInfo
	.dw	word _fwdAmpl_RegInfo
	.dw	word _revAmpl_RegInfo
	.dw	word _revAmpl_RegInfo
	.dw	word _cavAmpl_RegInfo
	.dw	word _cavAmpl_RegInfo
	.dw	word _ref0Ampl_RegInfo
	.dw	word _ref0Ampl_RegInfo
	.dw	word _spare0Ampl_RegInfo
	.dw	word _spare0Ampl_RegInfo
	.dw	word _spare1Ampl_RegInfo
	.dw	word _spare1Ampl_RegInfo
	.dw	word _spare2Ampl_RegInfo
	.dw	word _spare2Ampl_RegInfo
	.dw	word _ref1Ampl_RegInfo
	.dw	word _ref1Ampl_RegInfo
	.dw	word _maxFwdAmpl_RegInfo
	.dw	word _maxFwdAmpl_RegInfo
	.dw	word _maxRevAmpl_RegInfo
	.dw	word _maxRevAmpl_RegInfo
	.dw	word _maxCavAmpl_RegInfo
	.dw	word _maxCavAmpl_RegInfo
	.dw	word _maxSpare0Ampl_RegInfo
	.dw	word _maxSpare0Ampl_RegInfo
	.dw	word _maxSpare1Ampl_RegInfo
	.dw	word _maxSpare1Ampl_RegInfo
	.dw	word _maxSpare2Ampl_RegInfo
	.dw	word _maxSpare2Ampl_RegInfo
	.dw	word _maxOutAmpl_RegInfo
	.dw	word _maxOutAmpl_RegInfo
	.dw	word _statusBitsDSP_RegInfo
	.dw	word _statusBitsDSP_RegInfo
	.dw	word _dspLoad_RegInfo
	.dw	word _dspLoad_RegInfo
	.dw	word _pcbTemp_RegInfo
	.dw	word _pcbTemp_RegInfo
	.dw	word _fan0Spd_RegInfo
	.dw	word _fan1Spd_RegInfo
	.dw	word _rfq_t1_output_RegInfo
	.dw	word _rfq_t1_output_RegInfo
	.dw	word _rfq_t2_output_RegInfo
	.dw	word _rfq_t2_output_RegInfo
	.dw	word _piezoReadPos_RegInfo
	.dw	word _stepperReadPos_RegInfo
	.dw	word _percentFFT_RegInfo
	.dw	word _sizeFFT_RegInfo
	.dw	word _romProgErrs_RegInfo
	.debug	I
	.dw	0
	.debug	G _r3Regs 37 0
	.debug	I 'C:\work\dev\Rabbit_ST\appl\LLRF2\LLRF2_Appl.c' 23 30 16 8 1 33
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I
_r4Regs::
	.dw	word _timeZone__RegInfo
	.dw	word _curTime_RegInfo
	.dw	word _curTime_RegInfo
	.dw	word _ipAddr_RegInfo
	.dw	word _ipAddr_RegInfo
	.dw	word _netmask_RegInfo
	.dw	word _netmask_RegInfo
	.dw	word _gateway_RegInfo
	.dw	word _gateway_RegInfo
	.dw	word _setACoil_RegInfo
	.dw	word _resetACoil_RegInfo
	.dw	word _digOutBits_RegInfo
	.dw	word _digOutBits_RegInfo
	.dw	word _cmdBitsStd_RegInfo
	.dw	word _cmdBitsStd_RegInfo
	.dw	word _cmdBitsAppl_RegInfo
	.dw	word _cmdBitsAppl_RegInfo
	.dw	word _ADC_err_rst_RegInfo
	.dw	word _RF_soft_on_RegInfo
	.dw	word _RF_ilk_rst_RegInfo
	.dw	word _ilk_ctr_rst1_RegInfo
	.dw	word _ilk_ctr_rst2_RegInfo
	.dw	word _trigPos_RegInfo
	.dw	word _trigPos_RegInfo
	.dw	word _warmLevel_RegInfo
	.dw	word _warmLevel_RegInfo
	.dw	word _warmTime_RegInfo
	.dw	word _warmTime_RegInfo
	.dw	word _rampTime_RegInfo
	.dw	word _rampTime_RegInfo
	.dw	word _fwdAtten_RegInfo
	.dw	word _revAtten_RegInfo
	.dw	word _cavAtten_RegInfo
	.dw	word _spare0Atten_RegInfo
	.dw	word _spare1Atten_RegInfo
	.dw	word _spare2Atten_RegInfo
	.dw	word _outAtten_RegInfo
	.dw	word _phaseSet_RegInfo
	.dw	word _phaseSet_RegInfo
	.dw	word _amplSet_RegInfo
	.dw	word _amplSet_RegInfo
	.dw	word _pulseFreq_RegInfo
	.dw	word _pulseFreq_RegInfo
	.dw	word _pulseDuty_RegInfo
	.dw	word _pulseDuty_RegInfo
	.dw	word _rf_P_RegInfo
	.dw	word _rf_P_RegInfo
	.dw	word _rf_I_RegInfo
	.dw	word _rf_I_RegInfo
	.dw	word _rf_D_RegInfo
	.dw	word _rf_D_RegInfo
	.dw	word _tuner_P_RegInfo
	.dw	word _tuner_P_RegInfo
	.dw	word _tuner_I_RegInfo
	.dw	word _tuner_I_RegInfo
	.dw	word _tuner_D_RegInfo
	.dw	word _tuner_D_RegInfo
	.dw	word _rf_B0_RegInfo
	.dw	word _rf_B0_RegInfo
	.dw	word _rf_WCtrl_RegInfo
	.dw	word _rf_WCtrl_RegInfo
	.dw	word _rf_WObs_RegInfo
	.dw	word _rf_WObs_RegInfo
	.dw	word _tuner_B0_RegInfo
	.dw	word _tuner_B0_RegInfo
	.dw	word _tuner_WCtrl_RegInfo
	.dw	word _tuner_WCtrl_RegInfo
	.dw	word _tuner_WObs_RegInfo
	.dw	word _tuner_WObs_RegInfo
	.dw	word _pulseAmplSet_RegInfo
	.dw	word _pulseAmplSet_RegInfo
	.dw	word _pulsePhaseSet_RegInfo
	.dw	word _pulsePhaseSet_RegInfo
	.dw	word _cavLowIlkLevel_RegInfo
	.dw	word _cavLowIlkLevel_RegInfo
	.dw	word _cavLowIlkTime_RegInfo
	.dw	word _cavLowIlkTime_RegInfo
	.dw	word _dvdtIlkRate_RegInfo
	.dw	word _dvdtIlkRate_RegInfo
	.dw	word _revHighIlkRatio_RegInfo
	.dw	word _revHighIlkRatio_RegInfo
	.dw	word _revHighIlkTime_RegInfo
	.dw	word _revHighIlkTime_RegInfo
	.dw	word _fwdLowIlkRatio_RegInfo
	.dw	word _fwdLowIlkRatio_RegInfo
	.dw	word _fwdLowIlkTime_RegInfo
	.dw	word _fwdLowIlkTime_RegInfo
	.dw	word _fwdHighIlkLevel_RegInfo
	.dw	word _fwdHighIlkLevel_RegInfo
	.dw	word _fwdHighIlkTime_RegInfo
	.dw	word _fwdHighIlkTime_RegInfo
	.dw	word _fwdCavPhase_RegInfo
	.dw	word _fwdCavPhase_RegInfo
	.dw	word _ffPhase_RegInfo
	.dw	word _ffPhase_RegInfo
	.dw	word _ffGain_RegInfo
	.dw	word _ffGain_RegInfo
	.dw	word _freqOffset_RegInfo
	.dw	word _freqOffset_RegInfo
	.dw	word _rfq_manTuneSet_RegInfo
	.dw	word _rfq_manTuneSet_RegInfo
	.dw	word _rfq_t1_offset_RegInfo
	.dw	word _rfq_t1_offset_RegInfo
	.dw	word _rfq_t2_offset_RegInfo
	.dw	word _rfq_t2_offset_RegInfo
	.dw	word _rfq_t1_feedback_RegInfo
	.dw	word _rfq_t1_feedback_RegInfo
	.dw	word _rfq_t2_feedback_RegInfo
	.dw	word _rfq_t2_feedback_RegInfo
	.dw	word _setDSPState_RegInfo
	.dw	word _setDSPState_RegInfo
	.dw	word _piezoSetPos_RegInfo
	.dw	word _stepperSetPos_RegInfo
	.dw	word _fwdMagCalib_RegInfo
	.dw	word _fwdMagCalib_RegInfo
	.dw	word _fwdPhaseCalib_RegInfo
	.dw	word _fwdPhaseCalib_RegInfo
	.dw	word _revMagCalib_RegInfo
	.dw	word _revMagCalib_RegInfo
	.dw	word _revPhaseCalib_RegInfo
	.dw	word _revPhaseCalib_RegInfo
	.dw	word _cavMagCalib_RegInfo
	.dw	word _cavMagCalib_RegInfo
	.dw	word _cavPhaseCalib_RegInfo
	.dw	word _cavPhaseCalib_RegInfo
	.dw	word _spare0MagCalib_RegInfo
	.dw	word _spare0MagCalib_RegInfo
	.dw	word _spare0PhaseCalib_RegInfo
	.dw	word _spare0PhaseCalib_RegInfo
	.dw	word _spare1MagCalib_RegInfo
	.dw	word _spare1MagCalib_RegInfo
	.dw	word _spare1PhaseCalib_RegInfo
	.dw	word _spare1PhaseCalib_RegInfo
	.dw	word _spare2MagCalib_RegInfo
	.dw	word _spare2MagCalib_RegInfo
	.dw	word _spare2PhaseCalib_RegInfo
	.dw	word _spare2PhaseCalib_RegInfo
	.dw	word _outMagCalib0_RegInfo
	.dw	word _outMagCalib0_RegInfo
	.dw	word _outPhaseCalib0_RegInfo
	.dw	word _outPhaseCalib0_RegInfo
	.dw	word _outMagCalib1_RegInfo
	.dw	word _outMagCalib1_RegInfo
	.dw	word _outPhaseCalib1_RegInfo
	.dw	word _outPhaseCalib1_RegInfo
	.dw	word _fwdPhaseGain_RegInfo
	.dw	word _fwdPhaseGain_RegInfo
	.dw	word _fwdPhaseOffset_RegInfo
	.dw	word _fwdPhaseOffset_RegInfo
	.dw	word _fwdAmplGain_RegInfo
	.dw	word _fwdAmplGain_RegInfo
	.dw	word _fwdAmplOffset_RegInfo
	.dw	word _fwdAmplOffset_RegInfo
	.dw	word _revPhaseGain_RegInfo
	.dw	word _revPhaseGain_RegInfo
	.dw	word _revPhaseOffset_RegInfo
	.dw	word _revPhaseOffset_RegInfo
	.dw	word _revAmplGain_RegInfo
	.dw	word _revAmplGain_RegInfo
	.dw	word _revAmplOffset_RegInfo
	.dw	word _revAmplOffset_RegInfo
	.dw	word _cavPhaseGain_RegInfo
	.dw	word _cavPhaseGain_RegInfo
	.dw	word _cavPhaseOffset_RegInfo
	.dw	word _cavPhaseOffset_RegInfo
	.dw	word _cavAmplGain_RegInfo
	.dw	word _cavAmplGain_RegInfo
	.dw	word _cavAmplOffset_RegInfo
	.dw	word _cavAmplOffset_RegInfo
	.dw	word _ref0PhaseGain_RegInfo
	.dw	word _ref0PhaseGain_RegInfo
	.dw	word _ref0PhaseOffset_RegInfo
	.dw	word _ref0PhaseOffset_RegInfo
	.dw	word _ref0AmplGain_RegInfo
	.dw	word _ref0AmplGain_RegInfo
	.dw	word _ref0AmplOffset_RegInfo
	.dw	word _ref0AmplOffset_RegInfo
	.dw	word _spare0PhaseGain_RegInfo
	.dw	word _spare0PhaseGain_RegInfo
	.dw	word _spare0PhaseOffset_RegInfo
	.dw	word _spare0PhaseOffset_RegInfo
	.dw	word _spare0AmplGain_RegInfo
	.dw	word _spare0AmplGain_RegInfo
	.dw	word _spare0AmplOffset_RegInfo
	.dw	word _spare0AmplOffset_RegInfo
	.dw	word _spare1PhaseGain_RegInfo
	.dw	word _spare1PhaseGain_RegInfo
	.dw	word _spare1PhaseOffset_RegInfo
	.dw	word _spare1PhaseOffset_RegInfo
	.dw	word _spare1AmplGain_RegInfo
	.dw	word _spare1AmplGain_RegInfo
	.dw	word _spare1AmplOffset_RegInfo
	.dw	word _spare1AmplOffset_RegInfo
	.dw	word _spare2PhaseGain_RegInfo
	.dw	word _spare2PhaseGain_RegInfo
	.dw	word _spare2PhaseOffset_RegInfo
	.dw	word _spare2PhaseOffset_RegInfo
	.dw	word _spare2AmplGain_RegInfo
	.dw	word _spare2AmplGain_RegInfo
	.dw	word _spare2AmplOffset_RegInfo
	.dw	word _spare2AmplOffset_RegInfo
	.dw	word _ref1PhaseGain_RegInfo
	.dw	word _ref1PhaseGain_RegInfo
	.dw	word _ref1PhaseOffset_RegInfo
	.dw	word _ref1PhaseOffset_RegInfo
	.dw	word _ref1AmplGain_RegInfo
	.dw	word _ref1AmplGain_RegInfo
	.dw	word _ref1AmplOffset_RegInfo
	.dw	word _ref1AmplOffset_RegInfo
	.dw	word _maxAmplSet_RegInfo
	.dw	word _maxAmplSet_RegInfo
	.debug	I
	.dw	0
	.debug	G _r4Regs 39 0
	.debug	I 'C:\work\dev\Rabbit_ST\appl\LLRF2\LLRF2_Appl.c' 23 30 16 8 1 33
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I
_nonRegPVs::
	.dw	word (phys @1+2193)
	.dw	phys @1+2193 >> 16
	.db	0x5
	.dw	0x16
	.dw	0x16
	.dw	0x1
	.dw	0x5
	.dl	phys _pvStrTest
	.dw	0x16
	.db	0
	.db	0
	.dw	word _PV_UpdateVal
	.dw	word _PV_LockVal
	.dw	word _PV_SendVal
	.dw	word _PV_RcvVal
	.dw	word (phys @1+2203)
	.dw	phys @1+2203 >> 16
	.db	0x3
	.dw	0x1000
	.dw	0x4
	.dw	0x400
	.dw	0x5
	.dl	phys _waveformData+4096
	.dw	0x1000
	.db	0
	.db	0
	.dw	word _PV_UpdateValWF
	.dw	word _PV_LockWave
	.dw	word _PV_SendVal
	.dw	word _PV_RcvVal
	.dw	word (phys @1+2217)
	.dw	phys @1+2217 >> 16
	.db	0x3
	.dw	0x1000
	.dw	0x4
	.dw	0x400
	.dw	0x5
	.dl	phys _waveformData+12288
	.dw	0x1000
	.db	0
	.db	0
	.dw	word _PV_UpdateValWF
	.dw	word _PV_LockWave
	.dw	word _PV_SendVal
	.dw	word _PV_RcvVal
	.dw	word (phys @1+748)
	.dw	phys @1+748 >> 16
	.db	0x4
	.dw	0x1000
	.dw	0x4
	.dw	0x400
	.dw	0x5
	.dl	phys _waveformData+20480
	.dw	0x1000
	.db	0
	.db	0
	.dw	word _PV_UpdateValWF
	.dw	word _PV_LockWave
	.dw	word _PV_SendVal
	.dw	word _PV_RcvVal
	.debug	I
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
	.dw	0
	.dw	0
	.dw	0, 0
	.dw	0
	.db	0
	.db	0
	.dw	0
	.dw	0
	.dw	0
	.dw	0
	.debug	G _nonRegPVs 40 0
	.debug	I 'C:\work\dev\Rabbit_ST\appl\LLRF2\LLRF2_Appl.c' 23 30 16 8 1 33
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I
	.defseg	CONST
	.seg	CONST
_PV_WF_Idx::
	.dw	0x1
	.dw	0x2
	.dw	0x3
	.debug	G _PV_WF_Idx 42 0
	.debug	I
	.debug	I
;799 #endif
;800 #ifndef RegDefs
;801 
;802 
;803 #include <LLRF2_Appl.h>
;804 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\LLRF2\LLRF2_Appl.h' 26 24 14 7 12 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\semaphore.h' 23 34 9 26 4 32
	.debug	I
	.debug	I
;805 
;806 
;807 //APPLCODE - change to the application name
;808 const char far applName[] = { "Low Level RF Controller" };
	.defseg	FARCONST
	.seg	FARCONST
_applName::
	.db	"Low Level RF Controller",0
	.debug	G _applName 43 8
;809 const char ApplAbbrev[] = { "LLRF2" };
	.seg	CONST
_ApplAbbrev::
	.db	"LLRF2",0
	.debug	G _ApplAbbrev 44 0
;810 
;811 //APPLCODE - add ID to unistd.h and place it here
;812 #define DeviceType  DT_LLRF2
;813 
;814 //APPLCODE - set the # of input and outputs used
;815 #define  NUM_APPL_AIN     1   // analog inputs
;816 #define  NUM_APPL_AOUT    1   // analog outputs
;817 #define  NUM_APPL_DIN     1   // digital inputs
;818 #define  NUM_APPL_DOUT    2   // digital outputs
;819 
;820 #define  ADC_MASK_APPL  0x00000000L
;821 #define  DAC_MASK_APPL  0x00000000L
;822 
;823 //APPLCODE - set to state we want outputs in during initialization
;824 #define InitialDigOutState  ( 0 )
;825 
;826 
;827 #define SharedINC
;828 #include <sharedAppl.c>
;829 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\socket.h' 2 54 9 7 6 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\netinet\in.h' 15 56 20 12 3 27
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip.h' 10 19 7 30 6 29
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\dcdefs.h' 18 42 23 19 7 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\dcrtcp.h' 1 9 9 21 2 32
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\tcp_config.h' 9 28 14 7 1 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\neterrno.h' 26 24 16 30 7 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\errors.h' 25 8 15 18 11 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\servlist.h' 7 29 16 30 7 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\net.h' 21 21 9 7 3 32
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ppp.h' 4 42 22 17 6 26
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ppplink.h' 18 1 2 19 9 26
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\chat.h' 12 7 15 6 10 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Rabbit.h' 24 40 11 19 5 30
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\net.h' 21 21 9 7 3 32
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\pktdrv.h' 16 19 17 14 3 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\arp.h' 21 56 14 30 7 24
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ip.h' 16 35 12 25 4 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ip.h' 16 35 12 25 4 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\udp.h' 23 23 20 4 3 30
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\icmp.h' 2 44 0 17 5 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\tcp.h' 14 5 18 19 4 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\bootp.h' 2 16 3 7 2 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\bsdname.h' 9 41 20 10 2 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\icmp.h' 2 44 0 17 5 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\dns.h' 18 15 17 7 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\servlist.h' 7 29 16 30 7 24
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\igmp.h' 10 41 20 10 2 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\smtp.h' 0 54 12 2 8 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\http.h' 28 52 16 7 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\zserver.h' 26 57 12 10 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\limits.h' 1 13 9 26 5 19
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\pool.h' 10 58 16 7 1 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\errno.h' 16 4 11 6 2 29
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\mib.h' 28 3 18 15 4 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\snmp.h' 22 42 15 4 11 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ftp_client.h' 9 41 20 10 2 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ftp_server.h' 5 50 18 7 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\zserver.h' 26 57 12 10 1 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ethernet.h' 21 56 23 21 3 26
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\fcntl.h' 29 48 12 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\stat.h' 0 48 10 31 3 29
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\time.h' 15 48 18 10 12 22
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\signal.h' 6 21 8 17 3 29
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\errno.h' 22 0 10 11 8 30
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\string.h' 26 27 14 2 6 25
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\backup.h' 9 9 8 10 9 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\hdwDefs.h' 20 2 12 27 10 31
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\srfs.h' 6 17 10 7 4 28
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Rabbit.h' 24 40 11 19 5 30
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\spi.h' 12 54 10 3 8 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\serial.h' 29 2 10 30 4 30
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\fcntl.h' 29 48 12 5 12 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\serial.h' 29 2 10 30 4 30
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\socket.h' 2 54 9 7 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\epics_cad.h' 8 56 9 31 7 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\fs\DLMiii.h' 3 21 14 4 6 30
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\ZWIDblock.h' 29 54 20 12 3 27
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\SoftoolsIDBlock.h' 18 52 15 4 6 26
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\fs\SHD_DLM.h' 7 53 11 8 2 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\socket.h' 2 54 9 7 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\fs\DLMIII.h' 3 21 14 4 6 30
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip.h' 10 19 7 30 6 29
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\inetd.h' 23 39 13 12 5 30
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\farprintf.h' 5 43 7 29 11 27
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\busybox.h' 23 57 14 6 3 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\backup.h' 9 9 8 10 9 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\applic.h' 28 10 10 5 12 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\busybox.h' 23 57 14 6 3 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\tsh.h' 3 45 16 21 3 28
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\backup.h' 9 9 8 10 9 32
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\ctype.h' 22 48 13 9 7 24
	.debug	I
	.dseg
_consCfg::
	.dw	0xffffc200, 0x1
	.dw	0x1
	.debug	G _consCfg 45 0
	.seg	CONST
_NumApplAIN::
	.dw	0x1
	.debug	G _NumApplAIN 7 0
_NumApplAOUT::
	.dw	0x1
	.debug	G _NumApplAOUT 7 0
_NumApplDIN::
	.dw	0x1
	.debug	G _NumApplDIN 7 0
_NumApplDOUT::
	.dw	0x2
	.debug	G _NumApplDOUT 7 0
_NumExtraIn::
	.dw	0x20
	.debug	G _NumExtraIn 7 0
	.debug	I
;830 #include <math.h>
;831 
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\math.h' 19 23 18 21 9 21
	.debug	I
;832 
;833 #include <LLRF2.h>
;834 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\arch\LLRF2\LLRF2.h' 12 58 9 16 11 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I
;835 
;836 
;837 //--- definitions for enabling diagnostic messages ---
;838 #define  DSPErrorBits      0x80
;839 
;840 #define  ShowDSPErrors     (DiagFlags & DSPErrorBits)
;841 
;842 
;843 /*-------------------------------------------------
;844  NOTES:
;845    port B ("spi1")  connected to DSP
;846    port C ("spi2")  connected to the EEPROMs
;847 ---------------------------------------------------*/
;848 
;849 //--- multipliers/divisors for converting between unsigned 16-bit int value
;850 //--- used by the DSP and engineering units (volts and degress in this case
;851 #define  AmplConvFactor   ( 0.0000426233445431 )
;852 //#define  PhaseConvFactor  ( 0.0054931640625 )  // 360 / 2^16
;853 #define  PhaseConvFactor  ( 0.0000000838190317 )  // ~ 360 / 2^32
;854 #define  FRACU16Divisor   ( 0.0000152590219 )  // div by this = mult by 655
;855 //#define  FreqConvFactor   (    0.0152590219 )  // scales 0-1000 -> 0-6553
;856 
;857 
;858 #define  DSP_Prog_File   "/srfs/dspprog.bin"
;859 #define  FPGA_Prog_File  "/srfs/llrf.bin"
;860 
;861 
;862 
;863 
;864 #define  ReadDig()  { digitalIn = ReadDigInputs(); }
;865 #define  WriteDig()  {  WriteDigOutputs(digitalOut); }
;866 
;867 
;868 
;869 #define  PI_  3.1415926
;870 
;871 //-------------------------------------------------------------------------
;872 // data types and function prototypes:
;873 //-------------------------------------------------------------------------
;874 
;875 //long fltofp(float value);
;876 //float fptofl(long value);
;877 
;878 
;879 
;880 bool MB_DSP_REG(SetRegFuncParams);
;881 
;882 
;883 
;884 
;885 //=========================================================================
;886 //  definitions related to use of the Digital and Analog I/O signals
;887 //=========================================================================
;888 
;889 //======================= digitalIn usage ===========================
;890 #define  FPGA_DONE    0x000001  // DI-00: FPGA loaded and running
;891 
;892 const uchar  LogicOrderDIn[NUM_APPL_DIN] = { 0 };
_LogicOrderDIn::
	.db	0
	.debug	G _LogicOrderDIn 46 0
;893 
;894 //--- msgs to log automatically when a digital input changes state ---
;895 //--- use NULL for bits that we don't want to log changes for ---
;896 const char far *digInDescr[NUM_APPL_DIN*2] = {
;897 //--0 state descr--    --1 state descr--
;898   NULL,                NULL
	.dseg
_digInDescr::
	.dw	0, 0
;899 
	.dw	0, 0
	.debug	G _digInDescr 47 0
;900 
;901 
;902 
;903 //====================== digitalOut usage ===========================
;904 
;905 //------------------------------
;906 #define  READY_LED    0x000001   // DO-00: CPU is running application
;907 #define  ActivityLED  0x000002   // DO-01: Activity LED
;908 
;909 const uchar  LogicOrderDOut[NUM_APPL_DOUT] = { 0, 1 };
	.seg	CONST
_LogicOrderDOut::
	.db	0
	.db	0x1
	.debug	G _LogicOrderDOut 49 0
;910 
;911 
;912 #define  TurnOnActivityLED()    ClearBits(Next.digitalOut, ActivityLED)
;913 #define  TurnOffActivityLED()   SetBits(Next.digitalOut, ActivityLED)
;914 
;915 //--- msgs to log automatically when a digital output changes state ---
;916 //--- use NULL for bits that we don't want to log changes for ---
;917 const char far *digOutDescr[NUM_APPL_DOUT*2] = {
;918 //--0 state descr--    --1 state descr--
;919   NULL,                NULL,
	.dseg
_digOutDescr::
	.dw	0, 0
	.dw	0, 0
;920   NULL,                NULL
	.dw	0, 0
;921 
	.dw	0, 0
	.debug	G _digOutDescr 50 0
;922 
;923 
;924 //=============== Analog input usage ===============
;925 const uchar  LogicOrderAIn[NUM_APPL_AIN] = { 0 };
	.seg	CONST
_LogicOrderAIn::
	.db	0
	.debug	G _LogicOrderAIn 46 0
;926 
;927 
;928 //=============== Analog output usage ===============
;929 const uchar  LogicOrderAOut[NUM_APPL_AOUT] = { 0 };
_LogicOrderAOut::
	.db	0
	.debug	G _LogicOrderAOut 46 0
;930 
;931 
;932 
;933 //=========================================================================
;934 //  ID's for values sent to or read from the DSP
;935 //  OR'ing an ID with 0x8000 allows reading without writing
;936 //=========================================================================
;937 
;938 //--- DSP register numbers (without the 0x01000000 mask shared by all regs)
;939 
;940 
;941 // Even # values used to read a register, Odd # values used to write to a r
;942 
;943 // P32 means the "PHASE" format, a 32-bit integer scaled to/from 0 to 360
;944 // B32 means 32-bit uint bit-mapped value
;945 
;946 //===== Read-Only registers =====
;947 #define  FwdPhase           0x00100000L  // P32: Forward Phase
;948 #define  RevPhase           0x00100002L  // P32: Reverse Phase
;949 #define  CavPhase           0x00100004L  // P32: Cavity Phase
;950 #define  Ref0Phase          0x00100006L  // P32: Reference 0 Phase
;951 
;952 #define  Spare0Phase        0x00100008L  // P32: Spare RF 0 Phase
;953 #define  Spare1Phase        0x0010000AL  // P32: Spare RF 1 Phase
;954 #define  Spare2Phase        0x0010000CL  // P32: Spare RF 2 Phase
;955 #define  Ref1Phase          0x0010000EL  // P32: Reference 1 Phase
;956 
;957 #define  FwdAmpl            0x00100010L  // F32: Forward Amplitude
;958 #define  RevAmpl            0x00100012L  // F32: Reverse Amplitude
;959 #define  CavAmpl            0x00100014L  // F32: Cavity Amplitude
;960 #define  Ref0Ampl           0x00100016L  // F32: Reference Amplitude
;961 
;962 #define  Spare0Ampl         0x00100018L  // F32: Spare RF 0 Amplitude
;963 #define  Spare1Ampl         0x0010001AL  // F32: Spare RF 1 Amplitude
;964 #define  Spare2Ampl         0x0010001CL  // F32: Spare RF 2 Amplitude
;965 #define  Ref1Ampl           0x0010001EL  // F32: Reference 1 Amplitude
;966 
;967 #define  MaxFwdAmpl         0x00100020L  // F32: Max Forward Amplitude
;968 #define  MaxRevAmpl         0x00100022L  // F32: Max Reverse Amplitude
;969 #define  MaxCavAmpl         0x00100024L  // F32: Max Cavity Amplitude
;970 
;971 #define  MaxSpare0Ampl      0x00100026L  // F32: Max Spare RF 0 Amplitude
;972 #define  MaxSpare1Ampl      0x00100028L  // F32: Max Spare RF 1 Amplitude
;973 #define  MaxSpare2Ampl      0x0010002AL  // F32: Max Spare RF 2 Amplitude
;974 #define  MaxOutAmpl         0x0010002CL  // F32: Max Output Amplitude
;975 
;976 #define  StatusBitsDSP      0x0010002EL  // B32: misc DSP status
;977 
;978 #define  DSPLoad            0x00100030L  // F32: DSP load percentage
;979 #define  PCBTemp            0x00100032L  // F32: PCB temperature
;980 #define  Fan0Spd            0x00100034L  // U32: Fan0 Speed (in rpm)
;981 #define  Fan1Spd            0x00100036L  // U32: Fan1 Speed (in rpm)
;982 
;983 
;984 //===== R/W registers (the addresses shown are the read addresses) =====
;985 #define  TrigPos            0x00100100L  // U32: trigger position (tuning p
;986 
;987 #define  WarmLevel          0x00100102L  // F32: Warm-up Amplitude
;988 #define  WarmTime           0x00100104L  // F32: time at warm-up Amplitude
;989 #define  RampTime           0x00100106L  // F32: time to ramp to set-point
;990 
;991 #define  FwdAtten           0x00100108L  // U32: Forward Attenuator setting
;992 #define  RevAtten           0x0010010AL  // U32: Reverse Attenuator setting
;993 #define  CavAtten           0x0010010CL  // U32: Cavity Attenuator setting
;994 
;995 #define  Spare0Atten        0x0010010EL  // U32: Spare RF 0 Attenuator sett
;996 #define  Spare1Atten        0x00100110L  // U32: Spare RF 1 Attenuator sett
;997 #define  Spare2Atten        0x00100112L  // U32: Spare RF 2 Attenuator sett
;998 #define  OutAtten           0x00100114L  // U32: Output Attenuator setting
;999 
;1000 #define  PhaseSet           0x00100116L  // P32: Phase setpoint
;1001 #define  AmplSet            0x00100118L  // F32: Ammplitude setpoint
;1002 
;1003 #define  PulseFreq          0x0010011AL  // F32: Tuning Pulse Freq
;1004 #define  PulseDuty          0x0010011CL  // F32: Freq Search Amplitude setp
;1005 
;1006 #define  RF_P               0x0010011EL  // F32: RF P value
;1007 #define  RF_I               0x00100120L  // F32: RF I value
;1008 #define  RF_D               0x00100122L  // F32: RF D value
;1009 
;1010 #define  Tuner_P            0x00100124L  // F32: Tuner P value
;1011 #define  Tuner_I            0x00100126L  // F32: Tuner I value
;1012 #define  Tuner_D            0x00100128L  // F32: Tuner D value
;1013 
;1014 #define  RF_B0              0x0010012AL  // F32: RF ADRC b0 value
;1015 #define  RF_WCtrl           0x0010012CL  // F32: RF ADCR control bandwidth
;1016 #define  RF_WObs            0x0010012EL  // F32: RF ADRC observer bandwidth
;1017 
;1018 #define  Tuner_B0           0x00100130L  // F32: Tuner ADRC b0 value
;1019 #define  Tuner_WCtrl        0x00100132L  // F32: Tuner ADCR control bandwid
;1020 #define  Tuner_WObs         0x00100134L  // F32: Tuner ADRC observer bandwi
;1021 
;1022 #define  PulseAmplSet       0x00100136L  // F32: Pulsed Amplitude setpoint
;1023 #define  PulsePhaseSet      0x00100138L  // P32: Pulsed Phase setpoint
;1024 
;1025 #define  CavLowIlkLevel     0x0010013AL  // F32: Cavity Low Interlock Level
;1026 #define  CavLowIlkTime      0x0010013CL  // F32: Cavity Log Interlock Time
;1027 
;1028 #define  DvDtIlkRate        0x0010013EL  // F32: -dV/dT Interlock Rate
;1029 
;1030 #define  RevHighIlkRatio    0x00100140L  // F32: V+/V- Interlock Ratio
;1031 #define  RevHighIlkTime     0x00100142L  // F32: V+/V- Interlock Time
;1032 
;1033 #define  FwdLowIlkRatio     0x00100144L  // F32: V+/V- Interlock Ratio
;1034 #define  FwdLowIlkTime      0x00100146L  // F32: V+/V- Interlock Time
;1035 
;1036 #define  FwdHighIlkLevel    0x00100148L  // F32: V+/V- Interlock Level
;1037 #define  FwdHighIlkTime     0x0010014AL  // F32: V+/V- Interlock Time
;1038 
;1039 #define  FwdCavPhase        0x0010014CL  // P32: Fwd-Cav Phase Calibration
;1040 
;1041 #define  FFPhase            0x0010014EL  // P32: Feed Forward Phase
;1042 #define  FFGain             0x00100150L  // F32: Feed Fwd Gain
;1043 
;1044 #define  FreqOffset         0x00100152L  // F32:  RF Frequency Offset
;1045 
;1046 
;1047 //===== RFQ Tuner values, some of which are forwarded by the control =====
;1048 //===== system to other devices/records =====
;1049 #define  RFQ_ManTuneSet     0x00100198L  // (RW) F32: manual tuner setpoint
;1050 
;1051 #define  RFQ_T1_Offset      0x0010019AL  // (RW) F32: tuner 1 calib offset
;1052 #define  RFQ_T2_Offset      0x0010019CL  // (RW) F32: tuner 2 calib offset
;1053 
;1054 #define  RFQ_T1_Output      0x0010019EL  // (RO) F32: tuner 1 output
;1055 #define  RFQ_T2_Output      0x001001A0L  // (RO) F32: tuner 2 output
;1056 
;1057 #define  RFQ_T1_Feedback    0x001001A2L  // (RW) F32: tuner 1 feedback
;1058 #define  RFQ_T2_Feedback    0x001001A4L  // (RW) F32: tuner 2 feedback
;1059 
;1060 
;1061 //===== Special Registers =====
;1062 #define  SetDSPState        0x00100200L  // (WO) Bits: set/reset DSP state 
;1063 
;1064 #define  PiezoTunerPos      0x00100202L  // (RW) U32: piezo tuner position
;1065 #define  StepperPos         0x00100204L  // (RW) U32: stepper position
;1066 
;1067 //#define  SetAFCsOnOff       0x00100206L  // (WO) Bits: turn AFC frequenci
;1068 
;1069 #define  Unused_208         0x00100208L  //
;1070 
;1071 #define  Rd_L_Buf           0x0010020AL  // (RN) S16x2: read next tuning pl
;1072 #define  Rd_T_Buf           0x0010020CL  // (RN) S16x2: read next tuning pl
;1073 
;1074 #define  Reserved_20E       0x0010020EL  //
;1075 #define  Reserved_210       0x00100210L  //
;1076 
;1077 //#define  WrMemBlock         0x00100212L  // (WO) U32: start prog mode for
;1078 //#define  WriteStatus        0x00100214L  // (RO) S32: (% done or -1 ==> b
;1079 
;1080 //#define  CalcCTF            0x00100216L  // (**) N/A: begin calc of Cavit
;1081 //#define  PercentCTF         0x00100218L  // (RO) U32: percent done for CT
;1082 //#define  AbortCTF           0x0010021AL  // (**) N/A: abort CTF calc; == 
;1083 //#define  SizeCTF            0x0010021CL  // (RO) U32: # of 32-bit words i
;1084 //#define  PhaseCTF           0x0010021EL  // (RO) P32: read next Phase val
;1085 //#define  AmplCTF            0x00100220L  // (RO) F32: read next Ampl valu
;1086 
;1087 #define  PercentFFT         0x00100222L  // (RO) U32: percent done for FFT 
;1088 #define  SizeFFT            0x00100224L  // (RO) U32: # of 32-bit words of 
;1089 
;1090 #define  AmplFFT            0x00100226L  // (RN) F32: read next Ampl value 
;1091 
;1092 //#define  CTFCalcRate        0x00100228L  // (RW) F32: CTF calc rate
;1093 
;1094 #define  FwdMagCalib        0x0010022AL  // (RW) F32: Forward Mag calib val
;1095 #define  FwdPhaseCalib      0x0010022CL  // (RW) P32: Forward Phase calib v
;1096 
;1097 #define  RevMagCalib        0x0010022EL  // (RW) F32: Reverse Mag calib val
;1098 #define  RevPhaseCalib      0x00100230L  // (RW) P32: Reverse Phase calib v
;1099 
;1100 #define  CavMagCalib        0x00100232L  // (RW) F32: Cavity Mag calib valu
;1101 #define  CavPhaseCalib      0x00100234L  // (RW) P32: Cavity Phase calib va
;1102 
;1103 #define  Spare0MagCalib     0x00100236L  // (RW) F32: Spare RF 0 Mag calib 
;1104 #define  Spare0PhaseCalib   0x00100238L  // (RW) P32: Spare RF 0 Phase cali
;1105 
;1106 #define  Spare1MagCalib     0x0010023AL  // (RW) F32: Spare RF 1 Mag calib 
;1107 #define  Spare1PhaseCalib   0x0010023CL  // (RW) P32: Spare RF 1 Phase cali
;1108 
;1109 #define  Spare2MagCalib     0x0010023EL  // (RW) F32: Spare RF 2 Mag calib 
;1110 #define  Spare2PhaseCalib   0x00100240L  // (RW) P32: Spare RF 2 Phase cali
;1111 
;1112 #define  OutMagCalib0       0x00100242L  // (RW) F32: Output Mag calib valu
;1113 #define  OutPhaseCalib0     0x00100244L  // (RW) P32: Output Phase calib va
;1114 
;1115 #define  OutMagCalib1       0x00100246L  // (RW) F32: Output Mag calib valu
;1116 #define  OutPhaseCalib1     0x00100248L  // (RW) P32: Output Phase calib va
;1117 
;1118 #define  SaveCalibToDSP     0x0010024AL  // (**) N/A: Save calib to DSP's R
;1119 
;1120 #define  ROMProgErrs        0x0010024CL  // (RO) U32: ROM programming error
;1121 
;1122 
;1123 //  (**) means a READ from the register triggers the stated functionality
;1124 
;1125 
;1126 //=========================================================================
;1127 //  List of daemon processes.  The inetd process listens for connection
;1128 //  requests on each port in the list and creates a new instance of the
;1129 //  specified process to service each new connection.
;1130 //=========================================================================
;1131 #include <services.h>
;1132 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\services.h' 2 22 6 20 2 29
	.debug	I
;1133 //--- service funcs declared the same as standard command funcs:
;1134 //     e.g.:   void _task fastdatad(int argc, char **argv);
;1135 void _task tsh(int argc, char **argv);
;1136 
;1137 //void _task readWave(int argc, char **argv);
;1138 
;1139 
;1140 ServiceInfo inetdConf[] = {
;1141 //--- add any application-specific inetd-type network services here ---
;1142 //--- (add port # names to services.h) ---
;1143 //  NAME          PORT           PROTOCOL  FUNC        PRI -1  Config Param
;1144 //  ------------  -------------  --------  ----------- --- --  ------------
;1145 //  { "readWave",   READ_WAVE,     ProtoTCP, readWave,   12, -1, NULL },
;1146 /*
;1147   NOTE:  The readWave approach is outdated as it is not a practical way
;1148   way to support reading multiple waveforms concurrently (as is now done
;1149   for EPICS, which has one waveform record for each of the waveforms).
;1150 
;1151   The new method uses the DEBROS EPICS CA (Channel Access) service (the
;1152   ca_tcpd task, which handles the TCP virtual circuits, subscriptions, etc)
;1153 
;1154   The UDP portion of the CA spec is supported if needed (handled by the ca_
;1155   task), but because of the limited buffering that is practical on the curr
;1156   hardware (i.e it drops lots of the CA UDP broadcast packets), this servic
;1157   is not normally run.
;1158 
;1159   The EPICS waveform records use the DirectCA device support module which
;1160   already knows the controller's IP address and opens a connection as neede
;1161   to the ca_tcpd task and uses the standard CA protocol messages to create
;1162   subscriptions for the waveforms and related values.
;1163 */
;1164 
;1165 //--- standard ones defined in sharedAppl.c ---
;1166 //{ "console",    TTY0,          ASYNC,    tsh,        10, -1, &consCfg },
;1167 //{ "fastdatad",  FASTDATA_PORT, ProtoTCP, fastdatad,   5, -1, NULL },
;1168 //{ "modbusTCPd", MODBUS_PORT,   ProtoTCP, modbusTCPd,  6, -1, NULL },
;1169 //{ "tsh",        TELNET_PORT,   ProtoTCP, tsh,        10, -1, NULL }
;1170 
;1171   #define SharedServices
;1172   #include <sharedAppl.c>
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
	.dseg
_inetdConf::
	.dw	@2
	.dw	0
	.db	0x3
	.dw	word _tsh
	.db	0xa
	.dw	0xffff
	.dw	word _consCfg
	.ds	6,0
	.dw	@2+8
	.dw	0x7d3
	.db	0x2
	.dw	word _fastdatad
	.db	0x5
	.dw	0xffff
	.dw	0
	.ds	6,0
	.dw	@2+18
	.dw	0x1f6
	.db	0x2
	.dw	word _modbusTCPd
	.db	0x6
	.dw	0xffff
	.dw	0
	.ds	6,0
	.dw	@2+29
	.dw	0x17
	.db	0x2
	.dw	word _tsh
	.db	0xa
	.dw	0xffff
	.dw	0
	.debug	I
;1173 };
	.ds	6,0
	.debug	G _inetdConf 51 0
;1174 
;1175 const int  NumServices = sizeof(inetdConf) / sizeof(ServiceInfo);
	.seg	CONST
_NumServices::
	.dw	0x4
	.debug	G _NumServices 6 0
;1176 
;1177 
;1178 //=========================================================================
;1179 //  Application-specific data structures and definitions
;1180 //=========================================================================
;1181 
;1182 // values that can be different between prev/cur or cur/next during process
;1183 
;1184 typedef struct {
;1185 //--- values updated in "cur" instance at start of each pass ---
;1186   ulong  digitalIn;       // state of digital inputs
;1187 
;1188 //--- values updated in "next" instance at start of each pass ---
;1189   ulong  digitalOut;      // state for digital outputs
;1190 } ApplState;
;1191 
;1192 
;1193 ApplState  prevState, curState, nextState;
;1194 
;1195 #define  Prev  prevState
;1196 #define  Cur   curState
;1197 #define  Next  nextState
;1198 
;1199 const uint ApplStateDataSize =  ( sizeof(ApplState)*3 + sizeof(setVal) );
_ApplStateDataSize::
	.dw	0x1c
	.debug	G _ApplStateDataSize 7 0
;1200 
;1201 
;1202 //--- values and flags set by the 1st stage of the control loop when ---
;1203 //--- interpreting the state of the inputs and processing the next ---
;1204 //--- event msg ---
;1205 
;1206 //ulong  calcStartTimeCTF;
;1207 
;1208 
;1209 
;1210 //=========================================================================
;1211 //  Defintions for variables that are accessable as ModBus registers
;1212 //=========================================================================
;1213 #else  // end of before-RegDefs portion
;1808 
;1809 #ifndef RegDefs // start of after-RegDefs portion
;1810 
;1811 
;1812 //---- bit usage for setDSPState (DSP SetDSPState 0x00100201) ---
;1813 //---------------------------------------------------------------
;1814 #define  CMD_STOP_IPULSING      0x00000001  // stop pulsing using intern sr
;1815 #define  CMD_START_IPULSING     0x00000002  // start pulsing using intern s
;1816 //       ---------------------
;1817 #define  CMD_IPULSING_BITS      0x00000003L
;1818 
;1819 #define  CMD_DISABLE_EPULSING   0x00000004  // disable pulsing via extern t
;1820 #define  CMD_ENABLE_EPULSING    0x00000008  // enable pulsing via extern tr
;1821 //       ---------------------
;1822 #define  CMD_EPULSING_BITS      0x0000000CL
;1823 
;1824 #define  CMD_L_DATA_READ        0x00000010  // Done reading Rising edge plo
;1825 #define  CMD_T_DATA_READ        0x00000020  // Done reading Falling edge pl
;1826 #define  CMD_FFT_DATA_READ      0x00000040  // Done reading FFT plot
;1827 
;1828 #define  CMD_UNUSED_7           0x00000080
;1829 //---------------------------------------------------------------
;1830 #define  CMD_CTL_MODE_AUTO      0x00000100  // set ctl to auto mode
;1831 #define  CMD_CTL_MODE_MANUAL    0x00000020  // set ctl to manual mode
;1832 //       -------------------
;1833 #define  CMD_CTL_MODE_BITS      0x00000300L
;1834 
;1835 // The following are ignored when CMD_CTL_MODE_AUTO is set
;1836 #define  CMD_DISAB_FREQ_OFFSET  0x00000400  // disable RF frequency offset
;1837 #define  CMD_ENAB_FREQ_OFFSET   0x00000800  // enable RF frequency offset
;1838 //       ---------------------
;1839 #define  CMD_FREQ_OFFSET_BITS   0x00000C00L
;1840 
;1841 #define  CMD_USE_CAV_CHAN       0x00001000  // use cav chan for RF ampl fee
;1842 #define  CMD_USE_FWD_CHAN       0x00002000  // use fwd chan for RF ampl fee
;1843 //       -----------------
;1844 #define  CMD_USE_CHAN_BITS      0x00003000L
;1845 
;1846 #define  CMD_AMPL_CTL_OFF       0x00004000  // disable RF ampl control
;1847 #define  CMD_AMPL_CTL_ON        0x00008000  // enable RF ampl control
;1848 //       ------------------
;1849 #define  CMD_AMPL_CTL_BITS      0x0000C000L
;1850 //---------------------------------------------------------------
;1851 #define  CMD_PHCTL_OFF_SEL_OFF  0x00010000L  // disble RF phase ctl and SEL
;1852 #define  CMD_PHCTL_ON_SEL_OFF   0x00020000L  // enable RF phase ctl, disabl
;1853 #define  CMD_PHCTL_OFF_SEL_ON   0x00040000L  // disable RF phase ctl, enabl
;1854 //       --------------------
;1855 #define  CMD_PHCTL_SEL_BITS     0x00070000L
;1856 
;1857 #define  CMD_UNUSED_19          0x00080000L
;1858 
;1859 #define  CMD_TUNER_CTL_OFF      0x00100000L  // disable tuner control
;1860 #define  CMD_TUNER_CTL_ON       0x00200000L  // enable tuner control
;1861 //       --------------------
;1862 #define  CMD_TUNER_CTL_BITS     0x00300000L
;1863 
;1864 #define  CMD_UNUSED_22          0x00400000
;1865 #define  CMD_UNUSED_23          0x00800000
;1866 //---------------------------------------------------------------
;1867 #define  CMD_UNUSED_24          0x01000000L
;1868 #define  CMD_UNUSED_25          0x02000000L
;1869 #define  CMD_UNUSED_26          0x04000000L
;1870 #define  CMD_UNUSED_27          0x08000000L
;1871 
;1872 #define  CMD_UNUSED_28          0x10000000L
;1873 #define  CMD_UNUSED_29          0x20000000L
;1874 #define  CMD_UNUSED_30          0x40000000L
;1875 #define  CMD_UNUSED_31          0x80000000L
;1876 //---------------------------------------------------------------
;1877 
;1878 
;1879 //--- bit-masks to determine which groups have a bit set ---
;1880 const ulong SetDSPStateMasks[] = {
;1881   CMD_IPULSING_BITS,
_SetDSPStateMasks::
	.dw	0x3, 0
;1882   CMD_EPULSING_BITS,
	.dw	0xc, 0
;1883   CMD_CTL_MODE_BITS,
	.dw	0x300, 0
;1884   CMD_FREQ_OFFSET_BITS,
	.dw	0xc00, 0
;1885   CMD_USE_CHAN_BITS,
	.dw	0x3000, 0
;1886   CMD_AMPL_CTL_BITS,
	.dw	0xffffc000, 0
;1887   CMD_PHCTL_SEL_BITS,
	.dw	0, 0x7
;1888   CMD_TUNER_CTL_BITS,
	.dw	0, 0x30
;1889   0  //indicates end of list
;1890 };
	.dw	0, 0
	.debug	G _SetDSPStateMasks 53 0
;1891 
;1892 
;1893 
;1894 const char far *SetDSPStateDesc[32] = {
;1895 //------------------------------------------
;1896   "Stop RF pulsing via intern src",
	.dseg
_SetDSPStateDesc::
	.dw	word (phys @1+2232)
	.dw	phys @1+2232 >> 16
;1897   "Start RF pulsing using intern src",
	.dw	word (phys @1+2263)
	.dw	phys @1+2263 >> 16
;1898 
;1899   "Disable RF pulsing via extern trig",
	.dw	word (phys @1+2297)
	.dw	phys @1+2297 >> 16
;1900   "Enable RF pulsing via extern trig",
	.dw	word (phys @1+2332)
	.dw	phys @1+2332 >> 16
;1901 
;1902   "", "", "", "",
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
;1903 //------------------------------------------
;1904   "Set Ctl to Auto Mode",
	.dw	word (phys @1+2366)
	.dw	phys @1+2366 >> 16
;1905   "Set Ctl to Manual Mode",
	.dw	word (phys @1+2387)
	.dw	phys @1+2387 >> 16
;1906 
;1907   "Disable RF Freq Offset",
	.dw	word (phys @1+2410)
	.dw	phys @1+2410 >> 16
;1908   "Enable RF Freq Offset",
	.dw	word (phys @1+2433)
	.dw	phys @1+2433 >> 16
;1909 
;1910   "Use cav chan for RF ampl feedback",
	.dw	word (phys @1+2455)
	.dw	phys @1+2455 >> 16
;1911   "Use fwd chan for RF ampl feedback",
	.dw	word (phys @1+2489)
	.dw	phys @1+2489 >> 16
;1912 
;1913   "Disable RF Ampl ctl",
	.dw	word (phys @1+2523)
	.dw	phys @1+2523 >> 16
;1914   "Enable RF Ampl ctl",
	.dw	word (phys @1+2543)
	.dw	phys @1+2543 >> 16
;1915 //------------------------------------------
;1916   "Disable RF Phase Ctl and SEL",
	.dw	word (phys @1+2562)
	.dw	phys @1+2562 >> 16
;1917   "Enable RF Phase Ctl, disable SEL",
	.dw	word (phys @1+2591)
	.dw	phys @1+2591 >> 16
;1918   "Disable RF Phase Ctl, enable SEL",
	.dw	word (phys @1+2624)
	.dw	phys @1+2624 >> 16
;1919 
;1920   "",
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
;1921 
;1922   "Disable Tuner Ctl",
	.dw	word (phys @1+2657)
	.dw	phys @1+2657 >> 16
;1923   "Enable Tuner Ctl",
	.dw	word (phys @1+2675)
	.dw	phys @1+2675 >> 16
;1924 
;1925   "", ""
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
;1926 //------------------------------------------
;1927   "", "", "", "",
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
;1928   "", "", "", ""
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
;1929 //------------------------------------------
;1930 };
	.dw	word (phys @1+9)
	.dw	phys @1+9 >> 16
	.ds	4,0
	.debug	G _SetDSPStateDesc 54 0
;1931 
;1932 
;1933 
;1934 
;1935 //--- parameter ID's for data in LLRF2 Device Data file ---
;1936 //-------------------------------------------------------------------------
;1937 // WARNING:  ID's can be added, but existing ID's (and the format of the
;1938 //           data they represent) should NEVER change!  NOTE:  Existing one
;1939 //           can be "eliminated" by changing their name and no longer
;1940 //           using them, but never REUSE an old one for something new!!!
;1941 //-------------------------------------------------------------------------
;1942 //  NOTE:  ID's >= 1000 are rsvd for values shared by most applications
;1943 //-------------------------------------------------------------------------
;1944 const ParamData far  devDataParams[] = {
;1945 //                ID   ---value---  --restoreFunc--
;1946   PERSISTENT_VAR(  1, trigPos,         NULL ),
	.seg	FARCONST
_devDataParams::
	.dw	0x1
	.dw	0x4
	.dw	0
	.dl	phys _trigPos
	.dl	phys @1+927
	.dw	0
;1947 
;1948   PERSISTENT_VAR(  2, warmLevel,       NULL ),
	.dw	0x2
	.dw	0x4
	.dw	0
	.dl	phys _warmLevel
	.dl	phys @1+935
	.dw	0
;1949   PERSISTENT_VAR(  3, warmTime,        NULL ),
	.dw	0x3
	.dw	0x4
	.dw	0
	.dl	phys _warmTime
	.dl	phys @1+945
	.dw	0
;1950   PERSISTENT_VAR(  4, rampTime,        NULL ),
	.dw	0x4
	.dw	0x4
	.dw	0
	.dl	phys _rampTime
	.dl	phys @1+954
	.dw	0
;1951 
;1952   PERSISTENT_VAR(  5, fwdAtten,        NULL ),
	.dw	0x5
	.dw	0x2
	.dw	0
	.dl	phys _fwdAtten
	.dl	phys @1+963
	.dw	0
;1953   PERSISTENT_VAR(  6, revAtten,        NULL ),
	.dw	0x6
	.dw	0x2
	.dw	0
	.dl	phys _revAtten
	.dl	phys @1+972
	.dw	0
;1954   PERSISTENT_VAR(  7, cavAtten,        NULL ),
	.dw	0x7
	.dw	0x2
	.dw	0
	.dl	phys _cavAtten
	.dl	phys @1+981
	.dw	0
;1955 
;1956   PERSISTENT_VAR(  8, spare0Atten,     NULL ),
	.dw	0x8
	.dw	0x2
	.dw	0
	.dl	phys _spare0Atten
	.dl	phys @1+990
	.dw	0
;1957   PERSISTENT_VAR(  9, spare1Atten,     NULL ),
	.dw	0x9
	.dw	0x2
	.dw	0
	.dl	phys _spare1Atten
	.dl	phys @1+1002
	.dw	0
;1958   PERSISTENT_VAR( 10, spare2Atten,     NULL ),
	.dw	0xa
	.dw	0x2
	.dw	0
	.dl	phys _spare2Atten
	.dl	phys @1+1014
	.dw	0
;1959 
;1960   PERSISTENT_VAR( 11, outAtten,        NULL ),
	.dw	0xb
	.dw	0x2
	.dw	0
	.dl	phys _outAtten
	.dl	phys @1+1026
	.dw	0
;1961 
;1962   PERSISTENT_VAR( 12, pulseFreq,       NULL ),
	.dw	0xc
	.dw	0x4
	.dw	0
	.dl	phys _pulseFreq
	.dl	phys @1+1052
	.dw	0
;1963   PERSISTENT_VAR( 13, pulseDuty,       NULL ),
	.dw	0xd
	.dw	0x4
	.dw	0
	.dl	phys _pulseDuty
	.dl	phys @1+1062
	.dw	0
;1964 
;1965   PERSISTENT_VAR( 14, rf_P,            NULL ),
	.dw	0xe
	.dw	0x4
	.dw	0
	.dl	phys _rf_P
	.dl	phys @1+1072
	.dw	0
;1966   PERSISTENT_VAR( 15, rf_I,            NULL ),
	.dw	0xf
	.dw	0x4
	.dw	0
	.dl	phys _rf_I
	.dl	phys @1+1077
	.dw	0
;1967   PERSISTENT_VAR( 16, rf_D,            NULL ),
	.dw	0x10
	.dw	0x4
	.dw	0
	.dl	phys _rf_D
	.dl	phys @1+1082
	.dw	0
;1968 
;1969   PERSISTENT_VAR( 17, tuner_P,         NULL ),
	.dw	0x11
	.dw	0x4
	.dw	0
	.dl	phys _tuner_P
	.dl	phys @1+1087
	.dw	0
;1970   PERSISTENT_VAR( 18, tuner_I,         NULL ),
	.dw	0x12
	.dw	0x4
	.dw	0
	.dl	phys _tuner_I
	.dl	phys @1+1095
	.dw	0
;1971   PERSISTENT_VAR( 19, tuner_D,         NULL ),
	.dw	0x13
	.dw	0x4
	.dw	0
	.dl	phys _tuner_D
	.dl	phys @1+1103
	.dw	0
;1972 
;1973   PERSISTENT_VAR( 20, rf_B0,           NULL ),
	.dw	0x14
	.dw	0x4
	.dw	0
	.dl	phys _rf_B0
	.dl	phys @1+1111
	.dw	0
;1974   PERSISTENT_VAR( 21, rf_WCtrl,        NULL ),
	.dw	0x15
	.dw	0x4
	.dw	0
	.dl	phys _rf_WCtrl
	.dl	phys @1+1117
	.dw	0
;1975   PERSISTENT_VAR( 22, rf_WObs,         NULL ),
	.dw	0x16
	.dw	0x4
	.dw	0
	.dl	phys _rf_WObs
	.dl	phys @1+1126
	.dw	0
;1976 
;1977   PERSISTENT_VAR( 23, tuner_B0,        NULL ),
	.dw	0x17
	.dw	0x4
	.dw	0
	.dl	phys _tuner_B0
	.dl	phys @1+1134
	.dw	0
;1978   PERSISTENT_VAR( 24, tuner_WCtrl,     NULL ),
	.dw	0x18
	.dw	0x4
	.dw	0
	.dl	phys _tuner_WCtrl
	.dl	phys @1+1143
	.dw	0
;1979   PERSISTENT_VAR( 25, tuner_WObs,      NULL ),
	.dw	0x19
	.dw	0x4
	.dw	0
	.dl	phys _tuner_WObs
	.dl	phys @1+1155
	.dw	0
;1980 
;1981   PERSISTENT_VAR( 26, cavLowIlkLevel,  NULL ),
	.dw	0x1a
	.dw	0x4
	.dw	0
	.dl	phys _cavLowIlkLevel
	.dl	phys @1+1193
	.dw	0
;1982   PERSISTENT_VAR( 27, cavLowIlkTime,   NULL ),
	.dw	0x1b
	.dw	0x4
	.dw	0
	.dl	phys _cavLowIlkTime
	.dl	phys @1+1208
	.dw	0
;1983 
;1984   PERSISTENT_VAR( 28, dvdtIlkRate,     NULL ),
	.dw	0x1c
	.dw	0x4
	.dw	0
	.dl	phys _dvdtIlkRate
	.dl	phys @1+1222
	.dw	0
;1985 
;1986   PERSISTENT_VAR( 29, revHighIlkRatio, NULL ),
	.dw	0x1d
	.dw	0x4
	.dw	0
	.dl	phys _revHighIlkRatio
	.dl	phys @1+1234
	.dw	0
;1987   PERSISTENT_VAR( 30, revHighIlkTime,  NULL ),
	.dw	0x1e
	.dw	0x4
	.dw	0
	.dl	phys _revHighIlkTime
	.dl	phys @1+1250
	.dw	0
;1988 
;1989   PERSISTENT_VAR( 31, fwdLowIlkRatio,  NULL ),
	.dw	0x1f
	.dw	0x4
	.dw	0
	.dl	phys _fwdLowIlkRatio
	.dl	phys @1+1265
	.dw	0
;1990   PERSISTENT_VAR( 32, fwdLowIlkTime,   NULL ),
	.dw	0x20
	.dw	0x4
	.dw	0
	.dl	phys _fwdLowIlkTime
	.dl	phys @1+1280
	.dw	0
;1991 
;1992   PERSISTENT_VAR( 33, fwdHighIlkLevel, NULL ),
	.dw	0x21
	.dw	0x4
	.dw	0
	.dl	phys _fwdHighIlkLevel
	.dl	phys @1+1294
	.dw	0
;1993   PERSISTENT_VAR( 34, fwdHighIlkTime,  NULL ),
	.dw	0x22
	.dw	0x4
	.dw	0
	.dl	phys _fwdHighIlkTime
	.dl	phys @1+1310
	.dw	0
;1994 
;1995   PERSISTENT_VAR( 35, fwdCavPhase,     NULL ),
	.dw	0x23
	.dw	0x4
	.dw	0
	.dl	phys _fwdCavPhase
	.dl	phys @1+1325
	.dw	0
;1996 
;1997   PERSISTENT_VAR( 36, ffPhase,         NULL ),
	.dw	0x24
	.dw	0x4
	.dw	0
	.dl	phys _ffPhase
	.dl	phys @1+1337
	.dw	0
;1998   PERSISTENT_VAR( 37, ffGain,          NULL ),
	.dw	0x25
	.dw	0x4
	.dw	0
	.dl	phys _ffGain
	.dl	phys @1+1345
	.dw	0
;1999 
;2000   PERSISTENT_VAR( 38, freqOffset,      NULL ),
	.dw	0x26
	.dw	0x4
	.dw	0
	.dl	phys _freqOffset
	.dl	phys @1+1352
	.dw	0
;2001 
;2002   PERSISTENT_VAR( 39, rfq_t1_offset,   NULL ),
	.dw	0x27
	.dw	0x4
	.dw	0
	.dl	phys _rfq_t1_offset
	.dl	phys @1+1378
	.dw	0
;2003   PERSISTENT_VAR( 40, rfq_t2_offset,   NULL ),
	.dw	0x28
	.dw	0x4
	.dw	0
	.dl	phys _rfq_t2_offset
	.dl	phys @1+1392
	.dw	0
;2004 
;2005   PERSISTENT_VAR( 41, fwdPhaseGain,      NULL ),
	.dw	0x29
	.dw	0x4
	.dw	0
	.dl	phys _fwdPhaseGain
	.dl	phys @1+1706
	.dw	0
;2006   PERSISTENT_VAR( 42, fwdPhaseOffset,    NULL ),
	.dw	0x2a
	.dw	0x4
	.dw	0
	.dl	phys _fwdPhaseOffset
	.dl	phys @1+1719
	.dw	0
;2007   PERSISTENT_VAR( 43, fwdAmplGain,       NULL ),
	.dw	0x2b
	.dw	0x4
	.dw	0
	.dl	phys _fwdAmplGain
	.dl	phys @1+1734
	.dw	0
;2008   PERSISTENT_VAR( 44, fwdAmplOffset,     NULL ),
	.dw	0x2c
	.dw	0x4
	.dw	0
	.dl	phys _fwdAmplOffset
	.dl	phys @1+1746
	.dw	0
;2009 
;2010   PERSISTENT_VAR( 45, revPhaseGain,      NULL ),
	.dw	0x2d
	.dw	0x4
	.dw	0
	.dl	phys _revPhaseGain
	.dl	phys @1+1760
	.dw	0
;2011   PERSISTENT_VAR( 46, revPhaseOffset,    NULL ),
	.dw	0x2e
	.dw	0x4
	.dw	0
	.dl	phys _revPhaseOffset
	.dl	phys @1+1773
	.dw	0
;2012   PERSISTENT_VAR( 47, revAmplGain,       NULL ),
	.dw	0x2f
	.dw	0x4
	.dw	0
	.dl	phys _revAmplGain
	.dl	phys @1+1788
	.dw	0
;2013   PERSISTENT_VAR( 48, revAmplOffset,     NULL ),
	.dw	0x30
	.dw	0x4
	.dw	0
	.dl	phys _revAmplOffset
	.dl	phys @1+1800
	.dw	0
;2014 
;2015   PERSISTENT_VAR( 49, cavPhaseGain,      NULL ),
	.dw	0x31
	.dw	0x4
	.dw	0
	.dl	phys _cavPhaseGain
	.dl	phys @1+1814
	.dw	0
;2016   PERSISTENT_VAR( 50, cavPhaseOffset,    NULL ),
	.dw	0x32
	.dw	0x4
	.dw	0
	.dl	phys _cavPhaseOffset
	.dl	phys @1+1827
	.dw	0
;2017   PERSISTENT_VAR( 51, cavAmplGain,       NULL ),
	.dw	0x33
	.dw	0x4
	.dw	0
	.dl	phys _cavAmplGain
	.dl	phys @1+1842
	.dw	0
;2018   PERSISTENT_VAR( 52, cavAmplOffset,     NULL ),
	.dw	0x34
	.dw	0x4
	.dw	0
	.dl	phys _cavAmplOffset
	.dl	phys @1+1854
	.dw	0
;2019 
;2020   PERSISTENT_VAR( 53, ref0PhaseGain,     NULL ),
	.dw	0x35
	.dw	0x4
	.dw	0
	.dl	phys _ref0PhaseGain
	.dl	phys @1+1868
	.dw	0
;2021   PERSISTENT_VAR( 54, ref0PhaseOffset,   NULL ),
	.dw	0x36
	.dw	0x4
	.dw	0
	.dl	phys _ref0PhaseOffset
	.dl	phys @1+1882
	.dw	0
;2022   PERSISTENT_VAR( 55, ref0AmplGain,      NULL ),
	.dw	0x37
	.dw	0x4
	.dw	0
	.dl	phys _ref0AmplGain
	.dl	phys @1+1898
	.dw	0
;2023   PERSISTENT_VAR( 56, ref0AmplOffset,    NULL ),
	.dw	0x38
	.dw	0x4
	.dw	0
	.dl	phys _ref0AmplOffset
	.dl	phys @1+1911
	.dw	0
;2024 
;2025   PERSISTENT_VAR( 57, spare0PhaseGain,   NULL ),
	.dw	0x39
	.dw	0x4
	.dw	0
	.dl	phys _spare0PhaseGain
	.dl	phys @1+1926
	.dw	0
;2026   PERSISTENT_VAR( 58, spare0PhaseOffset, NULL ),
	.dw	0x3a
	.dw	0x4
	.dw	0
	.dl	phys _spare0PhaseOffset
	.dl	phys @1+1942
	.dw	0
;2027   PERSISTENT_VAR( 59, spare0AmplGain,    NULL ),
	.dw	0x3b
	.dw	0x4
	.dw	0
	.dl	phys _spare0AmplGain
	.dl	phys @1+1960
	.dw	0
;2028   PERSISTENT_VAR( 60, spare0AmplOffset,  NULL ),
	.dw	0x3c
	.dw	0x4
	.dw	0
	.dl	phys _spare0AmplOffset
	.dl	phys @1+1975
	.dw	0
;2029 
;2030   PERSISTENT_VAR( 61, spare1PhaseGain,   NULL ),
	.dw	0x3d
	.dw	0x4
	.dw	0
	.dl	phys _spare1PhaseGain
	.dl	phys @1+1992
	.dw	0
;2031   PERSISTENT_VAR( 62, spare1PhaseOffset, NULL ),
	.dw	0x3e
	.dw	0x4
	.dw	0
	.dl	phys _spare1PhaseOffset
	.dl	phys @1+2008
	.dw	0
;2032   PERSISTENT_VAR( 63, spare1AmplGain,    NULL ),
	.dw	0x3f
	.dw	0x4
	.dw	0
	.dl	phys _spare1AmplGain
	.dl	phys @1+2026
	.dw	0
;2033   PERSISTENT_VAR( 64, spare1AmplOffset,  NULL ),
	.dw	0x40
	.dw	0x4
	.dw	0
	.dl	phys _spare1AmplOffset
	.dl	phys @1+2041
	.dw	0
;2034 
;2035   PERSISTENT_VAR( 65, spare2PhaseGain,   NULL ),
	.dw	0x41
	.dw	0x4
	.dw	0
	.dl	phys _spare2PhaseGain
	.dl	phys @1+2058
	.dw	0
;2036   PERSISTENT_VAR( 66, spare2PhaseOffset, NULL ),
	.dw	0x42
	.dw	0x4
	.dw	0
	.dl	phys _spare2PhaseOffset
	.dl	phys @1+2074
	.dw	0
;2037   PERSISTENT_VAR( 67, spare2AmplGain,    NULL ),
	.dw	0x43
	.dw	0x4
	.dw	0
	.dl	phys _spare2AmplGain
	.dl	phys @1+2092
	.dw	0
;2038   PERSISTENT_VAR( 68, spare2AmplOffset,  NULL ),
	.dw	0x44
	.dw	0x4
	.dw	0
	.dl	phys _spare2AmplOffset
	.dl	phys @1+2107
	.dw	0
;2039 
;2040   PERSISTENT_VAR( 69, ref1PhaseGain,     NULL ),
	.dw	0x45
	.dw	0x4
	.dw	0
	.dl	phys _ref1PhaseGain
	.dl	phys @1+2124
	.dw	0
;2041   PERSISTENT_VAR( 70, ref1PhaseOffset,   NULL ),
	.dw	0x46
	.dw	0x4
	.dw	0
	.dl	phys _ref1PhaseOffset
	.dl	phys @1+2138
	.dw	0
;2042   PERSISTENT_VAR( 71, ref1AmplGain,      NULL ),
	.dw	0x47
	.dw	0x4
	.dw	0
	.dl	phys _ref1AmplGain
	.dl	phys @1+2154
	.dw	0
;2043   PERSISTENT_VAR( 72, ref1AmplOffset,    NULL ),
	.dw	0x48
	.dw	0x4
	.dw	0
	.dl	phys _ref1AmplOffset
	.dl	phys @1+2167
	.dw	0
;2044 
;2045   PERSISTENT_VAR( 73, maxAmplSet,        NULL ),
	.dw	0x49
	.dw	0x4
	.dw	0
	.dl	phys _maxAmplSet
	.dl	phys @1+2182
	.dw	0
;2046 
;2047   #define SharedDevParams
;2048   #include <sharedAppl.c>
;2049 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
	.dw	0x3ec
	.dw	0x2
	.dw	0
	.dl	phys _timeZone
	.dl	phys @1+2692
	.dw	0
	.dw	0x3ed
	.dw	0x4
	.dw	0
	.dl	phys _srfsIPAddr
	.dl	phys @1+2701
	.dw	0
	.dw	0x3e8
	.dw	0x18
	.dw	0x18
	.dl	phys _ainLabel
	.dl	phys @1+2712
	.dw	0
	.dw	0x3e9
	.dw	0x18
	.dw	0x18
	.dl	phys _aoutLabel
	.dl	phys @1+2721
	.dw	0
	.dw	0x3ea
	.dw	0x18
	.dw	0x18
	.dl	phys _dinLabel
	.dl	phys @1+2731
	.dw	0
	.dw	0x3eb
	.dw	0x30
	.dw	0x18
	.dl	phys _doutLabel
	.dl	phys @1+2740
	.dw	0
	.dw	0x3ee
	.dw	0x4
	.dw	0
	.dl	phys _readGain
	.dl	phys @1+2750
	.dw	0
	.dw	0x3ef
	.dw	0x4
	.dw	0
	.dl	phys _readOffset
	.dl	phys @1+2759
	.dw	0
	.dw	0x3f0
	.dw	0x4
	.dw	0
	.dl	phys _setGain
	.dl	phys @1+2770
	.dw	0
	.dw	0x3f1
	.dw	0x4
	.dw	0
	.dl	phys _setOffset
	.dl	phys @1+2778
	.dw	word _ReadSetCalibsRestore
	.dw	0x3f2
	.dw	0x20
	.dw	0
	.dl	phys _hostName
	.dl	phys @1+2788
	.dw	word _hostNameRestore
	.dw	0x3f3
	.dw	0x4
	.dw	0
	.dl	phys _srfsAltAddr
	.dl	phys @1+2797
	.dw	0
	.dw	0x3f4
	.dw	0x300
	.dw	0x18
	.dl	phys _extraInLabel
	.dl	phys @1+2809
	.dw	0
	.dw	0x3f5
	.dw	0x4
	.dw	0
	.dl	phys _netmask
	.dl	phys @1+797
	.dw	0
	.dw	0x3f6
	.dw	0x4
	.dw	0
	.dl	phys _gateway
	.dl	phys @1+805
	.dw	0
	.debug	I
;2050   END_PARAM_DATA
	.dw	0
	.dw	0
	.dw	0
	.dw	0, 0
	.dw	0, 0
;2051 
	.ds	2,0
	.debug	G _devDataParams 55 8
;2052 
;2053 //----------------------------------------------------
;2054 
;2055 //RESTORE_FUNC(ampSet);
;2056 
;2057 const ParamData far  settingsParams[] = {
;2058 //APPLDATA
;2059 //_nearcall void xxxSetRestore(void far *curVal, char *newVal, uint count);
;2060 //
;2061 //                ID   ---value---  --restoreFunc--
;2062 //  PERSISTENT_VAR(  1,  xxxSet,     PVRF(xxxSet) ),
;2063   PERSISTENT_VAR(  1, piezoSetPos,    NULL ),
_settingsParams::
	.dw	0x1
	.dw	0x2
	.dw	0
	.dl	phys _piezoSetPos
	.dl	phys @1+1450
	.dw	0
;2064   PERSISTENT_VAR(  2, stepperSetPos,  NULL ),
	.dw	0x2
	.dw	0x2
	.dw	0
	.dl	phys _stepperSetPos
	.dl	phys @1+1462
	.dw	0
;2065 
;2066   PERSISTENT_VAR(  3, phaseSet,       NULL ),
	.dw	0x3
	.dw	0x4
	.dw	0
	.dl	phys _phaseSet
	.dl	phys @1+1035
	.dw	0
;2067   PERSISTENT_VAR(  4, amplSet,        NULL ),
	.dw	0x4
	.dw	0x4
	.dw	0
	.dl	phys _amplSet
	.dl	phys @1+1044
	.dw	0
;2068 
;2069   PERSISTENT_VAR(  5, pulseAmplSet,    NULL ),
	.dw	0x5
	.dw	0x4
	.dw	0
	.dl	phys _pulseAmplSet
	.dl	phys @1+1166
	.dw	0
;2070   PERSISTENT_VAR(  6, pulsePhaseSet,   NULL ),
	.dw	0x6
	.dw	0x4
	.dw	0
	.dl	phys _pulsePhaseSet
	.dl	phys @1+1179
	.dw	0
;2071 
;2072   PERSISTENT_VAR(  7, rfq_manTuneSet,  NULL ),
	.dw	0x7
	.dw	0x4
	.dw	0
	.dl	phys _rfq_manTuneSet
	.dl	phys @1+1363
	.dw	0
;2073 
;2074   END_PARAM_DATA
	.dw	0
	.dw	0
	.dw	0
	.dw	0, 0
	.dw	0, 0
;2075 
	.ds	2,0
	.debug	G _settingsParams 57 8
;2076 
;2077 
;2078 //-----------------------------------------------------------------
;2079 //  Application-specific values needed to restore the state of the
;2080 //  controller following a "soft" reset (i.e. a restart without a
;2081 //  CPU reset)
;2082 //-----------------------------------------------------------------
;2083 const ParamData far  ctlrStateParams[] = {
;2084 // sample values from sharedAppl.c, common to all applications
;2085 //                   --ID--  ---value,granularity---  --restoreFunc--
;2086 //    PERSISTENT_VARx(  1000,  prevState,               NULL ),
;2087 //    PERSISTENT_VARx(  1001,  curState,                NULL ),
;2088 //    PERSISTENT_VARx(  1002,  nextState,               NULL ),
;2089 //    PERSISTENT_TBL (  1003,  setVal,4,                NULL ),
;2090 // // PERSISTENT_TBL (  1004,  shadowRegValues,4,       NULL ),
;2091 
;2092 //APPLDATA
;2093 //    PERSISTENT_VAR(  1,  myApplStateVar,        NULL ),
;2094 
;2095 
;2096   #define SharedCtlrStateParams
;2097   #include <sharedAppl.c>
;2098 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
	.debug	I
;2099   END_PARAM_DATA
_ctlrStateParams::
	.dw	0
	.dw	0
	.dw	0
	.dw	0, 0
	.dw	0, 0
;2100 
	.ds	2,0
	.debug	G _ctlrStateParams 58 8
;2101 
;2102 
;2103 //-------------------------------------------------------------------------
;2104 //  A list of parameterized "persistent" data.  Backup copies of such value
;2105 //  are stored in files and restored on startup.
;2106 //-------------------------------------------------------------------------
;2107 
;2108 //    Base File Name    Category  ParamData
;2109 //    ----------------  --------  ---------
;2110 //--- add any application-specific parameterized file defs here ---
;2111 //const ParamFile far  myAppCfgFile =
;2112 //  { "myFileName",     PDC_CTLR, myCfgParams,   0 };
;2113 
;2114 ParamFile  settingsFile =
;2115     { "settings",       PDC_SET,  settingsParams  };
	.dseg
_settingsFile::
	.dw	@2+33
	.db	0x4
	.dl	phys _settingsParams
	.ds	42,0
	.debug	G _settingsFile 59 0
;2116 
;2117 
;2118 #define SharedParams
;2119 #include <sharedAppl.c>
;2120 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
_devDataFile::
	.dw	@2+42
	.db	0x3
	.dl	phys _devDataParams
	.dw	0
	.ds	30,0
	.ds	10,0
	.debug	G _devDataFile 59 0
_hdwCfgFile::
	.dw	@2+50
	.db	0x1
	.dl	phys _hdwCfgParams
	.dw	0
	.ds	30,0
	.ds	10,0
	.debug	G _hdwCfgFile 59 0
	.debug	I
;2121 
;2122 ParamFile *paramFiles[] = {
;2123   #define SharedParamsList
;2124   #include <sharedAppl.c>
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
_paramFiles::
	.dw	word _hdwCfgFile
	.dw	word _devDataFile
	.debug	I
;2125 //--- add any application-specific ParamData references here ---
;2126 //  &myAppCfgFile
;2127   &settingsFile
;2128 };
	.dw	word _settingsFile
	.debug	G _paramFiles 60 0
;2129 
;2130 const int NumParamFiles = sizeof(paramFiles) / sizeof(paramFiles[0]);
	.seg	CONST
_NumParamFiles::
	.dw	0x3
	.debug	G _NumParamFiles 6 0
;2131 
;2132 
;2133 
;2134 float  ilkRatioGain, ilkRatioOffset, maxOutAmplGain, maxOutAmplOffset;
;2135 
;2136 
;2137 
;2138 //===== FPGA registers that require special handling =====
;2139 #define  RF_SOFT_ON  0x0E
;2140 
;2141 
;2142 
;2143 #define  DSP_RD( dspName, dspType, mbReg, gain, offset, typ, noise )  
;2144     { dspName, RT_##dspType, &mbReg##_RegInfo, gain, offset, typ, noise }
;2145 
;2146 const RD_DSP_REG far rdDSPregs[] = {
;2147   //      DSP Reg ID      DSP fmt ModBus Reg      Gain              Offset
;2148   //      --------------  ------- --------------  --------------    -------
;2149   DSP_RD( FwdPhase,       PHASE,  fwdPhase,       &fwdPhaseGain,    &fwdPha
	.seg	FARCONST
_rdDSPregs::
	.dw	0, 0x10
	.db	0x63
	.dw	word _fwdPhase_RegInfo
	.dw	word _fwdPhaseGain
	.dw	word _fwdPhaseOffset
	.dw	0xffffd9e6, 0x8230
	.dw	0x5
;2150   DSP_RD( RevPhase,       PHASE,  revPhase,       &revPhaseGain,    &revPha
	.dw	0x2, 0x10
	.db	0x63
	.dw	word _revPhase_RegInfo
	.dw	word _revPhaseGain
	.dw	word _revPhaseOffset
	.dw	0xffff88e8, 0xe642
	.dw	0x5
;2151   DSP_RD( CavPhase,       PHASE,  cavPhase,       &cavPhaseGain,    &cavPha
	.dw	0x4, 0x10
	.db	0x63
	.dw	word _cavPhase_RegInfo
	.dw	word _cavPhaseGain
	.dw	word _cavPhaseOffset
	.dw	0xffffa22c, 0xda93
	.dw	0x5
;2152   DSP_RD( Ref0Phase,      PHASE,  ref0Phase,      &ref0PhaseGain,   &ref0Ph
	.dw	0x6, 0x10
	.db	0x63
	.dw	word _ref0Phase_RegInfo
	.dw	word _ref0PhaseGain
	.dw	word _ref0PhaseOffset
	.dw	0xffff809c, 0xebf0
	.dw	0x5
;2153 
;2154   DSP_RD( Spare0Phase,    PHASE,  spare0Phase,    &spare0PhaseGain, &spare0
	.dw	0x8, 0x10
	.db	0x63
	.dw	word _spare0Phase_RegInfo
	.dw	word _spare0PhaseGain
	.dw	word _spare0PhaseOffset
	.dw	0xffffd9e6, 0x8230
	.dw	0x5
;2155   DSP_RD( Spare1Phase,    PHASE,  spare1Phase,    &spare1PhaseGain, &spare1
	.dw	0xa, 0x10
	.db	0x63
	.dw	word _spare1Phase_RegInfo
	.dw	word _spare1PhaseGain
	.dw	word _spare1PhaseOffset
	.dw	0xffff88e8, 0xe642
	.dw	0x5
;2156   DSP_RD( Spare2Phase,    PHASE,  spare2Phase,    &spare2PhaseGain, &spare2
	.dw	0xc, 0x10
	.db	0x63
	.dw	word _spare2Phase_RegInfo
	.dw	word _spare2PhaseGain
	.dw	word _spare2PhaseOffset
	.dw	0xffffa22c, 0xda93
	.dw	0x5
;2157   DSP_RD( Ref1Phase,      PHASE,  ref1Phase,      &ref1PhaseGain,   &ref1Ph
	.dw	0xe, 0x10
	.db	0x63
	.dw	word _ref1Phase_RegInfo
	.dw	word _ref1PhaseGain
	.dw	word _ref1PhaseOffset
	.dw	0xffff809c, 0xebf0
	.dw	0x5
;2158 
;2159   DSP_RD( FwdAmpl,        F32,    fwdAmpl,        &fwdAmplGain,     &fwdAmp
	.dw	0x10, 0x10
	.db	0x4
	.dw	word _fwdAmpl_RegInfo
	.dw	word _fwdAmplGain
	.dw	word _fwdAmplOffset
	.dw	0xba5, 0x3f8d
	.dw	0x5
;2160   DSP_RD( RevAmpl,        F32,    revAmpl,        &revAmplGain,     &revAmp
	.dw	0x12, 0x10
	.db	0x4
	.dw	word _revAmpl_RegInfo
	.dw	word _revAmplGain
	.dw	word _revAmplOffset
	.dw	0x6060, 0x3e60
	.dw	0x5
;2161   DSP_RD( CavAmpl,        F32,    cavAmpl,        &cavAmplGain,     &cavAmp
	.dw	0x14, 0x10
	.db	0x4
	.dw	word _cavAmpl_RegInfo
	.dw	word _cavAmplGain
	.dw	word _cavAmplOffset
	.dw	0xffffa76b, 0x4013
	.dw	0x5
;2162   DSP_RD( Ref0Ampl,       F32,    ref0Ampl,       &ref0AmplGain,    &ref0Am
	.dw	0x16, 0x10
	.db	0x4
	.dw	word _ref0Ampl_RegInfo
	.dw	word _ref0AmplGain
	.dw	word _ref0AmplOffset
	.dw	0xfffffe6e, 0x3ee4
	.dw	0x5
;2163 
;2164   DSP_RD( Spare0Ampl,     F32,    spare0Ampl,     &spare0AmplGain,  &spare0
	.dw	0x18, 0x10
	.db	0x4
	.dw	word _spare0Ampl_RegInfo
	.dw	word _spare0AmplGain
	.dw	word _spare0AmplOffset
	.dw	0xba5, 0x3f8d
	.dw	0x5
;2165   DSP_RD( Spare1Ampl,     F32,    spare1Ampl,     &spare1AmplGain,  &spare1
	.dw	0x1a, 0x10
	.db	0x4
	.dw	word _spare1Ampl_RegInfo
	.dw	word _spare1AmplGain
	.dw	word _spare1AmplOffset
	.dw	0x6060, 0x3e60
	.dw	0x5
;2166   DSP_RD( Spare2Ampl,     F32,    spare2Ampl,     &spare2AmplGain,  &spare2
	.dw	0x1c, 0x10
	.db	0x4
	.dw	word _spare2Ampl_RegInfo
	.dw	word _spare2AmplGain
	.dw	word _spare2AmplOffset
	.dw	0xffffa76b, 0x4013
	.dw	0x5
;2167   DSP_RD( Ref1Ampl,       F32,    ref1Ampl,       &ref1AmplGain,    &ref1Am
	.dw	0x1e, 0x10
	.db	0x4
	.dw	word _ref1Ampl_RegInfo
	.dw	word _ref1AmplGain
	.dw	word _ref1AmplOffset
	.dw	0xfffffe6e, 0x3ee4
	.dw	0x5
;2168 
;2169   DSP_RD( MaxFwdAmpl,     F32,    maxFwdAmpl,     &fwdAmplGain,     &fwdAmp
	.dw	0x20, 0x10
	.db	0x4
	.dw	word _maxFwdAmpl_RegInfo
	.dw	word _fwdAmplGain
	.dw	word _fwdAmplOffset
	.dw	0x18de, 0x4079
	.dw	0
;2170   DSP_RD( MaxRevAmpl,     F32,    maxRevAmpl,     &revAmplGain,     &revAmp
	.dw	0x22, 0x10
	.db	0x4
	.dw	word _maxRevAmpl_RegInfo
	.dw	word _revAmplGain
	.dw	word _revAmplOffset
	.dw	0xffffc7ee, 0x3fe3
	.dw	0
;2171   DSP_RD( MaxCavAmpl,     F32,    maxCavAmpl,     &cavAmplGain,     &cavAmp
	.dw	0x24, 0x10
	.db	0x4
	.dw	word _maxCavAmpl_RegInfo
	.dw	word _cavAmplGain
	.dw	word _cavAmplOffset
	.dw	0x6f19, 0x40df
	.dw	0
;2172 
;2173   DSP_RD( MaxSpare0Ampl,  F32,    maxSpare0Ampl,  &spare0AmplGain,  &spare0
	.dw	0x26, 0x10
	.db	0x4
	.dw	word _maxSpare0Ampl_RegInfo
	.dw	word _spare0AmplGain
	.dw	word _spare0AmplOffset
	.dw	0x18de, 0x4079
	.dw	0
;2174   DSP_RD( MaxSpare1Ampl,  F32,    maxSpare1Ampl,  &spare1AmplGain,  &spare1
	.dw	0x28, 0x10
	.db	0x4
	.dw	word _maxSpare1Ampl_RegInfo
	.dw	word _spare1AmplGain
	.dw	word _spare1AmplOffset
	.dw	0xffffc7ee, 0x3fe3
	.dw	0
;2175   DSP_RD( MaxSpare2Ampl,  F32,    maxSpare2Ampl,  &spare2AmplGain,  &spare2
	.dw	0x2a, 0x10
	.db	0x4
	.dw	word _maxSpare2Ampl_RegInfo
	.dw	word _spare2AmplGain
	.dw	word _spare2AmplOffset
	.dw	0x6f19, 0x40df
	.dw	0
;2176 
;2177   DSP_RD( MaxOutAmpl,     F32,    maxOutAmpl,     &maxOutAmplGain,  &maxOut
	.dw	0x2c, 0x10
	.db	0x4
	.dw	word _maxOutAmpl_RegInfo
	.dw	word _maxOutAmplGain
	.dw	word _maxOutAmplOffset
	.dw	0x6fce, 0x3fcd
	.dw	0
;2178 
;2179   DSP_RD( StatusBitsDSP,  BITS,   statusBitsDSP,  NULL,             NULL,  
	.dw	0x2e, 0x10
	.db	0x3
	.dw	word _statusBitsDSP_RegInfo
	.dw	0
	.dw	0
	.dw	0x2088, 0x30
	.dw	0
;2180 
;2181   DSP_RD( DSPLoad,        F32,    dspLoad,        NULL,             NULL,  
	.dw	0x30, 0x10
	.db	0x4
	.dw	word _dspLoad_RegInfo
	.dw	0
	.dw	0
	.dw	0xffffb886, 0x4284
	.dw	0xa
;2182   DSP_RD( PCBTemp,        F32,    pcbTemp,        NULL,             NULL,  
	.dw	0x32, 0x10
	.db	0x4
	.dw	word _pcbTemp_RegInfo
	.dw	0
	.dw	0
	.dw	0xffffb886, 0x4284
	.dw	0x5
;2183   DSP_RD( Fan0Spd,        U32,    fan0Spd,        NULL,             NULL,  
	.dw	0x34, 0x10
	.db	0x3
	.dw	word _fan0Spd_RegInfo
	.dw	0
	.dw	0
	.dw	0x3e8, 0
	.dw	0x5
;2184   DSP_RD( Fan1Spd,        U32,    fan1Spd,        NULL,             NULL,  
	.dw	0x36, 0x10
	.db	0x3
	.dw	word _fan1Spd_RegInfo
	.dw	0
	.dw	0
	.dw	0x3e8, 0
	.dw	0x5
;2185 
;2186   DSP_RD( RFQ_T1_Output,  F32,    rfq_t1_output,  NULL,             NULL,  
	.dw	0x19e, 0x10
	.db	0x4
	.dw	word _rfq_t1_output_RegInfo
	.dw	0
	.dw	0
	.dw	0, 0
	.dw	0
;2187   DSP_RD( RFQ_T2_Output,  F32,    rfq_t2_output,  NULL,             NULL,  
	.dw	0x1a0, 0x10
	.db	0x4
	.dw	word _rfq_t2_output_RegInfo
	.dw	0
	.dw	0
	.dw	0, 0
	.dw	0
;2188 
;2189 //--- R/W registers that we want to read from each pass ---
;2190   DSP_RD( PiezoTunerPos,  U32,    piezoReadPos,   NULL,             NULL,  
	.dw	0x202, 0x10
	.db	0x3
	.dw	word _piezoReadPos_RegInfo
	.dw	0
	.dw	0
	.dw	0, 0
	.dw	0
;2191   DSP_RD( StepperPos,     U32,    stepperReadPos, NULL,             NULL,  
	.dw	0x204, 0x10
	.db	0x3
	.dw	word _stepperReadPos_RegInfo
	.dw	0
	.dw	0
	.dw	0, 0
	.dw	0
;2192 
;2193 //--- RO registers in "special" registers group ---
;2194   DSP_RD( PercentFFT,     U32,    percentFFT,     NULL,             NULL,  
	.dw	0x222, 0x10
	.db	0x3
	.dw	word _percentFFT_RegInfo
	.dw	0
	.dw	0
	.dw	0x64, 0
	.dw	0
;2195   DSP_RD( SizeFFT,        U32,    sizeFFT,        NULL,             NULL,  
	.dw	0x224, 0x10
	.db	0x3
	.dw	word _sizeFFT_RegInfo
	.dw	0
	.dw	0
	.dw	0x3e9, 0
	.dw	0
;2196   DSP_RD( ROMProgErrs,    U32,    romProgErrs,    NULL,             NULL,  
	.dw	0x24c, 0x10
	.db	0x3
	.dw	word _romProgErrs_RegInfo
	.dw	0
	.dw	0
	.dw	0, 0
	.dw	0
;2197 
;2198 //--- R/W registers in "special" group that provide the current value from 
;2199 //--- a table of values ---
;2200   DSP_RD( FwdMagCalib,      F32,    fwdMagCalib,      NULL,         NULL,  
	.dw	0x22a, 0x10
	.db	0x4
	.dw	word _fwdMagCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0x43d5, 0x3117
	.dw	0
;2201   DSP_RD( FwdPhaseCalib,    PHASE,  fwdPhaseCalib,    NULL,         NULL,  
	.dw	0x22c, 0x10
	.db	0x63
	.dw	word _fwdPhaseCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0xffffafce, 0x8c7e
	.dw	0
;2202 
;2203   DSP_RD( RevMagCalib,      F32,    revMagCalib,      NULL,         NULL,  
	.dw	0x22e, 0x10
	.db	0x4
	.dw	word _revMagCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0x5217, 0x308a
	.dw	0
;2204   DSP_RD( RevPhaseCalib,    PHASE,  revPhaseCalib,    NULL,         NULL,  
	.dw	0x230, 0x10
	.db	0x63
	.dw	word _revPhaseCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0xffffa5a2, 0x96f5
	.dw	0
;2205 
;2206   DSP_RD( CavMagCalib,      F32,    cavMagCalib,      NULL,         NULL,  
	.dw	0x232, 0x10
	.db	0x4
	.dw	word _cavMagCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0xffffae52, 0x3187
	.dw	0
;2207   DSP_RD( CavPhaseCalib,    PHASE,  cavPhaseCalib,    NULL,         NULL,  
	.dw	0x234, 0x10
	.db	0x63
	.dw	word _cavPhaseCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0xffff91bc, 0x8f7f
	.dw	0
;2208 
;2209   DSP_RD( Spare0MagCalib,   F32,    spare0MagCalib,   NULL,         NULL,  
	.dw	0x236, 0x10
	.db	0x4
	.dw	word _spare0MagCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0x43d5, 0x3117
	.dw	0
;2210   DSP_RD( Spare0PhaseCalib, PHASE,  spare0PhaseCalib, NULL,         NULL,  
	.dw	0x238, 0x10
	.db	0x63
	.dw	word _spare0PhaseCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0xffffafce, 0x8c7e
	.dw	0
;2211 
;2212   DSP_RD( Spare1MagCalib,   F32,    spare1MagCalib,   NULL,         NULL,  
	.dw	0x23a, 0x10
	.db	0x4
	.dw	word _spare1MagCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0x5217, 0x308a
	.dw	0
;2213   DSP_RD( Spare1PhaseCalib, PHASE,  spare1PhaseCalib, NULL,         NULL,  
	.dw	0x23c, 0x10
	.db	0x63
	.dw	word _spare1PhaseCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0xffffa5a2, 0x96f5
	.dw	0
;2214 
;2215   DSP_RD( Spare2MagCalib,   F32,    spare2MagCalib,   NULL,         NULL,  
	.dw	0x23e, 0x10
	.db	0x4
	.dw	word _spare2MagCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0xffffae52, 0x3187
	.dw	0
;2216   DSP_RD( Spare2PhaseCalib, PHASE,  spare2PhaseCalib, NULL,         NULL,  
	.dw	0x240, 0x10
	.db	0x63
	.dw	word _spare2PhaseCalib_RegInfo
	.dw	0
	.dw	0
	.dw	0xffffafce, 0x8c7e
	.dw	0
;2217 
;2218   DSP_RD( OutMagCalib0,     F32,    outMagCalib0,     NULL,         NULL,  
	.dw	0x242, 0x10
	.db	0x4
	.dw	word _outMagCalib0_RegInfo
	.dw	0
	.dw	0
	.dw	0x3dd5, 0x30a9
	.dw	0
;2219   DSP_RD( OutPhaseCalib0,   PHASE,  outPhaseCalib0,   NULL,         NULL,  
	.dw	0x244, 0x10
	.db	0x63
	.dw	word _outPhaseCalib0_RegInfo
	.dw	0
	.dw	0
	.dw	0, 0
	.dw	0
;2220 
;2221   DSP_RD( OutMagCalib1,     F32,    outMagCalib1,     NULL,         NULL,  
	.dw	0x246, 0x10
	.db	0x4
	.dw	word _outMagCalib1_RegInfo
	.dw	0
	.dw	0
	.dw	0x3dd5, 0x30a9
	.dw	0
;2222   DSP_RD( OutPhaseCalib1,   PHASE,  outPhaseCalib1,   NULL,         NULL,  
	.dw	0x248, 0x10
	.db	0x63
	.dw	word _outPhaseCalib1_RegInfo
	.dw	0
	.dw	0
	.dw	0, 0
	.dw	0
;2223 
	.debug	G _rdDSPregs 62 8
;2224 
;2225 #define  NUM_RD_DSP_REGS  (sizeof(rdDSPregs) / sizeof(RD_DSP_REG))
;2226 
;2227 const uint  Num_RD_DSP_Regs = NUM_RD_DSP_REGS;
	.seg	CONST
_Num_RD_DSP_Regs::
	.dw	0x33
	.debug	G _Num_RD_DSP_Regs 7 0
;2228 
;2229 
;2230 #define  DSP_WR( dspName, dspType, mbReg, min, max, mode, gain, offset )  
;2231     { dspName, RT_##dspType, &mbReg##_RegInfo, min, max, WM_##mode, gain, o
;2232 
;2233 const WR_DSP_REG far wrDSPregs[] = {
;2234   //      DSP Reg ID      DSP fmt ModBus Reg       Min    Max     Mode  Gai
;2235   //      --------------  ------- --------------   ----   ------  ----  ---
;2236   DSP_WR( TrigPos,        U32,    trigPos,         0,     1023,   EVERY, NU
	.seg	FARCONST
_wrDSPregs::
	.dw	0x100, 0x10
	.db	0x3
	.dw	word _trigPos_RegInfo
	.dw	0, 0
	.dw	0xffffc000, 0x447f
	.db	0x1
	.dw	0
	.dw	0
;2237   DSP_WR( WarmLevel,      F32,    warmLevel,       0.0,   0.0,    EVERY, &c
	.dw	0x102, 0x10
	.db	0x4
	.dw	word _warmLevel_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	word _cavAmplGain
	.dw	word _cavAmplOffset
;2238   DSP_WR( WarmTime,       F32,    warmTime,        0.0,   32767,  EVERY, NU
	.dw	0x104, 0x10
	.db	0x4
	.dw	word _warmTime_RegInfo
	.dw	0, 0
	.dw	0xfffffe00, 0x46ff
	.db	0x1
	.dw	0
	.dw	0
;2239   DSP_WR( RampTime,       F32,    rampTime,        0.0,   32767,  EVERY, NU
	.dw	0x106, 0x10
	.db	0x4
	.dw	word _rampTime_RegInfo
	.dw	0, 0
	.dw	0xfffffe00, 0x46ff
	.db	0x1
	.dw	0
	.dw	0
;2240 
;2241   DSP_WR( FwdAtten,       U32,    fwdAtten,        0,     63,     EVERY, NU
	.dw	0x108, 0x10
	.db	0x3
	.dw	word _fwdAtten_RegInfo
	.dw	0, 0
	.dw	0, 0x427c
	.db	0x1
	.dw	0
	.dw	0
;2242   DSP_WR( RevAtten,       U32,    revAtten,        0,     63,     EVERY, NU
	.dw	0x10a, 0x10
	.db	0x3
	.dw	word _revAtten_RegInfo
	.dw	0, 0
	.dw	0, 0x427c
	.db	0x1
	.dw	0
	.dw	0
;2243   DSP_WR( CavAtten,       U32,    cavAtten,        0,     63,     EVERY, NU
	.dw	0x10c, 0x10
	.db	0x3
	.dw	word _cavAtten_RegInfo
	.dw	0, 0
	.dw	0, 0x427c
	.db	0x1
	.dw	0
	.dw	0
;2244 
;2245   DSP_WR( Spare0Atten,    U32,    spare0Atten,     0,     63,     EVERY, NU
	.dw	0x10e, 0x10
	.db	0x3
	.dw	word _spare0Atten_RegInfo
	.dw	0, 0
	.dw	0, 0x427c
	.db	0x1
	.dw	0
	.dw	0
;2246   DSP_WR( Spare1Atten,    U32,    spare1Atten,     0,     63,     EVERY, NU
	.dw	0x110, 0x10
	.db	0x3
	.dw	word _spare1Atten_RegInfo
	.dw	0, 0
	.dw	0, 0x427c
	.db	0x1
	.dw	0
	.dw	0
;2247   DSP_WR( Spare2Atten,    U32,    spare2Atten,     0,     63,     EVERY, NU
	.dw	0x112, 0x10
	.db	0x3
	.dw	word _spare2Atten_RegInfo
	.dw	0, 0
	.dw	0, 0x427c
	.db	0x1
	.dw	0
	.dw	0
;2248 
;2249   DSP_WR( OutAtten,       U32,    outAtten,        0,     63,     EVERY, NU
	.dw	0x114, 0x10
	.db	0x3
	.dw	word _outAtten_RegInfo
	.dw	0, 0
	.dw	0, 0x427c
	.db	0x1
	.dw	0
	.dw	0
;2250 
;2251   DSP_WR( PhaseSet,       PHASE,  phaseSet,        0.0,   0.0,    EVERY, &c
	.dw	0x116, 0x10
	.db	0x63
	.dw	word _phaseSet_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	word _cavPhaseGain
	.dw	word _cavPhaseOffset
;2252   DSP_WR( AmplSet,        F32,    amplSet,         0.0,   0.0,    EVERY, &c
	.dw	0x118, 0x10
	.db	0x4
	.dw	word _amplSet_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	word _cavAmplGain
	.dw	word _cavAmplOffset
;2253 
;2254   DSP_WR( PulseFreq,      F32,    pulseFreq,       0.0,   0.0,    EVERY, NU
	.dw	0x11a, 0x10
	.db	0x4
	.dw	word _pulseFreq_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2255   DSP_WR( PulseDuty,      F32,    pulseDuty,       0.0,   0.0,    EVERY, NU
	.dw	0x11c, 0x10
	.db	0x4
	.dw	word _pulseDuty_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2256 
;2257   DSP_WR( RF_P,           F32,    rf_P,            0.0,   0.0,    EVERY, NU
	.dw	0x11e, 0x10
	.db	0x4
	.dw	word _rf_P_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2258   DSP_WR( RF_I,           F32,    rf_I,            0.0,   0.0,    EVERY, NU
	.dw	0x120, 0x10
	.db	0x4
	.dw	word _rf_I_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2259   DSP_WR( RF_D,           F32,    rf_D,            0.0,   0.0,    EVERY, NU
	.dw	0x122, 0x10
	.db	0x4
	.dw	word _rf_D_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2260 
;2261   DSP_WR( Tuner_P,        F32,    tuner_P,         0.0,   0.0,    EVERY, NU
	.dw	0x124, 0x10
	.db	0x4
	.dw	word _tuner_P_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2262   DSP_WR( Tuner_I,        F32,    tuner_I,         0.0,   0.0,    EVERY, NU
	.dw	0x126, 0x10
	.db	0x4
	.dw	word _tuner_I_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2263   DSP_WR( Tuner_D,        F32,    tuner_D,         0.0,   0.0,    EVERY, NU
	.dw	0x128, 0x10
	.db	0x4
	.dw	word _tuner_D_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2264 
;2265   DSP_WR( RF_B0,          F32,    rf_B0,           0.0,   0.0,    EVERY, NU
	.dw	0x12a, 0x10
	.db	0x4
	.dw	word _rf_B0_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2266   DSP_WR( RF_WCtrl,       F32,    rf_WCtrl,        0.0,   0.0,    EVERY, NU
	.dw	0x12c, 0x10
	.db	0x4
	.dw	word _rf_WCtrl_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2267   DSP_WR( RF_WObs,        F32,    rf_WObs,         0.0,   0.0,    EVERY, NU
	.dw	0x12e, 0x10
	.db	0x4
	.dw	word _rf_WObs_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2268 
;2269   DSP_WR( Tuner_B0,       F32,    tuner_B0,        0.0,   0.0,    EVERY, NU
	.dw	0x130, 0x10
	.db	0x4
	.dw	word _tuner_B0_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2270   DSP_WR( Tuner_WCtrl,    F32,    tuner_WCtrl,     0.0,   0.0,    EVERY, NU
	.dw	0x132, 0x10
	.db	0x4
	.dw	word _tuner_WCtrl_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2271   DSP_WR( Tuner_WObs,     F32,    tuner_WObs,      0.0,   0.0,    EVERY, NU
	.dw	0x134, 0x10
	.db	0x4
	.dw	word _tuner_WObs_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2272 
;2273   DSP_WR( PulseAmplSet,   F32,    pulseAmplSet,    0.0,   0.0,    EVERY, &c
	.dw	0x136, 0x10
	.db	0x4
	.dw	word _pulseAmplSet_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	word _cavAmplGain
	.dw	word _cavAmplOffset
;2274   DSP_WR( PulsePhaseSet,  PHASE,  pulsePhaseSet,   0.0,   0.0,    EVERY, &c
	.dw	0x138, 0x10
	.db	0x63
	.dw	word _pulsePhaseSet_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	word _cavPhaseGain
	.dw	word _cavPhaseOffset
;2275 
;2276   DSP_WR( CavLowIlkLevel, F32,    cavLowIlkLevel,  0.0,   0.0,    EVERY, &c
	.dw	0x13a, 0x10
	.db	0x4
	.dw	word _cavLowIlkLevel_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	word _cavAmplGain
	.dw	word _cavAmplOffset
;2277   DSP_WR( CavLowIlkTime,  F32,    cavLowIlkTime,   0.0,   0.0,    EVERY, NU
	.dw	0x13c, 0x10
	.db	0x4
	.dw	word _cavLowIlkTime_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2278 
;2279   DSP_WR( DvDtIlkRate,    F32,    dvdtIlkRate,     0.0,   0.0,    EVERY, &c
	.dw	0x13e, 0x10
	.db	0x4
	.dw	word _dvdtIlkRate_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	word _cavAmplGain
	.dw	word _cavAmplOffset
;2280 
;2281   DSP_WR( RevHighIlkRatio,F32,    revHighIlkRatio, 0.0,   0.0,    EVERY, &i
	.dw	0x140, 0x10
	.db	0x4
	.dw	word _revHighIlkRatio_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	word _ilkRatioGain
	.dw	word _ilkRatioOffset
;2282   DSP_WR( RevHighIlkTime, F32,    revHighIlkTime,  0.0,   0.0,    EVERY, NU
	.dw	0x142, 0x10
	.db	0x4
	.dw	word _revHighIlkTime_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2283 
;2284   DSP_WR( FwdLowIlkRatio, F32,    fwdLowIlkRatio,  0.0,   0.0,    EVERY, &i
	.dw	0x144, 0x10
	.db	0x4
	.dw	word _fwdLowIlkRatio_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	word _ilkRatioGain
	.dw	word _ilkRatioOffset
;2285   DSP_WR( FwdLowIlkTime,  F32,    fwdLowIlkTime,   0.0,   0.0,    EVERY, NU
	.dw	0x146, 0x10
	.db	0x4
	.dw	word _fwdLowIlkTime_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2286 
;2287   DSP_WR( FwdHighIlkLevel,F32,    fwdHighIlkLevel, 0.0,   0.0,    EVERY, NU
	.dw	0x148, 0x10
	.db	0x4
	.dw	word _fwdHighIlkLevel_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2288   DSP_WR( FwdHighIlkTime, F32,    fwdHighIlkTime,  0.0,   0.0,    EVERY, NU
	.dw	0x14a, 0x10
	.db	0x4
	.dw	word _fwdHighIlkTime_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2289 
;2290   DSP_WR( FwdCavPhase,    PHASE,  fwdCavPhase,     0.0,   0.0,    EVERY, NU
	.dw	0x14c, 0x10
	.db	0x63
	.dw	word _fwdCavPhase_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2291 
;2292   DSP_WR( FFPhase,        PHASE,  ffPhase,         0.0,   0.0,    EVERY, NU
	.dw	0x14e, 0x10
	.db	0x63
	.dw	word _ffPhase_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2293   DSP_WR( FFGain,         F32,    ffGain,          0.0,   0.0,    EVERY, NU
	.dw	0x150, 0x10
	.db	0x4
	.dw	word _ffGain_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2294 
;2295   DSP_WR( FreqOffset,     F32,    freqOffset,      0.0,   0.0,    EVERY, NU
	.dw	0x152, 0x10
	.db	0x4
	.dw	word _freqOffset_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2296 
;2297   DSP_WR( RFQ_ManTuneSet, F32,    rfq_manTuneSet,  0.0,   0.0,    EVERY, NU
	.dw	0x198, 0x10
	.db	0x4
	.dw	word _rfq_manTuneSet_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2298   DSP_WR( RFQ_T1_Offset,  F32,    rfq_t1_offset,   0.0,   0.0,    EVERY, NU
	.dw	0x19a, 0x10
	.db	0x4
	.dw	word _rfq_t1_offset_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2299   DSP_WR( RFQ_T2_Offset,  F32,    rfq_t2_offset,   0.0,   0.0,    EVERY, NU
	.dw	0x19c, 0x10
	.db	0x4
	.dw	word _rfq_t2_offset_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2300   DSP_WR( RFQ_T1_Feedback,F32,    rfq_t1_feedback, 0.0,   0.0,    EVERY, NU
	.dw	0x1a2, 0x10
	.db	0x4
	.dw	word _rfq_t1_feedback_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2301   DSP_WR( RFQ_T2_Feedback,F32,    rfq_t2_feedback, 0.0,   0.0,    EVERY, NU
	.dw	0x1a4, 0x10
	.db	0x4
	.dw	word _rfq_t2_feedback_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2302 
;2303 //--- WO registers in the "special" registers group ---
;2304   DSP_WR( SetDSPState,    U32,    setDSPState,     0,     0,      EVERY, NU
	.dw	0x200, 0x10
	.db	0x3
	.dw	word _setDSPState_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2305 
;2306 //--- R/W registers in the "special" registers group ---
;2307   DSP_WR( PiezoTunerPos,  U32,    piezoSetPos,     0,     0,      EVERY, NU
	.dw	0x202, 0x10
	.db	0x3
	.dw	word _piezoSetPos_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2308   DSP_WR( StepperPos,     U32,    stepperSetPos,   0,     0,      EVERY, NU
	.dw	0x204, 0x10
	.db	0x3
	.dw	word _stepperSetPos_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0x1
	.dw	0
	.dw	0
;2309 
;2310 //--- R/W registers in "special" group that write a value into a table ---
;2311 //--- and cannot be continuously written to ---
;2312   DSP_WR( FwdMagCalib,      F32,    fwdMagCalib,     0.0,   0.0,     ONCE, 
	.dw	0x22a, 0x10
	.db	0x4
	.dw	word _fwdMagCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2313   DSP_WR( FwdPhaseCalib,    PHASE,  fwdPhaseCalib,   0.0,   0.0,     ONCE, 
	.dw	0x22c, 0x10
	.db	0x63
	.dw	word _fwdPhaseCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2314 
;2315   DSP_WR( RevMagCalib,      F32,    revMagCalib,     0.0,   0.0,     ONCE, 
	.dw	0x22e, 0x10
	.db	0x4
	.dw	word _revMagCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2316   DSP_WR( RevPhaseCalib,    PHASE,  revPhaseCalib,   0.0,   0.0,     ONCE, 
	.dw	0x230, 0x10
	.db	0x63
	.dw	word _revPhaseCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2317 
;2318   DSP_WR( CavMagCalib,      F32,    cavMagCalib,     0.0,   0.0,     ONCE, 
	.dw	0x232, 0x10
	.db	0x4
	.dw	word _cavMagCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2319   DSP_WR( CavPhaseCalib,    PHASE,  cavPhaseCalib,   0.0,   0.0,     ONCE, 
	.dw	0x234, 0x10
	.db	0x63
	.dw	word _cavPhaseCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2320 
;2321   DSP_WR( Spare0MagCalib,   F32,   spare0MagCalib,   0.0,   0.0,     ONCE, 
	.dw	0x236, 0x10
	.db	0x4
	.dw	word _spare0MagCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2322   DSP_WR( Spare0PhaseCalib, PHASE, spare0PhaseCalib, 0.0,   0.0,     ONCE, 
	.dw	0x238, 0x10
	.db	0x63
	.dw	word _spare0PhaseCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2323 
;2324   DSP_WR( Spare1MagCalib,   F32,   spare1MagCalib,   0.0,   0.0,     ONCE, 
	.dw	0x23a, 0x10
	.db	0x4
	.dw	word _spare1MagCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2325   DSP_WR( Spare1PhaseCalib, PHASE, spare1PhaseCalib, 0.0,   0.0,     ONCE, 
	.dw	0x23c, 0x10
	.db	0x63
	.dw	word _spare1PhaseCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2326 
;2327   DSP_WR( Spare2MagCalib,   F32,   spare2MagCalib,   0.0,   0.0,     ONCE, 
	.dw	0x23e, 0x10
	.db	0x4
	.dw	word _spare2MagCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2328   DSP_WR( Spare2PhaseCalib, PHASE, spare2PhaseCalib, 0.0,   0.0,     ONCE, 
	.dw	0x240, 0x10
	.db	0x63
	.dw	word _spare2PhaseCalib_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2329 
;2330   DSP_WR( OutMagCalib0,     F32,   outMagCalib0,     0.0,   0.0,     ONCE, 
	.dw	0x242, 0x10
	.db	0x4
	.dw	word _outMagCalib0_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2331   DSP_WR( OutPhaseCalib0,   PHASE, outPhaseCalib0,   0.0,   0.0,     ONCE, 
	.dw	0x244, 0x10
	.db	0x63
	.dw	word _outPhaseCalib0_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2332 
;2333   DSP_WR( OutMagCalib1,     F32,   outMagCalib1,     0.0,   0.0,     ONCE, 
	.dw	0x246, 0x10
	.db	0x4
	.dw	word _outMagCalib1_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2334   DSP_WR( OutPhaseCalib1,   PHASE, outPhaseCalib1,   0.0,   0.0,     ONCE, 
	.dw	0x248, 0x10
	.db	0x63
	.dw	word _outPhaseCalib1_RegInfo
	.dw	0, 0
	.dw	0, 0
	.db	0
	.dw	0
	.dw	0
;2335 
	.debug	G _wrDSPregs 64 8
;2336 
;2337 #define  NUM_WR_DSP_REGS  (sizeof(wrDSPregs) / sizeof(WR_DSP_REG))
;2338 
;2339 const uint  Num_WR_DSP_Regs = NUM_WR_DSP_REGS;
	.seg	CONST
_Num_WR_DSP_Regs::
	.dw	0x42
	.debug	G _Num_WR_DSP_Regs 7 0
;2340 
;2341 
;2342 
;2343 U16 *rdFPGAregs[] = {
;2344   &statusBitsFPGA,
	.dseg
_rdFPGAregs::
	.dw	word _statusBitsFPGA
;2345   &statusBitsTuner,
	.dw	word _statusBitsTuner
;2346   &statusBitsRF,
	.dw	word _statusBitsRF
;2347   &statusBitsILK,
	.dw	word _statusBitsILK
;2348 
;2349   &spark0_ctr,
	.dw	word _spark0_ctr
;2350   &spark1_ctr,
	.dw	word _spark1_ctr
;2351   &cavlow_ctr,
	.dw	word _cavlow_ctr
;2352   &rev_high_ctr,
	.dw	word _rev_high_ctr
;2353   &dvdt_ctr,
	.dw	word _dvdt_ctr
;2354   &fwd_low_ctr,
	.dw	word _fwd_low_ctr
;2355   &fwd_high_ctr,
	.dw	word _fwd_high_ctr
;2356   &PLC_ctr,
	.dw	word _PLC_ctr
;2357   &safety_ctr
;2358 };
	.dw	word _safety_ctr
	.debug	G _rdFPGAregs 66 0
;2359 
;2360 #define  NUM_RD_FPGA_REGS  (sizeof(rdFPGAregs) / sizeof(U16 *))
;2361 
;2362 
;2363 U16 *wrFPGAregs[] = {
;2364   &ADC_err_rst,
_wrFPGAregs::
	.dw	word _ADC_err_rst
;2365   &RF_ilk_rst,
	.dw	word _RF_ilk_rst
;2366   &ilk_ctr_rst1,
	.dw	word _ilk_ctr_rst1
;2367   &ilk_ctr_rst2
;2368 };
	.dw	word _ilk_ctr_rst2
	.debug	G _wrFPGAregs 68 0
;2369 
;2370 #define  NUM_WR_FPGA_REGS  (sizeof(wrFPGAregs) / sizeof(U16 *))
;2371 
;2372 
;2373 
;2374 //=========================================================================
;2375 //  Definitions for event msgs.  Event msgs are the means that other proces
;2376 //  use to inform the application process of an event that it needs to hand
;2377 //
;2378 //  In general, allowing multiple processes to concurrently modify the stat
;2379 //  of the controller is a sure recipie for chaos.  By restricting all such
;2380 //  changes to the context of a single process, and sending messages to tha
;2381 //  process to request changes, we eliminate all such concurrency issues (n
;2382 //  the least of which is keeping track of the order of events).
;2383 //
;2384 //  The most common event is a request for change of state that corresponds
;2385 //  a message received from an external client via a protocol like ModBus.
;2386 //=========================================================================
;2387 
;2388 //EVENT_DEF( myEvent )
;2389 
;2390 
;2391 //=========================================================================
;2392 //  This is where misc global variables should be defined
;2393 //=========================================================================
;2394 #define SharedGlobals
;2395 #include <sharedAppl.c>
;2396 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
	.seg	CONST
_DigOutInitState::
	.dw	0, 0
	.debug	G _DigOutInitState 9 0
	.dseg
_applicReady::
	.db	0
	.debug	G _applicReady 3 0
_restoredApplState::
	.db	0
	.debug	G _restoredApplState 3 0
_allowSoftReset::
	.db	0
	.debug	G _allowSoftReset 3 0
_startupComplete::
	.db	0
	.debug	G _startupComplete 3 0
	.debug	I
;2397 //--- add definitions for application-specific global variables here ---
;2398 
;2399 LockData  locks[NUM_LOCKS];
;2400 
;2401 
;2402 bool  fpgaInitialized, pulseAmpl, pulsePhase;
;2403 
;2404 uint  ADCVal;
;2405 
;2406 bool  autoReadDSP, autoWriteDSP, autoReadFPGA, autoWriteFPGA, showFiller;
;2407 
;2408 bool  rfIsOn, rfPulsing, tunerCtlisON, rfAtWarm, rfRamping, readWaves,
;2409       cavLocked, amplCtlisON;
;2410 
;2411 uint  curProgNum, phaseMode, lastIOTime;
;2412 
;2413 const char *PhaseModeDesc[] = { "    OPEN", "PhaseCtl", "SEL Ctl ", "??????
	.seg	CONST
_PhaseModeDesc::
	.dw	@2+57
	.dw	@2+66
	.dw	@2+75
	.dw	@2+84
	.debug	G _PhaseModeDesc 69 0
;2414 
;2415 //=========================================================================
;2416 //  Temporary functions to fill in for not-yet implemented system functions
;2417 //=========================================================================
;2418 
;2419 
;2420 #define LockOwnedBy(id)  ( locks[id].owner )
;2421 #define NotLockOwner(id)  (LockOwnedBy(id) != getpid())
;2422 
;2423 //-------------------------------------------------------------------------
;2424 //  Get the lock for the specified resource.
;2425 //
;2426 //  Returns:
;2427 //    If lock acquired:  depth of locks aquired by caller
;2428 //    On error:  < 0
;2429 //-------------------------------------------------------------------------
;2430 int  getLock(uint id, ulong msTimeout)
;2431 {
	.defseg	FARCODE
	.seg	FARCODE
$getLock::
	.debug	G $getLock 71 9
	.debug	{ $getLock 71 2430 @5 @4
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-12
	.debug	A id 5 7
	.debug	A msTimeout 7 9
;2432   PID  mypid, owner;
	.debug	L mypid -2 6
	.debug	L owner -4 6
;2433   int  stat, count;
	.debug	L stat -6 6
	.debug	L count -8 6
;2434   task_t  *task;
	.debug	L task -10 72
;2435   LockData  *lock;
	.debug	L lock -12 74
;2436 
;2437 
;2438   if (id >= NUM_LOCKS)  { errno = ENODEV;  return -1; }
	.debug	# 2438
	ld	hl,(ix+5)
	ld	de,0xb
	or	a,a
	sbc	hl,de
	jr	c,@6
@7:
	.debug	< @7 @8
	ld	hl,(_current)
	ld	e,0xb9
	add	hl,de
	ld	e,0x13
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,0xffff
	jp	@4
@8:
	.debug	>
;2439 
;2440   mypid = getpid();   lock = locks + id;
@6:
	.debug	# 2440
	ld	hl,(_current)
	ld	de,0xb4
	add	hl,de
	ld	hl,(hl)
	ld	(ix-2),hl
	ld	hl,(ix+5)
	ld	bc,0x1a
	ex	de,hl
	mul
	ld	hl,_locks
	add	hl,bc
	ld	(ix-12),hl
;2441 
;2442   _EnterCritical_();  owner = lock->owner;  _ExitCritical_();
	.debug	# 2442
	ipset	2
	inc	hl
	inc	hl
	ld	hl,(hl)
	ld	(ix-4),hl
	ipres
;2443 
;2444 //--- If we already own the lock, then just increment the count.  Since ---
;2445 //--- only the owner of the lock changes the count, we can safely incr ---
;2446 //--- and return the current value w/o worrying about preemption. ---
;2447   if (owner == mypid)  { ++lock->count;  return lock->count; }
	.debug	# 2447
	ex	de,hl
	ld	hl,(ix-2)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	nz,@9
@10:
	.debug	< @10 @11
	ld	hl,(ix-12)
	push	hl
	ld	hl,(hl)
	inc	hl
	pop	de
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,(ix-12)
	ld	hl,(hl)
	jp	@4
@11:
	.debug	>
;2448 
;2449 //--- If we aren't the owner, then try to get the lock.  If the previous --
;2450 //--- owner died, then the LockData will be invalid but the semaphore for -
;2451 //--- the lock will have been released so another task will be able to get 
;2452 //--- it (and change the LockData values to indicate the new owner) ---
;2453   stat = sem_wait_(&lock->sem, msToTicks(msTimeout));
@9:
	.debug	# 2453
	ld	hl,(ix+7)
	ex	de,hl
	ld	hl,(ix+9)
	ld	bc,hl
	push	bc
	push	de
	lcall	$msToTicks
	add	sp,4
	push	bc
	push	de
	ld	hl,(ix-12)
	ld	de,0x4
	add	hl,de
	push	hl
	lcall	$sem_wait_
	add	sp,6
	ld	(ix-6),hl
;2454 
;2455   if (stat < 0)  {
	.debug	# 2455
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@12
;2456     if (msTimeout > 100)  {
@13:
	.debug	< @13 @14
	.debug	# 2456
	ld	hl,(ix+7)
	ex	de,hl
	ld	hl,(ix+9)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x64
	ld	c,d
	ld	b,d
	call	__lsub##
	jr	c,@15
	jr	z,@15
;2457       _EnterCritical_();
@16:
	.debug	< @16 @17
	.debug	# 2457
	ipset	2
;2458       owner = lock->owner;  count = lock->count;
	.debug	# 2458
	ld	hl,(ix-12)
	inc	hl
	inc	hl
	ld	hl,(hl)
	ld	(ix-4),hl
	ld	hl,(ix-12)
	ld	hl,(hl)
	ld	(ix-8),hl
;2459       _ExitCritical_();
	.debug	# 2459
	ipres
;2460       task = tasks + _activeTaskIdx_(owner);
	.debug	# 2460
	ld	hl,(ix-4)
	push	hl
	lcall	$_activeTaskIdx_
	pop	de
	ld	bc,0xe8
	ex	de,hl
	mul
	ld	hl,_tasks
	add	hl,bc
	ld	(ix-10),hl
;2461       printf("Lock %d still held by %.15s[%d], count %d\r\n",
;2462               id, task->argv[0], owner, count );
	.debug	# 2462
	ld	hl,(ix-8)
	push	hl
	ld	hl,(ix-4)
	push	hl
	ld	hl,(ix-10)
	ld	de,0x1a
	add	hl,de
	ld	hl,(hl)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	de,word (phys @1+2822)
	ld	c,phys @1+2822 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,12
;2463     }
@17:
	.debug	>
;2464     return stat;
@15:
	.debug	# 2464
	ld	hl,(ix-6)
	jr	@4
;2465   }
@14:
	.debug	>
;2466 
;2467 
;2468 //--- Again, if we got here, we own the lock and only the owner will change
;2469 //--- the LockData values ---
;2470   lock->owner = mypid;
@12:
	.debug	# 2470
	ld	hl,(ix-12)
	inc	hl
	inc	hl
	ex	de,hl
	ld	hl,(ix-2)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;2471   lock->count = 1;
	.debug	# 2471
	ld	hl,(ix-12)
	ld	de,0x1
	ld	(hl),e
	inc	hl
	ld	(hl),d
;2472 
;2473 //  fprintf(DiagLog, "%u now has lock %d\n", mypid, id);  //tdebug
;2474 
;2475 
;2476   return lock->count;
	.debug	# 2476
	ld	hl,(ix-12)
	ld	hl,(hl)
;2477 }
@4:
	.debug	} @4 2477

@3	equ	12
	ld	sp,ix
	pop	ix
	lret
@5	equ	14
;2478 
;2479 
;2480 
;2481 //-------------------------------------------------------------------------
;2482 //  If the calling process owns the specified lock, then reduce its lock co
;2483 //  (or release the lock if no more counts)
;2484 //
;2485 //  Returns:
;2486 //    On success:  New lock depth held by caller
;2487 //    On error:  < 0
;2488 //-------------------------------------------------------------------------
;2489 int releaseLock(uint id)
;2490 {
$releaseLock::
	.debug	G $releaseLock 76 9
	.debug	{ $releaseLock 76 2489 @20 @19
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-4
	.debug	A id 5 7
;2491   uint  owner;
	.debug	L owner -2 7
;2492   LockData  *lock;
	.debug	L lock -4 74
;2493 
;2494 
;2495   if (id >= NUM_LOCKS)  {
	.debug	# 2495
	ld	hl,(ix+5)
	ld	de,0xb
	or	a,a
	sbc	hl,de
	jr	c,@21
;2496     fprintf(DiagLog, "*** releaseLock(%u): Invalid lock ID ***\n", id);
@22:
	.debug	< @22 @23
	.debug	# 2496
	ld	hl,(ix+5)
	push	hl
	ld	de,word (phys @1+2866)
	ld	c,phys @1+2866 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,8
;2497     return -1;
	.debug	# 2497
	jr	@1134
;2498   }
@23:
	.debug	>
;2499 
;2500   lock = locks + id;
@21:
	.debug	# 2500
	ld	hl,(ix+5)
	ld	bc,0x1a
	ex	de,hl
	mul
	ld	hl,_locks
	add	hl,bc
	ld	(ix-4),hl
;2501 
;2502   _EnterCritical_();  owner = lock->owner;  _ExitCritical_();
	.debug	# 2502
	ipset	2
	inc	hl
	inc	hl
	ld	hl,(hl)
	ld	(ix-2),hl
	ipres
;2503 
;2504   if (owner != getpid())  {
	.debug	# 2504
	ld	hl,(_current)
	ld	de,0xb4
	add	hl,de
	ld	hl,(hl)
	ex	de,hl
	ld	hl,(ix-2)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	z,@24
;2505     fprintf(DiagLog, "*** releaseLock(%u): Lock owned by: %u ***\n",
@25:
	.debug	< @25 @26
;2506             id, owner);
	.debug	# 2506
	ld	h,d
	ld	l,e
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	de,word (phys @1+2908)
	ld	c,phys @1+2908 >> 16
	jr	@1133
;2507     return -1;
	.debug	# 2507
;2508   }
@26:
	.debug	>
;2509 
;2510   // At this point, we know we own the lock, so no other task will make cha
;2511   // to the values in the LockData structure until we release the semaphore
;2512 
;2513   if (lock->count < 1)  {
@24:
	.debug	# 2513
	ld	hl,(ix-4)
	ld	hl,(hl)
	ld	de,0x8001
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@27
;2514     fprintf(DiagLog, "*** releaseLock(%u) with count of %u ***\n",
@28:
	.debug	< @28 @29
;2515             id, lock->count);
	.debug	# 2515
	ld	hl,(ix-4)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	de,word (phys @1+2952)
	ld	c,phys @1+2952 >> 16
@1133:
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,10
;2516     return -1;
	.debug	# 2516
@1134:
	ld	hl,0xffff
	jr	@19
;2517   }
@29:
	.debug	>
;2518 
;2519   if (--lock->count == 0)  {
@27:
	.debug	# 2519
	ld	hl,(ix-4)
	push	hl
	ld	hl,(hl)
	dec	hl
	pop	de
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	a,e
	or	a,d
	jr	nz,@30
;2520 //    fprintf(DiagLog, "%u releasing lock %d\n",  getpid(), id);  //tdebug
;2521     lock->owner = -1;  sem_post(&lock->sem);
@31:
	.debug	< @31 @32
	.debug	# 2521
	ld	hl,(ix-4)
	inc	hl
	inc	hl
	ld	de,0xffff
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,(ix-4)
	ld	de,0x4
	add	hl,de
	push	hl
	lcall	$sem_post
	pop	de
;2522   }
@32:
	.debug	>
;2523 
;2524   return lock->count;
@30:
	.debug	# 2524
	ld	hl,(ix-4)
	ld	hl,(hl)
;2525 }
@19:
	.debug	} @19 2525

@18	equ	4
	ld	sp,ix
	pop	ix
	lret
@20	equ	6
;2526 
;2527 
;2528 
;2529 //=========================================================================
;2530 
;2531 
;2532 //-------------------------------------------------------------------------
;2533 //  Convert a new ModBus register value to a floating point number
;2534 //
;2535 //  Note that the size of the buffer pointed to by newVal must match the
;2536 //  size of the specified ModBus register value.
;2537 //-------------------------------------------------------------------------
;2538 float mb2float(void *newVal, RegInfo *mbRegInfo)
;2539 {
$mb2float::
	.debug	G $mb2float 77 9
	.debug	{ $mb2float 77 2538 @35 @34
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-1
	.debug	A newVal 5 78
	.debug	A mbRegInfo 7 38
;2540   uchar  type = mbRegInfo->dataType;
	.debug	L type -1 3
	.debug	# 2540
	ld	hl,(ix+7)
	ld	de,0xa
	add	hl,de
	ld	a,(hl)
	ld	(ix-1),a
	.seg	FARCODE
;2541 
;2542   switch (type)  {
	.debug	# 2542
	jr	@36
;2543     case DT_S16:  return (float) MB_VAL(S16);
@38:
	.debug	< @38 @39
	.debug	# 2543
@40:
	ld	hl,(ix+5)
	ld	hl,(hl)
	ex	de,hl
	call	__itof##
	jr	@34
;2544     case DT_U16:  return (float) MB_VAL(U16);
	.debug	# 2544
@41:
	ld	hl,(ix+5)
	ld	hl,(hl)
	ex	de,hl
	call	__uitof##
	jr	@34
;2545     case DT_S32:  return (float) MB_VAL(S32);
	.debug	# 2545
@42:
	ld	hl,(ix+5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__ltof##
	jr	@34
;2546     case DT_U32:  return (float) MB_VAL(U32);
	.debug	# 2546
@43:
	ld	hl,(ix+5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__ultof##
	jr	@34
;2547     case DT_F32:  return         MB_VAL(F32);
	.debug	# 2547
@44:
	ld	hl,(ix+5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	jr	@34
;2548   }
@39:
	.debug	>
;2549 
;2550   return 0.0;
@36:
	or	a,a
	jr	z,@40
	cp	0x1
	jr	z,@41
	cp	0x2
	jr	z,@42
	cp	0x3
	jr	z,@43
	cp	0x4
	jr	z,@44
@37:
	.debug	# 2550
	ld	de,0
	ld	c,e
	ld	b,e
;2551 }
@34:
	.debug	} @34 2551

@33	equ	1
	ld	sp,ix
	pop	ix
	lret
@35	equ	3
;2552 
;2553 //-------------------------------------------------------------------------
;2554 //  Convert a new ModBus register value to a signed long number
;2555 //
;2556 //  Note that the size of the buffer pointed to by newVal must match the
;2557 //  size of the specified ModBus register value.
;2558 //-------------------------------------------------------------------------
;2559 S32 mb2S32(void *newVal, RegInfo *mbRegInfo)
;2560 {
$mb2S32::
	.debug	G $mb2S32 79 9
	.debug	{ $mb2S32 79 2559 @47 @46
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-1
	.debug	A newVal 5 78
	.debug	A mbRegInfo 7 38
;2561   uchar  type = mbRegInfo->dataType;
	.debug	L type -1 3
	.debug	# 2561
	ld	hl,(ix+7)
	ld	de,0xa
	add	hl,de
	ld	a,(hl)
	ld	(ix-1),a
	.seg	FARCODE
;2562 
;2563   switch (type)  {
	.debug	# 2563
	jr	@48
;2564     case DT_S16:  return (S32) MB_VAL(S16);
@50:
	.debug	< @50 @51
	.debug	# 2564
@52:
	ld	hl,(ix+5)
	ld	hl,(hl)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	jr	@46
;2565     case DT_U16:  return (S32) MB_VAL(U16);
	.debug	# 2565
@53:
	ld	hl,(ix+5)
	ld	hl,(hl)
	ex	de,hl
	ld	bc,0
	jr	@46
;2566     case DT_S32:  return       MB_VAL(S32);
	.debug	# 2566
;2567     case DT_U32:  return (S32) MB_VAL(U32);
	.debug	# 2567
@55:
	ld	hl,(ix+5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	jr	@46
;2568     case DT_F32:  return (S32) MB_VAL(F32);
	.debug	# 2568
@56:
	ld	hl,(ix+5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__ftol##
	jr	@46
;2569   }
@51:
	.debug	>
;2570 
;2571   return 0;
@48:
	or	a,a
	jr	z,@52
	cp	0x1
	jr	z,@53
	cp	0x2
	jr	z,@55
	cp	0x3
	jr	z,@55
	cp	0x4
	jr	z,@56
@49:
	.debug	# 2571
	ld	de,0
	ld	c,e
	ld	b,e
;2572 }
@46:
	.debug	} @46 2572

@45	equ	1
	ld	sp,ix
	pop	ix
	lret
@47	equ	3
;2573 
;2574 //-------------------------------------------------------------------------
;2575 //  Convert a new ModBus register value to an unsigned long number
;2576 //
;2577 //  Note that the size of the buffer pointed to by newVal must match the
;2578 //  size for the type of the specified ModBus register value.
;2579 //-------------------------------------------------------------------------
;2580 U32 mb2U32(void *newVal, RegInfo *mbRegInfo)
;2581 {
$mb2U32::
	.debug	G $mb2U32 80 9
	.debug	{ $mb2U32 80 2580 @59 @58
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-1
	.debug	A newVal 5 78
	.debug	A mbRegInfo 7 38
;2582   uchar  type = mbRegInfo->dataType;
	.debug	L type -1 3
	.debug	# 2582
	ld	hl,(ix+7)
	ld	de,0xa
	add	hl,de
	ld	a,(hl)
	ld	(ix-1),a
	.seg	FARCODE
;2583 
;2584   switch (type)  {
	.debug	# 2584
	jr	@60
;2585     case DT_S16:  return (U32) MB_VAL(S16);
@62:
	.debug	< @62 @63
	.debug	# 2585
@64:
	ld	hl,(ix+5)
	ld	hl,(hl)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	jr	@58
;2586     case DT_U16:  return (U32) MB_VAL(U16);
	.debug	# 2586
@65:
	ld	hl,(ix+5)
	ld	hl,(hl)
	ex	de,hl
	ld	bc,0
	jr	@58
;2587     case DT_S32:  return (U32) MB_VAL(S32);
	.debug	# 2587
;2588     case DT_U32:  return       MB_VAL(U32);
	.debug	# 2588
@67:
	ld	hl,(ix+5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	jr	@58
;2589     case DT_F32:  return (U32) MB_VAL(F32);
	.debug	# 2589
@68:
	ld	hl,(ix+5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__ftol##
	jr	@58
;2590   }
@63:
	.debug	>
;2591 
;2592   return 0;
@60:
	or	a,a
	jr	z,@64
	cp	0x1
	jr	z,@65
	cp	0x2
	jr	z,@67
	cp	0x3
	jr	z,@67
	cp	0x4
	jr	z,@68
@61:
	.debug	# 2592
	ld	de,0
	ld	c,e
	ld	b,e
;2593 }
@58:
	.debug	} @58 2593

@57	equ	1
	ld	sp,ix
	pop	ix
	lret
@59	equ	3
;2594 
;2595 //-------------------------------------------------------------------------
;2596 //  Convert a floating value to the format of the specified ModBus register
;2597 //
;2598 //  Note that the size of the buffer pointed to by newVal must match the
;2599 //  size of the specified ModBus register value.
;2600 //-------------------------------------------------------------------------
;2601 void float2mb(void *newVal, RegInfo *mbRegInfo, float fromVal)
;2602 {
$float2mb::
	.debug	G $float2mb 81 9
	.debug	{ $float2mb 81 2601 @71 @70
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-1
	.debug	A newVal 5 78
	.debug	A mbRegInfo 7 38
	.debug	A fromVal 9 10
;2603   uchar  type = mbRegInfo->dataType;
	.debug	L type -1 3
	.debug	# 2603
	ld	hl,(ix+7)
	ld	de,0xa
	add	hl,de
	ld	a,(hl)
	ld	(ix-1),a
	.seg	FARCODE
;2604 
;2605   switch (type)  {
	.debug	# 2605
	jr	@72
;2606     case DT_S16:  MB_VAL(S16) = (int) fromVal;  break;
@74:
	.debug	< @74 @75
	.debug	# 2606
;2607     case DT_U16:  MB_VAL(U16) = (uint) fromVal;  break;
	.debug	# 2607
@77:
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	call	__ftol##
	ld	hl,(ix+5)
	ld	(hl),e
	inc	hl
	ld	(hl),d
;2608     case DT_S32:  MB_VAL(S32) = (long) fromVal;  break;
	jr	@73
	.debug	# 2608
@1132:
;2609     case DT_U32:  MB_VAL(U32) = (ulong) fromVal;  break;
	.debug	# 2609
@79:
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	call	__ftol##
	jr	@1131
;2610     case DT_F32:  MB_VAL(float) = fromVal;  break;
	.debug	# 2610
@80:
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
@1131:
	ld	hl,(ix+5)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;2611   }
	jr	@73
@75:
	.debug	>
;2612 }
@72:
	or	a,a
	jr	z,@77
	cp	0x1
	jr	z,@77
	cp	0x2
	jr	z,@1132
	cp	0x3
	jr	z,@79
	cp	0x4
	jr	z,@80
@73:
@70:
	.debug	} @70 2612

@69	equ	1
	ld	sp,ix
	pop	ix
	lret
@71	equ	3
;2613 
;2614 //-------------------------------------------------------------------------
;2615 //  Convert a DSP register value to a format that is compatible with the
;2616 //  specified ModBus register (i.e. make sure it is a floating point or
;2617 //  an integer value as needed).
;2618 //-------------------------------------------------------------------------
;2619 void dsp2mb(RegInfo *mbReg, S16 *mbVal,
;2620             const RD_DSP_REG far *dspReg, ulong *dspVal)
;2621 {
$dsp2mb::
	.debug	G $dsp2mb 82 9
	.debug	{ $dsp2mb 82 2619 @83 @82
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-17
	.debug	A mbReg 5 38
	.debug	A mbVal 7 83
	.debug	A dspReg 9 84
	.debug	A dspVal 13 85
;2622   uchar  mbType = mbReg->dataType, dspType = dspReg->type;
	.debug	L mbType -1 3
	.debug	# 2622
	ld	hl,(ix+5)
	ld	de,0xa
	add	hl,de
	ld	a,(hl)
	ld	(ix-1),a
	.seg	FARCODE
	.debug	L dspType -2 3
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,de
	ld	a,c
	ldp	hl,(hl)
	ld	a,l
	ld	(ix-2),a
	.seg	FARCODE
;2623   bool  offset0, gain1, scaleIt;
	.debug	L offset0 -3 3
	.debug	L gain1 -4 3
	.debug	L scaleIt -5 3
;2624   float  gain, offset, fval;
	.debug	L gain -9 10
	.debug	L offset -13 10
	.debug	L fval -17 10
;2625 
;2626 
;2627 //--- if DSP value is a scaled PHASE value, rescale it into a float ---
;2628   if (dspType == RT_PHASE)  {
	.debug	# 2628
	sub	a,99
	jr	nz,@84
;2629     *(float *)dspVal = (float)*dspVal * PhaseConvFactor;  dspType = RT_F32;
@85:
	.debug	< @85 @86
	.debug	# 2629
	ld	hl,(ix+13)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__ultof##
	push	bc
	push	de
	ld	de,0
	ld	bc,0x33b4
	call	__fmlt##
	ld	hl,(ix+13)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	(ix-2),0x4
@86:
	.debug	>
;2630 
;2631 
;2632 //--- determine if the value needs to be scaled ---
;2633   offset = (dspReg->offset ? *dspReg->offset : 0.0);  // NULL == no offset
@84:
	.debug	# 2633
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x9
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	a,e
	or	a,d
	jr	z,@87
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x9
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	jr	@88
@87:
	ld	de,0
	ld	c,e
	ld	b,e
@88:
	ld	hl,bc
	ld	(ix-11),hl
	ex	de,hl
	ld	(ix-13),hl
	ex	de,hl
;2634   gain = (dspReg->gain ? *dspReg->gain : 1.0);  // NULL == no gain
	.debug	# 2634
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x7
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	a,e
	or	a,d
	jr	z,@89
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x7
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	jr	@90
@89:
	ld	de,0
	ld	bc,0x3f80
@90:
	ld	hl,bc
	ld	(ix-7),hl
	ex	de,hl
	ld	(ix-9),hl
	ex	de,hl
;2635 
;2636   if (gain == 0.0)  gain = 1.0;
	.debug	# 2636
	ld	a,b
	and	a,7Fh
	or	a,c
	or	a,d
	or	a,e
	jr	nz,@91
	ld	de,0
	ld	hl,0x3f80
	ld	(ix-7),hl
	ex	de,hl
	ld	(ix-9),hl
	ex	de,hl
;2637 
;2638   offset0 = (offset == 0.0);  gain1 = (gain == 1.0);
@91:
	.debug	# 2638
	ld	hl,(ix-13)
	ex	de,hl
	ld	hl,(ix-11)
	ld	bc,hl
	ld	a,b
	and	a,7Fh
	or	a,c
	or	a,d
	or	a,e
	ld	de,0
	jr	nz,@92
	inc	e
@92:
	ld	(ix-3),e
	ld	hl,(ix-9)
	ex	de,hl
	ld	hl,(ix-7)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0
	ld	bc,0x3f80
	call	__fequ##
	ld	de,0
	jr	nz,@93
	inc	e
@93:
	ld	(ix-4),e
;2639 
;2640   scaleIt = (!offset0 OR !gain1);
	.debug	# 2640
	ld	a,(ix-3)
	or	a,a
	jr	z,@94
	ld	a,e
	or	a,a
	jr	nz,@95
@94:
	ld	a,1
	jr	@96
@95:
	xor	a
@96:
	ld	(ix-5),a
;2641 
;2642   if (scaleIt)  {
	.debug	# 2642
	or	a,a
	jp	z,@97
;2643     switch (dspType)  {
@98:
	.debug	< @98 @99
	.debug	# 2643
	ld	a,(ix-2)
	jr	@100
;2644       case RT_S32:  fval = (float)((S32)*dspVal);  break;
@102:
	.debug	< @102 @103
	.debug	# 2644
@104:
	ld	hl,(ix+13)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__ltof##
	jr	@1130
;2645       case RT_U32:  fval = (float)((U32)*dspVal);  break;
	.debug	# 2645
@105:
	ld	hl,(ix+13)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__ultof##
	jr	@1130
;2646       case RT_F32:  fval = *(float *)dspVal;  break;
	.debug	# 2646
@106:
	ld	hl,(ix+13)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
@1130:
	ld	hl,bc
	ld	(ix-15),hl
	ex	de,hl
	ld	(ix-17),hl
	ex	de,hl
;2647     }
	jr	@101
@103:
	.debug	>
;2648     if (!offset0)  fval -= offset;
@100:
	cp	0x2
	jr	z,@104
	cp	0x3
	jr	z,@105
	cp	0x4
	jr	z,@106
@101:
	.debug	# 2648
	ld	a,(ix-3)
	or	a,a
	jr	nz,@107
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-15)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-13)
	ex	de,hl
	ld	hl,(ix-11)
	ld	bc,hl
	call	__fsub##
	ld	hl,bc
	ld	(ix-15),hl
	ex	de,hl
	ld	(ix-17),hl
	ex	de,hl
;2649     if (!gain1)  fval = fval * gain;
@107:
	.debug	# 2649
	ld	a,(ix-4)
	or	a,a
	jr	nz,@108
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-15)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-9)
	ex	de,hl
	ld	hl,(ix-7)
	ld	bc,hl
	call	__fmlt##
	ld	hl,bc
	ld	(ix-15),hl
	ex	de,hl
	ld	(ix-17),hl
	ex	de,hl
;2650     *(float *)dspVal = fval;  dspType = RT_F32;
@108:
	.debug	# 2650
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-15)
	ld	bc,hl
	ld	hl,(ix+13)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	(ix-2),0x4
;2651   }
@99:
	.debug	>
;2652 
;2653 
;2654 //--- Make sure the DSP format is the same type as the ModBus format ---
;2655 //--- (i.e. both are 16 or 32-bit integers or both are 32-bit floats) ---
;2656   if ((dspType == DT_F32) != (mbType == DT_F32))  {
@97:
	.debug	# 2656
	ld	a,(ix-2)
	sub	a,4
	ld	a,0
	jr	nz,@109
	inc	a
@109:
	ld	hl,(ix-1)
	ld	e,0x4
	ld	h,a
	ld	a,l
	ld	l,h
	sub	a,e
	ld	a,0
	jr	nz,@110
	inc	a
@110:
	sub	a,l
	jp	z,@111
;2657     if (mbType == DT_F32)  {  // non-float to float
@112:
	.debug	< @112 @113
	.debug	# 2657
	ld	a,(ix-1)
	sub	a,4
	jr	nz,@114
;2658       if (dspType == DT_S32)  {  // signed or unsigned?
@115:
	.debug	< @115 @116
	.debug	# 2658
	ld	a,(ix-2)
	sub	a,2
	jr	nz,@117
;2659         *(float *)dspVal = (float)((S32)*dspVal);  dspType = DT_F32;  }
@118:
	.debug	< @118 @119
	.debug	# 2659
	ld	hl,(ix+13)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__ltof##
	jr	@1129
@119:
	.debug	>
;2660       else  {
@117:
;2661         *(float *)dspVal = (float)((U32)*dspVal);  dspType = DT_F32;  }
@121:
	.debug	< @121 @122
	.debug	# 2661
	ld	hl,(ix+13)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__ultof##
@1129:
	ld	hl,(ix+13)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	a,0x4
	ld	(ix-2),a
@122:
	.debug	>
;2662     }
@116:
	.debug	>
;2663     else  {  // float to non-float
	jr	@123
@114:
;2664       fval = *(float *)dspVal;
@124:
	.debug	< @124 @125
	.debug	# 2664
	ld	hl,(ix+13)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-15),hl
	ex	de,hl
	ld	(ix-17),hl
	ex	de,hl
;2665       if ((mbType == DT_S16) OR (mbType == DT_S32)) { // signed or unsigned
	.debug	# 2665
	ld	a,(ix-1)
	or	a,a
	jr	z,@126
	sub	a,2
	jr	nz,@127
@126:
;2666         *(S32 *)dspVal = (S32) fval;  dspType = DT_S32; }
@128:
	.debug	< @128 @129
	.debug	# 2666
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-15)
	ld	bc,hl
	call	__ftol##
	ld	hl,(ix+13)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	a,0x2
	ld	(ix-2),a
@129:
	.debug	>
;2667       else  {
	jr	@130
@127:
;2668         *(U32 *)dspVal = (U32) fval;  dspType = DT_U32; }
@131:
	.debug	< @131 @132
	.debug	# 2668
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-15)
	ld	bc,hl
	call	__ftol##
	ld	hl,(ix+13)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	(ix-2),0x3
@132:
	.debug	>
;2669     }
@130:
@125:
	.debug	>
;2670   }
@123:
@113:
	.debug	>
;2671 
;2672   if (mbReg->numRegs < 2)  // ModBus is 16-bit int, DSP is 32-bit int
@111:
	.debug	# 2672
	ld	hl,(ix+5)
	ld	de,0xb
	add	hl,de
	ld	a,(hl)
	sub	a,2
	jr	nc,@133
;2673     *(uint *)mbVal = (uint)(*dspVal);
	.debug	# 2673
	ld	hl,(ix+13)
	ld	hl,(hl)
	ex	de,hl
	ld	hl,(ix+7)
	ld	(hl),e
	inc	hl
	ld	(hl),d
;2674   else  // both are 32-bits (either float or long ints)
;2675     memcpy(mbVal, dspVal, 4);
	jr	@134
@133:
	.debug	# 2675
	ld	hl,0x4
	push	hl
	ld	hl,(ix+13)
	push	hl
	ld	hl,(ix+7)
	push	hl
	call	_memcpy
	add	sp,6
;2676 }
@134:
@82:
	.debug	} @82 2676

@81	equ	17
	ld	sp,ix
	pop	ix
	lret
@83	equ	19
;2677 
;2678 
;2679 //-------------------------------------------------------------------------
;2680 //  Convert a ModBus register value to the specified DSP register format
;2681 //-------------------------------------------------------------------------
;2682 void mb2dsp(RegInfo *mbReg, S16 *mbVal,
;2683             const WR_DSP_REG far *dspReg, ulong *dspVal)
;2684 {
$mb2dsp::
	.debug	G $mb2dsp 86 9
	.debug	{ $mb2dsp 86 2682 @137 @136
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-16
	.debug	A mbReg 5 38
	.debug	A mbVal 7 83
	.debug	A dspReg 9 87
	.debug	A dspVal 13 85
;2685   uchar  dspType = dspReg->type;
	.debug	L dspType -1 3
	.debug	# 2685
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,de
	ld	a,c
	ldp	hl,(hl)
	ld	a,l
	ld	(ix-1),a
	.seg	FARCODE
;2686   bool  offset0, gain1, scaleIt;
	.debug	L offset0 -2 3
	.debug	L gain1 -3 3
	.debug	L scaleIt -4 3
;2687   float  gain, offset, fval;
	.debug	L gain -8 10
	.debug	L offset -12 10
	.debug	L fval -16 10
;2688 
;2689 
;2690 //--- determine if the value needs to be scaled ---
;2691   offset = (dspReg->offset ? *dspReg->offset : 0.0);  // NULL == no offset
	.debug	# 2691
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x12
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	a,e
	or	a,d
	jr	z,@138
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x12
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	jr	@139
@138:
	ld	de,0
	ld	c,e
	ld	b,e
@139:
	ld	hl,bc
	ld	(ix-10),hl
	ex	de,hl
	ld	(ix-12),hl
	ex	de,hl
;2692   gain = (dspReg->gain ? *dspReg->gain : 1.0);  // NULL == no gain
	.debug	# 2692
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x10
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	a,e
	or	a,d
	jr	z,@140
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x10
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	jr	@141
@140:
	ld	de,0
	ld	bc,0x3f80
@141:
	ld	hl,bc
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
;2693 
;2694   if (gain == 0.0)  gain = 1.0;
	.debug	# 2694
	ld	a,b
	and	a,7Fh
	or	a,c
	or	a,d
	or	a,e
	jr	nz,@142
	ld	de,0
	ld	hl,0x3f80
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
;2695 
;2696   offset0 = (offset == 0.0);  gain1 = (gain == 1.0);
@142:
	.debug	# 2696
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	ld	a,b
	and	a,7Fh
	or	a,c
	or	a,d
	or	a,e
	ld	de,0
	jr	nz,@143
	inc	e
@143:
	ld	(ix-2),e
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0
	ld	bc,0x3f80
	call	__fequ##
	ld	de,0
	jr	nz,@144
	inc	e
@144:
	ld	(ix-3),e
;2697 
;2698   scaleIt = (!offset0 OR !gain1);
	.debug	# 2698
	ld	a,(ix-2)
	or	a,a
	jr	z,@145
	ld	a,e
	or	a,a
	jr	nz,@146
@145:
	ld	a,1
	jr	@147
@146:
	xor	a
@147:
	ld	(ix-4),a
;2699 
;2700   if (scaleIt)  {
	.debug	# 2700
	or	a,a
	jr	z,@148
;2701     fval = mb2float(mbVal, mbReg);
@149:
	.debug	< @149 @150
	.debug	# 2701
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix+7)
	push	hl
	lcall	$mb2float
	add	sp,4
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
;2702     if (!gain1)  fval = fval / gain;
	.debug	# 2702
	ld	a,(ix-3)
	or	a,a
	jr	nz,@151
	push	bc
	push	de
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	call	__fdiv##
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
;2703     if (!offset0)  fval += offset;
@151:
	.debug	# 2703
	ld	a,(ix-2)
	or	a,a
	jr	nz,@152
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	call	__fadd##
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
;2704   }
@152:
@150:
	.debug	>
;2705 
;2706 //--- convert either the scaled or unscaled ModBus value to the DSP format 
;2707   switch (dspType)  {
@148:
	.debug	# 2707
	ld	a,(ix-1)
	jp	@153
;2708     case RT_S32:
@155:
	.debug	< @155 @156
;2709       if (scaleIt)
	.debug	# 2709
@157:
	ld	a,(ix-4)
	or	a,a
	jr	z,@158
;2710         *(S32 *)dspVal = (S32)fval;
	.debug	# 2710
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__ftol##
	ld	hl,(ix+13)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;2711       else
;2712         *(S32 *)dspVal = mb2S32(mbVal, mbReg);
	jr	@159
@158:
	.debug	# 2712
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix+7)
	push	hl
	lcall	$mb2S32
	add	sp,4
	ld	hl,(ix+13)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;2713       break;
@159:
	.debug	# 2713
;2714 
;2715     case RT_U32:
	jp	@154
;2716       if (scaleIt)
	.debug	# 2716
@160:
	ld	a,(ix-4)
	or	a,a
	jr	z,@161
;2717         *(U32 *)dspVal = (U32)fval;
	.debug	# 2717
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__ftol##
	ld	hl,(ix+13)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;2718       else
;2719         *(U32 *)dspVal = mb2U32(mbVal, mbReg);
	jr	@154
@161:
	.debug	# 2719
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix+7)
	push	hl
	lcall	$mb2U32
	add	sp,4
	ld	hl,(ix+13)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;2720       break;
	.debug	# 2720
;2721 
;2722     case RT_F32:
	jr	@154
;2723       if (scaleIt)
	.debug	# 2723
@163:
	ld	a,(ix-4)
	or	a,a
	jr	z,@164
;2724         *(float *)dspVal = fval;
	.debug	# 2724
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	jr	@1128
;2725       else
;2726         *(float *)dspVal = mb2float(mbVal, mbReg);
@164:
	.debug	# 2726
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix+7)
	push	hl
	lcall	$mb2float
	add	sp,4
	jr	@1128
;2727       break;
	.debug	# 2727
;2728 
;2729     case RT_PHASE:
;2730       if (scaleIt)
	.debug	# 2730
@166:
	ld	a,(ix-4)
	or	a,a
	jr	z,@167
;2731         *dspVal = (ulong)(fval / PhaseConvFactor);
	.debug	# 2731
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	jr	@1127
;2732       else
;2733         *dspVal = (ulong)(mb2float(mbVal, mbReg) / PhaseConvFactor);
@167:
	.debug	# 2733
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix+7)
	push	hl
	lcall	$mb2float
	add	sp,4
@1127:
	push	bc
	push	de
	ld	de,0
	ld	bc,0x33b4
	call	__fdiv##
	call	__ftol##
@1128:
	ld	hl,(ix+13)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;2734       break;
	.debug	# 2734
;2735   }
	jr	@154
@156:
	.debug	>
;2736 
;2737 }
@153:
	cp	0x2
	jp	z,@157
	cp	0x3
	jp	z,@160
	cp	0x4
	jr	z,@163
	cp	0x63
	jr	z,@166
@154:
@136:
	.debug	} @136 2737

@135	equ	16
	ld	sp,ix
	pop	ix
	lret
@137	equ	18
;2738 
;2739 
;2740 //=========================================================================
;2741 //  Functions referenced in the Modbus register definitions to handle write
;2742 //  to specific global values defined as ModBus registers
;2743 //=========================================================================
;2744 
;2745 //-------------------------------------------------------------------------
;2746 //  Got a new value for the register that enables/disables RF output.  This
;2747 //  register is different from the other writable FPGA registers in that th
;2748 //  value does not represents bits that need to be toggled to reset some ot
;2749 //  value.
;2750 //-------------------------------------------------------------------------
;2751 bool MB_RF_soft_on(SetRegFuncParams)
;2752 {
	.align	4
$$MB_RF_soft_on::
	ex	(sp),iy
$MB_RF_soft_on::
	.debug	G $MB_RF_soft_on 88 9
	.debug	{ $MB_RF_soft_on 88 2751 @171 @170
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A info 5 78
	.debug	A chanNum 7 3
	.debug	A newVal 9 78
	.debug	A eventsfd 11 6
;2753   writeFPGA(RF_SOFT_ON, MB_VAL(U16));
	.debug	# 2753
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	hl,0xe
	push	hl
	lcall	$writeFPGA
	add	sp,4
;2754 
;2755   return setU16(_SetRegFuncParams_);
	.debug	# 2755
	ld	hl,(ix+11)
	push	hl
	ld	hl,(ix+9)
	push	hl
	bool	hl
	ld	l,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$setU16
	add	sp,8
;2756 }
@170:
	.debug	} @170 2756

@169	equ	0
	pop	ix
	lret
@171	equ	2
;2757 
;2758 
;2759 //-------------------------------------------------------------------------
;2760 //  Got a new value for one of the registers that mirrors one of the DSP's 
;2761 //  registers.  In response, we check the new value against the range of va
;2762 //  given for the DSP register and modify the ModBus register value if need
;2763 //
;2764 //  The writeMode specified for the DSP register determines when the new va
;2765 //  is written to the DSP as follows:
;2766 //
;2767 //  WM_ONCE:
;2768 //     The value must be written just once ASAP.  Writing to one of these D
;2769 //     registers triggers some action on the part of the DSP, so the regist
;2770 //     must not be written to except in response to a write to the correspo
;2771 //     ModBus register.  The most common use of this mode by the DSP is to
;2772 //     copy some related values to a table (possibly including the value ju
;2773 //     written, or using the new value as an index in to the table?) as par
;2774 //     of a calibration process.
;2775 //
;2776 //  WM_EVERY:
;2777 //     The value represents some run-time setting and can be written to the
;2778 //     repeatedly without causing a problem.  While writing such values to 
;2779 //     DSP just once would be sufficient if the the DSP never rebooted, suc
;2780 //     values are not retained by the DSP across a reboot.  For that reason
;2781 //     these values are resent to the DSP each pass through the application
;2782 //     main loop (i.e. by the applic task).  The result is that if the DSP
;2783 //     restarts, or if a write should fail one time, the correct settings w
;2784 //     still be quickly (re)asserted.
;2785 //
;2786 //-------------------------------------------------------------------------
;2787 bool MB_DSP_REG(SetRegFuncParams)
;2788 {
	.align	4
$$MB_DSP_REG::
	ex	(sp),iy
$MB_DSP_REG::
	.debug	G $MB_DSP_REG 88 9
	.debug	{ $MB_DSP_REG 88 2787 @174 @173
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-48
	.debug	A info 5 78
	.debug	A chanNum 7 3
	.debug	A newVal 9 78
	.debug	A eventsfd 11 6
;2789   uint  ui;
	.debug	L ui -2 7
;2790   RegInfo *mbRegInfo = (RegInfo *)info;
	.debug	L mbRegInfo -4 38
	.debug	# 2790
	ld	hl,(ix+5)
	ld	(ix-4),hl
	.seg	FARCODE
;2791   ulong  dspVal;
	.debug	L dspVal -8 9
;2792   float  fval;
	.debug	L fval -12 10
;2793   const WR_DSP_REG far *dspReg = wrDSPregs;
	.debug	L dspReg -16 87
	.debug	# 2793
	ld	de,word (phys _wrDSPregs)
	ld	c,phys _wrDSPregs >> 16
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
	.seg	FARCODE
;2794   char  temp[32];
	.debug	L temp -48 89
;2795 
;2796   for (ui=0; ui<NUM_WR_DSP_REGS; ++ui,++dspReg)  {
@175:
	.debug	< @175 @176
	.debug	# 2796
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@177:
	ld	hl,(ix-2)
	ld	de,0x42
	or	a,a
	sbc	hl,de
	jp	nc,@178
;2797     if (dspReg->mbReg != mbRegInfo)  continue;
@180:
	.debug	< @180 @181
	.debug	# 2797
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x5
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	hl,(ix-4)
	or	a,a
	sbc	hl,de
	jp	nz,@179
;2798 
;2799   //--- if a phase value, then make sure it is within the range 0 to 360 --
;2800     if (dspReg->type == RT_PHASE)  {
	.debug	# 2800
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,de
	ld	a,c
	ldp	hl,(hl)
	ld	a,l
	sub	a,99
	jp	nz,@183
;2801       fval = mb2float(newVal, mbRegInfo);
@184:
	.debug	< @184 @185
	.debug	# 2801
	ld	hl,(ix-4)
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$mb2float
	add	sp,4
	ld	hl,bc
	ld	(ix-10),hl
	ex	de,hl
	ld	(ix-12),hl
	ex	de,hl
;2802       if ((fval < 0.0) OR (fval >= 360.0))  {
	.debug	# 2802
	push	bc
	push	de
	ld	de,0
	ld	c,e
	ld	b,e
	call	__fsub##
	jp	m,@186
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0
	ld	bc,0x43b4
	call	__fsub##
	jp	m,@187
@186:
;2803         while (fval < 0.0)  fval += 360.0;
@188:
	.debug	< @188 @189
@190:
	.debug	# 2803
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0
	ld	c,e
	ld	b,e
	call	__fsub##
	jp	p,@191
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0
	ld	bc,0x43b4
	call	__fadd##
	ld	hl,bc
	ld	(ix-10),hl
	ex	de,hl
	ld	(ix-12),hl
	ex	de,hl
;2804         while (fval >= 360.0)  fval -= 360.0;
	jr	@190
@191:
@192:
	.debug	# 2804
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0
	ld	bc,0x43b4
	call	__fsub##
	jp	m,@193
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0
	ld	bc,0xc3b4
	call	__fadd##
	ld	hl,bc
	ld	(ix-10),hl
	ex	de,hl
	ld	(ix-12),hl
	ex	de,hl
;2805         float2mb(newVal, mbRegInfo, fval);
	jr	@192
@193:
	.debug	# 2805
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-4)
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$float2mb
	add	sp,8
;2806       }
@189:
	.debug	>
;2807     }
@187:
@185:
	.debug	>
;2808 
;2809   //--- check the new value against the specified range of allowed values -
;2810     if (dspReg->minVal != dspReg->maxVal)  {
@183:
	.debug	# 2810
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x7
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xb
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	call	__fequ##
	jp	z,@194
;2811       fval = mb2float(newVal, mbRegInfo);
@195:
	.debug	< @195 @196
	.debug	# 2811
	ld	hl,(ix-4)
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$mb2float
	add	sp,4
	ld	hl,bc
	ld	(ix-10),hl
	ex	de,hl
	ld	(ix-12),hl
	ex	de,hl
;2812       if (fval < dspReg->minVal)  float2mb(newVal, mbRegInfo, dspReg->minVa
	.debug	# 2812
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x7
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	call	__fsubr##
	jp	p,@197
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x7
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix-4)
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$float2mb
	add	sp,8
;2813       if (fval > dspReg->maxVal)  float2mb(newVal, mbRegInfo, dspReg->maxVa
@197:
	.debug	# 2813
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xb
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	call	__fsubr##
	jp	m,@198
	jr	z,@198
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xb
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix-4)
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$float2mb
	add	sp,8
;2814     }
@198:
@196:
	.debug	>
;2815 
;2816     if (mbRegInfo->flags & LogChanges)
@194:
	.debug	# 2816
	ld	hl,(ix-4)
	ld	de,0xd
	add	hl,de
	ld	a,(hl)
	and	a,2
	jr	z,@199
;2817       printf("new value for DSP reg: 0x%08lX: %.32s\r\n",
;2818              dspReg->id, fmtRegVal(mbRegInfo, newVal, 0, temp) );
	.debug	# 2818
	ld	hl,-48+@172
	add	hl,sp
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix-4)
	push	hl
	lcall	$fmtRegVal
	add	sp,8
	push	hl
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__farld4##
	push	bc
	push	de
	ld	de,word (phys @1+2994)
	ld	c,phys @1+2994 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,10
;2819 
;2820   //--- write the value to the DSP now? ---
;2821     if (dspReg->writeMode == WM_ONCE)  {
@199:
	.debug	# 2821
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xf
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,de
	ld	a,c
	ldp	hl,(hl)
	ld	a,l
	or	a,a
	jr	nz,@200
;2822       mb2dsp(mbRegInfo, newVal, dspReg, &dspVal);
@201:
	.debug	< @201 @202
	.debug	# 2822
	ld	hl,-8+@172
	add	hl,sp
	push	hl
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix-4)
	push	hl
	lcall	$mb2dsp
	add	sp,10
;2823       if (writeDSPReg(dspReg->id, dspVal, true) < 0)  return false;
	.debug	# 2823
	ld	hl,0x1
	push	hl
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__farld4##
	push	bc
	push	de
	lcall	$writeDSPReg
	add	sp,10
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@203
	xor	a,a
	jp	@173
;2824     }
@203:
@202:
	.debug	>
;2825 
;2826   //--- update the modbus register value using the new value ---
;2827   // NOTE:  Can't use setMBReg() since that would just call this func again
;2828     switch (mbRegInfo->dataType)  {
@200:
	.debug	# 2828
	ld	hl,(ix-4)
	ld	de,0xa
	add	hl,de
	ld	a,(hl)
	jr	@204
;2829       case DT_S16:  setS16(_SetRegFuncParams_);  break;
@206:
	.debug	< @206 @207
	.debug	# 2829
@208:
	ld	hl,(ix+11)
	push	hl
	ld	hl,(ix+9)
	push	hl
	bool	hl
	ld	l,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$setS16
	jr	@1126
;2830       case DT_U16:  setU16(_SetRegFuncParams_);  break;
	.debug	# 2830
@209:
	ld	hl,(ix+11)
	push	hl
	ld	hl,(ix+9)
	push	hl
	bool	hl
	ld	l,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$setU16
	jr	@1126
;2831       case DT_S32:  setS32(_SetRegFuncParams_);  break;
	.debug	# 2831
@210:
	ld	hl,(ix+11)
	push	hl
	ld	hl,(ix+9)
	push	hl
	bool	hl
	ld	l,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$setS32
	jr	@1126
;2832       case DT_U32:  setU32(_SetRegFuncParams_);  break;
	.debug	# 2832
@211:
	ld	hl,(ix+11)
	push	hl
	ld	hl,(ix+9)
	push	hl
	bool	hl
	ld	l,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$setU32
	jr	@1126
;2833       case DT_F32:  setF32(_SetRegFuncParams_);  break;
	.debug	# 2833
@212:
	ld	hl,(ix+11)
	push	hl
	ld	hl,(ix+9)
	push	hl
	bool	hl
	ld	l,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$setF32
@1126:
	add	sp,8
;2834     }
	jr	@178
@207:
	.debug	>
;2835     break;
@204:
	or	a,a
	jr	z,@208
	cp	0x1
	jr	z,@209
	cp	0x2
	jr	z,@210
	cp	0x3
	jr	z,@211
	cp	0x4
	jr	z,@212
	jr	@178
@205:
	.debug	# 2835
;2836   }
	jr	@178
@181:
	.debug	>
;2837 
;2838   return true;
@179:
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x14
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
	jp	@177
@178:
@176:
	.debug	>
	.debug	# 2838
	ld	a,0x1
;2839 }
@173:
	.debug	} @173 2839

@172	equ	48
	ld	sp,ix
	pop	ix
	lret
@174	equ	50
;2840 
;2841 //-------------------------------------------------------------------------
;2842 //  process a new value for "cmdBitsAppl"
;2843 //-------------------------------------------------------------------------
;2844 bool MB_cmdBitsAppl(SetRegFuncParams)
;2845 {
	.align	4
$$MB_cmdBitsAppl::
	ex	(sp),iy
$MB_cmdBitsAppl::
	.debug	G $MB_cmdBitsAppl 88 9
	.debug	{ $MB_cmdBitsAppl 88 2844 @215 @214
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A info 5 78
	.debug	A chanNum 7 3
	.debug	A newVal 9 78
	.debug	A eventsfd 11 6
;2846 /*
;2847   ulong newFmt = 0;
;2848 
;2849   cmdBitsAppl = MB_VAL(ulong);
;2850 
;2851 //--- This is the older version of the bit-mapped cmds, so we translate ---
;2852 //--- the best we can in to the newer ones and pass them on ---
;2853 
;2854   if (cmdBitsAppl & OLDC_AMPL_OPEN_LOOP)  newFmt |= CMD_AMPL_CTL_OFF;
;2855   if (cmdBitsAppl & OLDC_AMPL_CLOSED_LOOP)  newFmt |= CMD_AMPL_CTL_ON;
;2856 
;2857   if (cmdBitsAppl & OLDC_PHASE_OPEN_LOOP)  newFmt |= CMD_PHASE_CTL_OFF;
;2858   if (cmdBitsAppl & OLDC_PHASE_CLOSED_LOOP)  newFmt |= CMD_PHASE_CTL_ON;
;2859 
;2860   if (cmdBitsAppl & OLDC_AMPL_TUNE_RUN)  newFmt |= CMD_RF_PULSING_ON;
;2861   if (cmdBitsAppl & OLDC_AMPL_TUNE_STOP)  newFmt |= CMD_RF_PULSING_OFF;
;2862   if (cmdBitsAppl & OLDC_PHASE_TUNE_RUN)  newFmt |= CMD_RF_PULSING_ON;
;2863   if (cmdBitsAppl & OLDC_PHASE_TUNE_STOP)  newFmt |= CMD_RF_PULSING_OFF;
;2864 
;2865   SetMBReg(setDSPState, 0, U32, newFmt);
;2866 */
;2867   return true;
	.debug	# 2867
	ld	a,0x1
;2868 }
@214:
	.debug	} @214 2868

@213	equ	0
	pop	ix
	lret
@215	equ	2
;2869 
;2870 //-------------------------------------------------------------------------
;2871 //  process a new value for "setDSPState"
;2872 //-------------------------------------------------------------------------
;2873 bool MB_setDSPState(SetRegFuncParams)
;2874 {
	.align	4
$$MB_setDSPState::
	ex	(sp),iy
$MB_setDSPState::
	.debug	G $MB_setDSPState 88 9
	.debug	{ $MB_setDSPState 88 2873 @218 @217
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-10
	.debug	A info 5 78
	.debug	A chanNum 7 3
	.debug	A newVal 9 78
	.debug	A eventsfd 11 6
;2875   uint  i;
	.debug	L i -2 7
;2876   ulong  mask, cmdBits;
	.debug	L mask -6 9
	.debug	L cmdBits -10 9
;2877 
;2878 
;2879   cmdBits = MB_VAL(ulong);
	.debug	# 2879
	ld	hl,(ix+9)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-8),hl
	ex	de,hl
	ld	(ix-10),hl
	ex	de,hl
;2880 
;2881 //--- setDSPState contains cmd bits in groups:  i.e. one bit is asserted to
;2882 //--- enable a state, the other to disable it.  None set within a group ---
;2883 //--- means do not change the current state for the group. ---
;2884 
;2885 //--- Writing of values to the DSP is handled primarily by the application 
;2886 //--- task.  To avoid slowing down the task that called this function ---
;2887 //--- waiting for access to the DSP interface, we just keep track of the --
;2888 //--- most recent setting for each group of bits.  That way the actual ---
;2889 //--- writing of the bits (whenever it happens) will always use the last --
;2890 //--- setting for each group, and we have a record of what state the DSP --
;2891 //--- should be in any given time (and a way to re-assert that state if ---
;2892 //--- the DSP is reset) ---
;2893 
;2894   for (i=0; SetDSPStateMasks[i]; ++i)  {
@219:
	.debug	< @219 @220
	.debug	# 2894
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@221:
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_SetDSPStateMasks
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	or	hl,de
	jr	z,@222
;2895     mask = SetDSPStateMasks[i];
@224:
	.debug	< @224 @225
	.debug	# 2895
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_SetDSPStateMasks
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
;2896     if (cmdBits & mask) setDSPState = (setDSPState & ~mask) | (cmdBits & ma
	.debug	# 2896
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@226
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	call	__lnot##
	push	bc
	push	de
	ld	de,(_setDSPState)
	ld	bc,(_setDSPState+2)
	call	__land##
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	call	__land##
	call	__lor##
	ld	(_setDSPState),de
	ld	(_setDSPState+2),bc
;2897   }
@226:
@225:
	.debug	>
;2898 
;2899 //--- Also, log a message showing what bits were set each time we get a new
;2900 //--- value ---
;2901   for (i=0,mask=1; i<32; ++i,mask<<=1)  if (cmdBits & mask)
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jp	@221
@222:
@220:
	.debug	>
@227:
	.debug	< @227 @228
	.debug	# 2901
	bool	hl
	ld	l,h
	ld	(ix-2),hl
	ld	de,0x1
	ld	c,d
	ld	b,d
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
@229:
	ld	hl,(ix-2)
	ld	de,0x20
	or	a,a
	sbc	hl,de
	jr	nc,@230
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@232
;2902     fprintf(DiagLog, "setDSPstate: Set: %ls\n", SetDSPStateDesc[i]);
	.debug	# 2902
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_SetDSPStateDesc
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,word (phys @1+3034)
	ld	c,phys @1+3034 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,10
;2903 
;2904 
;2905   return true;
@232:
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	ld	a,0x1
	call	__lshl##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	jr	@229
@230:
@228:
	.debug	>
	.debug	# 2905
	ld	a,0x1
;2906 }
@217:
	.debug	} @217 2906

@216	equ	10
	ld	sp,ix
	pop	ix
	lret
@218	equ	12
;2907 
;2908 
;2909 //-------------------------------------------------------------------------
;2910 //  process a new value for an amplitude value that needs to be checked
;2911 //  against maxAmplSet
;2912 //-------------------------------------------------------------------------
;2913 bool MB_maxAmpl(SetRegFuncParams)
;2914 {
	.align	4
$$MB_maxAmpl::
	ex	(sp),iy
$MB_maxAmpl::
	.debug	G $MB_maxAmpl 88 9
	.debug	{ $MB_maxAmpl 88 2913 @235 @234
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A info 5 78
	.debug	A chanNum 7 3
	.debug	A newVal 9 78
	.debug	A eventsfd 11 6
;2915   if (REG->dataType != DT_F32)  return false;
	.debug	# 2915
	ld	hl,(ix+5)
	ld	de,0xa
	add	hl,de
	ld	a,(hl)
	sub	a,4
	jr	z,@236
	xor	a,a
	jr	@234
;2916 
;2917   if (MB_VAL(F32) > maxAmplSet)  MB_VAL(F32) = maxAmplSet;
@236:
	.debug	# 2917
	ld	hl,(ix+9)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,(_maxAmplSet)
	ld	bc,(_maxAmplSet+2)
	call	__fsub##
	jp	m,@237
	jr	z,@237
	ld	de,(_maxAmplSet)
	ld	bc,(_maxAmplSet+2)
	ld	hl,(ix+9)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;2918 
;2919   return setF32(_SetRegFuncParams_);
@237:
	.debug	# 2919
	ld	hl,(ix+11)
	push	hl
	ld	hl,(ix+9)
	push	hl
	bool	hl
	ld	l,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$setF32
	add	sp,8
;2920 }
@234:
	.debug	} @234 2920

@233	equ	0
	pop	ix
	lret
@235	equ	2
;2921 
;2922 
;2923 //-------------------------------------------------------------------------
;2924 //  process a new value for one of the xxxGain registers
;2925 //-------------------------------------------------------------------------
;2926 bool MB_setGain(SetRegFuncParams)
;2927 {
	.align	4
$$MB_setGain::
	ex	(sp),iy
$MB_setGain::
	.debug	G $MB_setGain 88 9
	.debug	{ $MB_setGain 88 2926 @240 @239
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A info 5 78
	.debug	A chanNum 7 3
	.debug	A newVal 9 78
	.debug	A eventsfd 11 6
;2928   if (REG->dataType != DT_F32)  return false;
	.debug	# 2928
	ld	hl,(ix+5)
	ld	de,0xa
	add	hl,de
	ld	a,(hl)
	sub	a,4
	jr	nz,@1125
;2929 
;2930   if (MB_VAL(float) == 0.0)  return false;
	.debug	# 2930
	ld	hl,(ix+9)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	a,(hl)
	and	a,7Fh
	or	a,c
	or	a,d
	or	a,e
	jr	nz,@242
@1125:
	xor	a,a
	jr	@239
;2931 
;2932   return setF32(_SetRegFuncParams_);
@242:
	.debug	# 2932
	ld	hl,(ix+11)
	push	hl
	ld	hl,(ix+9)
	push	hl
	bool	hl
	ld	l,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$setF32
	add	sp,8
;2933 }
@239:
	.debug	} @239 2933

@238	equ	0
	pop	ix
	lret
@240	equ	2
;2934 
;2935 
;2936 //-------------------------------------------------------------------------
;2937 //  process a new value for a register with a phase value
;2938 //-------------------------------------------------------------------------
;2939 bool MB_setPhase(SetRegFuncParams)
;2940 {
$MB_setPhase::
	.debug	G $MB_setPhase 88 9
	.debug	{ $MB_setPhase 88 2939 @245 @244
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A info 5 78
	.debug	A chanNum 7 3
	.debug	A newVal 9 78
	.debug	A eventsfd 11 6
;2941   if (REG->dataType != DT_F32)  return false;
	.debug	# 2941
	ld	hl,(ix+5)
	ld	de,0xa
	add	hl,de
	ld	a,(hl)
	sub	a,4
	jr	z,@246
	xor	a,a
	jp	@244
;2942 
;2943   if (MB_VAL(float) < -720.0)  MB_VAL(float) = -360.0;
@246:
	.debug	# 2943
	ld	hl,(ix+9)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,0
	ld	bc,0xc434
	call	__fsub##
	jp	p,@247
	ld	de,0
	ld	bc,0xc3b4
	ld	hl,(ix+9)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;2944   if (MB_VAL(float) >  720.0)  MB_VAL(float) =  360.0;
@247:
	.debug	# 2944
	ld	hl,(ix+9)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,0
	ld	bc,0x4434
	call	__fsub##
	jp	m,@248
	jr	z,@248
	ld	de,0
	ld	bc,0x43b4
	ld	hl,(ix+9)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;2945 
;2946   while (MB_VAL(float) < -360.0)  MB_VAL(float) += 360.0;
@248:
@249:
	.debug	# 2946
	ld	hl,(ix+9)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,0
	ld	bc,0xc3b4
	call	__fsub##
	jp	p,@250
	ld	hl,(ix+9)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,0
	ld	bc,0x43b4
	call	__fadd##
	ld	hl,(ix+9)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;2947   while (MB_VAL(float) >  360.0)  MB_VAL(float) -= 360.0;
	jr	@249
@250:
@251:
	.debug	# 2947
	ld	hl,(ix+9)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,0
	ld	bc,0x43b4
	call	__fsub##
	jp	m,@252
	jr	z,@252
	ld	hl,(ix+9)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,0
	ld	bc,0xc3b4
	call	__fadd##
	ld	hl,(ix+9)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;2948 
;2949   return setF32(_SetRegFuncParams_);
	jr	@251
@252:
	.debug	# 2949
	ld	hl,(ix+11)
	push	hl
	ld	hl,(ix+9)
	push	hl
	bool	hl
	ld	l,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$setF32
	add	sp,8
;2950 }
@244:
	.debug	} @244 2950

@243	equ	0
	pop	ix
	lret
@245	equ	2
;2951 
;2952 
;2953 //-------------------------------------------------------------------------
;2954 //
;2955 //-------------------------------------------------------------------------
;2956 int _nearcall PV_UpdateVal(PV_OP_Params)
;2957 {
	.align	4
$$PV_UpdateVal::
	ex	(sp),iy
$PV_UpdateVal::
	.debug	G $PV_UpdateVal 90 9
	.debug	{ $PV_UpdateVal 90 2956 @255 @254
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A pv 5 91
;2958   //finish:  Need to update pv->newEvents and pv->val for any non-waveform
;2959   //         PVs that don't do it elsewhere
;2960 
;2961 //  printf("PV_UpdateVal(%ls)\r\n", pv->pvInfo.name);  //tdebug
;2962 
;2963   return -1;
	.debug	# 2963
	ld	hl,0xffff
;2964 }
@254:
	.debug	} @254 2964

@253	equ	0
	pop	ix
	lret
@255	equ	2
;2965 
;2966 //-------------------------------------------------------------------------
;2967 //
;2968 //-------------------------------------------------------------------------
;2969 int _nearcall PV_LockVal(PV_Lock_Params)
;2970 {
	.align	4
$$PV_LockVal::
	ex	(sp),iy
$PV_LockVal::
	.debug	G $PV_LockVal 92 9
	.debug	{ $PV_LockVal 92 2969 @258 @257
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A pv 5 91
	.debug	A lockMode 7 6
	.debug	A lockType 9 6
;2971   //finish:  Implement locking for non-waveform values
;2972 
;2973 //tdebug
;2974 //  printf("\r\nPV_LockVal(%ls, %d, %d)\r\n", pv->pvInfo.name, lockMode, lo
;2975 
;2976   return 0;
	.debug	# 2976
	bool	hl
	ld	l,h
;2977 }
@257:
	.debug	} @257 2977

@256	equ	0
	pop	ix
	lret
@258	equ	2
;2978 
;2979 //-------------------------------------------------------------------------
;2980 //
;2981 //-------------------------------------------------------------------------
;2982 int _nearcall PV_SendVal(PV_RW_Params)
;2983 {
	.align	4
$$PV_SendVal::
	ex	(sp),iy
$PV_SendVal::
	.debug	G $PV_SendVal 93 9
	.debug	{ $PV_SendVal 93 2982 @261 @260
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-13
	.debug	A id 5 6
	.debug	A numElems 7 7
	.debug	A conInfo 9 78
	.debug	A copyFunc 11 94
;2984   PV_Info  pvInfo;
	.debug	L pvInfo -13 96
;2985 
;2986 
;2987   //finish:  call (*copyFunc)(void *conInfo, void far *from, uint count)
;2988   //         as needed to send PV value
;2989 
;2990   if (!pvGetInfo(id, &pvInfo))  pvInfo.name = "<invalid>";
	.debug	# 2990
	ld	hl,-13+@259
	add	hl,sp
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$pvGetInfo
	add	sp,4
	bool	hl
	jr	nz,@262
	ld	de,word (phys @1+3057)
	ld	c,phys @1+3057 >> 16
	ld	hl,bc
	ld	(ix-11),hl
	ex	de,hl
	ld	(ix-13),hl
	ex	de,hl
;2991 
;2992   printf("PV_SendVal(%ls)\r\n", pvInfo.name);  //tdebug
@262:
	.debug	# 2992
	ld	hl,(ix-13)
	ex	de,hl
	ld	hl,(ix-11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,word (phys @1+3067)
	ld	c,phys @1+3067 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
;2993 
;2994   return -1;
	.debug	# 2994
	ld	hl,0xffff
;2995 }
@260:
	.debug	} @260 2995

@259	equ	13
	ld	sp,ix
	pop	ix
	lret
@261	equ	15
;2996 
;2997 //-------------------------------------------------------------------------
;2998 //
;2999 //-------------------------------------------------------------------------
;3000 int _nearcall PV_RcvVal(PV_RW_Params)
;3001 {
	.align	4
$$PV_RcvVal::
	ex	(sp),iy
$PV_RcvVal::
	.debug	G $PV_RcvVal 93 9
	.debug	{ $PV_RcvVal 93 3000 @265 @264
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-13
	.debug	A id 5 6
	.debug	A numElems 7 7
	.debug	A conInfo 9 78
	.debug	A copyFunc 11 94
;3002   PV_Info  pvInfo;
	.debug	L pvInfo -13 96
;3003 
;3004 
;3005   //finish:  call (*copyFunc)(void *conInfo, void far *from, uint count)
;3006   //         as needed to get new PV value
;3007   //         (probably need more info so we know how big a payload is in
;3008   //         the message that caused this call)
;3009 
;3010   if (!pvGetInfo(id, &pvInfo))  pvInfo.name = "<invalid>";
	.debug	# 3010
	ld	hl,-13+@263
	add	hl,sp
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$pvGetInfo
	add	sp,4
	bool	hl
	jr	nz,@266
	ld	de,word (phys @1+3057)
	ld	c,phys @1+3057 >> 16
	ld	hl,bc
	ld	(ix-11),hl
	ex	de,hl
	ld	(ix-13),hl
	ex	de,hl
;3011 
;3012   printf("PV_RcvVal(%ls)\r\n", pvInfo.name);  //tdebug
@266:
	.debug	# 3012
	ld	hl,(ix-13)
	ex	de,hl
	ld	hl,(ix-11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,word (phys @1+3085)
	ld	c,phys @1+3085 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
;3013 
;3014   return -1;
	.debug	# 3014
	ld	hl,0xffff
;3015 }
@264:
	.debug	} @264 3015

@263	equ	13
	ld	sp,ix
	pop	ix
	lret
@265	equ	15
;3016 
;3017 
;3018 //-------------------------------------------------------------------------
;3019 //  If newer data available, update the copy of a PV's value that is sent t
;3020 //  external clients (and update the newEvents flag indicating what events 
;3021 //  new data applies to).
;3022 //-------------------------------------------------------------------------
;3023 int _nearcall PV_UpdateValWF(PV_OP_Params)
;3024 {
	.align	4
$$PV_UpdateValWF::
	ex	(sp),iy
$PV_UpdateValWF::
	.debug	G $PV_UpdateValWF 90 9
	.debug	{ $PV_UpdateValWF 90 3023 @269 @268
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-21
	.debug	A pv 5 91
;3025   bool noChange;
	.debug	L noChange -1 3
;3026   int  stat;
	.debug	L stat -3 6
;3027   uint  src, cpy, numBytes, srcLockID, cpyLockID;
	.debug	L src -5 7
	.debug	L cpy -7 7
	.debug	L numBytes -9 7
	.debug	L srcLockID -11 7
	.debug	L cpyLockID -13 7
;3028   char far  *srcBuf, *cpyBuf;
	.debug	L srcBuf -17 48
	.debug	L cpyBuf -21 48
;3029 
;3030 
;3031   if ((src = WF_IDX(pv->val) - 1) >= NUM_WAVEFORMS*2)  return -1;
	.debug	# 3031
	ld	hl,(ix+5)
	ld	de,0xd
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,word (phys _waveformData)
	ld	c,phys _waveformData >> 16
	call	__lsub##
	ld	a,0x2
	call	__lshr##
	ld	a,0xa
	call	__lshr##
	dec	de
	ex	de,hl
	ld	(ix-5),hl
	ld	de,0x6
	or	a,a
	sbc	hl,de
	jp	nc,@275
;3032   cpy = src + 1;
	.debug	# 3032
	ld	hl,(ix-5)
	inc	hl
	ld	(ix-7),hl
;3033 
;3034 //--- Don't even TRY to get the locks unless we KNOW there is new data ---
;3035   _EnterCritical_();
	.debug	# 3035
	ipset	2
;3036   noChange = (dataTimeMS[cpy] == dataTimeMS[src]);
	.debug	# 3036
	add	hl,hl
	add	hl,hl
	ld	de,_dataTimeMS
	add	hl,de
	ex	de,hl
	ld	hl,(ix-5)
	add	hl,hl
	add	hl,hl
	ld	bc,de
	ld	de,_dataTimeMS
	add	hl,de
	push	bc
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	pop	hl
	push	bc
	push	de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__lsub##
	ld	de,0
	jr	nz,@271
	inc	e
@271:
	ld	(ix-1),e
;3037   _ExitCritical_();
	.debug	# 3037
	ipres
;3038   if (noChange)  return 0;
	.debug	# 3038
	ld	a,e
	or	a,a
	jr	z,@272
	bool	hl
	ld	l,h
	jp	@268
;3039 
;3040   if (ShowDebugMsgs)
@272:
	.debug	# 3040
	ld	hl,(_current)
	ld	de,0xae
	add	hl,de
	ld	hl,(hl)
	ld	de,0x800
	and	hl,de
	jr	z,@273
;3041     printf("New data avail for PV val: [%ls]\r\n", pv->pvInfo.name);
	.debug	# 3041
	ld	hl,(ix+5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,word (phys @1+3102)
	ld	c,phys @1+3102 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
;3042 
;3043 
;3044   srcBuf = (char far *) waveformData[src];    srcLockID = WAVE_DATA_0 + src
@273:
	.debug	# 3044
	ld	hl,(ix-5)
	ex	de,hl
	ld	bc,0
	ld	a,0xc
	call	__lshl##
	push	bc
	push	de
	ld	de,word (phys _waveformData)
	ld	c,phys _waveformData >> 16
	call	__ladd##
	ld	hl,bc
	ld	(ix-15),hl
	ex	de,hl
	ld	(ix-17),hl
	ex	de,hl
	ld	hl,(ix-5)
	inc	hl
	ld	(ix-11),hl
;3045   cpyBuf = (char far *) waveformData[cpy];    cpyLockID = WAVE_DATA_0 + cpy
	.debug	# 3045
	ld	hl,(ix-7)
	ex	de,hl
	ld	bc,0
	ld	a,0xc
	call	__lshl##
	push	bc
	push	de
	ld	de,word (phys _waveformData)
	ld	c,phys _waveformData >> 16
	call	__ladd##
	ld	hl,bc
	ld	(ix-19),hl
	ex	de,hl
	ld	(ix-21),hl
	ex	de,hl
	ld	hl,(ix-7)
	inc	hl
	ld	(ix-13),hl
;3046 
;3047 
;3048 //--- Get a lock on the dest/cpy buffer so it won't change while we are ---
;3049 //--- accessing (and possibly updating) it ---
;3050 
;3051   if (getLock(cpyLockID, 0) <= 0)  return -1;  // try to get lock on dest b
	.debug	# 3051
	ld	de,0
	ld	c,e
	ld	b,e
	push	bc
	push	de
	push	hl
	lcall	$getLock
	add	sp,6
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	z,@275
	jr	nc,@274
@275:
	ld	hl,0xffff
	jp	@268
;3052 
;3053 //--- If the timestamps for the source and destination buffers are ---
;3054 //--- different, then update the contents and timestamp for the ---
;3055 //--- destination buffer.  NOTE that we don't really care about the ---
;3056 //--- possibility of the src date/time changing during the check as ---
;3057 //--- all we really care about is whether or not it IS different. ---
;3058 
;3059   stat = 0;
@274:
	.debug	# 3059
	bool	hl
	ld	l,h
	ld	(ix-3),hl
;3060   if (dataTimeMS[cpy] != dataTimeMS[src])  do {
	.debug	# 3060
	ld	hl,(ix-7)
	add	hl,hl
	add	hl,hl
	ld	de,_dataTimeMS
	add	hl,de
	ex	de,hl
	ld	hl,(ix-5)
	add	hl,hl
	add	hl,hl
	ld	bc,de
	ld	de,_dataTimeMS
	add	hl,de
	push	bc
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	pop	hl
	push	bc
	push	de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__lsub##
	jp	z,@276
;3061     if (getLock(srcLockID, 0) <= 0)  { stat = -1;  break; }  // try to get 
@280:
	.debug	< @280 @281
	.debug	# 3061
	ld	de,0
	ld	c,e
	ld	b,e
	push	bc
	push	de
	ld	hl,(ix-11)
	push	hl
	lcall	$getLock
	add	sp,6
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	z,@283
	jr	nc,@282
@283:
@284:
	.debug	< @284 @285
	ld	hl,0xffff
	ld	(ix-3),hl
	jp	@278
@285:
	.debug	>
;3062 
;3063     numBytes = numDataPoints[src]<<2;
@282:
	.debug	# 3063
	ld	hl,(ix-5)
	add	hl,hl
	ld	de,_numDataPoints
	add	hl,de
	ld	hl,(hl)
	add	hl,hl
	add	hl,hl
	ld	(ix-9),hl
;3064 
;3065     fmemcpy(cpyBuf, srcBuf, numBytes);
	.debug	# 3065
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-15)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-21)
	ex	de,hl
	ld	hl,(ix-19)
	ld	bc,hl
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
;3066     numDataPoints[cpy] = numDataPoints[src];
	.debug	# 3066
	ld	hl,(ix-5)
	add	hl,hl
	ld	de,_numDataPoints
	add	hl,de
	ex	de,hl
	ld	hl,(ix-7)
	add	hl,hl
	ld	bc,de
	ld	de,_numDataPoints
	add	hl,de
	ld	de,bc
	ex	de,hl
	ld	hl,(hl)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;3067     dataTimeSecs[cpy] = dataTimeSecs[src];
	.debug	# 3067
	ld	hl,(ix-5)
	add	hl,hl
	add	hl,hl
	ld	de,_dataTimeSecs
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,(ix-7)
	add	hl,hl
	add	hl,hl
	push	de
	ld	de,_dataTimeSecs
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;3068     dataTimeMS[cpy] = dataTimeMS[src];
	.debug	# 3068
	ld	hl,(ix-5)
	add	hl,hl
	add	hl,hl
	ld	de,_dataTimeMS
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,(ix-7)
	add	hl,hl
	add	hl,hl
	push	de
	ld	de,_dataTimeMS
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;3069 
;3070     releaseLock(srcLockID);  // release lock on src buffer
	.debug	# 3070
	ld	hl,(ix-11)
	push	hl
	lcall	$releaseLock
	pop	de
;3071 
;3072     pv->pvInfo.size = numBytes;
	.debug	# 3072
	ld	hl,(ix+5)
	ld	de,0x5
	add	hl,de
	ex	de,hl
	ld	hl,(ix-9)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;3073     pv->pvInfo.numElems = numDataPoints[src];
	.debug	# 3073
	ld	hl,(ix-5)
	add	hl,hl
	ld	de,_numDataPoints
	add	hl,de
	ex	de,hl
	ld	hl,(ix+5)
	ex	de,hl
	ld	bc,hl
	ld	hl,0x9
	add	hl,de
	ld	de,bc
	ex	de,hl
	ld	hl,(hl)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;3074 
;3075     pv->newEvents |= DBE_VALUE;
	.debug	# 3075
	ld	hl,(ix+5)
	ld	de,0x13
	add	hl,de
	ld	a,(hl)
	or	a,1
	ld	(hl),a
;3076 
;3077     if (ShowDebugMsgs)
	.debug	# 3077
	ld	hl,(_current)
	ld	e,0xae
	add	hl,de
	ld	hl,(hl)
	ld	de,0x800
	and	hl,de
	jr	z,@286
;3078       printf("Updated copy of PV val: [%ls]\r\n", pv->pvInfo.name);
	.debug	# 3078
	ld	hl,(ix+5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,word (phys @1+3137)
	ld	c,phys @1+3137 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
;3079   } while (0);
@286:
@281:
	.debug	>
	.debug	# 3079
;3080 
;3081   releaseLock(cpyLockID);
@278:
@276:
	.debug	# 3081
	ld	hl,(ix-13)
	push	hl
	lcall	$releaseLock
	pop	de
;3082 
;3083   return stat;
	.debug	# 3083
	ld	hl,(ix-3)
;3084 }
@268:
	.debug	} @268 3084

@267	equ	21
	ld	sp,ix
	pop	ix
	lret
@269	equ	23
;3085 
;3086 
;3087 //-------------------------------------------------------------------------
;3088 //  Attempt to obtain or release a read or write lock on a waveform PV.
;3089 //
;3090 //  Returns the depth of the lock or < 0 if not successsful.
;3091 //
;3092 //  For lockMode LM_TryLock, a return value < 0 and errno == EAGAIN means
;3093 //  it could not get the lock w/o blocking.
;3094 //-------------------------------------------------------------------------
;3095 int _nearcall PV_LockWave(PV_Lock_Params)
;3096 {
	.align	4
$$PV_LockWave::
	ex	(sp),iy
$PV_LockWave::
	.debug	G $PV_LockWave 92 9
	.debug	{ $PV_LockWave 92 3095 @289 @288
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-6
	.debug	A pv 5 91
	.debug	A lockMode 7 6
	.debug	A lockType 9 6
;3097   uint  src, cpy, lockID;
	.debug	L src -2 7
	.debug	L cpy -4 7
	.debug	L lockID -6 7
;3098 
;3099 
;3100 //tdebug
;3101 //  printf("\r\nLock PV val: [%ls]  mode:[%d]  type:[%d]\r\n",
;3102 //         pv->pvInfo.name, lockMode, lockType);
;3103 
;3104   if ((src = WF_IDX(pv->val) - 1) >= NUM_WAVEFORMS*2)  return -1;
	.debug	# 3104
	ld	hl,(ix+5)
	ld	de,0xd
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	de,word (phys _waveformData)
	ld	c,phys _waveformData >> 16
	call	__lsub##
	ld	a,0x2
	call	__lshr##
	ld	a,0xa
	call	__lshr##
	dec	de
	ex	de,hl
	ld	(ix-2),hl
	ld	de,0x6
	or	a,a
	sbc	hl,de
	jr	nc,@297
;3105   cpy = src + 1;
	.debug	# 3105
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-4),hl
;3106 
;3107   switch (lockType)  {
	.debug	# 3107
	ld	hl,(ix+9)
	jr	@291
;3108     case LT_READ:  lockID = WAVE_DATA_0 + cpy;  break;
@293:
	.debug	< @293 @294
	.debug	# 3108
@295:
	ld	hl,(ix-4)
	jr	@1124
;3109     case LT_WRITE:  lockID = WAVE_DATA_0 + src;  break;
	.debug	# 3109
@296:
	ld	hl,(ix-2)
@1124:
	inc	hl
	ld	(ix-6),hl
;3110     default:  return -1;
	jr	@292
	.debug	# 3110
@297:
	ld	hl,0xffff
	jr	@288
;3111   }
@294:
	.debug	>
;3112 
;3113   switch (lockMode)  {
@291:
	ex	de,hl
	ld	a,d
	or	a,e
	jr	z,@295
	ld	hl,0xffffffff
	add	hl,de
	bool	hl
	jr	z,@296
	jr	@297
@292:
	.debug	# 3113
	ld	hl,(ix+7)
	jr	@298
;3114     case LM_Unlock:    return releaseLock(lockID);
@300:
	.debug	< @300 @301
	.debug	# 3114
@302:
	ld	hl,(ix-6)
	push	hl
	lcall	$releaseLock
	pop	de
	jr	@288
;3115     case LM_WaitLock:  return getLock(lockID, 5000);
	.debug	# 3115
@303:
	ld	de,0x1388
	ld	bc,0
	jr	@1123
;3116     case LM_TryLock:   return getLock(lockID, 0);
	.debug	# 3116
@304:
	ld	de,0
	ld	c,e
	ld	b,e
@1123:
	push	bc
	push	de
	ld	hl,(ix-6)
	push	hl
	lcall	$getLock
	add	sp,6
	jr	@288
;3117   }
@301:
	.debug	>
;3118 
;3119   return -1;
@298:
	ex	de,hl
	ld	a,d
	or	a,e
	jr	z,@302
	ld	hl,0xffffffff
	add	hl,de
	bool	hl
	jr	z,@303
	ld	hl,0xfffffffe
	add	hl,de
	bool	hl
	jr	z,@304
@299:
	.debug	# 3119
	ld	hl,0xffff
;3120 }
@288:
	.debug	} @288 3120

@287	equ	6
	ld	sp,ix
	pop	ix
	lret
@289	equ	8
;3121 
;3122 
;3123 //-------------------------------------------------------------------------
;3124 
;3125 
;3126 #define SharedFuncs
;3127 #include <sharedAppl.c>
;3128 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
$initModBusRegs::
	.debug	G $initModBusRegs 97 9
	.debug	{ $initModBusRegs 97 454 @307 @306
	.debug	# 456
	ld	hl,0x64
	ld	(_C0ChanMult),hl
	ld	l,0x1
	ld	(_NumC0Chans),hl
	.debug	# 457
	ld	l,0xc8
	ld	(_R3ChanMult),hl
	ld	l,0x1
	ld	(_NumR3Chans),hl
	.debug	# 458
	ld	hl,0x12c
	ld	(_R4ChanMult),hl
	ld	hl,0x1
	ld	(_NumR4Chans),hl
	.debug	# 460
	lcall	$pvInitLists
	bool	hl
	jr	z,@308
@309:
	.debug	< @309 @310
	.debug	# 461
	ld	a,0x1
	ld	(_doShutdown),a
	.debug	# 462
	ld	a,(_runningPrimary)
	or	a,a
	jr	z,@311
	ld	a,0x1
	ld	(_runBackup),a
	ld	(_doReboot),a
@311:
@310:
	.debug	>
@308:
@306:
	.debug	} @306 464

@305	equ	0
	lret
@307	equ	0
	.align	4
$$MB_timeZone::
	ex	(sp),iy
$MB_timeZone::
	.debug	G $MB_timeZone 88 9
	.debug	{ $MB_timeZone 88 470 @314 @313
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-2
	.debug	A info 5 78
	.debug	A chanNum 7 3
	.debug	A newVal 9 78
	.debug	A eventsfd 11 6
	.debug	L intVal -2 6
	.debug	# 474
	ld	hl,(ix+9)
	ld	hl,(hl)
	ld	(ix-2),hl
	.debug	# 476
	ld	de,0x7ff4
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@315
	ld	hl,(ix-2)
	ld	de,0x800c
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@316
	jr	z,@316
@315:
	xor	a,a
	jr	@313
@316:
	.debug	# 478
	ld	hl,(ix-2)
	ld	(_timeZone),hl
	.debug	# 479
	ld	a,(_startupComplete)
	or	a,a
	jr	z,@317
	push	hl
	ld	de,word (phys @1+3169)
	ld	c,phys @1+3169 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,8
@317:
	.debug	# 481
	ipset	2
	.debug	# 482
	ld	hl,(ix+5)
	ld	hl,(hl)
	ld	d,0
	ld	e,(ix+7)
	ex	de,hl
	add	hl,hl
	add	hl,de
	ex	de,hl
	ld	hl,(ix+9)
	ld	hl,(hl)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	.debug	# 483
	ipres
	.debug	# 485
	ld	a,0x1
@313:
	.debug	} @313 486

@312	equ	2
	ld	sp,ix
	pop	ix
	lret
@314	equ	4
	.align	4
$$MB_cmdBitsStd::
	ex	(sp),iy
$MB_cmdBitsStd::
	.debug	G $MB_cmdBitsStd 88 9
	.debug	{ $MB_cmdBitsStd 88 494 @320 @319
	ld	hl,-132
	call	__fent##
	.debug	A info 5 78
	.debug	A chanNum 7 3
	.debug	A newVal 9 78
	.debug	A eventsfd 11 6
	.debug	L cmdBits -4 9
	.debug	L buf -132 98
	.debug	# 503
	ld	hl,(ix+9)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
	.debug	# 505
	push	bc
	push	de
	ld	de,0x1
	ld	c,d
	ld	b,d
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@321
	ld	a,0x1
	ld	(_doReboot),a
	ld	(_doShutdown),a
@321:
	.debug	# 507
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@322
	ld	a,0x1
	ld	(_doShutdown),a
@322:
	.debug	# 509
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@323
	.debug	# 510
	ld	hl,0x1
	push	hl
	ld	hl,_devDataFile
	push	hl
	ld	hl,0x80
	push	hl
	ld	hl,-126+@318
	add	hl,sp
	push	hl
	lcall	$saveParams
	add	sp,8
@323:
	.debug	# 511
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@324
	ld	hl,_devDataFile
	push	hl
	lcall	$loadParamsMsg
	pop	de
@324:
	.debug	# 513
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x10
	ld	c,d
	ld	b,d
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@325
	.debug	# 514
	ld	hl,0x1
	push	hl
	ld	hl,_hdwCfgFile
	push	hl
	ld	hl,0x80
	push	hl
	ld	hl,-126+@318
	add	hl,sp
	push	hl
	lcall	$saveParams
	add	sp,8
@325:
	.debug	# 515
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x20
	ld	c,d
	ld	b,d
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@326
	ld	hl,_hdwCfgFile
	push	hl
	lcall	$loadParamsMsg
	pop	de
@326:
	.debug	# 523
	ld	a,0x1
@319:
	.debug	} @319 524

@318	equ	132
	ld	sp,ix
	pop	ix
	lret
@320	equ	134
$rwIO::
	.debug	G $rwIO 97 9
	.debug	{ $rwIO 97 531 @329 @328
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-28
	.bseg
@330:
	.ds	2
	.debug	S ioTicks @330 6
	.seg	FARCODE
	.debug	L ui -2 7
	.debug	L chgdMask -4 7
	.debug	L chgBit -6 7
	.debug	L aout -10 99
	.debug	L update -12 70
	.debug	L startTicks -16 9
	.debug	L endTicks -20 9
	.debug	L rwStartTicks -24 9
	.debug	L rwEndTicks -28 9
	.debug	# 541
	ld	hl,(@330)
	inc	hl
	ld	(@330),hl
	.debug	# 543
	lcall	$clockTicks
	ld	hl,bc
	ld	(ix-22),hl
	ex	de,hl
	ld	(ix-24),hl
	ex	de,hl
	.debug	# 546
	bool	hl
	ld	l,h
	ld	(ix-4),hl
	inc	l
	ld	(ix-6),hl
	.debug	# 547
	ld	de,(_DAC_Mask)
	ld	bc,(_DAC_Mask+2)
	ld	hl,bc
	or	hl,de
	jp	z,@331
@332:
	.debug	< @332 @333
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@334:
	ld	hl,(ix-2)
	ld	de,(_NumHdwDAC)
	or	a,a
	sbc	hl,de
	jp	nc,@335
@337:
	.debug	< @337 @338
	.debug	# 548
	ld	hl,(ix-2)
	ld	de,_updateAOut
	add	hl,de
	ld	(ix-12),hl
	.debug	# 550
	ld	hl,(ix-2)
	ld	de,0x1
	or	a,a
	sbc	hl,de
	jr	nc,@339
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_setVal
	add	hl,de
	ex	de,hl
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	bc,de
	ld	de,_lastSetVal
	add	hl,de
	push	bc
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	pop	hl
	push	bc
	push	de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fequ##
	jr	z,@340
@341:
	.debug	< @341 @342
	.debug	# 551
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_setVal
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	push	de
	ld	de,_lastSetVal
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	hl,(ix-12)
	ld	(hl),0x1
@342:
	.debug	>
@340:
@339:
	.debug	# 553
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_anaOut
	add	hl,de
	ex	de,hl
	call	__farcvp##
	ld	hl,bc
	ld	(ix-8),hl
	ex	de,hl
	ld	(ix-10),hl
	ex	de,hl
	.debug	# 554
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_lastAnaOut
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	call	__farld4##
	call	__fequ##
	jr	z,@343
	ld	hl,(ix-12)
	ld	(hl),0x1
@343:
	.debug	# 556
	ld	hl,(ix-12)
	ld	a,(hl)
	or	a,a
	jp	z,@336
	.debug	# 557
	xor	a,a
	ld	(hl),a
	.debug	# 559
	ld	hl,(ix-2)
	ld	de,_dacForced
	add	hl,de
	ld	a,(hl)
	or	a,a
	jr	z,@345
	.debug	# 560
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_dacForcedVal
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	call	__farst4p##
	jr	@346
@345:
	.debug	# 562
	ld	hl,(ix-2)
	ld	de,(_NumApplAOUT)
	or	a,a
	sbc	hl,de
	jr	nc,@347
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_setVal
	add	hl,de
	ex	de,hl
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	bc,de
	ld	de,_setGain
	add	hl,de
	push	bc
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	pop	hl
	push	bc
	push	de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fdivr##
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_setOffset
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fadd##
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	call	__farst4p##
@347:
@346:
	.debug	# 563
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_aoutMin
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	call	__farld4##
	call	__fsubr##
	jp	p,@348
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_aoutMin
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	call	__farst4p##
@348:
	.debug	# 564
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_aoutMax
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	call	__farld4##
	call	__fsubr##
	jp	m,@349
	jr	z,@349
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_aoutMax
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	call	__farst4p##
@349:
	.debug	# 565
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	call	__farld4##
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	push	de
	ld	de,_lastAnaOut
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	.debug	# 567
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_dacGain
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	call	__farld4##
	call	__fdivr##
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_dacOffset
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fadd##
	call	__ftol##
	ld	hl,(ix-2)
	add	hl,hl
	ld	bc,de
	ld	de,_dacRaw
	add	hl,de
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-6)
	ex	de,hl
	or	hl,de
	ld	(ix-4),hl
@338:
	.debug	>
@336:
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	ld	hl,(ix-6)
	add	hl,hl
	ld	(ix-6),hl
	jp	@334
@335:
@333:
	.debug	>
@331:
	.debug	# 571
	ld	hl,(_hdwType)
	ld	de,0x1
	or	a,a
	sbc	hl,de
	jr	z,@350
	ld	hl,(@330)
	ld	e,0x1f
	and	hl,de
	ld	a,l
	ld	hl,0x1
	or	a,a
	jr	z,@351
	ld	b,a
@352:
	add	hl,hl
	djnz	@352
@351:
	ex	de,hl
	ld	hl,(ix-4)
	ex	de,hl
	or	hl,de
	ld	(ix-4),hl
@350:
	.debug	# 574
	ld	hl,(__TimerMilliSeconds+4)
	ld	(_lastReadTime),hl
	.debug	# 575
	lcall	$clockTicks
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
	.debug	# 576
	ld	de,(_ADC_Mask)
	ld	bc,(_ADC_Mask+2)
	ld	hl,bc
	or	hl,de
	jr	z,@353
	push	bc
	push	de
	lcall	$readAnalogInputs
	add	sp,4
@353:
	.debug	# 577
	lcall	$clockTicks
	ld	hl,bc
	ld	(ix-18),hl
	ex	de,hl
	ld	(ix-20),hl
	ex	de,hl
	.debug	# 578
	lcall	$ReadDigInputs
	ld	(_digitalIn),de
	ld	(_digitalIn+2),bc
	.debug	# 579
	ld	hl,(ix-20)
	ex	de,hl
	ld	hl,(ix-18)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__lsub##
	ld	(_readTicks),de
	.debug	# 582
	lcall	$clockTicks
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
	.debug	# 583
	ld	de,(_DAC_Mask)
	ld	bc,(_DAC_Mask+2)
	ld	hl,bc
	or	hl,de
	jr	z,@354
	ld	hl,(ix-4)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	d,h
	ld	e,l
	ld	bc,(_DAC_Mask+2)
	call	__land##
	push	bc
	push	de
	lcall	$writeAnalogOutputs
	add	sp,4
@354:
	.debug	# 584
	lcall	$clockTicks
	ld	hl,bc
	ld	(ix-18),hl
	ex	de,hl
	ld	(ix-20),hl
	ex	de,hl
	.debug	# 585
	ld	de,(_digitalOut)
	ld	bc,(_digitalOut+2)
	push	bc
	push	de
	lcall	$WriteDigOutputs
	add	sp,4
	.debug	# 586
	ld	hl,(ix-20)
	ex	de,hl
	ld	hl,(ix-18)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__lsub##
	ld	(_writeTicks),de
	.debug	# 589
	ld	de,(_ADC_Mask)
	ld	bc,(_ADC_Mask+2)
	ld	hl,bc
	or	hl,de
	jp	z,@355
@356:
	.debug	< @356 @357
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@358:
	ld	hl,(ix-2)
	ld	de,(_NumHdwADC)
	or	a,a
	sbc	hl,de
	jp	nc,@359
@361:
	.debug	< @361 @362
	.debug	# 590
	ld	hl,(ix-2)
	ld	de,_updateAIn
	add	hl,de
	ld	(ix-12),hl
	.debug	# 591
	ld	hl,(ix-2)
	add	hl,hl
	ld	de,_adcRaw
	add	hl,de
	ex	de,hl
	ld	hl,(ix-2)
	add	hl,hl
	ld	bc,de
	ld	de,_lastadcRaw
	add	hl,de
	ld	hl,(hl)
	ld	de,bc
	ex	de,hl
	ld	hl,(hl)
	or	a,a
	sbc	hl,de
	jr	z,@363
@364:
	.debug	< @364 @365
	.debug	# 592
	ld	hl,(ix-2)
	add	hl,hl
	ld	de,_adcRaw
	add	hl,de
	ex	de,hl
	ld	hl,(ix-2)
	add	hl,hl
	ld	bc,de
	ld	de,_lastadcRaw
	add	hl,de
	ld	de,bc
	ex	de,hl
	ld	hl,(hl)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,(ix-12)
	ld	(hl),0x1
@365:
	.debug	>
@363:
	.debug	# 593
	ld	hl,(ix-12)
	ld	a,(hl)
	or	a,a
	jp	z,@360
	.debug	# 594
	xor	a,a
	ld	(hl),a
	.debug	# 595
	ld	hl,(ix-2)
	ld	de,_adcForced
	add	hl,de
	ld	a,(hl)
	or	a,a
	jr	z,@367
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_adcForcedVal
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	jr	@368
@367:
	ld	hl,(ix-2)
	add	hl,hl
	ld	de,_adcRaw
	add	hl,de
	ld	hl,(hl)
	ex	de,hl
	call	__uitof##
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_adcOffset
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fsub##
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_adcGain
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fmlt##
@368:
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	push	de
	ld	de,_anaIn
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	.debug	# 596
	ld	hl,(ix-2)
	ld	de,(_NumApplAIN)
	or	a,a
	sbc	hl,de
	jr	nc,@369
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	de,_anaIn
	add	hl,de
	ex	de,hl
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	ld	bc,de
	ld	de,_readOffset
	add	hl,de
	push	bc
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	pop	hl
	push	bc
	push	de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fsubr##
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_readGain
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fmlt##
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,hl
	push	de
	ld	de,_readVal
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
@369:
	.debug	# 597
	ld	hl,(ix-2)
	ld	de,_newRdg
	add	hl,de
	ld	a,0x1
	ld	(hl),a
@362:
	.debug	>
@360:
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jp	@358
@359:
@357:
	.debug	>
@355:
	.debug	# 601
	lcall	$updateAvgsADC
	.debug	# 604
	lcall	$clockTicks
	ld	hl,bc
	ld	(ix-26),hl
	ex	de,hl
	ld	(ix-28),hl
	ex	de,hl
	.debug	# 606
	push	bc
	push	de
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	ld	bc,hl
	call	__lsub##
	ld	(_rwTicks),de
@328:
	.debug	} @328 607

@327	equ	28
	ld	sp,ix
	pop	ix
	lret
@329	equ	30
$logDigIOChgs::
	.debug	G $logDigIOChgs 100 9
	.debug	{ $logDigIOChgs 100 613 @372 @371
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-20
	.debug	A firstTime 5 3
	.debug	L ui -2 7
	.debug	L offset -4 7
	.debug	L digInChgs -8 9
	.debug	# 616
	ld	de,(_curState)
	ld	bc,(_curState+2)
	push	bc
	push	de
	ld	de,(_prevState)
	ld	bc,(_prevState+2)
	call	__lxor##
	ld	hl,bc
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
	.seg	FARCODE
	.debug	L digOutChgs -12 9
	.debug	# 617
	ld	de,(_curState+4)
	ld	bc,(_curState+6)
	push	bc
	push	de
	ld	de,(_prevState+4)
	ld	bc,(_prevState+6)
	call	__lxor##
	ld	hl,bc
	ld	(ix-10),hl
	ex	de,hl
	ld	(ix-12),hl
	ex	de,hl
	.seg	FARCODE
	.debug	L mask -16 9
	.debug	L msg -20 48
	.debug	# 622
	ld	a,(ix+5)
	or	a,a
	jr	z,@373
	ld	de,word (phys @1+3193)
	ld	c,phys @1+3193 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
@373:
	.debug	# 625
	ld	a,(ix+5)
	or	a,a
	jr	nz,@374
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	or	hl,de
	jp	z,@375
@374:
@376:
	.debug	< @376 @377
	.debug	# 626
	ld	de,0x1
	ld	c,d
	ld	b,d
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
	ld	(ix-2),hl
@378:
	ld	hl,(ix-2)
	ld	de,(_NumApplDIN)
	or	a,a
	sbc	hl,de
	jp	nc,@379
@381:
	.debug	< @381 @382
	.debug	# 627
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	nz,@383
	ld	a,(ix+5)
	or	a,a
	jr	z,@384
@383:
@385:
	.debug	< @385 @386
	.debug	# 628
	ld	hl,(ix-2)
	add	hl,hl
	ld	(ix-4),hl
	ld	de,(_curState)
	ld	bc,(_curState+2)
	push	bc
	push	de
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@387
	ld	hl,(ix-4)
	inc	hl
	ld	(ix-4),hl
@387:
	.debug	# 629
	ld	hl,(ix-4)
	add	hl,hl
	add	hl,hl
	ld	de,_digInDescr
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-18),hl
	ex	de,hl
	ld	(ix-20),hl
	ex	de,hl
	.debug	# 630
	ld	a,e
	or	a,d
	or	a,c
	jr	z,@388
	ld	h,b
	ld	l,c
	ld	bc,hl
	push	bc
	push	de
	ld	de,word (phys @1+3238)
	ld	c,phys @1+3238 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
@388:
@386:
	.debug	>
@384:
	.debug	# 632
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	ld	a,0x1
	call	__lshl##
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
@382:
	.debug	>
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jp	@378
@379:
@377:
	.debug	>
@375:
	.debug	# 635
	ld	a,(ix+5)
	or	a,a
	jr	z,@389
	ld	de,word (phys @1+3244)
	ld	c,phys @1+3244 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
@389:
	.debug	# 637
	ld	a,(ix+5)
	or	a,a
	jr	nz,@390
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	or	hl,de
	jp	z,@391
@390:
@392:
	.debug	< @392 @393
	.debug	# 638
	ld	de,0x1
	ld	c,d
	ld	b,d
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
	ld	(ix-2),hl
@394:
	ld	hl,(ix-2)
	ld	de,(_NumApplDOUT)
	or	a,a
	sbc	hl,de
	jp	nc,@395
@397:
	.debug	< @397 @398
	.debug	# 639
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	nz,@399
	ld	a,(ix+5)
	or	a,a
	jr	z,@400
@399:
@401:
	.debug	< @401 @402
	.debug	# 640
	ld	hl,(ix-2)
	add	hl,hl
	ld	(ix-4),hl
	ld	de,(_curState+4)
	ld	bc,(_curState+6)
	push	bc
	push	de
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@403
	ld	hl,(ix-4)
	inc	hl
	ld	(ix-4),hl
@403:
	.debug	# 641
	ld	hl,(ix-4)
	add	hl,hl
	add	hl,hl
	ld	de,_digOutDescr
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-18),hl
	ex	de,hl
	ld	(ix-20),hl
	ex	de,hl
	.debug	# 642
	ld	a,e
	or	a,d
	or	a,c
	jr	z,@404
	ld	h,b
	ld	l,c
	ld	bc,hl
	push	bc
	push	de
	ld	de,word (phys @1+3238)
	ld	c,phys @1+3238 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
@404:
@402:
	.debug	>
@400:
	.debug	# 644
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	ld	a,0x1
	call	__lshl##
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
@398:
	.debug	>
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jp	@394
@395:
@393:
	.debug	>
@391:
	.debug	# 647
	ld	a,(ix+5)
	or	a,a
	jr	z,@405
	ld	de,word (phys @1+2863)
	ld	c,phys @1+2863 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
@405:
@371:
	.debug	} @371 648

@370	equ	20
	ld	sp,ix
	pop	ix
	lret
@372	equ	22
	.align	4
$$EVENT_RestoredParamFile::
	ex	(sp),iy
$EVENT_RestoredParamFile::
	.debug	G $EVENT_RestoredParamFile 101 9
	.debug	{ $EVENT_RestoredParamFile 101 654 @408 @407
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-4
	.debug	A event 5 102
	.debug	L idx -2 7
	.debug	L paramFile -4 61
	.debug	# 660
	ld	hl,(ix+5)
	ld	de,0x6
	add	hl,de
	ld	hl,(hl)
	ld	(ix-2),hl
	.debug	# 661
	add	hl,hl
	ld	de,_paramFiles
	add	hl,de
	ld	hl,(hl)
	ld	(ix-4),hl
	.debug	# 663
	ld	hl,(hl)
	push	hl
	ld	de,word (phys @1+3290)
	ld	c,phys @1+3290 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,8
	.debug	# 665
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-4)
	push	hl
	ld	hl,0x100
	push	hl
	ld	hl,_applBuf
	push	hl
	lcall	$loadParams
	add	sp,8
@407:
	.debug	} @407 666

@406	equ	4
	ld	sp,ix
	pop	ix
	lret
@408	equ	6
	.align	4
$$hostNameRestore::
	ex	(sp),iy
$hostNameRestore::
	.debug	G $hostNameRestore 104 9
	.debug	{ $hostNameRestore 104 671 @411 @410
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A curVal 5 105
	.debug	A newVal 9 70
	.debug	A count 11 7
	.debug	# 673
	ld	hl,(ix+9)
	push	hl
	call	_strlen
	pop	de
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$SetHostName
	add	sp,4
@410:
	.debug	} @410 674

@409	equ	0
	pop	ix
	lret
@411	equ	2
	.align	4
$$ReadSetCalibsRestore::
	ex	(sp),iy
$ReadSetCalibsRestore::
	.debug	G $ReadSetCalibsRestore 104 9
	.debug	{ $ReadSetCalibsRestore 104 679 @414 @413
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-2
	.debug	A curVal 5 105
	.debug	A newVal 9 70
	.debug	A count 11 7
	.debug	L ui -2 7
	.debug	# 684
	call	_KLOCK
	ld	hl,(ix+11)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix+9)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
	call	_KUNLOCK
@415:
	.debug	< @415 @416
	.debug	# 687
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@417:
	ld	hl,(ix-2)
	ld	de,(_NumHdwADC)
	or	a,a
	sbc	hl,de
	jr	nc,@418
	ld	hl,(ix-2)
	push	hl
	lcall	$newADCCalib
	pop	hl
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@417
@418:
@416:
	.debug	>
@420:
	.debug	< @420 @421
	.debug	# 688
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@422:
	ld	hl,(ix-2)
	ld	de,(_NumHdwDAC)
	or	a,a
	sbc	hl,de
	jr	nc,@423
	ld	hl,(ix-2)
	push	hl
	lcall	$newDACCalib
	pop	hl
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@422
@423:
@421:
	.debug	>
@413:
	.debug	} @413 689

@412	equ	2
	ld	sp,ix
	pop	ix
	lret
@414	equ	4
$setApplDTM::
	.debug	G $setApplDTM 97 9
	.debug	{ $setApplDTM 97 694 @427 @426
	.debug	# 696
	ld	hl,@2+93
	push	hl
	ld	hl,@2+102
	push	hl
	ld	de,word (phys @1+3322)
	ld	c,phys @1+3322 >> 16
	push	bc
	push	de
	ld	de,word (phys _applDTM)
	ld	c,phys _applDTM >> 16
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,12
	.debug	# 698
	ld	hl,@2+93
	push	hl
	ld	hl,@2+102
	push	hl
	lcall	$compileTime
	add	sp,4
	ld	(_applVersTime),de
	ld	(_applVersTime+2),bc
@426:
	.debug	} @426 699

@425	equ	0
	lret
@427	equ	0
$saveApplState::
	.debug	G $saveApplState 106 9
	.debug	{ $saveApplState 106 709 @430 @429
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-10
	.debug	L dataSize -2 7
	.debug	L ptr -6 48
	.debug	L hdr -10 107
	.debug	# 716
	ld	de,(_stateDataBuf)
	ld	bc,(_stateDataBuf+2)
	ld	a,e
	or	a,d
	or	a,c
	jr	nz,@431
	ld	hl,0xffff
	jp	@429
@431:
	.debug	# 718
	call	_KLOCK
	.debug	# 720
	ld	de,word (phys @1+3328)
	ld	c,phys @1+3328 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,6
	.debug	# 722
	ld	hl,(_ApplStateDataSize)
	ld	(ix-2),hl
	.debug	# 724
	ld	de,(_stateDataBuf)
	ld	bc,(_stateDataBuf+2)
	ld	hl,bc
	ld	(ix-8),hl
	ex	de,hl
	ld	(ix-10),hl
	ex	de,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	.debug	# 726
	ld	de,0x8
	ld	c,d
	ld	b,d
	push	bc
	push	de
	ld	hl,_prevState
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	.debug	# 727
	ld	de,0x8
	ld	c,d
	ld	b,d
	push	bc
	push	de
	ld	hl,_curState
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	.debug	# 728
	ld	de,0x8
	ld	c,d
	ld	b,d
	push	bc
	push	de
	ld	hl,_nextState
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	.debug	# 730
	ld	de,0x4
	ld	c,d
	ld	b,d
	push	bc
	push	de
	ld	hl,_setVal
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	.debug	# 732
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	.debug	# 734
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	ld	hl,(ix-2)
	call	__farst2##
	.debug	# 735
	ld	hl,(ix-2)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	ld	b,0
	push	bc
	push	de
	lcall	$memCRC16
	add	sp,8
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	ld	hl,(ix-10)
	ex	de,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farst2##
	.debug	# 736
	bool	hl
	ld	l,h
	push	hl
	lcall	$time
	pop	hl
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farst4p##
	.debug	# 743
	call	_KUNLOCK
	.debug	# 745
	bool	hl
	ld	l,h
@429:
	.debug	} @429 746

@428	equ	10
	ld	sp,ix
	pop	ix
	lret
@430	equ	12
$restoreApplState::
	.debug	G $restoreApplState 106 9
	.debug	{ $restoreApplState 106 764 @434 @433
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-10
	.debug	L dataSize -2 7
	.debug	L ptr -6 48
	.debug	L hdr -10 107
	.debug	# 771
	ld	de,(_stateDataBuf)
	ld	bc,(_stateDataBuf+2)
	ld	a,e
	or	a,d
	or	a,c
	jr	nz,@435
	ld	hl,0xffff
	jp	@433
@435:
	.debug	# 773
	ld	hl,(_ApplStateDataSize)
	ld	(ix-2),hl
	ld	de,(_stateDataBuf)
	ld	bc,(_stateDataBuf+2)
	ld	hl,bc
	ld	(ix-8),hl
	ex	de,hl
	ld	(ix-10),hl
	ex	de,hl
	.debug	# 777
	call	__farld2##
	ld	hl,(ix-2)
	or	a,a
	sbc	hl,de
	jr	z,@436
	ld	hl,0x1
	jp	@433
@436:
	.debug	# 778
	bool	hl
	ld	l,h
	push	hl
	lcall	$time
	pop	hl
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	call	__lsub##
	push	bc
	push	de
	ld	de,0x3c
	ld	c,d
	ld	b,d
	call	__lsub##
	jr	c,@437
	jr	z,@437
	ld	hl,0x2
	jp	@433
@437:
	.debug	# 779
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	.debug	# 780
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	push	de
	ld	hl,(ix-2)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	ld	b,0
	push	bc
	push	de
	lcall	$memCRC16
	add	sp,8
	pop	de
	or	a,a
	sbc	hl,de
	jr	z,@438
	ld	hl,0x3
	jp	@433
@438:
	.debug	# 782
	ld	de,0x8
	ld	c,d
	ld	b,d
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,_prevState
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	.debug	# 783
	ld	de,0x8
	ld	c,d
	ld	b,d
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,_curState
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	.debug	# 784
	ld	de,0x8
	ld	c,d
	ld	b,d
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,_nextState
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	.debug	# 786
	ld	de,0x4
	ld	c,d
	ld	b,d
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,_setVal
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	.debug	# 795
	ld	de,(_digitalIn)
	ld	bc,(_digitalIn+2)
	ld	(_nextState),de
	ld	(_nextState+2),bc
	ld	(_curState),de
	ld	(_curState+2),bc
	.debug	# 796
	ld	de,(_curState+4)
	ld	bc,(_curState+6)
	ld	(_digitalOut),de
	ld	(_digitalOut+2),bc
	.debug	# 798
	ld	de,word (phys @1+3355)
	ld	c,phys @1+3355 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	.debug	# 800
	bool	hl
	ld	l,h
@433:
	.debug	} @433 801

@432	equ	10
	ld	sp,ix
	pop	ix
	lret
@434	equ	12
$setGenericLabels::
	.debug	G $setGenericLabels 97 9
	.debug	{ $setGenericLabels 97 806 @441 @440
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-2
	.debug	L ui -2 7
@442:
	.debug	< @442 @443
	.debug	# 810
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@444:
	ld	hl,(ix-2)
	ld	de,(_NumApplAIN)
	or	a,a
	sbc	hl,de
	jr	nc,@445
@447:
	.debug	< @447 @448
	.debug	# 811
	ld	de,0x18
	ld	c,d
	ld	b,d
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-2)
	ex	de,hl
	push	bc
	push	de
	ld	d,h
	ld	e,l
	call	__ulmlt##
	push	bc
	push	de
	ld	de,word (phys _ainLabel)
	ld	c,phys _ainLabel >> 16
	call	__ladd##
	push	bc
	push	de
	call	_fmemset
	add	sp,10
	.debug	# 812
	ld	hl,(ix-2)
	push	hl
	ld	de,word (phys @1+3395)
	ld	c,phys @1+3395 >> 16
	push	bc
	push	de
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	de,0x18
	call	__ulmlt##
	push	bc
	push	de
	ld	de,word (phys _ainLabel)
	ld	c,phys _ainLabel >> 16
	call	__ladd##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,10
@448:
	.debug	>
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@444
@445:
@443:
	.debug	>
@449:
	.debug	< @449 @450
	.debug	# 814
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@451:
	ld	hl,(ix-2)
	ld	de,(_NumApplAOUT)
	or	a,a
	sbc	hl,de
	jr	nc,@452
@454:
	.debug	< @454 @455
	.debug	# 815
	ld	de,0x18
	ld	c,d
	ld	b,d
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-2)
	ex	de,hl
	push	bc
	push	de
	ld	d,h
	ld	e,l
	call	__ulmlt##
	push	bc
	push	de
	ld	de,word (phys _aoutLabel)
	ld	c,phys _aoutLabel >> 16
	call	__ladd##
	push	bc
	push	de
	call	_fmemset
	add	sp,10
	.debug	# 816
	ld	hl,(ix-2)
	push	hl
	ld	de,word (phys @1+3402)
	ld	c,phys @1+3402 >> 16
	push	bc
	push	de
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	de,0x18
	call	__ulmlt##
	push	bc
	push	de
	ld	de,word (phys _aoutLabel)
	ld	c,phys _aoutLabel >> 16
	call	__ladd##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,10
@455:
	.debug	>
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@451
@452:
@450:
	.debug	>
@456:
	.debug	< @456 @457
	.debug	# 819
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@458:
	ld	hl,(ix-2)
	ld	de,(_NumApplDIN)
	or	a,a
	sbc	hl,de
	jr	nc,@459
@461:
	.debug	< @461 @462
	.debug	# 820
	ld	de,0x18
	ld	c,d
	ld	b,d
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-2)
	ex	de,hl
	push	bc
	push	de
	ld	d,h
	ld	e,l
	call	__ulmlt##
	push	bc
	push	de
	ld	de,word (phys _dinLabel)
	ld	c,phys _dinLabel >> 16
	call	__ladd##
	push	bc
	push	de
	call	_fmemset
	add	sp,10
	.debug	# 821
	ld	hl,(ix-2)
	push	hl
	ld	de,word (phys @1+3410)
	ld	c,phys @1+3410 >> 16
	push	bc
	push	de
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	de,0x18
	call	__ulmlt##
	push	bc
	push	de
	ld	de,word (phys _dinLabel)
	ld	c,phys _dinLabel >> 16
	call	__ladd##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,10
@462:
	.debug	>
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@458
@459:
@457:
	.debug	>
@463:
	.debug	< @463 @464
	.debug	# 823
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@465:
	ld	hl,(ix-2)
	ld	de,(_NumApplDOUT)
	or	a,a
	sbc	hl,de
	jr	nc,@466
@468:
	.debug	< @468 @469
	.debug	# 824
	ld	de,0x18
	ld	c,d
	ld	b,d
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-2)
	ex	de,hl
	push	bc
	push	de
	ld	d,h
	ld	e,l
	call	__ulmlt##
	push	bc
	push	de
	ld	de,word (phys _doutLabel)
	ld	c,phys _doutLabel >> 16
	call	__ladd##
	push	bc
	push	de
	call	_fmemset
	add	sp,10
	.debug	# 825
	ld	hl,(ix-2)
	push	hl
	ld	de,word (phys @1+3417)
	ld	c,phys @1+3417 >> 16
	push	bc
	push	de
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	de,0x18
	call	__ulmlt##
	push	bc
	push	de
	ld	de,word (phys _doutLabel)
	ld	c,phys _doutLabel >> 16
	call	__ladd##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,10
@469:
	.debug	>
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@465
@466:
@464:
	.debug	>
@470:
	.debug	< @470 @471
	.debug	# 827
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@472:
	ld	hl,(ix-2)
	ld	de,(_NumExtraIn)
	or	a,a
	sbc	hl,de
	jr	nc,@473
@475:
	.debug	< @475 @476
	.debug	# 828
	ld	de,0x18
	ld	c,d
	ld	b,d
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-2)
	ex	de,hl
	push	bc
	push	de
	ld	d,h
	ld	e,l
	call	__ulmlt##
	push	bc
	push	de
	ld	de,word (phys _extraInLabel)
	ld	c,phys _extraInLabel >> 16
	call	__ladd##
	push	bc
	push	de
	call	_fmemset
	add	sp,10
	.debug	# 829
	ld	hl,(ix-2)
	push	hl
	ld	de,word (phys @1+3425)
	ld	c,phys @1+3425 >> 16
	push	bc
	push	de
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	de,0x18
	call	__ulmlt##
	push	bc
	push	de
	ld	de,word (phys _extraInLabel)
	ld	c,phys _extraInLabel >> 16
	call	__ladd##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,10
@476:
	.debug	>
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@472
@473:
@471:
	.debug	>
@440:
	.debug	} @440 831

@439	equ	2
	ld	sp,ix
	pop	ix
	lret
@441	equ	4
	.debug	I
;3129 
;3130 
;3131 //=========================================================================
;3132 //  Misc Application-specific functions
;3133 //=========================================================================
;3134 
;3135 
;3136 //-------------------------------------------------------------------------
;3137 //  phaseDiff:  compute difference between 2 phases, returning a value betw
;3138 //  -180.0 and +180.0
;3139 //-------------------------------------------------------------------------
;3140 float  phaseDiff(float ph1, float ph2)
;3141 {
$phaseDiff::
	.debug	G $phaseDiff 109 9
	.debug	{ $phaseDiff 109 3140 @479 @478
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-4
	.debug	A ph1 5 10
	.debug	A ph2 9 10
;3142   float diff = ph1 - ph2;
	.debug	L diff -4 10
	.debug	# 3142
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	call	__fsub##
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
	.seg	FARCODE
;3143 
;3144   if (diff < -180.0)
	.debug	# 3144
	push	bc
	push	de
	ld	de,0
	ld	bc,0xc334
	call	__fsub##
	jp	p,@480
;3145     diff += 360.0;
	.debug	# 3145
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0
	ld	bc,0x43b4
	call	__fadd##
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
;3146   else
;3147     if (diff > 180.0)  diff -= 360.0;
	jr	@481
@480:
	.debug	# 3147
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0
	ld	bc,0x4334
	call	__fsub##
	jp	m,@482
	jr	z,@482
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0
	ld	bc,0xc3b4
	call	__fadd##
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
;3148 
;3149   return diff;
@482:
@481:
	.debug	# 3149
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
;3150 }
@478:
	.debug	} @478 3150

@477	equ	4
	ld	sp,ix
	pop	ix
	lret
@479	equ	6
;3151 
;3152 
;3153 /*
;3154 //-------------------------------------------------------------------------
;3155 //  "fptofl": Converts a decimal number in fixed point format to floating p
;3156 //-------------------------------------------------------------------------
;3157 float fptofl(long value)
;3158 {
;3159   float out;
;3160   bool neg = value < 0;
;3161   ulong *p = (ulong *)&value;
;3162 
;3163   if (neg)  value *= -1;
;3164 
;3165 //finish
;3166   out = (*p) & 0x0000FFFFL;
;3167   out /= 65536.0;
;3168   out += (float) ((*p) >> 16);
;3169 
;3170   if (neg)  out *= -1.0;
;3171 
;3172   return out;
;3173 }
;3174 
;3175 //-------------------------------------------------------------------------
;3176 //  "fltofp": Converts a decimal number in floating point format to fixed p
;3177 //-------------------------------------------------------------------------
;3178 long fltofp(float value)
;3179 {
;3180   long fixed;
;3181   bool neg = (value < 0.0);
;3182 
;3183   if (neg)  value *= -1.0;
;3184 
;3185 //finish
;3186   fixed = (long) value;
;3187   value -= (float) fixed;
;3188   fixed <<= 16;
;3189   fixed += (long) (65536.0 * value);
;3190 
;3191   if (neg)  fixed *= -1;
;3192 
;3193   return fixed;
;3194 }
;3195 */
;3196 
;3197 //=========================================================================
;3198 typedef struct {
;3199   fd_t   fromfd;
;3200   bool   eof;
;3201 }  ReadState;
;3202 
;3203 #define RDSTATE ((ReadState *)rdState)
;3204 
;3205 typedef struct {
;3206   uint   progNum;     // 0 - 3
;3207   uint   blkNum;      // 0 - 15
;3208   ulong  ttlWritten;  // ttl bytes written to DSP mem for this program
;3209   ulong  blkWritten;  // # bytes in current block written so far
;3210   bool   showProgress;
;3211 }  WriteState;
;3212 
;3213 #define WRSTATE ((WriteState *)wrState)
;3214 
;3215 #define  DSP_BLK_SIZE  (0x10000L)
;3216 
;3217 
;3218 //-------------------------------------------------------------------------
;3219 //  Read the sepcified # of bytes from a file descriptor.  If the end of th
;3220 //  file is reached, bytes with 0 value are returned instead.
;3221 //
;3222 //  Parameters:
;3223 //    *rdState:  Pointer to ReadState structure that contains the following
;3224 //                 fromfd:  the file descriptor to read from
;3225 //                 eof:  flag to indicate that the End-Of-File was reached
;3226 //    *buf:      pointer to buffer to read the data in to
;3227 //    count:     # of bytes to read (or fill with zeros)
;3228 //
;3229 //  This function is designed to be called by the copyBytes() function whic
;3230 //  provides a generic means of managing the transfer of a stream of bytes.
;3231 //
;3232 //  On return:
;3233 //    rdState values updated as needed
;3234 //    Returns the # of bytes successfully read or set to 0
;3235 //-------------------------------------------------------------------------
;3236 int _nearcall readFromFile(void *rdState, char *buf, int count)
;3237 {
	.align	4
$$readFromFile::
	ex	(sp),iy
$readFromFile::
	.debug	G $readFromFile 110 9
	.debug	{ $readFromFile 110 3236 @485 @484
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-2
	.debug	A rdState 5 78
	.debug	A buf 7 70
	.debug	A count 9 6
;3238   int  stat;
	.debug	L stat -2 6
;3239 
;3240   if (RDSTATE->eof)  { memset(buf, 0xff, count);  return count; }
	.debug	# 3240
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	ld	a,(hl)
	or	a,a
	jr	z,@486
@487:
	.debug	< @487 @488
	ld	hl,(ix+9)
	push	hl
	ld	hl,0xff
	push	hl
	ld	hl,(ix+7)
	push	hl
	call	_memset
	add	sp,6
	ld	hl,(ix+9)
	jr	@484
@488:
	.debug	>
;3241 
;3242   if ((stat = read(RDSTATE->fromfd, buf, count)) < 0)  return stat;
@486:
	.debug	# 3242
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	ld	hl,(hl)
	push	hl
	lcall	$read
	add	sp,6
	ld	(ix-2),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@489
	ld	hl,(ix-2)
	jr	@484
;3243 
;3244   if (stat < count)  { memset(buf+stat, 0, count-stat);  RDSTATE->eof = tru
@489:
	.debug	# 3244
	ld	hl,(ix-2)
	ex	de,hl
	ld	hl,(ix+9)
	ex	de,hl
	call	__icmp##
	jr	nc,@490
@491:
	.debug	< @491 @492
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix-2)
	ex	de,hl
	or	a,a
	sbc	hl,de
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix+7)
	ex	de,hl
	add	hl,de
	push	hl
	call	_memset
	add	sp,6
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	ld	a,0x1
	ld	(hl),a
@492:
	.debug	>
;3245 
;3246   return count;
@490:
	.debug	# 3246
	ld	hl,(ix+9)
;3247 }
@484:
	.debug	} @484 3247

@483	equ	2
	ld	sp,ix
	pop	ix
	lret
@485	equ	4
;3248 
;3249 //-------------------------------------------------------------------------
;3250 //  initialize various default settings
;3251 //-------------------------------------------------------------------------
;3252 void initDefaults(void)
;3253 {
$initDefaults::
	.debug	G $initDefaults 97 9
	.debug	{ $initDefaults 97 3252 @495 @494
;3254   //finish
;3255 }
@494:
	.debug	} @494 3255

@493	equ	0
	lret
@495	equ	0
;3256 
;3257 
;3258 //=========================================================================
;3259 //  Event-Handler functions called by the appl task to process event messag
;3260 //=========================================================================
;3261 
;3262 //-------------------------------------------------------------------------
;3263 //  Event-Handler for a request to ...
;3264 //-------------------------------------------------------------------------
;3265 /*
;3266 void EVENT_MyEvent(EventMsg *event)
;3267 {
;3268   uint  idx;
;3269   float  setpoint;
;3270   char  temp[16];
;3271 
;3272 
;3273   idx = event->chanNum;
;3274 
;3275   if (error)  {
;3276     fprintf(DiagLog, "error msg, chan:\n", idx+1);
;3277     return;
;3278   }
;3279 
;3280   setpoint = *((float *)event->data);
;3281 
;3282   fprintf(DiagLog, "New SetPoint for PS%d: %s\n",
;3283      idx+1, fmtFloat(setpoint, 8,3, temp) );
;3284 
;3285   *((float *)((&iSet_1_RegInfo + idx)->regVal)) = setpoint;
;3286 
;3287   anaOut[idx] = ISETtoVolts(idx, setpoint);
;3288 }
;3289 */
;3290 
;3291 
;3292 
;3293 //=========================================================================
;3294 //  Task functions to handle application-specific commands
;3295 //=========================================================================
;3296 
;3297 
;3298 //-------------------------------------------------------------------------
;3299 //  example func to process a command entered via a user shell
;3300 //-------------------------------------------------------------------------
;3301 /*
;3302 void _task miscTest(int argc, char **argv)
;3303 {
;3304   int  idx;
;3305   bool  validCmd;
;3306   // used by EVENT macro
;3307   fd_t  eventsfd = -1;
;3308   EventMsg  eventMsg;
;3309 
;3310 
;3311   validCmd = false;
;3312 
;3313   for (;;)  {
;3314     if (argc < 3)  break;
;3315     if (! isdigit(argv[1][0]))  break;
;3316 
;3317     idx = atoi(argv[1]);
;3318     if ((idx < 1) OR (idx > NumChan))  {
;3319       printf("Invalid chan #\n");  Exit(-2);  }
;3320 
;3321     --idx;  validCmd = true;
;3322 
;3323     if (StriEq(argv[2], "on"))     { EVENT(TurnOn,  idx, NULL, 0);  break; 
;3324     if (StriEq(argv[2], "off"))    { EVENT(TurnOff, idx, NULL, 0);  break; 
;3325     if (StriEq(argv[2], "reset"))  { EVENT(Reset,   idx, NULL, 0);  break; 
;3326 
;3327     validCmd = false;  break;
;3328   }
;3329 
;3330   if (! validCmd)  { printf("Usage:  misc # on|off|reset\r\n");  Exit(-1); 
;3331 
;3332   Exit(0);
;3333 }
;3334 */
;3335 
;3336 
;3337 //-------------------------------------------------------------------------
;3338 //  auto-calibrate the fwd, rev, or cav atten value by cycling through
;3339 //  the range of atten values for a given pair of V-peak and phase values.
;3340 //-------------------------------------------------------------------------
;3341 void _task autocal(int argc, char **argv)
;3342 {
	.align	4
$$autocal::
	ex	(sp),iy
$autocal::
	.debug	G $autocal 111 9
	.debug	{ $autocal 111 3341 @498 @497
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-21
	.debug	A argc 5 6
	.debug	A argv 7 112
;3343   U16  u, curAtten;
	.debug	L u -2 5
	.debug	L curAtten -4 5
;3344   F32  vPeak, phase;
	.debug	L vPeak -8 10
	.debug	L phase -12 10
;3345   byte  flags[3];
	.debug	L flags -15 113
;3346   RegInfo  *reg[3];
	.debug	L reg -21 114
;3347 
;3348 
;3349   if (showHelp(4, argc, argv))  {
	.debug	# 3349
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,0x4
	push	hl
	lcall	$showHelp
	add	sp,6
	or	a,a
	jr	z,@499
;3350     printf("Usage:  autocal fwd|rev|cav <V-peak> <Phase>\r\n");
@500:
	.debug	< @500 @501
	.debug	# 3350
	ld	de,word (phys @1+3436)
	ld	c,phys @1+3436 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3351     Exit(-1);
	.debug	# 3351
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
;3352   }
@501:
	.debug	>
;3353 
;3354   reg[0] = NULL;
@499:
	.debug	# 3354
	bool	hl
	ld	l,h
	ld	(ix-21),hl
;3355 
;3356   if (StriEq(argv[1], "fwd"))  {
	.debug	# 3356
	ld	hl,@2+118
	push	hl
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@502
;3357     curAtten = fwdAtten;
@503:
	.debug	< @503 @504
	.debug	# 3357
	ld	hl,(_fwdAtten)
	ld	(ix-4),hl
;3358     reg[0] = &fwdAtten_RegInfo;
	.debug	# 3358
	ld	hl,_fwdAtten_RegInfo
	ld	(ix-21),hl
;3359     reg[1] = &fwdMagCalib_RegInfo;
	.debug	# 3359
	ld	hl,_fwdMagCalib_RegInfo
	ld	(ix-19),hl
;3360     reg[2] = &fwdPhaseCalib_RegInfo;
	.debug	# 3360
	ld	hl,_fwdPhaseCalib_RegInfo
	ld	(ix-17),hl
;3361   }  else
@504:
	.debug	>
;3362   if (StriEq(argv[1], "rev"))  {
	jr	@505
@502:
	.debug	# 3362
	ld	hl,@2+122
	push	hl
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@506
;3363     curAtten = revAtten;
@507:
	.debug	< @507 @508
	.debug	# 3363
	ld	hl,(_revAtten)
	ld	(ix-4),hl
;3364     reg[0] = &revAtten_RegInfo;
	.debug	# 3364
	ld	hl,_revAtten_RegInfo
	ld	(ix-21),hl
;3365     reg[1] = &revMagCalib_RegInfo;
	.debug	# 3365
	ld	hl,_revMagCalib_RegInfo
	ld	(ix-19),hl
;3366     reg[2] = &revPhaseCalib_RegInfo;
	.debug	# 3366
	ld	hl,_revPhaseCalib_RegInfo
	ld	(ix-17),hl
;3367   }  else
@508:
	.debug	>
;3368   if (StriEq(argv[1], "cav"))  {
	jr	@509
@506:
	.debug	# 3368
	ld	hl,@2+126
	push	hl
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@510
;3369     curAtten = cavAtten;
@511:
	.debug	< @511 @512
	.debug	# 3369
	ld	hl,(_cavAtten)
	ld	(ix-4),hl
;3370     reg[0] = &cavAtten_RegInfo;
	.debug	# 3370
	ld	hl,_cavAtten_RegInfo
	ld	(ix-21),hl
;3371     reg[1] = &cavMagCalib_RegInfo;
	.debug	# 3371
	ld	hl,_cavMagCalib_RegInfo
	ld	(ix-19),hl
;3372     reg[2] = &cavPhaseCalib_RegInfo;
	.debug	# 3372
	ld	hl,_cavPhaseCalib_RegInfo
	ld	(ix-17),hl
;3373   }
@512:
	.debug	>
;3374 
;3375   if (! reg[0])  {
@510:
@509:
@505:
	.debug	# 3375
	ld	hl,(ix-21)
	bool	hl
	jr	nz,@513
;3376     printf("Invalid argument: %.32s\r\n", argv[1]);  Exit(-1); }
@514:
	.debug	< @514 @515
	.debug	# 3376
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	de,word (phys @1+3483)
	ld	c,phys @1+3483 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
@515:
	.debug	>
;3377 
;3378   vPeak = atof(argv[2]);  phase = atof(argv[3]);
@513:
	.debug	# 3378
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix+7)
	ld	de,0x4
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_strtod
	add	sp,4
	ld	hl,bc
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix+7)
	ld	de,0x6
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_strtod
	add	sp,4
	ld	hl,bc
	ld	(ix-10),hl
	ex	de,hl
	ld	(ix-12),hl
	ex	de,hl
;3379 
;3380 //--- save the flags value for each register so we can restore it after ---
;3381 //--- temporarily changing it to avoid seeing the normal log messages on --
;3382 //--- the terminal screen ---
;3383   for (u=0; u<3; ++u)  flags[u] = reg[u]->flags;
@516:
	.debug	< @516 @517
	.debug	# 3383
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@518:
	ld	hl,(ix-2)
	ld	de,0x3
	or	a,a
	sbc	hl,de
	jr	nc,@519
	ld	hl,(ix-2)
	add	hl,hl
	ex	de,hl
	ld	hl,-21+@496
	add	hl,sp
	add	hl,de
	ld	hl,(hl)
	ld	de,0xd
	add	hl,de
	ex	de,hl
	ld	hl,(ix-2)
	ex	de,hl
	ld	bc,hl
	ld	hl,-15+@496
	add	hl,sp
	add	hl,de
	ld	a,(bc)
	ld	(hl),a
;3384 
;3385 //--- cycle the specified attenuation from 0 to 63, waiting 1 second after 
;3386 //--- each change and then telling the DSP the v-peak and phase values ---
;3387 //--- that it should associate with the current conditions ---
;3388 
;3389   for (u=0; u<64; ++u)  {
	ld	h,d
	ld	l,e
	inc	hl
	ld	(ix-2),hl
	jr	@518
@519:
@517:
	.debug	>
@521:
	.debug	< @521 @522
	.debug	# 3389
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@523:
	ld	hl,(ix-2)
	ld	de,0x40
	or	a,a
	sbc	hl,de
	jp	nc,@524
;3390     printf("%3d\r", u);  fflush(stdout);
@526:
	.debug	< @526 @527
	.debug	# 3390
	ld	hl,(ix-2)
	push	hl
	ld	de,word (phys @1+3509)
	ld	c,phys @1+3509 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
	ld	hl,(__stdio)
	push	hl
	lcall	$fflush
	pop	de
;3391 
;3392     reg[0]->flags &= ~(byte)LogChanges;
	.debug	# 3392
	ld	hl,(ix-21)
	ld	de,0xd
	add	hl,de
	res	1,(hl)
;3393     setRegU16(reg[0], 0, u);
	.debug	# 3393
	ld	hl,(ix-2)
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-21)
	push	hl
	lcall	$setRegU16
	add	sp,6
;3394     reg[0]->flags = flags[0];
	.debug	# 3394
	ld	hl,(ix-21)
	ld	de,0xd
	add	hl,de
	ld	a,(ix-15)
	ld	(hl),a
;3395 
;3396     sleepMS(1000);
	.debug	# 3396
	ld	de,0x3e8
	ld	bc,0
	push	bc
	push	de
	lcall	$sleepMS
	add	sp,4
;3397 
;3398     reg[1]->flags &= ~(byte)LogChanges;
	.debug	# 3398
	ld	hl,(ix-19)
	ld	de,0xd
	add	hl,de
	res	1,(hl)
;3399     setRegF32(reg[1], 0, vPeak);
	.debug	# 3399
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-19)
	push	hl
	lcall	$setRegF32
	add	sp,8
;3400     reg[1]->flags = flags[1];
	.debug	# 3400
	ld	hl,(ix-19)
	ld	de,0xd
	add	hl,de
	ld	a,(ix-14)
	ld	(hl),a
;3401 
;3402     reg[2]->flags &= ~(byte)LogChanges;
	.debug	# 3402
	ld	hl,(ix-17)
	add	hl,de
	res	1,(hl)
;3403     setRegF32(reg[2], 0, phase);
	.debug	# 3403
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-17)
	push	hl
	lcall	$setRegF32
	add	sp,8
;3404     reg[2]->flags = flags[2];
	.debug	# 3404
	ld	hl,(ix-17)
	ld	de,0xd
	add	hl,de
	ld	a,(ix-13)
	ld	(hl),a
;3405 
;3406     CheckETX();
	.debug	# 3406
	lcall	$CheckETX
;3407   }
@527:
	.debug	>
;3408   printf("\r\n");
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jp	@523
@524:
@522:
	.debug	>
	.debug	# 3408
	ld	de,word (phys @1+2863)
	ld	c,phys @1+2863 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3409 
;3410   reg[0]->flags &= ~(byte)LogChanges;
	.debug	# 3410
	ld	hl,(ix-21)
	ld	de,0xd
	add	hl,de
	res	1,(hl)
;3411   setRegU16(reg[0], 0, curAtten);
	.debug	# 3411
	ld	hl,(ix-4)
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-21)
	push	hl
	lcall	$setRegU16
	add	sp,6
;3412   reg[0]->flags = flags[0];
	.debug	# 3412
	ld	hl,(ix-21)
	ld	de,0xd
	add	hl,de
	ld	a,(ix-15)
	ld	(hl),a
;3413 
;3414 
;3415   Exit(0);
	.debug	# 3415
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
;3416 }
@497:
	.debug	} @497 3416

@496	equ	21
	ld	sp,ix
	pop	ix
	lret
@498	equ	23
;3417 
;3418 
;3419 //-------------------------------------------------------------------------
;3420 //  "cmd": set various DSP states
;3421 //-------------------------------------------------------------------------
;3422 void _task cmd(int argc, char **argv)
;3423 {
	.align	4
$$cmd::
	ex	(sp),iy
$cmd::
	.debug	G $cmd 111 9
	.debug	{ $cmd 111 3422 @530 @529
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-4
	.debug	A argc 5 6
	.debug	A argv 7 112
;3424   ulong  bits = 0;
	.debug	L bits -4 9
	.debug	# 3424
	ld	de,0
	ld	c,e
	ld	b,e
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
	.seg	FARCODE
;3425 
;3426 
;3427   if (showHelp(3, argc, argv))  {
	.debug	# 3427
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,0x3
	push	hl
	lcall	$showHelp
	add	sp,6
	or	a,a
	jr	z,@531
;3428     printf("Usage:  cmd amplCtl on|off\r\n"
@532:
	.debug	< @532 @533
;3429 //           "            phaseCtl off|sel|auto\r\n"
;3430 //           "            rfPulse on|off\r\n"
;3431            "            tunerCtl on|off\r\n"
;3432           );
	.debug	# 3428
	ld	de,word (phys @1+3514)
	ld	c,phys @1+3514 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3433     Exit(-1);
	.debug	# 3433
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
;3434   }
@533:
	.debug	>
;3435 
;3436 //finish
;3437   do {
@531:
;3438     if (StriEq(argv[1], "amplCtl"))  {
@537:
	.debug	< @537 @538
	.debug	# 3438
	ld	hl,@2+130
	push	hl
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@539
;3439       if (StriEq(argv[2], "on"))  { bits = CMD_AMPL_CTL_ON;  break; }
@540:
	.debug	< @540 @541
	.debug	# 3439
	ld	hl,@2+138
	push	hl
	ld	hl,(ix+7)
	ld	de,0x4
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@542
@543:
	.debug	< @543 @544
	ld	de,0x8000
	ld	b,e
	jr	@1122
@544:
	.debug	>
;3440       if (StriEq(argv[2], "off"))  { bits = CMD_AMPL_CTL_OFF;  break; }
@542:
	.debug	# 3440
	ld	hl,@2+141
	push	hl
	ld	hl,(ix+7)
	ld	de,0x4
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@545
@546:
	.debug	< @546 @547
	ld	de,0x4000
	ld	b,e
@1122:
	ld	l,e
	jr	@1121
@547:
	.debug	>
;3441     }
@545:
@541:
	.debug	>
;3442 
;3443 /*  //finish
;3444     if (StriEq(argv[1], "phaseCtl"))  {
;3445       if (StriEq(argv[2], "off"))  { bits = CMD_PHASE_CTL_OFF;  break; }
;3446       if (StriEq(argv[2], "sel"))  { bits = CMD_PHASE_CTL_SEL;  break; }
;3447       if (StriEq(argv[2], "auto"))  { bits = CMD_PHASE_CTL_AUTO;  break; }
;3448     }
;3449 
;3450     if (StriEq(argv[1], "rfPulse"))  {
;3451       if (StriEq(argv[2], "on"))  { bits = CMD_RF_PULSING_ON;  break; }
;3452       if (StriEq(argv[2], "off"))  { bits = CMD_RF_PULSING_OFF;  break; }
;3453     }
;3454 */
;3455     if (StriEq(argv[1], "tunerCtl"))  {
@539:
	.debug	# 3455
	ld	hl,@2+145
	push	hl
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@548
;3456       if (StriEq(argv[2], "on"))  { bits = CMD_TUNER_CTL_ON;  break; }
@549:
	.debug	< @549 @550
	.debug	# 3456
	ld	hl,@2+138
	push	hl
	ld	hl,(ix+7)
	ld	de,0x4
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@551
@552:
	.debug	< @552 @553
	ld	d,h
	ld	e,h
	ld	hl,0x20
@1121:
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
	jr	@535
@553:
	.debug	>
;3457       if (StriEq(argv[2], "off"))  { bits = CMD_TUNER_CTL_OFF;  break; }
@551:
	.debug	# 3457
	ld	hl,@2+141
	push	hl
	ld	hl,(ix+7)
	ld	de,0x4
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@554
@555:
	.debug	< @555 @556
	ld	d,h
	ld	e,h
	ld	hl,0x10
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
@556:
	.debug	>
;3458     }
@554:
@550:
	.debug	>
;3459 
;3460   } while (false);
@548:
@538:
	.debug	>
	.debug	# 3460
;3461 
;3462 
;3463   if (bits)
@535:
	.debug	# 3463
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	or	hl,de
	jr	z,@557
;3464     SetMBReg(setDSPState, 0, U32, bits);
	.debug	# 3464
	ld	h,b
	ld	l,c
	ld	bc,hl
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_setDSPState_RegInfo
	push	hl
	lcall	$setRegU32
	add	sp,8
;3465   else
;3466     printf("Invalid parameters\r\n");
	jr	@558
@557:
	.debug	# 3466
	ld	de,word (phys @1+3572)
	ld	c,phys @1+3572 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3467 
;3468   Exit(0);
@558:
	.debug	# 3468
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
;3469 }
@529:
	.debug	} @529 3469

@528	equ	4
	ld	sp,ix
	pop	ix
	lret
@530	equ	6
;3470 
;3471 
;3472 
;3473 //-------------------------------------------------------------------------
;3474 //  "defaults": initialize FPGA settings to default values
;3475 //-------------------------------------------------------------------------
;3476 void _task defaults(int argc, char **argv)
;3477 {
	.align	4
$$defaults::
	ex	(sp),iy
$defaults::
	.debug	G $defaults 111 9
	.debug	{ $defaults 111 3476 @561 @560
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A argc 5 6
	.debug	A argv 7 112
;3478   initDefaults();  Exit(0);
	.debug	# 3478
	lcall	$initDefaults
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
;3479 }
@560:
	.debug	} @560 3479

@559	equ	0
	pop	ix
	lret
@561	equ	2
;3480 
;3481 
;3482 
;3483 //-------------------------------------------------------------------------
;3484 //  Reset the DSP and FPGA, causing them to reload their firmware from EEPR
;3485 //-------------------------------------------------------------------------
;3486 void _task reset_DSP_FPGA(int argc, char **argv)
;3487 {
	.align	4
$$reset_DSP_FPGA::
	ex	(sp),iy
$reset_DSP_FPGA::
	.debug	G $reset_DSP_FPGA 111 9
	.debug	{ $reset_DSP_FPGA 111 3486 @564 @563
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-2
	.debug	A argc 5 6
	.debug	A argv 7 112
;3488   int  stat;
	.debug	L stat -2 6
;3489 
;3490   if (getLock(APPL_LOCK, 10000) <= 0)  Exit(-1);
	.debug	# 3490
	ld	de,0x2710
	ld	bc,0
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	lcall	$getLock
	add	sp,6
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	z,@566
	jr	nc,@565
@566:
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
;3491 
;3492   stop_DSP_FPGA();  stat = restart_DSP_FPGA(10000);
@565:
	.debug	# 3492
	lcall	$stop_DSP_FPGA
	ld	de,0x2710
	ld	bc,0
	push	bc
	push	de
	lcall	$restart_DSP_FPGA
	add	sp,4
	ld	(ix-2),hl
;3493 
;3494   releaseLock(APPL_LOCK);
	.debug	# 3494
	bool	hl
	ld	l,h
	push	hl
	lcall	$releaseLock
	pop	de
;3495 
;3496   Exit(stat);
	.debug	# 3496
	ld	hl,(ix-2)
	push	hl
	lcall	$Exit
	pop	hl
;3497 }
@563:
	.debug	} @563 3497

@562	equ	2
	ld	sp,ix
	pop	ix
	lret
@564	equ	4
;3498 
;3499 
;3500 //-------------------------------------------------------------------------
;3501 //  "setdspfirm": load a new DSP firmware file in to the serial EEPROM
;3502 //-------------------------------------------------------------------------
;3503 void _task setDSPfirm(int argc, char **argv)
;3504 {
	.align	4
$$setDSPfirm::
	ex	(sp),iy
$setDSPfirm::
	.debug	G $setDSPfirm 111 9
	.debug	{ $setDSPfirm 111 3503 @569 @568
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-11
	.debug	A argc 5 6
	.debug	A argv 7 112
;3505   int  i, fromfd;
	.debug	L i -2 6
	.debug	L fromfd -4 6
;3506   ulong  limit = 0x1F8000L;
	.debug	L limit -8 9
	.debug	# 3506
	ld	de,0x8000
	ld	hl,0x1f
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
	.seg	FARCODE
;3507   bool  verify = false;
	.debug	L verify -9 3
	.debug	# 3507
	xor	a,a
	ld	(ix-9),a
	.seg	FARCODE
;3508   char  *fileName;
	.debug	L fileName -11 70
;3509 
;3510 
;3511   if (showHelp(1, argc, argv))  {
	.debug	# 3511
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,0x1
	push	hl
	lcall	$showHelp
	add	sp,6
	or	a,a
	jr	z,@570
;3512     printf("\nusage:  setdspfirm [-v] [fileName] [-l<limit>]\r\n");
@571:
	.debug	< @571 @572
	.debug	# 3512
	ld	de,word (phys @1+3593)
	ld	c,phys @1+3593 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3513     Exit(-1);
	.debug	# 3513
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
;3514   }
@572:
	.debug	>
;3515 
;3516   fileName = DSP_Prog_File;
@570:
	.debug	# 3516
	ld	hl,@2+154
	ld	(ix-11),hl
;3517   for (i=1; i<argc; ++i)  {
@573:
	.debug	< @573 @574
	.debug	# 3517
	ld	hl,0x1
	ld	(ix-2),hl
@575:
	ld	hl,(ix-2)
	ex	de,hl
	ld	hl,(ix+5)
	ex	de,hl
	call	__icmp##
	jr	nc,@576
;3518     if (argv[i][0] != '-')  { fileName = argv[i];  continue; }
@578:
	.debug	< @578 @579
	.debug	# 3518
	ld	hl,(ix-2)
	add	hl,hl
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	add	hl,de
	ld	hl,(hl)
	ld	a,(hl)
	sub	a,45
	jr	z,@580
@581:
	.debug	< @581 @582
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,de
	ld	hl,(hl)
	ld	(ix-11),hl
	jr	@577
@582:
	.debug	>
;3519     if (StrEq(argv[i], "-v"))  verify = true;
@580:
	.debug	# 3519
	ld	hl,@2+172
	push	hl
	ld	hl,(ix-2)
	add	hl,hl
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_strcmp
	add	sp,4
	bool	hl
	jr	nz,@583
	ld	(ix-9),0x1
;3520     if (strncmp(argv[i], "-l", 2) == 0)  limit = ATOL(argv[i]+2);
@583:
	.debug	# 3520
	ld	hl,0x2
	push	hl
	ld	hl,@2+175
	push	hl
	ld	hl,(ix-2)
	add	hl,hl
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_strncmp
	add	sp,6
	bool	hl
	jr	nz,@584
	bool	hl
	ld	l,h
	push	hl
	push	hl
	ld	hl,(ix-2)
	add	hl,hl
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	add	hl,de
	ld	hl,(hl)
	inc	hl
	inc	hl
	push	hl
	lcall	$strtol
	add	sp,6
	ld	hl,bc
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
;3521   }
@584:
@579:
	.debug	>
;3522 
;3523   if ((fromfd = open(fileName, O_RDONLY, 0)) < 0)  {
@577:
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jp	@575
@576:
@574:
	.debug	>
	.debug	# 3523
	bool	hl
	ld	l,h
	push	hl
	inc	l
	push	hl
	ld	hl,(ix-11)
	push	hl
	lcall	$open
	add	sp,6
	ld	(ix-4),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@585
;3524     perror(fileName);  Exit(-1); }
@586:
	.debug	< @586 @587
	.debug	# 3524
	ld	hl,(ix-11)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
@587:
	.debug	>
;3525 
;3526   printf("\r\nDownloading %.40s to EEPROM\r\n", fileName);
@585:
	.debug	# 3526
	ld	hl,(ix-11)
	push	hl
	ld	de,word (phys @1+3643)
	ld	c,phys @1+3643 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;3527 
;3528   updateEEPROM(DSP_EEPROM, fromfd, limit, true, verify);
	.debug	# 3528
	bool	hl
	ld	l,(ix-9)
	push	hl
	ld	l,0x1
	push	hl
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-4)
	push	hl
	ld	hl,0x1
	push	hl
	lcall	$updateEEPROM
	add	sp,12
;3529 
;3530   close(fromfd);  Exit(0);
	.debug	# 3530
	ld	hl,(ix-4)
	push	hl
	lcall	$close
	pop	de
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
;3531 }
@568:
	.debug	} @568 3531

@567	equ	11
	ld	sp,ix
	pop	ix
	lret
@569	equ	13
;3532 
;3533 //-------------------------------------------------------------------------
;3534 //  "fpgaread": command to read one or more 8-bit values from the FPGA
;3535 //-------------------------------------------------------------------------
;3536 void _task fpgaRead(int argc, char **argv)
;3537 {
	.align	4
$$fpgaRead::
	ex	(sp),iy
$fpgaRead::
	.debug	G $fpgaRead 111 9
	.debug	{ $fpgaRead 111 3536 @590 @589
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-10
	.debug	A argc 5 6
	.debug	A argv 7 112
;3538   uint  u, regNum, count, maxCount;
	.debug	L u -2 7
	.debug	L regNum -4 7
	.debug	L count -6 7
	.debug	L maxCount -8 7
;3539   int  regVal;
	.debug	L regVal -10 6
;3540 
;3541 
;3542   if (showHelp(3, argc, argv))  {
	.debug	# 3542
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,0x3
	push	hl
	lcall	$showHelp
	add	sp,6
	or	a,a
	jr	z,@591
;3543     printf("Usage:  fpgaread <reg> <count>\r\n"
@592:
	.debug	< @592 @593
;3544            "                 auto on|off\r\n");
	.debug	# 3543
	ld	de,word (phys @1+3675)
	ld	c,phys @1+3675 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3545     Exit(-1);
	.debug	# 3545
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
;3546   }
@593:
	.debug	>
;3547 
;3548   if (StriEq(argv[1], "auto"))  {
@591:
	.debug	# 3548
	ld	hl,@2+178
	push	hl
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@594
;3549     autoReadFPGA = StriEq(argv[2], "on");  Exit(0); }
@595:
	.debug	< @595 @596
	.debug	# 3549
	ld	hl,@2+138
	push	hl
	ld	hl,(ix+7)
	ld	de,0x4
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	ld	l,h
	jr	nz,@597
	inc	l
@597:
	ld	a,l
	ld	(_autoReadFPGA),a
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
@596:
	.debug	>
;3550 
;3551 
;3552   regNum = ATOU(argv[1]) & 0x3F;
@594:
	.debug	# 3552
	bool	hl
	ld	l,h
	push	hl
	push	hl
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	lcall	$strtoui
	add	sp,6
	ld	de,0x3f
	and	hl,de
	ld	(ix-4),hl
;3553   count = ATOU(argv[2]);
	.debug	# 3553
	bool	hl
	ld	l,h
	push	hl
	push	hl
	ld	hl,(ix+7)
	ld	e,0x4
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$strtoui
	add	sp,6
	ld	(ix-6),hl
;3554 
;3555   maxCount = 64 - regNum;  if (count > maxCount)  count = maxCount;
	.debug	# 3555
	ld	hl,0x40
	ex	de,hl
	ld	hl,(ix-4)
	ex	de,hl
	or	a,a
	sbc	hl,de
	ld	(ix-8),hl
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-8)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	c,@598
	jr	z,@598
	ld	h,d
	ld	l,e
	ld	(ix-6),hl
;3556 
;3557   if (getLock(APPL_LOCK, 5000) <= 0)  { perror("getLock(...)");  Exit(-1); 
@598:
	.debug	# 3557
	ld	de,0x1388
	ld	bc,0
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	lcall	$getLock
	add	sp,6
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	z,@600
	jr	nc,@599
@600:
@601:
	.debug	< @601 @602
	ld	de,word (phys @1+3738)
	ld	c,phys @1+3738 >> 16
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
@602:
	.debug	>
;3558 
;3559   for (u=0; u<count; ++u,++regNum)  {
@599:
@603:
	.debug	< @603 @604
	.debug	# 3559
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@605:
	ld	hl,(ix-2)
	ex	de,hl
	ld	hl,(ix-6)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	nc,@606
;3560     regVal = readFPGA(regNum);
@608:
	.debug	< @608 @609
	.debug	# 3560
	ld	hl,(ix-4)
	push	hl
	lcall	$readFPGA
	pop	de
	ld	(ix-10),hl
;3561     printf("  0x%02X:  ", regNum);
	.debug	# 3561
	ld	hl,(ix-4)
	push	hl
	ld	de,word (phys @1+3751)
	ld	c,phys @1+3751 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;3562     if (regVal < 0)
	.debug	# 3562
	ld	hl,(ix-10)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@610
;3563       printf("<error>\r\n");
	.debug	# 3563
	ld	de,word (phys @1+3763)
	ld	c,phys @1+3763 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3564     else
;3565       printf("0x%02X\r\n", regVal);
	jr	@611
@610:
	.debug	# 3565
	ld	hl,(ix-10)
	push	hl
	ld	de,word (phys @1+3773)
	ld	c,phys @1+3773 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;3566   }
@611:
@609:
	.debug	>
;3567 
;3568   releaseLock(APPL_LOCK);
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	ld	hl,(ix-4)
	inc	hl
	ld	(ix-4),hl
	jr	@605
@606:
@604:
	.debug	>
	.debug	# 3568
	bool	hl
	ld	l,h
	push	hl
	lcall	$releaseLock
	pop	de
;3569 
;3570   Exit(0);
	.debug	# 3570
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
;3571 }
@589:
	.debug	} @589 3571

@588	equ	10
	ld	sp,ix
	pop	ix
	lret
@590	equ	12
;3572 
;3573 
;3574 //-------------------------------------------------------------------------
;3575 //  "fpgawrite": command to write a value to the FPGA
;3576 //-------------------------------------------------------------------------
;3577 void _task fpgaWrite(int argc, char **argv)
;3578 {
	.align	4
$$fpgaWrite::
	ex	(sp),iy
$fpgaWrite::
	.debug	G $fpgaWrite 111 9
	.debug	{ $fpgaWrite 111 3577 @614 @613
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-4
	.debug	A argc 5 6
	.debug	A argv 7 112
;3579   uint  regNum, setVal;
	.debug	L regNum -2 7
	.debug	L setVal -4 7
;3580 
;3581 
;3582   if (showHelp(3, argc, argv))  {
	.debug	# 3582
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,0x3
	push	hl
	lcall	$showHelp
	add	sp,6
	or	a,a
	jr	z,@615
;3583     printf("Usage:  fpgawrite <reg> <value>\r\n"
@616:
	.debug	< @616 @617
;3584            "                  auto on/off\r\n");
	.debug	# 3583
	ld	de,word (phys @1+3782)
	ld	c,phys @1+3782 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3585     Exit(-1);
	.debug	# 3585
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
;3586   }
@617:
	.debug	>
;3587 
;3588   if (StriEq(argv[1], "auto"))  {
@615:
	.debug	# 3588
	ld	hl,@2+178
	push	hl
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@618
;3589     autoWriteFPGA = StriEq(argv[2], "on");  Exit(0); }
@619:
	.debug	< @619 @620
	.debug	# 3589
	ld	hl,@2+138
	push	hl
	ld	hl,(ix+7)
	ld	de,0x4
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	ld	l,h
	jr	nz,@621
	inc	l
@621:
	ld	a,l
	ld	(_autoWriteFPGA),a
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
@620:
	.debug	>
;3590 
;3591   regNum = ATOU(argv[1]) & 0x3F;
@618:
	.debug	# 3591
	bool	hl
	ld	l,h
	push	hl
	push	hl
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	lcall	$strtoui
	add	sp,6
	ld	de,0x3f
	and	hl,de
	ld	(ix-2),hl
;3592   setVal = ATOU(argv[2]) & 0xFF;
	.debug	# 3592
	bool	hl
	ld	l,h
	push	hl
	push	hl
	ld	hl,(ix+7)
	ld	e,0x4
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$strtoui
	add	sp,6
	ld	h,0
	ld	(ix-4),hl
;3593 
;3594   printf("  write 0x%02X to 0x%02X\r\n", setVal, regNum);
	.debug	# 3594
	ld	hl,(ix-2)
	push	hl
	ld	hl,(ix-4)
	push	hl
	ld	de,word (phys @1+3847)
	ld	c,phys @1+3847 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
;3595 
;3596   if (writeFPGA(regNum, setVal))  { perror("writeFPGA()");  Exit(-1); }
	.debug	# 3596
	ld	hl,(ix-4)
	push	hl
	ld	hl,(ix-2)
	push	hl
	lcall	$writeFPGA
	add	sp,4
	bool	hl
	jr	z,@622
@623:
	.debug	< @623 @624
	ld	de,word (phys @1+3874)
	ld	c,phys @1+3874 >> 16
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
@624:
	.debug	>
;3597 
;3598   Exit(0);
@622:
	.debug	# 3598
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
;3599 }
@613:
	.debug	} @613 3599

@612	equ	4
	ld	sp,ix
	pop	ix
	lret
@614	equ	6
;3600 
;3601 
;3602 //-------------------------------------------------------------------------
;3603 //  "setfpgafirm": load a new FPGA firmware file in to the serial EEPROM
;3604 //-------------------------------------------------------------------------
;3605 void _task setFPGAfirm(int argc, char **argv)
;3606 {
	.align	4
$$setFPGAfirm::
	ex	(sp),iy
$setFPGAfirm::
	.debug	G $setFPGAfirm 111 9
	.debug	{ $setFPGAfirm 111 3605 @627 @626
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-11
	.debug	A argc 5 6
	.debug	A argv 7 112
;3607   int  i, fromfd;
	.debug	L i -2 6
	.debug	L fromfd -4 6
;3608   ulong  limit = 0;
	.debug	L limit -8 9
	.debug	# 3608
	ld	de,0
	ld	c,e
	ld	b,e
	ld	hl,bc
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
	.seg	FARCODE
;3609   bool  verify = false;
	.debug	L verify -9 3
	.debug	# 3609
	xor	a,a
	ld	(ix-9),a
	.seg	FARCODE
;3610   char  *fileName;
	.debug	L fileName -11 70
;3611 
;3612 
;3613   if (showHelp(1, argc, argv))  {
	.debug	# 3613
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,0x1
	push	hl
	lcall	$showHelp
	add	sp,6
	or	a,a
	jr	z,@628
;3614     printf("\nusage:  setfpgafirm [-v] [fileName] [-l<limit>]\r\n");
@629:
	.debug	< @629 @630
	.debug	# 3614
	ld	de,word (phys @1+3886)
	ld	c,phys @1+3886 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3615     Exit(-1);
	.debug	# 3615
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
;3616   }
@630:
	.debug	>
;3617 
;3618   fileName = FPGA_Prog_File;
@628:
	.debug	# 3618
	ld	hl,@2+183
	ld	(ix-11),hl
;3619   for (i=1; i<argc; ++i)  {
@631:
	.debug	< @631 @632
	.debug	# 3619
	ld	hl,0x1
	ld	(ix-2),hl
@633:
	ld	hl,(ix-2)
	ex	de,hl
	ld	hl,(ix+5)
	ex	de,hl
	call	__icmp##
	jr	nc,@634
;3620     if (argv[i][0] != '-')  { fileName = argv[i];  continue; }
@636:
	.debug	< @636 @637
	.debug	# 3620
	ld	hl,(ix-2)
	add	hl,hl
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	add	hl,de
	ld	hl,(hl)
	ld	a,(hl)
	sub	a,45
	jr	z,@638
@639:
	.debug	< @639 @640
	ld	hl,(ix-2)
	add	hl,hl
	add	hl,de
	ld	hl,(hl)
	ld	(ix-11),hl
	jr	@635
@640:
	.debug	>
;3621     if (StrEq(argv[i], "-v"))  verify = true;
@638:
	.debug	# 3621
	ld	hl,@2+172
	push	hl
	ld	hl,(ix-2)
	add	hl,hl
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_strcmp
	add	sp,4
	bool	hl
	jr	nz,@641
	ld	(ix-9),0x1
;3622     if (strncmp(argv[i], "-l", 2) == 0)  limit = ATOL(argv[i]+2);
@641:
	.debug	# 3622
	ld	hl,0x2
	push	hl
	ld	hl,@2+175
	push	hl
	ld	hl,(ix-2)
	add	hl,hl
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	add	hl,de
	ld	hl,(hl)
	push	hl
	call	_strncmp
	add	sp,6
	bool	hl
	jr	nz,@642
	bool	hl
	ld	l,h
	push	hl
	push	hl
	ld	hl,(ix-2)
	add	hl,hl
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	add	hl,de
	ld	hl,(hl)
	inc	hl
	inc	hl
	push	hl
	lcall	$strtol
	add	sp,6
	ld	hl,bc
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
;3623   }
@642:
@637:
	.debug	>
;3624 
;3625   if ((fromfd = open(fileName, O_RDONLY, 0)) < 0)  {
@635:
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jp	@633
@634:
@632:
	.debug	>
	.debug	# 3625
	bool	hl
	ld	l,h
	push	hl
	inc	l
	push	hl
	ld	hl,(ix-11)
	push	hl
	lcall	$open
	add	sp,6
	ld	(ix-4),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@643
;3626     perror(fileName);  Exit(-1); }
@644:
	.debug	< @644 @645
	.debug	# 3626
	ld	hl,(ix-11)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
@645:
	.debug	>
;3627 
;3628   printf("\r\nDownloading %.40s to EEPROM\r\n", fileName);
@643:
	.debug	# 3628
	ld	hl,(ix-11)
	push	hl
	ld	de,word (phys @1+3643)
	ld	c,phys @1+3643 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;3629 
;3630   updateEEPROM(FPGA_EEPROM, fromfd, limit, true, verify);
	.debug	# 3630
	bool	hl
	ld	l,(ix-9)
	push	hl
	ld	l,0x1
	push	hl
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-4)
	push	hl
	bool	hl
	ld	l,h
	push	hl
	lcall	$updateEEPROM
	add	sp,12
;3631 
;3632   close(fromfd);  Exit(0);
	.debug	# 3632
	ld	hl,(ix-4)
	push	hl
	lcall	$close
	pop	de
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
;3633 }
@626:
	.debug	} @626 3633

@625	equ	11
	ld	sp,ix
	pop	ix
	lret
@627	equ	13
;3634 
;3635 
;3636 //=========================================================================
;3637 //  Generic structure to handle application-specific presentation of the
;3638 //  digital and analog input/output values.  Code to handle additional
;3639 //  formatting that cannot be handled in this generic manner should be
;3640 //  placed in the showState() task routine.
;3641 //=========================================================================
;3642 
;3643 //-------------------------------------------------------------------------
;3644 _nearcall char *fmtPhaseMode(DispElem far *elem, char *buf)
;3645 {
	.align	4
$$fmtPhaseMode::
	ex	(sp),iy
$fmtPhaseMode::
	.debug	G $fmtPhaseMode 115 9
	.debug	{ $fmtPhaseMode 115 3644 @648 @647
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A elem 5 116
	.debug	A buf 9 70
;3646   return (char *) PhaseModeDesc[ELEM_VAL(uint)];
	.debug	# 3646
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	call	__farld2##
	ex	de,hl
	add	hl,hl
	ld	de,_PhaseModeDesc
	add	hl,de
	ld	hl,(hl)
;3647 }
@647:
	.debug	} @647 3647

@646	equ	0
	pop	ix
	lret
@648	equ	2
;3648 
;3649 //-------------------------------------------------------------------------
;3650 _nearcall char *fmtWarming(DispElem far *elem, char *buf)
;3651 {
	.align	4
$$fmtWarming::
	ex	(sp),iy
$fmtWarming::
	.debug	G $fmtWarming 115 9
	.debug	{ $fmtWarming 115 3650 @651 @650
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A elem 5 116
	.debug	A buf 9 70
;3652   return (ELEM_VAL(bool) ? " WARM  " : "WARMING");
	.debug	# 3652
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	ld	hl,de
	ld	a,c
	ldp	hl,(hl)
	ld	a,l
	or	a,a
	jr	z,@652
	ld	hl,@2+198
	jr	@650
@652:
	ld	hl,@2+206
;3653 }
@650:
	.debug	} @650 3653

@649	equ	0
	pop	ix
	lret
@651	equ	2
;3654 
;3655 //-------------------------------------------------------------------------
;3656 _nearcall char *fmtRamping(DispElem far *elem, char *buf)
;3657 {
	.align	4
$$fmtRamping::
	ex	(sp),iy
$fmtRamping::
	.debug	G $fmtRamping 115 9
	.debug	{ $fmtRamping 115 3656 @657 @656
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A elem 5 116
	.debug	A buf 9 70
;3658   return (ELEM_VAL(bool) ? "RAMPING" : "STEADY ");
	.debug	# 3658
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	ld	hl,de
	ld	a,c
	ldp	hl,(hl)
	ld	a,l
	or	a,a
	jr	z,@658
	ld	hl,@2+214
	jr	@656
@658:
	ld	hl,@2+222
;3659 }
@656:
	.debug	} @656 3659

@655	equ	0
	pop	ix
	lret
@657	equ	2
;3660 
;3661 //-------------------------------------------------------------------------
;3662 _nearcall char *fmtLocked(DispElem far *elem, char *buf)
;3663 {
	.align	4
$$fmtLocked::
	ex	(sp),iy
$fmtLocked::
	.debug	G $fmtLocked 115 9
	.debug	{ $fmtLocked 115 3662 @663 @662
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A elem 5 116
	.debug	A buf 9 70
;3664   return (ELEM_VAL(bool) ? "LOCKED  " : "UNLOCKED");
	.debug	# 3664
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	ld	hl,de
	ld	a,c
	ldp	hl,(hl)
	ld	a,l
	or	a,a
	jr	z,@664
	ld	hl,@2+230
	jr	@662
@664:
	ld	hl,@2+239
;3665 }
@662:
	.debug	} @662 3665

@661	equ	0
	pop	ix
	lret
@663	equ	2
;3666 //-------------------------------------------------------------------------
;3667   const char far DevStateStr[] = {
;3668 "--INPUTS--  Max  Ampl   Phase Atten   -------- RF State --------   ---DSP-
;3669 "Forward:  ##.## ##.## -###.##  ##     WarmLevel:##.##      ###     Load:##
;3670 "Reverse:  ##.## ##.## -###.##  ##     WarmTime: ##.##    #######   Temp: #
;3671 "Cavity:   ##.## ##.## -###.##  ##     RampTime: ##.##    #######   Prog ID
;3672 "Reference:      ##.## -###.##                                      Errs:##
;3673 "Setpoints:##.## ##.## -###.##         CavLocked: ########\r\n"
;3674 "\r\n"
;3675 "                                -TUNE-  -Ampl-  -Phase-   Pulsing: ###\r\n
;3676 "                                State:  ###### ########   Pulse Rate:###.#
;3677 "                                Pulsed:   #.##  -###.##   Pulse Duty:###.#
;3678 "                            \r\n"
;3679 "                                  --RF-- -Tuner-  ---- Tuner ----\r\n"
;3680 "                                P: #.###  #.###   State:   ######     \r\n
;3681 "                                I: #.###  #.###   Piezo:   -#####     FFT:
;3682 "                                D: #.###  #.###   Stepper: -#####\r\n"
;3683 "                            \r\n"
;3684 "                            \r\n"
;3685 "                            \r\n"
;3686 "\r\n"
;3687   };
	.seg	FARCONST
_DevStateStr::
	.db	"--INPUTS--  Max  Ampl   Phase Atten   -------- RF State --------   ---DSP---",13,10,"Forward"
	.db	":  ##.## ##.## -###.##  ##     WarmLevel:##.##      ###     Load:###.#%%",13,10,"Reverse:  #"
	.db	"#.## ##.## -###.##  ##     WarmTime: ##.##    #######   Temp: ##.#C",13,10,"Cavity:   ##.## "
	.db	"##.## -###.##  ##     RampTime: ##.##    #######   Prog ID: #",13,10,"Reference:      ##.## "
	.db	"-###.##                                      Errs:#####",13,10,"Setpoints:##.## ##.## -###.#"
	.db	"#         CavLocked: ########",13,10,13,10,"                                -TUNE-  -Ampl-  -Pha"
	.db	"se-   Pulsing: ###",13,10,"                                State:  ###### ########   Pulse R"
	.db	"ate:###.##",13,10,"                                Pulsed:   #.##  -###.##   Pulse Duty:###."
	.db	"##",13,10,"                            ",13,10,"                                  --RF-- -Tuner-  "
	.db	"---- Tuner ----",13,10,"                                P: #.###  #.###   State:   ######   "
	.db	"  ",13,10,"                                I: #.###  #.###   Piezo:   -#####     FFT:###%%",13
	.db	10,"                                D: #.###  #.###   Stepper: -#####",13,10,"                "
	.db	"            ",13,10,"                            ",13,10,"                            ",13,10,13,10,0
	.debug	G _DevStateStr 118 8
;3688 
;3689 const DispElem  far dispElems[] = {
;3690   { 10, 3,  VFPV(maxFwdAmpl),    BLUE,  fmtF32, 5,2 },
_dispElems::
	.db	0xa
	.db	0x3
	.dl	phys _maxFwdAmpl
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3691   { 10, 4,  VFPV(maxRevAmpl),    BLUE,  fmtF32, 5,2 },
	.db	0xa
	.db	0x4
	.dl	phys _maxRevAmpl
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3692   { 10, 5,  VFPV(maxCavAmpl),    BLUE,  fmtF32, 5,2 },
	.db	0xa
	.db	0x5
	.dl	phys _maxCavAmpl
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3693 
;3694   { 10, 7,  VFPV(maxOutAmpl), BLUE,  fmtF32, 5,2 },
	.db	0xa
	.db	0x7
	.dl	phys _maxOutAmpl
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3695 
;3696   { 16, 3,  VFPV(fwdAmpl),  BLUE,  fmtF32, 5,2 },
	.db	0x10
	.db	0x3
	.dl	phys _fwdAmpl
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3697   { 16, 4,  VFPV(revAmpl),  BLUE,  fmtF32, 5,2 },
	.db	0x10
	.db	0x4
	.dl	phys _revAmpl
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3698   { 16, 5,  VFPV(cavAmpl),  BLUE,  fmtF32, 5,2 },
	.db	0x10
	.db	0x5
	.dl	phys _cavAmpl
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3699   { 16, 6,  VFPV(ref0Ampl), BLUE,  fmtF32, 5,2 },
	.db	0x10
	.db	0x6
	.dl	phys _ref0Ampl
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3700   { 16, 7,  VFPV(amplSet),  BLUE,  fmtF32, 5,2 },
	.db	0x10
	.db	0x7
	.dl	phys _amplSet
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3701 
;3702   { 22, 3,  VFPV(fwdPhase),  BLUE,  fmtF32, 7,2 },
	.db	0x16
	.db	0x3
	.dl	phys _fwdPhase
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x7
	.dw	0x2
;3703   { 22, 4,  VFPV(revPhase),  BLUE,  fmtF32, 7,2 },
	.db	0x16
	.db	0x4
	.dl	phys _revPhase
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x7
	.dw	0x2
;3704   { 22, 5,  VFPV(cavPhase),  BLUE,  fmtF32, 7,2 },
	.db	0x16
	.db	0x5
	.dl	phys _cavPhase
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x7
	.dw	0x2
;3705   { 22, 6,  VFPV(ref0Phase), BLUE,  fmtF32, 7,2 },
	.db	0x16
	.db	0x6
	.dl	phys _ref0Phase
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x7
	.dw	0x2
;3706   { 22, 7,  VFPV(phaseSet),  BLUE,  fmtF32, 7,2 },
	.db	0x16
	.db	0x7
	.dl	phys _phaseSet
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x7
	.dw	0x2
;3707 
;3708   { 31, 3,  VFPV(fwdAtten),   BLUE,  fmtU16, 2,2 },
	.db	0x1f
	.db	0x3
	.dl	phys _fwdAtten
	.dw	@2+248
	.dw	word _fmtU16
	.dw	0x2
	.dw	0x2
;3709   { 31, 4,  VFPV(revAtten),   BLUE,  fmtU16, 2,2 },
	.db	0x1f
	.db	0x4
	.dl	phys _revAtten
	.dw	@2+248
	.dw	word _fmtU16
	.dw	0x2
	.dw	0x2
;3710   { 31, 5,  VFPV(cavAtten),   BLUE,  fmtU16, 2,2 },
	.db	0x1f
	.db	0x5
	.dl	phys _cavAtten
	.dw	@2+248
	.dw	word _fmtU16
	.dw	0x2
	.dw	0x2
;3711 
;3712   { 59, 3,  VFPV(rfIsOn),     BLUE,  fmtBool, 3, FMT_ON_OFF },
	.db	0x3b
	.db	0x3
	.dl	phys _rfIsOn
	.dw	@2+248
	.dw	word _fmtBool
	.dw	0x3
	.dw	0x2
;3713   { 57, 4,  VFPV(rfAtWarm),   BLUE,  fmtWarming, 7,7 },
	.db	0x39
	.db	0x4
	.dl	phys _rfAtWarm
	.dw	@2+248
	.dw	word _fmtWarming
	.dw	0x7
	.dw	0x7
;3714   { 57, 5,  VFPV(rfRamping),  BLUE,  fmtRamping, 7,7 },
	.db	0x39
	.db	0x5
	.dl	phys _rfRamping
	.dw	@2+248
	.dw	word _fmtRamping
	.dw	0x7
	.dw	0x7
;3715 
;3716   { 49, 7,  VFPV(cavLocked),  BLUE,  fmtLocked, 8,8 },
	.db	0x31
	.db	0x7
	.dl	phys _cavLocked
	.dw	@2+248
	.dw	word _fmtLocked
	.dw	0x8
	.dw	0x8
;3717 
;3718   { 48, 3,  VFPV(warmLevel),  BLUE,  fmtF32, 5,2 },
	.db	0x30
	.db	0x3
	.dl	phys _warmLevel
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3719   { 48, 4,  VFPV(warmTime),   BLUE,  fmtF32, 5,2 },
	.db	0x30
	.db	0x4
	.dl	phys _warmTime
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3720   { 48, 5,  VFPV(rampTime),   BLUE,  fmtF32, 5,2 },
	.db	0x30
	.db	0x5
	.dl	phys _rampTime
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x2
;3721 
;3722   { 72, 3,  VFPV(dspLoad),     BLUE,  fmtF32, 5,1 },
	.db	0x48
	.db	0x3
	.dl	phys _dspLoad
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x1
;3723   { 72, 4,  VFPV(pcbTemp),     BLUE,  fmtF32, 5,1 },
	.db	0x48
	.db	0x4
	.dl	phys _pcbTemp
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x1
;3724   { 76, 5,  VFPV(curProgNum),  BLUE,  fmtU16, 1,1 },
	.db	0x4c
	.db	0x5
	.dl	phys _curProgNum
	.dw	@2+248
	.dw	word _fmtU16
	.dw	0x1
	.dw	0x1
;3725   { 72, 6,  VFPV(romProgErrs), BLUE,  fmtU16, 5,5 },
	.db	0x48
	.db	0x6
	.dl	phys _romProgErrs
	.dw	@2+248
	.dw	word _fmtU16
	.dw	0x5
	.dw	0x5
;3726 
;3727   { 40,10,  VFPV(amplCtlisON),  BLUE,  fmtBool, 6, FMT_CLOSED_OPEN },
	.db	0x28
	.db	0xa
	.dl	phys _amplCtlisON
	.dw	@2+248
	.dw	word _fmtBool
	.dw	0x6
	.dw	0x3
;3728   { 42,11,  VFPV(pulseAmplSet), BLUE,  fmtF32, 4,2 },
	.db	0x2a
	.db	0xb
	.dl	phys _pulseAmplSet
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x4
	.dw	0x2
;3729 
;3730   { 47,10,  VFPV(phaseMode),     BLUE,  fmtPhaseMode, 8,8 },
	.db	0x2f
	.db	0xa
	.dl	phys _phaseMode
	.dw	@2+248
	.dw	word _fmtPhaseMode
	.dw	0x8
	.dw	0x8
;3731   { 48,11,  VFPV(pulsePhaseSet), BLUE,  fmtF32, 7,2 },
	.db	0x30
	.db	0xb
	.dl	phys _pulsePhaseSet
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x7
	.dw	0x2
;3732 
;3733   { 67, 9,  VFPV(rfPulsing),  BLUE,  fmtBool, 3, FMT_ON_OFF },
	.db	0x43
	.db	0x9
	.dl	phys _rfPulsing
	.dw	@2+248
	.dw	word _fmtBool
	.dw	0x3
	.dw	0x2
;3734   { 69,10,  VFPV(pulseFreq),  BLUE,  fmtF32, 6,2 },
	.db	0x45
	.db	0xa
	.dl	phys _pulseFreq
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x6
	.dw	0x2
;3735   { 69,11,  VFPV(pulseDuty),  BLUE,  fmtF32, 6,2 },
	.db	0x45
	.db	0xb
	.dl	phys _pulseDuty
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x6
	.dw	0x2
;3736 
;3737   { 35,14,  VFPV(rf_P),  BLUE,  fmtF32, 5,3 },
	.db	0x23
	.db	0xe
	.dl	phys _rf_P
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x3
;3738   { 35,15,  VFPV(rf_I),  BLUE,  fmtF32, 5,3 },
	.db	0x23
	.db	0xf
	.dl	phys _rf_I
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x3
;3739   { 35,16,  VFPV(rf_D),  BLUE,  fmtF32, 5,3 },
	.db	0x23
	.db	0x10
	.dl	phys _rf_D
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x3
;3740 
;3741   { 42,14,  VFPV(tuner_P),  BLUE,  fmtF32, 5,3 },
	.db	0x2a
	.db	0xe
	.dl	phys _tuner_P
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x3
;3742   { 42,15,  VFPV(tuner_I),  BLUE,  fmtF32, 5,3 },
	.db	0x2a
	.db	0xf
	.dl	phys _tuner_I
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x3
;3743   { 42,16,  VFPV(tuner_D),  BLUE,  fmtF32, 5,3 },
	.db	0x2a
	.db	0x10
	.dl	phys _tuner_D
	.dw	@2+248
	.dw	word _fmtF32
	.dw	0x5
	.dw	0x3
;3744 
;3745   { 59,14,  VFPV(tunerCtlisON),    BLUE,  fmtBool, 6, FMT_CLOSED_OPEN },
	.db	0x3b
	.db	0xe
	.dl	phys _tunerCtlisON
	.dw	@2+248
	.dw	word _fmtBool
	.dw	0x6
	.dw	0x3
;3746   { 59,15,  VFPV(piezoReadPos),    BLUE,  fmtS16, 6,6 },
	.db	0x3b
	.db	0xf
	.dl	phys _piezoReadPos
	.dw	@2+248
	.dw	word _fmtS16
	.dw	0x6
	.dw	0x6
;3747   { 59,16,  VFPV(stepperReadPos),  BLUE,  fmtS16, 6,6 },
	.db	0x3b
	.db	0x10
	.dl	phys _stepperReadPos
	.dw	@2+248
	.dw	word _fmtS16
	.dw	0x6
	.dw	0x6
;3748 
;3749   { 74,15,  VFPV(percentFFT),   BLUE,  fmtU16, 3,3 },
	.db	0x4a
	.db	0xf
	.dl	phys _percentFFT
	.dw	@2+248
	.dw	word _fmtU16
	.dw	0x3
	.dw	0x3
;3750 
;3751   { 0,0, NULL, NULL, NULL, 0,0 }
	.db	0
	.db	0
	.dw	0, 0
	.dw	0
	.dw	0
	.dw	0
	.dw	0
;3752 };
	.debug	G _dispElems 119 8
;3753 
;3754 
;3755 //--- ATTR_NAME macros currently defined in fcntl.h ---
;3756 const DigDispElem far digInElems[] = {
;3757 //{ xx,yy,  BIT_MASK,          "0STR"  ,ATTR_NAME,  "1STR"  ,ATTR_NAME },
;3758 
;3759   { 0,0, 0, NULL,NULL, NULL,NULL }
_digInElems::
	.db	0
	.db	0
	.dw	0, 0
	.dw	0
	.dw	0
	.dw	0
	.dw	0
;3760 };
	.debug	G _digInElems 120 8
;3761 
;3762 const DigDispElem far digOutElems[] = {
;3763 //{ xx,yy,  BIT_MASK,      "0STR"  ,ATTR_NAME,  "1STR" ,ATTR_NAME },
;3764 
;3765   { 0,0,  0,  NULL,NULL,  NULL,NULL }
_digOutElems::
	.db	0
	.db	0
	.dw	0, 0
	.dw	0
	.dw	0
	.dw	0
	.dw	0
;3766 };
	.debug	G _digOutElems 120 8
;3767 
;3768 
;3769 //-------------------------------------------------------------------------
;3770 //  "state":  Show continuously updated state of the system
;3771 //-------------------------------------------------------------------------
;3772 void _task showState(int argc, char **argv)
;3773 {
	.seg	FARCODE
	.align	4
$$showState::
	ex	(sp),iy
$showState::
	.debug	G $showState 111 9
	.debug	{ $showState 111 3772 @669 @668
	ld	hl,-129
	call	__fent##
	.debug	A argc 5 6
	.debug	A argv 7 112
;3774   bool  blinkOff;
	.debug	L blinkOff -1 3
;3775   char  temp[128];
	.debug	L temp -129 98
;3776 
;3777 
;3778   printf(clsCmd);  blinkOff = false;
	.debug	# 3778
	ld	hl,_clsCmd
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	xor	a,a
	ld	(ix-1),a
;3779 
;3780   CursorTo(0, 2);  printf(DevStateStr);
	.debug	# 3780
	ld	hl,0x1
	push	hl
	ld	l,0x3
	push	hl
	ld	de,word (phys @1+3937)
	ld	c,phys @1+3937 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
	ld	de,word (phys _DevStateStr)
	ld	c,phys _DevStateStr >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3781 
;3782   for (;;)  {
@670:
	.debug	< @670 @671
@672:
;3783     CursorTo(0, 0);  blinkOff = !blinkOff;
@675:
	.debug	< @675 @676
	.debug	# 3783
	ld	hl,0x1
	push	hl
	push	hl
	ld	de,word (phys @1+3937)
	ld	c,phys @1+3937 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
	ld	a,(ix-1)
	or	a,a
	ld	a,0
	jr	nz,@677
	inc	a
@677:
	ld	(ix-1),a
;3784 
;3785     stdHeader();  displayElems((DispElem far *)dispElems, blinkOff, temp);
	.debug	# 3785
	lcall	$stdHeader
	ld	hl,-129+@667
	add	hl,sp
	push	hl
	bool	hl
	ld	l,(ix-1)
	push	hl
	ld	de,word (phys _dispElems)
	ld	c,phys _dispElems >> 16
	push	bc
	push	de
	lcall	$displayElems
	add	sp,8
;3786 
;3787     displayDigElems(digitalIn, (DigDispElem far *)digInElems, blinkOff, tem
	.debug	# 3787
	ld	hl,-129+@667
	add	hl,sp
	push	hl
	bool	hl
	ld	l,(ix-1)
	push	hl
	ld	de,word (phys _digInElems)
	ld	c,phys _digInElems >> 16
	push	bc
	push	de
	ld	de,(_digitalIn)
	ld	bc,(_digitalIn+2)
	push	bc
	push	de
	lcall	$displayDigElems
	add	sp,12
;3788     displayDigElems(digitalOut, (DigDispElem far *)digOutElems, blinkOff, t
	.debug	# 3788
	ld	hl,-129+@667
	add	hl,sp
	push	hl
	bool	hl
	ld	l,(ix-1)
	push	hl
	ld	de,word (phys _digOutElems)
	ld	c,phys _digOutElems >> 16
	push	bc
	push	de
	ld	de,(_digitalOut)
	ld	bc,(_digitalOut+2)
	push	bc
	push	de
	lcall	$displayDigElems
	add	sp,12
;3789 
;3790     CursorTo(0,22);  fflush(stdout);  CheckETX();  sleepMS(500);
	.debug	# 3790
	ld	hl,0x1
	push	hl
	ld	l,0x17
	push	hl
	ld	de,word (phys @1+3937)
	ld	c,phys @1+3937 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
	ld	hl,(__stdio)
	push	hl
	lcall	$fflush
	pop	de
	lcall	$CheckETX
	ld	de,0x1f4
	ld	bc,0
	push	bc
	push	de
	lcall	$sleepMS
	add	sp,4
;3791   }
@676:
	.debug	>
;3792 
;3793 }
	jp	@672
@671:
	.debug	>
@668:
	.debug	} @668 3793

@667	equ	129
	ld	sp,ix
	pop	ix
	lret
@669	equ	131
;3794 
;3795 //-------------------------------------------------------------------------
;3796 //  return a string with a human-readable format for the specified DSP valu
;3797 //-------------------------------------------------------------------------
;3798 char *dspVal2Str(ulong dspVal, uchar dspType, char *buf)
;3799 {
$dspVal2Str::
	.debug	G $dspVal2Str 122 9
	.debug	{ $dspVal2Str 122 3798 @680 @679
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-4
	.debug	A dspVal 5 9
	.debug	A dspType 9 3
	.debug	A buf 11 70
;3800   float  fval;
	.debug	L fval -4 10
;3801 
;3802   switch (dspType)  {
	.debug	# 3802
	ld	a,(ix+9)
	jr	@681
;3803     case DT_S32:
@683:
	.debug	< @683 @684
;3804       sprintf(buf, "%12ld", (S32)dspVal);  break;
	.debug	# 3804
@685:
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	push	bc
	push	de
	ld	de,word (phys @1+3950)
	ld	c,phys @1+3950 >> 16
	jr	@1120
;3805 
;3806     case DT_U32:
;3807       sprintf(buf, "%12lu", (U32)dspVal);  break;
	.debug	# 3807
@686:
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	push	bc
	push	de
	ld	de,word (phys @1+3956)
	ld	c,phys @1+3956 >> 16
@1120:
	push	bc
	push	de
	ld	hl,(ix+11)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,12
;3808 
;3809     case DT_F32:
	jr	@682
;3810       Engineering(*(float *)&dspVal, 6, buf);  break;
	.debug	# 3810
@687:
	ld	hl,(ix+11)
	push	hl
	ld	hl,0x6
	push	hl
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	jr	@1119
;3811 
;3812     case RT_PHASE:
;3813       fval = (float)dspVal * PhaseConvFactor;
	.debug	# 3813
@688:
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	call	__ultof##
	push	bc
	push	de
	ld	de,0
	ld	bc,0x33b4
	call	__fmlt##
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
;3814       Engineering(fval, 6, buf);  break;
	.debug	# 3814
	ld	hl,(ix+11)
	push	hl
	ld	hl,0x6
	push	hl
	ld	hl,(ix-2)
@1119:
	ld	bc,hl
	push	bc
	push	de
	lcall	$Engineering
	add	sp,8
;3815   }
	jr	@682
@684:
	.debug	>
;3816 
;3817 
;3818   return buf;
@681:
	cp	0x2
	jr	z,@685
	cp	0x3
	jr	z,@686
	cp	0x4
	jr	z,@687
	cp	0x63
	jr	z,@688
@682:
	.debug	# 3818
	ld	hl,(ix+11)
;3819 }
@679:
	.debug	} @679 3819

@678	equ	4
	ld	sp,ix
	pop	ix
	lret
@680	equ	6
;3820 
;3821 //-------------------------------------------------------------------------
;3822 //  Check the list of known DSP registers against the specified register ID
;3823 //  If it matches one of the known registers, then format the provided DSP
;3824 //  value based on the expected data format.  Otherwise, format the value a
;3825 //  a float.
;3826 //-------------------------------------------------------------------------
;3827 char *fmtDSPval(ulong dspVal, ulong regNum, char *buf)
;3828 {
$fmtDSPval::
	.debug	G $fmtDSPval 123 9
	.debug	{ $fmtDSPval 123 3827 @691 @690
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-17
	.debug	A dspVal 5 9
	.debug	A regNum 9 9
	.debug	A buf 13 70
;3829   uchar  dspType = DT_F32;
	.debug	L dspType -1 3
	.debug	# 3829
	ld	(ix-1),0x4
	.seg	FARCODE
;3830   uint  i, j;
	.debug	L i -3 7
	.debug	L j -5 7
;3831   const RD_DSP_REG far *rdReg;
	.debug	L rdReg -9 84
;3832   const WR_DSP_REG far *wrReg;
	.debug	L wrReg -13 87
;3833   const char far *name = NULL;
	.debug	L name -17 48
	.debug	# 3833
	ld	de,0
	ld	c,e
	ld	b,e
	ld	hl,bc
	ld	(ix-15),hl
	ex	de,hl
	ld	(ix-17),hl
	ex	de,hl
	.seg	FARCODE
;3834 
;3835 
;3836   for (i=0,rdReg=rdDSPregs; i<NUM_RD_DSP_REGS; ++i,++rdReg)  {
@692:
	.debug	< @692 @693
	.debug	# 3836
	ld	(ix-3),hl
	ld	de,word (phys _rdDSPregs)
	ld	c,phys _rdDSPregs >> 16
	ld	l,c
	ld	(ix-7),hl
	ex	de,hl
	ld	(ix-9),hl
	ex	de,hl
@694:
	ld	hl,(ix-3)
	ld	de,0x33
	or	a,a
	sbc	hl,de
	jr	nc,@695
;3837     if (rdReg->id != regNum)  continue;
@697:
	.debug	< @697 @698
	.debug	# 3837
	ld	hl,(ix-9)
	ex	de,hl
	ld	hl,(ix-7)
	ld	bc,hl
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	call	__lsub##
	jr	nz,@696
;3838     name = rdReg->mbReg->name;  dspType = rdReg->type;  break;
	.debug	# 3838
	ld	hl,(ix-9)
	ex	de,hl
	ld	hl,(ix-7)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x5
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	hl,0x4
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-15),hl
	ex	de,hl
	ld	(ix-17),hl
	ex	de,hl
	ld	hl,(ix-9)
	ex	de,hl
	ld	hl,(ix-7)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,de
	ld	a,c
	ldp	hl,(hl)
	ld	a,l
	ld	(ix-1),a
;3839   }
	jr	@695
@698:
	.debug	>
;3840 
;3841   if (! name)  for (i=0,wrReg=wrDSPregs; i<NUM_WR_DSP_REGS; ++i,++wrReg)  {
@696:
	ld	hl,(ix-3)
	inc	hl
	ld	(ix-3),hl
	ld	hl,(ix-9)
	ex	de,hl
	ld	hl,(ix-7)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x11
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-7),hl
	ex	de,hl
	ld	(ix-9),hl
	ex	de,hl
	jp	@694
@695:
@693:
	.debug	>
	.debug	# 3841
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-15)
	ld	bc,hl
	ld	a,e
	or	a,d
	or	a,c
	jp	nz,@700
@701:
	.debug	< @701 @702
	bool	hl
	ld	l,h
	ld	(ix-3),hl
	ld	de,word (phys _wrDSPregs)
	ld	c,phys _wrDSPregs >> 16
	ld	hl,bc
	ld	(ix-11),hl
	ex	de,hl
	ld	(ix-13),hl
	ex	de,hl
@703:
	ld	hl,(ix-3)
	ld	de,0x42
	or	a,a
	sbc	hl,de
	jr	nc,@704
;3842     if (wrReg->id != regNum)  continue;
@706:
	.debug	< @706 @707
	.debug	# 3842
	ld	hl,(ix-13)
	ex	de,hl
	ld	hl,(ix-11)
	ld	bc,hl
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix+9)
	ex	de,hl
	ld	hl,(ix+11)
	ld	bc,hl
	call	__lsub##
	jr	nz,@705
;3843     name = wrReg->mbReg->name;  dspType = wrReg->type;  break;
	.debug	# 3843
	ld	hl,(ix-13)
	ex	de,hl
	ld	hl,(ix-11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x5
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	hl,0x4
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-15),hl
	ex	de,hl
	ld	(ix-17),hl
	ex	de,hl
	ld	hl,(ix-13)
	ex	de,hl
	ld	hl,(ix-11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,de
	ld	a,c
	ldp	hl,(hl)
	ld	a,l
	ld	(ix-1),a
;3844   }
	jr	@704
@707:
	.debug	>
;3845 
;3846   if (name)  {
@705:
	ld	hl,(ix-3)
	inc	hl
	ld	(ix-3),hl
	ld	hl,(ix-13)
	ex	de,hl
	ld	hl,(ix-11)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x14
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-11),hl
	ex	de,hl
	ld	(ix-13),hl
	ex	de,hl
	jp	@703
@704:
@702:
	.debug	>
@700:
	.debug	# 3846
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-15)
	ld	bc,hl
	ld	a,e
	or	a,d
	or	a,c
	jr	z,@709
;3847     strcpyfn(buf, name);
@710:
	.debug	< @710 @711
	.debug	# 3847
	ld	h,b
	ld	l,c
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix+13)
	push	hl
	call	_strcpyfn
	add	sp,6
;3848     for (j=strlen(buf); j<16; ++j)  buf[j] = ' ';
@712:
	.debug	< @712 @713
	.debug	# 3848
	ld	hl,(ix+13)
	push	hl
	call	_strlen
	pop	de
	ld	(ix-5),hl
@714:
	ld	hl,(ix-5)
	ld	de,0x10
	or	a,a
	sbc	hl,de
	jr	nc,@715
	ld	hl,(ix-5)
	ex	de,hl
	ld	hl,(ix+13)
	ex	de,hl
	add	hl,de
	ld	a,0x20
	ld	(hl),a
;3849     buf[15] = ' ';
	ld	hl,(ix-5)
	inc	hl
	ld	(ix-5),hl
	jr	@714
@715:
@713:
	.debug	>
	.debug	# 3849
	ld	hl,(ix+13)
	ld	de,0xf
	add	hl,de
	ld	a,0x20
	ld	(hl),a
;3850     dspVal2Str(dspVal, dspType, buf+16);
	.debug	# 3850
	ld	hl,(ix+13)
	inc	e
	add	hl,de
	push	hl
	bool	hl
	ld	l,(ix-1)
	push	hl
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	push	bc
	push	de
	lcall	$dspVal2Str
	add	sp,8
;3851   }
@711:
	.debug	>
;3852   else  {
	jr	@717
@709:
;3853     sprintf(buf, "%14ld  ", dspVal);
@718:
	.debug	< @718 @719
	.debug	# 3853
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	push	bc
	push	de
	ld	de,word (phys @1+3962)
	ld	c,phys @1+3962 >> 16
	push	bc
	push	de
	ld	hl,(ix+13)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,12
;3854     dspVal2Str(dspVal, DT_F32, buf+16);
	.debug	# 3854
	ld	hl,(ix+13)
	ld	de,0x10
	add	hl,de
	push	hl
	ld	hl,0x4
	push	hl
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	push	bc
	push	de
	lcall	$dspVal2Str
	add	sp,8
;3855   }
@719:
	.debug	>
;3856 
;3857   return buf;
@717:
	.debug	# 3857
	ld	hl,(ix+13)
;3858 }
@690:
	.debug	} @690 3858

@689	equ	17
	ld	sp,ix
	pop	ix
	lret
@691	equ	19
;3859 
;3860 //-------------------------------------------------------------------------
;3861 //  "dspcalib": calibrate gain/offset for conversion to/from certain DSP
;3862 //  values and Engineering units.
;3863 //
;3864 //  The value to calibrate is specified by giving the name of a ModBus
;3865 //  register value.  If the register is referenced in one of the DSP regist
;3866 //  tables, and there is a gain and offset value listed for it, then the
;3867 //  gain and offset can be calibrated by running the device at 2 different
;3868 //  states and providing the actual (measured) value for each state.
;3869 //
;3870 //  NOTE that some calibrations apply to more than one register, so changin
;3871 //  them for one can affect other readings/settings.
;3872 //-------------------------------------------------------------------------
;3873 void _task dspcalib(int argc, char **argv)
;3874 {
	.align	4
$$dspcalib::
	ex	(sp),iy
$dspcalib::
	.debug	G $dspcalib 111 9
	.debug	{ $dspcalib 111 3873 @722 @721
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-101
	.debug	A argc 5 6
	.debug	A argv 7 112
;3875   bool  foundReg;
	.debug	L foundReg -1 3
;3876   uint  ui;
	.debug	L ui -3 7
;3877   float  *gain, *offset;
	.debug	L gain -5 124
	.debug	L offset -7 124
;3878   float  mbVal, egu[2], raw[2];
	.debug	L mbVal -11 10
	.debug	L egu -19 125
	.debug	L raw -27 125
;3879   const RD_DSP_REG far *rdReg;
	.debug	L rdReg -31 84
;3880   const WR_DSP_REG far *wrReg;
	.debug	L wrReg -35 87
;3881   RegInfo *mbReg;
	.debug	L mbReg -37 38
;3882   char  temp[64];
	.debug	L temp -101 126
;3883 
;3884 
;3885   if (showHelp(2, argc, argv))  {
	.debug	# 3885
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,0x2
	push	hl
	lcall	$showHelp
	add	sp,6
	or	a,a
	jr	z,@723
;3886     printf("\r\n Usage:  dspcalib <regName>\r\n");  Exit(-1); }
@724:
	.debug	< @724 @725
	.debug	# 3886
	ld	de,word (phys @1+3970)
	ld	c,phys @1+3970 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
@725:
	.debug	>
;3887 
;3888 
;3889   foundReg = false;
@723:
	.debug	# 3889
	xor	a,a
	ld	(ix-1),a
;3890 
;3891 //--- check RO DSP register list ---
;3892   for (rdReg=rdDSPregs,ui=0; ui<NUM_RD_DSP_REGS; ++ui,++rdReg)  {
@726:
	.debug	< @726 @727
	.debug	# 3892
	ld	de,word (phys _rdDSPregs)
	ld	c,phys _rdDSPregs >> 16
	ld	hl,bc
	ld	(ix-29),hl
	ex	de,hl
	ld	(ix-31),hl
	ex	de,hl
	bool	hl
	ld	l,h
	ld	(ix-3),hl
@728:
	ld	hl,(ix-3)
	ld	de,0x33
	or	a,a
	sbc	hl,de
	jp	nc,@729
;3893     mbReg = rdReg->mbReg;  strcpyfn(temp, mbReg->name);
@731:
	.debug	< @731 @732
	.debug	# 3893
	ld	hl,(ix-31)
	ex	de,hl
	ld	hl,(ix-29)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x5
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-37),hl
	ld	de,0x4
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,-97+@720
	add	hl,sp
	push	hl
	call	_strcpyfn
	add	sp,6
;3894     if (stricmp(temp, argv[1]))  continue;
	.debug	# 3894
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	hl,-99+@720
	add	hl,sp
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@730
;3895     gain = rdReg->gain;  offset = rdReg->offset;
	.debug	# 3895
	ld	hl,(ix-31)
	ex	de,hl
	ld	hl,(ix-29)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x7
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-5),hl
	ld	hl,(ix-31)
	ex	de,hl
	ld	hl,(ix-29)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x9
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-7),hl
;3896     foundReg = true;  break;
	.debug	# 3896
	ld	a,0x1
	ld	(ix-1),a
;3897   }
	jr	@729
@732:
	.debug	>
;3898 
;3899 //--- check RW DSP register list ---
;3900   if (!foundReg)  for (wrReg=wrDSPregs,ui=0; ui<NUM_WR_DSP_REGS; ++ui,++wrR
@730:
	ld	hl,(ix-3)
	inc	hl
	ld	(ix-3),hl
	ld	hl,(ix-31)
	ex	de,hl
	ld	hl,(ix-29)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x11
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-29),hl
	ex	de,hl
	ld	(ix-31),hl
	ex	de,hl
	jp	@728
@729:
@727:
	.debug	>
	.debug	# 3900
	ld	a,(ix-1)
	or	a,a
	jp	nz,@734
@735:
	.debug	< @735 @736
	ld	de,word (phys _wrDSPregs)
	ld	c,phys _wrDSPregs >> 16
	ld	hl,bc
	ld	(ix-33),hl
	ex	de,hl
	ld	(ix-35),hl
	ex	de,hl
	bool	hl
	ld	l,h
	ld	(ix-3),hl
@737:
	ld	hl,(ix-3)
	ld	de,0x42
	or	a,a
	sbc	hl,de
	jp	nc,@738
;3901     mbReg = wrReg->mbReg;  strcpyfn(temp, mbReg->name);
@740:
	.debug	< @740 @741
	.debug	# 3901
	ld	hl,(ix-35)
	ex	de,hl
	ld	hl,(ix-33)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x5
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-37),hl
	ld	de,0x4
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,-97+@720
	add	hl,sp
	push	hl
	call	_strcpyfn
	add	sp,6
;3902     if (stricmp(temp, argv[1]))  continue;
	.debug	# 3902
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	hl,-99+@720
	add	hl,sp
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@739
;3903     gain = wrReg->gain;  offset = wrReg->offset;
	.debug	# 3903
	ld	hl,(ix-35)
	ex	de,hl
	ld	hl,(ix-33)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x10
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-5),hl
	ld	hl,(ix-35)
	ex	de,hl
	ld	hl,(ix-33)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x12
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-7),hl
;3904     foundReg = true;  break;
	.debug	# 3904
	ld	a,0x1
	ld	(ix-1),a
;3905   }
	jr	@738
@741:
	.debug	>
;3906 
;3907   if (!foundReg)  {
@739:
	ld	hl,(ix-3)
	inc	hl
	ld	(ix-3),hl
	ld	hl,(ix-35)
	ex	de,hl
	ld	hl,(ix-33)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x14
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-33),hl
	ex	de,hl
	ld	(ix-35),hl
	ex	de,hl
	jp	@737
@738:
@736:
	.debug	>
@734:
	.debug	# 3907
	ld	a,(ix-1)
	or	a,a
	jr	nz,@743
;3908     printf("\r\n Register not related to a DSP value\r\n");  Exit(-2); }
@744:
	.debug	< @744 @745
	.debug	# 3908
	ld	de,word (phys @1+4002)
	ld	c,phys @1+4002 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	ld	hl,0xfffe
	push	hl
	lcall	$Exit
	pop	hl
@745:
	.debug	>
;3909 
;3910   if (!gain AND !offset)  {
@743:
	.debug	# 3910
	ld	hl,(ix-5)
	bool	hl
	jr	nz,@747
	ld	hl,(ix-7)
	bool	hl
	jr	nz,@747
;3911     printf("\r\n Value does not require conversion\r\n");  Exit(-3); }
@748:
	.debug	< @748 @749
	.debug	# 3911
	ld	de,word (phys @1+4043)
	ld	c,phys @1+4043 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	ld	hl,0xfffd
	push	hl
	lcall	$Exit
	pop	hl
@749:
	.debug	>
;3912 
;3913 
;3914   printf("\r\n Existing:");
@747:
	.debug	# 3914
	ld	de,word (phys @1+4082)
	ld	c,phys @1+4082 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3915   if (gain)  printf("  Gain: %s", Engineering(*gain, 6, temp));
	.debug	# 3915
	ld	hl,(ix-5)
	bool	hl
	jr	z,@750
	ld	hl,-101+@720
	add	hl,sp
	push	hl
	ld	hl,0x6
	push	hl
	ld	hl,(ix-5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	lcall	$Engineering
	add	sp,8
	push	hl
	ld	de,word (phys @1+4095)
	ld	c,phys @1+4095 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;3916   if (offset)  printf("  Offset: %s", Engineering(*offset, 6, temp));
@750:
	.debug	# 3916
	ld	hl,(ix-7)
	bool	hl
	jr	z,@751
	ld	hl,-101+@720
	add	hl,sp
	push	hl
	ld	hl,0x6
	push	hl
	ld	hl,(ix-7)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	lcall	$Engineering
	add	sp,8
	push	hl
	ld	de,word (phys @1+4106)
	ld	c,phys @1+4106 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;3917   printf("\r\n\n");
@751:
	.debug	# 3917
	ld	de,word (phys @1+3391)
	ld	c,phys @1+3391 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3918 
;3919 //--- get real EGU and DSP values for two test points ---
;3920   for (ui=0; ui<2; ++ui)  {
@752:
	.debug	< @752 @753
	.debug	# 3920
	bool	hl
	ld	l,h
	ld	(ix-3),hl
@754:
	ld	hl,(ix-3)
	ld	de,0x2
	or	a,a
	sbc	hl,de
	jp	nc,@755
;3921     printf(" Test point %u:  Enter measured/actual value for %ls: ",
@757:
	.debug	< @757 @758
;3922            ui+1, mbReg->name);
	.debug	# 3922
	ld	hl,(ix-37)
	ld	e,0x4
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix-3)
	inc	hl
	push	hl
	ld	de,word (phys @1+4119)
	ld	c,phys @1+4119 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,10
;3923     GETS(temp);  if (!*temp)  Exit(-4);
	.debug	# 3923
	bool	hl
	ld	l,h
	push	hl
	push	hl
	push	hl
	ld	hl,-95+@720
	add	hl,sp
	push	hl
	lcall	$Gets
	add	sp,8
	ld	a,(ix-101)
	or	a,a
	jr	nz,@759
	ld	hl,0xfffc
	push	hl
	lcall	$Exit
	pop	hl
;3924     egu[ui] = atof(temp);  mbVal = mb2float(mbReg->regVal, mbReg);
@759:
	.debug	# 3924
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-99+@720
	add	hl,sp
	push	hl
	call	_strtod
	add	sp,4
	ld	hl,(ix-3)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ex	de,hl
	ld	hl,-15+@720
	add	hl,sp
	add	hl,de
	pop	de
	pop	bc
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	hl,(ix-37)
	push	hl
	ld	hl,(hl)
	push	hl
	lcall	$mb2float
	add	sp,4
	ld	hl,bc
	ld	(ix-9),hl
	ex	de,hl
	ld	(ix-11),hl
	ex	de,hl
;3925     if (gain)  if (*gain != 0.0)  mbVal = mbVal / *gain;
	.debug	# 3925
	ld	hl,(ix-5)
	bool	hl
	jr	z,@760
	ld	hl,(ix-5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	a,(hl)
	and	a,7Fh
	or	a,c
	or	a,d
	or	a,e
	jr	z,@761
	ld	hl,(ix-5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix-11)
	ex	de,hl
	ld	hl,(ix-9)
	ld	bc,hl
	call	__fdivr##
	ld	hl,bc
	ld	(ix-9),hl
	ex	de,hl
	ld	(ix-11),hl
	ex	de,hl
;3926     if (offset)  mbVal += *offset;
@761:
@760:
	.debug	# 3926
	ld	hl,(ix-7)
	bool	hl
	jr	z,@762
	ld	hl,(ix-7)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix-11)
	ex	de,hl
	ld	hl,(ix-9)
	ld	bc,hl
	call	__fadd##
	ld	hl,bc
	ld	(ix-9),hl
	ex	de,hl
	ld	(ix-11),hl
	ex	de,hl
;3927     raw[ui] = mbVal;
@762:
	.debug	# 3927
	ld	hl,(ix-3)
	add	hl,hl
	add	hl,hl
	ex	de,hl
	ld	hl,-27+@720
	add	hl,sp
	add	hl,de
	ex	de,hl
	ld	hl,(ix-9)
	ld	bc,hl
	ld	hl,(ix-11)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;3928   }
@758:
	.debug	>
;3929 
;3930   if ((raw[0] == raw[1]) OR (egu[0] == egu[1]))  {
	ld	hl,(ix-3)
	inc	hl
	ld	(ix-3),hl
	jp	@754
@755:
@753:
	.debug	>
	.debug	# 3930
	ld	hl,(ix-27)
	ex	de,hl
	ld	hl,(ix-25)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-23)
	ex	de,hl
	ld	hl,(ix-21)
	ld	bc,hl
	call	__fequ##
	jr	z,@763
	ld	hl,(ix-19)
	ex	de,hl
	ld	hl,(ix-17)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-15)
	ex	de,hl
	ld	hl,(ix-13)
	ld	bc,hl
	call	__fequ##
	jr	nz,@764
@763:
;3931     printf("\r\n Unusable calibration data\r\n");  Exit(-4); }
@765:
	.debug	< @765 @766
	.debug	# 3931
	ld	de,word (phys @1+4174)
	ld	c,phys @1+4174 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	ld	hl,0xfffc
	push	hl
	lcall	$Exit
	pop	hl
@766:
	.debug	>
;3932 
;3933   KLOCK();
@764:
	.debug	# 3933
	call	_KLOCK
;3934   if (gain)  *gain = (egu[1] - egu[0]) / (raw[1] - raw[0]);
	.debug	# 3934
	ld	hl,(ix-5)
	bool	hl
	jr	z,@767
	ld	hl,(ix-15)
	ex	de,hl
	ld	hl,(ix-13)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-19)
	ex	de,hl
	ld	hl,(ix-17)
	ld	bc,hl
	call	__fsub##
	push	bc
	push	de
	ld	hl,(ix-23)
	ex	de,hl
	ld	hl,(ix-21)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-27)
	ex	de,hl
	ld	hl,(ix-25)
	ld	bc,hl
	call	__fsub##
	call	__fdiv##
	ld	hl,(ix-5)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;3935   if (offset)  *offset = raw[0] - (gain ? egu[0]/(*gain) : raw[1]);
@767:
	.debug	# 3935
	ld	hl,(ix-7)
	bool	hl
	jr	z,@768
	ld	hl,(ix-5)
	bool	hl
	jr	z,@769
	ld	hl,(ix-5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	ld	hl,(ix-19)
	ex	de,hl
	ld	hl,(ix-17)
	ld	bc,hl
	call	__fdivr##
	jr	@770
@769:
	ld	hl,(ix-23)
	ex	de,hl
	ld	hl,(ix-21)
	ld	bc,hl
@770:
	push	bc
	push	de
	ld	hl,(ix-27)
	ex	de,hl
	ld	hl,(ix-25)
	ld	bc,hl
	call	__fsubr##
	ld	hl,(ix-7)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;3936   KUNLOCK();
@768:
	.debug	# 3936
	call	_KUNLOCK
;3937 
;3938   printf("\r\n New:");
	.debug	# 3938
	ld	de,word (phys @1+4205)
	ld	c,phys @1+4205 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3939   if (gain)  printf("  Gain: %s", Engineering(*gain, 6,temp));
	.debug	# 3939
	ld	hl,(ix-5)
	bool	hl
	jr	z,@771
	ld	hl,-101+@720
	add	hl,sp
	push	hl
	ld	hl,0x6
	push	hl
	ld	hl,(ix-5)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	lcall	$Engineering
	add	sp,8
	push	hl
	ld	de,word (phys @1+4095)
	ld	c,phys @1+4095 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;3940   if (offset)  printf("  Offset: %s", Engineering(*offset, 6, temp));
@771:
	.debug	# 3940
	ld	hl,(ix-7)
	bool	hl
	jr	z,@772
	ld	hl,-101+@720
	add	hl,sp
	push	hl
	ld	hl,0x6
	push	hl
	ld	hl,(ix-7)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	push	bc
	push	de
	lcall	$Engineering
	add	sp,8
	push	hl
	ld	de,word (phys @1+4106)
	ld	c,phys @1+4106 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;3941   printf("\r\n\n");
@772:
	.debug	# 3941
	ld	de,word (phys @1+3391)
	ld	c,phys @1+3391 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;3942 
;3943   Exit(0);
	.debug	# 3943
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
;3944 }
@721:
	.debug	} @721 3944

@720	equ	101
	ld	sp,ix
	pop	ix
	lret
@722	equ	103
;3945 
;3946 //-------------------------------------------------------------------------
;3947 //  "dspio": command to enable/disable normal communication with the DSP
;3948 //-------------------------------------------------------------------------
;3949 void _task dspio(int argc, char **argv)
;3950 {
	.align	4
$$dspio::
	ex	(sp),iy
$dspio::
	.debug	G $dspio 111 9
	.debug	{ $dspio 111 3949 @775 @774
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-1
	.debug	A argc 5 6
	.debug	A argv 7 112
;3951   bool  val;
	.debug	L val -1 3
;3952 
;3953 
;3954   if (showHelp(3, argc, argv))  {
	.debug	# 3954
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,0x3
	push	hl
	lcall	$showHelp
	add	sp,6
	or	a,a
	jr	z,@776
;3955     printf("Usage:  dspio <read|write|filler|waves> <on|off>\r\n");  Exit(-
@777:
	.debug	< @777 @778
	.debug	# 3955
	ld	de,word (phys @1+4213)
	ld	c,phys @1+4213 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
@778:
	.debug	>
;3956 
;3957   if (StriEq("on", argv[2]))  val = true;  else
@776:
	.debug	# 3957
	ld	hl,(ix+7)
	ld	de,0x4
	add	hl,de
	ld	hl,(hl)
	push	hl
	ld	hl,@2+138
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@779
	ld	a,0x1
	ld	(ix-1),a
;3958   if (StriEq("off", argv[2])) val = false;  else  {
	jr	@780
@779:
	.debug	# 3958
	ld	hl,(ix+7)
	ld	de,0x4
	add	hl,de
	ld	hl,(hl)
	push	hl
	ld	hl,@2+141
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@781
	xor	a,a
	ld	(ix-1),a
	jr	@782
@781:
;3959     printf("invalid option\r\n");  Exit(-1);  }
@783:
	.debug	< @783 @784
	.debug	# 3959
	ld	de,word (phys @1+4264)
	ld	c,phys @1+4264 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
@784:
	.debug	>
;3960 
;3961   if (StriEq("read", argv[1]))    { autoReadDSP = val;   Exit(0); }
@782:
@780:
	.debug	# 3961
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	hl,@2+250
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@785
@786:
	.debug	< @786 @787
	ld	a,(ix-1)
	ld	(_autoReadDSP),a
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
@787:
	.debug	>
;3962   if (StriEq("write", argv[1]))   { autoWriteDSP = val;  Exit(0); }
@785:
	.debug	# 3962
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	hl,@2+255
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@788
@789:
	.debug	< @789 @790
	ld	a,(ix-1)
	ld	(_autoWriteDSP),a
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
@790:
	.debug	>
;3963   if (StriEq("filler", argv[1]))  { showFiller = val;    Exit(0); }
@788:
	.debug	# 3963
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	hl,@2+261
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@791
@792:
	.debug	< @792 @793
	ld	a,(ix-1)
	ld	(_showFiller),a
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
@793:
	.debug	>
;3964   if (StriEq("waves", argv[1]))   { readWaves = val;     Exit(0); }
@791:
	.debug	# 3964
	ld	hl,(ix+7)
	inc	hl
	inc	hl
	ld	hl,(hl)
	push	hl
	ld	hl,@2+268
	push	hl
	call	_stricmp
	add	sp,4
	bool	hl
	jr	nz,@794
@795:
	.debug	< @795 @796
	ld	a,(ix-1)
	ld	(_readWaves),a
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
@796:
	.debug	>
;3965 
;3966   printf("invalid option\r\n");  Exit(-1);
@794:
	.debug	# 3966
	ld	de,word (phys @1+4264)
	ld	c,phys @1+4264 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
;3967 }
@774:
	.debug	} @774 3967

@773	equ	1
	ld	sp,ix
	pop	ix
	lret
@775	equ	3
;3968 
;3969 
;3970 //-------------------------------------------------------------------------
;3971 //  Function to read the data from one of the DSP's data buffers that conta
;3972 //  new waveform data for the Phase and Amplitude values surrounding a rece
;3973 //  change to the Phase and/or Amplitude setpoints.
;3974 //-------------------------------------------------------------------------
;3975 int readWaveform(ulong bufID)
;3976 {
$readWaveform::
	.debug	G $readWaveform 127 9
	.debug	{ $readWaveform 127 3975 @799 @798
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-38
	.debug	A bufID 5 9
;3977   static uint readCount;  //test
	.bseg
@800:
	.ds	2
	.debug	S readCount @800 7
	.seg	FARCODE
;3978   ulong far  *buf;
	.debug	L buf -4 128
;3979   uint  w, lockID;
	.debug	L w -6 7
	.debug	L lockID -8 7
;3980   U32  u32;
	.debug	L u32 -12 9
;3981   float  fval;
	.debug	L fval -16 10
;3982   ulong  regNum, ackBits, val, count, countReg;
	.debug	L regNum -20 9
	.debug	L ackBits -24 9
	.debug	L val -28 9
	.debug	L count -32 9
	.debug	L countReg -36 9
;3983 
;3984 
;3985   ackBits = setDSPState;
	.debug	# 3985
	ld	de,(_setDSPState)
	ld	bc,(_setDSPState+2)
	ld	hl,bc
	ld	(ix-22),hl
	ex	de,hl
	ld	(ix-24),hl
	ex	de,hl
;3986 
;3987   switch (bufID)  {
	.debug	# 3987
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	jr	@801
;3988     case Rd_L_Buf:
@803:
	.debug	< @803 @804
;3989       w = WAVE_AP_L_SRC;  ackBits |= CMD_L_DATA_READ;  countReg = 0;
	.debug	# 3989
@805:
	bool	hl
	ld	l,h
	ld	(ix-6),hl
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	set	4,e
	jr	@1118
;3990       break;
	.debug	# 3990
;3991 
;3992     case Rd_T_Buf:
;3993       w = WAVE_AP_T_SRC;  ackBits |= CMD_T_DATA_READ;  countReg = 0;
	.debug	# 3993
@806:
	ld	hl,0x2
	ld	(ix-6),hl
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	set	5,e
@1118:
	ld	(ix-22),hl
	ex	de,hl
	ld	(ix-24),hl
	ex	de,hl
	ld	de,0
	ld	c,e
	ld	b,e
	ld	hl,bc
	jr	@1117
;3994       break;
	.debug	# 3994
;3995 
;3996     case AmplFFT:
;3997       w = WAVE_FFT_SRC;  ackBits |= CMD_FFT_DATA_READ;  countReg = SizeFFT;
	.debug	# 3997
@807:
	ld	hl,0x4
	ld	(ix-6),hl
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	ld	bc,hl
	set	6,e
	ld	(ix-22),hl
	ex	de,hl
	ld	(ix-24),hl
	ex	de,hl
	ld	de,0x224
	ld	hl,0x10
@1117:
	ld	(ix-34),hl
	ex	de,hl
	ld	(ix-36),hl
	ex	de,hl
;3998       break;
	.debug	# 3998
;3999 
;4000 //    case PhaseCTF:
;4001 //      if (hdwType == HT_PHASE_CTLR)  return -1;
;4002 //      w = WAVE_CTF_P_SRC;  ackBits = 0;  countReg = SizeCTF;
;4003 //      break;
;4004 
;4005 //    case AmplCTF:
;4006 //      if (hdwType == HT_PHASE_CTLR)  return -1;
;4007 //      w = WAVE_CTF_A_SRC;  ackBits = 0;  countReg = SizeCTF;
;4008 //      break;
;4009 
;4010     default:
	jr	@802
;4011       return -1;
	.debug	# 4011
@808:
	jp	@815
;4012   }
@804:
	.debug	>
;4013 
;4014   if (msSince_(lastWaveReadTime[w]) < 500)  return - 1;
@801:
	ld	hl,0xfffffdf6
	add	hl,de
	bool	hl
	jr	nz,@809
	ld	hl,0xfffffff0
	add	hl,bc
	bool	hl
	jr	z,@805
@809:
	ld	hl,0xfffffdf4
	add	hl,de
	bool	hl
	jr	nz,@810
	ld	hl,0xfffffff0
	add	hl,bc
	bool	hl
	jr	z,@806
@810:
	ld	hl,0xfffffdda
	add	hl,de
	bool	hl
	jr	nz,@808
	ld	hl,0xfffffff0
	add	hl,bc
	bool	hl
	jr	z,@807
	jr	@808
@802:
	.debug	# 4014
	ld	hl,(ix-6)
	add	hl,hl
	ld	de,_lastWaveReadTime
	add	hl,de
	ld	hl,(hl)
	ld	de,(__TimerMilliSeconds+4)
	ex	de,hl
	or	a,a
	sbc	hl,de
	ld	de,0x1f4
	or	a,a
	sbc	hl,de
	jr	c,@815
;4015 
;4016   if (msSince_(lastReadPlotTime) < 200)  return -1;
	.debug	# 4016
	ld	hl,(__TimerMilliSeconds+4)
	ld	de,(_lastReadPlotTime)
	or	a,a
	sbc	hl,de
	ld	de,0xc8
	or	a,a
	sbc	hl,de
	jr	c,@815
;4017 
;4018   buf = waveformData[w];  lockID = WAVE_DATA_0 + w;
	.debug	# 4018
	ld	hl,(ix-6)
	ex	de,hl
	ld	bc,0
	ld	a,0xc
	call	__lshl##
	push	bc
	push	de
	ld	de,word (phys _waveformData)
	ld	c,phys _waveformData >> 16
	call	__ladd##
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
	ld	hl,(ix-6)
	inc	hl
	ld	(ix-8),hl
;4019 
;4020   regNum = bufID;
	.debug	# 4020
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	(ix-18),hl
	ex	de,hl
	ld	(ix-20),hl
	ex	de,hl
;4021 
;4022 
;4023   if (getLock(lockID, 0) <= 0)  return -1;
	.debug	# 4023
	ld	de,0
	ld	c,e
	ld	b,e
	push	bc
	push	de
	ld	hl,(ix-8)
	push	hl
	lcall	$getLock
	add	sp,6
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	z,@815
	jr	nc,@814
@815:
	ld	hl,0xffff
	jp	@798
;4024 
;4025 //  if (ShowDebugMsgs)  fprintf(DiagLog, "readWaveform(w=%d)\n", w);  //tde
;4026 
;4027   if (readWaves)  {
@814:
	.debug	# 4027
	ld	a,(_readWaves)
	or	a,a
	jp	z,@816
;4028 
;4029   //--- get the # of long words in the waveform ---
;4030     if (countReg)  {
@817:
	.debug	< @817 @818
	.debug	# 4030
	ld	hl,(ix-36)
	ex	de,hl
	ld	hl,(ix-34)
	ld	bc,hl
	or	hl,de
	jp	z,@819
;4031       if (hdwType == HT_PHASE_CTLR)
@820:
	.debug	< @820 @821
	.debug	# 4031
	ld	hl,(_hdwType)
	ld	de,0x3
	or	a,a
	sbc	hl,de
	jr	nz,@822
;4032         readDSPReg(countReg, &count);
	.debug	# 4032
	ld	hl,-32+@797
	add	hl,sp
	push	hl
	ld	hl,(ix-36)
	ex	de,hl
	ld	h,b
	ld	l,c
	ld	bc,hl
	push	bc
	push	de
	lcall	$readDSPReg
	add	sp,6
;4033       else  {
	jr	@823
@822:
;4034         switch (bufID)  {
@824:
	.debug	< @824 @825
	.debug	# 4034
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	jr	@826
;4035           case Rd_L_Buf:  case Rd_T_Buf:  count = 1024;  break;
@828:
	.debug	< @828 @829
	.debug	# 4035
@830:
@831:
	ld	de,0x400
	ld	c,e
	ld	b,e
	ld	hl,bc
	jr	@1116
;4036           case AmplFFT:                   count = 1001;  break;
	.debug	# 4036
@832:
	ld	de,0x3e9
	ld	hl,0
@1116:
	ld	(ix-30),hl
	ex	de,hl
	ld	(ix-32),hl
	ex	de,hl
;4037 //          case PhaseCTF:  case AmplCTF:   count = 1000;  break;
;4038         }
	jr	@827
@829:
	.debug	>
;4039       }
@826:
	ld	hl,0xfffffdf6
	add	hl,de
	bool	hl
	jr	nz,@833
	ld	hl,0xfffffff0
	add	hl,bc
	bool	hl
	jr	z,@830
@833:
	ld	hl,0xfffffdf4
	add	hl,de
	bool	hl
	jr	nz,@834
	ld	hl,0xfffffff0
	add	hl,bc
	bool	hl
	jr	z,@831
@834:
	ld	hl,0xfffffdda
	add	hl,de
	bool	hl
	jr	nz,@827
	ld	hl,0xfffffff0
	add	hl,bc
	bool	hl
	jr	z,@832
@827:
@825:
	.debug	>
;4040       if (count > MaxDataPoints)  count = MaxDataPoints;
@823:
	.debug	# 4040
	ld	hl,(ix-32)
	ex	de,hl
	ld	hl,(ix-30)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x400
	ld	c,e
	ld	b,e
	call	__lsub##
	jr	c,@836
	jr	z,@836
	ld	de,0x400
	ld	c,e
	ld	b,e
	ld	hl,bc
	ld	(ix-30),hl
	ex	de,hl
	ld	(ix-32),hl
	ex	de,hl
;4041       numDataPoints[w] = (uint) count;
@836:
	.debug	# 4041
	ld	hl,(ix-6)
	add	hl,hl
	ld	de,_numDataPoints
	add	hl,de
	ex	de,hl
	ld	hl,(ix-32)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;4042     }
@821:
	.debug	>
;4043 
;4044   //--- read specified DSP buffer with waveform data ---
;4045     readPlot(regNum, buf, numDataPoints[w]);  lastReadPlotTime = GetMS_();
@819:
	.debug	# 4045
	ld	hl,(ix-6)
	add	hl,hl
	ld	de,_numDataPoints
	add	hl,de
	ld	hl,(hl)
	push	hl
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-20)
	ex	de,hl
	ld	hl,(ix-18)
	ld	bc,hl
	push	bc
	push	de
	lcall	$readPlot
	add	sp,10
	ld	hl,(__TimerMilliSeconds+4)
	ld	(_lastReadPlotTime),hl
;4046 
;4047     if (hdwType != HT_PHASE_CTLR)  {  // generate a simple waveform
	.debug	# 4047
	ld	hl,(_hdwType)
	ld	de,0x3
	or	a,a
	sbc	hl,de
	jp	z,@837
;4048       readCount += 5;  val = readCount;
@838:
	.debug	< @838 @839
	.debug	# 4048
	ld	hl,(@800)
	ld	e,0x5
	add	hl,de
	ld	(@800),hl
	ex	de,hl
	ld	b,h
	ld	l,h
	ld	(ix-26),hl
	ex	de,hl
	ld	(ix-28),hl
	ex	de,hl
;4049       for (uint u=0; u<numDataPoints[w]; ++u)  {
@840:
	.debug	< @840 @841
	.debug	L u -38 7
	.debug	# 4049
	ld	(ix-38),hl
	.seg	FARCODE
@842:
	ld	hl,(ix-6)
	add	hl,hl
	ld	de,_numDataPoints
	add	hl,de
	ld	hl,(hl)
	ex	de,hl
	ld	hl,(ix-38)
	or	a,a
	sbc	hl,de
	jp	nc,@843
;4050         val = ++val & 0xff;
@845:
	.debug	< @845 @846
	.debug	# 4050
	ld	hl,(ix-28)
	ex	de,hl
	ld	hl,(ix-26)
	ld	bc,hl
	call	__linc##
	ld	hl,bc
	ld	(ix-26),hl
	ex	de,hl
	ld	(ix-28),hl
	ex	de,hl
	push	bc
	push	de
	ld	de,0xff
	ld	c,d
	ld	b,d
	call	__land##
	ld	hl,bc
	ld	(ix-26),hl
	ex	de,hl
	ld	(ix-28),hl
	ex	de,hl
;4051 
;4052         switch (bufID)  {
	.debug	# 4052
	ld	hl,(ix+5)
	ex	de,hl
	ld	hl,(ix+7)
	ld	bc,hl
	jp	@847
;4053           case Rd_L_Buf:  // 2 16-bit LSB 1st values for Ampl and Phase
@849:
	.debug	< @849 @850
;4054           case Rd_T_Buf:  // ampl: raw ADC rdg, phase: int mult of 360/2^16
	.debug	# 4054
@851:
;4055             u32 = (val << 16) | (~val & 0xff);  buf[u] = htonl(u32);  break
	.debug	# 4055
@852:
	ld	hl,(ix-28)
	ex	de,hl
	ld	b,d
	ld	c,e
	ld	de,0
	push	bc
	push	de
	ld	hl,(ix-28)
	ex	de,hl
	ld	hl,(ix-26)
	ld	bc,hl
	call	__lnot##
	push	bc
	push	de
	ld	de,0xff
	ld	c,d
	ld	b,d
	call	__land##
	call	__lor##
	ld	hl,bc
	ld	(ix-10),hl
	ex	de,hl
	ld	(ix-12),hl
	ex	de,hl
	ex	de,hl
	ld	e,b
	ld	d,c
	ld	b,l
	ld	c,h
	jp	@1115
;4056 
;4057           case AmplFFT:  // Amplitude FFT (32-bit IEEE float)
;4058             fval = (float) val / 256.0;  buf[u] = htonl(*((ulong *)&fval));
	.debug	# 4058
@853:
	ld	hl,(ix-28)
	ex	de,hl
	ld	hl,(ix-26)
	ld	bc,hl
	call	__ultof##
	push	bc
	push	de
	ld	de,0
	ld	bc,0x4380
	call	__fdiv##
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
	ex	de,hl
	ld	e,b
	ld	d,c
	ld	b,l
	ld	c,h
@1115:
	ld	hl,(ix-38)
	push	bc
	push	de
	ex	de,hl
	ld	bc,0
	ld	a,0x2
	call	__lshl##
	push	bc
	push	de
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	call	__ladd##
	call	__farst4p##
;4059 
;4060 //          case PhaseCTF:  // Phase CTF (expr as int mult of 360/2^32)
;4061 //            u32 = ~(val << 24);  buf[u] = htonl(u32);  break;
;4062 
;4063 //          case AmplCTF:  // Ampl CTF (32-bit IEEE float) (?)
;4064 //  //          u32 = val << 24;  buf[u] = htonl(u32);  break;
;4065 //            fval = (float) val / 256.0;  buf[u] = htonl(*((ulong *)&fval)
;4066         }
	jr	@848
@850:
	.debug	>
;4067       }
@847:
	ld	hl,0xfffffdf6
	add	hl,de
	bool	hl
	jr	nz,@854
	ld	hl,0xfffffff0
	add	hl,bc
	bool	hl
	jp	z,@851
@854:
	ld	hl,0xfffffdf4
	add	hl,de
	bool	hl
	jr	nz,@855
	ld	hl,0xfffffff0
	add	hl,bc
	bool	hl
	jp	z,@852
@855:
	ld	hl,0xfffffdda
	add	hl,de
	bool	hl
	jr	nz,@848
	ld	hl,0xfffffff0
	add	hl,bc
	bool	hl
	jp	z,@853
@848:
@846:
	.debug	>
;4068     }
	ld	hl,(ix-38)
	inc	hl
	ld	(ix-38),hl
	jp	@842
@843:
@841:
	.debug	>
@839:
	.debug	>
;4069 
;4070   //--- let DSP know it is OK to write to the buffer again ---
;4071     if (ackBits)  writeDSPRegQuick(SetDSPState, ackBits);
@837:
	.debug	# 4071
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	ld	bc,hl
	or	hl,de
	jr	z,@857
	ld	h,b
	ld	l,c
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x200
	ld	bc,0x10
	push	bc
	push	de
	lcall	$writeDSPRegQuick
	add	sp,8
;4072 
;4073     _EnterCritical_();
@857:
	.debug	# 4073
	ipset	2
;4074     dataTimeSecs[w] = time(NULL);  dataTimeMS[w] = GetMS();
	.debug	# 4074
	bool	hl
	ld	l,h
	push	hl
	lcall	$time
	pop	hl
	ld	hl,(ix-6)
	add	hl,hl
	add	hl,hl
	push	de
	ld	de,_dataTimeSecs
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	ld	hl,(ix-6)
	add	hl,hl
	add	hl,hl
	push	de
	ld	de,_dataTimeMS
	add	hl,de
	pop	de
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;4075     _ExitCritical_();
	.debug	# 4075
	ipres
;4076 
;4077     lastWaveReadTime[w] = GetMS_();
	.debug	# 4077
	ld	hl,(ix-6)
	add	hl,hl
	ld	de,_lastWaveReadTime
	add	hl,de
	ld	de,(__TimerMilliSeconds+4)
	ld	(hl),e
	inc	hl
	ld	(hl),d
;4078   }
@818:
	.debug	>
;4079 
;4080   releaseLock(lockID);
@816:
	.debug	# 4080
	ld	hl,(ix-8)
	push	hl
	lcall	$releaseLock
	pop	de
;4081 
;4082   return 0;
	.debug	# 4082
	bool	hl
	ld	l,h
;4083 }
@798:
	.debug	} @798 4083

@797	equ	38
	ld	sp,ix
	pop	ix
	lret
@799	equ	40
;4084 
;4085 
;4086 
;4087 //=========================================================================
;4088 //  Copy the "cur" state from last time to "prev", and update "cur"
;4089 //=========================================================================
;4090 //  Just before this function is called, readings were taken of the current
;4091 //  state of the hardware, and any changes to the output states specified b
;4092 //  the last pass were applied (just after taking the readings).
;4093 //
;4094 //  This func first copies "cur" to "prev" and then "next" to "cur" which d
;4095 //  2 things for us:
;4096 //
;4097 //    - We have a copy of the controller's logical state from the last pass
;4098 //      (so we can determine what has changed)
;4099 //
;4100 //    - We have a copy of values set by other tasks indicating events/reque
;4101 //      that happend/were made since the beginning of the last pass
;4102 //
;4103 //  We then reset the values (as needed) that are set by other tasks to
;4104 //  indicate events and requests, and update the current state based on the
;4105 //  readings just taken and the requests and events communicated by other
;4106 //  tasks.
;4107 //
;4108 //  All of this is in preperation for the updateNextState() function, which
;4109 //  implements the main "decision making" portion of the application logic,
;4110 //  determining what the next state should be based on the latest condition
;4111 //  (as determined by the updateCurState() function).
;4112 //
;4113 //=========================================================================
;4114 void updateCurState(void)
;4115 {
$updateCurState::
	.debug	G $updateCurState 97 9
	.debug	{ $updateCurState 97 4114 @860 @859
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-10
;4116   uint  x;
	.debug	L x -2 7
;4117   ulong  maxWait1, maxWait2;
	.debug	L maxWait1 -6 9
	.debug	L maxWait2 -10 9
;4118   static uint  check1st = 0;
	.dseg
@861:
	.dw	0
	.debug	S check1st @861 7
	.seg	FARCODE
;4119 
;4120 
;4121   if (current != applTask)  return;
	.debug	# 4121
	ld	hl,(_applTask)
	ld	de,(_current)
	or	a,a
	sbc	hl,de
	jp	nz,@859
;4122 
;4123 //-----------------------------------------------------------------
;4124 // The following conditions exist at this point:
;4125 // - The inputs have just been read
;4126 // - The outputs were just set to values computed during the last pass
;4127 // - "cur" contains the state the inputs were in at the START of
;4128 //   of the previous pass, and the state the outputs were in DURING
;4129 //   the previous pass
;4130 // - "next" contains the values that the outputs were just set
;4131 //   to and any "input" values that were changed by other tasks
;4132 //------------------------------------------------------------------
;4133 
;4134 // Now we update the values in "Next" that are based solely on the
;4135 // state of the hardware inputs we just read and values maintained by
;4136 // by other tasks (basically, we deal with READINGS here: Values that
;4137 // we need but are not written to "Next" by another task)
;4138 
;4139 //  Next.processTime = GetMS();  curTime = time(NULL);
;4140 
;4141 //  msSincePrev = msSince(Cur.processTime);
;4142 
;4143   Next.digitalIn = digitalIn;
	.debug	# 4143
	ld	de,(_digitalIn)
	ld	bc,(_digitalIn+2)
	ld	(_nextState),de
	ld	(_nextState+2),bc
;4144 
;4145 
;4146 // Note that a copy is done (rather than just switching pointer values) to
;4147 // insure all values are set in both the source and destination.  If we
;4148 // simply kept pointers and swapped the pointer values, then EVERY value
;4149 // would have to be updated independently by the code, which is a pain to
;4150 // program, adds overhead and code, and is a VERY easy way to introduce odd
;4151 // bugs.  Overall, copying everything here is actually faster overall,
;4152 // simpler, and much safer.
;4153 
;4154   Prev = Cur;  Cur = Next;
	.debug	# 4154
	ld	hl,_curState
	ld	de,_prevState
	push	hl
	ld	bc,8
	ldir
	pop	hl
	ld	hl,_nextState
	ld	de,_curState
	push	hl
	ld	bc,8
	ldir
	pop	hl
;4155 
;4156 //--- At this point the Cur copy contains what the inputs were at the start
;4157 //--- of this pass, and the Prev copy contains the output values from the -
;4158 //--- the previous pass (i.e. the output values that resulted in the input 
;4159 //--- values in Cur) ---
;4160 
;4161 
;4162 //--------------------------------------------------------------------
;4163 // Next we update the rest of the "cur" values needed by the control
;4164 // logic (the ones that do NOT come from "next" values set by other
;4165 // tasks)
;4166 //
;4167 // NOTES:
;4168 //  - This portion of the logic has order-of-execution dependencies,
;4169 //    so be sure that all values referenced in an expression are ones
;4170 //    set before they are used.
;4171 //
;4172 //  - References to "prev" values are always safe since they are just
;4173 //    a copy of values from the last pass
;4174 //--------------------------------------------------------------------
;4175 
;4176   //--- log msgs for bit chgs only if not starving other processes for cycl
;4177   if (applLoopTime < 8)  logDigIOChgs(false);
	.debug	# 4177
	ld	hl,(_applLoopTime)
	ld	de,0x8
	or	a,a
	sbc	hl,de
	jr	nc,@863
	bool	hl
	ld	l,h
	push	hl
	lcall	$logDigIOChgs
	pop	hl
;4178 
;4179 
;4180 //APPLCODE
;4181 //ignore for now:  if (!fpgaInitialized)  return;
;4182 
;4183   if (!autoReadDSP AND !autoWriteDSP)  return;
@863:
	.debug	# 4183
	ld	a,(_autoReadDSP)
	or	a,a
	jr	nz,@865
	ld	a,(_autoWriteDSP)
	or	a,a
	jp	z,@859
;4184 
;4185   if (getLockDSP(10) <= 0)  return;
@865:
	.debug	# 4185
	ld	de,0xa
	ld	c,d
	ld	b,d
	push	bc
	push	de
	lcall	$getLockDSP
	add	sp,4
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	z,@867
	jr	nc,@866
@867:
;4186 
;4187 
;4188 //--- Amplitude and phase waveform data is read as a single stream of ---
;4189 //--- 32-bit values, where 16 of each 32 bits is an amplitude value and ---
;4190 //--- the other 16 is a phase value.  When operating in "pulsed" mode ---
;4191 //--- (where the Amplitude and/or Phase setpoints are being toggled ---
;4192 //--- between 2 values), the waveform data is updated based on when ---
;4193 //--- the change happens, and it provides data surrounding the "rising" ---
;4194 //--- edge and the "falling" edge.  When not operating in pulsed mode, ---
;4195 //--- the exact timing of the updates to the buffers is not known. ---
;4196 
;4197 //--- The FFT waveform is 32-bits per value and, like the Ampl/Phase data -
;4198 //--- is updated by the DSP whenever we tell it we have finished reading --
;4199 //--- the last copy in its buffer ---
;4200 
;4201 //--- In each case, status bits from the DSP indicate when the buffers ---
;4202 //--- have new data, and we need to read the data then tell the DSP that --
;4203 //--- we have done so and that it may write new data to the buffers ---
;4204 
;4205   if (hdwType == HT_PHASE_CTLR)  {  //test
	jp	@859
@866:
	.debug	# 4205
	ld	hl,(_hdwType)
	ld	de,0x3
	or	a,a
	sbc	hl,de
	jr	nz,@868
;4206     maxWait1 = 2000;  maxWait2 = 10000;  }
@869:
	.debug	< @869 @870
	.debug	# 4206
	ld	de,0x7d0
	ld	hl,0
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	ld	de,0x2710
	ld	b,h
	ld	c,l
	ld	(ix-8),hl
	ex	de,hl
	ld	(ix-10),hl
	ex	de,hl
@870:
	.debug	>
;4207   else  {
	jr	@871
@868:
;4208     maxWait1 = 500;  maxWait2 = 2000;  }
@872:
	.debug	< @872 @873
	.debug	# 4208
	ld	de,0x1f4
	ld	hl,0
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	ld	de,0x7d0
	ld	b,h
	ld	c,l
	ld	(ix-8),hl
	ex	de,hl
	ld	(ix-10),hl
	ex	de,hl
@873:
	.debug	>
;4209 
;4210 
;4211 //--- Rotate the order we use to check for new waveform data and do not ---
;4212 //--- read more than one each pass ---
;4213   for (x=0; x<3; ++x)  {
@871:
@874:
	.debug	< @874 @875
	.debug	# 4213
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@876:
	ld	hl,(ix-2)
	ld	de,0x3
	or	a,a
	sbc	hl,de
	jp	nc,@877
;4214     if (++check1st >= 3)  check1st = 0;
@879:
	.debug	< @879 @880
	.debug	# 4214
	ld	hl,(@861)
	inc	hl
	ld	(@861),hl
	or	a,a
	sbc	hl,de
	jr	c,@881
	bool	hl
	ld	l,h
	ld	(@861),hl
;4215 
;4216     if (check1st == 0)  if (NEW_L_DATA
@881:
	.debug	# 4216
	ld	hl,(@861)
	bool	hl
	jr	nz,@882
;4217         OR (msSince_(lastWaveReadTime[WAVE_AP_L_SRC]) > maxWait1))
	.debug	# 4217
	ld	de,(_statusBitsDSP)
	ld	bc,(_statusBitsDSP+2)
	push	bc
	push	de
	ld	de,0x40
	ld	c,d
	ld	b,d
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	nz,@883
	ld	hl,(__TimerMilliSeconds+4)
	ld	de,(_lastWaveReadTime)
	or	a,a
	sbc	hl,de
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	call	__lsub##
	jr	c,@884
	jr	z,@884
@883:
;4218       if (!readWaveform(Rd_L_Buf))  { lastIOTime = GetMS_();  break; }
	.debug	# 4218
	ld	de,0x20a
	ld	bc,0x10
	push	bc
	push	de
	lcall	$readWaveform
	add	sp,4
	bool	hl
	jp	z,@1114
@886:
	.debug	< @886 @887
@887:
	.debug	>
;4219 
;4220     if (check1st == 1)  if (NEW_T_DATA
@884:
@882:
	.debug	# 4220
	ld	hl,(@861)
	ld	de,0x1
	or	a,a
	sbc	hl,de
	jr	nz,@888
;4221         OR (msSince_(lastWaveReadTime[WAVE_AP_T_SRC]) > maxWait1))
	.debug	# 4221
	ld	de,(_statusBitsDSP)
	ld	bc,(_statusBitsDSP+2)
	push	bc
	push	de
	ld	de,0x80
	ld	c,d
	ld	b,d
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	nz,@889
	ld	hl,(__TimerMilliSeconds+4)
	ld	de,(_lastWaveReadTime+4)
	or	a,a
	sbc	hl,de
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	call	__lsub##
	jr	c,@890
	jr	z,@890
@889:
;4222       if (!readWaveform(Rd_T_Buf))  { lastIOTime = GetMS_();  break; }
	.debug	# 4222
	ld	de,0x20c
	ld	bc,0x10
	push	bc
	push	de
	lcall	$readWaveform
	add	sp,4
	bool	hl
	jr	z,@1114
@892:
	.debug	< @892 @893
@893:
	.debug	>
;4223 
;4224     if (check1st == 2)  if (FFT_DATA_VALID
@890:
@888:
	.debug	# 4224
	ld	hl,(@861)
	ld	de,0x2
	or	a,a
	sbc	hl,de
	jr	nz,@894
;4225         OR (msSince_(lastWaveReadTime[WAVE_FFT_SRC]) > maxWait2))
	.debug	# 4225
	ld	de,(_statusBitsDSP)
	ld	bc,(_statusBitsDSP+2)
	push	bc
	push	de
	ld	de,0x400
	ld	c,e
	ld	b,e
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	nz,@895
	ld	hl,(__TimerMilliSeconds+4)
	ld	de,(_lastWaveReadTime+8)
	or	a,a
	sbc	hl,de
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	call	__lsub##
	jr	c,@896
	jr	z,@896
@895:
;4226       if (!readWaveform(AmplFFT))   { lastIOTime = GetMS_();  break; }
	.debug	# 4226
	ld	de,0x226
	ld	bc,0x10
	push	bc
	push	de
	lcall	$readWaveform
	add	sp,4
	bool	hl
	jr	nz,@897
@898:
	.debug	< @898 @899
@1114:
	ld	hl,(__TimerMilliSeconds+4)
	ld	(_lastIOTime),hl
	jr	@877
@899:
	.debug	>
;4227   }
@897:
@896:
@894:
@880:
	.debug	>
;4228 
;4229 
;4230 /*
;4231   Reading waveforms and doing the periodic I/O to/from the DSP are both
;4232   lengthy processes, so we minimize latency on the rest of the system
;4233   by not allowing them both to happen in the same pass.
;4234 */
;4235   if (msSince_(lastIOTime) > 20)  {
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jp	@876
@877:
@875:
	.debug	>
	.debug	# 4235
	ld	hl,(__TimerMilliSeconds+4)
	ld	de,(_lastIOTime)
	or	a,a
	sbc	hl,de
	ld	de,0x14
	or	a,a
	sbc	hl,de
	jp	c,@900
	jp	z,@900
;4236 
;4237     ilkRatioGain = fwdAmplGain / revAmplGain;
@901:
	.debug	< @901 @902
	.debug	# 4237
	ld	de,(_fwdAmplGain)
	ld	bc,(_fwdAmplGain+2)
	push	bc
	push	de
	ld	de,(_revAmplGain)
	ld	bc,(_revAmplGain+2)
	call	__fdiv##
	ld	(_ilkRatioGain),de
	ld	(_ilkRatioGain+2),bc
;4238     ilkRatioOffset = 0;  //finish - what to set this to?
	.debug	# 4238
	ld	de,0
	ld	c,e
	ld	b,e
	ld	(_ilkRatioOffset),de
	ld	(_ilkRatioOffset+2),bc
;4239 
;4240     maxOutAmplGain = cavAmplGain * ffGain;
	.debug	# 4240
	ld	de,(_cavAmplGain)
	ld	bc,(_cavAmplGain+2)
	push	bc
	push	de
	ld	de,(_ffGain)
	ld	bc,(_ffGain+2)
	call	__fmlt##
	ld	(_maxOutAmplGain),de
	ld	(_maxOutAmplGain+2),bc
;4241     maxOutAmplOffset = 0;  //finish - what to set this to?
	.debug	# 4241
	ld	de,0
	ld	c,e
	ld	b,e
	ld	(_maxOutAmplOffset),de
	ld	(_maxOutAmplOffset+2),bc
;4242 
;4243 
;4244     if (autoReadDSP)
	.debug	# 4244
	ld	a,(_autoReadDSP)
	or	a,a
	jr	z,@903
;4245       if (!read_DSP_Regs(rdDSPregs))   lastIOTime = GetMS_();
	.debug	# 4245
	ld	de,word (phys _rdDSPregs)
	ld	c,phys _rdDSPregs >> 16
	push	bc
	push	de
	lcall	$read_DSP_Regs
	add	sp,4
	bool	hl
	jr	nz,@904
	ld	hl,(__TimerMilliSeconds+4)
	ld	(_lastIOTime),hl
;4246     if (autoWriteDSP)
@904:
@903:
	.debug	# 4246
	ld	a,(_autoWriteDSP)
	or	a,a
	jr	z,@905
;4247       if (!write_DSP_Regs(wrDSPregs))  lastIOTime = GetMS_();
	.debug	# 4247
	ld	de,word (phys _wrDSPregs)
	ld	c,phys _wrDSPregs >> 16
	push	bc
	push	de
	lcall	$write_DSP_Regs
	add	sp,4
	bool	hl
	jr	nz,@906
	ld	hl,(__TimerMilliSeconds+4)
	ld	(_lastIOTime),hl
;4248     if (autoReadFPGA)
@906:
@905:
	.debug	# 4248
	ld	a,(_autoReadFPGA)
	or	a,a
	jr	z,@907
;4249       if (!read_FPGA_Regs(rdFPGAregs, 0, NUM_RD_FPGA_REGS))  lastIOTime = G
	.debug	# 4249
	ld	hl,0xd
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_rdFPGAregs
	push	hl
	lcall	$read_FPGA_Regs
	add	sp,6
	bool	hl
	jr	nz,@908
	ld	hl,(__TimerMilliSeconds+4)
	ld	(_lastIOTime),hl
;4250     if (autoWriteFPGA)
@908:
@907:
	.debug	# 4250
	ld	a,(_autoWriteFPGA)
	or	a,a
	jr	z,@909
;4251       if (!write_FPGA_Regs(wrFPGAregs, 0, NUM_WR_FPGA_REGS))  lastIOTime = 
	.debug	# 4251
	ld	hl,0x4
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_wrFPGAregs
	push	hl
	lcall	$write_FPGA_Regs
	add	sp,6
	bool	hl
	jr	nz,@910
	ld	hl,(__TimerMilliSeconds+4)
	ld	(_lastIOTime),hl
;4252 
;4253 
;4254     if (dspLoad > 999)  dspLoad = 999;
@910:
@909:
	.debug	# 4254
	ld	de,(_dspLoad)
	ld	bc,(_dspLoad+2)
	push	bc
	push	de
	ld	de,0xc000
	ld	bc,0x4479
	call	__fsub##
	jp	m,@911
	jr	z,@911
	ld	de,0xc000
	ld	bc,0x4479
	ld	(_dspLoad),de
	ld	(_dspLoad+2),bc
;4255     if (pcbTemp > 99.9)  pcbTemp = 99.9;
@911:
	.debug	# 4255
	ld	de,(_pcbTemp)
	ld	bc,(_pcbTemp+2)
	push	bc
	push	de
	ld	de,0xcccd
	ld	bc,0x42c7
	call	__fsub##
	jp	m,@912
	jr	z,@912
	ld	de,0xcccd
	ld	bc,0x42c7
	ld	(_pcbTemp),de
	ld	(_pcbTemp+2),bc
;4256 
;4257     if (percentFFT > 999)  percentFFT = 999;
@912:
	.debug	# 4257
	ld	hl,(_percentFFT)
	ld	de,0x3e7
	or	a,a
	sbc	hl,de
	jr	c,@913
	jr	z,@913
	ld	h,d
	ld	l,e
	ld	(_percentFFT),hl
;4258 
;4259     rfIsOn = BOOL(RF_ON);
@913:
	.debug	# 4259
	ld	de,(_statusBitsDSP)
	ld	bc,(_statusBitsDSP+2)
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@914
	ld	a,0x1
	jr	@915
@914:
	xor	a,a
@915:
	ld	(_rfIsOn),a
;4260   //  rfPulsing = BOOL(RF_PULSES);  //finish
;4261     tunerCtlisON = BOOL(TUNER_CTL_ON);
	.debug	# 4261
	ld	de,(_statusBitsDSP)
	ld	bc,(_statusBitsDSP+2)
	push	bc
	push	de
	ld	de,0
	ld	bc,0x80
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@916
	ld	a,0x1
	jr	@917
@916:
	xor	a,a
@917:
	ld	(_tunerCtlisON),a
;4262   //  curProgNum = (uint) CUR_PROG_NUM;  //finish
;4263     rfAtWarm = BOOL(RF_OUT_WARM);
	.debug	# 4263
	ld	de,(_statusBitsDSP)
	ld	bc,(_statusBitsDSP+2)
	push	bc
	push	de
	ld	de,0x2000
	ld	c,e
	ld	b,e
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@918
	ld	a,0x1
	jr	@919
@918:
	xor	a,a
@919:
	ld	(_rfAtWarm),a
;4264     rfRamping = BOOL(OUT_RAMPING);
	.debug	# 4264
	ld	de,(_statusBitsDSP)
	ld	bc,(_statusBitsDSP+2)
	push	bc
	push	de
	ld	de,0x4000
	ld	c,e
	ld	b,e
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@920
	ld	a,0x1
	jr	@921
@920:
	xor	a,a
@921:
	ld	(_rfRamping),a
;4265     cavLocked = BOOL(CAV_LOCKED);
	.debug	# 4265
	ld	de,(_statusBitsDSP)
	ld	bc,(_statusBitsDSP+2)
	push	bc
	push	de
	ld	de,0
	ld	bc,0x1
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@922
	ld	a,0x1
	jr	@923
@922:
	xor	a,a
@923:
	ld	(_cavLocked),a
;4266     amplCtlisON = BOOL(AMPL_CTL_ON);
	.debug	# 4266
	ld	de,(_statusBitsDSP)
	ld	bc,(_statusBitsDSP+2)
	push	bc
	push	de
	ld	de,0
	ld	bc,0x10
	call	__land##
	ld	hl,bc
	or	hl,de
	jr	z,@924
	ld	a,0x1
	jr	@925
@924:
	xor	a,a
@925:
	ld	(_amplCtlisON),a
;4267     phaseMode = (uint) PHASE_MODE;
	.debug	# 4267
	ld	de,(_statusBitsDSP)
	ld	bc,(_statusBitsDSP+2)
	ld	a,0x15
	call	__ulshr##
	push	bc
	push	de
	ld	de,0x3
	ld	c,d
	ld	b,d
	call	__land##
	ld	(_phaseMode),de
;4268 
;4269   }
@902:
	.debug	>
;4270 
;4271   releaseLockDSP();
@900:
	.debug	# 4271
	lcall	$releaseLockDSP
;4272 }
@859:
	.debug	} @859 4272

@858	equ	10
	ld	sp,ix
	pop	ix
	lret
@860	equ	12
;4273 
;4274 
;4275 //=========================================================================
;4276 //  Now we update the values in "next" based on the "cur" and "prev"
;4277 //  values.
;4278 //=========================================================================
;4279 //  All values changed by updateNextState() are "outputs" from this func.
;4280 //  All other values referenced in this function are "inputs" to this func.
;4281 //
;4282 //  All the values in "next" provide information about the next logical sta
;4283 //  for the controller:  Values to be applied during the next pass of the
;4284 //  control logic.  All "next" values fall in to one of 2 categories:
;4285 //
;4286 //    - They are set/updated ONLY by this function (which is executed only
;4287 //      by the "applic" task)
;4288 //
;4289 //    - They are set ONLY by other tasks (within a lock), and possibly
;4290 //      reset/cleared by the "applic" task in updateCurState() (after copyi
;4291 //      "next" to "cur")
;4292 //
;4293 //
;4294 //  All values that act as input to this func must fall in to one of these
;4295 //  categories:
;4296 //
;4297 //    - "prev" values
;4298 //
;4299 //    - "cur" values updated directly by statements in updateCurState()
;4300 //
;4301 //    - "cur" values updated indirectly by updateCurState() due to the copy
;4302 //      of "next" to "cur", which includes the values in "next" that are
;4303 //      set by other tasks
;4304 //
;4305 //    - a shared/global value that is the equivalent of a "cur" value but
;4306 //      for which we don't need to track changes
;4307 //
;4308 //
;4309 //  All "outputs" from this function must be altered ONLY within this funct
;4310 //
;4311 //  All "inputs" to this function must NEVER be altered within this functio
;4312 //
;4313 //  Each output value (or VERY small "set" of ouput values) must be updated
;4314 //  in a single, self-contained block of code.  NO output may be referenced
;4315 //  OR altered in ANOTHER block of code:  This is a declarative style of
;4316 //  programming and has multiple advantages:
;4317 //
;4318 //     - There are no order-of-execution dependencies across nodes/blocks
;4319 //       within a pass.  They can be executed in any order and the result
;4320 //       will be the same (get used to it - this method will be the norm as
;4321 //       computer languages and hardware add support for atomic transaction
;4322 //       and computers evolve in to neural-net-like processor arrays - FPGA
;4323 //       are just the beginning of this trend).
;4324 //
;4325 //     - Encapsulating all logic for an output within a single node/block
;4326 //       leaves just one place to start when tracking a problem backward fr
;4327 //       the result (far simpler than guessing at the possible causes and
;4328 //       trying to work forward, looking for paths to the observed result).
;4329 //
;4330 //     - Drastically reduces the complexity of the code by greatly reducing
;4331 //       the number of possible paths through the code.  This simplifies
;4332 //       testing and verification processes.
;4333 //
;4334 //  Each node/block-of-code should be prefaced with the following informati
;4335 //   - The names of the values it modifies
;4336 //   - A descr of the meaning of the values and the logic used to set them
;4337 //   - A list of the values that serve as "inputs" to the node
;4338 //
;4339 //=========================================================================
;4340 void updateNextState(void)
;4341 {
$updateNextState::
	.debug	G $updateNextState 97 9
	.debug	{ $updateNextState 97 4340 @928 @927
;4342   if (current != applTask)  return;
	.debug	# 4342
	ld	hl,(_applTask)
	ld	de,(_current)
	or	a,a
	sbc	hl,de
;4343 
;4344 }
@927:
	.debug	} @927 4344

@926	equ	0
	lret
@928	equ	0
;4345 
;4346 
;4347 //=========================================================================
;4348 //  Next we update output signal values to be written to the outputs at
;4349 //  the beginning of the next pass (just after the inputs are read).
;4350 //=========================================================================
;4351 void updateOutputState(void)
;4352 {
$updateOutputState::
	.debug	G $updateOutputState 97 9
	.debug	{ $updateOutputState 97 4351 @932 @931
;4353 //  digitalOut = 0;
;4354 
;4355 //  if (rfIsOn)
;4356 //    SetBits(Next.digitalOut, ON_OFF_LED);
;4357 //  else
;4358 //    ClearBits(Next.digitalOut, ON_OFF_LED);
;4359 
;4360 //APPLCODE
;4361 
;4362 }
@931:
	.debug	} @931 4362

@930	equ	0
	lret
@932	equ	0
;4363 
;4364 
;4365 //=========================================================================
;4366 // The last thing we do is update values read by other tasks
;4367 //=========================================================================
;4368 // IMPORTANT NOTES and WARNINGS:
;4369 //
;4370 //   To avoid concurrency issues, other tasks need to use locks when readin
;4371 //   values set here.  But be careful to keep such locks short to avoid
;4372 //   delaying the timing of the main application task.  If necessary, split
;4373 //   such code in to sections, getting and releasing the lock(s) in each
;4374 //   section to avoid increasing latency for the main application task.
;4375 //
;4376 //   Values in the "next" instance of the application state fall in to 2
;4377 //   categories:  Inputs and Outputs.
;4378 //
;4379 //   The "next" Outputs are those that may be changed in updateOutputState(
;4380 //   The reset are Inputs, and may be changed by other tasks within a lock.
;4381 //
;4382 //   Since each pass through the application logic starts by setting
;4383 //   cur = next, any "next" values set by other tasks become part of the
;4384 //   "cur" instance used by the rest of the applic logic.  Because the
;4385 //   "applic" task at a higher priority than any other real-time task, the
;4386 //   changes to "next" values by other tasks can ONLY happen between passes
;4387 //   through the applic logic loop.
;4388 //
;4389 //   NOTES:
;4390 //     - Some of these "next" input values values must be initialized in
;4391 //       updateCurState() in order to tell when a value is set by another
;4392 //       task.  For example, if we have a boolean flag that is set when we
;4393 //       get an ON command, then (after its "next" value is copied to "cur"
;4394 //       we have to clear the flag in order to tell if another ON command i
;4395 //       received between passes.
;4396 //
;4397 //     - As with reads to these values, other tasks must use locks when
;4398 //       writing to the "next" input values to insure they are not preempte
;4399 //       by the "applic" task in the middle of a write.
;4400 //
;4401 //=========================================================================
;4402 void updateExternals(void)
;4403 {
$updateExternals::
	.debug	G $updateExternals 97 9
	.debug	{ $updateExternals 97 4402 @935 @934
;4404   if (current != applTask)  return;
	.debug	# 4404
	ld	hl,(_applTask)
	ld	de,(_current)
	or	a,a
	sbc	hl,de
	jr	nz,@934
;4405 
;4406 
;4407   digInBits = Cur.digitalIn;
	.debug	# 4407
	ld	de,(_curState)
	ld	bc,(_curState+2)
	ld	(_digInBits),de
	ld	(_digInBits+2),bc
;4408   digOutBits = Cur.digitalOut;
	.debug	# 4408
	ld	de,(_curState+4)
	ld	bc,(_curState+6)
	ld	(_digOutBits),de
	ld	(_digOutBits+2),bc
;4409 
;4410 //--- set bits in older statusBitsAppl format ---
;4411   statusBitsAppl = statusBitsDSP & 0x0000FFC8L;
	.debug	# 4411
	ld	de,(_statusBitsDSP)
	ld	bc,(_statusBitsDSP+2)
	push	bc
	push	de
	ld	de,0xffc8
	ld	bc,0
	call	__land##
	ld	(_statusBitsAppl),de
	ld	(_statusBitsAppl+2),bc
;4412 //  if (RF_PULSES)  statusBitsAppl |= (OLD_PHASE_PULSES | OLD_AMPL_PULSES);
;4413 
;4414 //APPLCODE
;4415 }
@934:
	.debug	} @934 4415

@933	equ	0
	lret
@935	equ	0
;4416 
;4417 
;4418 //=========================================================================
;4419 _nearcall float readGetRaw(uint ui)  {
	.align	4
$$readGetRaw::
	ex	(sp),iy
$readGetRaw::
	.debug	G $readGetRaw 129 9
	.debug	{ $readGetRaw 129 4419 @939 @938
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
;4420   return (ui < NumHdwADC ? anaIn[ui] : 0.0);  }
	.debug	# 4420
	ld	hl,(ix+5)
	ld	de,(_NumHdwADC)
	or	a,a
	sbc	hl,de
	jr	nc,@940
	ld	hl,(ix+5)
	add	hl,hl
	add	hl,hl
	ld	de,_anaIn
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	jr	@938
@940:
	ld	de,0
	ld	c,e
	ld	b,e
@938:
	.debug	} @938 4420

@937	equ	0
	pop	ix
	lret
@939	equ	2
;4421 
;4422 _nearcall float readMinRaw(uint ui)  {
	.align	4
$$readMinRaw::
	ex	(sp),iy
$readMinRaw::
	.debug	G $readMinRaw 129 9
	.debug	{ $readMinRaw 129 4422 @944 @943
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
;4423   return (ui < NumHdwADC) ? (adcMinRaw(ui) - adcOffset[ui]) * adcGain[ui] :
	.debug	# 4423
	ld	hl,(ix+5)
	ld	de,(_NumHdwADC)
	or	a,a
	sbc	hl,de
	jr	nc,@945
	ld	hl,(ix+5)
	push	hl
	lcall	$adcMinRaw
	pop	hl
	ld	hl,(ix+5)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_adcOffset
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fsub##
	ld	hl,(ix+5)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_adcGain
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fmlt##
	jr	@943
@945:
	ld	de,0
	ld	c,e
	ld	b,e
@943:
	.debug	} @943 4423

@942	equ	0
	pop	ix
	lret
@944	equ	2
;4424 
;4425 _nearcall float readMaxRaw(uint ui)  {
	.align	4
$$readMaxRaw::
	ex	(sp),iy
$readMaxRaw::
	.debug	G $readMaxRaw 129 9
	.debug	{ $readMaxRaw 129 4425 @949 @948
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
;4426   return (ui < NumHdwADC) ? (adcMaxRaw(ui) - adcOffset[ui]) * adcGain[ui] :
	.debug	# 4426
	ld	hl,(ix+5)
	ld	de,(_NumHdwADC)
	or	a,a
	sbc	hl,de
	jr	nc,@950
	ld	hl,(ix+5)
	push	hl
	lcall	$adcMaxRaw
	pop	hl
	ld	hl,(ix+5)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_adcOffset
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fsub##
	ld	hl,(ix+5)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_adcGain
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fmlt##
	jr	@948
@950:
	ld	de,0xcccd
	ld	bc,0x411c
@948:
	.debug	} @948 4426

@947	equ	0
	pop	ix
	lret
@949	equ	2
;4427 
;4428 _nearcall void readDefaults(uint ui, float *gain, float *offset)  {
	.align	4
$$readDefaults::
	ex	(sp),iy
$readDefaults::
	.debug	G $readDefaults 130 9
	.debug	{ $readDefaults 130 4428 @954 @953
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
	.debug	A gain 7 124
	.debug	A offset 9 124
;4429   *gain = 1.0;  *offset = 0.0; }
	.debug	# 4429
	ld	de,0
	ld	bc,0x3f80
	ld	hl,(ix+7)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	c,e
	ld	b,e
	ld	hl,(ix+9)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
@953:
	.debug	} @953 4429

@952	equ	0
	pop	ix
	lret
@954	equ	2
;4430 
;4431 _nearcall void readCalibPoints(uint ui, float *low, float *high)  {
	.align	4
$$readCalibPoints::
	ex	(sp),iy
$readCalibPoints::
	.debug	G $readCalibPoints 130 9
	.debug	{ $readCalibPoints 130 4431 @957 @956
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
	.debug	A low 7 124
	.debug	A high 9 124
;4432   *low = -9.5;  *high = 9.5; }
	.debug	# 4432
	ld	de,0
	ld	bc,0xc118
	ld	hl,(ix+7)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	b,0x41
	ld	hl,(ix+9)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
@956:
	.debug	} @956 4432

@955	equ	0
	pop	ix
	lret
@957	equ	2
;4433 
;4434 _nearcall char *readUnits(uint ui, char *buf)  { return "V"; }
	.align	4
$$readUnits::
	ex	(sp),iy
$readUnits::
	.debug	G $readUnits 131 9
	.debug	{ $readUnits 131 4434 @960 @959
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
	.debug	A buf 7 70
	.debug	# 4434
	ld	hl,@2+274
@959:
	.debug	} @959 4434

@958	equ	0
	pop	ix
	lret
@960	equ	2
;4435 
;4436 
;4437 //=========================================================================
;4438 _nearcall float setGetRaw(uint ui)  {
	.align	4
$$setGetRaw::
	ex	(sp),iy
$setGetRaw::
	.debug	G $setGetRaw 129 9
	.debug	{ $setGetRaw 129 4438 @963 @962
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
;4439   return (ui < NumHdwADC ? anaOut[ui] : 0.0);  }
	.debug	# 4439
	ld	hl,(ix+5)
	ld	de,(_NumHdwADC)
	or	a,a
	sbc	hl,de
	jr	nc,@964
	ld	hl,(ix+5)
	add	hl,hl
	add	hl,hl
	ld	de,_anaOut
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	jr	@962
@964:
	ld	de,0
	ld	c,e
	ld	b,e
@962:
	.debug	} @962 4439

@961	equ	0
	pop	ix
	lret
@963	equ	2
;4440 
;4441 _nearcall float setMinRaw(uint ui)  {
	.align	4
$$setMinRaw::
	ex	(sp),iy
$setMinRaw::
	.debug	G $setMinRaw 129 9
	.debug	{ $setMinRaw 129 4441 @968 @967
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
;4442   return (ui < NumHdwDAC) ? (dacMinRaw(ui) - dacOffset[ui]) * dacGain[ui] :
	.debug	# 4442
	ld	hl,(ix+5)
	ld	de,(_NumHdwDAC)
	or	a,a
	sbc	hl,de
	jr	nc,@969
	ld	hl,(ix+5)
	push	hl
	lcall	$dacMinRaw
	pop	hl
	ld	hl,(ix+5)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_dacOffset
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fsub##
	ld	hl,(ix+5)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_dacGain
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fmlt##
	jr	@967
@969:
	ld	de,0
	ld	c,e
	ld	b,e
@967:
	.debug	} @967 4442

@966	equ	0
	pop	ix
	lret
@968	equ	2
;4443 
;4444 _nearcall float setMaxRaw(uint ui)  {
	.align	4
$$setMaxRaw::
	ex	(sp),iy
$setMaxRaw::
	.debug	G $setMaxRaw 129 9
	.debug	{ $setMaxRaw 129 4444 @973 @972
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
;4445   return (ui < NumHdwDAC) ? (dacMaxRaw(ui) - dacOffset[ui]) * dacGain[ui] :
	.debug	# 4445
	ld	hl,(ix+5)
	ld	de,(_NumHdwDAC)
	or	a,a
	sbc	hl,de
	jr	nc,@974
	ld	hl,(ix+5)
	push	hl
	lcall	$dacMaxRaw
	pop	hl
	ld	hl,(ix+5)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_dacOffset
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fsub##
	ld	hl,(ix+5)
	add	hl,hl
	add	hl,hl
	push	bc
	push	de
	ld	de,_dacGain
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__fmlt##
	jr	@972
@974:
	ld	de,0xcccd
	ld	bc,0x411c
@972:
	.debug	} @972 4445

@971	equ	0
	pop	ix
	lret
@973	equ	2
;4446 
;4447 _nearcall void setDefaults(uint ui, float *gain, float *offset)  {
	.align	4
$$setDefaults::
	ex	(sp),iy
$setDefaults::
	.debug	G $setDefaults 130 9
	.debug	{ $setDefaults 130 4447 @978 @977
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
	.debug	A gain 7 124
	.debug	A offset 9 124
;4448   *gain = 1.0;  *offset = 0.0;  }
	.debug	# 4448
	ld	de,0
	ld	bc,0x3f80
	ld	hl,(ix+7)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	c,e
	ld	b,e
	ld	hl,(ix+9)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
@977:
	.debug	} @977 4448

@976	equ	0
	pop	ix
	lret
@978	equ	2
;4449 
;4450 _nearcall void setCalibPoints(uint ui, float *low, float *high)  {
	.align	4
$$setCalibPoints::
	ex	(sp),iy
$setCalibPoints::
	.debug	G $setCalibPoints 130 9
	.debug	{ $setCalibPoints 130 4450 @981 @980
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
	.debug	A low 7 124
	.debug	A high 9 124
;4451   *low = -9.5;  *high = 9.5;  }
	.debug	# 4451
	ld	de,0
	ld	bc,0xc118
	ld	hl,(ix+7)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	b,0x41
	ld	hl,(ix+9)
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
@980:
	.debug	} @980 4451

@979	equ	0
	pop	ix
	lret
@981	equ	2
;4452 
;4453 _nearcall char *setUnits(uint ui, char *buf)  { return "V"; }
	.align	4
$$setUnits::
	ex	(sp),iy
$setUnits::
	.debug	G $setUnits 131 9
	.debug	{ $setUnits 131 4453 @984 @983
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A ui 5 7
	.debug	A buf 7 70
	.debug	# 4453
	ld	hl,@2+274
@983:
	.debug	} @983 4453

@982	equ	0
	pop	ix
	lret
@984	equ	2
;4454 
;4455 
;4456 //=========================================================================
;4457 //  Set default labels for all the I/O signals
;4458 //=========================================================================
;4459 void setDefaultLabels(void)
;4460 {
$setDefaultLabels::
	.debug	G $setDefaultLabels 97 9
	.debug	{ $setDefaultLabels 97 4459 @987 @986
;4461   printf("Setting default labels for I/O signals\r\n");
	.debug	# 4461
	ld	de,word (phys @1+4281)
	ld	c,phys @1+4281 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;4462 
;4463 //--- set application-appropriate default labels ---
;4464 //APPLCODE
;4465 
;4466 //finish
;4467   fstrcpy(dinLabel[0], "FPGA_DONE");
	.debug	# 4467
	ld	de,word (phys @1+4322)
	ld	c,phys @1+4322 >> 16
	push	bc
	push	de
	ld	de,word (phys _dinLabel)
	ld	c,phys _dinLabel >> 16
	push	bc
	push	de
	call	_fstrcpy
	add	sp,8
;4468 
;4469   fstrcpy(doutLabel[0], "CPU_RDY");
	.debug	# 4469
	ld	de,word (phys @1+4332)
	ld	c,phys @1+4332 >> 16
	push	bc
	push	de
	ld	de,word (phys _doutLabel)
	ld	c,phys _doutLabel >> 16
	push	bc
	push	de
	call	_fstrcpy
	add	sp,8
;4470   fstrcpy(doutLabel[1], "ACTY");
	.debug	# 4470
	ld	de,word (phys @1+4340)
	ld	c,phys @1+4340 >> 16
	push	bc
	push	de
	ld	de,word (phys _doutLabel+24)
	ld	c,phys _doutLabel+24 >> 16
	push	bc
	push	de
	call	_fstrcpy
	add	sp,8
;4471 
;4472 }
@986:
	.debug	} @986 4472

@985	equ	0
	lret
@987	equ	0
;4473 
;4474 
;4475 //=========================================================================
;4476 //  This function is called to force software configurable values to defaul
;4477 //  values.
;4478 //=========================================================================
;4479 void setDefaultConfig(void)
;4480 {
$setDefaultConfig::
	.debug	G $setDefaultConfig 97 9
	.debug	{ $setDefaultConfig 97 4479 @990 @989
;4481   printf("Setting in-RAM device config to default values\r\n");
	.debug	# 4481
	ld	de,word (phys @1+4345)
	ld	c,phys @1+4345 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;4482 
;4483 //===== set default values for persistent data values =====
;4484 //APPLCODE
;4485 
;4486 //--- default values for application-specific data elements ---
;4487   rf_P = 0.9;   tuner_P = 0.9;
	.debug	# 4487
	ld	de,0x6666
	ld	bc,0x3f66
	ld	(_rf_P),de
	ld	(_rf_P+2),bc
	ld	(_tuner_P),de
	ld	(_tuner_P+2),bc
;4488   rf_I = 0.1;   tuner_I = 0.1;
	.debug	# 4488
	ld	de,0xcccd
	ld	bc,0x3dcc
	ld	(_rf_I),de
	ld	(_rf_I+2),bc
	ld	(_tuner_I),de
	ld	(_tuner_I+2),bc
;4489   rf_D = 0.01;  tuner_D = 0.01;
	.debug	# 4489
	ld	de,0xd70a
	ld	bc,0x3c23
	ld	(_rf_D),de
	ld	(_rf_D+2),bc
	ld	(_tuner_D),de
	ld	(_tuner_D+2),bc
;4490 
;4491   pulseFreq = 10;  trigPos = 128;
	.debug	# 4491
	ld	de,0
	ld	bc,0x4120
	ld	(_pulseFreq),de
	ld	(_pulseFreq+2),bc
	ld	e,0x80
	ld	c,d
	ld	b,d
	ld	(_trigPos),de
	ld	(_trigPos+2),bc
;4492 
;4493   fwdAtten = revAtten = cavAtten = 4;
	.debug	# 4493
	ld	hl,0x4
	ld	(_cavAtten),hl
	ld	(_revAtten),hl
	ld	(_fwdAtten),hl
;4494   amplSet = 1.0;  phaseSet = 90;
	.debug	# 4494
	ld	e,c
	ld	bc,0x3f80
	ld	(_amplSet),de
	ld	(_amplSet+2),bc
	ld	bc,0x42b4
	ld	(_phaseSet),de
	ld	(_phaseSet+2),bc
;4495 
;4496   warmLevel = 1.0;
	.debug	# 4496
	ld	bc,0x3f80
	ld	(_warmLevel),de
	ld	(_warmLevel+2),bc
;4497   warmTime = 1;
	.debug	# 4497
	ld	(_warmTime),de
	ld	(_warmTime+2),bc
;4498   rampTime = 2;
	.debug	# 4498
	ld	bc,0x4000
	ld	(_rampTime),de
	ld	(_rampTime+2),bc
;4499 
;4500 //  setAmplGain = setPhaseGain =
;4501   ilkRatioGain = maxOutAmplGain =
;4502   fwdAmplGain = fwdPhaseGain = revAmplGain = revPhaseGain =
;4503   cavAmplGain = cavPhaseGain = ref0AmplGain = ref0PhaseGain =
;4504   spare0PhaseGain = spare0AmplGain = spare1PhaseGain = spare1AmplGain  =
;4505   spare2PhaseGain = spare2AmplGain = ref1PhaseGain = ref1AmplGain = 1.0;
	.debug	# 4505
	ld	bc,0x3f80
	ld	(_ref1AmplGain),de
	ld	(_ref1AmplGain+2),bc
	ld	(_ref1PhaseGain),de
	ld	(_ref1PhaseGain+2),bc
	ld	(_spare2AmplGain),de
	ld	(_spare2AmplGain+2),bc
	ld	(_spare2PhaseGain),de
	ld	(_spare2PhaseGain+2),bc
	ld	(_spare1AmplGain),de
	ld	(_spare1AmplGain+2),bc
	ld	(_spare1PhaseGain),de
	ld	(_spare1PhaseGain+2),bc
	ld	(_spare0AmplGain),de
	ld	(_spare0AmplGain+2),bc
	ld	(_spare0PhaseGain),de
	ld	(_spare0PhaseGain+2),bc
	ld	(_ref0PhaseGain),de
	ld	(_ref0PhaseGain+2),bc
	ld	(_ref0AmplGain),de
	ld	(_ref0AmplGain+2),bc
	ld	(_cavPhaseGain),de
	ld	(_cavPhaseGain+2),bc
	ld	(_cavAmplGain),de
	ld	(_cavAmplGain+2),bc
	ld	(_revPhaseGain),de
	ld	(_revPhaseGain+2),bc
	ld	(_revAmplGain),de
	ld	(_revAmplGain+2),bc
	ld	(_fwdPhaseGain),de
	ld	(_fwdPhaseGain+2),bc
	ld	(_fwdAmplGain),de
	ld	(_fwdAmplGain+2),bc
	ld	(_maxOutAmplGain),de
	ld	(_maxOutAmplGain+2),bc
	ld	(_ilkRatioGain),de
	ld	(_ilkRatioGain+2),bc
;4506 
;4507 //  setAmplOffset = setPhaseOffset =
;4508   ilkRatioOffset = maxOutAmplOffset =
;4509   fwdAmplOffset = fwdPhaseOffset = revAmplOffset = revPhaseOffset =
;4510   cavAmplOffset = cavPhaseOffset = ref0AmplOffset = ref0PhaseOffset = 0.0;
	.debug	# 4510
	ld	c,e
	ld	b,e
	ld	(_ref0PhaseOffset),de
	ld	(_ref0PhaseOffset+2),bc
	ld	(_ref0AmplOffset),de
	ld	(_ref0AmplOffset+2),bc
	ld	(_cavPhaseOffset),de
	ld	(_cavPhaseOffset+2),bc
	ld	(_cavAmplOffset),de
	ld	(_cavAmplOffset+2),bc
	ld	(_revPhaseOffset),de
	ld	(_revPhaseOffset+2),bc
	ld	(_revAmplOffset),de
	ld	(_revAmplOffset+2),bc
	ld	(_fwdPhaseOffset),de
	ld	(_fwdPhaseOffset+2),bc
	ld	(_fwdAmplOffset),de
	ld	(_fwdAmplOffset+2),bc
	ld	(_maxOutAmplOffset),de
	ld	(_maxOutAmplOffset+2),bc
	ld	(_ilkRatioOffset),de
	ld	(_ilkRatioOffset+2),bc
;4511 
;4512   maxAmplSet = 999.0;
	.debug	# 4512
	ld	de,0xc000
	ld	bc,0x4479
	ld	(_maxAmplSet),de
	ld	(_maxAmplSet+2),bc
;4513 }
@989:
	.debug	} @989 4513

@988	equ	0
	lret
@990	equ	0
;4514 
;4515 //=========================================================================
;4516 //  This function should be called to prepare for a DSP or ZWorld reset
;4517 //=========================================================================
;4518 void prepForReset(void)
;4519 {
$prepForReset::
	.debug	G $prepForReset 97 9
	.debug	{ $prepForReset 97 4518 @993 @992
;4520 //finish  writeSetDSPState(CMD_RF_OFF);
;4521 
;4522 // do something more MB_setDSPState(...)
;4523 }
@992:
	.debug	} @992 4523

@991	equ	0
	lret
@993	equ	0
;4524 
;4525 //=========================================================================
;4526 //  This function is called by the application loop just before a shutdown.
;4527 //  The appliation must do whatever is necessary for a clean shutdown and
;4528 //  then exit
;4529 //=========================================================================
;4530 void applicShutdown(void)
;4531 {
$applicShutdown::
	.debug	G $applicShutdown 97 9
	.debug	{ $applicShutdown 97 4530 @996 @995
;4532   printf("Application prepping for shutdown\r\n");  fflush(stdout);
	.debug	# 4532
	ld	de,word (phys @1+4394)
	ld	c,phys @1+4394 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	ld	hl,(__stdio)
	push	hl
	lcall	$fflush
	pop	de
;4533 
;4534   prepForReset();
	.debug	# 4534
	lcall	$prepForReset
;4535 
;4536   Exit(0);
	.debug	# 4536
	bool	hl
	ld	l,h
	push	hl
	lcall	$Exit
	pop	hl
;4537 }
@995:
	.debug	} @995 4537

@994	equ	0
	lret
@996	equ	0
;4538 
;4539 //=========================================================================
;4540 //  This is the highest priority task, usually resumed at a regular interva
;4541 //  so that it can do what it needs and sleep again until the start of the
;4542 //  next interval.
;4543 //=========================================================================
;4544 void _task applic(int argc, char **argv)
;4545 {
	.align	4
$$applic::
	ex	(sp),iy
$applic::
	.debug	G $applic 111 9
	.debug	{ $applic 111 4544 @999 @998
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-118
	.debug	A argc 5 6
	.debug	A argv 7 112
;4546   #define SharedApplLocals
;4547   #include <sharedAppl.c>
;4548 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
	.debug	L idleMS -4 9
	.debug	L secs -8 9
	.debug	L nanoSecs -12 9
	.debug	L terminateTime -16 9
	.debug	L startTime -20 9
	.debug	L u -22 7
	.debug	L ui -24 7
	.debug	L intTime -26 7
	.debug	L idleInterval -28 7
	.debug	L numBytes -30 7
	.debug	L ms -32 7
	.debug	L loopCountdown -34 7
	.debug	L stat -36 6
	.debug	L eventsfd -38 6
	.debug	L argv_ -44 132
	.debug	L sigSet -48 9
	.debug	L reg -50 38
	.debug	I
;4549 //--- add local variable definitions here ---
;4550 //  fd_t  fpgaProgfd;
;4551   LockData  *lock, *endLocks;
	.debug	L lock -52 74
	.debug	L endLocks -54 74
;4552 
;4553 
;4554   #define SharedApplLocals2
;4555   #include <sharedAppl.c>
;4556 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
	.debug	L buf -118 126
	.debug	I
;4557 
;4558 
;4559   #define SharedApplInit
;4560   #include <sharedAppl.c>
;4561 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
	.debug	# 866
	ld	hl,0x2
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_prevState
	push	hl
	call	_memset
	add	sp,6
	ld	hl,0x2
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_curState
	push	hl
	call	_memset
	add	sp,6
	ld	hl,0x2
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_nextState
	push	hl
	call	_memset
	add	sp,6
	.debug	# 868
	bool	hl
	ld	l,h
	ld	(_numScopeRegs),hl
	.debug	# 870
	lcall	$resetWDT
	.debug	# 873
	ld	de,0
	ld	c,e
	ld	b,e
	ld	(_ADC_Mask),de
	ld	(_ADC_Mask+2),bc
	ld	(_DAC_Mask),de
	ld	(_DAC_Mask+2),bc
	.debug	# 876
	ld	(_capturePtr),de
	ld	(_capturePtr+2),bc
@1000:
	.debug	< @1000 @1001
	.debug	# 880
	bool	hl
	ld	l,h
	ld	(ix-24),hl
@1002:
	ld	hl,(ix-24)
	ld	de,(_NumHdwADC)
	or	a,a
	sbc	hl,de
	jr	nc,@1003
@1005:
	.debug	< @1005 @1006
	.debug	# 881
	ld	hl,(ix-24)
	add	hl,hl
	add	hl,hl
	ld	de,_adcOffset
	add	hl,de
	push	hl
	ld	hl,(ix-24)
	add	hl,hl
	add	hl,hl
	ld	de,_adcGain
	add	hl,de
	push	hl
	ld	hl,(ix-24)
	push	hl
	lcall	$adcDefaults
	add	sp,6
	.debug	# 882
	ld	hl,(ix-24)
	add	hl,hl
	ld	de,_lastadcRaw
	add	hl,de
	ld	de,0xbad
	ld	(hl),e
	inc	hl
	ld	(hl),d
	.debug	# 883
	ld	hl,(ix-24)
	push	hl
	lcall	$newADCCalib
	pop	hl
@1006:
	.debug	>
	ld	hl,(ix-24)
	inc	hl
	ld	(ix-24),hl
	jr	@1002
@1003:
@1001:
	.debug	>
@1007:
	.debug	< @1007 @1008
	.debug	# 885
	bool	hl
	ld	l,h
	ld	(ix-24),hl
@1009:
	ld	hl,(ix-24)
	ld	de,(_NumHdwDAC)
	or	a,a
	sbc	hl,de
	jr	nc,@1010
@1012:
	.debug	< @1012 @1013
	.debug	# 886
	ld	hl,(ix-24)
	add	hl,hl
	add	hl,hl
	ld	de,_dacOffset
	add	hl,de
	push	hl
	ld	hl,(ix-24)
	add	hl,hl
	add	hl,hl
	ld	de,_dacGain
	add	hl,de
	push	hl
	ld	hl,(ix-24)
	push	hl
	lcall	$dacDefaults
	add	sp,6
	.debug	# 887
	ld	hl,(ix-24)
	push	hl
	lcall	$newDACCalib
	pop	hl
@1013:
	.debug	>
	ld	hl,(ix-24)
	inc	hl
	ld	(ix-24),hl
	jr	@1009
@1010:
@1008:
	.debug	>
@1014:
	.debug	< @1014 @1015
	.debug	# 889
	bool	hl
	ld	l,h
	ld	(ix-24),hl
@1016:
	ld	hl,(ix-24)
	ld	de,(_NumApplAIN)
	or	a,a
	sbc	hl,de
	jr	nc,@1017
@1019:
	.debug	< @1019 @1020
	.debug	# 890
	ld	hl,(ix-24)
	add	hl,hl
	add	hl,hl
	ld	de,_readOffset
	add	hl,de
	push	hl
	ld	hl,(ix-24)
	add	hl,hl
	add	hl,hl
	ld	de,_readGain
	add	hl,de
	push	hl
	ld	hl,(ix-24)
	push	hl
	lcall	$readDefaults
	add	sp,6
	.debug	# 891
	ld	hl,(ix-24)
	add	hl,hl
	add	hl,hl
	ld	de,_readVal
	add	hl,de
	ld	de,0
	ld	c,e
	ld	b,e
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	.debug	# 892
	ld	hl,(ix-24)
	ld	de,(_NumHdwADC)
	or	a,a
	sbc	hl,de
	jr	z,@1022
	jr	nc,@1021
@1022:
	ld	hl,(ix-24)
	push	hl
	lcall	$newADCCalib
	pop	hl
@1021:
@1020:
	.debug	>
	ld	hl,(ix-24)
	inc	hl
	ld	(ix-24),hl
	jr	@1016
@1017:
@1015:
	.debug	>
@1023:
	.debug	< @1023 @1024
	.debug	# 894
	bool	hl
	ld	l,h
	ld	(ix-24),hl
@1025:
	ld	hl,(ix-24)
	ld	de,(_NumApplAOUT)
	or	a,a
	sbc	hl,de
	jr	nc,@1026
@1028:
	.debug	< @1028 @1029
	.debug	# 895
	ld	hl,(ix-24)
	add	hl,hl
	add	hl,hl
	ld	de,_setOffset
	add	hl,de
	push	hl
	ld	hl,(ix-24)
	add	hl,hl
	add	hl,hl
	ld	de,_setGain
	add	hl,de
	push	hl
	ld	hl,(ix-24)
	push	hl
	lcall	$setDefaults
	add	sp,6
	.debug	# 896
	ld	hl,(ix-24)
	add	hl,hl
	add	hl,hl
	ld	de,_setVal
	add	hl,de
	ld	de,0
	ld	c,e
	ld	b,e
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	.debug	# 897
	ld	hl,(ix-24)
	ld	de,(_NumHdwDAC)
	or	a,a
	sbc	hl,de
	jr	z,@1031
	jr	nc,@1030
@1031:
	ld	hl,(ix-24)
	push	hl
	lcall	$newDACCalib
	pop	hl
@1030:
@1029:
	.debug	>
	ld	hl,(ix-24)
	inc	hl
	ld	(ix-24),hl
	jr	@1025
@1026:
@1024:
	.debug	>
	.debug	# 900
	lcall	$setGenericLabels
	.debug	# 904
	ld	de,0x4000
	ld	c,e
	ld	b,e
	ld	hl,bc
	ld	(ix-46),hl
	ex	de,hl
	ld	(ix-48),hl
	ex	de,hl
	push	hl
	ld	hl,-46+@997
	add	hl,sp
	push	hl
	ld	hl,0x1
	push	hl
	lcall	$sigprocmask
	add	sp,6
	.debug	# 906
	ld	de,word (phys @1+4430)
	ld	c,phys @1+4430 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	.debug	# 908
	lcall	$initModBusRegs
	.debug	# 910
	ld	hl,0x57
	ld	(_devType),hl
	.debug	# 915
	ld	hl,_netmask
	push	hl
	ld	hl,@2+276
	push	hl
	ld	hl,0x2
	push	hl
	lcall	$inet_ptoh
	add	sp,6
	.debug	# 916
	ld	hl,_gateway
	push	hl
	ld	hl,@2+288
	push	hl
	ld	hl,0x2
	push	hl
	lcall	$inet_ptoh
	add	sp,6
	.debug	# 920
	bool	hl
	ld	l,h
	push	hl
	ld	hl,@2+300
	push	hl
	lcall	$mkfifo
	add	sp,4
	.debug	# 921
	bool	hl
	ld	l,h
	push	hl
	ld	l,0x21
	push	hl
	ld	hl,@2+300
	push	hl
	lcall	$open
	add	sp,6
	ld	(ix-38),hl
	.debug	# 924
	ld	hl,(__TimerMilliSeconds+4)
	ld	(_lastIdleTime),hl
	ld	de,0
	ld	c,e
	ld	b,e
	ld	(_lastIdleMS),de
	ld	(_lastIdleMS+2),bc
	.debug	# 930
	bool	hl
	ld	l,h
	ld	(_ctlLoopTicks),hl
	ld	(_lastCtlLoopTime),hl
	.debug	# 932
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
	.debug	# 935
	lcall	$setDefaultLabels
	lcall	$setDefaultConfig
	.debug	# 938
	lcall	$initIOvalues
	.debug	# 953
	ld	de,word (phys _prevLogInfo+116)
	ld	c,phys _prevLogInfo+116 >> 16
	ld	hl,de
	ld	a,c
	ldp	hl,(hl)
	ld	a,l
	or	a,a
	jr	z,@1032
@1033:
	.debug	< @1033 @1034
	.debug	# 954
	lcall	$restoreApplState
	ld	(ix-36),hl
	.debug	# 955
	bool	hl
	jr	z,@1035
@1036:
	.debug	< @1036 @1037
	.debug	# 957
	ld	hl,(ix-36)
	push	hl
	ld	de,word (phys @1+4455)
	ld	c,phys @1+4455 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
	.debug	# 958
	ld	a,(_runningPrimary)
	or	a,a
	jr	z,@1038
	xor	a,a
	ld	(_runBackup),a
@1038:
	.debug	# 959
	ld	hl,0x1
	push	hl
	lcall	$REBOOT
	pop	hl
@1037:
	.debug	>
@1035:
	.debug	# 961
	ld	a,0x1
	ld	(_restoredApplState),a
@1034:
	.debug	>
@1032:
	.debug	# 965
	ld	hl,0x100
	push	hl
	ld	hl,_applBuf
	push	hl
	lcall	$initParamModule
	add	sp,4
	.debug	# 968
	ld	a,(_restoredApplState)
	or	a,a
	jr	nz,@1039
@1040:
	.debug	< @1040 @1041
	.debug	# 969
	ld	de,(_digitalIn)
	ld	bc,(_digitalIn+2)
	ld	(_nextState),de
	ld	(_nextState+2),bc
	ld	(_curState),de
	ld	(_curState+2),bc
	ld	(_prevState),de
	ld	(_prevState+2),bc
	.debug	# 970
	ld	de,(_digitalOut)
	ld	bc,(_digitalOut+2)
	ld	(_nextState+4),de
	ld	(_nextState+6),bc
	ld	(_curState+4),de
	ld	(_curState+6),bc
	ld	(_prevState+4),de
	ld	(_prevState+6),bc
@1041:
	.debug	>
@1039:
	.debug	# 974
	bool	hl
	ld	l,h
	push	hl
	lcall	$startNetUp
	pop	de
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	ld	hl,bc
	ld	(ix-18),hl
	ex	de,hl
	ld	(ix-20),hl
	ex	de,hl
@1042:
	.debug	# 975
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	push	bc
	push	de
	ld	hl,(ix-20)
	ex	de,hl
	ld	hl,(ix-18)
	ld	bc,hl
	call	__lsub##
	push	bc
	push	de
	ld	de,0x2710
	ld	bc,0
	call	__lsub##
	jr	nc,@1044
	bool	hl
	ld	l,h
	push	hl
	lcall	$ifpending
	pop	de
	ld	de,0x2
	or	a,a
	sbc	hl,de
	jr	z,@1044
	.debug	# 976
	lcall	$resetWDT
	jp	@1042
@1044:
	.debug	# 977
	bool	hl
	ld	l,h
	push	hl
	lcall	$ifpending
	pop	de
	ld	de,0x2
	or	a,a
	sbc	hl,de
	jr	z,@1045
	.debug	# 978
	ld	de,word (phys @1+4514)
	ld	c,phys @1+4514 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	jr	@1046
@1045:
	.debug	# 980
	bool	hl
	ld	l,h
	push	hl
	lcall	$showNetState
	pop	hl
@1046:
@1047:
	.debug	< @1047 @1048
	.debug	# 983
	bool	hl
	ld	l,h
	ld	(ix-24),hl
@1049:
	ld	hl,(ix-24)
	ld	de,(_NumHdwADC)
	or	a,a
	sbc	hl,de
	jr	nc,@1050
	ld	hl,(ix-24)
	push	hl
	lcall	$newADCCalib
	pop	hl
	ld	hl,(ix-24)
	inc	hl
	ld	(ix-24),hl
	jr	@1049
@1050:
@1048:
	.debug	>
@1052:
	.debug	< @1052 @1053
	.debug	# 984
	bool	hl
	ld	l,h
	ld	(ix-24),hl
@1054:
	ld	hl,(ix-24)
	ld	de,(_NumHdwDAC)
	or	a,a
	sbc	hl,de
	jr	nc,@1055
	ld	hl,(ix-24)
	push	hl
	lcall	$newDACCalib
	pop	hl
	.debug	I
;4562   _EnterCritical_();  SchedMS = 20;  _ExitCritical_();
	ld	hl,(ix-24)
	inc	hl
	ld	(ix-24),hl
	jr	@1054
@1055:
@1053:
	.debug	>
	.debug	# 4562
	ipset	2
	ld	hl,0x14
	ld	(_SchedMS),hl
	ipres
;4563 
;4564 //--- add application initialization code here ---
;4565 
;4566   autoReadDSP = autoWriteDSP = autoReadFPGA = autoWriteFPGA = readWaves = t
	.debug	# 4566
	ld	a,0x1
	ld	(_readWaves),a
	ld	(_autoWriteFPGA),a
	ld	(_autoReadFPGA),a
	ld	(_autoWriteDSP),a
	ld	(_autoReadDSP),a
;4567   showFiller = false;
	.debug	# 4567
	xor	a,a
	ld	(_showFiller),a
;4568 
;4569   rfq_t1_feedback_RegInfo.flags ^= LogChanges;
	.debug	# 4569
	ld	a,(_rfq_t1_feedback_RegInfo+13)
	xor	a,2
	ld	(_rfq_t1_feedback_RegInfo+13),a
;4570   rfq_t2_feedback_RegInfo.flags ^= LogChanges;
	.debug	# 4570
	ld	a,(_rfq_t2_feedback_RegInfo+13)
	xor	a,2
	ld	(_rfq_t2_feedback_RegInfo+13),a
;4571 
;4572   rfq_t1_output_RegInfo.flags ^= LogChanges;
	.debug	# 4572
	ld	a,(_rfq_t1_output_RegInfo+13)
	xor	a,2
	ld	(_rfq_t1_output_RegInfo+13),a
;4573   rfq_t2_output_RegInfo.flags ^= LogChanges;
	.debug	# 4573
	ld	a,(_rfq_t2_output_RegInfo+13)
	xor	a,2
	ld	(_rfq_t2_output_RegInfo+13),a
;4574 
;4575 
;4576 //--- initialize the locks ---
;4577   endLocks = locks + NUM_LOCKS;
	.debug	# 4577
	ld	hl,_locks+286
	ld	(ix-54),hl
;4578   for (lock=locks; lock<endLocks; ++lock)  {
@1057:
	.debug	< @1057 @1058
	.debug	# 4578
	ld	hl,_locks
	ld	(ix-52),hl
@1059:
	ld	hl,(ix-52)
	ex	de,hl
	ld	hl,(ix-54)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	nc,@1060
;4579     lock->count = 0;  lock->owner = 0;
@1062:
	.debug	< @1062 @1063
	.debug	# 4579
	ld	hl,(ix-52)
	ld	de,0
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,(ix-52)
	inc	hl
	inc	hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;4580     sem_init(&lock->sem, true, 1);
	.debug	# 4580
	ld	hl,0x1
	push	hl
	push	hl
	ld	hl,(ix-52)
	ld	e,0x4
	add	hl,de
	push	hl
	lcall	$sem_init
	add	sp,6
;4581 //    init_waitqueue_head(&lock->tasks);
;4582   }
@1063:
	.debug	>
;4583 
;4584 
;4585 //  dspProgNum = 0;
;4586 
;4587 
;4588   fpgaInitialized = pulseAmpl = pulsePhase = false;
	ld	hl,(ix-52)
	ld	de,0x1a
	add	hl,de
	ld	(ix-52),hl
	jr	@1059
@1060:
@1058:
	.debug	>
	.debug	# 4588
	xor	a,a
	ld	(_pulsePhase),a
	ld	(_pulseAmpl),a
	ld	(_fpgaInitialized),a
;4589 
;4590 //--- set defaults for # of data points in each waveform ---
;4591   FBZERO(waveformData);  BZERO(readerRdy);
	.debug	# 4591
	ld	de,0x6000
	ld	c,e
	ld	b,e
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	ld	de,word (phys _waveformData)
	ld	c,phys _waveformData >> 16
	push	bc
	push	de
	call	_fmemset
	add	sp,10
	ld	hl,0x6
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_readerRdy
	push	hl
	call	_memset
	add	sp,6
;4592   BZERO(dataTimeSecs);  BZERO(dataTimeMS);
	.debug	# 4592
	ld	hl,0x18
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_dataTimeSecs
	push	hl
	call	_memset
	add	sp,6
	ld	hl,0x18
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_dataTimeMS
	push	hl
	call	_memset
	add	sp,6
;4593   for (ui=0; ui<NUM_WAVEFORMS*2; ++ui)  {
@1064:
	.debug	< @1064 @1065
	.debug	# 4593
	bool	hl
	ld	l,h
	ld	(ix-24),hl
@1066:
	ld	hl,(ix-24)
	ld	de,0x6
	or	a,a
	sbc	hl,de
	jr	nc,@1067
;4594     numDataPoints[ui] = MaxDataPoints;
@1069:
	.debug	< @1069 @1070
	.debug	# 4594
	ld	hl,(ix-24)
	add	hl,hl
	ld	de,_numDataPoints
	add	hl,de
	ld	de,0x400
	ld	(hl),e
	inc	hl
	ld	(hl),d
;4595     lastWaveReadTime[ui] = GetMS_();
	.debug	# 4595
	ld	hl,(ix-24)
	add	hl,hl
	ld	de,_lastWaveReadTime
	add	hl,de
	ld	de,(__TimerMilliSeconds+4)
	ld	(hl),e
	inc	hl
	ld	(hl),d
;4596   }
@1070:
	.debug	>
;4597 
;4598 
;4599 
;4600 //--- initialize ... ---
;4601   initDefaults();
	ld	hl,(ix-24)
	inc	hl
	ld	(ix-24),hl
	jr	@1066
@1067:
@1065:
	.debug	>
	.debug	# 4601
	lcall	$initDefaults
;4602 
;4603   SetBits(Next.digitalOut, READY_LED);
	.debug	# 4603
	ld	de,(_nextState+4)
	ld	bc,(_nextState+6)
	set	0,e
	ld	(_nextState+4),de
	ld	(_nextState+6),bc
;4604 
;4605   if (getLock(APPL_LOCK, 10000) > 0)  {
	.debug	# 4605
	ld	de,0x2710
	ld	bc,0
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	lcall	$getLock
	add	sp,6
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@1071
	jr	z,@1071
;4606     stop_DSP_FPGA();  stat = restart_DSP_FPGA(10000);
@1072:
	.debug	< @1072 @1073
	.debug	# 4606
	lcall	$stop_DSP_FPGA
	ld	de,0x2710
	ld	bc,0
	push	bc
	push	de
	lcall	$restart_DSP_FPGA
	add	sp,4
	ld	(ix-36),hl
;4607     releaseLock(APPL_LOCK);
	.debug	# 4607
	bool	hl
	ld	l,h
	push	hl
	lcall	$releaseLock
	pop	de
;4608   }
@1073:
	.debug	>
;4609 
;4610 
;4611   #define SharedApplInit2
;4612   #include <sharedAppl.c>
;4613 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
@1071:
	.debug	# 993
	lcall	$resetWDT
	.debug	# 996
	ld	hl,0x1
	push	hl
	lcall	$logDigIOChgs
	pop	hl
	.debug	# 999
	ld	hl,@2+311
	ld	(ix-44),hl
	bool	hl
	ld	l,h
	ld	(ix-42),hl
	.debug	# 1000
	ld	l,0x2
	push	hl
	dec	l
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-38+@997
	add	hl,sp
	push	hl
	ld	hl,0x1
	push	hl
	ld	l,0x3
	push	hl
	ld	hl,_inetd
	push	hl
	lcall	$createProc
	add	sp,14
	.debug	# 1002
	ld	hl,@2+317
	ld	(ix-44),hl
	bool	hl
	ld	l,h
	ld	(ix-42),hl
	.debug	# 1003
	ld	l,0x2
	push	hl
	dec	l
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-38+@997
	add	hl,sp
	push	hl
	ld	hl,0x1
	push	hl
	ld	l,0x7
	push	hl
	ld	hl,_configd
	push	hl
	lcall	$createProc
	add	sp,14
	.debug	# 1005
	ld	hl,@2+325
	ld	(ix-44),hl
	bool	hl
	ld	l,h
	ld	(ix-42),hl
	.debug	# 1006
	ld	l,0x2
	push	hl
	dec	l
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-38+@997
	add	hl,sp
	push	hl
	ld	hl,0x1
	push	hl
	ld	l,0x4
	push	hl
	ld	hl,_backupd
	push	hl
	lcall	$createProc
	add	sp,14
	.debug	# 1008
	ld	hl,@2+333
	ld	(ix-44),hl
	bool	hl
	ld	l,h
	ld	(ix-42),hl
	.debug	# 1009
	ld	l,0x2
	push	hl
	dec	l
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-38+@997
	add	hl,sp
	push	hl
	ld	hl,0x1
	push	hl
	ld	l,0xb
	push	hl
	ld	hl,_ca_tcpd
	push	hl
	lcall	$createProc
	add	sp,14
	.debug	# 1012
	ld	hl,0x5dc
	ld	(ix-34),hl
	.debug	# 1014
	lcall	$resetWDT
	.debug	# 1016
	bool	hl
	ld	l,h
	ld	(_applicTicks),hl
	.debug	# 1018
	ld	hl,(__TimerMilliSeconds+4)
	ld	(ix-26),hl
	bool	hl
	ld	l,h
	ld	(_applLoopTime),hl
	.debug	# 1020
	lcall	$xavail
	push	bc
	push	de
	ld	de,word (phys @1+4555)
	ld	c,phys @1+4555 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
	.debug	# 1035
	ld	a,0x1
	ld	(_startupComplete),a
	.debug	I
;4614   lastIOTime = GetMS_();
	.debug	# 4614
	ld	hl,(__TimerMilliSeconds+4)
	ld	(_lastIOTime),hl
;4615 
;4616 //  calcStartTimeCTF = startCTFWaitTime = 0;
;4617 
;4618 //  resetDSP();
;4619 
;4620 //===================================================
;4621 //===== start loop that runs once per timer msg =====
;4622 //===================================================
;4623 
;4624 //--- The following loop is run at a periodic interval (normally 100 times 
;4625 //--- per second, or every 10ms).  The application process normally ---
;4626 //--- consists of reading the current state of the digital and analog ---
;4627 //--- inputs, changing the state of the analog and digital outputs to the -
;4628 //--- values computed during the previous pass, then computing new values -
;4629 //--- for the outputs and updating values that outside clients can view ---
;4630 //--- (currently accessable as a set of ModBus registers) ---
;4631 
;4632   for (;;)  {
@1074:
	.debug	< @1074 @1075
@1076:
;4633     #define SharedApplLoopStart
;4634     #include <sharedAppl.c>
;4635 
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
@1079:
	.debug	< @1079 @1080
	.debug	# 1046
	ld	hl,(_applicTicks)
	inc	hl
	ld	(_applicTicks),hl
	.debug	# 1052
	ld	hl,(ix-34)
	bool	hl
	jr	z,@1081
	.debug	# 1053
	ld	hl,(ix-34)
	dec	hl
	ld	(ix-34),hl
	bool	hl
	jr	nz,@1082
	xor	a,a
	ld	(_runBackup),a
@1082:
@1081:
	.debug	# 1056
	lcall	$resetWDT
	.debug	# 1058
	ld	hl,(__TimerMilliSeconds+4)
	ex	de,hl
	ld	hl,(ix-26)
	ex	de,hl
	or	a,a
	sbc	hl,de
	ld	(_applLoopTime),hl
	.debug	# 1063
	ld	hl,(_current)
	ld	de,0x13
	add	hl,de
	ld	e,0x2
	ld	(hl),e
	inc	hl
	ld	(hl),d
	lcall	$schedule
	.debug	# 1069
	ld	hl,(__TimerMilliSeconds+4)
	ld	(ix-26),hl
	.debug	# 1072
	ipset	2
	ld	a,(_intTicks)
	bool	hl
	ld	l,a
	ld	(_latency),hl
	ipres
	.debug	# 1078
	ld	hl,(_ctlLoopTicks)
	inc	hl
	ld	(_ctlLoopTicks),hl
	.debug	# 1079
	ld	hl,(ix-26)
	ld	de,(_lastCtlLoopTime)
	or	a,a
	sbc	hl,de
	ld	de,0x7d0
	or	a,a
	sbc	hl,de
	jr	c,@1083
@1084:
	.debug	< @1084 @1085
	.debug	# 1080
	ld	hl,(_ctlLoopTicks)
	or	a,a
	rr	hl
	ld	(_ctlLoopTicksPerSec),hl
	.debug	# 1081
	bool	hl
	ld	l,h
	ld	(_ctlLoopTicks),hl
	ld	hl,(ix-26)
	ld	(_lastCtlLoopTime),hl
@1085:
	.debug	>
	.debug	I
;4636   //--- read the current state of the analog and digital Input signals, and
;4637   //--- write the values computed at the end of the last pass ---
;4638     digitalOut = Next.digitalOut;  rwIO();
@1083:
	.debug	# 4638
	ld	de,(_nextState+4)
	ld	bc,(_nextState+6)
	ld	(_digitalOut),de
	ld	(_digitalOut+2),bc
	lcall	$rwIO
;4639 
;4640   //--- limit DSP/FPGA communication to aprox 15 times per second ---
;4641     if (msSince_(lastIOTime) >= 50)  {
	.debug	# 4641
	ld	hl,(__TimerMilliSeconds+4)
	ld	de,(_lastIOTime)
	or	a,a
	sbc	hl,de
	ld	de,0x32
	or	a,a
	sbc	hl,de
	jr	c,@1086
;4642 
;4643     //--- process next event message (if any) ---
;4644       processMsg(eventsfd, (EventMsg *)buf, sizeof(buf));
@1087:
	.debug	< @1087 @1088
	.debug	# 4644
	ld	hl,0x40
	push	hl
	ld	hl,-116+@997
	add	hl,sp
	push	hl
	ld	hl,(ix-38)
	push	hl
	lcall	$processMsg
	add	sp,6
;4645 
;4646     //--- update the current state and determine the next state ---
;4647       updateCurState();  updateNextState();
	.debug	# 4647
	lcall	$updateCurState
	lcall	$updateNextState
;4648       updateOutputState();  updateExternals();
	.debug	# 4648
	lcall	$updateOutputState
	lcall	$updateExternals
;4649     }
@1088:
	.debug	>
;4650 
;4651     #define SharedApplLoopEnd
;4652     #include <sharedAppl.c>
	.debug	I 'C:\work\dev\Rabbit_ST\appl\_shared_\sharedAppl.c' 17 3 10 5 12 32
@1086:
	.debug	# 1093
	lcall	$ReadDeviceID
	ld	(_deviceID),de
	ld	(_deviceID+2),bc
	.debug	# 1096
	ld	hl,(__TimerMilliSeconds+4)
	ld	de,(_lastIdleTime)
	or	a,a
	sbc	hl,de
	ld	(ix-28),hl
	ld	de,0xbb8
	or	a,a
	sbc	hl,de
	jr	c,@1089
@1090:
	.debug	< @1090 @1091
	.debug	# 1097
	ld	hl,(_idleTask)
	ld	de,0xd3
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
	.debug	# 1098
	push	bc
	push	de
	ld	de,(_lastIdleMS)
	ld	bc,(_lastIdleMS+2)
	call	__lsub##
	push	bc
	push	de
	ld	de,0x64
	ld	c,d
	ld	b,d
	call	__ulmlt##
	ld	hl,(ix-28)
	push	bc
	push	de
	ex	de,hl
	ld	bc,0
	call	__uldiv##
	ld	(_percentIdle),de
	.debug	# 1099
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	ld	(_lastIdleMS),de
	ld	(_lastIdleMS+2),bc
	ld	hl,(__TimerMilliSeconds+4)
	ld	(_lastIdleTime),hl
@1091:
	.debug	>
@1089:
	.debug	# 1102
	ld	hl,(_devType)
	ld	(_devType_),hl
	.debug	# 1103
	ld	de,(_deviceID)
	ld	bc,(_deviceID+2)
	ld	(_deviceID_),de
	ld	(_deviceID_+2),bc
	.debug	# 1104
ipset 3
	ld	de,(__TimerSeconds)
	ld	bc,(__TimerSeconds+2)
ipres
	ld	(_upSecs),de
	ld	(_upSecs+2),bc
	.debug	# 1105
	ld	hl,(_percentIdle)
	ld	(_percentIdle_),hl
	.debug	# 1107
	ld	hl,(_timeZone)
	ld	(_timeZone_),hl
	.debug	# 1108
	bool	hl
	ld	l,h
	push	hl
	lcall	$time
	pop	hl
	ld	(_curTime),de
	ld	(_curTime+2),bc
	.debug	# 1110
	ld	de,(_digitalIn)
	ld	bc,(_digitalIn+2)
	ld	(_digInBits),de
	ld	(_digInBits+2),bc
	.debug	# 1111
	ld	de,(_digitalOut)
	ld	bc,(_digitalOut+2)
	ld	(_digOutBits),de
	ld	(_digOutBits+2),bc
	.debug	# 1113
	ld	hl,-48+@997
	add	hl,sp
	push	hl
	lcall	$sigpending
	pop	de
	bool	hl
	jp	nz,@1092
	ld	l,0xf
	push	hl
	ld	hl,-46+@997
	add	hl,sp
	push	hl
	lcall	$sigismember
	add	sp,4
	bool	hl
	jp	z,@1093
	.debug	# 1114
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	or	hl,de
	jp	nz,@1094
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
@1094:
@1093:
@1092:
	.debug	# 1116
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	or	hl,de
	jp	z,@1096
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	push	bc
	push	de
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__lsub##
	push	bc
	push	de
	ld	de,0x1f40
	ld	bc,0
	call	__lsub##
	jr	c,@1096
	jr	z,@1096
	lcall	$applicShutdown
@1096:
	.debug	# 1119
	ld	hl,(__TimerMilliSeconds+4)
	ld	de,0x40
	and	hl,de
	jr	z,@1097
	.debug	# 1120
	ld	de,(_nextState+4)
	ld	bc,(_nextState+6)
	res	1,e
	ld	(_nextState+4),de
	ld	(_nextState+6),bc
	jr	@1098
@1097:
	.debug	# 1122
	ld	de,(_nextState+4)
	ld	bc,(_nextState+6)
	set	1,e
	ld	(_nextState+4),de
	ld	(_nextState+6),bc
@1098:
	.debug	# 1126
	ld	de,(_capturePtr)
	ld	bc,(_capturePtr+2)
	ld	a,e
	or	a,d
	or	a,c
	jp	z,@1099
@1100:
	.debug	< @1100 @1101
	.debug	# 1129
	ipset	2
	.debug	# 1130
	ld	de,(_hdwResetTime)
	ld	bc,(_hdwResetTime+2)
	push	bc
	push	de
	ld	de,(__TimerSeconds)
	ld	bc,(__TimerSeconds+2)
	call	__ladd##
	ld	hl,bc
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
	ld	hl,(_milliTicks)
	ld	(ix-32),hl
	.debug	# 1131
	ipres
	.debug	# 1134
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	de,0x4240
	ld	c,0xf
	call	__ulmlt##
	ld	hl,bc
	ld	(ix-10),hl
	ex	de,hl
	ld	(ix-12),hl
	ex	de,hl
	.debug	# 1135
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	ex	de,hl
	ld	e,b
	ld	d,c
	ld	b,l
	ld	c,h
	push	bc
	push	de
	ld	de,(_capturePtr)
	ld	bc,(_capturePtr+2)
	call	__farst4p##
	ld	de,(_capturePtr)
	ld	bc,(_capturePtr+2)
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	(_capturePtr),de
	ld	(_capturePtr+2),bc
	.debug	# 1136
	ld	hl,(ix-12)
	ex	de,hl
	ld	hl,(ix-10)
	ld	bc,hl
	ex	de,hl
	ld	e,b
	ld	d,c
	ld	b,l
	ld	c,h
	push	bc
	push	de
	ld	de,(_capturePtr)
	ld	bc,(_capturePtr+2)
	call	__farst4p##
	ld	de,(_capturePtr)
	ld	bc,(_capturePtr+2)
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	(_capturePtr),de
	ld	(_capturePtr+2),bc
@1102:
	.debug	< @1102 @1103
	.debug	# 1138
	bool	hl
	ld	l,h
	ld	(ix-22),hl
	ld	hl,(_scopeList)
	ld	(ix-50),hl
@1104:
	ld	hl,(ix-22)
	ld	de,(_numScopeRegs)
	or	a,a
	sbc	hl,de
	jp	nc,@1105
@1107:
	.debug	< @1107 @1108
	.debug	# 1139
	ld	hl,(ix-50)
	ld	de,0xb
	add	hl,de
	ld	a,(hl)
	bool	hl
	ld	l,a
	add	hl,hl
	ld	(ix-30),hl
	.debug	# 1140
	ld	e,0x2
	or	a,a
	sbc	hl,de
	jp	nz,@1109
	.debug	# 1141
	ld	hl,(ix-50)
	ld	hl,(hl)
	ld	hl,(hl)
	ld	a,l
	ld	l,h
	ld	h,a
	ld	de,(_capturePtr)
	ld	bc,(_capturePtr+2)
	call	__farst2##
	jp	@1110
@1109:
	.debug	# 1143
	ld	hl,(ix-50)
	ld	hl,(hl)
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ex	de,hl
	ld	e,b
	ld	d,c
	ld	b,l
	ld	c,h
	push	bc
	push	de
	ld	de,(_capturePtr)
	ld	bc,(_capturePtr+2)
	call	__farst4p##
@1110:
	.debug	# 1144
	ld	hl,(ix-30)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	de,(_capturePtr)
	ld	bc,(_capturePtr+2)
	call	__ladd##
	ld	(_capturePtr),de
	ld	(_capturePtr+2),bc
@1108:
	.debug	>
	ld	hl,(ix-22)
	inc	hl
	ld	(ix-22),hl
	add	hl,hl
	ld	de,_scopeList
	add	hl,de
	ld	hl,(hl)
	ld	(ix-50),hl
	jp	@1104
@1105:
@1103:
	.debug	>
	.debug	# 1146
	ld	de,(_captureCount)
	ld	bc,(_captureCount+2)
	call	__linc##
	ld	(_captureCount),de
	ld	(_captureCount+2),bc
	.debug	# 1148
	ld	hl,(_captureIdx)
	inc	hl
	ld	(_captureIdx),hl
	ld	de,(_scopeBufSampleSize)
	or	a,a
	sbc	hl,de
	jr	nz,@1111
@1112:
	.debug	< @1112 @1113
	.debug	# 1149
	bool	hl
	ld	l,h
	ld	(_captureIdx),hl
	ld	de,(_scopeBuf)
	ld	bc,(_scopeBuf+2)
	ld	(_capturePtr),de
	ld	(_capturePtr+2),bc
@1113:
	.debug	>
@1111:
@1101:
	.debug	>
@1099:
	.debug	# 1152
	ld	a,0x1
	ld	(_applicReady),a
	.debug	I
;4653   }
@1080:
	.debug	>
;4654 
;4655 }
	jp	@1076
@1075:
	.debug	>
@998:
	.debug	} @998 4655

@997	equ	118
	ld	sp,ix
	pop	ix
	lret
@999	equ	120
;4656 
;4657 
;4658 //=========================================================================
;4659 
;4660 const CmdInfo far applCmds[] = {
;4661   { "autocal",      autocal,        "calibrate fwd, rev, or cav Atten value
	.seg	CONST
_applCmds::
	.dw	word (phys @1+4574)
	.dw	phys @1+4574 >> 16
	.dw	word _autocal
	.dw	word (phys @1+4582)
	.dw	phys @1+4582 >> 16
;4662   { "cmd",          cmd,            "control DSP state" },
	.dw	word (phys @1+4621)
	.dw	phys @1+4621 >> 16
	.dw	word _cmd
	.dw	word (phys @1+4625)
	.dw	phys @1+4625 >> 16
;4663   { "defaults",     defaults,       "set FPGA params to default values" },
	.dw	word (phys @1+4643)
	.dw	phys @1+4643 >> 16
	.dw	word _defaults
	.dw	word (phys @1+4652)
	.dw	phys @1+4652 >> 16
;4664   { "dspcalib",     dspcalib,       "calibrate gain/offset for DSP/EGU conv
	.dw	word (phys @1+4686)
	.dw	phys @1+4686 >> 16
	.dw	word _dspcalib
	.dw	word (phys @1+4695)
	.dw	phys @1+4695 >> 16
;4665   { "dspio",        dspio,          "enable/disable normal com with DSP" },
	.dw	word (phys @1+4740)
	.dw	phys @1+4740 >> 16
	.dw	word _dspio
	.dw	word (phys @1+4746)
	.dw	phys @1+4746 >> 16
;4666   { "dspread",      dspRead,        "read values from the DSP" },
	.dw	word (phys @1+4781)
	.dw	phys @1+4781 >> 16
	.dw	word _dspRead
	.dw	word (phys @1+4789)
	.dw	phys @1+4789 >> 16
;4667   { "dspwrite",     dspWrite,       "write a value to the DSP" },
	.dw	word (phys @1+4814)
	.dw	phys @1+4814 >> 16
	.dw	word _dspWrite
	.dw	word (phys @1+4823)
	.dw	phys @1+4823 >> 16
;4668   { "fpgaread",     fpgaRead,       "read values from the FPGA" },
	.dw	word (phys @1+4848)
	.dw	phys @1+4848 >> 16
	.dw	word _fpgaRead
	.dw	word (phys @1+4857)
	.dw	phys @1+4857 >> 16
;4669   { "fpgawrite",    fpgaWrite,      "write a value to the FPGA" },
	.dw	word (phys @1+4883)
	.dw	phys @1+4883 >> 16
	.dw	word _fpgaWrite
	.dw	word (phys @1+4893)
	.dw	phys @1+4893 >> 16
;4670   { "resetfpgadsp", reset_DSP_FPGA, "Reload the FPGA firmware from EEPROM" 
	.dw	word (phys @1+4919)
	.dw	phys @1+4919 >> 16
	.dw	word _reset_DSP_FPGA
	.dw	word (phys @1+4932)
	.dw	phys @1+4932 >> 16
;4671   { "setdspfirm",   setDSPfirm,     "Update and reload DSP firmware image" 
	.dw	word (phys @1+4969)
	.dw	phys @1+4969 >> 16
	.dw	word _setDSPfirm
	.dw	word (phys @1+4980)
	.dw	phys @1+4980 >> 16
;4672   { "setfpgafirm",  setFPGAfirm,    "Update and reload FPGA firmware image"
	.dw	word (phys @1+5017)
	.dw	phys @1+5017 >> 16
	.dw	word _setFPGAfirm
	.dw	word (phys @1+5029)
	.dw	phys @1+5029 >> 16
;4673   { "state",        showState,      "Show state of controller" }
	.dw	word (phys @1+4637)
	.dw	phys @1+4637 >> 16
	.dw	word _showState
	.dw	word (phys @1+5067)
	.dw	phys @1+5067 >> 16
;4674 };
	.debug	G _applCmds 133 8
;4675 
;4676 int const NumApplCmds = ( sizeof(applCmds) / sizeof(CmdInfo) );
_NumApplCmds::
	.dw	0xd
	.debug	G _NumApplCmds 6 0
;4677 
;4678 //-------------------------------------------------------------------------
;4679 #endif
;4680 
;4681 // progdsp /srfs/RFController2.0.bin
;4682 // setfpgafirm /srfs/llrf2.0.bin
	.extern	$time 135
	.extern	_memcpy 136
	.extern	_memset 137
	.extern	_strcmp 138
	.extern	_stricmp 138
	.extern	_strlen 139
	.extern	_strncmp 140
	.extern	$fmemcpy 141
	.extern	_fmemset 142
	.extern	_fstrcpy 143
	.extern	_strcpyfn 144
	.extern	$clockTicks 145
	.extern	$memCRC16 146
	.extern	$Engineering 147
	.extern	$stdHeader 97
	.extern	$showNetState 148
	.extern	$CheckETX 106
	.extern	$compileTime 149
	.extern	$processMsg 150
	.extern	$inet_ptoh 151
	.extern	_clsCmd 152
	.extern	$sigismember 153
	.extern	$sigprocmask 154
	.extern	$sigpending 155
	.extern	_tasks 156
	.extern	_current 72
	.extern	_idleTask 72
	.extern	_applTask 72
	.extern	_SchedMS 7
	.cseg
_schedule:
	pop	iy
	lcall	$$schedule
	.extern	$$schedule
	.extern	$schedule 97
	.extern	__stdio 157
	.extern	$farfprintf 159
	.extern	$farprintf 160
	.extern	$farsprintf 161
	.extern	__TimerSeconds 9
	.extern	__TimerMilliSeconds 162
	.extern	$xavail 145
	.extern	_strtod 163
	.extern	$strtol 164
	.extern	$strtoui 165
	.extern	_updateAIn 152
	.extern	_updateAOut 152
	.extern	_adcGain 166
	.extern	_adcOffset 166
	.extern	_dacGain 166
	.extern	_dacOffset 166
	.extern	_aoutMin 166
	.extern	_aoutMax 166
	.extern	_anaIn 166
	.extern	_anaOut 166
	.extern	_lastAnaOut 166
	.extern	_adcRaw 167
	.extern	_dacRaw 167
	.extern	_lastadcRaw 167
	.extern	_newRdg 152
	.extern	_adcForced 152
	.extern	_dacForced 152
	.extern	_adcForcedVal 166
	.extern	_dacForcedVal 166
	.extern	_NumHdwADC 7
	.extern	_NumHdwDAC 7
	.bseg
_readGain::
	.ds	4
	.debug	G _readGain 168 0
_readOffset::
	.ds	4
	.debug	G _readOffset 168 0
_setGain::
	.ds	4
	.debug	G _setGain 168 0
_setOffset::
	.ds	4
	.debug	G _setOffset 168 0
_readVal::
	.ds	4
	.debug	G _readVal 168 0
_setVal::
	.ds	4
	.debug	G _setVal 168 0
_lastSetVal::
	.ds	4
	.debug	G _lastSetVal 168 0
	.extern	_digitalIn 9
	.extern	_digitalOut 9
	.defseg	FARBSS
	.seg	FARBSS
_ainLabel::
	.ds	24
	.debug	G _ainLabel 169 8
_aoutLabel::
	.ds	24
	.debug	G _aoutLabel 169 8
_dinLabel::
	.ds	24
	.debug	G _dinLabel 169 8
_doutLabel::
	.ds	48
	.debug	G _doutLabel 170 8
_extraInLabel::
	.ds	768
	.debug	G _extraInLabel 171 8
	.extern	$initIOvalues 97
	.extern	$ReadDigInputs 145
	.extern	$WriteDigOutputs 172
	.extern	$ReadDeviceID 145
	.cseg
_newDACCalib:
	pop	iy
	lcall	$$newDACCalib
	.extern	$$newDACCalib
	.extern	$newDACCalib 173
_newADCCalib:
	pop	iy
	lcall	$$newADCCalib
	.extern	$$newADCCalib
	.extern	$newADCCalib 173
	.extern	$writeAnalogOutputs 172
	.extern	$readAnalogInputs 172
	.extern	$updateAvgsADC 97
	.extern	$open 174
	.extern	$close 175
	.extern	$read 176
	.extern	$fflush 177
	.extern	_KLOCK 97
	.extern	_KUNLOCK 97
	.extern	$resetWDT 97
	.extern	$_activeTaskIdx_ 175
	.extern	_hostName 89
	.bseg
_ctlLoopTicksPerSec::
	.ds	2
	.debug	G _ctlLoopTicksPerSec 7 0
	.extern	_prevLogInfo 178
	.extern	_runBackup 3
	.extern	_runningPrimary 3
	.extern	_doReboot 3
	.extern	_doShutdown 3
	.extern	_diagLog 158
	.extern	_intTicks 3
	.extern	_milliTicks 7
	.extern	_hdwResetTime 9
	.extern	_applicTicks 7
	.extern	_deviceID 9
	.extern	_devType 7
	.extern	_hdwType 7
	.extern	_percentIdle 7
	.extern	_timeZone 6
	.extern	$REBOOT 100
	.extern	$createProc 179
	.extern	$sleepMS 180
	.extern	$msToTicks 180
	.extern	$Gets 181
	.extern	$Exit 148
	.extern	$SetHostName 182
	.extern	$sem_wait_ 183
	.extern	$sem_init 184
	.extern	$sem_post 185
_C0ChanMult::
	.ds	2
	.debug	G _C0ChanMult 7 0
_NumC0Chans::
	.ds	2
	.debug	G _NumC0Chans 7 0
_R3ChanMult::
	.ds	2
	.debug	G _R3ChanMult 7 0
_NumR3Chans::
	.ds	2
	.debug	G _NumR3Chans 7 0
_R4ChanMult::
	.ds	2
	.debug	G _R4ChanMult 7 0
_NumR4Chans::
	.ds	2
	.debug	G _NumR4Chans 7 0
	.extern	$setRegU16 186
	.extern	$setRegU32 187
	.extern	$setRegF32 188
	.extern	_numScopeRegs 7
	.extern	_captureIdx 7
	.extern	_scopeBufSampleSize 7
	.extern	_captureCount 9
	.extern	_scopeList 189
	.extern	_scopeBuf 48
	.extern	_capturePtr 48
	.cseg
_setS16:
	pop	iy
	lcall	$$setS16
	.extern	$$setS16
	.extern	$setS16 88
_setU16:
	pop	iy
	lcall	$$setU16
	.extern	$$setU16
	.extern	$setU16 88
_setS32:
	pop	iy
	lcall	$$setS32
	.extern	$$setS32
	.extern	$setS32 88
_setU32:
	pop	iy
	lcall	$$setU32
	.extern	$$setU32
	.extern	$setU32 88
_setF32:
	pop	iy
	lcall	$$setF32
	.extern	$$setF32
	.extern	$setF32 88
	.extern	$fmtRegVal 190
	.extern	$pvInitLists 106
_modbusTCPd:
	pop	iy
	lcall	$$modbusTCPd
	.extern	$$modbusTCPd
	.extern	$modbusTCPd 111
	.extern	$pvGetInfo 191
	.bseg
_DoNothing::
	.ds	1
	.debug	G _DoNothing 3 0
	.cseg
_setCoil:
	pop	iy
	lcall	$$setCoil
	.extern	$$setCoil
	.extern	$setCoil 192
	.bseg
_dlmVersTime::
	.ds	4
	.debug	G _dlmVersTime 9 0
_dlmVersTime_ChgEventVal::
	.ds	4
	.debug	G _dlmVersTime_ChgEventVal 9 0
_dlmVersTime_MinChg::
	.ds	4
	.debug	G _dlmVersTime_MinChg 9 0
_rtosVersTime::
	.ds	4
	.debug	G _rtosVersTime 9 0
_rtosVersTime_ChgEventVal::
	.ds	4
	.debug	G _rtosVersTime_ChgEventVal 9 0
_rtosVersTime_MinChg::
	.ds	4
	.debug	G _rtosVersTime_MinChg 9 0
_applVersTime::
	.ds	4
	.debug	G _applVersTime 9 0
_applVersTime_ChgEventVal::
	.ds	4
	.debug	G _applVersTime_ChgEventVal 9 0
_applVersTime_MinChg::
	.ds	4
	.debug	G _applVersTime_MinChg 9 0
_devType_::
	.ds	2
	.debug	G _devType_ 5 0
_devType__ChgEventVal::
	.ds	2
	.debug	G _devType__ChgEventVal 5 0
_devType__MinChg::
	.ds	2
	.debug	G _devType__MinChg 5 0
_deviceID_::
	.ds	4
	.debug	G _deviceID_ 9 0
_deviceID__ChgEventVal::
	.ds	4
	.debug	G _deviceID__ChgEventVal 9 0
_deviceID__MinChg::
	.ds	4
	.debug	G _deviceID__MinChg 9 0
_upSecs::
	.ds	4
	.debug	G _upSecs 9 0
_upSecs_ChgEventVal::
	.ds	4
	.debug	G _upSecs_ChgEventVal 9 0
_upSecs_MinChg::
	.ds	4
	.debug	G _upSecs_MinChg 9 0
_percentIdle_::
	.ds	2
	.debug	G _percentIdle_ 5 0
_percentIdle__ChgEventVal::
	.ds	2
	.debug	G _percentIdle__ChgEventVal 5 0
_percentIdle__MinChg::
	.ds	2
	.debug	G _percentIdle__MinChg 5 0
_numChans::
	.ds	2
	.debug	G _numChans 5 0
_numChans_ChgEventVal::
	.ds	2
	.debug	G _numChans_ChgEventVal 5 0
_numChans_MinChg::
	.ds	2
	.debug	G _numChans_MinChg 5 0
_coilChanMult::
	.ds	2
	.debug	G _coilChanMult 5 0
_coilChanMult_ChgEventVal::
	.ds	2
	.debug	G _coilChanMult_ChgEventVal 5 0
_coilChanMult_MinChg::
	.ds	2
	.debug	G _coilChanMult_MinChg 5 0
_reg3ChanMult::
	.ds	2
	.debug	G _reg3ChanMult 5 0
_reg3ChanMult_ChgEventVal::
	.ds	2
	.debug	G _reg3ChanMult_ChgEventVal 5 0
_reg3ChanMult_MinChg::
	.ds	2
	.debug	G _reg3ChanMult_MinChg 5 0
_reg4ChanMult::
	.ds	2
	.debug	G _reg4ChanMult 5 0
_reg4ChanMult_ChgEventVal::
	.ds	2
	.debug	G _reg4ChanMult_ChgEventVal 5 0
_reg4ChanMult_MinChg::
	.ds	2
	.debug	G _reg4ChanMult_MinChg 5 0
_digInBits::
	.ds	4
	.debug	G _digInBits 9 0
_digInBits_ChgEventVal::
	.ds	4
	.debug	G _digInBits_ChgEventVal 9 0
_digInBits_MinChg::
	.ds	4
	.debug	G _digInBits_MinChg 9 0
_statusBitsStdA::
	.ds	2
	.debug	G _statusBitsStdA 5 0
_statusBitsStdA_ChgEventVal::
	.ds	2
	.debug	G _statusBitsStdA_ChgEventVal 5 0
_statusBitsStdA_MinChg::
	.ds	2
	.debug	G _statusBitsStdA_MinChg 5 0
_statusBitsStdB::
	.ds	2
	.debug	G _statusBitsStdB 5 0
_statusBitsStdB_ChgEventVal::
	.ds	2
	.debug	G _statusBitsStdB_ChgEventVal 5 0
_statusBitsStdB_MinChg::
	.ds	2
	.debug	G _statusBitsStdB_MinChg 5 0
_statusBitsAppl::
	.ds	4
	.debug	G _statusBitsAppl 9 0
_statusBitsAppl_ChgEventVal::
	.ds	4
	.debug	G _statusBitsAppl_ChgEventVal 9 0
_statusBitsAppl_MinChg::
	.ds	4
	.debug	G _statusBitsAppl_MinChg 9 0
_latency::
	.ds	2
	.debug	G _latency 5 0
_latency_ChgEventVal::
	.ds	2
	.debug	G _latency_ChgEventVal 5 0
_latency_MinChg::
	.ds	2
	.debug	G _latency_MinChg 5 0
_tcpTickTime::
	.ds	2
	.debug	G _tcpTickTime 5 0
_tcpTickTime_ChgEventVal::
	.ds	2
	.debug	G _tcpTickTime_ChgEventVal 5 0
_tcpTickTime_MinChg::
	.ds	2
	.debug	G _tcpTickTime_MinChg 5 0
_applLoopTime::
	.ds	2
	.debug	G _applLoopTime 5 0
_applLoopTime_ChgEventVal::
	.ds	2
	.debug	G _applLoopTime_ChgEventVal 5 0
_applLoopTime_MinChg::
	.ds	2
	.debug	G _applLoopTime_MinChg 5 0
_kernelLoopTime::
	.ds	2
	.debug	G _kernelLoopTime 5 0
_kernelLoopTime_ChgEventVal::
	.ds	2
	.debug	G _kernelLoopTime_ChgEventVal 5 0
_kernelLoopTime_MinChg::
	.ds	2
	.debug	G _kernelLoopTime_MinChg 5 0
_statusBitsFPGA::
	.ds	2
	.debug	G _statusBitsFPGA 5 0
_statusBitsFPGA_ChgEventVal::
	.ds	2
	.debug	G _statusBitsFPGA_ChgEventVal 5 0
_statusBitsFPGA_MinChg::
	.ds	2
	.debug	G _statusBitsFPGA_MinChg 5 0
_statusBitsTuner::
	.ds	2
	.debug	G _statusBitsTuner 5 0
_statusBitsTuner_ChgEventVal::
	.ds	2
	.debug	G _statusBitsTuner_ChgEventVal 5 0
_statusBitsTuner_MinChg::
	.ds	2
	.debug	G _statusBitsTuner_MinChg 5 0
_statusBitsRF::
	.ds	2
	.debug	G _statusBitsRF 5 0
_statusBitsRF_ChgEventVal::
	.ds	2
	.debug	G _statusBitsRF_ChgEventVal 5 0
_statusBitsRF_MinChg::
	.ds	2
	.debug	G _statusBitsRF_MinChg 5 0
_statusBitsILK::
	.ds	2
	.debug	G _statusBitsILK 5 0
_statusBitsILK_ChgEventVal::
	.ds	2
	.debug	G _statusBitsILK_ChgEventVal 5 0
_statusBitsILK_MinChg::
	.ds	2
	.debug	G _statusBitsILK_MinChg 5 0
_spark0_ctr::
	.ds	2
	.debug	G _spark0_ctr 5 0
_spark0_ctr_ChgEventVal::
	.ds	2
	.debug	G _spark0_ctr_ChgEventVal 5 0
_spark0_ctr_MinChg::
	.ds	2
	.debug	G _spark0_ctr_MinChg 5 0
_spark1_ctr::
	.ds	2
	.debug	G _spark1_ctr 5 0
_spark1_ctr_ChgEventVal::
	.ds	2
	.debug	G _spark1_ctr_ChgEventVal 5 0
_spark1_ctr_MinChg::
	.ds	2
	.debug	G _spark1_ctr_MinChg 5 0
_cavlow_ctr::
	.ds	2
	.debug	G _cavlow_ctr 5 0
_cavlow_ctr_ChgEventVal::
	.ds	2
	.debug	G _cavlow_ctr_ChgEventVal 5 0
_cavlow_ctr_MinChg::
	.ds	2
	.debug	G _cavlow_ctr_MinChg 5 0
_rev_high_ctr::
	.ds	2
	.debug	G _rev_high_ctr 5 0
_rev_high_ctr_ChgEventVal::
	.ds	2
	.debug	G _rev_high_ctr_ChgEventVal 5 0
_rev_high_ctr_MinChg::
	.ds	2
	.debug	G _rev_high_ctr_MinChg 5 0
_dvdt_ctr::
	.ds	2
	.debug	G _dvdt_ctr 5 0
_dvdt_ctr_ChgEventVal::
	.ds	2
	.debug	G _dvdt_ctr_ChgEventVal 5 0
_dvdt_ctr_MinChg::
	.ds	2
	.debug	G _dvdt_ctr_MinChg 5 0
_fwd_low_ctr::
	.ds	2
	.debug	G _fwd_low_ctr 5 0
_fwd_low_ctr_ChgEventVal::
	.ds	2
	.debug	G _fwd_low_ctr_ChgEventVal 5 0
_fwd_low_ctr_MinChg::
	.ds	2
	.debug	G _fwd_low_ctr_MinChg 5 0
_fwd_high_ctr::
	.ds	2
	.debug	G _fwd_high_ctr 5 0
_fwd_high_ctr_ChgEventVal::
	.ds	2
	.debug	G _fwd_high_ctr_ChgEventVal 5 0
_fwd_high_ctr_MinChg::
	.ds	2
	.debug	G _fwd_high_ctr_MinChg 5 0
_PLC_ctr::
	.ds	2
	.debug	G _PLC_ctr 5 0
_PLC_ctr_ChgEventVal::
	.ds	2
	.debug	G _PLC_ctr_ChgEventVal 5 0
_PLC_ctr_MinChg::
	.ds	2
	.debug	G _PLC_ctr_MinChg 5 0
_safety_ctr::
	.ds	2
	.debug	G _safety_ctr 5 0
_safety_ctr_ChgEventVal::
	.ds	2
	.debug	G _safety_ctr_ChgEventVal 5 0
_safety_ctr_MinChg::
	.ds	2
	.debug	G _safety_ctr_MinChg 5 0
_fwdPhase::
	.ds	4
	.debug	G _fwdPhase 10 0
_fwdPhase_ChgEventVal::
	.ds	4
	.debug	G _fwdPhase_ChgEventVal 10 0
_fwdPhase_MinChg::
	.ds	4
	.debug	G _fwdPhase_MinChg 10 0
_revPhase::
	.ds	4
	.debug	G _revPhase 10 0
_revPhase_ChgEventVal::
	.ds	4
	.debug	G _revPhase_ChgEventVal 10 0
_revPhase_MinChg::
	.ds	4
	.debug	G _revPhase_MinChg 10 0
_cavPhase::
	.ds	4
	.debug	G _cavPhase 10 0
_cavPhase_ChgEventVal::
	.ds	4
	.debug	G _cavPhase_ChgEventVal 10 0
_cavPhase_MinChg::
	.ds	4
	.debug	G _cavPhase_MinChg 10 0
_ref0Phase::
	.ds	4
	.debug	G _ref0Phase 10 0
_ref0Phase_ChgEventVal::
	.ds	4
	.debug	G _ref0Phase_ChgEventVal 10 0
_ref0Phase_MinChg::
	.ds	4
	.debug	G _ref0Phase_MinChg 10 0
_spare0Phase::
	.ds	4
	.debug	G _spare0Phase 10 0
_spare0Phase_ChgEventVal::
	.ds	4
	.debug	G _spare0Phase_ChgEventVal 10 0
_spare0Phase_MinChg::
	.ds	4
	.debug	G _spare0Phase_MinChg 10 0
_spare1Phase::
	.ds	4
	.debug	G _spare1Phase 10 0
_spare1Phase_ChgEventVal::
	.ds	4
	.debug	G _spare1Phase_ChgEventVal 10 0
_spare1Phase_MinChg::
	.ds	4
	.debug	G _spare1Phase_MinChg 10 0
_spare2Phase::
	.ds	4
	.debug	G _spare2Phase 10 0
_spare2Phase_ChgEventVal::
	.ds	4
	.debug	G _spare2Phase_ChgEventVal 10 0
_spare2Phase_MinChg::
	.ds	4
	.debug	G _spare2Phase_MinChg 10 0
_ref1Phase::
	.ds	4
	.debug	G _ref1Phase 10 0
_ref1Phase_ChgEventVal::
	.ds	4
	.debug	G _ref1Phase_ChgEventVal 10 0
_ref1Phase_MinChg::
	.ds	4
	.debug	G _ref1Phase_MinChg 10 0
_fwdAmpl::
	.ds	4
	.debug	G _fwdAmpl 10 0
_fwdAmpl_ChgEventVal::
	.ds	4
	.debug	G _fwdAmpl_ChgEventVal 10 0
_fwdAmpl_MinChg::
	.ds	4
	.debug	G _fwdAmpl_MinChg 10 0
_revAmpl::
	.ds	4
	.debug	G _revAmpl 10 0
_revAmpl_ChgEventVal::
	.ds	4
	.debug	G _revAmpl_ChgEventVal 10 0
_revAmpl_MinChg::
	.ds	4
	.debug	G _revAmpl_MinChg 10 0
_cavAmpl::
	.ds	4
	.debug	G _cavAmpl 10 0
_cavAmpl_ChgEventVal::
	.ds	4
	.debug	G _cavAmpl_ChgEventVal 10 0
_cavAmpl_MinChg::
	.ds	4
	.debug	G _cavAmpl_MinChg 10 0
_ref0Ampl::
	.ds	4
	.debug	G _ref0Ampl 10 0
_ref0Ampl_ChgEventVal::
	.ds	4
	.debug	G _ref0Ampl_ChgEventVal 10 0
_ref0Ampl_MinChg::
	.ds	4
	.debug	G _ref0Ampl_MinChg 10 0
_spare0Ampl::
	.ds	4
	.debug	G _spare0Ampl 10 0
_spare0Ampl_ChgEventVal::
	.ds	4
	.debug	G _spare0Ampl_ChgEventVal 10 0
_spare0Ampl_MinChg::
	.ds	4
	.debug	G _spare0Ampl_MinChg 10 0
_spare1Ampl::
	.ds	4
	.debug	G _spare1Ampl 10 0
_spare1Ampl_ChgEventVal::
	.ds	4
	.debug	G _spare1Ampl_ChgEventVal 10 0
_spare1Ampl_MinChg::
	.ds	4
	.debug	G _spare1Ampl_MinChg 10 0
_spare2Ampl::
	.ds	4
	.debug	G _spare2Ampl 10 0
_spare2Ampl_ChgEventVal::
	.ds	4
	.debug	G _spare2Ampl_ChgEventVal 10 0
_spare2Ampl_MinChg::
	.ds	4
	.debug	G _spare2Ampl_MinChg 10 0
_ref1Ampl::
	.ds	4
	.debug	G _ref1Ampl 10 0
_ref1Ampl_ChgEventVal::
	.ds	4
	.debug	G _ref1Ampl_ChgEventVal 10 0
_ref1Ampl_MinChg::
	.ds	4
	.debug	G _ref1Ampl_MinChg 10 0
_maxFwdAmpl::
	.ds	4
	.debug	G _maxFwdAmpl 10 0
_maxFwdAmpl_ChgEventVal::
	.ds	4
	.debug	G _maxFwdAmpl_ChgEventVal 10 0
_maxFwdAmpl_MinChg::
	.ds	4
	.debug	G _maxFwdAmpl_MinChg 10 0
_maxRevAmpl::
	.ds	4
	.debug	G _maxRevAmpl 10 0
_maxRevAmpl_ChgEventVal::
	.ds	4
	.debug	G _maxRevAmpl_ChgEventVal 10 0
_maxRevAmpl_MinChg::
	.ds	4
	.debug	G _maxRevAmpl_MinChg 10 0
_maxCavAmpl::
	.ds	4
	.debug	G _maxCavAmpl 10 0
_maxCavAmpl_ChgEventVal::
	.ds	4
	.debug	G _maxCavAmpl_ChgEventVal 10 0
_maxCavAmpl_MinChg::
	.ds	4
	.debug	G _maxCavAmpl_MinChg 10 0
_maxSpare0Ampl::
	.ds	4
	.debug	G _maxSpare0Ampl 10 0
_maxSpare0Ampl_ChgEventVal::
	.ds	4
	.debug	G _maxSpare0Ampl_ChgEventVal 10 0
_maxSpare0Ampl_MinChg::
	.ds	4
	.debug	G _maxSpare0Ampl_MinChg 10 0
_maxSpare1Ampl::
	.ds	4
	.debug	G _maxSpare1Ampl 10 0
_maxSpare1Ampl_ChgEventVal::
	.ds	4
	.debug	G _maxSpare1Ampl_ChgEventVal 10 0
_maxSpare1Ampl_MinChg::
	.ds	4
	.debug	G _maxSpare1Ampl_MinChg 10 0
_maxSpare2Ampl::
	.ds	4
	.debug	G _maxSpare2Ampl 10 0
_maxSpare2Ampl_ChgEventVal::
	.ds	4
	.debug	G _maxSpare2Ampl_ChgEventVal 10 0
_maxSpare2Ampl_MinChg::
	.ds	4
	.debug	G _maxSpare2Ampl_MinChg 10 0
_maxOutAmpl::
	.ds	4
	.debug	G _maxOutAmpl 10 0
_maxOutAmpl_ChgEventVal::
	.ds	4
	.debug	G _maxOutAmpl_ChgEventVal 10 0
_maxOutAmpl_MinChg::
	.ds	4
	.debug	G _maxOutAmpl_MinChg 10 0
_statusBitsDSP::
	.ds	4
	.debug	G _statusBitsDSP 9 0
_statusBitsDSP_ChgEventVal::
	.ds	4
	.debug	G _statusBitsDSP_ChgEventVal 9 0
_statusBitsDSP_MinChg::
	.ds	4
	.debug	G _statusBitsDSP_MinChg 9 0
_dspLoad::
	.ds	4
	.debug	G _dspLoad 10 0
_dspLoad_ChgEventVal::
	.ds	4
	.debug	G _dspLoad_ChgEventVal 10 0
_dspLoad_MinChg::
	.ds	4
	.debug	G _dspLoad_MinChg 10 0
_pcbTemp::
	.ds	4
	.debug	G _pcbTemp 10 0
_pcbTemp_ChgEventVal::
	.ds	4
	.debug	G _pcbTemp_ChgEventVal 10 0
_pcbTemp_MinChg::
	.ds	4
	.debug	G _pcbTemp_MinChg 10 0
_fan0Spd::
	.ds	2
	.debug	G _fan0Spd 5 0
_fan0Spd_ChgEventVal::
	.ds	2
	.debug	G _fan0Spd_ChgEventVal 5 0
_fan0Spd_MinChg::
	.ds	2
	.debug	G _fan0Spd_MinChg 5 0
_fan1Spd::
	.ds	2
	.debug	G _fan1Spd 5 0
_fan1Spd_ChgEventVal::
	.ds	2
	.debug	G _fan1Spd_ChgEventVal 5 0
_fan1Spd_MinChg::
	.ds	2
	.debug	G _fan1Spd_MinChg 5 0
_rfq_t1_output::
	.ds	4
	.debug	G _rfq_t1_output 10 0
_rfq_t1_output_ChgEventVal::
	.ds	4
	.debug	G _rfq_t1_output_ChgEventVal 10 0
_rfq_t1_output_MinChg::
	.ds	4
	.debug	G _rfq_t1_output_MinChg 10 0
_rfq_t2_output::
	.ds	4
	.debug	G _rfq_t2_output 10 0
_rfq_t2_output_ChgEventVal::
	.ds	4
	.debug	G _rfq_t2_output_ChgEventVal 10 0
_rfq_t2_output_MinChg::
	.ds	4
	.debug	G _rfq_t2_output_MinChg 10 0
_piezoReadPos::
	.ds	2
	.debug	G _piezoReadPos 4 0
_piezoReadPos_ChgEventVal::
	.ds	2
	.debug	G _piezoReadPos_ChgEventVal 4 0
_piezoReadPos_MinChg::
	.ds	2
	.debug	G _piezoReadPos_MinChg 4 0
_stepperReadPos::
	.ds	2
	.debug	G _stepperReadPos 4 0
_stepperReadPos_ChgEventVal::
	.ds	2
	.debug	G _stepperReadPos_ChgEventVal 4 0
_stepperReadPos_MinChg::
	.ds	2
	.debug	G _stepperReadPos_MinChg 4 0
_percentFFT::
	.ds	2
	.debug	G _percentFFT 5 0
_percentFFT_ChgEventVal::
	.ds	2
	.debug	G _percentFFT_ChgEventVal 5 0
_percentFFT_MinChg::
	.ds	2
	.debug	G _percentFFT_MinChg 5 0
_sizeFFT::
	.ds	2
	.debug	G _sizeFFT 5 0
_sizeFFT_ChgEventVal::
	.ds	2
	.debug	G _sizeFFT_ChgEventVal 5 0
_sizeFFT_MinChg::
	.ds	2
	.debug	G _sizeFFT_MinChg 5 0
_romProgErrs::
	.ds	2
	.debug	G _romProgErrs 5 0
_romProgErrs_ChgEventVal::
	.ds	2
	.debug	G _romProgErrs_ChgEventVal 5 0
_romProgErrs_MinChg::
	.ds	2
	.debug	G _romProgErrs_MinChg 5 0
_timeZone_::
	.ds	2
	.debug	G _timeZone_ 4 0
_timeZone__ChgEventVal::
	.ds	2
	.debug	G _timeZone__ChgEventVal 4 0
_timeZone__MinChg::
	.ds	2
	.debug	G _timeZone__MinChg 4 0
	.cseg
_MB_timeZone:
	pop	iy
	lcall	$$MB_timeZone
	.bseg
_curTime::
	.ds	4
	.debug	G _curTime 9 0
_curTime_ChgEventVal::
	.ds	4
	.debug	G _curTime_ChgEventVal 9 0
_curTime_MinChg::
	.ds	4
	.debug	G _curTime_MinChg 9 0
_ipAddr::
	.ds	4
	.debug	G _ipAddr 9 0
_ipAddr_ChgEventVal::
	.ds	4
	.debug	G _ipAddr_ChgEventVal 9 0
_ipAddr_MinChg::
	.ds	4
	.debug	G _ipAddr_MinChg 9 0
_netmask::
	.ds	4
	.debug	G _netmask 9 0
_netmask_ChgEventVal::
	.ds	4
	.debug	G _netmask_ChgEventVal 9 0
_netmask_MinChg::
	.ds	4
	.debug	G _netmask_MinChg 9 0
_gateway::
	.ds	4
	.debug	G _gateway 9 0
_gateway_ChgEventVal::
	.ds	4
	.debug	G _gateway_ChgEventVal 9 0
_gateway_MinChg::
	.ds	4
	.debug	G _gateway_MinChg 9 0
_setACoil::
	.ds	2
	.debug	G _setACoil 5 0
_setACoil_ChgEventVal::
	.ds	2
	.debug	G _setACoil_ChgEventVal 5 0
_setACoil_MinChg::
	.ds	2
	.debug	G _setACoil_MinChg 5 0
_resetACoil::
	.ds	2
	.debug	G _resetACoil 5 0
_resetACoil_ChgEventVal::
	.ds	2
	.debug	G _resetACoil_ChgEventVal 5 0
_resetACoil_MinChg::
	.ds	2
	.debug	G _resetACoil_MinChg 5 0
_digOutBits::
	.ds	4
	.debug	G _digOutBits 9 0
_digOutBits_ChgEventVal::
	.ds	4
	.debug	G _digOutBits_ChgEventVal 9 0
_digOutBits_MinChg::
	.ds	4
	.debug	G _digOutBits_MinChg 9 0
_cmdBitsStd::
	.ds	4
	.debug	G _cmdBitsStd 9 0
_cmdBitsStd_ChgEventVal::
	.ds	4
	.debug	G _cmdBitsStd_ChgEventVal 9 0
_cmdBitsStd_MinChg::
	.ds	4
	.debug	G _cmdBitsStd_MinChg 9 0
	.cseg
_MB_cmdBitsStd:
	pop	iy
	lcall	$$MB_cmdBitsStd
	.bseg
_cmdBitsAppl::
	.ds	4
	.debug	G _cmdBitsAppl 9 0
_cmdBitsAppl_ChgEventVal::
	.ds	4
	.debug	G _cmdBitsAppl_ChgEventVal 9 0
_cmdBitsAppl_MinChg::
	.ds	4
	.debug	G _cmdBitsAppl_MinChg 9 0
	.cseg
_MB_cmdBitsAppl:
	pop	iy
	lcall	$$MB_cmdBitsAppl
	.bseg
_ADC_err_rst::
	.ds	2
	.debug	G _ADC_err_rst 5 0
_ADC_err_rst_ChgEventVal::
	.ds	2
	.debug	G _ADC_err_rst_ChgEventVal 5 0
_ADC_err_rst_MinChg::
	.ds	2
	.debug	G _ADC_err_rst_MinChg 5 0
_RF_soft_on::
	.ds	2
	.debug	G _RF_soft_on 5 0
_RF_soft_on_ChgEventVal::
	.ds	2
	.debug	G _RF_soft_on_ChgEventVal 5 0
_RF_soft_on_MinChg::
	.ds	2
	.debug	G _RF_soft_on_MinChg 5 0
	.cseg
_MB_RF_soft_on:
	pop	iy
	lcall	$$MB_RF_soft_on
	.bseg
_RF_ilk_rst::
	.ds	2
	.debug	G _RF_ilk_rst 5 0
_RF_ilk_rst_ChgEventVal::
	.ds	2
	.debug	G _RF_ilk_rst_ChgEventVal 5 0
_RF_ilk_rst_MinChg::
	.ds	2
	.debug	G _RF_ilk_rst_MinChg 5 0
_ilk_ctr_rst1::
	.ds	2
	.debug	G _ilk_ctr_rst1 5 0
_ilk_ctr_rst1_ChgEventVal::
	.ds	2
	.debug	G _ilk_ctr_rst1_ChgEventVal 5 0
_ilk_ctr_rst1_MinChg::
	.ds	2
	.debug	G _ilk_ctr_rst1_MinChg 5 0
_ilk_ctr_rst2::
	.ds	2
	.debug	G _ilk_ctr_rst2 5 0
_ilk_ctr_rst2_ChgEventVal::
	.ds	2
	.debug	G _ilk_ctr_rst2_ChgEventVal 5 0
_ilk_ctr_rst2_MinChg::
	.ds	2
	.debug	G _ilk_ctr_rst2_MinChg 5 0
_trigPos::
	.ds	4
	.debug	G _trigPos 9 0
_trigPos_ChgEventVal::
	.ds	4
	.debug	G _trigPos_ChgEventVal 9 0
_trigPos_MinChg::
	.ds	4
	.debug	G _trigPos_MinChg 9 0
	.cseg
_MB_DSP_REG:
	pop	iy
	lcall	$$MB_DSP_REG
	.bseg
_warmLevel::
	.ds	4
	.debug	G _warmLevel 10 0
_warmLevel_ChgEventVal::
	.ds	4
	.debug	G _warmLevel_ChgEventVal 10 0
_warmLevel_MinChg::
	.ds	4
	.debug	G _warmLevel_MinChg 10 0
	.cseg
_MB_maxAmpl:
	pop	iy
	lcall	$$MB_maxAmpl
	.bseg
_warmTime::
	.ds	4
	.debug	G _warmTime 10 0
_warmTime_ChgEventVal::
	.ds	4
	.debug	G _warmTime_ChgEventVal 10 0
_warmTime_MinChg::
	.ds	4
	.debug	G _warmTime_MinChg 10 0
_rampTime::
	.ds	4
	.debug	G _rampTime 10 0
_rampTime_ChgEventVal::
	.ds	4
	.debug	G _rampTime_ChgEventVal 10 0
_rampTime_MinChg::
	.ds	4
	.debug	G _rampTime_MinChg 10 0
_fwdAtten::
	.ds	2
	.debug	G _fwdAtten 5 0
_fwdAtten_ChgEventVal::
	.ds	2
	.debug	G _fwdAtten_ChgEventVal 5 0
_fwdAtten_MinChg::
	.ds	2
	.debug	G _fwdAtten_MinChg 5 0
_revAtten::
	.ds	2
	.debug	G _revAtten 5 0
_revAtten_ChgEventVal::
	.ds	2
	.debug	G _revAtten_ChgEventVal 5 0
_revAtten_MinChg::
	.ds	2
	.debug	G _revAtten_MinChg 5 0
_cavAtten::
	.ds	2
	.debug	G _cavAtten 5 0
_cavAtten_ChgEventVal::
	.ds	2
	.debug	G _cavAtten_ChgEventVal 5 0
_cavAtten_MinChg::
	.ds	2
	.debug	G _cavAtten_MinChg 5 0
_spare0Atten::
	.ds	2
	.debug	G _spare0Atten 5 0
_spare0Atten_ChgEventVal::
	.ds	2
	.debug	G _spare0Atten_ChgEventVal 5 0
_spare0Atten_MinChg::
	.ds	2
	.debug	G _spare0Atten_MinChg 5 0
_spare1Atten::
	.ds	2
	.debug	G _spare1Atten 5 0
_spare1Atten_ChgEventVal::
	.ds	2
	.debug	G _spare1Atten_ChgEventVal 5 0
_spare1Atten_MinChg::
	.ds	2
	.debug	G _spare1Atten_MinChg 5 0
_spare2Atten::
	.ds	2
	.debug	G _spare2Atten 5 0
_spare2Atten_ChgEventVal::
	.ds	2
	.debug	G _spare2Atten_ChgEventVal 5 0
_spare2Atten_MinChg::
	.ds	2
	.debug	G _spare2Atten_MinChg 5 0
_outAtten::
	.ds	2
	.debug	G _outAtten 5 0
_outAtten_ChgEventVal::
	.ds	2
	.debug	G _outAtten_ChgEventVal 5 0
_outAtten_MinChg::
	.ds	2
	.debug	G _outAtten_MinChg 5 0
_phaseSet::
	.ds	4
	.debug	G _phaseSet 10 0
_phaseSet_ChgEventVal::
	.ds	4
	.debug	G _phaseSet_ChgEventVal 10 0
_phaseSet_MinChg::
	.ds	4
	.debug	G _phaseSet_MinChg 10 0
_amplSet::
	.ds	4
	.debug	G _amplSet 10 0
_amplSet_ChgEventVal::
	.ds	4
	.debug	G _amplSet_ChgEventVal 10 0
_amplSet_MinChg::
	.ds	4
	.debug	G _amplSet_MinChg 10 0
_pulseFreq::
	.ds	4
	.debug	G _pulseFreq 10 0
_pulseFreq_ChgEventVal::
	.ds	4
	.debug	G _pulseFreq_ChgEventVal 10 0
_pulseFreq_MinChg::
	.ds	4
	.debug	G _pulseFreq_MinChg 10 0
_pulseDuty::
	.ds	4
	.debug	G _pulseDuty 10 0
_pulseDuty_ChgEventVal::
	.ds	4
	.debug	G _pulseDuty_ChgEventVal 10 0
_pulseDuty_MinChg::
	.ds	4
	.debug	G _pulseDuty_MinChg 10 0
_rf_P::
	.ds	4
	.debug	G _rf_P 10 0
_rf_P_ChgEventVal::
	.ds	4
	.debug	G _rf_P_ChgEventVal 10 0
_rf_P_MinChg::
	.ds	4
	.debug	G _rf_P_MinChg 10 0
_rf_I::
	.ds	4
	.debug	G _rf_I 10 0
_rf_I_ChgEventVal::
	.ds	4
	.debug	G _rf_I_ChgEventVal 10 0
_rf_I_MinChg::
	.ds	4
	.debug	G _rf_I_MinChg 10 0
_rf_D::
	.ds	4
	.debug	G _rf_D 10 0
_rf_D_ChgEventVal::
	.ds	4
	.debug	G _rf_D_ChgEventVal 10 0
_rf_D_MinChg::
	.ds	4
	.debug	G _rf_D_MinChg 10 0
_tuner_P::
	.ds	4
	.debug	G _tuner_P 10 0
_tuner_P_ChgEventVal::
	.ds	4
	.debug	G _tuner_P_ChgEventVal 10 0
_tuner_P_MinChg::
	.ds	4
	.debug	G _tuner_P_MinChg 10 0
_tuner_I::
	.ds	4
	.debug	G _tuner_I 10 0
_tuner_I_ChgEventVal::
	.ds	4
	.debug	G _tuner_I_ChgEventVal 10 0
_tuner_I_MinChg::
	.ds	4
	.debug	G _tuner_I_MinChg 10 0
_tuner_D::
	.ds	4
	.debug	G _tuner_D 10 0
_tuner_D_ChgEventVal::
	.ds	4
	.debug	G _tuner_D_ChgEventVal 10 0
_tuner_D_MinChg::
	.ds	4
	.debug	G _tuner_D_MinChg 10 0
_rf_B0::
	.ds	4
	.debug	G _rf_B0 10 0
_rf_B0_ChgEventVal::
	.ds	4
	.debug	G _rf_B0_ChgEventVal 10 0
_rf_B0_MinChg::
	.ds	4
	.debug	G _rf_B0_MinChg 10 0
_rf_WCtrl::
	.ds	4
	.debug	G _rf_WCtrl 10 0
_rf_WCtrl_ChgEventVal::
	.ds	4
	.debug	G _rf_WCtrl_ChgEventVal 10 0
_rf_WCtrl_MinChg::
	.ds	4
	.debug	G _rf_WCtrl_MinChg 10 0
_rf_WObs::
	.ds	4
	.debug	G _rf_WObs 10 0
_rf_WObs_ChgEventVal::
	.ds	4
	.debug	G _rf_WObs_ChgEventVal 10 0
_rf_WObs_MinChg::
	.ds	4
	.debug	G _rf_WObs_MinChg 10 0
_tuner_B0::
	.ds	4
	.debug	G _tuner_B0 10 0
_tuner_B0_ChgEventVal::
	.ds	4
	.debug	G _tuner_B0_ChgEventVal 10 0
_tuner_B0_MinChg::
	.ds	4
	.debug	G _tuner_B0_MinChg 10 0
_tuner_WCtrl::
	.ds	4
	.debug	G _tuner_WCtrl 10 0
_tuner_WCtrl_ChgEventVal::
	.ds	4
	.debug	G _tuner_WCtrl_ChgEventVal 10 0
_tuner_WCtrl_MinChg::
	.ds	4
	.debug	G _tuner_WCtrl_MinChg 10 0
_tuner_WObs::
	.ds	4
	.debug	G _tuner_WObs 10 0
_tuner_WObs_ChgEventVal::
	.ds	4
	.debug	G _tuner_WObs_ChgEventVal 10 0
_tuner_WObs_MinChg::
	.ds	4
	.debug	G _tuner_WObs_MinChg 10 0
_pulseAmplSet::
	.ds	4
	.debug	G _pulseAmplSet 10 0
_pulseAmplSet_ChgEventVal::
	.ds	4
	.debug	G _pulseAmplSet_ChgEventVal 10 0
_pulseAmplSet_MinChg::
	.ds	4
	.debug	G _pulseAmplSet_MinChg 10 0
_pulsePhaseSet::
	.ds	4
	.debug	G _pulsePhaseSet 10 0
_pulsePhaseSet_ChgEventVal::
	.ds	4
	.debug	G _pulsePhaseSet_ChgEventVal 10 0
_pulsePhaseSet_MinChg::
	.ds	4
	.debug	G _pulsePhaseSet_MinChg 10 0
_cavLowIlkLevel::
	.ds	4
	.debug	G _cavLowIlkLevel 10 0
_cavLowIlkLevel_ChgEventVal::
	.ds	4
	.debug	G _cavLowIlkLevel_ChgEventVal 10 0
_cavLowIlkLevel_MinChg::
	.ds	4
	.debug	G _cavLowIlkLevel_MinChg 10 0
_cavLowIlkTime::
	.ds	4
	.debug	G _cavLowIlkTime 10 0
_cavLowIlkTime_ChgEventVal::
	.ds	4
	.debug	G _cavLowIlkTime_ChgEventVal 10 0
_cavLowIlkTime_MinChg::
	.ds	4
	.debug	G _cavLowIlkTime_MinChg 10 0
_dvdtIlkRate::
	.ds	4
	.debug	G _dvdtIlkRate 10 0
_dvdtIlkRate_ChgEventVal::
	.ds	4
	.debug	G _dvdtIlkRate_ChgEventVal 10 0
_dvdtIlkRate_MinChg::
	.ds	4
	.debug	G _dvdtIlkRate_MinChg 10 0
_revHighIlkRatio::
	.ds	4
	.debug	G _revHighIlkRatio 10 0
_revHighIlkRatio_ChgEventVal::
	.ds	4
	.debug	G _revHighIlkRatio_ChgEventVal 10 0
_revHighIlkRatio_MinChg::
	.ds	4
	.debug	G _revHighIlkRatio_MinChg 10 0
_revHighIlkTime::
	.ds	4
	.debug	G _revHighIlkTime 10 0
_revHighIlkTime_ChgEventVal::
	.ds	4
	.debug	G _revHighIlkTime_ChgEventVal 10 0
_revHighIlkTime_MinChg::
	.ds	4
	.debug	G _revHighIlkTime_MinChg 10 0
_fwdLowIlkRatio::
	.ds	4
	.debug	G _fwdLowIlkRatio 10 0
_fwdLowIlkRatio_ChgEventVal::
	.ds	4
	.debug	G _fwdLowIlkRatio_ChgEventVal 10 0
_fwdLowIlkRatio_MinChg::
	.ds	4
	.debug	G _fwdLowIlkRatio_MinChg 10 0
_fwdLowIlkTime::
	.ds	4
	.debug	G _fwdLowIlkTime 10 0
_fwdLowIlkTime_ChgEventVal::
	.ds	4
	.debug	G _fwdLowIlkTime_ChgEventVal 10 0
_fwdLowIlkTime_MinChg::
	.ds	4
	.debug	G _fwdLowIlkTime_MinChg 10 0
_fwdHighIlkLevel::
	.ds	4
	.debug	G _fwdHighIlkLevel 10 0
_fwdHighIlkLevel_ChgEventVal::
	.ds	4
	.debug	G _fwdHighIlkLevel_ChgEventVal 10 0
_fwdHighIlkLevel_MinChg::
	.ds	4
	.debug	G _fwdHighIlkLevel_MinChg 10 0
_fwdHighIlkTime::
	.ds	4
	.debug	G _fwdHighIlkTime 10 0
_fwdHighIlkTime_ChgEventVal::
	.ds	4
	.debug	G _fwdHighIlkTime_ChgEventVal 10 0
_fwdHighIlkTime_MinChg::
	.ds	4
	.debug	G _fwdHighIlkTime_MinChg 10 0
_fwdCavPhase::
	.ds	4
	.debug	G _fwdCavPhase 10 0
_fwdCavPhase_ChgEventVal::
	.ds	4
	.debug	G _fwdCavPhase_ChgEventVal 10 0
_fwdCavPhase_MinChg::
	.ds	4
	.debug	G _fwdCavPhase_MinChg 10 0
_ffPhase::
	.ds	4
	.debug	G _ffPhase 10 0
_ffPhase_ChgEventVal::
	.ds	4
	.debug	G _ffPhase_ChgEventVal 10 0
_ffPhase_MinChg::
	.ds	4
	.debug	G _ffPhase_MinChg 10 0
_ffGain::
	.ds	4
	.debug	G _ffGain 10 0
_ffGain_ChgEventVal::
	.ds	4
	.debug	G _ffGain_ChgEventVal 10 0
_ffGain_MinChg::
	.ds	4
	.debug	G _ffGain_MinChg 10 0
_freqOffset::
	.ds	4
	.debug	G _freqOffset 10 0
_freqOffset_ChgEventVal::
	.ds	4
	.debug	G _freqOffset_ChgEventVal 10 0
_freqOffset_MinChg::
	.ds	4
	.debug	G _freqOffset_MinChg 10 0
_rfq_manTuneSet::
	.ds	4
	.debug	G _rfq_manTuneSet 10 0
_rfq_manTuneSet_ChgEventVal::
	.ds	4
	.debug	G _rfq_manTuneSet_ChgEventVal 10 0
_rfq_manTuneSet_MinChg::
	.ds	4
	.debug	G _rfq_manTuneSet_MinChg 10 0
_rfq_t1_offset::
	.ds	4
	.debug	G _rfq_t1_offset 10 0
_rfq_t1_offset_ChgEventVal::
	.ds	4
	.debug	G _rfq_t1_offset_ChgEventVal 10 0
_rfq_t1_offset_MinChg::
	.ds	4
	.debug	G _rfq_t1_offset_MinChg 10 0
_rfq_t2_offset::
	.ds	4
	.debug	G _rfq_t2_offset 10 0
_rfq_t2_offset_ChgEventVal::
	.ds	4
	.debug	G _rfq_t2_offset_ChgEventVal 10 0
_rfq_t2_offset_MinChg::
	.ds	4
	.debug	G _rfq_t2_offset_MinChg 10 0
_rfq_t1_feedback::
	.ds	4
	.debug	G _rfq_t1_feedback 10 0
_rfq_t1_feedback_ChgEventVal::
	.ds	4
	.debug	G _rfq_t1_feedback_ChgEventVal 10 0
_rfq_t1_feedback_MinChg::
	.ds	4
	.debug	G _rfq_t1_feedback_MinChg 10 0
_rfq_t2_feedback::
	.ds	4
	.debug	G _rfq_t2_feedback 10 0
_rfq_t2_feedback_ChgEventVal::
	.ds	4
	.debug	G _rfq_t2_feedback_ChgEventVal 10 0
_rfq_t2_feedback_MinChg::
	.ds	4
	.debug	G _rfq_t2_feedback_MinChg 10 0
_setDSPState::
	.ds	4
	.debug	G _setDSPState 9 0
_setDSPState_ChgEventVal::
	.ds	4
	.debug	G _setDSPState_ChgEventVal 9 0
_setDSPState_MinChg::
	.ds	4
	.debug	G _setDSPState_MinChg 9 0
	.cseg
_MB_setDSPState:
	pop	iy
	lcall	$$MB_setDSPState
	.bseg
_piezoSetPos::
	.ds	2
	.debug	G _piezoSetPos 4 0
_piezoSetPos_ChgEventVal::
	.ds	2
	.debug	G _piezoSetPos_ChgEventVal 4 0
_piezoSetPos_MinChg::
	.ds	2
	.debug	G _piezoSetPos_MinChg 4 0
_stepperSetPos::
	.ds	2
	.debug	G _stepperSetPos 4 0
_stepperSetPos_ChgEventVal::
	.ds	2
	.debug	G _stepperSetPos_ChgEventVal 4 0
_stepperSetPos_MinChg::
	.ds	2
	.debug	G _stepperSetPos_MinChg 4 0
_fwdMagCalib::
	.ds	4
	.debug	G _fwdMagCalib 10 0
_fwdMagCalib_ChgEventVal::
	.ds	4
	.debug	G _fwdMagCalib_ChgEventVal 10 0
_fwdMagCalib_MinChg::
	.ds	4
	.debug	G _fwdMagCalib_MinChg 10 0
_fwdPhaseCalib::
	.ds	4
	.debug	G _fwdPhaseCalib 10 0
_fwdPhaseCalib_ChgEventVal::
	.ds	4
	.debug	G _fwdPhaseCalib_ChgEventVal 10 0
_fwdPhaseCalib_MinChg::
	.ds	4
	.debug	G _fwdPhaseCalib_MinChg 10 0
_revMagCalib::
	.ds	4
	.debug	G _revMagCalib 10 0
_revMagCalib_ChgEventVal::
	.ds	4
	.debug	G _revMagCalib_ChgEventVal 10 0
_revMagCalib_MinChg::
	.ds	4
	.debug	G _revMagCalib_MinChg 10 0
_revPhaseCalib::
	.ds	4
	.debug	G _revPhaseCalib 10 0
_revPhaseCalib_ChgEventVal::
	.ds	4
	.debug	G _revPhaseCalib_ChgEventVal 10 0
_revPhaseCalib_MinChg::
	.ds	4
	.debug	G _revPhaseCalib_MinChg 10 0
_cavMagCalib::
	.ds	4
	.debug	G _cavMagCalib 10 0
_cavMagCalib_ChgEventVal::
	.ds	4
	.debug	G _cavMagCalib_ChgEventVal 10 0
_cavMagCalib_MinChg::
	.ds	4
	.debug	G _cavMagCalib_MinChg 10 0
_cavPhaseCalib::
	.ds	4
	.debug	G _cavPhaseCalib 10 0
_cavPhaseCalib_ChgEventVal::
	.ds	4
	.debug	G _cavPhaseCalib_ChgEventVal 10 0
_cavPhaseCalib_MinChg::
	.ds	4
	.debug	G _cavPhaseCalib_MinChg 10 0
_spare0MagCalib::
	.ds	4
	.debug	G _spare0MagCalib 10 0
_spare0MagCalib_ChgEventVal::
	.ds	4
	.debug	G _spare0MagCalib_ChgEventVal 10 0
_spare0MagCalib_MinChg::
	.ds	4
	.debug	G _spare0MagCalib_MinChg 10 0
_spare0PhaseCalib::
	.ds	4
	.debug	G _spare0PhaseCalib 10 0
_spare0PhaseCalib_ChgEventVal::
	.ds	4
	.debug	G _spare0PhaseCalib_ChgEventVal 10 0
_spare0PhaseCalib_MinChg::
	.ds	4
	.debug	G _spare0PhaseCalib_MinChg 10 0
_spare1MagCalib::
	.ds	4
	.debug	G _spare1MagCalib 10 0
_spare1MagCalib_ChgEventVal::
	.ds	4
	.debug	G _spare1MagCalib_ChgEventVal 10 0
_spare1MagCalib_MinChg::
	.ds	4
	.debug	G _spare1MagCalib_MinChg 10 0
_spare1PhaseCalib::
	.ds	4
	.debug	G _spare1PhaseCalib 10 0
_spare1PhaseCalib_ChgEventVal::
	.ds	4
	.debug	G _spare1PhaseCalib_ChgEventVal 10 0
_spare1PhaseCalib_MinChg::
	.ds	4
	.debug	G _spare1PhaseCalib_MinChg 10 0
_spare2MagCalib::
	.ds	4
	.debug	G _spare2MagCalib 10 0
_spare2MagCalib_ChgEventVal::
	.ds	4
	.debug	G _spare2MagCalib_ChgEventVal 10 0
_spare2MagCalib_MinChg::
	.ds	4
	.debug	G _spare2MagCalib_MinChg 10 0
_spare2PhaseCalib::
	.ds	4
	.debug	G _spare2PhaseCalib 10 0
_spare2PhaseCalib_ChgEventVal::
	.ds	4
	.debug	G _spare2PhaseCalib_ChgEventVal 10 0
_spare2PhaseCalib_MinChg::
	.ds	4
	.debug	G _spare2PhaseCalib_MinChg 10 0
_outMagCalib0::
	.ds	4
	.debug	G _outMagCalib0 10 0
_outMagCalib0_ChgEventVal::
	.ds	4
	.debug	G _outMagCalib0_ChgEventVal 10 0
_outMagCalib0_MinChg::
	.ds	4
	.debug	G _outMagCalib0_MinChg 10 0
_outPhaseCalib0::
	.ds	4
	.debug	G _outPhaseCalib0 10 0
_outPhaseCalib0_ChgEventVal::
	.ds	4
	.debug	G _outPhaseCalib0_ChgEventVal 10 0
_outPhaseCalib0_MinChg::
	.ds	4
	.debug	G _outPhaseCalib0_MinChg 10 0
_outMagCalib1::
	.ds	4
	.debug	G _outMagCalib1 10 0
_outMagCalib1_ChgEventVal::
	.ds	4
	.debug	G _outMagCalib1_ChgEventVal 10 0
_outMagCalib1_MinChg::
	.ds	4
	.debug	G _outMagCalib1_MinChg 10 0
_outPhaseCalib1::
	.ds	4
	.debug	G _outPhaseCalib1 10 0
_outPhaseCalib1_ChgEventVal::
	.ds	4
	.debug	G _outPhaseCalib1_ChgEventVal 10 0
_outPhaseCalib1_MinChg::
	.ds	4
	.debug	G _outPhaseCalib1_MinChg 10 0
_fwdPhaseGain::
	.ds	4
	.debug	G _fwdPhaseGain 10 0
_fwdPhaseGain_ChgEventVal::
	.ds	4
	.debug	G _fwdPhaseGain_ChgEventVal 10 0
_fwdPhaseGain_MinChg::
	.ds	4
	.debug	G _fwdPhaseGain_MinChg 10 0
	.cseg
_MB_setGain:
	pop	iy
	lcall	$$MB_setGain
	.bseg
_fwdPhaseOffset::
	.ds	4
	.debug	G _fwdPhaseOffset 10 0
_fwdPhaseOffset_ChgEventVal::
	.ds	4
	.debug	G _fwdPhaseOffset_ChgEventVal 10 0
_fwdPhaseOffset_MinChg::
	.ds	4
	.debug	G _fwdPhaseOffset_MinChg 10 0
_fwdAmplGain::
	.ds	4
	.debug	G _fwdAmplGain 10 0
_fwdAmplGain_ChgEventVal::
	.ds	4
	.debug	G _fwdAmplGain_ChgEventVal 10 0
_fwdAmplGain_MinChg::
	.ds	4
	.debug	G _fwdAmplGain_MinChg 10 0
_fwdAmplOffset::
	.ds	4
	.debug	G _fwdAmplOffset 10 0
_fwdAmplOffset_ChgEventVal::
	.ds	4
	.debug	G _fwdAmplOffset_ChgEventVal 10 0
_fwdAmplOffset_MinChg::
	.ds	4
	.debug	G _fwdAmplOffset_MinChg 10 0
_revPhaseGain::
	.ds	4
	.debug	G _revPhaseGain 10 0
_revPhaseGain_ChgEventVal::
	.ds	4
	.debug	G _revPhaseGain_ChgEventVal 10 0
_revPhaseGain_MinChg::
	.ds	4
	.debug	G _revPhaseGain_MinChg 10 0
_revPhaseOffset::
	.ds	4
	.debug	G _revPhaseOffset 10 0
_revPhaseOffset_ChgEventVal::
	.ds	4
	.debug	G _revPhaseOffset_ChgEventVal 10 0
_revPhaseOffset_MinChg::
	.ds	4
	.debug	G _revPhaseOffset_MinChg 10 0
_revAmplGain::
	.ds	4
	.debug	G _revAmplGain 10 0
_revAmplGain_ChgEventVal::
	.ds	4
	.debug	G _revAmplGain_ChgEventVal 10 0
_revAmplGain_MinChg::
	.ds	4
	.debug	G _revAmplGain_MinChg 10 0
_revAmplOffset::
	.ds	4
	.debug	G _revAmplOffset 10 0
_revAmplOffset_ChgEventVal::
	.ds	4
	.debug	G _revAmplOffset_ChgEventVal 10 0
_revAmplOffset_MinChg::
	.ds	4
	.debug	G _revAmplOffset_MinChg 10 0
_cavPhaseGain::
	.ds	4
	.debug	G _cavPhaseGain 10 0
_cavPhaseGain_ChgEventVal::
	.ds	4
	.debug	G _cavPhaseGain_ChgEventVal 10 0
_cavPhaseGain_MinChg::
	.ds	4
	.debug	G _cavPhaseGain_MinChg 10 0
_cavPhaseOffset::
	.ds	4
	.debug	G _cavPhaseOffset 10 0
_cavPhaseOffset_ChgEventVal::
	.ds	4
	.debug	G _cavPhaseOffset_ChgEventVal 10 0
_cavPhaseOffset_MinChg::
	.ds	4
	.debug	G _cavPhaseOffset_MinChg 10 0
_cavAmplGain::
	.ds	4
	.debug	G _cavAmplGain 10 0
_cavAmplGain_ChgEventVal::
	.ds	4
	.debug	G _cavAmplGain_ChgEventVal 10 0
_cavAmplGain_MinChg::
	.ds	4
	.debug	G _cavAmplGain_MinChg 10 0
_cavAmplOffset::
	.ds	4
	.debug	G _cavAmplOffset 10 0
_cavAmplOffset_ChgEventVal::
	.ds	4
	.debug	G _cavAmplOffset_ChgEventVal 10 0
_cavAmplOffset_MinChg::
	.ds	4
	.debug	G _cavAmplOffset_MinChg 10 0
_ref0PhaseGain::
	.ds	4
	.debug	G _ref0PhaseGain 10 0
_ref0PhaseGain_ChgEventVal::
	.ds	4
	.debug	G _ref0PhaseGain_ChgEventVal 10 0
_ref0PhaseGain_MinChg::
	.ds	4
	.debug	G _ref0PhaseGain_MinChg 10 0
_ref0PhaseOffset::
	.ds	4
	.debug	G _ref0PhaseOffset 10 0
_ref0PhaseOffset_ChgEventVal::
	.ds	4
	.debug	G _ref0PhaseOffset_ChgEventVal 10 0
_ref0PhaseOffset_MinChg::
	.ds	4
	.debug	G _ref0PhaseOffset_MinChg 10 0
_ref0AmplGain::
	.ds	4
	.debug	G _ref0AmplGain 10 0
_ref0AmplGain_ChgEventVal::
	.ds	4
	.debug	G _ref0AmplGain_ChgEventVal 10 0
_ref0AmplGain_MinChg::
	.ds	4
	.debug	G _ref0AmplGain_MinChg 10 0
_ref0AmplOffset::
	.ds	4
	.debug	G _ref0AmplOffset 10 0
_ref0AmplOffset_ChgEventVal::
	.ds	4
	.debug	G _ref0AmplOffset_ChgEventVal 10 0
_ref0AmplOffset_MinChg::
	.ds	4
	.debug	G _ref0AmplOffset_MinChg 10 0
_spare0PhaseGain::
	.ds	4
	.debug	G _spare0PhaseGain 10 0
_spare0PhaseGain_ChgEventVal::
	.ds	4
	.debug	G _spare0PhaseGain_ChgEventVal 10 0
_spare0PhaseGain_MinChg::
	.ds	4
	.debug	G _spare0PhaseGain_MinChg 10 0
_spare0PhaseOffset::
	.ds	4
	.debug	G _spare0PhaseOffset 10 0
_spare0PhaseOffset_ChgEventVal::
	.ds	4
	.debug	G _spare0PhaseOffset_ChgEventVal 10 0
_spare0PhaseOffset_MinChg::
	.ds	4
	.debug	G _spare0PhaseOffset_MinChg 10 0
_spare0AmplGain::
	.ds	4
	.debug	G _spare0AmplGain 10 0
_spare0AmplGain_ChgEventVal::
	.ds	4
	.debug	G _spare0AmplGain_ChgEventVal 10 0
_spare0AmplGain_MinChg::
	.ds	4
	.debug	G _spare0AmplGain_MinChg 10 0
_spare0AmplOffset::
	.ds	4
	.debug	G _spare0AmplOffset 10 0
_spare0AmplOffset_ChgEventVal::
	.ds	4
	.debug	G _spare0AmplOffset_ChgEventVal 10 0
_spare0AmplOffset_MinChg::
	.ds	4
	.debug	G _spare0AmplOffset_MinChg 10 0
_spare1PhaseGain::
	.ds	4
	.debug	G _spare1PhaseGain 10 0
_spare1PhaseGain_ChgEventVal::
	.ds	4
	.debug	G _spare1PhaseGain_ChgEventVal 10 0
_spare1PhaseGain_MinChg::
	.ds	4
	.debug	G _spare1PhaseGain_MinChg 10 0
_spare1PhaseOffset::
	.ds	4
	.debug	G _spare1PhaseOffset 10 0
_spare1PhaseOffset_ChgEventVal::
	.ds	4
	.debug	G _spare1PhaseOffset_ChgEventVal 10 0
_spare1PhaseOffset_MinChg::
	.ds	4
	.debug	G _spare1PhaseOffset_MinChg 10 0
_spare1AmplGain::
	.ds	4
	.debug	G _spare1AmplGain 10 0
_spare1AmplGain_ChgEventVal::
	.ds	4
	.debug	G _spare1AmplGain_ChgEventVal 10 0
_spare1AmplGain_MinChg::
	.ds	4
	.debug	G _spare1AmplGain_MinChg 10 0
_spare1AmplOffset::
	.ds	4
	.debug	G _spare1AmplOffset 10 0
_spare1AmplOffset_ChgEventVal::
	.ds	4
	.debug	G _spare1AmplOffset_ChgEventVal 10 0
_spare1AmplOffset_MinChg::
	.ds	4
	.debug	G _spare1AmplOffset_MinChg 10 0
_spare2PhaseGain::
	.ds	4
	.debug	G _spare2PhaseGain 10 0
_spare2PhaseGain_ChgEventVal::
	.ds	4
	.debug	G _spare2PhaseGain_ChgEventVal 10 0
_spare2PhaseGain_MinChg::
	.ds	4
	.debug	G _spare2PhaseGain_MinChg 10 0
_spare2PhaseOffset::
	.ds	4
	.debug	G _spare2PhaseOffset 10 0
_spare2PhaseOffset_ChgEventVal::
	.ds	4
	.debug	G _spare2PhaseOffset_ChgEventVal 10 0
_spare2PhaseOffset_MinChg::
	.ds	4
	.debug	G _spare2PhaseOffset_MinChg 10 0
_spare2AmplGain::
	.ds	4
	.debug	G _spare2AmplGain 10 0
_spare2AmplGain_ChgEventVal::
	.ds	4
	.debug	G _spare2AmplGain_ChgEventVal 10 0
_spare2AmplGain_MinChg::
	.ds	4
	.debug	G _spare2AmplGain_MinChg 10 0
_spare2AmplOffset::
	.ds	4
	.debug	G _spare2AmplOffset 10 0
_spare2AmplOffset_ChgEventVal::
	.ds	4
	.debug	G _spare2AmplOffset_ChgEventVal 10 0
_spare2AmplOffset_MinChg::
	.ds	4
	.debug	G _spare2AmplOffset_MinChg 10 0
_ref1PhaseGain::
	.ds	4
	.debug	G _ref1PhaseGain 10 0
_ref1PhaseGain_ChgEventVal::
	.ds	4
	.debug	G _ref1PhaseGain_ChgEventVal 10 0
_ref1PhaseGain_MinChg::
	.ds	4
	.debug	G _ref1PhaseGain_MinChg 10 0
_ref1PhaseOffset::
	.ds	4
	.debug	G _ref1PhaseOffset 10 0
_ref1PhaseOffset_ChgEventVal::
	.ds	4
	.debug	G _ref1PhaseOffset_ChgEventVal 10 0
_ref1PhaseOffset_MinChg::
	.ds	4
	.debug	G _ref1PhaseOffset_MinChg 10 0
_ref1AmplGain::
	.ds	4
	.debug	G _ref1AmplGain 10 0
_ref1AmplGain_ChgEventVal::
	.ds	4
	.debug	G _ref1AmplGain_ChgEventVal 10 0
_ref1AmplGain_MinChg::
	.ds	4
	.debug	G _ref1AmplGain_MinChg 10 0
_ref1AmplOffset::
	.ds	4
	.debug	G _ref1AmplOffset 10 0
_ref1AmplOffset_ChgEventVal::
	.ds	4
	.debug	G _ref1AmplOffset_ChgEventVal 10 0
_ref1AmplOffset_MinChg::
	.ds	4
	.debug	G _ref1AmplOffset_MinChg 10 0
_maxAmplSet::
	.ds	4
	.debug	G _maxAmplSet 10 0
_maxAmplSet_ChgEventVal::
	.ds	4
	.debug	G _maxAmplSet_ChgEventVal 10 0
_maxAmplSet_MinChg::
	.ds	4
	.debug	G _maxAmplSet_MinChg 10 0
_numDataPoints::
	.ds	12
	.debug	G _numDataPoints 193 0
	.seg	FARBSS
_waveformData::
	.ds	24576
	.debug	G _waveformData 194 8
	.bseg
_dataTimeMS::
	.ds	24
	.debug	G _dataTimeMS 196 0
_dataTimeSecs::
	.ds	24
	.debug	G _dataTimeSecs 196 0
_lastWaveReadTime::
	.ds	12
	.debug	G _lastWaveReadTime 193 0
_readerRdy::
	.ds	6
	.debug	G _readerRdy 44 0
_lastReadPlotTime::
	.ds	2
	.debug	G _lastReadPlotTime 7 0
	.cseg
_PV_UpdateVal:
	pop	iy
	lcall	$$PV_UpdateVal
_PV_LockVal:
	pop	iy
	lcall	$$PV_LockVal
_PV_SendVal:
	pop	iy
	lcall	$$PV_SendVal
_PV_RcvVal:
	pop	iy
	lcall	$$PV_RcvVal
_PV_UpdateValWF:
	pop	iy
	lcall	$$PV_UpdateValWF
_PV_LockWave:
	pop	iy
	lcall	$$PV_LockWave
	.bseg
_locks::
	.ds	286
	.debug	G _locks 197 0
_fpgaInitialized::
	.ds	1
	.debug	G _fpgaInitialized 3 0
_showFiller::
	.ds	1
	.debug	G _showFiller 3 0
	.extern	$ifpending 175
	.extern	$startNetUp 175
	.extern	$mkfifo 182
	.extern	$perror 198
	.extern	_hdwCfgParams 199
	.extern	$initParamModule 200
	.extern	$saveParams 201
	.extern	$loadParams 201
	.extern	$loadParamsMsg 202
	.cseg
_backupd:
	pop	iy
	lcall	$$backupd
	.extern	$$backupd
	.extern	$backupd 111
	.extern	_srfsIPAddr 9
	.extern	_srfsAltAddr 9
_ca_tcpd:
	pop	iy
	lcall	$$ca_tcpd
	.extern	$$ca_tcpd
	.extern	$ca_tcpd 111
_inetd:
	pop	iy
	lcall	$$inetd
	.extern	$$inetd
	.extern	$inetd 111
	.extern	$showHelp 203
	.extern	$displayElems 204
	.extern	$displayDigElems 205
_fmtF32:
	pop	iy
	lcall	$$fmtF32
	.extern	$$fmtF32
	.extern	$fmtF32 115
_fmtU16:
	pop	iy
	lcall	$$fmtU16
	.extern	$$fmtU16
	.extern	$fmtU16 115
_fmtS16:
	pop	iy
	lcall	$$fmtS16
	.extern	$$fmtS16
	.extern	$fmtS16 115
_fmtBool:
	pop	iy
	lcall	$$fmtBool
	.extern	$$fmtBool
	.extern	$fmtBool 115
	.seg	FARBSS
_applDTM::
	.ds	32
	.debug	G _applDTM 89 8
	.bseg
_ctlLoopTicks::
	.ds	2
	.debug	G _ctlLoopTicks 7 0
_lastCtlLoopTime::
	.ds	2
	.debug	G _lastCtlLoopTime 7 0
_lastReadTime::
	.ds	2
	.debug	G _lastReadTime 7 0
_readTicks::
	.ds	2
	.debug	G _readTicks 7 0
_writeTicks::
	.ds	2
	.debug	G _writeTicks 7 0
_rwTicks::
	.ds	2
	.debug	G _rwTicks 7 0
_ADC_Mask::
	.ds	4
	.debug	G _ADC_Mask 9 0
_DAC_Mask::
	.ds	4
	.debug	G _DAC_Mask 9 0
_stateDataBuf::
	.ds	4
	.debug	G _stateDataBuf 105 0
	.cseg
_EVENT_RestoredParamFile:
	pop	iy
	lcall	$$EVENT_RestoredParamFile
_adcMinRaw:
	pop	iy
	lcall	$$adcMinRaw
	.extern	$$adcMinRaw
	.extern	$adcMinRaw 129
_adcMaxRaw:
	pop	iy
	lcall	$$adcMaxRaw
	.extern	$$adcMaxRaw
	.extern	$adcMaxRaw 129
_adcDefaults:
	pop	iy
	lcall	$$adcDefaults
	.extern	$$adcDefaults
	.extern	$adcDefaults 130
_dacMinRaw:
	pop	iy
	lcall	$$dacMinRaw
	.extern	$$dacMinRaw
	.extern	$dacMinRaw 129
_dacMaxRaw:
	pop	iy
	lcall	$$dacMaxRaw
	.extern	$$dacMaxRaw
	.extern	$dacMaxRaw 129
_dacDefaults:
	pop	iy
	lcall	$$dacDefaults
	.extern	$$dacDefaults
	.extern	$dacDefaults 130
_readGetRaw:
	pop	iy
	lcall	$$readGetRaw
_readMinRaw:
	pop	iy
	lcall	$$readMinRaw
_readMaxRaw:
	pop	iy
	lcall	$$readMaxRaw
_readDefaults:
	pop	iy
	lcall	$$readDefaults
_readCalibPoints:
	pop	iy
	lcall	$$readCalibPoints
_setGetRaw:
	pop	iy
	lcall	$$setGetRaw
_setMinRaw:
	pop	iy
	lcall	$$setMinRaw
_setMaxRaw:
	pop	iy
	lcall	$$setMaxRaw
_setDefaults:
	pop	iy
	lcall	$$setDefaults
_setCalibPoints:
	pop	iy
	lcall	$$setCalibPoints
_applic:
	pop	iy
	lcall	$$applic
_tsh:
	pop	iy
	lcall	$$tsh
	.extern	$$tsh
	.extern	$tsh 111
_configd:
	pop	iy
	lcall	$$configd
	.extern	$$configd
	.extern	$configd 111
_fastdatad:
	pop	iy
	lcall	$$fastdatad
	.extern	$$fastdatad
	.extern	$fastdatad 111
_ReadSetCalibsRestore:
	pop	iy
	lcall	$$ReadSetCalibsRestore
_hostNameRestore:
	pop	iy
	lcall	$$hostNameRestore
	.extern	$getLockDSP 127
	.extern	$releaseLockDSP 106
	.extern	$stop_DSP_FPGA 106
	.extern	$restart_DSP_FPGA 127
	.extern	$writeDSPRegQuick 206
	.extern	$writeDSPReg 207
	.extern	$readDSPReg 208
	.extern	$readPlot 209
	.extern	$read_DSP_Regs 210
	.extern	$write_DSP_Regs 211
	.extern	$readFPGA 76
	.extern	$writeFPGA 212
	.extern	$read_FPGA_Regs 213
	.extern	$write_FPGA_Regs 213
	.extern	$updateEEPROM 214
_dspRead:
	pop	iy
	lcall	$$dspRead
	.extern	$$dspRead
	.extern	$dspRead 111
_dspWrite:
	pop	iy
	lcall	$$dspWrite
	.extern	$$dspWrite
	.extern	$dspWrite 111
	.bseg
_prevState::
	.ds	8
	.debug	G _prevState 215 0
_curState::
	.ds	8
	.debug	G _curState 215 0
_nextState::
	.ds	8
	.debug	G _nextState 215 0
_ilkRatioGain::
	.ds	4
	.debug	G _ilkRatioGain 10 0
_ilkRatioOffset::
	.ds	4
	.debug	G _ilkRatioOffset 10 0
_maxOutAmplGain::
	.ds	4
	.debug	G _maxOutAmplGain 10 0
_maxOutAmplOffset::
	.ds	4
	.debug	G _maxOutAmplOffset 10 0
_lastIdleMS:
	.ds	4
	.debug	F _lastIdleMS 9 0
_lastIdleTime:
	.ds	2
	.debug	F _lastIdleTime 7 0
_applBuf::
	.ds	256
	.debug	G _applBuf 216 0
_pulseAmpl::
	.ds	1
	.debug	G _pulseAmpl 3 0
_pulsePhase::
	.ds	1
	.debug	G _pulsePhase 3 0
_ADCVal::
	.ds	2
	.debug	G _ADCVal 7 0
_autoReadDSP::
	.ds	1
	.debug	G _autoReadDSP 3 0
_autoWriteDSP::
	.ds	1
	.debug	G _autoWriteDSP 3 0
_autoReadFPGA::
	.ds	1
	.debug	G _autoReadFPGA 3 0
_autoWriteFPGA::
	.ds	1
	.debug	G _autoWriteFPGA 3 0
_rfIsOn::
	.ds	1
	.debug	G _rfIsOn 3 0
_rfPulsing::
	.ds	1
	.debug	G _rfPulsing 3 0
_tunerCtlisON::
	.ds	1
	.debug	G _tunerCtlisON 3 0
_rfAtWarm::
	.ds	1
	.debug	G _rfAtWarm 3 0
_rfRamping::
	.ds	1
	.debug	G _rfRamping 3 0
_readWaves::
	.ds	1
	.debug	G _readWaves 3 0
_cavLocked::
	.ds	1
	.debug	G _cavLocked 3 0
_amplCtlisON::
	.ds	1
	.debug	G _amplCtlisON 3 0
_curProgNum::
	.ds	2
	.debug	G _curProgNum 7 0
_phaseMode::
	.ds	2
	.debug	G _phaseMode 7 0
_lastIOTime::
	.ds	2
	.debug	G _lastIOTime 7 0
	.cseg
_readFromFile:
	pop	iy
	lcall	$$readFromFile
_autocal:
	pop	iy
	lcall	$$autocal
_cmd:
	pop	iy
	lcall	$$cmd
_defaults:
	pop	iy
	lcall	$$defaults
_reset_DSP_FPGA:
	pop	iy
	lcall	$$reset_DSP_FPGA
_setDSPfirm:
	pop	iy
	lcall	$$setDSPfirm
_fpgaRead:
	pop	iy
	lcall	$$fpgaRead
_fpgaWrite:
	pop	iy
	lcall	$$fpgaWrite
_setFPGAfirm:
	pop	iy
	lcall	$$setFPGAfirm
_fmtPhaseMode:
	pop	iy
	lcall	$$fmtPhaseMode
_fmtWarming:
	pop	iy
	lcall	$$fmtWarming
_fmtRamping:
	pop	iy
	lcall	$$fmtRamping
_fmtLocked:
	pop	iy
	lcall	$$fmtLocked
_showState:
	pop	iy
	lcall	$$showState
_dspcalib:
	pop	iy
	lcall	$$dspcalib
_dspio:
	pop	iy
	lcall	$$dspio
	.debug	T	49000320000A00050001004600000008636F696C56616C00030002
	.debug	T	0008636F696C4E756D0030000300056E616D6500D90007000C
	.debug	T	736574436F696C46756E63000300090006666C61677300
	.debug	T	AD0003210014000C000100530000000772656756616C0005000200
	.debug	T	077265674E756D0030000400056E616D6500DA0008000B7365
	.debug	T	7452656746756E630003000A00096461746154797065000300
	.debug	T	0B00086E756D526567730003000C000777724D61736B000300
	.debug	T	0D0006666C6167730003000E000C636865636B4576656E7473
	.debug	T	0003000F000E6576656E74734F636375726564004E0010000B
	.debug	T	6C61737443686756616C004E001200076D696E43686700
	.debug	T	0900012200160003001600
	.debug	T	0900012300040024000200
	.debug	T	070000240002002000
	.debug	T	0900012500CE0026006700
	.debug	T	070000260002002100
	.debug	T	0900012700A8012600D400
	.debug	T	0900012800910029000500
	.debug	T	8A000329001D0009000950565F446573635F006000000007707649
	.debug	T	6E666F0069000D000476616C00070011000876616C53697A65
	.debug	T	00030013000A6E65774576656E747300030014000E6576656E
	.debug	T	74734F63637572656400DB0015000A75706461746556616C00
	.debug	T	DC001700086C6F636B56616C00DD0019000873656E6456616C
	.debug	T	00DD001B000772637656616C00
	.debug	T	0900012A00060007000300
	.debug	T	0900012B00180003001800
	.debug	T	0900012C00060003000600
	.debug	T	1D00032D000600020001000900000005626175640006000400056D
	.debug	T	6F646500
	.debug	T	0900012E00010003000100
	.debug	T	0900012F00080030000200
	.debug	T	070008300004000300
	.debug	T	0900013100020003000200
	.debug	T	0900013200100030000400
	.debug	T	0900013300480034000400
	.debug	T	820003340012000900010046000000087372764E616D6500070002
	.debug	T	0005706F727400030004000970726F746F636F6C00DE000500
	.debug	T	0970726F634D61696E0003000700097072696F726974790006
	.debug	T	000800066465766664004E000A000A636667506172616D7300
	.debug	T	06000C00086C6173744572720009000E000C6C617374457272
	.debug	T	54696D6500
	.debug	T	0900013500240009000900
	.debug	T	0900013600800030002000
	.debug	T	0900013700900538005900
	.debug	T	49000338001000060001000400000003696400040002000573697A
	.debug	T	650004000400056772616E006900060005646174610030000A
	.debug	T	00056E616D6500DF000E000B726573746F726556616C00
	.debug	T	0900013900800038000800
	.debug	T	0900013A00100038000100
	.debug	T	7200033B003100080001004600000009626173654E616D65000300
	.debug	T	02000963617465676F727900E000030007706172616D7300E1
	.debug	T	0007000B7772697465436F756E74000600270003666400E200
	.debug	T	29000962626D43616368650007002D000A636163686553697A
	.debug	T	650007002F00076F666673657400
	.debug	T	0900013C0006003D000300
	.debug	T	0700003D0002003B00
	.debug	T	0900013E0063033F003300
	.debug	T	5800033F0011000700010009000000036964000300040005747970
	.debug	T	650026000500066D62526567007C000700056761696E007C00
	.debug	T	0900076F66667365740009000B00087479706963616C000700
	.debug	T	0F000A706572634E6F69736500
	.debug	T	0900014000280541004200
	.debug	T	630003410014000800010009000000036964000300040005747970
	.debug	T	650026000500066D62526567000A000700076D696E56616C00
	.debug	T	0A000B00076D617856616C0003000F000A77726974654D6F64
	.debug	T	65007C001000056761696E007C001200076F666673657400
	.debug	T	09000142001A0043000D00
	.debug	T	070000430002000500
	.debug	T	0900014400080043000400
	.debug	T	0900014500080046000400
	.debug	T	070000460002000300
	.debug	T	0F00024700020006000700000009000200
	.debug	T	070000480002004900
	.debug	T	7702034900E8002D000C7461736B5F73747275637400E400000006
	.debug	T	737461636B00E40002000C737461636B5F7374617274000700
	.debug	T	04000B737461636B5F73697A65000300060005737365670003
	.debug	T	0007000773746B6C6C7200030008000773746B686C72000300
	.debug	T	09000677707361720007000A0008777073616C68720003000C
	.debug	T	000677707362720007000D0008777073626C687200E5000F00
	.debug	T	064644536574000300110009757365436F756E740003001200
	.debug	T	08636C61696D65640007001300067374617465004800150007
	.debug	T	6D794164647200030017000A7A65726F416674657200060018
	.debug	T	0005617267630070001A0005617267760062001C0005617267
	.debug	T	7300E6009C0007776169745173000900A0000C7369675F7065
	.debug	T	6E64696E67000900A4000C7369675F626C6F636B6564000600
	.debug	T	A8000E707265656D70745F636F756E74000300AA0006696E55
	.debug	T	7365000300AB000565786974000300AC000A6C6F675374646F
	.debug	T	7574000300AD000B7265646972537464696F000900AE000A64
	.debug	T	696167466C61677300DE00B2000970726F634D61696E000600
	.debug	T	B40004706964000600B6000570706964000300B80009707269
	.debug	T	6F72697479000600B900086C617374457272000900BB000873
	.debug	T	746172746564000900BF000C6C617374526573756D65640009
	.debug	T	00C3000C6C61737453746F70706564000900C7000C6C617374
	.debug	T	52756E6E696E67000900CB000D726573756D65645469636B73
	.debug	T	000900CF000A7469636B7355736564000900D300076D735573
	.debug	T	6564000300D700096F7574436F756E74003000D800076F7574
	.debug	T	507472000900DC000B736C6565705469636B73000900E0000D
	.debug	T	736C65657053746172746564000900E4000D74696D656F7574
	.debug	T	5469636B7300
	.debug	T	0700004A0002004B00
	.debug	T	2800034B001A00030001000600000006636F756E74000600020006
	.debug	T	6F776E657200E70004000473656D00
	.debug	T	0B00024C000100060007000000
	.debug	T	0F00024D0002000A004E00000026000200
	.debug	T	0700004E0002000100
	.debug	T	0F00024F00020008004E00000026000200
	.debug	T	0F00025000020009004E00000026000200
	.debug	T	1300025100030001004E000000260002000A000400
	.debug	T	17000252000400010026000000530002005400040055000800
	.debug	T	070000530002000400
	.debug	T	070008540004003F00
	.debug	T	070000550002000900
	.debug	T	17000256000400010026000000530002005700040055000800
	.debug	T	070008570004004100
	.debug	T	1700025800040003004E000000030002004E00040006000600
	.debug	T	0900015900200003002000
	.debug	T	0B00025A00010006005B000000
	.debug	T	0700005B0002002900
	.debug	T	1300025C00030006005B0000000600020006000400
	.debug	T	1700025D000400060006000000070002004E0004005E000600
	.debug	T	0700005E0002005F00
	.debug	T	1300025F00030006004E0000006900020007000600
	.debug	T	52000360000D000600010030000000056E616D6500030004000964
	.debug	T	6174615479706500070005000573697A65000700070009656C
	.debug	T	656D53697A650007000900096E756D456C656D730007000B00
	.debug	T	06666C61677300
	.debug	T	0B000261000100010001000000
	.debug	T	0900016200800003008000
	.debug	T	070008630004000A00
	.debug	T	0B000264000100010003000000
	.debug	T	0B000265000100010066000000
	.debug	T	070000660002006700
	.debug	T	3C000367001600040001000700000008646174614C656E004E0002
	.debug	T	000A6576656E7446756E630006000400086368616E4E756D00
	.debug	T	E8000600056461746100
	.debug	T	130002680003000100690000004600040007000600
	.debug	T	070008690004000100
	.debug	T	0B00026A000100060001000000
	.debug	T	0700086B0004006C00
	.debug	T	2F00036C0008000300010007000000096461746153697A65000700
	.debug	T	02000463726300090004000A726573657454696D6500
	.debug	T	0F00026D0002000A000A0000000A000400
	.debug	T	1300026E00030006004E0000004600020006000400
	.debug	T	0F00026F00020001000600000070000200
	.debug	T	070000700002004600
	.debug	T	0900017100030003000300
	.debug	T	0900017200060026000300
	.debug	T	0F00027300020046007400000046000400
	.debug	T	070008740004007500
	.debug	T	56000375000E0007000A44697370456C656D5F0003000000027800
	.debug	T	03000100027900690002000476616C00460006000661747472
	.debug	T	7300E900080008666D7446756E630006000A00067769647468
	.debug	T	0006000C00057072656300
	.debug	T	0900017600460403004604
	.debug	T	0900017700840275002E00
	.debug	T	09000178000E0079000100
	.debug	T	4B000379000E000700010003000000027800030001000279000900
	.debug	T	0200056D61736B004600060005737472300046000800066174
	.debug	T	7472300046000A0005737472310046000C0006617474723100
	.debug	T	1300027A0003004600090000000300040046000600
	.debug	T	1300027B0003004600090000000900040046000800
	.debug	T	0700007C0002000A00
	.debug	T	0900017D0008000A000200
	.debug	T	0900017E00400003004000
	.debug	T	0B00027F000100060009000000
	.debug	T	070008800004000900
	.debug	T	0B0002810001000A0007000000
	.debug	T	130002820003000100070000007C0002007C000400
	.debug	T	0F00028300020046000700000046000200
	.debug	T	0900018400060046000300
	.debug	T	0900018500820086000D00
	.debug	T	2F000386000A00030001003000000008636D644E616D6500DE0004
	.debug	T	000970726F634D61696E003000060006646573637200
	.debug	T	0B000287000100090055000000
	.debug	T	130002880003004E004E0000004E00020007000400
	.debug	T	130002890003004E004E0000000600020007000400
	.debug	T	0F00028A00020006004600000046000200
	.debug	T	0B00028B000100060046000000
	.debug	T	1300028C0003000600460000004600020007000400
	.debug	T	1300028D0003006900690000006900040009000800
	.debug	T	1300028E0003006900690000000600040009000600
	.debug	T	0F00028F00020001003000000030000400
	.debug	T	0F00029000020001004600000030000200
	.debug	T	0B000291000100090001000000
	.debug	T	0F00029200020007000900000009000400
	.debug	T	1300029300030046000A0000000600040046000600
	.debug	T	0B000294000100010006000000
	.debug	T	0F00029500020009004600000046000200
	.debug	T	130002960003000100060000006600020006000400
	.debug	T	13000297000300060006000000460002004E000400
	.debug	T	0900019800000003000000
	.debug	T	0F00029900020006005500000006000200
	.debug	T	1300029A0003000600060000005500020055000400
	.debug	T	0B00029B000100060055000000
	.debug	T	0900019C00000049000000
	.debug	T	0700009D0002009E00
	.debug	T	5400039E000C0006000100EA000000065F5F6F757400EC00020005
	.debug	T	5F5F696E00EE000400085F5F777269746500EE000600075F5F
	.debug	T	726561640006000800085F5F756E676574004E000A000B5F5F
	.debug	T	757365724461746100
	.debug	T	1300029F00030006009D000000300002000D000000
	.debug	T	0F0002A00002000600300000000D000000
	.debug	T	130002A1000300060030000000300004000D000000
	.debug	T	090001A200080009000200
	.debug	T	0F0002A30002000B004600000070000200
	.debug	T	130002A40003000800460000007000020006000400
	.debug	T	130002A50003000700460000007000020006000400
	.debug	T	090001A60000000A000000
	.debug	T	090001A700000007000000
	.debug	T	090001A80004000A000100
	.debug	T	090001A90018002B000100
	.debug	T	090001AA0030002B000200
	.debug	T	090001AB0000032B002000
	.debug	T	0B0002AC000100010009000000
	.debug	T	0B0002AD000100010007000000
	.debug	T	130002AE0003000600460000000600020006000400
	.debug	T	0B0002AF000100060006000000
	.debug	T	130002B00003000600060000004E00020006000400
	.debug	T	0B0002B100010006009D000000
	.debug	T	270103B200F900140001000900000008696E666F4B657900090004
	.debug	T	00086262436F756E74000700080006746F4964780007000A00
	.debug	T	0866726F6D4964780003000C0008777261707065640003000D
	.debug	T	000D63746C6453687574646F776E0007000E00097764745469
	.debug	T	636B730009001000087365636F6E647300070014000A6D696C
	.debug	T	6C69536563730009001600076D7354696D6500F0001A000866
	.debug	T	696C6C6572310006006A0003747A0009006C000D6864775265
	.debug	T	73657454696D6500090070000B7265626F6F7454696D650003
	.debug	T	0074000D646964536F6674526573657400090075000C646967
	.debug	T	4F7574537461746500090079000972746F7356657273000900
	.debug	T	7D000D74696D65725365636F6E647300090081000B696E7649
	.debug	T	6E666F4B657900F10085000866696C6C65723200
	.debug	T	230002B30007000600DE0000000300020006000400700006000600
	.debug	T	080006000A0006000C00
	.debug	T	0B0002B4000100090009000000
	.debug	T	170002B5000400460046000000460002000700040007000600
	.debug	T	0F0002B600020006004600000006000200
	.debug	T	0F0002B70002000600F200000009000200
	.debug	T	130002B80003000600F20000000600020007000400
	.debug	T	0B0002B90001000600F2000000
	.debug	T	130002BA0003000300260000000300020005000400
	.debug	T	130002BB0003000300260000000300020009000400
	.debug	T	130002BC000300030026000000030002000A000400
	.debug	T	090001BD00200026001000
	.debug	T	170002BE000400460026000000530002000700040046000600
	.debug	T	0F0002BF000200060006000000F3000200
	.debug	T	170002C000040003004E000000030002000300040006000600
	.debug	T	090001C1000C0007000600
	.debug	T	090001C2000060C3000600
	.debug	T	090001C300001009000004
	.debug	T	090001C400180009000600
	.debug	T	090001C5001E014B000B00
	.debug	T	0B0002C6000100010030000000
	.debug	T	090001C700000038000000
	.debug	T	0F0002C800020006004600000007000200
	.debug	T	170002C9000400060046000000070002003D00040003000600
	.debug	T	0B0002CA00010006003D000000
	.debug	T	130002CB0003000300060000000600020070000400
	.debug	T	130002CC0003000100740000000300040046000600
	.debug	T	170002CD000400010009000000F40004000300080046000A00
	.debug	T	0F0002CE00020006000900000009000400
	.debug	T	130002CF0003000600090000000900040003000800
	.debug	T	0F0002D000020006000900000055000400
	.debug	T	130002D10003000600090000008000040007000800
	.debug	T	0B0002D2000100060054000000
	.debug	T	0B0002D3000100060057000000
	.debug	T	0F0002D400020006000700000007000200
	.debug	T	130002D50003000600F50000000700020007000400
	.debug	T	1B0002D60005000600070000000600020009000400030008000300
	.debug	T	0A00
	.debug	T	280003D700080002000100090000000A6469676974616C496E0009
	.debug	T	0004000B6469676974616C4F757400
	.debug	T	090001D800000103000001
	.debug	T	070000D9000200C000
	.debug	T	070000DA0002005800
	.debug	T	070000DB0002005A00
	.debug	T	070000DC0002005C00
	.debug	T	070000DD0002005D00
	.debug	T	070000DE0002006F00
	.debug	T	070000DF0002006800
	.debug	T	070008E00004003800
	.debug	T	090001E100200007001000
	.debug	T	070008E2000400E300
	.debug	T	330003E3000A0004000100090000000574696D6500070004000573
	.debug	T	697A6500070006000864657654797065000700080004637263
	.debug	T	00
	.debug	T	070000E40002000700
	.debug	T	070000E50002006900
	.debug	T	280003E60004000100125F5F776169745F71756575655F68656164
	.debug	T	00F6000000096C6973744865616400
	.debug	T	680003E70016000800010030000000056E616D6500E6000400076F
	.debug	T	70656E656400E6000800076C6F636B656400E6000C0008626C
	.debug	T	6F636B6564000300100009756E6C696E6B6564000300110006
	.debug	T	696E55736500060012000676616C7565004E001400076D7941
	.debug	T	64647200
	.debug	T	090001E800100003001000
	.debug	T	070000E90002007300
	.debug	T	070000EA000200EB00
	.debug	T	0F0002EB0002000600030000004E000200
	.debug	T	070000EC000200ED00
	.debug	T	0B0002ED00010006004E000000
	.debug	T	070000EE000200EF00
	.debug	T	130002EF000300080030000000080004004E000800
	.debug	T	090001F000500003005000
	.debug	T	090001F100740003007400
	.debug	T	070000F2000200E700
	.debug	T	070000F30002006000
	.debug	T	070008F40004007900
	.debug	T	070000F50002004300
	.debug	T	260003F600040002000A6C6973745F6865616400F7000000056E65
	.debug	T	787400F7000200057072657600
	.debug	T	070000F7000200F600
	.seg	CONST
@2:
	.db	"console",0,"fastdatad",0,"modbusTCPd",0,"tsh",0,"settings",0,"devData",0,"hdwcfg",0,"    OPEN",0,"PhaseCtl",0,"SEL Ctl ",0,"?"
	.db	"???????",0,"16:38:59",0,"Fri Jan 11 2013",0,"fwd",0,"rev",0,"cav",0,"amplCtl",0,"on",0,"off",0,"tunerCtl",0,"/srfs/dspprog.bi"
	.db	"n",0,"-v",0,"-l",0,"auto",0,"/srfs/llrf.bin",0," WARM  ",0,"WARMING",0,"RAMPING",0,"STEADY ",0,"LOCKED  ",0,"UNLOCKED",0,34,0,"read",0
	.db	"write",0,"filler",0,"waves",0,"V",0,"255.255.0.0",0,"192.168.0.1",0,"fifoEvents",0,"inetd",0,"configd",0,"backupd",0,"ca_tcpd"
	.db	0
	.seg	FARCONST
@1:
	.db	"DoNothing",0,"dlmVersTime",0,"rtosVersTime",0,"applVersTime",0,"devType_",0,"deviceID_",0,"upSecs",0,"percentIdle"
	.db	"_",0,"numChans",0,"coilChanMult",0,"reg3ChanMult",0,"reg4ChanMult",0,"digInBits",0,"statusBitsStdA",0,"statusBits"
	.db	"StdB",0,"statusBitsAppl",0,"latency",0,"tcpTickTime",0,"applLoopTime",0,"kernelLoopTime",0,"statusBitsFPGA",0,"st"
	.db	"atusBitsTuner",0,"statusBitsRF",0,"statusBitsILK",0,"spark0_ctr",0,"spark1_ctr",0,"cavlow_ctr",0,"rev_high_ct"
	.db	"r",0,"dvdt_ctr",0,"fwd_low_ctr",0,"fwd_high_ctr",0,"PLC_ctr",0,"safety_ctr",0,"fwdPhase",0,"revPhase",0,"cavPhase",0,"ref"
	.db	"0Phase",0,"spare0Phase",0,"spare1Phase",0,"spare2Phase",0,"ref1Phase",0,"fwdAmpl",0,"revAmpl",0,"cavAmpl",0,"ref0Ampl"
	.db	0,"spare0Ampl",0,"spare1Ampl",0,"spare2Ampl",0,"ref1Ampl",0,"maxFwdAmpl",0,"maxRevAmpl",0,"maxCavAmpl",0,"maxSpare0"
	.db	"Ampl",0,"maxSpare1Ampl",0,"maxSpare2Ampl",0,"maxOutAmpl",0,"statusBitsDSP",0,"dspLoad",0,"pcbTemp",0,"fan0Spd",0,"fan"
	.db	"1Spd",0,"rfq_t1_output",0,"rfq_t2_output",0,"piezoReadPos",0,"stepperReadPos",0,"percentFFT",0,"sizeFFT",0,"romPr"
	.db	"ogErrs",0,"timeZone_",0,"curTime",0,"ipAddr",0,"netmask",0,"gateway",0,"setACoil",0,"resetACoil",0,"digOutBits",0,"cmdBit"
	.db	"sStd",0,"cmdBitsAppl",0,"ADC_err_rst",0,"RF_soft_on",0,"RF_ilk_rst",0,"ilk_ctr_rst1",0,"ilk_ctr_rst2",0,"trigPos",0
	.db	"warmLevel",0,"warmTime",0,"rampTime",0,"fwdAtten",0,"revAtten",0,"cavAtten",0,"spare0Atten",0,"spare1Atten",0,"spare2"
	.db	"Atten",0,"outAtten",0,"phaseSet",0,"amplSet",0,"pulseFreq",0,"pulseDuty",0,"rf_P",0,"rf_I",0,"rf_D",0,"tuner_P",0,"tuner_I",0,"tu"
	.db	"ner_D",0,"rf_B0",0,"rf_WCtrl",0,"rf_WObs",0,"tuner_B0",0,"tuner_WCtrl",0,"tuner_WObs",0,"pulseAmplSet",0,"pulsePhaseS"
	.db	"et",0,"cavLowIlkLevel",0,"cavLowIlkTime",0,"dvdtIlkRate",0,"revHighIlkRatio",0,"revHighIlkTime",0,"fwdLowIlkR"
	.db	"atio",0,"fwdLowIlkTime",0,"fwdHighIlkLevel",0,"fwdHighIlkTime",0,"fwdCavPhase",0,"ffPhase",0,"ffGain",0,"freqOffs"
	.db	"et",0,"rfq_manTuneSet",0,"rfq_t1_offset",0,"rfq_t2_offset",0,"rfq_t1_feedback",0,"rfq_t2_feedback",0,"setDSPS"
	.db	"tate",0,"piezoSetPos",0,"stepperSetPos",0,"fwdMagCalib",0,"fwdPhaseCalib",0,"revMagCalib",0,"revPhaseCalib",0,"ca"
	.db	"vMagCalib",0,"cavPhaseCalib",0,"spare0MagCalib",0,"spare0PhaseCalib",0,"spare1MagCalib",0,"spare1PhaseCal"
	.db	"ib",0,"spare2MagCalib",0,"spare2PhaseCalib",0,"outMagCalib0",0,"outPhaseCalib0",0,"outMagCalib1",0,"outPhaseC"
	.db	"alib1",0,"fwdPhaseGain",0,"fwdPhaseOffset",0,"fwdAmplGain",0,"fwdAmplOffset",0,"revPhaseGain",0,"revPhaseOffs"
	.db	"et",0,"revAmplGain",0,"revAmplOffset",0,"cavPhaseGain",0,"cavPhaseOffset",0,"cavAmplGain",0,"cavAmplOffset",0,"re"
	.db	"f0PhaseGain",0,"ref0PhaseOffset",0,"ref0AmplGain",0,"ref0AmplOffset",0,"spare0PhaseGain",0,"spare0PhaseOf"
	.db	"fset",0,"spare0AmplGain",0,"spare0AmplOffset",0,"spare1PhaseGain",0,"spare1PhaseOffset",0,"spare1AmplGain"
	.db	0,"spare1AmplOffset",0,"spare2PhaseGain",0,"spare2PhaseOffset",0,"spare2AmplGain",0,"spare2AmplOffset",0,"r"
	.db	"ef1PhaseGain",0,"ref1PhaseOffset",0,"ref1AmplGain",0,"ref1AmplOffset",0,"maxAmplSet",0,"pvStrTest",0,"AmplPha"
	.db	"seLead",0,"AmplPhaseTrail",0,"Stop RF pulsing via intern src",0,"Start RF pulsing using intern sr"
	.db	"c",0,"Disable RF pulsing via extern trig",0,"Enable RF pulsing via extern trig",0,"Set Ctl to Aut"
	.db	"o Mode",0,"Set Ctl to Manual Mode",0,"Disable RF Freq Offset",0,"Enable RF Freq Offset",0,"Use cav ch"
	.db	"an for RF ampl feedback",0,"Use fwd chan for RF ampl feedback",0,"Disable RF Ampl ctl",0,"Enable "
	.db	"RF Ampl ctl",0,"Disable RF Phase Ctl and SEL",0,"Enable RF Phase Ctl, disable SEL",0,"Disable RF "
	.db	"Phase Ctl, enable SEL",0,"Disable Tuner Ctl",0,"Enable Tuner Ctl",0,"timeZone",0,"srfsIPAddr",0,"ainLabel"
	.db	0,"aoutLabel",0,"dinLabel",0,"doutLabel",0,"readGain",0,"readOffset",0,"setGain",0,"setOffset",0,"hostName",0,"srfsAltA"
	.db	"ddr",0,"extraInLabel",0,"Lock %d still held by %.15s[%d], count %d",13,10,0,"*** releaseLock(%u): Inv"
	.db	"alid lock ID ***",10,0,"*** releaseLock(%u): Lock owned by: %u ***",10,0,"*** releaseLock(%u) wit"
	.db	"h count of %u ***",10,0,"new value for DSP reg: 0x%08lX: %.32s",13,10,0,"setDSPstate: Set: %ls",10,0,"<in"
	.db	"valid>",0,"PV_SendVal(%ls)",13,10,0,"PV_RcvVal(%ls)",13,10,0,"New data avail for PV val: [%ls]",13,10,0,"Updated "
	.db	"copy of PV val: [%ls]",13,10,0,"new timezone value: %d",10,0,13,10,"===== Initial Digital Input states "
	.db	"=====",13,10,0,"%ls",13,10,0,13,10,"===== Initial Digital Output states =====",13,10,0,"restore from param file: "
	.db	"%.24s",10,0,"%s %s",0,"*** Saving ApplState ***",13,10,0,13,10,"*** Restored Application State ***",13,10,10,0,"AIN-%"
	.db	"u",0,"AOUT-%u",0,"DIN-%u",0,"DOUT-%u",0,"ExtraIN-%u",0,"Usage:  autocal fwd|rev|cav <V-peak> <Phase>",13,10,0,"In"
	.db	"valid argument: %.32s",13,10,0,"%3d",13,0,"Usage:  cmd amplCtl on|off",13,10,"            tunerCtl on|off",13
	.db	10,0,"Invalid parameters",13,10,0,10,"usage:  setdspfirm [-v] [fileName] [-l<limit>]",13,10,0,13,10,"Downloadin"
	.db	"g %.40s to EEPROM",13,10,0,"Usage:  fpgaread <reg> <count>",13,10,"                 auto on|off",13,10,0,"ge"
	.db	"tLock(...)",0,"  0x%02X:  ",0,"<error>",13,10,0,"0x%02X",13,10,0,"Usage:  fpgawrite <reg> <value>",13,10,"          "
	.db	"        auto on/off",13,10,0,"  write 0x%02X to 0x%02X",13,10,0,"writeFPGA()",0,10,"usage:  setfpgafirm [-v"
	.db	"] [fileName] [-l<limit>]",13,10,0,27,"[%02d;%02dH",0,"%12ld",0,"%12lu",0,"%14ld  ",0,13,10," Usage:  dspcalib <regN"
	.db	"ame>",13,10,0,13,10," Register not related to a DSP value",13,10,0,13,10," Value does not require conversion",13
	.db	10,0,13,10," Existing:",0,"  Gain: %s",0,"  Offset: %s",0," Test point %u:  Enter measured/actual value f"
	.db	"or %ls: ",0,13,10," Unusable calibration data",13,10,0,13,10," New:",0,"Usage:  dspio <read|write|filler|wave"
	.db	"s> <on|off>",13,10,0,"invalid option",13,10,0,"Setting default labels for I/O signals",13,10,0,"FPGA_DONE",0,"CPU"
	.db	"_RDY",0,"ACTY",0,"Setting in-RAM device config to default values",13,10,0,"Application prepping for s"
	.db	"hutdown",13,10,0,"applic process started",13,10,0,"*** restoreApplState() failed (%d):  Doing CPU res"
	.db	"et ***",13,10,0,13,10,"***** Network did not come up! *****",13,10,0,"free memory: %ld",13,10,0,"autocal",0,"calibrat"
	.db	"e fwd, rev, or cav Atten value",0,"cmd",0,"control DSP state",0,"defaults",0,"set FPGA params to defa"
	.db	"ult values",0,"dspcalib",0,"calibrate gain/offset for DSP/EGU conversion",0,"dspio",0,"enable/disable"
	.db	" normal com with DSP",0,"dspread",0,"read values from the DSP",0,"dspwrite",0,"write a value to the D"
	.db	"SP",0,"fpgaread",0,"read values from the FPGA",0,"fpgawrite",0,"write a value to the FPGA",0,"resetfpgads"
	.db	"p",0,"Reload the FPGA firmware from EEPROM",0,"setdspfirm",0,"Update and reload DSP firmware imag"
	.db	"e",0,"setfpgafirm",0,"Update and reload FPGA firmware image",0,"Show state of controller",0
	.end
