	.objcmt	Softools, Inc. ** Control Cross-C Globally Optimizing Compiler ** Version 2.18.00
	.objcmt	Copyright (c) 1989 - 2003, Softools, Inc.  All rights reserved.
	.debug	P 5
	.debug	X 11 10
	.debug	X 12 10
	.debug	N 'C:\work\dev\Rabbit_ST\clients\backup.c' 29 25 12 9 1 33
;1 //----- backup.c ----- 10/02/12 -----(07/19/06)-----
;2 
;3 //-------------------------------------------------------------------------
;4 //  Process to automatically handle the following:
;5 //   - backup and restore "persistent data" values
;6 //   - copy all new entries the in-RAM log to a remote file
;7 //   - keep the controller's clock in sync with an external time server
;8 //-------------------------------------------------------------------------
;9 // Persistent values:
;10 //
;11 //  Each application defines a "paramFiles" table for data that it wants to
;12 //  be "persistent".  Persistent, in this case, means values that can be
;13 //  changed while the application is running and need to be restored to the
;14 //  value they last had whenever the controller is rebooted.
;15 //
;16 //  This is accomplished by use of 2 structures:
;17 //
;18 //  ParamData:  Each ParamData structure references a single logical value
;19 //              or data structure.  An array of ParamData structures is use
;20 //              to define a group of related values that will be backed up
;21 //              in to a single file.
;22 //
;23 //  ParamFile:  A ParamFile structure defines the relationshsip between an
;24 //              array of ParamData structures and a file for storing a copy
;25 //              of the current values referenced by the ParamData elements.
;26 //              On startup, the controller attempts to open each file in
;27 //              a list of BackupFile elements and restore any values found
;28 //              that match the corresponding ParamData elements.
;29 //
;30 //  Use of the ParamData structure to reference individual elements makes
;31 //  it much simpler to alter the persistent data elements used by an
;32 //  applicaton.  By organizing data elements in to paramterized lists rathe
;33 //  than fixed monolithic structures, each version of an application can
;34 //  use the elements it understands and ignore those it doesn't.  Some elem
;35 //  (such as board-specific values like ADC/DAC calibration values) can eve
;36 //  be accessed independent of the application.
;37 //
;38 //  Newer versions of the application can read a file created by an older
;39 //  version, using the elements that still apply, and ignoring values that 
;40 //  no longer needed.  Older versions can read a file written by a newer
;41 //  version, ignoring newer elements that they don't know about.
;42 //
;43 //  Not perfect, of course, since values might be missing either way.  But 
;44 //  is far better than having it crash because some fixed structure changed
;45 //  the data elements don't line up (or wasting space with placeholders for
;46 //  elements no longer in use).
;47 //
;48 //  NOTE:  Because some values may NOT be in the file (or the file might no
;49 //         even exist yet), all persistent values should be initialized to
;50 //         reasonable default values on startup before attempting to restor
;51 //         them from the corresponding file.
;52 //-------------------------------------------------------------------------
;53 //  Update Log (most recent first)
;54 //
;55 // ---------------------------------------------------
;56 //   *** TO-DO ***
;57 //               - Add locks to avoid possible concurrent calls (by the bac
;58 //                 task and commands like save and reload) to the functions
;59 //                 that access the param files.
;60 // ---------------------------------------------------
;61 //
;62 //  2012-10-02 - Mark Davis (*)
;63 //               - Modified the parameters for how often changes to cached
;64 //                 copies of persistent values may be flushed to flash.  Pr
;65 //                 to this, flash updates were happing frequently enough th
;66 //                 the limited # of writes to a block of flash was likely t
;67 //                 be exceeded on some devices before too long.
;68 //
;69 //  2012-03-07 - Mark Davis
;70 //               - diagLogInfo->hdwResetTime is now updated when the global
;71 //                 hdwResetTime is
;72 //
;73 //  2011-12-08 - Mark Davis (*)
;74 //               - Until I can add proper locking, and get all the opens an
;75 //                 closes balanced/in sync, I changed the functions that
;76 //                 need to open a file to close it first if it is already o
;77 //                 Without this, the last change resulted in errors because
;78 //                 the opens/closes were not balanced, causing EBUSY errors
;79 //                 when an attempt to open a file found it already/still op
;80 //
;81 //  2011-12-07 - Mark Davis (*)
;82 //               - Moved the call to closeParamFile() in checkCache() so th
;83 //                 param file will be closed when the next attempt to open 
;84 //                 occurs.  This is necessary because the previous set of
;85 //                 changes uses the short-term (incomplete) method of guard
;86 //                 against concurrent use of a file by having the functions
;87 //                 that open a param file fail if it is already open.
;88 //
;89 //  2011-11-21 - Mark Davis (*)
;90 //               - Changes to eliminate "orphaned" fd's for param files.  P
;91 //                 to this, a param file of the wrong size would result in
;92 //                 repeated loops through the logic to rewrite the file tha
;93 //                 would accumulate "orphaned" open fd's for the file, quic
;94 //                 making it impossible to access ANY flash file.
;95 //
;96 //  2010-12-14 - Mark Davis
;97 //               - Changed secTimerOffset to hdwResetTime
;98 //
;99 //  2010-06-29 - Mark Davis
;100 //               - Added log message for memory allocation calls
;101 //
;102 //  2010-05-02 - Mark Davis (*)
;103 //               - Fixed bug in new code that stops a cached copy of settin
;104 //                 data from being flushed to the flash file if the file si
;105 //                 was not the same size as the cache.
;106 //
;107 //  2010-04-01 - Mark Davis
;108 //               - Parameter lists with no entries are now ignored.  This
;109 //                 avoids the creation of 0-length files and reduces the
;110 //                 overhead a bit of such "lists" (some lists are standard 
;111 //                 expected to exist even if they are empty).
;112 //
;113 //  2010-03-30 - Mark Davis
;114 //               - A BBRAM (Battery-Backed RAM) cache is now created and us
;115 //                 for all parameter lists of type PDC_SET (e.g. settings)
;116 //                 to avoid excessive wear on the flash.  backupd now copie
;117 //                 updated runtime values to the cache frequently, and flus
;118 //                 any newer cache values to flash much less frequently.  T
;119 //                 previous version had the barest beginning of this and, i
;120 //                 its barely started state, was writing all the changes it
;121 //                 saw to flash, causing lots of wear and slowing down
;122 //                 response times to fast series of updates.
;123 //
;124 //  2010-03-23 - Mark Davis
;125 //               - The version date/time for the OS and the application are
;126 //                 now written to the log file each time it is (re)opened.
;127 //               - Added size info to data logged for remote/local paramete
;128 //                 files
;129 //
;130 //  2010-03-12 - Mark Davis
;131 //               - getTime() now checks the state of the network interface
;132 //                 before bothering to try to get a connection to the time
;133 //                 server.  This eliminates pointless and frequent messages
;134 //                 in the log.
;135 //
;136 //  2010-02-24 - Mark Davis
;137 //               - When renaming a remote backup copy of the devData file,
;138 //                 the last modification date and time of the file are used
;139 //                 as a suffix (rather than simply using .bak all the time)
;140 //
;141 //  2009-12-18 - Mark Davis
;142 //               - Increased max # of params readParams() can handle to 128
;143 //
;144 //  2009-08-24 - Mark Davis
;145 //               - Parameter files for settings (PDC_SET) files are created
;146 //                 if they don't already exist.
;147 //               - readParams() now returns a positve number not just if on
;148 //                 of the saved values != an in-RAM value, but if any of th
;149 //                 in-RAM values are not saved in the file.  This insures t
;150 //                 changes to settings are actually saved.
;151 //
;152 //  2009-08-06 - Mark Davis
;153 //               - updateParamFiles() no longer tries to continue if the ca
;154 //                 to loadParams() returns an error (which happens if there
;155 //                 is no devData file).
;156 //
;157 //  2009-06-21 - Mark Davis
;158 //               - The time the log file is opened by a module is now inclu
;159 //                 in the line written directly to the file at the time.
;160 //
;161 //  2009-06-19 - Mark Davis
;162 //               - The Core Module ID (last 3 bytes of MAC addr) is now wri
;163 //                 to the remote copy of the log file each time the module
;164 //                 reopens the connection to the remote file.
;165 //
;166 //  2009-06-11 - Mark Davis
;167 //               - Added month to the name used for the remote copy of the
;168 //                 system log so we will always have a least a year's worth
;169 //                 the log files.
;170 //
;171 //  2009-05-07 - Mark Davis
;172 //               - Corrected value used compute ZWorld RTC value from the U
;173 //                 value returned by the time server.
;174 //
;175 //  2009-04-13 - Mark Davis
;176 //               - Application now waits 5 seconds from time a SIGTERM sign
;177 //                 is rcvd before shutting down (to allow entries that were
;178 //                 likely added to the system log to be saved to the server
;179 //
;180 //  2009-03-23 - Mark Davis
;181 //               - 10 attempts (one per sec) are now made (if necessary) to
;182 //                 get the date/time for a new remote backup copy of a file
;183 //                 to be used for setting the local copy's date/time.
;184 //               - If we fail to get the date/time for the new remote file,
;185 //                 the date/time of the local file is not changed (before i
;186 //                 was changed to whatever value happened to be in the
;187 //                 struct used to return the remote's date/time value).  Th
;188 //                 could account for the invalid date/times seen in message
;189 //                 in the system log.
;190 //
;191 //  2009-03-13 - Mark Davis
;192 //               - saveParamsToFile() now checks the return value of unlink
;193 //                 and retries for up to 5 seconds if needed to successfull
;194 //                 delete the file.  If it cannot delete the file it does n
;195 //                 try to overwrite it.  This eliminates the most likely ca
;196 //                 of problems caused by concurrent access to parameter fil
;197 //                 like devdata.  Prior to this it was not uncommon to end 
;198 //                 with a corrupted devData file from doing multiple saves,
;199 //                 causing it to be open by both backupd and the save cmd.
;200 //
;201 //  2008-04-07 - Mark Davis
;202 //               - a size of 0 for a parameter is now considered invalid an
;203 //                 a sign of a corrupt param file
;204 //
;205 //  2008-02-28 - Mark Davis
;206 //               - age of log files now correct (the # of hours was off by 
;207 //                 factor of 10)
;208 //
;209 //  2008-02-20 - Mark Davis
;210 //               - Added 5 sec delay after file I/O errors
;211 //
;212 //  2007-12-12 - Mark Davis
;213 //               - Added logic to automatically save changes to PERSISTENT
;214 //                 (ParamData/ParamFile) values (up to 5 times in a 15-minu
;215 //                 window, and no more often then every 30 seconds)
;216 //
;217 //  2007-12-11 - Mark Davis
;218 //               - Fixed interval for checking on time when it fails so it
;219 //                 doesn't do it continuously
;220 //
;221 //  2007-10-02 - Mark Davis
;222 //               - ParamData struct now includes a pointer to a func to cal
;223 //                 whenever the value is restored from a file
;224 //
;225 //  2007-08-01 - Mark Davis
;226 //               - update of remote backup copy of a file now requires that
;227 //                 the local copy was successfully opened
;228 //
;229 //  2007-07-13 - Mark Davis
;230 //               - added code to automatically maintain remote files with a
;231 //                 copy of system log entries
;232 //
;233 //  2006-07-19 - Mark Davis
;234 //               - original version started
;235 //
;236 //-------------------------------------------------------------------------
;237 
;238 
;239 #include <common.h>
;240 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\time.h' 15 48 18 10 12 22
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\string.h' 26 27 14 2 6 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stddef.h' 21 20 12 5 6 22
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\math.h' 19 23 18 21 9 21
	.debug	I
	.debug	I
;241 #include <sys/types.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
;242 #include <unistd.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\sched.h' 3 49 6 8 6 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\signal.h' 6 21 8 17 3 29
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\list.h' 21 49 15 26 7 30
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\dummy.h' 25 58 21 12 10 27
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\list.h' 21 49 15 26 7 30
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\common.h' 6 8 9 6 12 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Rabbit.h' 24 40 11 19 5 30
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stdio.h' 22 16 12 9 4 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\SoftoolsIDblock.h' 18 52 15 4 6 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\FlashBlock.h' 5 10 16 30 11 24
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Flash.h' 0 55 21 17 7 26
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Stdlib.h' 2 30 16 6 7 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\dcdefs.h' 18 42 23 19 7 26
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Rabbit.h' 24 40 11 19 5 30
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\SoftoolsIDBlock.h' 18 52 15 4 6 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\errors.h' 25 8 15 18 11 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stdlib.h' 2 30 16 6 7 24
	.debug	I
	.debug	# 109
_LIN2SEG	.macro

	.debug	# 110
add hl,hl

	.debug	# 111
rla

	.debug	# 112
add hl,hl

	.debug	# 113
rla

	.debug	# 114
add hl,hl

	.debug	# 115
rla

	.debug	# 116
add hl,hl

	.debug	# 117
rla

	.debug	# 118
sub 0x0E

	.debug	# 119
or a

	.debug	# 120
rr hl

	.debug	# 121
scf

	.debug	# 122
rr hl

	.debug	# 123
scf

	.debug	# 124
rr hl

	.debug	# 125
scf

	.debug	# 126
rr hl

	.debug	# 127
.endm

	.cseg
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\hdwdefs.h' 20 2 12 27 10 31
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\sched.h' 3 49 6 8 6 32
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\fcntl.h' 29 48 12 5 12 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\ioctl.h' 2 15 10 21 5 30
	.debug	I
	.debug	I
	.debug	I
;243 #include <sys/socket.h>
;244 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\socket.h' 2 54 9 7 6 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\netinet\in.h' 15 56 20 12 3 27
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip.h' 10 19 7 30 6 29
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\dcdefs.h' 18 42 23 19 7 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\dcrtcp.h' 1 9 9 21 2 32
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\tcp_config.h' 9 28 14 7 1 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\neterrno.h' 26 24 16 30 7 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\errors.h' 25 8 15 18 11 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\servlist.h' 7 29 16 30 7 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\net.h' 21 21 9 7 3 32
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ppp.h' 4 42 22 17 6 26
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ppplink.h' 18 1 2 19 9 26
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\chat.h' 12 7 15 6 10 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\Rabbit.h' 24 40 11 19 5 30
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\net.h' 21 21 9 7 3 32
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\pktdrv.h' 16 19 17 14 3 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\arp.h' 21 56 14 30 7 24
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ip.h' 16 35 12 25 4 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ip.h' 16 35 12 25 4 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\udp.h' 23 23 20 4 3 30
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\icmp.h' 2 44 0 17 5 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\tcp.h' 14 5 18 19 4 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\bootp.h' 2 16 3 7 2 26
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\bsdname.h' 9 41 20 10 2 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\icmp.h' 2 44 0 17 5 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\dns.h' 18 15 17 7 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\servlist.h' 7 29 16 30 7 24
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\igmp.h' 10 41 20 10 2 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\smtp.h' 0 54 12 2 8 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\http.h' 28 52 16 7 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\zserver.h' 26 57 12 10 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\limits.h' 1 13 9 26 5 19
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\pool.h' 10 58 16 7 1 25
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\errno.h' 16 4 11 6 2 29
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\mib.h' 28 3 18 15 4 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\snmp.h' 22 42 15 4 11 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ftp_client.h' 9 41 20 10 2 24
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ftp_server.h' 5 50 18 7 1 25
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\zserver.h' 26 57 12 10 1 25
	.debug	I
	.debug	I
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\stcpip\ethernet.h' 21 56 23 21 3 26
	.debug	I
	.debug	I
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\limits.h' 25 24 9 6 6 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\linux\wait.h' 10 13 9 20 4 32
	.debug	I
	.debug	I
;245 #include <fcntl.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\fcntl.h' 29 48 12 5 12 32
	.debug	I
;246 #include <arpa/inet.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\arpa\inet.h' 14 56 20 12 3 27
	.debug	I
;247 #include <errno.h>
;248 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\errno.h' 22 0 10 11 8 30
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\unistd.h' 5 0 10 5 12 32
	.debug	I
	.debug	I
;249 #include <string.h>
;250 
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\string.h' 26 27 14 2 6 25
	.debug	I
;251 #include <sys/backup.h>
;252 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\backup.h' 9 9 8 10 9 32
	.debug	I
;253 #include <sys/modbus.h>
;254 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\modbus.h' 21 56 10 7 12 32
	.debug	I
;255 #include <time.h>
;256 
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\time.h' 15 48 18 10 12 22
	.debug	I
;257 #include <sys/srfs.h>
;258 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\srfs.h' 6 17 10 7 4 28
	.debug	I
;259 #include <services.h>
;260 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\services.h' 2 22 6 20 2 29
	.debug	I
;261 #include <sys/farprintf.h>
;262 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\farprintf.h' 5 43 7 29 11 27
	.debug	I
;263 #include <sys/stat.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\stat.h' 0 48 10 31 3 29
	.debug	I 'C:\Program Files (x86)\Softools WinIDE\Rabbit 1.70\Include\time.h' 15 48 18 10 12 22
	.debug	I
	.debug	I
;264 #include <utime.h>
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\utime.h' 29 54 20 12 3 27
	.debug	I
;265 #include <sys/time.h>
;266 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\time.h' 10 37 11 8 3 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I
;267 #include <applic.h>
;268 
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\applic.h' 28 10 10 5 12 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\busybox.h' 23 57 14 6 3 32
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\types.h' 26 55 13 6 4 32
	.debug	I
	.debug	I 'C:\work\dev\Rabbit_ST\rtos\include\sys\backup.h' 9 9 8 10 9 32
	.debug	I
	.debug	I
	.debug	I
;269 
;270 
;271 EVENT_DEF(RestoredParamFile);
;272 
;273 static ulong timeChkInterval, lastTimeChk, lastDateChk, lastChgChk;
;274 
;275 static fd_t  logfd;
;276 static uint  logDay, logMonth;
;277 
;278 static char  logFileName[PATH_MAX];
;279 
;280 
;281 #define  HdrSize  (sizeof(BBM_Cache_Hdr))
;282 
;283 
;284 //-------------------------------------------------------------------------
;285 //  query the SRFS server for the time of day
;286 //-------------------------------------------------------------------------
;287 void getTime(void)
;288 {
	.defseg	FARCODE
	.seg	FARCODE
$getTime::
	.debug	G $getTime 32 9
	.debug	{ $getTime 32 287 @3 @2
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-75
;289   int  sockfd;
	.debug	L sockfd -2 6
;290   ulong  diff;
	.debug	L diff -6 9
;291   time_t  curSecs, newSecs;
	.debug	L curSecs -10 9
	.debug	L newSecs -14 9
;292   uchar  buf[4];
	.debug	L buf -18 33
;293   struct tm  tms;
	.debug	L tms -36 34
;294   struct sockaddr_in  srvAddr;
	.debug	L srvAddr -50 35
;295   IFConfig  curConfig;
	.debug	L curConfig -75 36
;296 
;297 
;298   if (msSince(lastTimeChk) < timeChkInterval)  return;
	.debug	# 298
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	push	bc
	push	de
	ld	de,(_lastTimeChk)
	ld	bc,(_lastTimeChk+2)
	call	__lsub##
	push	bc
	push	de
	ld	de,(_timeChkInterval)
	ld	bc,(_timeChkInterval+2)
	call	__lsub##
	jp	c,@2
;299 
;300   lastTimeChk = GetMS();
	.debug	# 300
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	ld	(_lastTimeChk),de
	ld	(_lastTimeChk+2),bc
;301   timeChkInterval = 5000;  // used if we exit this func due to an error
	.debug	# 301
	ld	de,0x1388
	ld	bc,0
	ld	(_timeChkInterval),de
	ld	(_timeChkInterval+2),bc
;302 
;303   getIFConfig(IF_ETH0, &curConfig);
	.debug	# 303
	ld	hl,-75+@1
	add	hl,sp
	push	hl
	bool	hl
	ld	l,h
	push	hl
	lcall	$getIFConfig
	add	sp,4
;304   if (curConfig.state != IF_UP)  return;
	.debug	# 304
	ld	a,(ix-51)
	sub	a,2
	jp	nz,@2
;305 
;306   errno = 0;
	.debug	# 306
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,d
	ld	(hl),e
	inc	hl
	ld	(hl),d
;307 
;308   srvAddr.sin_family = AF_INET;
	.debug	# 308
	ld	hl,0x2
	ld	(ix-50),hl
;309   srvAddr.sin_port   = htons(TIME_PORT);
	.debug	# 309
	ld	hl,0x25
	ld	a,l
	ld	l,h
	ld	h,a
	ld	(ix-48),hl
;310   srvAddr.sin_addr   = htonl(srfsIPAddr);
	.debug	# 310
	ld	de,(_srfsIPAddr)
	ld	bc,(_srfsIPAddr+2)
	ex	de,hl
	ld	e,b
	ld	d,c
	ld	b,l
	ld	c,h
	ld	hl,bc
	ld	(ix-44),hl
	ex	de,hl
	ld	(ix-46),hl
	ex	de,hl
;311 
;312   if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)  {
	.debug	# 312
	bool	hl
	ld	l,h
	push	hl
	ld	l,0x2
	push	hl
	push	hl
	lcall	$socket
	add	sp,6
	ld	(ix-2),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@6
;313     perror("getTime: socket()");  return;  }
@7:
	.debug	< @7 @8
	.debug	# 313
	ld	de,word (phys @9)
	ld	c,phys @9 >> 16
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	jp	@2
@8:
	.debug	>
;314 
;315   if (connect(sockfd, (struct sockaddr *) &srvAddr, sizeof(srvAddr)) < 0)  
@6:
	.debug	# 315
	ld	hl,0xe
	push	hl
	ld	hl,-48+@1
	add	hl,sp
	push	hl
	ld	hl,(ix-2)
	push	hl
	lcall	$connect
	add	sp,6
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@10
;316     perror("getTime: connect()");  close(sockfd);  return;  }
@11:
	.debug	< @11 @12
	.debug	# 316
	ld	de,word (phys @9+18)
	ld	c,phys @9+18 >> 16
	jr	@579
@12:
	.debug	>
;317 
;318   if (read(sockfd, buf, 4) != 4)  {
@10:
	.debug	# 318
	ld	hl,0x4
	push	hl
	ld	hl,-16+@1
	add	hl,sp
	push	hl
	ld	hl,(ix-2)
	push	hl
	lcall	$read
	add	sp,6
	ld	de,0x4
	or	a,a
	sbc	hl,de
	jr	z,@13
;319     perror("getTime: read()");  close(sockfd);  return; }
@14:
	.debug	< @14 @15
	.debug	# 319
	ld	de,word (phys @9+37)
	ld	c,phys @9+37 >> 16
@579:
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	hl,(ix-2)
	push	hl
	lcall	$close
	pop	de
	jp	@2
@15:
	.debug	>
;320 
;321   close(sockfd);
@13:
	.debug	# 321
	ld	hl,(ix-2)
	push	hl
	lcall	$close
	pop	de
;322 
;323   time(&curSecs);
	.debug	# 323
	ld	hl,-10+@1
	add	hl,sp
	push	hl
	lcall	$time
	pop	hl
;324   newSecs = ntohl(*((ulong *)buf)) - 2208988800L;
	.debug	# 324
	ld	hl,-18+@1
	add	hl,sp
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ex	de,hl
	ld	e,b
	ld	d,c
	ld	b,l
	ld	c,h
	push	bc
	push	de
	ld	de,0x8180
	ld	bc,0x7c55
	call	__ladd##
	ld	hl,bc
	ld	(ix-12),hl
	ex	de,hl
	ld	(ix-14),hl
	ex	de,hl
;325 
;326   diff = ULDIF(curSecs, newSecs);
	.debug	# 326
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-14)
	ex	de,hl
	ld	hl,(ix-12)
	ld	bc,hl
	call	__lsub##
	jr	c,@16
	jr	z,@16
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-14)
	ex	de,hl
	ld	hl,(ix-12)
	ld	bc,hl
	call	__lsub##
	jr	@17
@16:
	ld	hl,(ix-14)
	ex	de,hl
	ld	hl,(ix-12)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	call	__lsub##
@17:
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
;327 
;328   if ((diff > 5) OR !gotTime)  {
	.debug	# 328
	push	bc
	push	de
	ld	de,0x5
	ld	c,d
	ld	b,d
	call	__lsub##
	jr	z,@20
	jr	nc,@18
@20:
	ld	a,(_gotTime)
	or	a,a
	jr	nz,@19
@18:
;329     gmtime_r(&newSecs, &tms);
@21:
	.debug	< @21 @22
	.debug	# 329
	ld	hl,-36+@1
	add	hl,sp
	push	hl
	ld	hl,-12+@1
	add	hl,sp
	push	hl
	lcall	$gmtime_r
	add	sp,4
;330     printf("UTC time: %04d-%02d-%02d %02d:%02d:%02d (%lu)\r\n",
;331             tms.tm_year+1900, tms.tm_mon+1, tms.tm_mday,
;332             tms.tm_hour, tms.tm_min, tms.tm_sec, newSecs );
	.debug	# 332
	ld	hl,(ix-14)
	ex	de,hl
	ld	hl,(ix-12)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-36)
	push	hl
	ld	hl,(ix-34)
	push	hl
	ld	hl,(ix-32)
	push	hl
	ld	hl,(ix-30)
	push	hl
	ld	hl,(ix-28)
	inc	hl
	push	hl
	ld	hl,(ix-26)
	ld	de,0x76c
	add	hl,de
	push	hl
	ld	de,word (phys @9+53)
	ld	c,phys @9+53 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,20
;333     gotTime = true;
	.debug	# 333
	ld	a,0x1
	ld	(_gotTime),a
;334   }
@22:
	.debug	>
;335 
;336   stime(&newSecs);
@19:
	.debug	# 336
	ld	hl,-14+@1
	add	hl,sp
	push	hl
	lcall	$stime
	pop	de
;337 
;338 //--- use a longer wait time between successful queries ---
;339   timeChkInterval = 300000L + ((ntohl(srvAddr.sin_addr) & 0x000000ff) << 4)
	.debug	# 339
	ld	hl,(ix-46)
	ex	de,hl
	ld	hl,(ix-44)
	ld	bc,hl
	ex	de,hl
	ld	e,b
	ld	d,c
	ld	b,l
	ld	c,h
	push	bc
	push	de
	ld	de,0xff
	ld	c,d
	ld	b,d
	call	__land##
	ld	a,0x4
	call	__lshl##
	push	bc
	push	de
	ld	de,0x93e0
	ld	bc,0x4
	call	__ladd##
	ld	(_timeChkInterval),de
	ld	(_timeChkInterval+2),bc
;340 }
@2:
	.debug	} @2 340

@1	equ	75
	ld	sp,ix
	pop	ix
	lret
@3	equ	77
;341 
;342 
;343 //-------------------------------------------------------------------------
;344 //  return the file name to use for the specified paramFile entity
;345 //-------------------------------------------------------------------------
;346 char *paramFileName(ParamFile *paramFile, char *fileName, uint maxLen)
;347 {
$paramFileName::
	.debug	G $paramFileName 37 9
	.debug	{ $paramFileName 37 346 @25 @24
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A paramFile 5 39
	.debug	A fileName 7 38
	.debug	A maxLen 9 7
;348   *fileName = 0;
	.debug	# 348
	xor	a,a
	ld	hl,(ix+7)
	ld	(hl),a
;349 
;350 
;351 //--- add prefix for device-specific files ---
;352   if ( (paramFile->category == PDC_DEV) OR (paramFile->category == PDC_SET)
	.debug	# 352
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	ld	a,(hl)
	sub	a,3
	jr	z,@26
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	ld	a,(hl)
	sub	a,4
	jr	nz,@27
@26:
;353     sprintf(fileName, "%.12s_x%02lX_", ApplAbbrev, deviceID);
	.debug	# 353
	ld	de,(_deviceID)
	ld	bc,(_deviceID+2)
	push	bc
	push	de
	ld	hl,_ApplAbbrev
	push	hl
	ld	de,word (phys @9+101)
	ld	c,phys @9+101 >> 16
	push	bc
	push	de
	ld	hl,(ix+7)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,14
;354 
;355   fstrncat(fileName, paramFile->baseName, maxLen-strlen(fileName)-1);
@27:
	.debug	# 355
	ld	hl,(ix+7)
	push	hl
	call	_strlen
	pop	de
	ex	de,hl
	ld	hl,(ix+9)
	or	a,a
	sbc	hl,de
	dec	hl
	push	hl
	ld	hl,(ix+5)
	ld	hl,(hl)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	ld	hl,(ix+7)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$fstrncat
	add	sp,10
;356 
;357   return fileName;
	.debug	# 357
	ld	hl,(ix+7)
;358 }
@24:
	.debug	} @24 358

@23	equ	0
	pop	ix
	lret
@25	equ	2
;359 
;360 
;361 //-------------------------------------------------------------------------
;362 //  Return the # of bytes needed to store a copy of the values in the
;363 //  specified list of parameters.
;364 //-------------------------------------------------------------------------
;365 uint paramStreamSize(ParamFile *paramFile)
;366 {
$paramStreamSize::
	.debug	G $paramStreamSize 41 9
	.debug	{ $paramStreamSize 41 365 @30 @29
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-6
	.debug	A paramFile 5 39
;367   uint  count = 0;
	.debug	L count -2 7
	.debug	# 367
	bool	hl
	ld	l,h
	ld	(ix-2),hl
	.seg	FARCODE
;368   const ParamData far *param;
	.debug	L param -6 42
;369 
;370   for (param=paramFile->params; param->id; ++param)
@31:
	.debug	< @31 @32
	.debug	# 370
	ld	hl,(ix+5)
	inc	hl
	inc	hl
	inc	hl
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
@33:
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	call	__farld2##
	ld	a,e
	or	a,d
	jr	z,@34
;371     count += param->size + 4;  // add 4 bytes for 2-byte ID & 2-byte size
	.debug	# 371
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	hl,0x4
	add	hl,de
	ex	de,hl
	ld	hl,(ix-2)
	ex	de,hl
	add	hl,de
	ld	(ix-2),hl
;372 
;373   return count;
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x10
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
	jr	@33
@34:
@32:
	.debug	>
	.debug	# 373
	ld	hl,(ix-2)
;374 }
@29:
	.debug	} @29 374

@28	equ	6
	ld	sp,ix
	pop	ix
	lret
@30	equ	8
;375 
;376 
;377 //-------------------------------------------------------------------------
;378 //  close the fd for the parameter file (if one is open)
;379 //-------------------------------------------------------------------------
;380 void closeParamFile(ParamFile *paramFile)
;381 {
$closeParamFile::
	.debug	G $closeParamFile 44 9
	.debug	{ $closeParamFile 44 380 @38 @37
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A paramFile 5 39
;382   if (paramFile->fd < 0)  return;
	.debug	# 382
	ld	hl,(ix+5)
	ld	de,0x27
	add	hl,de
	ld	hl,(hl)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@37
;383 
;384   close(paramFile->fd);  paramFile->fd = -1;
	.debug	# 384
	ld	hl,(ix+5)
	ld	de,0x27
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$close
	pop	de
	ld	hl,(ix+5)
	ld	de,0x27
	add	hl,de
	ld	de,0xffff
	ld	(hl),e
	inc	hl
	ld	(hl),d
;385 }
@37:
	.debug	} @37 385

@36	equ	0
	pop	ix
	lret
@38	equ	2
;386 
;387 //-------------------------------------------------------------------------
;388 //  Allocate BBRAM for the parameter files that may be written to often so 
;389 //  can use it as a cache to minimize the # of writes to flash (yet still
;390 //  have the most recent values if we reboot before the cache is flushed to
;391 //  the flash file).
;392 //
;393 //  Once we have allocated a block, check to see if it contains data from
;394 //  before the reboot (and if it is newer than the flash file)
;395 //-------------------------------------------------------------------------
;396 int initParamModule(char *buf, uint bufSize)
;397 {
$initParamModule::
	.debug	G $initParamModule 45 9
	.debug	{ $initParamModule 45 396 @42 @41
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-83
	.debug	A buf 5 38
	.debug	A bufSize 7 7
;398   bool  useCache;
	.debug	L useCache -1 3
;399   uint  u, ttlSize, dataSize;
	.debug	L u -3 7
	.debug	L ttlSize -5 7
	.debug	L dataSize -7 7
;400   char far *cache;
	.debug	L cache -11 46
;401   BBM_Cache_Hdr far *hdr;
	.debug	L hdr -15 47
;402   ParamFile *paramFile;
	.debug	L paramFile -17 39
;403   struct stat  fileInfo;
	.debug	L fileInfo -59 49
;404   char fileName[NAME_MAX];
	.debug	L fileName -83 50
;405 
;406 
;407 //=== For each file of category PDC_SET (i.e. runtime settings), allocate =
;408 //=== the space for a cache in BBRAM and see if it contains valid data ===
;409 //=== from the previous run ===
;410 
;411   for (u=0; u<NumParamFiles; ++u)  {
@43:
	.debug	< @43 @44
	.debug	# 411
	bool	hl
	ld	l,h
	ld	(ix-3),hl
@45:
	ld	hl,(ix-3)
	ld	de,(_NumParamFiles)
	or	a,a
	sbc	hl,de
	jp	nc,@46
;412     paramFile = paramFiles[u];
@48:
	.debug	< @48 @49
	.debug	# 412
	ld	hl,(ix-3)
	add	hl,hl
	ld	de,_paramFiles
	add	hl,de
	ld	hl,(hl)
	ld	(ix-17),hl
;413 
;414     BZERO(paramFile->writeCount);
	.debug	# 414
	ld	hl,0x20
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-17)
	ld	de,0x7
	add	hl,de
	push	hl
	call	_memset
	add	sp,6
;415     paramFile->fd = -1;
	.debug	# 415
	ld	hl,(ix-17)
	ld	de,0x27
	add	hl,de
	ld	de,0xffff
	ld	(hl),e
	inc	hl
	ld	(hl),d
;416     paramFile->bbmCache = NULL;
	.debug	# 416
	ld	hl,(ix-17)
	ld	de,0x29
	add	hl,de
	ld	e,d
	ld	c,e
	ld	b,e
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;417 
;418     if (paramFile->category != PDC_SET) continue;
	.debug	# 418
	ld	hl,(ix-17)
	inc	hl
	inc	hl
	ld	a,(hl)
	sub	a,4
	jp	nz,@47
;419 
;420     if (paramStreamSize(paramFile) < 1)  continue;
	.debug	# 420
	ld	hl,(ix-17)
	push	hl
	lcall	$paramStreamSize
	pop	de
	ld	de,0x1
	or	a,a
	sbc	hl,de
	jp	c,@47
;421 
;422   //--- allocate a cache in BBRAM (if file contains run-time settings) ---
;423     dataSize = paramStreamSize(paramFile);  ttlSize = dataSize + HdrSize;
	.debug	# 423
	ld	hl,(ix-17)
	push	hl
	lcall	$paramStreamSize
	pop	de
	ld	(ix-7),hl
	ld	de,0xa
	add	hl,de
	ld	(ix-5),hl
;424     fprintf(DiagLog, "For '%.32s' file cache:\r\n", paramFile->baseName);
	.debug	# 424
	ld	hl,(ix-17)
	ld	hl,(hl)
	push	hl
	ld	de,word (phys @9+115)
	ld	c,phys @9+115 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,8
;425     hdr = (BBM_Cache_Hdr far *) bb_xalloc(ttlSize);
	.debug	# 425
	ld	hl,(ix-5)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	lcall	$bb_xalloc
	add	sp,4
	ld	hl,bc
	ld	(ix-13),hl
	ex	de,hl
	ld	(ix-15),hl
	ex	de,hl
;426     if (!hdr)  {
	.debug	# 426
	ld	a,e
	or	a,d
	or	a,c
	jr	nz,@52
;427       printf("Unable to allocate BBRAM cache for %.32s file\r\n",
@53:
	.debug	< @53 @54
;428              paramFile->baseName);
	.debug	# 428
	ld	hl,(ix-17)
	ld	hl,(hl)
	push	hl
	ld	de,word (phys @9+141)
	ld	c,phys @9+141 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;429       continue;
	.debug	# 429
;430     }
	jp	@47
@54:
	.debug	>
;431     paramFile->cacheSize = dataSize;  paramFile->bbmCache = hdr;
@52:
	.debug	# 431
	ld	hl,(ix-17)
	ld	de,0x2d
	add	hl,de
	ex	de,hl
	ld	hl,(ix-7)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,(ix-17)
	ld	de,0x29
	add	hl,de
	ex	de,hl
	ld	hl,(ix-13)
	ld	bc,hl
	ld	hl,(ix-15)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;432     cache = (char far *)hdr + HdrSize;
	.debug	# 432
	ld	h,b
	ld	l,c
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xa
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-9),hl
	ex	de,hl
	ld	(ix-11),hl
	ex	de,hl
;433 
;434   //--- check to see if there is data from last time and if it is newer ---
;435   //--- than the data in the flash file ---
;436     useCache = true;
	.debug	# 436
	ld	(ix-1),0x1
;437     if (hdr->devType != devType)  useCache = false;
	.debug	# 437
	ld	hl,(ix-15)
	ex	de,hl
	ld	hl,(ix-13)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x6
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	hl,(_devType)
	or	a,a
	sbc	hl,de
	jr	z,@55
	xor	a,a
	ld	(ix-1),a
;438     else if (hdr->size != dataSize)  useCache = false;
	jp	@56
@55:
	.debug	# 438
	ld	hl,(ix-15)
	ex	de,hl
	ld	hl,(ix-13)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	hl,(ix-7)
	or	a,a
	sbc	hl,de
	jr	z,@57
	xor	a,a
	ld	(ix-1),a
;439     else if (hdr->crc != memCRC16((ulong)cache, dataSize))  useCache = fals
	jr	@58
@57:
	.debug	# 439
	ld	hl,(ix-15)
	ex	de,hl
	ld	hl,(ix-13)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	push	de
	ld	hl,(ix-7)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-11)
	ex	de,hl
	ld	hl,(ix-9)
	ld	bc,hl
	ld	b,0
	push	bc
	push	de
	lcall	$memCRC16
	add	sp,8
	pop	de
	or	a,a
	sbc	hl,de
	jr	z,@59
	xor	a,a
	ld	(ix-1),a
;440     else {
	jr	@60
@59:
;441       paramFileName(paramFile, fileName, sizeof(fileName));
@61:
	.debug	< @61 @62
	.debug	# 441
	ld	hl,0x18
	push	hl
	ld	hl,-81+@40
	add	hl,sp
	push	hl
	ld	hl,(ix-17)
	push	hl
	lcall	$paramFileName
	add	sp,6
;442       stat(fileName, &fileInfo);
	.debug	# 442
	ld	hl,-59+@40
	add	hl,sp
	push	hl
	ld	hl,-81+@40
	add	hl,sp
	push	hl
	lcall	$stat
	add	sp,4
;443       if (fileInfo.st_mtime > hdr->time)  useCache = false;
	.debug	# 443
	ld	hl,(ix-15)
	ex	de,hl
	ld	hl,(ix-13)
	ld	bc,hl
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix-25)
	ex	de,hl
	ld	hl,(ix-23)
	ld	bc,hl
	call	__lsubr##
	jr	c,@63
	jr	z,@63
	xor	a,a
	ld	(ix-1),a
;444     }
@63:
@62:
	.debug	>
;445 
;446     if (!useCache)  {
@60:
@58:
@56:
	.debug	# 446
	ld	a,(ix-1)
	or	a,a
	jr	nz,@64
;447       fmemset(hdr, 0, ttlSize);
@65:
	.debug	< @65 @66
	.debug	# 447
	ld	hl,(ix-5)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-15)
	ex	de,hl
	ld	hl,(ix-13)
	ld	bc,hl
	push	bc
	push	de
	call	_fmemset
	add	sp,10
;448       hdr->devType = devType;  hdr->size = dataSize;
	.debug	# 448
	ld	hl,(ix-15)
	ex	de,hl
	ld	hl,(ix-13)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x6
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,(_devType)
	call	__farst2##
	ld	hl,(ix-15)
	ex	de,hl
	ld	hl,(ix-13)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,(ix-7)
	call	__farst2##
;449     }
@66:
	.debug	>
;450 
;451     printf("%s %u byte BBRAM cache for '%s' at: 0x%05lX\r\n",
@64:
;452            (useCache ? "Found" : "Created"), dataSize,
;453            paramFile->baseName, (ulong)cache );
	.debug	# 453
	ld	hl,(ix-11)
	ex	de,hl
	ld	hl,(ix-9)
	ld	bc,hl
	ld	b,0
	push	bc
	push	de
	ld	hl,(ix-17)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix-7)
	push	hl
	ld	a,(ix-1)
	or	a,a
	jr	z,@67
	ld	hl,@68
	jr	@69
@67:
	ld	hl,@68+6
@69:
	push	hl
	ld	de,word (phys @9+189)
	ld	c,phys @9+189 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,14
;454   }
@49:
	.debug	>
;455 
;456 
;457 //--- load all the values from the parameter files (or their cache) in to -
;458 //--- the runtime values ---
;459   loadAllParams(buf, bufSize);
@47:
	ld	hl,(ix-3)
	inc	hl
	ld	(ix-3),hl
	jp	@45
@46:
@44:
	.debug	>
	.debug	# 459
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$loadAllParams
	add	sp,4
;460 
;461   return 0;
	.debug	# 461
	bool	hl
	ld	l,h
;462 }
@41:
	.debug	} @41 462

@40	equ	83
	ld	sp,ix
	pop	ix
	lret
@42	equ	85
;463 
;464 //-------------------------------------------------------------------------
;465 //  Call to prepare a paramaterized list's flash file for writing.
;466 //
;467 //  If the file already exists and is the correct size, it is left alone.
;468 //
;469 //  If the file already exists but is not the correct size, it is deleted
;470 //  and a new empty one is created.
;471 //
;472 //  If the file doesn't exist, a new empty one is created.
;473 //
;474 //  Returns the fd the file was opened on, or < 0 if an error
;475 //-------------------------------------------------------------------------
;476 int initParamFile(ParamFile *paramFile)
;477 {
$initParamFile::
	.debug	G $initParamFile 51 9
	.debug	{ $initParamFile 51 476 @73 @72
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-74
	.debug	A paramFile 5 39
;478   int  fd, status;
	.debug	L fd -2 6
	.debug	L status -4 6
;479   ulong  startTime;
	.debug	L startTime -8 9
;480   struct stat  fileInfo;
	.debug	L fileInfo -50 49
;481   char  fileName[NAME_MAX];
	.debug	L fileName -74 50
;482 
;483 
;484   paramFileName(paramFile, fileName, sizeof(fileName));
	.debug	# 484
	ld	hl,0x18
	push	hl
	ld	hl,-72+@71
	add	hl,sp
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$paramFileName
	add	sp,6
;485 
;486   fd = paramFile->fd;
	.debug	# 486
	ld	hl,(ix+5)
	ld	de,0x27
	add	hl,de
	ld	hl,(hl)
	ld	(ix-2),hl
;487 
;488 //--- open the existing file (if any) ---
;489   if (fd < 0)  fd = open(fileName, O_WRONLY, 0);
	.debug	# 489
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@74
	bool	hl
	ld	l,h
	push	hl
	inc	l
	inc	l
	push	hl
	ld	hl,-70+@71
	add	hl,sp
	push	hl
	lcall	$open
	add	sp,6
	ld	(ix-2),hl
;490   if (fd < 0)  if (errno != ENOENT)  { perror(fileName);  return -1; }
@74:
	.debug	# 490
	ld	hl,(ix-2)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@75
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0x2
	or	a,a
	sbc	hl,de
	jp	nz,@578
@77:
	.debug	< @77 @78
@78:
	.debug	>
;491 
;492 //--- if the file exists and it is the wrong size, then delete it ---
;493   if (fd >= 0)  {
@75:
	.debug	# 493
	ld	hl,(ix-2)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jp	c,@79
;494     if (fstat(fd, &fileInfo))  {
@80:
	.debug	< @80 @81
	.debug	# 494
	ld	hl,-50+@71
	add	hl,sp
	push	hl
	ld	hl,(ix-2)
	push	hl
	lcall	$fstat
	add	sp,4
	bool	hl
	jr	z,@82
;495       perror(fileName);  close(fd);  paramFile->fd = -1;  return -1; }
@83:
	.debug	< @83 @84
	.debug	# 495
	ld	hl,-74+@71
	add	hl,sp
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	hl,(ix-2)
	push	hl
	lcall	$close
	pop	de
	ld	hl,(ix+5)
	ld	de,0x27
	add	hl,de
	ld	de,0xffff
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	h,d
	ld	l,e
	jp	@72
@84:
	.debug	>
;496     if (fileInfo.st_size != paramStreamSize(paramFile))  {
@82:
	.debug	# 496
	ld	hl,(ix+5)
	push	hl
	lcall	$paramStreamSize
	pop	de
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-28)
	ex	de,hl
	ld	hl,(ix-26)
	ld	bc,hl
	call	__lsub##
	jp	z,@85
;497       close(fd);  paramFile->fd = fd = -1;
@86:
	.debug	< @86 @87
	.debug	# 497
	ld	hl,(ix-2)
	push	hl
	lcall	$close
	pop	de
	ld	hl,0xffff
	ld	(ix-2),hl
	ex	de,hl
	ld	hl,(ix+5)
	ex	de,hl
	ld	bc,hl
	ld	hl,0x27
	add	hl,de
	ld	(hl),c
	inc	hl
	ld	(hl),b
;498       // wait for up to 5 seconds to delete an existing file
;499       for (startTime=GetMS(); msSince(startTime)<5000; )  {
@88:
	.debug	< @88 @89
	.debug	# 499
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	ld	hl,bc
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
@90:
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	push	bc
	push	de
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	call	__lsub##
	push	bc
	push	de
	ld	de,0x1388
	ld	bc,0
	call	__lsub##
	jr	nc,@91
;500         status = unlink(fileName);  if (!status)  break;
@92:
	.debug	< @92 @93
	.debug	# 500
	ld	hl,-74+@71
	add	hl,sp
	push	hl
	lcall	$unlink
	pop	de
	ld	(ix-4),hl
	bool	hl
	jr	z,@91
;501         if (errno == ENOENT)  { status = 0;  break; }
	.debug	# 501
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0x2
	or	a,a
	sbc	hl,de
	jr	nz,@95
@96:
	.debug	< @96 @97
	bool	hl
	ld	l,h
	ld	(ix-4),hl
	jr	@91
@97:
	.debug	>
;502         if (errno != EBUSY)  break;
@95:
	.debug	# 502
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0x10
	or	a,a
	sbc	hl,de
	jr	nz,@91
;503         sleepMS(50);
	.debug	# 503
	ld	e,0x32
	ld	c,d
	ld	b,d
	push	bc
	push	de
	lcall	$sleepMS
	add	sp,4
;504       }
@93:
	.debug	>
;505       if (status)  { perror(fileName);  return -1; }
	jp	@90
@91:
@89:
	.debug	>
	.debug	# 505
	ld	hl,(ix-4)
	bool	hl
	jr	nz,@578
@100:
	.debug	< @100 @101
@101:
	.debug	>
;506     }
@87:
	.debug	>
;507   }
@85:
@81:
	.debug	>
;508 
;509 //--- create a new empty file if necessary ---
;510   if (fd < 0)  fd = open(fileName, O_RDWR | O_CREAT, 0666);
@79:
	.debug	# 510
	ld	hl,(ix-2)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@102
	ld	hl,0x1b6
	push	hl
	ld	hl,0x6
	push	hl
	ld	hl,-70+@71
	add	hl,sp
	push	hl
	lcall	$open
	add	sp,6
	ld	(ix-2),hl
;511 
;512   if (fd < 0)  { perror(fileName);  return -1; }
@102:
	.debug	# 512
	ld	hl,(ix-2)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@103
@104:
	.debug	< @104 @105
@578:
	ld	hl,-74+@71
	add	hl,sp
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	hl,0xffff
	jr	@72
@105:
	.debug	>
;513 
;514   paramFile->fd = fd;  return fd;
@103:
	.debug	# 514
	ld	hl,(ix+5)
	ld	de,0x27
	add	hl,de
	ex	de,hl
	ld	hl,(ix-2)
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	h,d
	ld	l,e
;515 }
@72:
	.debug	} @72 515

@71	equ	74
	ld	sp,ix
	pop	ix
	lret
@73	equ	76
;516 
;517 
;518 //-------------------------------------------------------------------------
;519 //  Call in preperation for saving a copy of the run-time values in a list
;520 //  to either its BBRAM cache or its flash file.
;521 //
;522 //  Returns 0 on success, < 0 if an error
;523 //-------------------------------------------------------------------------
;524 int prepForWrites(ParamFile *paramFile)
;525 {
$prepForWrites::
	.debug	G $prepForWrites 51 9
	.debug	{ $prepForWrites 51 524 @108 @107
	push	ix
	ld	ix,0
	add	ix,sp
	.debug	A paramFile 5 39
;526 //  if (paramFile->fd >= 0)  { errno = EBUSY;  return -1; }
;527   if (paramFile->fd >= 0)  closeParamFile(paramFile);
	.debug	# 527
	ld	hl,(ix+5)
	ld	de,0x27
	add	hl,de
	ld	hl,(hl)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@109
	ld	hl,(ix+5)
	push	hl
	lcall	$closeParamFile
	pop	hl
;528 
;529   paramFile->offset = 0;
@109:
	.debug	# 529
	ld	hl,(ix+5)
	ld	de,0x2f
	add	hl,de
	ld	e,d
	ld	(hl),e
	inc	hl
	ld	(hl),d
;530 
;531 //--- just use the cache if one exists for this file ---
;532   if (paramFile->bbmCache)  if (paramFile->bbmCache->time)  return 0;
	.debug	# 532
	ld	hl,(ix+5)
	ld	e,0x29
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	a,e
	or	a,d
	or	a,c
	jr	z,@110
	ld	hl,(ix+5)
	ld	de,0x29
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__farld4##
	ld	hl,bc
	or	hl,de
	jr	nz,@577
;533 
;534 //--- otherwise get the file ready for writing ---
;535   if (initParamFile(paramFile) >= 0)  return 0;
@110:
	.debug	# 535
	ld	hl,(ix+5)
	push	hl
	lcall	$initParamFile
	pop	de
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@112
@577:
	bool	hl
	ld	l,h
	jr	@107
;536 
;537   return -1;
@112:
	.debug	# 537
	ld	hl,0xffff
;538 }
@107:
	.debug	} @107 538

@106	equ	0
	pop	ix
	lret
@108	equ	2
;539 
;540 //-------------------------------------------------------------------------
;541 //  Call in preperation for reading a copy of the run-time values in a list
;542 //  from either its BBRAM cache or its flash file.
;543 //
;544 //  Returns 0 on sucess, < 0 if an error.
;545 //-------------------------------------------------------------------------
;546 int prepForReads(ParamFile *paramFile)
;547 {
$prepForReads::
	.debug	G $prepForReads 51 9
	.debug	{ $prepForReads 51 546 @115 @114
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-24
	.debug	A paramFile 5 39
;548   char  fileName[NAME_MAX];
	.debug	L fileName -24 50
;549 
;550 
;551 //  if (paramFile->fd >= 0)  { errno = EBUSY;  return -1; }
;552   if (paramFile->fd >= 0)  closeParamFile(paramFile);
	.debug	# 552
	ld	hl,(ix+5)
	ld	de,0x27
	add	hl,de
	ld	hl,(hl)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@116
	ld	hl,(ix+5)
	push	hl
	lcall	$closeParamFile
	pop	hl
;553 
;554   paramFile->offset = 0;
@116:
	.debug	# 554
	ld	hl,(ix+5)
	ld	de,0x2f
	add	hl,de
	ld	e,d
	ld	(hl),e
	inc	hl
	ld	(hl),d
;555 
;556   if (paramFile->bbmCache)  if (paramFile->bbmCache->time)  return 0;
	.debug	# 556
	ld	hl,(ix+5)
	ld	e,0x29
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	a,e
	or	a,d
	or	a,c
	jr	z,@117
	ld	hl,(ix+5)
	ld	de,0x29
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	call	__farld4##
	ld	hl,bc
	or	hl,de
	jr	z,@118
	bool	hl
	ld	l,h
	jr	@114
;557 
;558   paramFileName(paramFile, fileName, sizeof(fileName));
@118:
@117:
	.debug	# 558
	ld	hl,0x18
	push	hl
	ld	hl,-22+@113
	add	hl,sp
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$paramFileName
	add	sp,6
;559 
;560   if ((paramFile->fd = open(fileName, O_RDONLY, 0)) < 0)   return -1;
	.debug	# 560
	bool	hl
	ld	l,h
	push	hl
	inc	l
	push	hl
	ld	hl,-20+@113
	add	hl,sp
	push	hl
	lcall	$open
	add	sp,6
	ex	de,hl
	ld	hl,(ix+5)
	ex	de,hl
	ld	bc,hl
	ld	hl,0x27
	add	hl,de
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	hl,0x8000
	ex	de,hl
	ld	hl,bc
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@119
	ld	hl,0xffff
	jr	@114
;561 
;562   return 0;
@119:
	.debug	# 562
	bool	hl
	ld	l,h
;563 }
@114:
	.debug	} @114 563

@113	equ	24
	ld	sp,ix
	pop	ix
	lret
@115	equ	26
;564 
;565 
;566 //-------------------------------------------------------------------------
;567 //  Write a block of bytes to the cache or the file copy.
;568 //
;569 //  Returns the number of bytes written or < 0 if an error
;570 //-------------------------------------------------------------------------
;571 int writeParamBytes(ParamFile *paramFile, char *from, uint count)
;572 {
$writeParamBytes::
	.debug	G $writeParamBytes 52 9
	.debug	{ $writeParamBytes 52 571 @122 @121
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-14
	.debug	A paramFile 5 39
	.debug	A from 7 38
	.debug	A count 9 7
;573   char far  *cachePtr, *endCache;
	.debug	L cachePtr -4 46
	.debug	L endCache -8 46
;574   int  status;
	.debug	L status -10 6
;575   BBM_Cache_Hdr far  *hdr = paramFile->bbmCache;
	.debug	L hdr -14 47
	.debug	# 575
	ld	hl,(ix+5)
	ld	de,0x29
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-12),hl
	ex	de,hl
	ld	(ix-14),hl
	ex	de,hl
	.seg	FARCODE
;576 
;577 
;578   if (!from)  return -1;
	.debug	# 578
	ld	hl,(ix+7)
	bool	hl
	jp	z,@576
;579 
;580   if (hdr)  {
	.debug	# 580
	ld	hl,(ix-14)
	ex	de,hl
	ld	hl,(ix-12)
	ld	bc,hl
	ld	a,e
	or	a,d
	or	a,c
	jp	z,@124
;581     cachePtr = (uchar far *)hdr + HdrSize;
@125:
	.debug	< @125 @126
	.debug	# 581
	ld	h,b
	ld	l,c
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xa
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
;582     endCache = cachePtr + paramFile->cacheSize;
	.debug	# 582
	ld	hl,(ix+5)
	ld	de,0x2d
	add	hl,de
	ld	hl,(hl)
	ex	de,hl
	ld	b,h
	ld	c,h
	push	bc
	push	de
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	call	__ladd##
	ld	hl,bc
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
;583     cachePtr += paramFile->offset;
	.debug	# 583
	ld	hl,(ix+5)
	ld	de,0x2f
	add	hl,de
	ld	hl,(hl)
	ex	de,hl
	ld	b,h
	ld	c,h
	push	bc
	push	de
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	call	__ladd##
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
;584     if (cachePtr + count > endCache)  { ERRNO = ENOSPC;  return -1; }
	.debug	# 584
	ld	hl,(ix+9)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	call	__ladd##
	push	bc
	push	de
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	call	__fpcmp##
	jr	c,@127
	jr	z,@127
@128:
	.debug	< @128 @129
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x1c
	ld	(hl),e
	inc	hl
	ld	(hl),d
@576:
	ld	hl,0xffff
	jr	@121
@129:
	.debug	>
;585     fmemcpy(cachePtr, from, count);
@127:
	.debug	# 585
	ld	hl,(ix+9)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix+7)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
;586     paramFile->offset += count;
	.debug	# 586
	ld	hl,(ix+5)
	ld	de,0x2f
	add	hl,de
	push	hl
	ld	hl,(hl)
	ex	de,hl
	ld	hl,(ix+9)
	ex	de,hl
	add	hl,de
	pop	de
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;587     return count;
	.debug	# 587
	ld	hl,(ix+9)
	jr	@121
;588   }
@126:
	.debug	>
;589 
;590   status = write(paramFile->fd, from, count);
@124:
	.debug	# 590
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	ld	de,0x27
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$write
	add	sp,6
	ld	(ix-10),hl
;591 
;592   if (status > 0)  paramFile->offset += status;
	.debug	# 592
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@130
	jr	z,@130
	ld	hl,(ix+5)
	ld	de,0x2f
	add	hl,de
	push	hl
	ld	hl,(hl)
	ex	de,hl
	ld	hl,(ix-10)
	ex	de,hl
	add	hl,de
	pop	de
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;593 
;594   return status;
@130:
	.debug	# 594
	ld	hl,(ix-10)
;595 }
@121:
	.debug	} @121 595

@120	equ	14
	ld	sp,ix
	pop	ix
	lret
@122	equ	16
;596 
;597 
;598 //-------------------------------------------------------------------------
;599 //  Read a block of bytes from the cache or the file copy.
;600 //
;601 //  Returns the number of bytes read or < 0 if an error
;602 //-------------------------------------------------------------------------
;603 int readParamBytes(ParamFile *paramFile, char *to, uint count)
;604 {
$readParamBytes::
	.debug	G $readParamBytes 52 9
	.debug	{ $readParamBytes 52 603 @133 @132
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-14
	.debug	A paramFile 5 39
	.debug	A to 7 38
	.debug	A count 9 7
;605   char far  *cachePtr, *endCache;
	.debug	L cachePtr -4 46
	.debug	L endCache -8 46
;606   int stat;
	.debug	L stat -10 6
;607   BBM_Cache_Hdr far  *hdr = paramFile->bbmCache;
	.debug	L hdr -14 47
	.debug	# 607
	ld	hl,(ix+5)
	ld	de,0x29
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-12),hl
	ex	de,hl
	ld	(ix-14),hl
	ex	de,hl
	.seg	FARCODE
;608 
;609 
;610   if (hdr)  if (hdr->time)  {
	.debug	# 610
	ld	a,e
	or	a,d
	or	a,c
	jp	z,@134
	ld	h,b
	ld	l,c
	ld	bc,hl
	call	__farld4##
	ld	hl,bc
	or	hl,de
	jp	z,@135
;611     cachePtr = (uchar far *)hdr + HdrSize;
@136:
	.debug	< @136 @137
	.debug	# 611
	ld	hl,(ix-14)
	ex	de,hl
	ld	hl,(ix-12)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xa
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
;612     endCache = cachePtr + paramFile->cacheSize;
	.debug	# 612
	ld	hl,(ix+5)
	ld	de,0x2d
	add	hl,de
	ld	hl,(hl)
	ex	de,hl
	ld	b,h
	ld	c,h
	push	bc
	push	de
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	call	__ladd##
	ld	hl,bc
	ld	(ix-6),hl
	ex	de,hl
	ld	(ix-8),hl
	ex	de,hl
;613     cachePtr += paramFile->offset;
	.debug	# 613
	ld	hl,(ix+5)
	ld	de,0x2f
	add	hl,de
	ld	hl,(hl)
	ex	de,hl
	ld	b,h
	ld	c,h
	push	bc
	push	de
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	call	__ladd##
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
;614     if (cachePtr + count > endCache)  { ERRNO = ENODATA;  return -1; }
	.debug	# 614
	ld	hl,(ix+9)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	call	__ladd##
	push	bc
	push	de
	ld	hl,(ix-8)
	ex	de,hl
	ld	hl,(ix-6)
	ld	bc,hl
	call	__fpcmp##
	jr	c,@138
	jr	z,@138
@139:
	.debug	< @139 @140
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	e,0x3d
	ld	(hl),e
	inc	hl
	ld	(hl),d
	ld	hl,0xffff
	jp	@132
@140:
	.debug	>
;615     if (to)  fmemcpy(to, cachePtr, count);
@138:
	.debug	# 615
	ld	hl,(ix+7)
	bool	hl
	jr	z,@141
	ld	hl,(ix+9)
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix+7)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
;616     paramFile->offset += count;
@141:
	.debug	# 616
	ld	hl,(ix+5)
	ld	de,0x2f
	add	hl,de
	push	hl
	ld	hl,(hl)
	ex	de,hl
	ld	hl,(ix+9)
	ex	de,hl
	add	hl,de
	pop	de
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;617     return count;
	.debug	# 617
	ld	hl,(ix+9)
	jr	@132
;618   }
@137:
	.debug	>
;619 
;620   if (to)
@135:
@134:
	.debug	# 620
	ld	hl,(ix+7)
	bool	hl
	jr	z,@142
;621     stat = read(paramFile->fd, to, count);
	.debug	# 621
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	ld	de,0x27
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$read
	add	sp,6
	ld	(ix-10),hl
;622   else
;623     stat = skipBytes(paramFile->fd, count, 500);
	jr	@143
@142:
	.debug	# 623
	ld	de,0x1f4
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix+5)
	ld	de,0x27
	add	hl,de
	ld	hl,(hl)
	push	hl
	lcall	$skipBytes
	add	sp,8
	ld	(ix-10),hl
;624 
;625   if (stat > 0)  paramFile->offset += stat;
@143:
	.debug	# 625
	ld	hl,(ix-10)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@144
	jr	z,@144
	ld	hl,(ix+5)
	ld	de,0x2f
	add	hl,de
	push	hl
	ld	hl,(hl)
	ex	de,hl
	ld	hl,(ix-10)
	ex	de,hl
	add	hl,de
	pop	de
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;626 
;627   return stat;
@144:
	.debug	# 627
	ld	hl,(ix-10)
;628 }
@132:
	.debug	} @132 628

@131	equ	14
	ld	sp,ix
	pop	ix
	lret
@133	equ	16
;629 
;630 
;631 //-------------------------------------------------------------------------
;632 //  If the specified parameter list has a BBRAM cache, and the cached copy 
;633 //  the data is newer, then copy its contents to the appropriate file.
;634 //-------------------------------------------------------------------------
;635 void flushParamCache(char *buf, uint bufSize, ParamFile *paramFile)
;636 {
$flushParamCache::
	.debug	G $flushParamCache 53 9
	.debug	{ $flushParamCache 53 635 @147 @146
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-56
	.debug	A buf 5 38
	.debug	A bufSize 7 7
	.debug	A paramFile 9 39
;637   int  fd;
	.debug	L fd -2 6
;638   uint  count, countIdx;
	.debug	L count -4 7
	.debug	L countIdx -6 7
;639   char far  *cache;
	.debug	L cache -10 46
;640   BBM_Cache_Hdr far  *hdr;
	.debug	L hdr -14 47
;641   struct stat  fileInfo;
	.debug	L fileInfo -56 49
;642 
;643 
;644   if (!gotTime)  return;
	.debug	# 644
	ld	a,(_gotTime)
	or	a,a
	jp	z,@146
;645 
;646   hdr = paramFile->bbmCache;  if (!hdr)  return;
	.debug	# 646
	ld	hl,(ix+9)
	ld	de,0x29
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-12),hl
	ex	de,hl
	ld	(ix-14),hl
	ex	de,hl
	ld	a,e
	or	a,d
	or	a,c
	jp	z,@146
;647   if (!hdr->time)  return;
	.debug	# 647
	ld	h,b
	ld	l,c
	ld	bc,hl
	call	__farld4##
	ld	hl,bc
	or	hl,de
	jp	z,@146
;648 
;649   if ((fd = initParamFile(paramFile)) < 0)   return;
	.debug	# 649
	ld	hl,(ix+9)
	push	hl
	lcall	$initParamFile
	pop	de
	ld	(ix-2),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jp	c,@146
;650 
;651   if (!fstat(fd, &fileInfo) AND
;652     ((hdr->time > fileInfo.st_mtime)
;653         OR (fileInfo.st_size != paramStreamSize(paramFile))) )  {
	.debug	# 653
	ld	hl,-56+@145
	add	hl,sp
	push	hl
	ld	hl,(ix-2)
	push	hl
	lcall	$fstat
	add	sp,4
	bool	hl
	jp	nz,@153
	ld	hl,(ix-14)
	ex	de,hl
	ld	hl,(ix-12)
	ld	bc,hl
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix-22)
	ex	de,hl
	ld	hl,(ix-20)
	ld	bc,hl
	call	__lsub##
	jr	z,@156
	jr	nc,@157
@156:
	ld	hl,(ix+9)
	push	hl
	lcall	$paramStreamSize
	pop	de
	ex	de,hl
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-34)
	ex	de,hl
	ld	hl,(ix-32)
	ld	bc,hl
	call	__lsub##
	jp	z,@153
@157:
;654     printf("Flushing '%.24s' to flash...\r\n", paramFile->baseName);
@158:
	.debug	< @158 @159
	.debug	# 654
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	de,word (phys @9+235)
	ld	c,phys @9+235 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;655 
;656     countIdx = (uint)(GetSeconds() / 60) & 0x0f;  // 15 minute sliding wind
	.debug	# 656
ipset 3
	ld	de,(__TimerSeconds)
	ld	bc,(__TimerSeconds+2)
ipres
	push	bc
	push	de
	ld	de,0x3c
	ld	c,d
	ld	b,d
	call	__uldiv##
	ld	hl,0xf
	and	hl,de
	ld	(ix-6),hl
;657     ++paramFile->writeCount[countIdx];
	.debug	# 657
	ld	hl,(ix+9)
	ld	de,0x7
	add	hl,de
	ex	de,hl
	ld	hl,(ix-6)
	add	hl,hl
	add	hl,de
	push	hl
	ld	hl,(hl)
	inc	hl
	pop	de
	ex	de,hl
	ld	(hl),e
	inc	hl
	ld	(hl),d
;658 
;659   //--- Copy the contents of the cache to the file and set the time for ---
;660   //--- the cached copy equal to the modification time of the file ---
;661     count = hdr->size;  cache = (char far *)hdr + HdrSize;
	.debug	# 661
	ld	hl,(ix-14)
	ex	de,hl
	ld	hl,(ix-12)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-4),hl
	ld	hl,(ix-14)
	ex	de,hl
	ld	hl,(ix-12)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xa
	ld	c,d
	ld	b,d
	call	__ladd##
	ld	hl,bc
	ld	(ix-8),hl
	ex	de,hl
	ld	(ix-10),hl
	ex	de,hl
;662 //  printf("  write %d bytes from 0x%05lX using fd %d\r\n", count, cache, f
;663     if (fwrite_(fd, cache, count, buf, bufSize, 5000) != count)
	.debug	# 663
	ld	de,0x1388
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix-4)
	push	hl
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-2)
	push	hl
	lcall	$fwrite_
	add	sp,16
	ex	de,hl
	ld	hl,(ix-4)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	z,@160
;664       perror("flushParamCache()");
	.debug	# 664
	ld	de,word (phys @9+266)
	ld	c,phys @9+266 >> 16
	push	bc
	push	de
	lcall	$perror
	add	sp,4
;665     else
;666       if (!fstat(fd, &fileInfo))  hdr->time = fileInfo.st_mtime;
	jr	@161
@160:
	.debug	# 666
	ld	hl,-56+@145
	add	hl,sp
	push	hl
	ld	hl,(ix-2)
	push	hl
	lcall	$fstat
	add	sp,4
	bool	hl
	jr	nz,@162
	ld	hl,(ix-14)
	ex	de,hl
	ld	hl,(ix-12)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-22)
	ex	de,hl
	ld	hl,(ix-20)
	ld	bc,hl
	call	__farst4d##
;667   }
@162:
@161:
@159:
	.debug	>
;668 
;669   closeParamFile(paramFile);
@153:
	.debug	# 669
	ld	hl,(ix+9)
	push	hl
	lcall	$closeParamFile
	pop	hl
;670 }
@146:
	.debug	} @146 670

@145	equ	56
	ld	sp,ix
	pop	ix
	lret
@147	equ	58
;671 
;672 
;673 //-------------------------------------------------------------------------
;674 //  Update the specified file (or cache) of parameterized values from the l
;675 //  of runtime in-RAM values.
;676 //-------------------------------------------------------------------------
;677 int saveParams(char *buf, uint bufSize, ParamFile *paramFile, bool forceFlu
;678 {
$saveParams::
	.debug	G $saveParams 54 9
	.debug	{ $saveParams 54 677 @165 @164
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-24
	.debug	A buf 5 38
	.debug	A bufSize 7 7
	.debug	A paramFile 9 39
	.debug	A forceFlush 11 3
;679   uchar far  *data;
	.debug	L data -4 46
;680   uchar  *ptr;
	.debug	L ptr -6 38
;681   short  i, id, size, remain, count;
	.debug	L i -8 4
	.debug	L id -10 4
	.debug	L size -12 4
	.debug	L remain -14 4
	.debug	L count -16 4
;682   BBM_Cache_Hdr far  *hdr;
	.debug	L hdr -20 47
;683   const ParamData far *params = paramFile->params;
	.debug	L params -24 42
	.debug	# 683
	ld	hl,(ix+9)
	inc	hl
	inc	hl
	inc	hl
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-22),hl
	ex	de,hl
	ld	(ix-24),hl
	ex	de,hl
	.seg	FARCODE
;684 
;685 
;686   if (paramStreamSize(paramFile) < 1)  return 0;
	.debug	# 686
	ld	hl,(ix+9)
	push	hl
	lcall	$paramStreamSize
	pop	de
	ld	de,0x1
	or	a,a
	sbc	hl,de
	jr	nc,@166
	bool	hl
	ld	l,h
	jp	@164
;687 
;688   if (!gotTime)  {
@166:
	.debug	# 688
	ld	a,(_gotTime)
	or	a,a
	jr	nz,@167
;689     printf("Can't save persistent data until the clock is synchronized.\r\n
@168:
	.debug	< @168 @169
;690            "Try again later.\r\n");
	.debug	# 689
	ld	de,word (phys @9+284)
	ld	c,phys @9+284 >> 16
	push	bc
	push	de
	lcall	$farprintf
	jr	@575
;691     return -1;
	.debug	# 691
;692   }
@169:
	.debug	>
;693 
;694   if (bufSize < 64)  {
@167:
	.debug	# 694
	ld	hl,(ix+7)
	ld	de,0x40
	or	a,a
	sbc	hl,de
	jr	nc,@170
;695     printf("saveParams(): Insufficient buffer size: %d\r\n", bufSize);
@171:
	.debug	< @171 @172
	.debug	# 695
	ld	hl,(ix+7)
	push	hl
	ld	de,word (phys @9+364)
	ld	c,phys @9+364 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;696     return -1;
	.debug	# 696
	jp	@574
;697   }
@172:
	.debug	>
;698 
;699   if (prepForWrites(paramFile))  { perror("saveParams()");  return -1; }
@170:
	.debug	# 699
	ld	hl,(ix+9)
	push	hl
	lcall	$prepForWrites
	pop	de
	bool	hl
	jr	z,@173
@174:
	.debug	< @174 @175
	ld	de,word (phys @9+409)
	ld	c,phys @9+409 >> 16
	push	bc
	push	de
	lcall	$perror
@575:
	add	sp,4
	jp	@574
@175:
	.debug	>
;700 
;701   hdr = paramFile->bbmCache;
@173:
	.debug	# 701
	ld	hl,(ix+9)
	ld	de,0x29
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-18),hl
	ex	de,hl
	ld	(ix-20),hl
	ex	de,hl
;702 
;703   printf("Updating parameter %s: %s\r\n",
;704          ((hdr AND !forceFlush) ? "cache" : "file"), paramFile->baseName);
	.debug	# 704
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix-18)
	ld	bc,hl
	ld	a,e
	or	a,d
	or	a,c
	jr	z,@177
	ld	a,(ix+11)
	or	a,a
	jr	nz,@177
	ld	hl,@68+14
	jr	@178
@177:
	ld	hl,@68+20
@178:
	push	hl
	ld	de,word (phys @9+422)
	ld	c,phys @9+422 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
;705 
;706 
;707   for (i=0; params[i].id; ++i)  {
@180:
	.debug	< @180 @181
	.debug	# 707
	bool	hl
	ld	l,h
	ld	(ix-8),hl
@182:
	ld	hl,(ix-8)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	ld	a,0x4
	call	__lshl##
	push	bc
	push	de
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	ld	bc,hl
	call	__ladd##
	call	__farld2##
	ld	a,e
	or	a,d
	jp	z,@183
;708     id = params[i].id;  size = params[i].size;  data = params[i].data;
@185:
	.debug	< @185 @186
	.debug	# 708
	ld	hl,(ix-8)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	ld	a,0x4
	call	__lshl##
	push	bc
	push	de
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	ld	bc,hl
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-10),hl
	ld	hl,(ix-8)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	ld	a,0x4
	call	__lshl##
	push	bc
	push	de
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	ld	bc,hl
	call	__ladd##
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-12),hl
	ld	hl,(ix-8)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	ld	a,0x4
	call	__lshl##
	push	bc
	push	de
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	ld	bc,hl
	call	__ladd##
	push	bc
	push	de
	ld	de,0x6
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
;709 
;710   //--- write the id and the size, MSB 1st for both values ---
;711     ptr = buf;  addShort(&ptr, id);  addShort(&ptr, size);
	.debug	# 711
	ld	hl,(ix+5)
	ld	(ix-6),hl
	ld	hl,(ix-10)
	push	hl
	ld	hl,-4+@163
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
	ld	hl,(ix-12)
	push	hl
	ld	hl,-4+@163
	add	hl,sp
	push	hl
	lcall	$addShort
	add	sp,4
;712     if (writeParamBytes(paramFile, buf, 4) != 4)  {
	.debug	# 712
	ld	hl,0x4
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$writeParamBytes
	add	sp,6
	ld	de,0x4
	or	a,a
	sbc	hl,de
	jr	z,@187
;713       sprintf(buf, "writeParams(): ID:%d  size:%d (%.24s)",
@188:
	.debug	< @188 @189
;714               id, size, paramFile->baseName);
	.debug	# 714
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix-12)
	push	hl
	ld	hl,(ix-10)
	push	hl
	ld	de,word (phys @9+450)
	ld	c,phys @9+450 >> 16
	jr	@573
;715       perror(buf);  closeParamFile(paramFile);  return -1;
	.debug	# 715
;716     }
@189:
	.debug	>
;717 
;718   //--- write the data ---
;719     remain = size;
@187:
	.debug	# 719
	ld	hl,(ix-12)
	ld	(ix-14),hl
;720     while (remain > 0)  {
@190:
	.debug	# 720
	ld	hl,(ix-14)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jp	c,@191
	jp	z,@191
;721       if ((count = remain) > sizeof(buf))  count = sizeof(buf);
@192:
	.debug	< @192 @193
	.debug	# 721
	ld	hl,(ix-14)
	ld	(ix-16),hl
	ld	e,0x2
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@194
	jr	z,@194
	ld	hl,0x2
	ld	(ix-16),hl
;722       fmemcpy(buf, data, count);
@194:
	.debug	# 722
	ld	hl,(ix-16)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	push	bc
	push	de
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix+5)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
;723       if (writeParamBytes(paramFile, buf, count) != count)  {
	.debug	# 723
	ld	hl,(ix-16)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$writeParamBytes
	add	sp,6
	ex	de,hl
	ld	hl,(ix-16)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	z,@195
;724         sprintf(buf, "writeParams(): ID:%d  size:%d (%.24)",
@196:
	.debug	< @196 @197
;725                 id, size, paramFile->baseName);
	.debug	# 725
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix-12)
	push	hl
	ld	hl,(ix-10)
	push	hl
	ld	de,word (phys @9+488)
	ld	c,phys @9+488 >> 16
@573:
	push	bc
	push	de
	ld	hl,(ix+5)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,14
;726         perror(buf);  closeParamFile(paramFile);  return -1;
	.debug	# 726
	ld	hl,(ix+5)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	hl,(ix+9)
	push	hl
	lcall	$closeParamFile
	pop	hl
@574:
	ld	hl,0xffff
	jp	@164
;727       }
@197:
	.debug	>
;728       remain -= count;  data += count;
@195:
	.debug	# 728
	ld	hl,(ix-14)
	ex	de,hl
	ld	hl,(ix-16)
	ex	de,hl
	or	a,a
	sbc	hl,de
	ld	(ix-14),hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	push	bc
	push	de
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	call	__ladd##
	ld	hl,bc
	ld	(ix-2),hl
	ex	de,hl
	ld	(ix-4),hl
	ex	de,hl
;729     }
@193:
	.debug	>
;730 
;731     if (ShowDebugMsgs)
	jp	@190
@191:
	.debug	# 731
	ld	hl,(_current)
	ld	de,0xae
	add	hl,de
	ld	hl,(hl)
	ld	de,0x800
	and	hl,de
	jr	z,@198
;732       fprintf(DiagLog, "wrote param:%d  size:%d  addr: %05lX\n",
;733               id, size, (ulong)data );
	.debug	# 733
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,(ix-2)
	ld	bc,hl
	ld	b,0
	push	bc
	push	de
	ld	hl,(ix-12)
	push	hl
	ld	hl,(ix-10)
	push	hl
	ld	de,word (phys @9+525)
	ld	c,phys @9+525 >> 16
	push	bc
	push	de
	ld	hl,_diagLog
	push	hl
	lcall	$farfprintf
	add	sp,14
;734   }
@198:
@186:
	.debug	>
;735 
;736 
;737   if (hdr)  {
	ld	hl,(ix-8)
	inc	hl
	ld	(ix-8),hl
	jp	@182
@183:
@181:
	.debug	>
	.debug	# 737
	ld	hl,(ix-20)
	ex	de,hl
	ld	hl,(ix-18)
	ld	bc,hl
	ld	a,e
	or	a,d
	or	a,c
	jr	z,@199
;738     hdr->size = paramStreamSize(paramFile);
@200:
	.debug	< @200 @201
	.debug	# 738
	ld	hl,(ix+9)
	push	hl
	lcall	$paramStreamSize
	pop	de
	ex	de,hl
	ld	hl,(ix-18)
	ld	bc,hl
	ld	hl,(ix-20)
	ex	de,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farst2##
;739     hdr->crc = memCRC16((ulong)hdr + HdrSize, hdr->size);
	.debug	# 739
	ld	hl,(ix-20)
	ex	de,hl
	ld	hl,(ix-18)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-20)
	ex	de,hl
	ld	hl,(ix-18)
	ld	bc,hl
	ld	b,0
	push	bc
	push	de
	ld	de,0xa
	ld	c,d
	call	__ladd##
	push	bc
	push	de
	lcall	$memCRC16
	add	sp,8
	ex	de,hl
	ld	hl,(ix-18)
	ld	bc,hl
	ld	hl,(ix-20)
	ex	de,hl
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farst2##
;740     hdr->time = time(NULL);
	.debug	# 740
	bool	hl
	ld	l,h
	push	hl
	lcall	$time
	pop	hl
	push	bc
	push	de
	ld	hl,(ix-20)
	ex	de,hl
	ld	hl,(ix-18)
	ld	bc,hl
	call	__farst4p##
;741   }
@201:
	.debug	>
;742 
;743 
;744   if (forceFlush)  flushParamCache(buf, bufSize, paramFile);
@199:
	.debug	# 744
	ld	a,(ix+11)
	or	a,a
	jr	z,@202
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$flushParamCache
	add	sp,6
;745 
;746   closeParamFile(paramFile);
@202:
	.debug	# 746
	ld	hl,(ix+9)
	push	hl
	lcall	$closeParamFile
	pop	hl
;747 
;748   return 0;
	.debug	# 748
	bool	hl
	ld	l,h
;749 }
@164:
	.debug	} @164 749

@163	equ	24
	ld	sp,ix
	pop	ix
	lret
@165	equ	26
;750 
;751 //-------------------------------------------------------------------------
;752 //  Read parameterized data for the specified list of values.  For each dat
;753 //  element in the list, the value is read from the file or its cache.
;754 //  Depending upon the value of the 'justCompare' flag, each value read is
;755 //  either compared to the current runtime value or overwrites the current
;756 //  runtime value.
;757 //
;758 //  The list of parameters is terminated by a parameter with an id of 0.
;759 //
;760 //  Returns:
;761 //     0 if it reaches EOF and no errors occured
;762 //     < 0 if an error occured
;763 //     > 0 if in-RAM != saved copy (including any values missing from the f
;764 //-------------------------------------------------------------------------
;765 int loadParams(char *buf, uint bufSize, ParamFile *paramFile, bool justComp
;766 {
$loadParams::
	.debug	G $loadParams 54 9
	.debug	{ $loadParams 54 765 @205 @204
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-59
	.debug	A buf 5 38
	.debug	A bufSize 7 7
	.debug	A paramFile 9 39
	.debug	A justCompare 11 3
;767 #define  MaxParams 128
;768   int  status;
	.debug	L status -2 6
;769   bool  useCache = false;
	.debug	L useCache -3 3
	.debug	# 769
	xor	a,a
	ld	(ix-3),a
	.seg	FARCODE
;770   short  i, idx, id, size, extra, minRead, ttl, count, remain, numParams;
	.debug	L i -5 4
	.debug	L idx -7 4
	.debug	L id -9 4
	.debug	L size -11 4
	.debug	L extra -13 4
	.debug	L minRead -15 4
	.debug	L ttl -17 4
	.debug	L count -19 4
	.debug	L remain -21 4
	.debug	L numParams -23 4
;771   uchar  *ptr;
	.debug	L ptr -25 38
;772   char far  *curVal;
	.debug	L curVal -29 46
;773   uint  offset;
	.debug	L offset -31 7
;774   BBM_Cache_Hdr far *hdr;
	.debug	L hdr -35 47
;775   const ParamData far *param, *params = paramFile->params;
	.debug	L param -39 42
	.debug	L params -43 42
	.debug	# 775
	ld	hl,(ix+9)
	inc	hl
	inc	hl
	inc	hl
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-41),hl
	ex	de,hl
	ld	(ix-43),hl
	ex	de,hl
	.seg	FARCODE
;776   bool verified[(MaxParams+7)/8];
	.debug	L verified -59 55
;777 
;778 
;779 #define  SetVerified(x)  ( verified[(x)>>3] |= (1 << ((x) & 7)) )
;780 #define  IsVerified(x)   ( verified[(x)>>3] & (1 << ((x) & 7)) )
;781 
;782   if (paramStreamSize(paramFile) < 1)  return 0;
	.debug	# 782
	ld	hl,(ix+9)
	push	hl
	lcall	$paramStreamSize
	pop	de
	ld	de,0x1
	or	a,a
	sbc	hl,de
	jr	nc,@206
	bool	hl
	ld	l,h
	jp	@204
;783 
;784   if (bufSize < 64)  {
@206:
	.debug	# 784
	ld	hl,(ix+7)
	ld	de,0x40
	or	a,a
	sbc	hl,de
	jr	nc,@207
;785     printf("loadParams(): Insufficient buffer size: %d\r\n", bufSize);
@208:
	.debug	< @208 @209
	.debug	# 785
	ld	hl,(ix+7)
	push	hl
	ld	de,word (phys @9+563)
	ld	c,phys @9+563 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;786     return -1;
	.debug	# 786
	jr	@572
;787   }
@209:
	.debug	>
;788 
;789   if (prepForReads(paramFile))  {
@207:
	.debug	# 789
	ld	hl,(ix+9)
	push	hl
	lcall	$prepForReads
	pop	de
	bool	hl
	jr	z,@210
;790     if (errno != ENOENT)  perror("loadParams()");
@211:
	.debug	< @211 @212
	.debug	# 790
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0x2
	or	a,a
	sbc	hl,de
	jr	z,@572
	ld	de,word (phys @9+608)
	ld	c,phys @9+608 >> 16
	push	bc
	push	de
	lcall	$perror
	add	sp,4
;791     return -1;
	.debug	# 791
	jr	@572
;792   }
@212:
	.debug	>
;793 
;794   //fix:  Move this to BEFORE prepForReads()
;795 //--- determine how many parameters there are ---
;796   for (numParams=0; params[numParams].id; ++numParams);
@210:
@214:
	.debug	< @214 @215
	.debug	# 796
	bool	hl
	ld	l,h
	ld	(ix-23),hl
@216:
	ld	hl,(ix-23)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	ld	a,0x4
	call	__lshl##
	push	bc
	push	de
	ld	hl,(ix-43)
	ex	de,hl
	ld	hl,(ix-41)
	ld	bc,hl
	call	__ladd##
	call	__farld2##
	ld	a,e
	or	a,d
	jr	z,@217
;797   if (numParams > MaxParams)  {
	ld	hl,(ix-23)
	inc	hl
	ld	(ix-23),hl
	jr	@216
@217:
@215:
	.debug	>
	.debug	# 797
	ld	hl,(ix-23)
	ld	de,0x8080
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@219
	jr	z,@219
;798     printf("loadParams(): Max # parameters exceeded\r\n");  return -1; }
@220:
	.debug	< @220 @221
	.debug	# 798
	ld	de,word (phys @9+621)
	ld	c,phys @9+621 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
@572:
	ld	hl,0xffff
	jp	@204
@221:
	.debug	>
;799 
;800   BZERO(verified);
@219:
	.debug	# 800
	ld	hl,0x10
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-55+@203
	add	hl,sp
	push	hl
	call	_memset
	add	sp,6
;801 
;802   hdr = paramFile->bbmCache;
	.debug	# 802
	ld	hl,(ix+9)
	ld	de,0x29
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-33),hl
	ex	de,hl
	ld	(ix-35),hl
	ex	de,hl
;803   if (hdr)  if (hdr->time)  useCache = true;
	.debug	# 803
	ld	a,e
	or	a,d
	or	a,c
	jr	z,@222
	ld	h,b
	ld	l,c
	ld	bc,hl
	call	__farld4##
	ld	hl,bc
	or	hl,de
	jr	z,@223
	ld	(ix-3),0x1
;804 
;805 
;806   if (!justCompare)
@223:
@222:
	.debug	# 806
	ld	a,(ix+11)
	or	a,a
	jr	nz,@224
;807     printf("\r\nreloading '%.24s' from %s\r\n",
;808            paramFile->baseName, (useCache ? "BBRAM cache" : "Flash") );
	.debug	# 808
	ld	a,(ix-3)
	or	a,a
	jr	z,@225
	ld	hl,@68+25
	jr	@226
@225:
	ld	hl,@68+37
@226:
	push	hl
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	de,word (phys @9+663)
	ld	c,phys @9+663 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
;809 
;810 
;811 //--- For each value in the file, read it and (if doing a compare only) ---
;812 //--- determine if it matches the current in-RAM value ---
;813   for (status=0;;)  {
@224:
@228:
	.debug	< @228 @229
	.debug	# 813
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@230:
@232:
;814 
;815   //--- get id and size of next parameter ---
;816     offset = paramFile->offset;
@233:
	.debug	< @233 @234
	.debug	# 816
	ld	hl,(ix+9)
	ld	de,0x2f
	add	hl,de
	ld	hl,(hl)
	ld	(ix-31),hl
;817     if (readParamBytes(paramFile, buf, 4) != 4)  break;
	.debug	# 817
	ld	hl,0x4
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$readParamBytes
	add	sp,6
	ld	de,0x4
	or	a,a
	sbc	hl,de
	jp	nz,@231
;818 
;819     ptr = buf;  id = getShort(&ptr);  size = getShort(&ptr);
	.debug	# 819
	ld	hl,(ix+5)
	ld	(ix-25),hl
	ld	hl,-25+@203
	add	hl,sp
	push	hl
	lcall	$getShort
	pop	de
	ld	(ix-9),hl
	ld	hl,-25+@203
	add	hl,sp
	push	hl
	lcall	$getShort
	pop	de
	ld	(ix-11),hl
;820 
;821     if ((id <= 0) OR (size <= 0))  {
	.debug	# 821
	ld	hl,(ix-9)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@236
	jr	z,@236
	ld	hl,(ix-11)
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	z,@238
	jr	nc,@237
@238:
@236:
;822       printf("loadParams(): offset: %u  ID:%d  size:%d (%.24s)\r\n",
@239:
	.debug	< @239 @240
;823              offset, id, size, paramFile->baseName);
	.debug	# 823
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix-11)
	push	hl
	ld	hl,(ix-9)
	push	hl
	ld	hl,(ix-31)
	push	hl
	ld	de,word (phys @9+693)
	ld	c,phys @9+693 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,12
;824       printf("*** invalid ID or size\r\n");
	.debug	# 824
	ld	de,word (phys @9+744)
	ld	c,phys @9+744 >> 16
	push	bc
	push	de
	lcall	$farprintf
	jp	@571
;825       status = -1;  break;
	.debug	# 825
;826     }
@240:
	.debug	>
;827 
;828   //--- see if the parameter just read is one of the ones in the list ---
;829     param = NULL;
@237:
	.debug	# 829
	ld	de,0
	ld	c,e
	ld	b,e
	ld	hl,bc
	ld	(ix-37),hl
	ex	de,hl
	ld	(ix-39),hl
	ex	de,hl
;830     for (i=0; params[i].id; ++i)
@241:
	.debug	< @241 @242
	.debug	# 830
	ld	(ix-5),hl
@243:
	ld	hl,(ix-5)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	ld	a,0x4
	call	__lshl##
	push	bc
	push	de
	ld	hl,(ix-43)
	ex	de,hl
	ld	hl,(ix-41)
	ld	bc,hl
	call	__ladd##
	call	__farld2##
	ld	a,e
	or	a,d
	jr	z,@244
;831       if (params[i].id == id)  { param = params + i;  idx = i;  break; }
	.debug	# 831
	ld	hl,(ix-5)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	ld	a,0x4
	call	__lshl##
	push	bc
	push	de
	ld	hl,(ix-43)
	ex	de,hl
	ld	hl,(ix-41)
	ld	bc,hl
	call	__ladd##
	call	__farld2##
	ld	hl,(ix-9)
	or	a,a
	sbc	hl,de
	jr	nz,@246
@247:
	.debug	< @247 @248
	ld	hl,(ix-5)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	ld	a,0x4
	call	__lshl##
	push	bc
	push	de
	ld	hl,(ix-43)
	ex	de,hl
	ld	hl,(ix-41)
	ld	bc,hl
	call	__ladd##
	ld	hl,bc
	ld	(ix-37),hl
	ex	de,hl
	ld	(ix-39),hl
	ex	de,hl
	ld	hl,(ix-5)
	ld	(ix-7),hl
	jr	@244
@248:
	.debug	>
;832 
;833   //--- if not in the list, just skip it ---
;834     if (! param)  {
@246:
	ld	hl,(ix-5)
	inc	hl
	ld	(ix-5),hl
	jr	@243
@244:
@242:
	.debug	>
	.debug	# 834
	ld	hl,(ix-39)
	ex	de,hl
	ld	hl,(ix-37)
	ld	bc,hl
	ld	a,e
	or	a,d
	or	a,c
	jr	nz,@249
;835       if (!justCompare)  {
@250:
	.debug	< @250 @251
	.debug	# 835
	ld	a,(ix+11)
	or	a,a
	jr	nz,@252
;836         printf("loadParams(): offset: %u  ID:%d  size:%d (%.24s)\r\n",
@253:
	.debug	< @253 @254
;837                offset, id, size, paramFile->baseName);
	.debug	# 837
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix-11)
	push	hl
	ld	hl,(ix-9)
	push	hl
	ld	hl,(ix-31)
	push	hl
	ld	de,word (phys @9+693)
	ld	c,phys @9+693 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,12
;838         printf("skipping unknown param\r\n");
	.debug	# 838
	ld	de,word (phys @9+769)
	ld	c,phys @9+769 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;839       }
@254:
	.debug	>
;840       if (readParamBytes(paramFile, NULL, size) < size)  {
@252:
	.debug	# 840
	ld	hl,(ix-11)
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$readParamBytes
	add	sp,6
	ex	de,hl
	ld	hl,(ix-11)
	ex	de,hl
	call	__icmp##
	jp	c,@570
;841         printf("loadParams(): offset: %u  ID:%d  size:%d (%.24s)\r\n",
@256:
	.debug	< @256 @257
;842                offset, id, size, paramFile->baseName);
	.debug	# 842
;843         perror(NULL);  status = -1;  break;
	.debug	# 843
;844       }
@257:
	.debug	>
;845       continue;  // successfully skipped this param
	.debug	# 845
;846     }
	jp	@232
@251:
	.debug	>
;847 
;848   //--- Set size to smaller of provided size/expected size and set the ---
;849   //--- minimum # of bytes we must read/copy each time to avoid major ---
;850   //--- corruption of the values ---
;851     extra = 0;
@249:
	.debug	# 851
	bool	hl
	ld	l,h
	ld	(ix-13),hl
;852     if (param->size < size)  {
	.debug	# 852
	ld	hl,(ix-39)
	ex	de,hl
	ld	hl,(ix-37)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	hl,(ix-11)
	ex	de,hl
	call	__icmp##
	jr	nc,@258
;853       extra = size - param->size;  size = param->size; }
@259:
	.debug	< @259 @260
	.debug	# 853
	ld	hl,(ix-39)
	ex	de,hl
	ld	hl,(ix-37)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	hl,(ix-11)
	or	a,a
	sbc	hl,de
	ld	(ix-13),hl
	ld	hl,(ix-39)
	ex	de,hl
	ld	hl,(ix-37)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x2
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-11),hl
@260:
	.debug	>
;854     if ((minRead = param->gran) < 1)  minRead = size;
@258:
	.debug	# 854
	ld	hl,(ix-39)
	ex	de,hl
	ld	hl,(ix-37)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-15),hl
	ld	de,0x8001
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@261
	ld	hl,(ix-11)
	ld	(ix-15),hl
;855     if (size <= bufSize)  minRead = size;
@261:
	.debug	# 855
	ld	hl,(ix-11)
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	z,@263
	jr	nc,@262
@263:
	ld	hl,(ix-11)
	ld	(ix-15),hl
;856 
;857   //--- Is the smallest logical-piece (defined by the granularity) too big 
;858   //--- for our buffer? (could read in pieces, but then we would be ---
;859   //--- taking a chance that we might not get the entire value, which ---
;860   //--- would corrupt the state of the parameter we were reading)
;861     if (minRead > bufSize)  {
@262:
	.debug	# 861
	ld	hl,(ix-15)
	ex	de,hl
	ld	hl,(ix+7)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	c,@264
	jr	z,@264
;862       printf("loadParams(): offset: %u  ID:%d  size:%d (%.24s)\r\n",
@265:
	.debug	< @265 @266
;863              offset, id, size, paramFile->baseName);
	.debug	# 863
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix-11)
	push	hl
	ld	hl,(ix-9)
	push	hl
	ld	hl,(ix-31)
	push	hl
	ld	de,word (phys @9+693)
	ld	c,phys @9+693 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,12
;864       printf("*** granularity too large: %d\r\n", minRead);
	.debug	# 864
	ld	hl,(ix-15)
	push	hl
	ld	de,word (phys @9+794)
	ld	c,phys @9+794 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;865       status = -1;  break;
	.debug	# 865
	jp	@569
;866     }
@266:
	.debug	>
;867 
;868   //--- read the value (in safe-size chunks if necessary) ---
;869     count = minRead;  remain = size;  ttl = 0;
@264:
	.debug	# 869
	ld	hl,(ix-15)
	ld	(ix-19),hl
	ld	hl,(ix-11)
	ld	(ix-21),hl
	bool	hl
	ld	l,h
	ld	(ix-17),hl
;870     if (!justCompare)  printf("  restoring: %ls\r\n", param->name);
	.debug	# 870
	ld	a,(ix+11)
	or	a,a
	jr	nz,@267
	ld	hl,(ix-39)
	ex	de,hl
	ld	hl,(ix-37)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xa
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	push	bc
	push	de
	ld	de,word (phys @9+826)
	ld	c,phys @9+826 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
;871 
;872     while (remain)  {
@267:
@268:
	.debug	# 872
	ld	hl,(ix-21)
	bool	hl
	jp	z,@269
;873       if (ShowDebugMsgs)
@270:
	.debug	< @270 @271
	.debug	# 873
	ld	hl,(_current)
	ld	de,0xae
	add	hl,de
	ld	hl,(hl)
	ld	de,0x800
	and	hl,de
	jr	z,@272
;874         printf("loadParams(): offset: %u  ID:%d  size:%d (%.24s)\r\n",
;875                offset, id, size, paramFile->baseName);
	.debug	# 875
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix-11)
	push	hl
	ld	hl,(ix-9)
	push	hl
	ld	hl,(ix-31)
	push	hl
	ld	de,word (phys @9+693)
	ld	c,phys @9+693 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,12
;876 
;877       if (count > remain)  count = remain;
@272:
	.debug	# 877
	ld	hl,(ix-19)
	ex	de,hl
	ld	hl,(ix-21)
	ex	de,hl
	call	__icmp##
	jr	c,@273
	jr	z,@273
	ld	hl,(ix-21)
	ld	(ix-19),hl
;878 
;879       if (readParamBytes(paramFile, buf, count) != count)  {
@273:
	.debug	# 879
	ld	hl,(ix-19)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$readParamBytes
	add	sp,6
	ex	de,hl
	ld	hl,(ix-19)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	z,@274
;880         if (!justCompare)  {
@275:
	.debug	< @275 @276
	.debug	# 880
	ld	a,(ix+11)
	or	a,a
	jr	nz,@277
;881           printf("loadParams(): offset: %u  ID:%d  size:%d (%.24s)\r\n",
@278:
	.debug	< @278 @279
;882                  offset, id, size, paramFile->baseName);
	.debug	# 882
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix-11)
	push	hl
	ld	hl,(ix-9)
	push	hl
	ld	hl,(ix-31)
	push	hl
	ld	de,word (phys @9+693)
	ld	c,phys @9+693 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,12
;883           perror(NULL);
	.debug	# 883
	ld	de,0
	ld	c,e
	ld	b,e
	push	bc
	push	de
	lcall	$perror
	add	sp,4
;884         }
@279:
	.debug	>
;885         status = -1;  break;
@277:
	.debug	# 885
	ld	hl,0xffff
	ld	(ix-2),hl
;886       }
	jp	@269
@276:
	.debug	>
;887       if (status)  break;
@274:
	.debug	# 887
	ld	hl,(ix-2)
	bool	hl
	jp	nz,@269
;888 
;889     //--- if just comparing in-memory to saved copy, then simply return ---
;890     //--- a positive value if they don't match ---
;891       curVal = (char far *)param->data + (uint)ttl;
	.debug	# 891
	ld	hl,(ix-39)
	ex	de,hl
	ld	hl,(ix-37)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x6
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	ld	hl,(ix-17)
	push	bc
	push	de
	ex	de,hl
	ld	bc,0
	call	__ladd##
	ld	hl,bc
	ld	(ix-27),hl
	ex	de,hl
	ld	(ix-29),hl
	ex	de,hl
;892       if (justCompare)  {
	.debug	# 892
	ld	a,(ix+11)
	or	a,a
	jr	z,@281
;893         if (fmemcmp(curVal, buf, count) != 0)  {
@282:
	.debug	< @282 @283
	.debug	# 893
	ld	hl,(ix-19)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	push	bc
	push	de
	ld	hl,(ix+5)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	ld	hl,(ix-29)
	ex	de,hl
	ld	hl,(ix-27)
	ld	bc,hl
	push	bc
	push	de
	call	_fmemcmp
	add	sp,12
	bool	hl
	jr	z,@284
;894           printf("Runtime value of %.24s.%ls has changed\r\n",
@285:
	.debug	< @285 @286
;895                  paramFile->baseName, param->name);
	.debug	# 895
	ld	hl,(ix-39)
	ex	de,hl
	ld	hl,(ix-37)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xa
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	de,word (phys @9+845)
	ld	c,phys @9+845 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,10
;896           //fix:  do:  status = idx + 1;  break;  instead of a return!
;897           return idx + 1;
	.debug	# 897
	ld	hl,(ix-7)
	jp	@568
;898         }
@286:
	.debug	>
;899       }
@284:
@283:
	.debug	>
;900       else  {
	jp	@287
@281:
;901     //--- Read a self-contained chunk without an error, so it is safe to --
;902     //--- overwrite the existing value with the one just read.  If the valu
;903     //--- has a restore func, then call it.  If not, check to see if it is 
;904     //--- a externally accessible (register/PV) value.  If neither of those
;905     //--- is the case, then just copy the new value. ---
;906         if (param->restoreVal)  // value-specific handling of new values?
@288:
	.debug	< @288 @289
	.debug	# 906
	ld	hl,(ix-39)
	ex	de,hl
	ld	hl,(ix-37)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xe
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	a,e
	or	a,d
	jr	z,@290
;907           (*param->restoreVal)(curVal, buf, count);
	.debug	# 907
	ld	hl,(ix-19)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix-29)
	ex	de,hl
	ld	hl,(ix-27)
	ld	bc,hl
	push	bc
	push	de
	push	bc
	push	de
	ld	hl,(ix-39)
	ex	de,hl
	ld	hl,(ix-37)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0xe
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	pop	de
	pop	bc
	call	__callhl##
	add	sp,8
;908         else
;909           if (!restoreIfRegValue(curVal, buf, count))  { // reg value?
	jr	@291
@290:
	.debug	# 909
	ld	hl,(ix-19)
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,(ix-29)
	ex	de,hl
	ld	hl,(ix-27)
	ld	bc,hl
	push	bc
	push	de
	lcall	$restoreIfRegValue
	add	sp,8
	bool	hl
	jr	nz,@292
;910             KLOCK();  fmemcpy(curVal, buf, count);  KUNLOCK();  }
@293:
	.debug	< @293 @294
	.debug	# 910
	call	_KLOCK
	ld	hl,(ix-19)
	ex	de,hl
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	push	bc
	push	de
	ld	hl,(ix+5)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	ld	hl,(ix-29)
	ex	de,hl
	ld	hl,(ix-27)
	ld	bc,hl
	push	bc
	push	de
	lcall	$fmemcpy
	add	sp,12
	call	_KUNLOCK
@294:
	.debug	>
;911       }
@292:
@291:
@289:
	.debug	>
;912 
;913       ttl += count;  remain -= count;
@287:
	.debug	# 913
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-19)
	ex	de,hl
	add	hl,de
	ld	(ix-17),hl
	ld	hl,(ix-21)
	or	a,a
	sbc	hl,de
	ld	(ix-21),hl
;914     }
@271:
	.debug	>
;915 
;916     SetVerified(idx);
	jp	@268
@269:
	.debug	# 916
	ld	hl,(ix-7)
	ld	de,0x7
	and	hl,de
	ld	a,l
	ld	hl,0x1
	or	a,a
	jr	z,@295
	ld	b,a
@296:
	add	hl,hl
	djnz	@296
@295:
	ex	de,hl
	ld	hl,(ix-7)
	ex	de,hl
	ld	b,3
@297:
	sra	d
	rr	e
	djnz	@297
	ld	bc,hl
	ld	hl,-59+@203
	add	hl,sp
	add	hl,de
	push	hl
	ld	a,(hl)
	bool	hl
	ld	l,a
	ld	de,bc
	or	hl,de
	pop	de
	ex	de,hl
	ld	(hl),e
;917 
;918     if (extra)  if (readParamBytes(paramFile, NULL, extra) < extra)  {
	.debug	# 918
	ld	hl,(ix-13)
	bool	hl
	jr	z,@298
	ld	hl,(ix-13)
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix+9)
	push	hl
	lcall	$readParamBytes
	add	sp,6
	ex	de,hl
	ld	hl,(ix-13)
	ex	de,hl
	call	__icmp##
	jr	nc,@299
;919       printf("loadParams(): offset: %u  ID:%d  size:%d (%.24s)\r\n",
@300:
	.debug	< @300 @301
;920              offset, id, size, paramFile->baseName);
	.debug	# 920
@570:
	ld	hl,(ix+9)
	ld	hl,(hl)
	push	hl
	ld	hl,(ix-11)
	push	hl
	ld	hl,(ix-9)
	push	hl
	ld	hl,(ix-31)
	push	hl
	ld	de,word (phys @9+693)
	ld	c,phys @9+693 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,12
;921       perror(NULL);  status = -1;  break;
	.debug	# 921
	ld	de,0
	ld	c,e
	ld	b,e
	push	bc
	push	de
	lcall	$perror
@571:
	add	sp,4
@569:
	ld	hl,0xffff
	ld	(ix-2),hl
;922     }
	jr	@231
@301:
	.debug	>
;923 
;924     if (ShowDebugMsgs)  printf("read param:%d size:%d\r\n", id, size);
@299:
@298:
	.debug	# 924
	ld	hl,(_current)
	ld	de,0xae
	add	hl,de
	ld	hl,(hl)
	ld	de,0x800
	and	hl,de
	jr	z,@302
	ld	hl,(ix-11)
	push	hl
	ld	hl,(ix-9)
	push	hl
	ld	de,word (phys @9+886)
	ld	c,phys @9+886 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,8
;925   }
@302:
@234:
	.debug	>
;926 
;927   closeParamFile(paramFile);
	jp	@230
@231:
@229:
	.debug	>
	.debug	# 927
	ld	hl,(ix+9)
	push	hl
	lcall	$closeParamFile
	pop	hl
;928 
;929   if (!justCompare)  printf("\r\n");
	.debug	# 929
	ld	a,(ix+11)
	or	a,a
	jr	nz,@303
	ld	de,word (phys @9+98)
	ld	c,phys @9+98 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;930 
;931   //fix:  do  if (status)  return status;  instead
;932   if (status < 0)  return status;
@303:
	.debug	# 932
	ld	hl,(ix-2)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@304
	ld	hl,(ix-2)
	jr	@204
;933 
;934 //--- Got through all the values in the file without finding any that ---
;935 //--- didn't match the ones in memory, but we still need to make sure ---
;936 //--- that all the ones in memory were in the file ---
;937   for (i=0; i<numParams; ++i)  if (!IsVerified(i))  return i+1;
@304:
@305:
	.debug	< @305 @306
	.debug	# 937
	bool	hl
	ld	l,h
	ld	(ix-5),hl
@307:
	ld	hl,(ix-5)
	ex	de,hl
	ld	hl,(ix-23)
	ex	de,hl
	call	__icmp##
	jr	nc,@308
	ld	hl,(ix-5)
	ld	b,3
@310:
	sra	h
	rr	l
	djnz	@310
	ex	de,hl
	ld	hl,-59+@203
	add	hl,sp
	add	hl,de
	ex	de,hl
	ld	hl,(ix-5)
	ex	de,hl
	ld	bc,hl
	ld	hl,0x7
	and	hl,de
	ld	a,l
	ld	hl,0x1
	or	a,a
	jr	z,@311
@312:
	add	hl,hl
	dec	a
	jr	nz,@312
@311:
	ld	a,(bc)
	ld	d,0
	ld	e,a
	and	hl,de
	jr	nz,@313
	ld	hl,(ix-5)
@568:
	inc	hl
	jr	@204
;938 
;939   status = 0;
@313:
	ld	hl,(ix-5)
	inc	hl
	ld	(ix-5),hl
	jr	@307
@308:
@306:
	.debug	>
	.debug	# 939
	bool	hl
	ld	l,h
	ld	(ix-2),hl
;940 
;941   return status;
	.debug	# 941
;942 }
@204:
	.debug	} @204 942

@203	equ	59
	ld	sp,ix
	pop	ix
	lret
@205	equ	61
;943 
;944 
;945 //-------------------------------------------------------------------------
;946 //  If the specified parameter list has a BBRAM cache, then update it as
;947 //  necessary and then determine if the file for the same list is also in
;948 //  need of updating.  If the file has older data than the cache, and we
;949 //  haven't updated much recently, then do so now.
;950 //-------------------------------------------------------------------------
;951 void checkCache(char *buf, uint bufSize, ParamFile *paramFile)
;952 {
$checkCache::
	.debug	G $checkCache 53 9
	.debug	{ $checkCache 53 951 @316 @315
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-59
	.debug	A buf 5 38
	.debug	A bufSize 7 7
	.debug	A paramFile 9 39
;953   bool  updateFile;
	.debug	L updateFile -1 3
;954   uint  ui, countIdx, numWrites;
	.debug	L ui -3 7
	.debug	L countIdx -5 7
	.debug	L numWrites -7 7
;955   int  fd;
	.debug	L fd -9 6
;956   ulong  now;
	.debug	L now -13 9
;957   BBM_Cache_Hdr far  *hdr;
	.debug	L hdr -17 47
;958   struct stat  fileInfo;
	.debug	L fileInfo -59 49
;959 
;960 
;961   if (!gotTime)  return;
	.debug	# 961
	ld	a,(_gotTime)
	or	a,a
	jp	z,@315
;962 
;963   hdr = paramFile->bbmCache;  if (!hdr)  return;
	.debug	# 963
	ld	hl,(ix+9)
	ld	de,0x29
	add	hl,de
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-15),hl
	ex	de,hl
	ld	(ix-17),hl
	ex	de,hl
	ld	a,e
	or	a,d
	or	a,c
	jp	z,@315
;964 
;965 //--- limit checks to one of the affected files every 5 seconds in a ---
;966 //--- round-robin fashion to minimize the overhead incurred ---
;967   if (msSince(lastChgChk) < 5000)  return;
	.debug	# 967
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	push	bc
	push	de
	ld	de,(_lastChgChk)
	ld	bc,(_lastChgChk+2)
	call	__lsub##
	push	bc
	push	de
	ld	de,0x1388
	ld	bc,0
	call	__lsub##
	jp	c,@315
;968   lastChgChk = GetMS();
	.debug	# 968
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	ld	(_lastChgChk),de
	ld	(_lastChgChk+2),bc
;969 
;970 //printf("--- checking on [%.20s] ---\r\n", paramFile->baseName);  //tdebug
;971 
;972 //--- if the run-time values have changed then update the cache copy ---
;973   if (loadParams(buf, bufSize, paramFile, JustCompare) > 0)  {
	.debug	# 973
	ld	hl,0x1
	push	hl
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$loadParams
	add	sp,8
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@320
	jr	z,@320
;974     saveParams(buf, bufSize, paramFile, DontFlush);  return; }
@321:
	.debug	< @321 @322
	.debug	# 974
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$saveParams
	add	sp,8
	jp	@315
@322:
	.debug	>
;975 
;976 
;977   if ((fd = initParamFile(paramFile)) < 0)  return;
@320:
	.debug	# 977
	ld	hl,(ix+9)
	push	hl
	lcall	$initParamFile
	pop	de
	ld	(ix-9),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jp	c,@315
;978 
;979   if (fstat(fd, &fileInfo))  {
	.debug	# 979
	ld	hl,-59+@314
	add	hl,sp
	push	hl
	ld	hl,(ix-9)
	push	hl
	lcall	$fstat
	add	sp,4
	bool	hl
	jr	z,@324
;980     perror("checkCache()");  closeParamFile(paramFile);  return; }
@325:
	.debug	< @325 @326
	.debug	# 980
	ld	de,word (phys @9+910)
	ld	c,phys @9+910 >> 16
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	hl,(ix+9)
	push	hl
	lcall	$closeParamFile
	pop	hl
	jp	@315
@326:
	.debug	>
;981 
;982 //--- 16 counters, one always set to 0, used in a round-robin fashion ---
;983 //--- to keep track of # of writes over a 15-minute sliding window ---
;984   countIdx = (uint)(GetSeconds() / 60) & 0x0f;  // 15 minute sliding window
@324:
	.debug	# 984
ipset 3
	ld	de,(__TimerSeconds)
	ld	bc,(__TimerSeconds+2)
ipres
	push	bc
	push	de
	ld	de,0x3c
	ld	c,d
	ld	b,d
	call	__uldiv##
	ld	hl,0xf
	and	hl,de
	ld	(ix-5),hl
;985   paramFile->writeCount[(countIdx + 1) & 0x0f] = 0;  // 16th pos always 0
	.debug	# 985
	inc	hl
	ld	de,0xf
	and	hl,de
	ex	de,hl
	ld	hl,(ix+9)
	ex	de,hl
	ld	bc,hl
	ld	hl,0x7
	add	hl,de
	ld	de,bc
	ex	de,hl
	add	hl,hl
	add	hl,de
	ld	de,0
	ld	(hl),e
	inc	hl
	ld	(hl),d
;986 
;987   now = time(NULL);
	.debug	# 987
	bool	hl
	ld	l,h
	push	hl
	lcall	$time
	pop	hl
	ld	hl,bc
	ld	(ix-11),hl
	ex	de,hl
	ld	(ix-13),hl
	ex	de,hl
;988 
;989   for (;;)  {
@327:
	.debug	< @327 @328
;990     updateFile = true;
@332:
	.debug	< @332 @333
	.debug	# 990
	ld	(ix-1),0x1
;991 
;992   //--- if the file is not the right size, then update it.  This includes -
;993   //--- new files which may have a newer time than the cached data. ---
;994     if (fileInfo.st_size != hdr->size)  break;
	.debug	# 994
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-15)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x4
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ld	bc,0
	push	bc
	push	de
	ld	hl,(ix-37)
	ex	de,hl
	ld	hl,(ix-35)
	ld	bc,hl
	call	__lsub##
	jr	nz,@330
;995 
;996   //--- The cached data must be stable for at least 60 secs before flushing
;997     if (now - hdr->time < 60)  { updateFile = false;  break; }
	.debug	# 997
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-15)
	ld	bc,hl
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix-13)
	ex	de,hl
	ld	hl,(ix-11)
	ld	bc,hl
	call	__lsubr##
	push	bc
	push	de
	ld	de,0x3c
	ld	c,d
	ld	b,d
	call	__lsub##
	jr	c,@567
@336:
	.debug	< @336 @337
@337:
	.debug	>
;998 
;999   //--- if the file copy is newer, then don't update it ---
;1000     if (fileInfo.st_mtime >= hdr->time)  { updateFile = false;  break; }
	.debug	# 1000
	ld	hl,(ix-17)
	ex	de,hl
	ld	hl,(ix-15)
	ld	bc,hl
	call	__farld4##
	push	bc
	push	de
	ld	hl,(ix-25)
	ex	de,hl
	ld	hl,(ix-23)
	ld	bc,hl
	call	__lsubr##
	jr	c,@338
@339:
	.debug	< @339 @340
@567:
	xor	a,a
	ld	(ix-1),a
	jr	@330
@340:
	.debug	>
;1001 //    printf("  file time: %lu  cache time: %lu\r\n", fileInfo.st_mtime, hd
;1002 
;1003   //--- Don't flush more often than once every 15 minutes ---
;1004     for (ui=0, numWrites=0; ui<16; ++ui) numWrites += paramFile->writeCount
@338:
@341:
	.debug	< @341 @342
	.debug	# 1004
	bool	hl
	ld	l,h
	ld	(ix-3),hl
	ld	(ix-7),hl
@343:
	ld	hl,(ix-3)
	ld	de,0x10
	or	a,a
	sbc	hl,de
	jr	nc,@344
	ld	hl,(ix+9)
	ld	e,0x7
	add	hl,de
	ex	de,hl
	ld	hl,(ix-3)
	add	hl,hl
	add	hl,de
	ld	hl,(hl)
	ex	de,hl
	ld	hl,(ix-7)
	ex	de,hl
	add	hl,de
	ld	(ix-7),hl
;1005 //    printf("  file '%s':  fd: %d  numWrites: %d,  age: %ld\r\n",  //tdebu
;1006 //       paramFile->baseName, fd, numWrites, (time(NULL) - fileInfo.st_mtim
;1007     if (numWrites)  updateFile = false;
	ld	hl,(ix-3)
	inc	hl
	ld	(ix-3),hl
	jr	@343
@344:
@342:
	.debug	>
	.debug	# 1007
	ld	hl,(ix-7)
	bool	hl
	jr	z,@330
	xor	a,a
	ld	(ix-1),a
;1008 
;1009     break;
	.debug	# 1009
;1010   }
@333:
	.debug	>
;1011 
;1012   closeParamFile(paramFile);
@330:
@328:
	.debug	>
	.debug	# 1012
	ld	hl,(ix+9)
	push	hl
	lcall	$closeParamFile
	pop	hl
;1013 
;1014 //--- copy the contents of the cache to the file ---
;1015   if (updateFile)  flushParamCache(buf, bufSize, paramFile);
	.debug	# 1015
	ld	a,(ix-1)
	or	a,a
	jr	z,@347
	ld	hl,(ix+9)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$flushParamCache
	add	sp,6
;1016 }
@347:
@315:
	.debug	} @315 1016

@314	equ	59
	ld	sp,ix
	pop	ix
	lret
@316	equ	61
;1017 
;1018 
;1019 
;1020 //-------------------------------------------------------------------------
;1021 //  Load values from all the parameter files
;1022 //-------------------------------------------------------------------------
;1023 void loadAllParams(char *buf, uint bufSize)
;1024 {
$loadAllParams::
	.debug	G $loadAllParams 56 9
	.debug	{ $loadAllParams 56 1023 @350 @349
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-2
	.debug	A buf 5 38
	.debug	A bufSize 7 7
;1025   for (uint i=0; i<NumParamFiles; ++i)
@351:
	.debug	< @351 @352
	.debug	L i -2 7
	.debug	# 1025
	bool	hl
	ld	l,h
	ld	(ix-2),hl
	.seg	FARCODE
@353:
	ld	hl,(ix-2)
	ld	de,(_NumParamFiles)
	or	a,a
	sbc	hl,de
	jr	nc,@354
;1026     loadParams(buf, bufSize, paramFiles[i], Reload);
	.debug	# 1026
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-2)
	add	hl,hl
	ld	de,_paramFiles
	add	hl,de
	ld	hl,(hl)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$loadParams
	add	sp,8
;1027 }
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@353
@354:
@352:
	.debug	>
@349:
	.debug	} @349 1027

@348	equ	2
	ld	sp,ix
	pop	ix
	lret
@350	equ	4
;1028 
;1029 
;1030 //-------------------------------------------------------------------------
;1031 //  Send a msg to the applic process to reload all the parameter files
;1032 //-------------------------------------------------------------------------
;1033 int loadAllParamsMsg(void)
;1034 {
$loadAllParamsMsg::
	.debug	G $loadAllParamsMsg 57 9
	.debug	{ $loadAllParamsMsg 57 1033 @358 @357
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-26
;1035   uint  i;
	.debug	L i -2 7
;1036   fd_t  eventsfd;
	.debug	L eventsfd -4 6
;1037   EventMsg  eventMsg;
	.debug	L eventMsg -26 58
;1038 
;1039 
;1040   if ((eventsfd = open(EventsFIFO, O_WRONLY, 0)) < 0)  return -1;
	.debug	# 1040
	bool	hl
	ld	l,h
	push	hl
	ld	l,0x2
	push	hl
	ld	hl,@68+43
	push	hl
	lcall	$open
	add	sp,6
	ld	(ix-4),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@359
	ld	hl,0xffff
	jr	@357
;1041 
;1042   for (i=0; i<NumParamFiles; ++i)  {
@359:
@360:
	.debug	< @360 @361
	.debug	# 1042
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@362:
	ld	hl,(ix-2)
	ld	de,(_NumParamFiles)
	or	a,a
	sbc	hl,de
	jr	nc,@363
;1043      *(uint *)(eventMsg.data) = i;
@365:
	.debug	< @365 @366
	.debug	# 1043
	ld	hl,(ix-2)
	ld	(ix-20),hl
;1044      EVENT(RestoredParamFile, 0, NULL, sizeof(uint));
	.debug	# 1044
	ld	hl,0x2
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-22+@356
	add	hl,sp
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_EVENT_RestoredParamFile
	push	hl
	ld	hl,(ix-4)
	push	hl
	lcall	$putMsg
	add	sp,12
;1045   }
@366:
	.debug	>
;1046 
;1047   close(eventsfd);  return 0;
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@362
@363:
@361:
	.debug	>
	.debug	# 1047
	ld	hl,(ix-4)
	push	hl
	lcall	$close
	pop	de
	bool	hl
	ld	l,h
;1048 }
@357:
	.debug	} @357 1048

@356	equ	26
	ld	sp,ix
	pop	ix
	lret
@358	equ	28
;1049 
;1050 //-------------------------------------------------------------------------
;1051 //  Send a msg to the applic process to reload a specific ParamFile
;1052 //-------------------------------------------------------------------------
;1053 int loadParamsMsg(ParamFile *paramFile)
;1054 {
$loadParamsMsg::
	.debug	G $loadParamsMsg 51 9
	.debug	{ $loadParamsMsg 51 1053 @369 @368
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-26
	.debug	A paramFile 5 39
;1055   uint  i;
	.debug	L i -2 7
;1056   fd_t  eventsfd;
	.debug	L eventsfd -4 6
;1057   EventMsg  eventMsg;
	.debug	L eventMsg -26 58
;1058 
;1059 
;1060   if ((eventsfd = open(EventsFIFO, O_WRONLY, 0)) < 0)  return -1;
	.debug	# 1060
	bool	hl
	ld	l,h
	push	hl
	ld	l,0x2
	push	hl
	ld	hl,@68+43
	push	hl
	lcall	$open
	add	sp,6
	ld	(ix-4),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@370
	ld	hl,0xffff
	jr	@368
;1061 
;1062   for (i=0; i<NumParamFiles; ++i)  {
@370:
@371:
	.debug	< @371 @372
	.debug	# 1062
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@373:
	ld	hl,(ix-2)
	ld	de,(_NumParamFiles)
	or	a,a
	sbc	hl,de
	jr	nc,@374
;1063     if (paramFiles[i] != paramFile)  continue;
@376:
	.debug	< @376 @377
	.debug	# 1063
	ld	hl,(ix-2)
	add	hl,hl
	ld	de,_paramFiles
	add	hl,de
	ld	hl,(hl)
	ex	de,hl
	ld	hl,(ix+5)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	nz,@375
;1064      *(uint *)(eventMsg.data) = i;
	.debug	# 1064
	ld	hl,(ix-2)
	ld	(ix-20),hl
;1065      EVENT(RestoredParamFile, 0, NULL, sizeof(uint));
	.debug	# 1065
	ld	hl,0x2
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-22+@367
	add	hl,sp
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_EVENT_RestoredParamFile
	push	hl
	ld	hl,(ix-4)
	push	hl
	lcall	$putMsg
	add	sp,12
;1066   }
@377:
	.debug	>
;1067 
;1068   close(eventsfd);  return 0;
@375:
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@373
@374:
@372:
	.debug	>
	.debug	# 1068
	ld	hl,(ix-4)
	push	hl
	lcall	$close
	pop	de
	bool	hl
	ld	l,h
;1069 }
@368:
	.debug	} @368 1069

@367	equ	26
	ld	sp,ix
	pop	ix
	lret
@369	equ	28
;1070 
;1071 
;1072 //-------------------------------------------------------------------------
;1073 //  Save values for all the parameter files
;1074 //-------------------------------------------------------------------------
;1075 void saveAllParams(char *buf, uint bufSize)
;1076 {
$saveAllParams::
	.debug	G $saveAllParams 56 9
	.debug	{ $saveAllParams 56 1075 @381 @380
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-2
	.debug	A buf 5 38
	.debug	A bufSize 7 7
;1077   for (uint i=0; i<NumParamFiles; ++i)
@382:
	.debug	< @382 @383
	.debug	L i -2 7
	.debug	# 1077
	bool	hl
	ld	l,h
	ld	(ix-2),hl
	.seg	FARCODE
@384:
	ld	hl,(ix-2)
	ld	de,(_NumParamFiles)
	or	a,a
	sbc	hl,de
	jr	nc,@385
;1078     saveParams(buf, bufSize, paramFiles[i], ForceFlush);
	.debug	# 1078
	ld	hl,0x1
	push	hl
	ld	hl,(ix-2)
	add	hl,hl
	ld	de,_paramFiles
	add	hl,de
	ld	hl,(hl)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$saveParams
	add	sp,8
;1079 }
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@384
@385:
@383:
	.debug	>
@380:
	.debug	} @380 1079

@379	equ	2
	ld	sp,ix
	pop	ix
	lret
@381	equ	4
;1080 
;1081 
;1082 //-------------------------------------------------------------------------
;1083 //  Flush any caches with data that is newer than the corresponding file
;1084 //-------------------------------------------------------------------------
;1085 void flushAllParams(char *buf, uint bufSize)
;1086 {
$flushAllParams::
	.debug	G $flushAllParams 56 9
	.debug	{ $flushAllParams 56 1085 @389 @388
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-2
	.debug	A buf 5 38
	.debug	A bufSize 7 7
;1087   for (uint i=0; i<NumParamFiles; ++i)
@390:
	.debug	< @390 @391
	.debug	L i -2 7
	.debug	# 1087
	bool	hl
	ld	l,h
	ld	(ix-2),hl
	.seg	FARCODE
@392:
	ld	hl,(ix-2)
	ld	de,(_NumParamFiles)
	or	a,a
	sbc	hl,de
	jr	nc,@393
;1088     flushParamCache(buf, bufSize, paramFiles[i]);
	.debug	# 1088
	ld	hl,(ix-2)
	add	hl,hl
	ld	de,_paramFiles
	add	hl,de
	ld	hl,(hl)
	push	hl
	ld	hl,(ix+7)
	push	hl
	ld	hl,(ix+5)
	push	hl
	lcall	$flushParamCache
	add	sp,6
;1089 }
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@392
@393:
@391:
	.debug	>
@388:
	.debug	} @388 1089

@387	equ	2
	ld	sp,ix
	pop	ix
	lret
@389	equ	4
;1090 
;1091 
;1092 //-------------------------------------------------------------------------
;1093 //  set the initial values for logDay and logMonth
;1094 //-------------------------------------------------------------------------
;1095 void setLogDayMonth(void)
;1096 {
$setLogDayMonth::
	.debug	G $setLogDayMonth 32 9
	.debug	{ $setLogDayMonth 32 1095 @397 @396
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-22
;1097   time_t  dateTime;
	.debug	L dateTime -4 9
;1098   struct tm  tms;
	.debug	L tms -22 34
;1099 
;1100   time(&dateTime);  localtime_r(&dateTime, &tms);
	.debug	# 1100
	ld	hl,-4+@395
	add	hl,sp
	push	hl
	lcall	$time
	pop	hl
	ld	hl,-22+@395
	add	hl,sp
	push	hl
	ld	hl,-2+@395
	add	hl,sp
	push	hl
	lcall	$localtime_r
	add	sp,4
;1101 
;1102   logDay = (uint)tms.tm_mday;  logMonth = (uint) tms.tm_mon + 1;
	.debug	# 1102
	ld	hl,(ix-16)
	ld	(_logDay),hl
	ld	hl,(ix-14)
	inc	hl
	ld	(_logMonth),hl
;1103 }
@396:
	.debug	} @396 1103

@395	equ	22
	ld	sp,ix
	pop	ix
	lret
@397	equ	24
;1104 
;1105 //-------------------------------------------------------------------------
;1106 //  return the current day of the month (1-31) using local time
;1107 //-------------------------------------------------------------------------
;1108 uint  dayOfMonth(void)
;1109 {
$dayOfMonth::
	.debug	G $dayOfMonth 59 9
	.debug	{ $dayOfMonth 59 1108 @400 @399
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-22
;1110   time_t  dateTime;
	.debug	L dateTime -4 9
;1111   struct tm  tms;
	.debug	L tms -22 34
;1112 
;1113   time(&dateTime);  localtime_r(&dateTime, &tms);
	.debug	# 1113
	ld	hl,-4+@398
	add	hl,sp
	push	hl
	lcall	$time
	pop	hl
	ld	hl,-22+@398
	add	hl,sp
	push	hl
	ld	hl,-2+@398
	add	hl,sp
	push	hl
	lcall	$localtime_r
	add	sp,4
;1114 
;1115   return (uint) tms.tm_mday;
	.debug	# 1115
	ld	hl,(ix-16)
;1116 }
@399:
	.debug	} @399 1116

@398	equ	22
	ld	sp,ix
	pop	ix
	lret
@400	equ	24
;1117 
;1118 //-------------------------------------------------------------------------
;1119 //  Copy the specified amount of data to the remote copy of the log file
;1120 //  Returns the # of bytes copied or < 0 if an error
;1121 //-------------------------------------------------------------------------
;1122 void reopenLogFile(char *buf)
;1123 {
$reopenLogFile::
	.debug	G $reopenLogFile 60 9
	.debug	{ $reopenLogFile 60 1122 @403 @402
	push	ix
	ld	ix,0
	add	ix,sp
	add	sp,-89
	.debug	A buf 5 38
;1124   uint  len;
	.debug	L len -2 7
;1125   ulong  hoursOld;
	.debug	L hoursOld -6 9
;1126   bool  err;
	.debug	L err -7 3
;1127   struct timeval  tv;
	.debug	L tv -15 61
;1128   struct stat  finfo;
	.debug	L finfo -57 49
;1129   char  temp[32];
	.debug	L temp -89 62
;1130 
;1131 
;1132   err = false;
	.debug	# 1132
	xor	a,a
	ld	(ix-7),a
;1133 
;1134 //===== if no open file descriptor then open one now =====
;1135   if (logfd < 0)  do  {
	.debug	# 1135
	ld	hl,(_logfd)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jp	nc,@404
;1136   //--- set name for log file ---
;1137     setLogDayMonth();
@408:
	.debug	< @408 @409
	.debug	# 1137
	lcall	$setLogDayMonth
;1138     sprintf(logFileName, "/srfs/%.10s_x%02lX_%02u-%02u.log",
;1139             ApplAbbrev, deviceID, logMonth, logDay);
	.debug	# 1139
	ld	hl,(_logDay)
	push	hl
	ld	hl,(_logMonth)
	push	hl
	ld	de,(_deviceID)
	ld	bc,(_deviceID+2)
	push	bc
	push	de
	ld	hl,_ApplAbbrev
	push	hl
	ld	de,word (phys @9+923)
	ld	c,phys @9+923 >> 16
	push	bc
	push	de
	ld	hl,_logFileName
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,18
;1140 
;1141   //--- if file already exists, and is > 72 hours old, then delete it ---
;1142     if (stat(logFileName, &finfo) == 0)  {
	.debug	# 1142
	ld	hl,-57+@401
	add	hl,sp
	push	hl
	ld	hl,_logFileName
	push	hl
	lcall	$stat
	add	sp,4
	bool	hl
	jr	nz,@410
;1143       hoursOld = (time(NULL) - finfo.st_mtime) / 3600;
@411:
	.debug	< @411 @412
	.debug	# 1143
	bool	hl
	ld	l,h
	push	hl
	lcall	$time
	pop	hl
	push	bc
	push	de
	ld	hl,(ix-23)
	ex	de,hl
	ld	hl,(ix-21)
	ld	bc,hl
	call	__lsub##
	push	bc
	push	de
	ld	de,0xe10
	ld	bc,0
	call	__uldiv##
	ld	hl,bc
	ld	(ix-4),hl
	ex	de,hl
	ld	(ix-6),hl
	ex	de,hl
;1144       if (hoursOld > 72)  {
	.debug	# 1144
	push	bc
	push	de
	ld	de,0x48
	ld	c,d
	ld	b,d
	call	__lsub##
	jr	c,@413
	jr	z,@413
;1145         printf("deleting log file: %s (%ld hours old)\r\n",
@414:
	.debug	< @414 @415
;1146                logFileName, hoursOld);
	.debug	# 1146
	ld	hl,(ix-6)
	ex	de,hl
	ld	hl,(ix-4)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,_logFileName
	push	hl
	ld	de,word (phys @9+956)
	ld	c,phys @9+956 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,10
;1147         unlink(logFileName);
	.debug	# 1147
	ld	hl,_logFileName
	push	hl
	lcall	$unlink
	pop	de
;1148       }
@415:
	.debug	>
;1149     }
@413:
@412:
	.debug	>
;1150 
;1151   //--- open file for appending, creating it if it doesn't exist yet ---
;1152     if ((logfd = open(logFileName, O_WRONLY | O_CREAT | O_APPEND, 0666)) < 
@410:
	.debug	# 1152
	ld	hl,0x1b6
	push	hl
	ld	hl,0x16
	push	hl
	ld	hl,_logFileName
	push	hl
	lcall	$open
	add	sp,6
	ld	(_logfd),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@416
;1153       err = true;  break; }
@417:
	.debug	< @417 @418
	.debug	# 1153
	ld	a,0x1
	ld	(ix-7),a
	jp	@406
@418:
	.debug	>
;1154 
;1155     printf("opened log file: %s\r\n", logFileName);
@416:
	.debug	# 1155
	ld	hl,_logFileName
	push	hl
	ld	de,word (phys @9+996)
	ld	c,phys @9+996 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;1156 
;1157     gettimeofday(&tv, NULL);
	.debug	# 1157
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-13+@401
	add	hl,sp
	push	hl
	lcall	$gettimeofday
	add	sp,4
;1158 
;1159     sprintf(buf,
;1160             "\r\n\n  ======== Module %s opened log file at %s ========\r\n\
;1161             moduleID, dateTimeMS(temp, tv.tv_sec, (uint)(tv.tv_usec/1000)))
	.debug	# 1161
	ld	hl,(ix-11)
	ex	de,hl
	ld	hl,(ix-9)
	ld	bc,hl
	push	bc
	push	de
	ld	de,0x3e8
	ld	bc,0
	call	__ldiv##
	push	de
	ld	hl,(ix-15)
	ex	de,hl
	ld	hl,(ix-13)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,-83+@401
	add	hl,sp
	push	hl
	lcall	$dateTimeMS
	add	sp,8
	push	hl
	ld	hl,_moduleID
	push	hl
	ld	de,word (phys @9+1018)
	ld	c,phys @9+1018 >> 16
	push	bc
	push	de
	ld	hl,(ix+5)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,12
;1162     len = strlen(buf);
	.debug	# 1162
	ld	hl,(ix+5)
	push	hl
	call	_strlen
	pop	de
	ld	(ix-2),hl
;1163 
;1164     if (write(logfd, buf, len) != len)  err = true;
	.debug	# 1164
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,(_logfd)
	push	hl
	lcall	$write
	add	sp,6
	ex	de,hl
	ld	hl,(ix-2)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	z,@419
	ld	(ix-7),0x1
;1165 
;1166     sprintf(buf, "  RTOS: %ls  %ls\r\n", osDTM, osName);  len = strlen(buf)
@419:
	.debug	# 1166
	ld	de,word (phys _osName)
	ld	c,phys _osName >> 16
	push	bc
	push	de
	ld	de,word (phys _osDTM)
	ld	c,phys _osDTM >> 16
	push	bc
	push	de
	ld	de,word (phys @9+1076)
	ld	c,phys @9+1076 >> 16
	push	bc
	push	de
	ld	hl,(ix+5)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,16
	ld	hl,(ix+5)
	push	hl
	call	_strlen
	pop	de
	ld	(ix-2),hl
;1167     if (write(logfd, buf, len) != len)  err = true;
	.debug	# 1167
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,(_logfd)
	push	hl
	lcall	$write
	add	sp,6
	ex	de,hl
	ld	hl,(ix-2)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	z,@420
	ld	(ix-7),0x1
;1168 
;1169     sprintf(buf, "  APPL: %ls  %ls\r\n", applDTM, applName);  len = strlen(
@420:
	.debug	# 1169
	ld	de,word (phys _applName)
	ld	c,phys _applName >> 16
	push	bc
	push	de
	ld	de,word (phys _applDTM)
	ld	c,phys _applDTM >> 16
	push	bc
	push	de
	ld	de,word (phys @9+1095)
	ld	c,phys @9+1095 >> 16
	push	bc
	push	de
	ld	hl,(ix+5)
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,16
	ld	hl,(ix+5)
	push	hl
	call	_strlen
	pop	de
	ld	(ix-2),hl
;1170     if (write(logfd, buf, len) != len)  err = true;
	.debug	# 1170
	push	hl
	ld	hl,(ix+5)
	push	hl
	ld	hl,(_logfd)
	push	hl
	lcall	$write
	add	sp,6
	ex	de,hl
	ld	hl,(ix-2)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	z,@421
	ld	(ix-7),0x1
;1171 
;1172   }  while(0);
@421:
@409:
	.debug	>
	.debug	# 1172
;1173 
;1174 
;1175   if (err)  {
@406:
@404:
	.debug	# 1175
	ld	a,(ix-7)
	or	a,a
	jr	z,@422
;1176     perror(logFileName);
@423:
	.debug	< @423 @424
	.debug	# 1176
	ld	hl,_logFileName
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$perror
	add	sp,4
;1177     if (logfd >= 0)  { close(logfd);  logfd = -1; }
	.debug	# 1177
	ld	hl,(_logfd)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@425
@426:
	.debug	< @426 @427
	ld	hl,(_logfd)
	push	hl
	lcall	$close
	pop	de
	ld	hl,0xffff
	ld	(_logfd),hl
@427:
	.debug	>
;1178     sleepMS(5000);  return;
@425:
	.debug	# 1178
	ld	de,0x1388
	ld	bc,0
	push	bc
	push	de
	lcall	$sleepMS
	add	sp,4
;1179   }
@424:
	.debug	>
;1180 
;1181 }
@422:
@402:
	.debug	} @402 1181

@401	equ	89
	ld	sp,ix
	pop	ix
	lret
@403	equ	91
;1182 
;1183 
;1184 //-------------------------------------------------------------------------
;1185 //  task to keep backup copies of device data up to date, and restore missi
;1186 //  or outdated local copies from backup
;1187 //-------------------------------------------------------------------------
;1188 void _task backupd(int argc, char **argv)
;1189 {
	.align	4
$$backupd::
	ex	(sp),iy
$backupd::
	.debug	G $backupd 63 9
	.debug	{ $backupd 63 1188 @430 @429
	ld	hl,-428
	call	__fent##
	.debug	A argc 5 6
	.debug	A argv 7 64
;1190   uint  ui, idx, dirLen;
	.debug	L ui -2 7
	.debug	L idx -4 7
	.debug	L dirLen -6 7
;1191   uint  toIdx, nextIdx;
	.debug	L toIdx -8 7
	.debug	L nextIdx -10 7
;1192   int  copied;
	.debug	L copied -12 6
;1193   time_t  fileTime;
	.debug	L fileTime -16 9
;1194   ulong  lastBackupCheck, terminateTime;
	.debug	L lastBackupCheck -20 9
	.debug	L terminateTime -24 9
;1195   fd_t  localfd, remotefd, eventsfd;
	.debug	L localfd -26 6
	.debug	L remotefd -28 6
	.debug	L eventsfd -30 6
;1196   sigset_t  sigSet;
	.debug	L sigSet -34 9
;1197   ParamFile  *paramFile;
	.debug	L paramFile -36 39
;1198   char  *errStr, *localName;
	.debug	L errStr -38 38
	.debug	L localName -40 38
;1199   struct stat  localInfo, remoteInfo;
	.debug	L localInfo -82 49
	.debug	L remoteInfo -124 49
;1200   struct utimbuf  fileTimes;
	.debug	L fileTimes -132 65
;1201   EventMsg  eventMsg;
	.debug	L eventMsg -154 58
;1202   struct tm  tms;
	.debug	L tms -172 34
;1203   char  fileName[PATH_MAX], temp[PATH_MAX], dataBuf[128];
	.debug	L fileName -236 66
	.debug	L temp -300 66
	.debug	L dataBuf -428 67
;1204   static uint  fileIdx = 0;
	.dseg
@431:
	.dw	0
	.debug	S fileIdx @431 7
	.seg	FARCODE
;1205 
;1206 
;1207   redirectStdio();  // so printf, perror, etc don't end up in data stream
	.debug	# 1207
	lcall	$redirectStdio
;1208 
;1209 
;1210 //--- block Terminate signal so we can shut down cleanly if needed ---
;1211   sigSet = sigmask(SIGTERM);  sigprocmask(SIG_BLOCK, &sigSet, NULL);
	.debug	# 1211
	ld	de,0x4000
	ld	c,e
	ld	b,e
	ld	hl,bc
	ld	(ix-32),hl
	ex	de,hl
	ld	(ix-34),hl
	ex	de,hl
	push	hl
	ld	hl,-32+@428
	add	hl,sp
	push	hl
	ld	hl,0x1
	push	hl
	lcall	$sigprocmask
	add	sp,6
;1212 
;1213 
;1214 //finish:  insure only one copy of this process can run!
;1215 
;1216   lastTimeChk = lastChgChk = 0;
	.debug	# 1216
	ld	de,0
	ld	c,e
	ld	b,e
	ld	(_lastChgChk),de
	ld	(_lastChgChk+2),bc
	ld	(_lastTimeChk),de
	ld	(_lastTimeChk+2),bc
;1217   timeChkInterval = 2000;  // made much longer after successful query
	.debug	# 1217
	ld	de,0x7d0
	ld	(_timeChkInterval),de
	ld	(_timeChkInterval+2),bc
;1218 
;1219 
;1220   if ((eventsfd = open(EventsFIFO, O_WRONLY, 0)) < 0)  {
	.debug	# 1220
	bool	hl
	ld	l,h
	push	hl
	inc	l
	inc	l
	push	hl
	ld	hl,@68+43
	push	hl
	lcall	$open
	add	sp,6
	ld	(ix-30),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@432
;1221     perror(EventsFIFO);  Exit(-1);  }
@433:
	.debug	< @433 @434
	.debug	# 1221
	ld	de,word (phys @9+1114)
	ld	c,phys @9+1114 >> 16
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
@434:
	.debug	>
;1222 
;1223   strcpy(fileName, "/srfs/");
@432:
	.debug	# 1223
	ld	hl,@68+54
	push	hl
	ld	hl,-234+@428
	add	hl,sp
	push	hl
	call	_strcpy
	add	sp,4
;1224   dirLen = strlen(fileName);  localName = fileName + dirLen;
	.debug	# 1224
	ld	hl,-236+@428
	add	hl,sp
	push	hl
	call	_strlen
	pop	de
	ld	(ix-6),hl
	ex	de,hl
	ld	hl,-236+@428
	add	hl,sp
	add	hl,de
	ld	(ix-40),hl
;1225 
;1226   localfd = remotefd = logfd = -1;  setLogDayMonth();
	.debug	# 1226
	ld	hl,0xffff
	ld	(_logfd),hl
	ld	(ix-28),hl
	ld	(ix-26),hl
	lcall	$setLogDayMonth
;1227 
;1228   lastBackupCheck = lastDateChk = GetMS();  terminateTime = 0;
	.debug	# 1228
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	ld	(_lastDateChk),de
	ld	(_lastDateChk+2),bc
	ld	hl,bc
	ld	(ix-18),hl
	ex	de,hl
	ld	(ix-20),hl
	ex	de,hl
	ld	de,0
	ld	c,e
	ld	b,e
	ld	hl,bc
	ld	(ix-22),hl
	ex	de,hl
	ld	(ix-24),hl
	ex	de,hl
;1229 
;1230 
;1231 //===== several chores to tend to in a loop =====
;1232   for (;;)  {
@435:
	.debug	< @435 @436
@437:
@439:
;1233 
;1234   //--- check for pending Terminate signal ---
;1235     if (sigpending(&sigSet) == 0)  if (sigismember(&sigSet, SIGTERM))
@440:
	.debug	< @440 @441
	.debug	# 1235
	ld	hl,-34+@428
	add	hl,sp
	push	hl
	lcall	$sigpending
	pop	de
	bool	hl
	jp	nz,@442
	ld	l,0xf
	push	hl
	ld	hl,-32+@428
	add	hl,sp
	push	hl
	lcall	$sigismember
	add	sp,4
	bool	hl
	jp	z,@443
;1236       if (! terminateTime)  {
	.debug	# 1236
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	ld	bc,hl
	or	hl,de
	jp	nz,@444
;1237         printf("cleaning up...\r\n");
@445:
	.debug	< @445 @446
	.debug	# 1237
	ld	de,word (phys @9+1125)
	ld	c,phys @9+1125 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;1238         terminateTime = GetMS();
	.debug	# 1238
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	ld	hl,bc
	ld	(ix-22),hl
	ex	de,hl
	ld	(ix-24),hl
	ex	de,hl
;1239         flushAllParams(dataBuf, sizeof(dataBuf));
	.debug	# 1239
	ld	hl,0x80
	push	hl
	ld	hl,-426+@428
	add	hl,sp
	push	hl
	lcall	$flushAllParams
	add	sp,4
;1240       }
@446:
	.debug	>
;1241 
;1242   //--- if terminate signal rcvd, wait 5 secs to allow log data to be saved
;1243     if (terminateTime AND (msSince(terminateTime) > 5000))  {
@444:
@443:
@442:
	.debug	# 1243
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	ld	bc,hl
	or	hl,de
	jp	z,@448
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	push	bc
	push	de
	ld	hl,(ix-24)
	ex	de,hl
	ld	hl,(ix-22)
	ld	bc,hl
	call	__lsub##
	push	bc
	push	de
	ld	de,0x1388
	ld	bc,0
	call	__lsub##
	jr	c,@448
	jr	z,@448
;1244       if (eventsfd >= 0)  close(eventsfd);
@449:
	.debug	< @449 @450
	.debug	# 1244
	ld	hl,(ix-30)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@451
	ld	hl,(ix-30)
	push	hl
	lcall	$close
	pop	de
;1245       if (logfd >= 0)  close(logfd);
@451:
	.debug	# 1245
	ld	hl,(_logfd)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@452
	ld	hl,(_logfd)
	push	hl
	lcall	$close
	pop	de
;1246       if (localfd >= 0)  close(localfd);
@452:
	.debug	# 1246
	ld	hl,(ix-26)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@453
	ld	hl,(ix-26)
	push	hl
	lcall	$close
	pop	de
;1247       if (remotefd >= 0)  close(remotefd);
@453:
	.debug	# 1247
	ld	hl,(ix-28)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@454
	ld	hl,(ix-28)
	push	hl
	lcall	$close
	pop	de
;1248       Exit(-1);
@454:
	.debug	# 1248
	ld	hl,0xffff
	push	hl
	lcall	$Exit
	pop	hl
;1249     }
@450:
	.debug	>
;1250 
;1251     sleepMS(20);
@448:
	.debug	# 1251
	ld	de,0x14
	ld	c,d
	ld	b,d
	push	bc
	push	de
	lcall	$sleepMS
	add	sp,4
;1252 
;1253   //--- periodically get time from SRFS server ---
;1254     getTime();
	.debug	# 1254
	lcall	$getTime
;1255 
;1256   //--- check for changes to persistent values ---
;1257     fileIdx = (fileIdx + 1) % NumParamFiles;
	.debug	# 1257
	ld	hl,(@431)
	inc	hl
	ld	de,(_NumParamFiles)
	call	__uidiv##
	ld	(@431),de
;1258     checkCache(dataBuf, sizeof(dataBuf), paramFiles[fileIdx]);
	.debug	# 1258
	ld	h,d
	ld	l,e
	add	hl,hl
	ld	de,_paramFiles
	add	hl,de
	ld	hl,(hl)
	push	hl
	ld	hl,0x80
	push	hl
	ld	hl,-424+@428
	add	hl,sp
	push	hl
	lcall	$checkCache
	add	sp,6
;1259 
;1260 
;1261   //================================================
;1262   //===== keep a remote copy of syslog entries =====
;1263   //================================================
;1264 
;1265     if (gotTime)  {
	.debug	# 1265
	ld	a,(_gotTime)
	or	a,a
	jp	z,@455
;1266 
;1267     //--- if date changed, then close the current log file ---
;1268       if (msSince(lastDateChk) >= 500)  {
@456:
	.debug	< @456 @457
	.debug	# 1268
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	push	bc
	push	de
	ld	de,(_lastDateChk)
	ld	bc,(_lastDateChk+2)
	call	__lsub##
	push	bc
	push	de
	ld	de,0x1f4
	ld	bc,0
	call	__lsub##
	jp	c,@458
;1269         if (logDay != dayOfMonth())  {
@459:
	.debug	< @459 @460
	.debug	# 1269
	lcall	$dayOfMonth
	ld	de,(_logDay)
	or	a,a
	sbc	hl,de
	jr	z,@461
;1270           printf("***** new day *****\n");
@462:
	.debug	< @462 @463
	.debug	# 1270
	ld	de,word (phys @9+1142)
	ld	c,phys @9+1142 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;1271           if (logfd >= 0)  { close(logfd);  logfd = -1; }
	.debug	# 1271
	ld	hl,(_logfd)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@464
@465:
	.debug	< @465 @466
	ld	hl,(_logfd)
	push	hl
	lcall	$close
	pop	de
	ld	hl,0xffff
	ld	(_logfd),hl
@466:
	.debug	>
;1272         }
@464:
@463:
	.debug	>
;1273         lastDateChk = GetMS();
@461:
	.debug	# 1273
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	ld	(_lastDateChk),de
	ld	(_lastDateChk+2),bc
;1274       }
@460:
	.debug	>
;1275 
;1276     //--- reopen log file if needed ---
;1277       if (logfd < 0)  reopenLogFile(dataBuf);
@458:
	.debug	# 1277
	ld	hl,(_logfd)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@467
	ld	hl,-428+@428
	add	hl,sp
	push	hl
	lcall	$reopenLogFile
	pop	hl
;1278 
;1279     //--- copy any new data in the syslog buffer to a remote file ---
;1280       if (logfd >= 0)  {
@467:
	.debug	# 1280
	ld	hl,(_logfd)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jp	c,@468
;1281         _EnterCritical_();
@469:
	.debug	< @469 @470
	.debug	# 1281
	ipset	2
;1282         toIdx = diagLogInfo->toIdx;  nextIdx = diagLogInfo->fromIdx;
	.debug	# 1282
	ld	de,(_diagLogInfo)
	ld	bc,(_diagLogInfo+2)
	push	bc
	push	de
	ld	de,0x8
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-8),hl
	ld	de,(_diagLogInfo)
	ld	bc,(_diagLogInfo+2)
	push	bc
	push	de
	ld	de,0xa
	ld	c,d
	ld	b,d
	call	__ladd##
	call	__farld2##
	ex	de,hl
	ld	(ix-10),hl
;1283         _ExitCritical_();
	.debug	# 1283
	ipres
;1284         for (ui=0; (nextIdx!=toIdx) AND (ui<sizeof(dataBuf)); ++ui)  {
@471:
	.debug	< @471 @472
	.debug	# 1284
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@473:
	ld	hl,(ix-10)
	ex	de,hl
	ld	hl,(ix-8)
	ex	de,hl
	or	a,a
	sbc	hl,de
	jr	z,@475
	ld	hl,(ix-2)
	ld	de,0x80
	or	a,a
	sbc	hl,de
	jr	nc,@475
;1285           dataBuf[ui] = diagLogBuf[nextIdx];
@478:
	.debug	< @478 @479
	.debug	# 1285
	ld	hl,(ix-10)
	ex	de,hl
	ld	b,h
	ld	c,h
	push	bc
	push	de
	ld	de,(_diagLogBuf)
	ld	bc,(_diagLogBuf+2)
	call	__ladd##
	ld	hl,de
	ld	a,c
	ldp	hl,(hl)
	ld	a,l
	ld	hl,(ix-2)
	ex	de,hl
	ld	hl,-428+@428
	add	hl,sp
	add	hl,de
	ld	(hl),a
;1286           if (++nextIdx == DiagLogSize)  nextIdx = 0;
	.debug	# 1286
	ld	hl,(ix-10)
	inc	hl
	ld	(ix-10),hl
	ld	de,(_DiagLogSize)
	or	a,a
	sbc	hl,de
	jr	nz,@480
	bool	hl
	ld	l,h
	ld	(ix-10),hl
;1287         }
@480:
@479:
	.debug	>
;1288         if (ui)  {
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@473
@475:
@472:
	.debug	>
	.debug	# 1288
	ld	hl,(ix-2)
	bool	hl
	jp	z,@481
;1289           copied = write(logfd, dataBuf, ui);
@482:
	.debug	< @482 @483
	.debug	# 1289
	ld	hl,(ix-2)
	push	hl
	ld	hl,-426+@428
	add	hl,sp
	push	hl
	ld	hl,(_logfd)
	push	hl
	lcall	$write
	add	sp,6
	ld	(ix-12),hl
;1290           if (copied < 0)  {
	.debug	# 1290
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@484
;1291             perror(logFileName);  close(logfd);  logfd = -1;  }
@485:
	.debug	< @485 @486
	.debug	# 1291
	ld	hl,_logFileName
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	hl,(_logfd)
	push	hl
	lcall	$close
	pop	de
	ld	hl,0xffff
	ld	(_logfd),hl
@486:
	.debug	>
;1292           if (copied > 0)  {
@484:
	.debug	# 1292
	ld	hl,(ix-12)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@487
	jr	z,@487
;1293             KLOCK();
@488:
	.debug	< @488 @489
	.debug	# 1293
	call	_KLOCK
;1294             if ((diagLogInfo->fromIdx += copied) >= DiagLogSize)
	.debug	# 1294
	ld	de,(_diagLogInfo)
	ld	bc,(_diagLogInfo+2)
	push	bc
	push	de
	ld	de,0xa
	ld	c,d
	ld	b,d
	call	__ladd##
	push	bc
	push	de
	call	__farld2##
	ld	hl,(ix-12)
	add	hl,de
	pop	de
	pop	bc
	call	__farst2##
	ld	de,(_DiagLogSize)
	or	a,a
	sbc	hl,de
	jr	c,@490
;1295               diagLogInfo->fromIdx -= DiagLogSize;
	.debug	# 1295
	ld	de,(_diagLogInfo)
	ld	bc,(_diagLogInfo+2)
	push	bc
	push	de
	ld	de,0xa
	ld	c,d
	ld	b,d
	call	__ladd##
	push	bc
	push	de
	call	__farld2##
	ld	hl,(_DiagLogSize)
	ex	de,hl
	or	a,a
	sbc	hl,de
	pop	de
	pop	bc
	call	__farst2##
;1296             KUNLOCK();
@490:
	.debug	# 1296
	call	_KUNLOCK
;1297           }
@489:
	.debug	>
;1298         }
@487:
@483:
	.debug	>
;1299       }
@481:
@470:
	.debug	>
;1300 
;1301     }  // end: if (gotTime)
@468:
@457:
	.debug	>
;1302 
;1303 
;1304   //=======================================================================
;1305   //===== automatically backup or restore the device configuration file ===
;1306   //===== as needed =====
;1307   //=======================================================================
;1308     if (msSince(lastBackupCheck) < 30000)  continue;
@455:
	.debug	# 1308
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	push	bc
	push	de
	ld	hl,(ix-20)
	ex	de,hl
	ld	hl,(ix-18)
	ld	bc,hl
	call	__lsub##
	push	bc
	push	de
	ld	de,0x7530
	ld	bc,0
	call	__lsub##
	jp	c,@439
;1309 
;1310     lastBackupCheck = GetMS();
	.debug	# 1310
ipset 3
	ld	de,(__TimerMilliSeconds+4)
	ld	bc,(__TimerMilliSeconds+6)
ipres
	ld	hl,bc
	ld	(ix-18),hl
	ex	de,hl
	ld	(ix-20),hl
	ex	de,hl
;1311 
;1312     for (idx=0; idx<NumParamFiles; ++idx)  {
@492:
	.debug	< @492 @493
	.debug	# 1312
	bool	hl
	ld	l,h
	ld	(ix-4),hl
@494:
	ld	hl,(ix-4)
	ld	de,(_NumParamFiles)
	or	a,a
	sbc	hl,de
	jp	nc,@495
;1313 
;1314     //--- close any open file descriptors ---
;1315       if (localfd >= 0)  { close(localfd);  localfd = -1; }
@497:
	.debug	< @497 @498
	.debug	# 1315
	ld	hl,(ix-26)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@499
@500:
	.debug	< @500 @501
	ld	hl,(ix-26)
	push	hl
	lcall	$close
	pop	de
	ld	hl,0xffff
	ld	(ix-26),hl
@501:
	.debug	>
;1316       if (remotefd >= 0)  { close(remotefd);  remotefd = -1; }
@499:
	.debug	# 1316
	ld	hl,(ix-28)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@502
@503:
	.debug	< @503 @504
	ld	hl,(ix-28)
	push	hl
	lcall	$close
	pop	de
	ld	hl,0xffff
	ld	(ix-28),hl
@504:
	.debug	>
;1317 
;1318       paramFile = paramFiles[idx];
@502:
	.debug	# 1318
	ld	hl,(ix-4)
	add	hl,hl
	ld	de,_paramFiles
	add	hl,de
	ld	hl,(hl)
	ld	(ix-36),hl
;1319 
;1320 
;1321     //--- For device-specific data, compare the timestamp on the local file
;1322     //--- with the remote backup copy.  If one is newer than the other, ---
;1323     //--- then update the one that is older ---
;1324       if (paramFile->category != PDC_DEV)  continue;
	.debug	# 1324
	inc	hl
	inc	hl
	ld	a,(hl)
	sub	a,3
	jp	nz,@496
;1325 
;1326     //--- generate unique name using base name, device type, and device id 
;1327       paramFileName(paramFile, localName, sizeof(fileName)-dirLen);
	.debug	# 1327
	ld	hl,0x40
	ex	de,hl
	ld	hl,(ix-6)
	ex	de,hl
	or	a,a
	sbc	hl,de
	push	hl
	ld	hl,(ix-40)
	push	hl
	ld	hl,(ix-36)
	push	hl
	lcall	$paramFileName
	add	sp,6
;1328 
;1329     //--- get date/time and size info for local and remote copy of the file
;1330       memset(&localInfo, 0, sizeof(localInfo));
	.debug	# 1330
	ld	hl,0x2a
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-78+@428
	add	hl,sp
	push	hl
	call	_memset
	add	sp,6
;1331       memset(&remoteInfo, 0, sizeof(remoteInfo));
	.debug	# 1331
	ld	hl,0x2a
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-120+@428
	add	hl,sp
	push	hl
	call	_memset
	add	sp,6
;1332 
;1333       if ((localfd = open(localName, O_RDONLY, 0)) >= 0)
	.debug	# 1333
	bool	hl
	ld	l,h
	push	hl
	inc	l
	push	hl
	ld	hl,(ix-40)
	push	hl
	lcall	$open
	add	sp,6
	ld	(ix-26),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@506
;1334         if (fstat(localfd, &localInfo))  {
	.debug	# 1334
	ld	hl,-82+@428
	add	hl,sp
	push	hl
	ld	hl,(ix-26)
	push	hl
	lcall	$fstat
	add	sp,4
	bool	hl
	jp	nz,@566
;1335           perror(localName);  sleepMS(5000);  continue; }
@508:
	.debug	< @508 @509
	.debug	# 1335
@509:
	.debug	>
;1336 
;1337       if ((remotefd = open(fileName, O_RDONLY, 0)) >= 0)  {
@506:
	.debug	# 1337
	bool	hl
	ld	l,h
	push	hl
	inc	l
	push	hl
	ld	hl,-232+@428
	add	hl,sp
	push	hl
	lcall	$open
	add	sp,6
	ld	(ix-28),hl
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@510
;1338         if (fstat(remotefd, &remoteInfo))  {
@511:
	.debug	< @511 @512
	.debug	# 1338
	ld	hl,-124+@428
	add	hl,sp
	push	hl
	ld	hl,(ix-28)
	push	hl
	lcall	$fstat
	add	sp,4
	bool	hl
	jr	z,@516
;1339           perror(fileName);  sleepMS(5000);  continue; } }
@514:
	.debug	< @514 @515
	.debug	# 1339
	jp	@565
@515:
	.debug	>
@512:
	.debug	>
;1340       else
;1341         if (errno != ENOENT)  { perror(fileName);  sleepMS(5000);  continue
@510:
	.debug	# 1341
	ld	hl,(_current)
	ld	de,0xb9
	add	hl,de
	ld	hl,(hl)
	ld	e,0x2
	or	a,a
	sbc	hl,de
	jp	nz,@565
@518:
	.debug	< @518 @519
@519:
	.debug	>
;1342 
;1343       if (ShowDebugMsgs)  {
@516:
	.debug	# 1343
	ld	hl,(_current)
	ld	de,0xae
	add	hl,de
	ld	hl,(hl)
	ld	de,0x800
	and	hl,de
	jr	z,@520
;1344         printf("Checking file: %s\r\n", localName);
@521:
	.debug	< @521 @522
	.debug	# 1344
	ld	hl,(ix-40)
	push	hl
	ld	de,word (phys @9+1163)
	ld	c,phys @9+1163 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;1345         printf("  local:  fd: %d, mtime: %lu  size: %lu\r\n",
;1346                localfd, localInfo.st_mtime, localInfo.st_size);
	.debug	# 1346
	ld	hl,(ix-60)
	ex	de,hl
	ld	hl,(ix-58)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-48)
	ex	de,hl
	ld	hl,(ix-46)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-26)
	push	hl
	ld	de,word (phys @9+1183)
	ld	c,phys @9+1183 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,14
;1347         printf("  remote: fd: %d, mtime: %lu  size: %lu\r\n",
;1348                remotefd, remoteInfo.st_mtime, remoteInfo.st_size);
	.debug	# 1348
	ld	hl,(ix-102)
	ex	de,hl
	ld	hl,(ix-100)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-90)
	ex	de,hl
	ld	hl,(ix-88)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-28)
	push	hl
	ld	de,word (phys @9+1225)
	ld	c,phys @9+1225 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,14
;1349       }
@522:
	.debug	>
;1350 
;1351 
;1352     //=== if backup copy of device-specific data is newer, then restore ===
;1353     //=== the backup copy ===
;1354       if ((remotefd >= 0) AND (localInfo.st_mtime < remoteInfo.st_mtime))  
@520:
	.debug	# 1354
	ld	hl,(ix-28)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jp	c,@524
	ld	hl,(ix-48)
	ex	de,hl
	ld	hl,(ix-46)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-90)
	ex	de,hl
	ld	hl,(ix-88)
	ld	bc,hl
	call	__lsub##
	jp	nc,@524
;1355         printf("restore from backup: %s\r\n", fileName);
@525:
	.debug	< @525 @526
	.debug	# 1355
	ld	hl,-236+@428
	add	hl,sp
	push	hl
	ld	de,word (phys @9+1267)
	ld	c,phys @9+1267 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;1356         if (localfd >= 0)
	.debug	# 1356
	ld	hl,(ix-26)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@527
;1357           printf("  local:  %.24s  size: %lu\r\n",
;1358                  ctime_r(&localInfo.st_mtime, dataBuf), localInfo.st_size);
	.debug	# 1358
	ld	hl,(ix-60)
	ex	de,hl
	ld	hl,(ix-58)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,-424+@428
	add	hl,sp
	push	hl
	ld	hl,-42+@428
	add	hl,sp
	push	hl
	lcall	$ctime_r
	add	sp,4
	push	hl
	ld	de,word (phys @9+1293)
	ld	c,phys @9+1293 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,10
;1359         printf(" remote:  %.24s  size: %lu\r\n",
@527:
;1360                ctime_r(&remoteInfo.st_mtime, dataBuf), remoteInfo.st_size);
	.debug	# 1360
	ld	hl,(ix-102)
	ex	de,hl
	ld	hl,(ix-100)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,-424+@428
	add	hl,sp
	push	hl
	ld	hl,-84+@428
	add	hl,sp
	push	hl
	lcall	$ctime_r
	add	sp,4
	push	hl
	ld	de,word (phys @9+1322)
	ld	c,phys @9+1322 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,10
;1361 
;1362       //--- delete any old local archive ---
;1363         strcpy(temp, localName);  strcat(temp, ".bak");  unlink(temp);
	.debug	# 1363
	ld	hl,(ix-40)
	push	hl
	ld	hl,-298+@428
	add	hl,sp
	push	hl
	call	_strcpy
	add	sp,4
	ld	hl,@68+61
	push	hl
	ld	hl,-298+@428
	add	hl,sp
	push	hl
	call	_strcat
	add	sp,4
	ld	hl,-300+@428
	add	hl,sp
	push	hl
	lcall	$unlink
	pop	de
;1364 
;1365       //--- rename any existing local as a backup ---
;1366         if (localfd >= 0)
	.debug	# 1366
	ld	hl,(ix-26)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@528
;1367           if (rename(localName, temp))  {
	.debug	# 1367
	ld	hl,-300+@428
	add	hl,sp
	push	hl
	ld	hl,(ix-40)
	push	hl
	lcall	$rename
	add	sp,4
	bool	hl
	jr	z,@529
;1368             perror(localName); sleepMS(5000); continue; }
@530:
	.debug	< @530 @531
	.debug	# 1368
@566:
	ld	hl,(ix-40)
	jp	@563
@531:
	.debug	>
;1369 
;1370       //--- copy remote-backup to new local file ---
;1371       if (copyFileByName(fileName, localName, false,
@529:
@528:
;1372                          dataBuf, sizeof(dataBuf), &errStr))  {
	.debug	# 1372
	ld	hl,-38+@428
	add	hl,sp
	push	hl
	ld	hl,0x80
	push	hl
	ld	hl,-424+@428
	add	hl,sp
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,(ix-40)
	push	hl
	ld	hl,-226+@428
	add	hl,sp
	push	hl
	lcall	$copyFileByName
	add	sp,12
	bool	hl
	jp	nz,@564
;1373         perror(errStr);  sleepMS(5000);  continue; }
@533:
	.debug	< @533 @534
	.debug	# 1373
@534:
	.debug	>
;1374 
;1375       //--- set date/time on restored file to match backup copy ---
;1376         fileTimes.actime = fileTimes.modtime = remoteInfo.st_mtime;
	.debug	# 1376
	ld	hl,(ix-90)
	ex	de,hl
	ld	hl,(ix-88)
	ld	bc,hl
	ld	hl,-128+@428
	add	hl,sp
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	hl,-132+@428
	add	hl,sp
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;1377         utime(localName, &fileTimes);
	.debug	# 1377
	ld	hl,-132+@428
	add	hl,sp
	push	hl
	ld	hl,(ix-40)
	push	hl
	lcall	$utime
	add	sp,4
;1378 
;1379       //--- send restored-param-file msg to applic process ---
;1380         *(uint *)(eventMsg.data) = idx;
	.debug	# 1380
	ld	hl,(ix-4)
	ex	de,hl
	ld	hl,-148+@428
	add	hl,sp
	ld	(hl),e
	inc	hl
	ld	(hl),d
;1381         EVENT(RestoredParamFile, 0, NULL, sizeof(uint));
	.debug	# 1381
	ld	hl,0x2
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-150+@428
	add	hl,sp
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,_EVENT_RestoredParamFile
	push	hl
	ld	hl,(ix-30)
	push	hl
	lcall	$putMsg
	add	sp,12
;1382 
;1383         continue;
	.debug	# 1383
;1384       }
	jp	@496
@526:
	.debug	>
;1385 
;1386 
;1387     //=== if local copy of parameter data is newer, then update the ===
;1388     //=== remote backup copy ===
;1389       if (localfd >= 0)  if (localInfo.st_mtime > remoteInfo.st_mtime)  {
@524:
	.debug	# 1389
	ld	hl,(ix-26)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jp	c,@535
	ld	hl,(ix-48)
	ex	de,hl
	ld	hl,(ix-46)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,(ix-90)
	ex	de,hl
	ld	hl,(ix-88)
	ld	bc,hl
	call	__lsub##
	jp	c,@536
	jp	z,@536
;1390         printf("update backup copy: %s\r\n", fileName);
@537:
	.debug	< @537 @538
	.debug	# 1390
	ld	hl,-236+@428
	add	hl,sp
	push	hl
	ld	de,word (phys @9+1351)
	ld	c,phys @9+1351 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,6
;1391         printf("  local:  %.24s  size: %lu\r\n",
;1392                ctime_r(&localInfo.st_mtime, dataBuf), localInfo.st_size);
	.debug	# 1392
	ld	hl,(ix-60)
	ex	de,hl
	ld	hl,(ix-58)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,-424+@428
	add	hl,sp
	push	hl
	ld	hl,-42+@428
	add	hl,sp
	push	hl
	lcall	$ctime_r
	add	sp,4
	push	hl
	ld	de,word (phys @9+1293)
	ld	c,phys @9+1293 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,10
;1393         if (remotefd >= 0)
	.debug	# 1393
	ld	hl,(ix-28)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@539
;1394           printf("  remote: %.24s  size: %lu\r\n",
;1395                  ctime_r(&remoteInfo.st_mtime, dataBuf), remoteInfo.st_size
	.debug	# 1395
	ld	hl,(ix-102)
	ex	de,hl
	ld	hl,(ix-100)
	ld	bc,hl
	push	bc
	push	de
	ld	hl,-424+@428
	add	hl,sp
	push	hl
	ld	hl,-84+@428
	add	hl,sp
	push	hl
	lcall	$ctime_r
	add	sp,4
	push	hl
	ld	de,word (phys @9+1376)
	ld	c,phys @9+1376 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,10
;1396 
;1397       //--- rename remote-backup copy if one exists ---
;1398         if (remotefd >= 0)  {
@539:
	.debug	# 1398
	ld	hl,(ix-28)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jp	c,@540
;1399           fileTime = remoteInfo.st_mtime;  fileTime += (long)(timeZone * 36
@541:
	.debug	< @541 @542
	.debug	# 1399
	ld	hl,(ix-90)
	ex	de,hl
	ld	hl,(ix-88)
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
	ld	hl,(_timeZone)
	ld	bc,0xe10
	ex	de,hl
	mul
	ld	de,bc
	ld	a,d
	rlca
	sbc	a,a
	ld	c,a
	ld	b,a
	push	bc
	push	de
	ld	hl,(ix-16)
	ex	de,hl
	ld	hl,(ix-14)
	ld	bc,hl
	call	__ladd##
	ld	hl,bc
	ld	(ix-14),hl
	ex	de,hl
	ld	(ix-16),hl
	ex	de,hl
;1400           gmtime_r(&fileTime, &tms);
	.debug	# 1400
	ld	hl,-172+@428
	add	hl,sp
	push	hl
	ld	hl,-14+@428
	add	hl,sp
	push	hl
	lcall	$gmtime_r
	add	sp,4
;1401           strcpy(temp, fileName);
	.debug	# 1401
	ld	hl,-236+@428
	add	hl,sp
	push	hl
	ld	hl,-298+@428
	add	hl,sp
	push	hl
	call	_strcpy
	add	sp,4
;1402           sprintf(temp+strlen(temp), "_%04d-%02d-%02d_%02d%02d%02d",
;1403                   tms.tm_year+1900, tms.tm_mon+1, tms.tm_mday,
;1404                   tms.tm_hour, tms.tm_min, tms.tm_sec);
	.debug	# 1404
	ld	hl,-172+@428
	add	hl,sp
	ld	hl,(hl)
	push	hl
	ld	hl,-168+@428
	add	hl,sp
	ld	hl,(hl)
	push	hl
	ld	hl,-164+@428
	add	hl,sp
	ld	hl,(hl)
	push	hl
	ld	hl,-160+@428
	add	hl,sp
	ld	hl,(hl)
	push	hl
	ld	hl,-156+@428
	add	hl,sp
	ld	hl,(hl)
	inc	hl
	push	hl
	ld	hl,-152+@428
	add	hl,sp
	ld	hl,(hl)
	ld	de,0x76c
	add	hl,de
	push	hl
	ld	de,word (phys @9+1405)
	ld	c,phys @9+1405 >> 16
	push	bc
	push	de
	ld	hl,-284+@428
	add	hl,sp
	push	hl
	call	_strlen
	pop	de
	ex	de,hl
	ld	hl,-284+@428
	add	hl,sp
	add	hl,de
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$farsprintf
	add	sp,20
;1405           if (rename(fileName, temp) < 0)  {
	.debug	# 1405
	ld	hl,-300+@428
	add	hl,sp
	push	hl
	ld	hl,-234+@428
	add	hl,sp
	push	hl
	lcall	$rename
	add	sp,4
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	nc,@543
;1406             perror(fileName);  sleepMS(5000);  continue; }
@544:
	.debug	< @544 @545
	.debug	# 1406
@565:
	ld	hl,-236+@428
	add	hl,sp
	jr	@563
@545:
	.debug	>
;1407         }
@543:
@542:
	.debug	>
;1408 
;1409       //--- copy local to new remote-backup ---
;1410         if (copyFileByName(localName, fileName, false,
@540:
;1411                            dataBuf, sizeof(dataBuf), &errStr))  {
	.debug	# 1411
	ld	hl,-38+@428
	add	hl,sp
	push	hl
	ld	hl,0x80
	push	hl
	ld	hl,-424+@428
	add	hl,sp
	push	hl
	bool	hl
	ld	l,h
	push	hl
	ld	hl,-228+@428
	add	hl,sp
	push	hl
	ld	hl,(ix-40)
	push	hl
	lcall	$copyFileByName
	add	sp,12
	bool	hl
	jr	z,@546
;1412           printf("copyFileByName(): ");
@547:
	.debug	< @547 @548
	.debug	# 1412
	ld	de,word (phys @9+1434)
	ld	c,phys @9+1434 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
;1413           perror(errStr);  sleepMS(5000);  continue;
	.debug	# 1413
@564:
	ld	hl,(ix-38)
@563:
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	de,0x1388
	ld	bc,0
	push	bc
	push	de
	lcall	$sleepMS
	add	sp,4
;1414         }
	jp	@496
@548:
	.debug	>
;1415 
;1416       //--- get date/time for new backup copy and set local to match ---
;1417         remoteInfo.st_mtime = fileTimes.modtime;
@546:
	.debug	# 1417
	ld	hl,-128+@428
	add	hl,sp
	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	ld	c,(hl)
	inc	hl
	ld	b,(hl)
	ld	hl,bc
	ld	(ix-88),hl
	ex	de,hl
	ld	(ix-90),hl
	ex	de,hl
;1418         for (ui=0; ui<10; ++ui)  {
@549:
	.debug	< @549 @550
	.debug	# 1418
	bool	hl
	ld	l,h
	ld	(ix-2),hl
@551:
	ld	hl,(ix-2)
	ld	de,0xa
	or	a,a
	sbc	hl,de
	jr	nc,@552
;1419           if (stat(fileName, &remoteInfo) == 0)  break;
@554:
	.debug	< @554 @555
	.debug	# 1419
	ld	hl,-124+@428
	add	hl,sp
	push	hl
	ld	hl,-234+@428
	add	hl,sp
	push	hl
	lcall	$stat
	add	sp,4
	bool	hl
	jr	z,@552
;1420           printf("stat(): ");  perror(fileName);  sleepMS(1000);
	.debug	# 1420
	ld	de,word (phys @9+1453)
	ld	c,phys @9+1453 >> 16
	push	bc
	push	de
	lcall	$farprintf
	add	sp,4
	ld	hl,-236+@428
	add	hl,sp
	ex	de,hl
	call	__farcvp##
	push	bc
	push	de
	lcall	$perror
	add	sp,4
	ld	de,0x3e8
	ld	bc,0
	push	bc
	push	de
	lcall	$sleepMS
	add	sp,4
;1421         }
@555:
	.debug	>
;1422         fileTimes.actime = fileTimes.modtime = remoteInfo.st_mtime;
	ld	hl,(ix-2)
	inc	hl
	ld	(ix-2),hl
	jr	@551
@552:
@550:
	.debug	>
	.debug	# 1422
	ld	hl,(ix-90)
	ex	de,hl
	ld	hl,(ix-88)
	ld	bc,hl
	ld	hl,-128+@428
	add	hl,sp
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
	ld	hl,-132+@428
	add	hl,sp
	ld	(hl),e
	inc	hl
	ld	(hl),d
	inc	hl
	ld	(hl),c
	inc	hl
	ld	(hl),b
;1423         utime(localName, &fileTimes);
	.debug	# 1423
	ld	hl,-132+@428
	add	hl,sp
	push	hl
	ld	hl,(ix-40)
	push	hl
	lcall	$utime
	add	sp,4
;1424 
;1425       //--- "thin-out" remote archive ---
;1426         //finish: don't let # remote backup files expand forever...
;1427         continue;
	.debug	# 1427
;1428       }
@538:
	.debug	>
;1429 
;1430     }
@536:
@535:
@498:
	.debug	>
;1431 
;1432     if (localfd >= 0)  { close(localfd);  localfd = -1; }
@496:
	ld	hl,(ix-4)
	inc	hl
	ld	(ix-4),hl
	jp	@494
@495:
@493:
	.debug	>
	.debug	# 1432
	ld	hl,(ix-26)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@557
@558:
	.debug	< @558 @559
	ld	hl,(ix-26)
	push	hl
	lcall	$close
	pop	de
	ld	hl,0xffff
	ld	(ix-26),hl
@559:
	.debug	>
;1433     if (remotefd >= 0)  { close(remotefd);  remotefd = -1; }
@557:
	.debug	# 1433
	ld	hl,(ix-28)
	ld	de,0x8000
	add	hl,hl
	ccf
	rr	hl
	sbc	hl,de
	jr	c,@560
@561:
	.debug	< @561 @562
	ld	hl,(ix-28)
	push	hl
	lcall	$close
	pop	de
	ld	hl,0xffff
	ld	(ix-28),hl
@562:
	.debug	>
;1434   }
@560:
@441:
	.debug	>
;1435 
;1436 }
	jp	@437
@436:
	.debug	>
@429:
	.debug	} @429 1436

@428	equ	428
	ld	sp,ix
	pop	ix
	lret
@430	equ	430
;1437 
;1438 //-------------------------------------------------------------------------
	.extern	$time 68
	.extern	_memset 69
	.extern	_strcat 71
	.extern	_strcpy 71
	.extern	_strlen 72
	.extern	_fmemcmp 73
	.extern	$fmemcpy 74
	.extern	_fmemset 76
	.extern	$redirectStdio 57
	.extern	$dateTimeMS 77
	.extern	$ctime_r 78
	.extern	$gmtime_r 79
	.extern	$localtime_r 79
	.extern	$addShort 81
	.extern	$getShort 82
	.extern	$memCRC16 83
	.extern	$fstrncat 84
	.extern	$skipBytes 85
	.extern	$fwrite_ 86
	.extern	$copyFileByName 87
	.extern	$putMsg 88
	.extern	$sigismember 89
	.extern	$sigprocmask 90
	.extern	$sigpending 91
	.extern	_current 92
	.extern	$farfprintf 94
	.extern	$farprintf 95
	.extern	$farsprintf 96
	.extern	__TimerSeconds 9
	.extern	__TimerMilliSeconds 97
	.extern	_diagLogInfo 98
	.extern	_DiagLogSize 7
	.extern	_diagLogBuf 46
	.extern	$open 100
	.extern	$close 101
	.extern	$read 102
	.extern	$write 102
	.extern	$unlink 72
	.extern	$rename 103
	.extern	_KLOCK 32
	.extern	_KUNLOCK 32
	.extern	_osName 66
	.extern	_osDTM 62
	.extern	_gotTime 3
	.extern	_diagLog 104
	.extern	_deviceID 9
	.extern	_devType 7
	.extern	_timeZone 6
	.extern	_moduleID 105
	.extern	$sleepMS 106
	.extern	$bb_xalloc 107
	.extern	$Exit 108
	.extern	$getIFConfig 109
	.extern	$connect 110
	.extern	$socket 111
	.extern	$perror 112
	.extern	_paramFiles 113
	.extern	_NumParamFiles 6
	.cseg
_backupd:
	pop	iy
	lcall	$$backupd
	.extern	$restoreIfRegValue 114
	.extern	_srfsIPAddr 9
	.extern	$fstat 115
	.extern	$stat 116
	.extern	$utime 117
	.extern	$stime 91
	.extern	$gettimeofday 118
	.extern	_applName 119
	.extern	_ApplAbbrev 119
	.extern	_applDTM 119
_EVENT_RestoredParamFile:
	pop	iy
	lcall	$$EVENT_RestoredParamFile
	.extern	$$EVENT_RestoredParamFile
	.extern	$EVENT_RestoredParamFile 120
	.bseg
_timeChkInterval:
	.ds	4
	.debug	F _timeChkInterval 9 0
_lastTimeChk:
	.ds	4
	.debug	F _lastTimeChk 9 0
_lastDateChk:
	.ds	4
	.debug	F _lastDateChk 9 0
_lastChgChk:
	.ds	4
	.debug	F _lastChgChk 9 0
_logfd:
	.ds	2
	.debug	F _logfd 6 0
_logDay:
	.ds	2
	.debug	F _logDay 7 0
_logMonth:
	.ds	2
	.debug	F _logMonth 7 0
_logFileName:
	.ds	64
	.debug	F _logFileName 66 0
	.debug	T	0B000220000100010001000000
	.debug	T	0900012100040003000400
	.debug	T	7E000322001200090003746D000600000007746D5F736563000600
	.debug	T	020007746D5F6D696E000600040008746D5F686F7572000600
	.debug	T	060008746D5F6D646179000600080007746D5F6D6F6E000600
	.debug	T	0A0008746D5F796561720006000C0008746D5F776461790006
	.debug	T	000E0008746D5F79646179000600100009746D5F6973647374
	.debug	T	00
	.debug	T	41000323000E00040009736F636B61646472000500000007735F74
	.debug	T	797065000500020007735F706F7274000900040005735F6970
	.debug	T	007900080009735F73706172657300
	.debug	T	5C0003240019000700010009000000076970416464720009000400
	.debug	T	076263416464720009000800086E65746D61736B0009000C00
	.debug	T	08676174657761790079001000086D61634164647200070016
	.debug	T	00046D7475000300180006737461746500
	.debug	T	130002250003002600270000002600020007000400
	.debug	T	070000260002000300
	.debug	T	070000270002002800
	.debug	T	72000328003100080001002600000009626173654E616D65000300
	.debug	T	02000963617465676F7279002A00030007706172616D73007A
	.debug	T	0007000B7772697465436F756E740006002700036664002F00
	.debug	T	29000962626D43616368650007002D000A636163686553697A
	.debug	T	650007002F00076F666673657400
	.debug	T	0B000229000100070027000000
	.debug	T	0700082A0004002B00
	.debug	T	4900032B001000060001000400000003696400040002000573697A
	.debug	T	650004000400056772616E004B0006000564617461002E000A
	.debug	T	00056E616D65007B000E000B726573746F726556616C00
	.debug	T	0B00022C000100010027000000
	.debug	T	0F00022D00020006002600000007000200
	.debug	T	0700082E0004000300
	.debug	T	0700082F0004003000
	.debug	T	33000330000A0004000100090000000574696D6500070004000573
	.debug	T	697A6500070006000864657654797065000700080004637263
	.debug	T	00
	.debug	T	BB000331002A000D00057374617400090000000773745F64657600
	.debug	T	090004000773745F696E6F00090008000873745F6D6F646500
	.debug	T	06000C000973745F6E6C696E6B0006000E000773745F756964
	.debug	T	00060010000773745F67696400090012000873745F72646576
	.debug	T	00090016000873745F73697A650006001A000B73745F626C6B
	.debug	T	73697A650006001C000A73745F626C6F636B730009001E0009
	.debug	T	73745F6174696D6500090022000973745F6D74696D65000900
	.debug	T	26000973745F6374696D6500
	.debug	T	0900013200180003001800
	.debug	T	0B000233000100060027000000
	.debug	T	130002340003000600270000002600020007000400
	.debug	T	130002350003000100260000000700020027000400
	.debug	T	17000236000400060026000000070002002700040003000600
	.debug	T	0900013700100003001000
	.debug	T	0F00023800020001002600000007000200
	.debug	T	0B000239000100060001000000
	.debug	T	3C00033A001600040001000700000008646174614C656E00460002
	.debug	T	000A6576656E7446756E630006000400086368616E4E756D00
	.debug	T	37000600056461746100
	.debug	T	0B00023B000100070001000000
	.debug	T	0B00023C000100010026000000
	.debug	T	2900033D00080002000874696D6576616C00090000000774765F73
	.debug	T	656300080004000874765F7573656300
	.debug	T	0900013E00200003002000
	.debug	T	0F00023F00020001000600000040000200
	.debug	T	070000400002002600
	.debug	T	290003410008000200087574696D62756600090000000761637469
	.debug	T	6D650009000400086D6F6474696D6500
	.debug	T	0900014200400003004000
	.debug	T	0900014300800003008000
	.debug	T	0B00024400010009007D000000
	.debug	T	130002450003004600460000000600020007000400
	.debug	T	070000460002000100
	.debug	T	0F00024700020026002600000026000200
	.debug	T	0B000248000100060026000000
	.debug	T	1300024900030006004B0000004B00040009000800
	.debug	T	1300024A0003004B004B0000004B00040009000800
	.debug	T	0700084B0004000100
	.debug	T	1300024C0003004B004B0000000600040009000600
	.debug	T	1300024D0003002600260000000900020007000600
	.debug	T	0F00024E00020026007D00000026000200
	.debug	T	0F00024F00020050007D00000050000200
	.debug	T	070000500002002200
	.debug	T	0F00025100020001004000000004000200
	.debug	T	0B000252000100040040000000
	.debug	T	0F00025300020007000900000009000400
	.debug	T	130002540003002E002E0000002E00040007000800
	.debug	T	130002550003000600060000000600020009000400
	.debug	T	1F0002560006000600060000002E00020006000600260008000700
	.debug	T	0A0009000C00
	.debug	T	1F0002570006000600260000002600020003000400260006000600
	.debug	T	080040000A00
	.debug	T	1F0002580006000600060000007E000200060004007F0006004600
	.debug	T	080006000A00
	.debug	T	0F00025900020006007D00000006000200
	.debug	T	1300025A0003000600060000007D0002007D000400
	.debug	T	0B00025B00010006007D000000
	.debug	T	0700005C0002005D00
	.debug	T	7702035D00E8002D000C7461736B5F737472756374008000000006
	.debug	T	737461636B00800002000C737461636B5F7374617274000700
	.debug	T	04000B737461636B5F73697A65000300060005737365670003
	.debug	T	0007000773746B6C6C7200030008000773746B686C72000300
	.debug	T	09000677707361720007000A0008777073616C68720003000C
	.debug	T	000677707362720007000D0008777073626C68720081000F00
	.debug	T	064644536574000300110009757365436F756E740003001200
	.debug	T	08636C61696D65640007001300067374617465005C00150007
	.debug	T	6D794164647200030017000A7A65726F416674657200060018
	.debug	T	0005617267630040001A0005617267760043001C0005617267
	.debug	T	730082009C0007776169745173000900A0000C7369675F7065
	.debug	T	6E64696E67000900A4000C7369675F626C6F636B6564000600
	.debug	T	A8000E707265656D70745F636F756E74000300AA0006696E55
	.debug	T	7365000300AB000565786974000300AC000A6C6F675374646F
	.debug	T	7574000300AD000B7265646972537464696F000900AE000A64
	.debug	T	696167466C616773008300B2000970726F634D61696E000600
	.debug	T	B40004706964000600B6000570706964000300B80009707269
	.debug	T	6F72697479000600B900086C617374457272000900BB000873
	.debug	T	746172746564000900BF000C6C617374526573756D65640009
	.debug	T	00C3000C6C61737453746F70706564000900C7000C6C617374
	.debug	T	52756E6E696E67000900CB000D726573756D65645469636B73
	.debug	T	000900CF000A7469636B7355736564000900D300076D735573
	.debug	T	6564000300D700096F7574436F756E74002E00D800076F7574
	.debug	T	507472000900DC000B736C6565705469636B73000900E0000D
	.debug	T	736C65657053746172746564000900E4000D74696D656F7574
	.debug	T	5469636B7300
	.debug	T	1300025E0003000600840000002E0002000D000000
	.debug	T	0F00025F00020006002E0000000D000000
	.debug	T	1300026000030006002E0000002E0004000D000000
	.debug	T	0900016100080009000200
	.debug	T	070008620004006300
	.debug	T	2701036300F900140001000900000008696E666F4B657900090004
	.debug	T	00086262436F756E74000700080006746F4964780007000A00
	.debug	T	0866726F6D4964780003000C0008777261707065640003000D
	.debug	T	000D63746C6453687574646F776E0007000E00097764745469
	.debug	T	636B730009001000087365636F6E647300070014000A6D696C
	.debug	T	6C69536563730009001600076D7354696D650085001A000866
	.debug	T	696C6C6572310006006A0003747A0009006C000D6864775265
	.debug	T	73657454696D6500090070000B7265626F6F7454696D650003
	.debug	T	0074000D646964536F6674526573657400090075000C646967
	.debug	T	4F7574537461746500090079000972746F7356657273000900
	.debug	T	7D000D74696D65725365636F6E647300090081000B696E7649
	.debug	T	6E666F4B657900860085000866696C6C65723200
	.debug	T	130002640003000600260000000600020006000400
	.debug	T	0B000265000100060006000000
	.debug	T	130002660003000600060000004600020006000400
	.debug	T	0F00026700020006002600000026000200
	.debug	T	54000368000C000600010087000000065F5F6F7574008900020005
	.debug	T	5F5F696E008B000400085F5F7772697465008B000600075F5F
	.debug	T	726561640006000800085F5F756E6765740046000A000B5F5F
	.debug	T	757365724461746100
	.debug	T	0900016900080003000800
	.debug	T	0B00026A000100090009000000
	.debug	T	0B00026B0001002E0009000000
	.debug	T	0B00026C000100010006000000
	.debug	T	0F00026D0002000600060000008D000200
	.debug	T	1300026E0003000600060000008E00020006000400
	.debug	T	1300026F0003000600060000000600020006000400
	.debug	T	0B00027000010001002E000000
	.debug	T	0900017100000027000000
	.debug	T	1300027200030006004B0000002600040007000600
	.debug	T	0F0002730002000600060000008F000200
	.debug	T	0F0002740002000600260000008F000200
	.debug	T	0F00027500020006002600000090000200
	.debug	T	0F00027600020006009100000046000200
	.debug	T	0900017700000003000000
	.debug	T	0B00027800010001007F000000
	.debug	T	0900017900060003000600
	.debug	T	0900017A00200007001000
	.debug	T	0700007B0002007C00
	.debug	T	1300027C00030001004B0000002600040007000600
	.debug	T	0700007D0002000900
	.debug	T	0700007E0002007800
	.debug	T	0700007F0002003A00
	.debug	T	070000800002000700
	.debug	T	070000810002004B00
	.debug	T	280003820004000100125F5F776169745F71756575655F68656164
	.debug	T	0092000000096C6973744865616400
	.debug	T	070000830002003F00
	.debug	T	070000840002006800
	.debug	T	0900018500500003005000
	.debug	T	0900018600740003007400
	.debug	T	070000870002008800
	.debug	T	0F00028800020006000300000046000200
	.debug	T	070000890002008A00
	.debug	T	0B00028A000100060046000000
	.debug	T	0700008B0002008C00
	.debug	T	1300028C00030008002E0000000800040046000800
	.debug	T	0700008D0002002400
	.debug	T	0700008E0002002300
	.debug	T	0700008F0002003100
	.debug	T	070000900002004100
	.debug	T	070000910002003D00
	.debug	T	2600039200040002000A6C6973745F686561640093000000056E65
	.debug	T	78740093000200057072657600
	.debug	T	070000930002009200
	.defseg	CONST
	.seg	CONST
@68:
	.db	"Found",0,"Created",0,"cache",0,"file",0,"BBRAM cache",0,"Flash",0,"fifoEvents",0,"/srfs/",0,".bak",0
	.defseg	FARCONST
	.seg	FARCONST
@9:
	.db	"getTime: socket()",0,"getTime: connect()",0,"getTime: read()",0,"UTC time: %04d-%02d-%02d %02d:%0"
	.db	"2d:%02d (%lu)",13,10,0,"%.12s_x%02lX_",0,"For '%.32s' file cache:",13,10,0,"Unable to allocate BBRAM cach"
	.db	"e for %.32s file",13,10,0,"%s %u byte BBRAM cache for '%s' at: 0x%05lX",13,10,0,"Flushing '%.24s' to "
	.db	"flash...",13,10,0,"flushParamCache()",0,"Can't save persistent data until the clock is synchroniz"
	.db	"ed.",13,10,"Try again later.",13,10,0,"saveParams(): Insufficient buffer size: %d",13,10,0,"saveParams()",0,"Upd"
	.db	"ating parameter %s: %s",13,10,0,"writeParams(): ID:%d  size:%d (%.24s)",0,"writeParams(): ID:%d  "
	.db	"size:%d (%.24)",0,"wrote param:%d  size:%d  addr: %05lX",10,0,"loadParams(): Insufficient buffe"
	.db	"r size: %d",13,10,0,"loadParams()",0,"loadParams(): Max # parameters exceeded",13,10,0,13,10,"reloading '%.24"
	.db	"s' from %s",13,10,0,"loadParams(): offset: %u  ID:%d  size:%d (%.24s)",13,10,0,"*** invalid ID or siz"
	.db	"e",13,10,0,"skipping unknown param",13,10,0,"*** granularity too large: %d",13,10,0,"  restoring: %ls",13,10,0,"Runti"
	.db	"me value of %.24s.%ls has changed",13,10,0,"read param:%d size:%d",13,10,0,"checkCache()",0,"/srfs/%.10s_"
	.db	"x%02lX_%02u-%02u.log",0,"deleting log file: %s (%ld hours old)",13,10,0,"opened log file: %s",13,10,0,13,10
	.db	10,"  ======== Module %s opened log file at %s ========",13,10,10,0,"  RTOS: %ls  %ls",13,10,0,"  APPL: %l"
	.db	"s  %ls",13,10,0,"fifoEvents",0,"cleaning up...",13,10,0,"***** new day *****",10,0,"Checking file: %s",13,10,0,"  local"
	.db	":  fd: %d, mtime: %lu  size: %lu",13,10,0,"  remote: fd: %d, mtime: %lu  size: %lu",13,10,0,"restore "
	.db	"from backup: %s",13,10,0,"  local:  %.24s  size: %lu",13,10,0," remote:  %.24s  size: %lu",13,10,0,"update ba"
	.db	"ckup copy: %s",13,10,0,"  remote: %.24s  size: %lu",13,10,0,"_%04d-%02d-%02d_%02d%02d%02d",0,"copyFileByN"
	.db	"ame(): ",0,"stat(): ",0
	.end
